from django.db import models

from django.core.validators import MaxValueValidator, MinValueValidator
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes.fields import GenericForeignKey
from professionals.models import Professional, Profession

from django.contrib.auth.models import User
from storages.backends.s3boto import S3BotoStorage

from static_data.models import State

class S3CustomStorage(S3BotoStorage):
    def _normalize_name(self, name):
        """
        S3 Storage function for storing files
        """
        return name



class ProfessionalRequestService(models.Model):
    """
    Professional Request Service
    """
    STATUS_CHOICE = (
        ("PENDING","PENDING"),
        ("ACCEPTED","ACCEPTED"),
        ("REJECTED","REJECTED")

    )
    professional = models.ForeignKey(Professional, on_delete=models.CASCADE, blank=True, null=True)
    profession = models.ForeignKey(Profession, on_delete=models.CASCADE, blank=True, null=True)
    service_name = models.CharField(blank=True, null=True, max_length=245)
    status = models.CharField(choices=STATUS_CHOICE, max_length=255, default=STATUS_CHOICE[0][0])
    comment = models.CharField(max_length=255, blank=True, null=True, help_text="Comment")
    status_change_by = models.ForeignKey(User, blank=True, null=True, on_delete=models.SET_NULL)

    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)
    is_deleted = models.BooleanField(
        default=False, help_text="activate/deactivate skills")

    def __str__(self):
        return "{}".format(self.service_name)

    class Meta:
        verbose_name_plural = "Professional Service Request"
        db_table = 'professional_service_request'




class ProfessionalRequestSkill(models.Model):
    """
    Professional Request Skill
    """
    STATUS_CHOICE = (
        ("PENDING","PENDING"),
        ("ACCEPTED","ACCE[TED"),
        ("REJECTED","REJECTED")

    )
    professional = models.ForeignKey(Professional, on_delete=models.CASCADE, blank=True, null=True)
    profession = models.ForeignKey(Profession, on_delete=models.CASCADE, blank=True, null=True)
    skill_name = models.CharField(blank=True, null=True, max_length=245)
    status = models.CharField(choices=STATUS_CHOICE, max_length=255, default=STATUS_CHOICE[0][0])
    comment = models.CharField(max_length=255, blank=True, null=True, help_text="Comment")
    status_change_by = models.ForeignKey(User, blank=True, null=True, on_delete=models.SET_NULL)

    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)
    is_deleted = models.BooleanField(
        default=False, help_text="activate/deactivate skills")

    def __str__(self):
        return "{}".format(self.skill_name)

    class Meta:
        verbose_name_plural = "Professional Skill Request"
        db_table = 'professional_skill_request'
