from rest_framework import serializers
from django.contrib.auth.models import User
from .models import *
from venue_owners.models import *
from players.models import Player
from professionals.models import *
from professionals.serializers import *
from advertisers.models import Advertiser
from commonAPI.models import ContactInfo, TelephoneNumbers, OnBoardAPI, Contact_Person
from commonAPI.serializers import *
from venues.models import *
from static_data.models import *
from static_data.serializers import *
from message.models import MessagesInbox
from campaigns.models import *
from campaigns.serializers import *
from django.db import transaction
from professionals.models import *
from venue_owners.serializers import *
from commonAPI.serializers import *
from venues.models import *
from venues.serializers import *
from advertisers.models import *
from advertisers.serializers import *
from utils.api_handle_errors import handle_errors
from django.contrib.auth.models import User
from rest_framework.validators import UniqueValidator
from django.contrib.auth.password_validation import validate_password
import uuid
import datetime

from utils.mail import send_emal_onboard_accept

from platform_central.models import RegisterPrivateKey


class AdvertiserOnboardSerializer(serializers.ModelSerializer):
    class Meta:
        model = Advertiser
        fields = "__all__"


class VenueOwnerOnboardSerializer(serializers.ModelSerializer):
    class Meta:
        model = VenueOwner
        fields = "__all__"


class VenueOnboardSerializer(serializers.ModelSerializer):
    class Meta:
        model = Venue
        fields = "__all__"


class ProfessionalOnboardSerializer(serializers.ModelSerializer):
    class Meta:
        model = Professional
        fields = "__all__"


class OnboardSerializer(serializers.ModelSerializer):
    class Meta:
        model = OnBoardAPI
        fields = "__all__"


class OnboardCreateProfileSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(
        required=False, help_text="ID of the venue owner")

    class Meta:
        model = OnBoardAPI
        fields = "__all__"


class VenueOwnerAutoFillSerializer(serializers.Serializer):
    """
    Venue Owner Auto Fill Serializer
    """
    key_word = serializers.CharField(
        required=True, help_text="Key Word to search venue owner")


class VenueOwnerSerializer(serializers.ModelSerializer):
    """
    Serializing Venue Owner Information
    """
    id = serializers.IntegerField(
        required=False, help_text="ID of the venue owner")
    country = CountrySerializer(read_only=True)
    city = CityGetSerializer(read_only=True)
    state = StateGetSerializer(read_only=True)
    area = AreaGetSerializer(read_only=True)
    vat = VatWithoutPageSerializer(read_only=True)
    vat_exemption_order = VatExemptionOrderWithoutPageSerializer(
        read_only=True)

    def validate_id(self, value):
        if VenueOwner.objects.filter(id=value).exists():
            return value
        else:
            raise serializers.FieldDoesNotExist("No Such Record")

    class Meta:
        model = VenueOwner
        fields = "__all__"
        read_only_fields = ['email', 'user', 'new_email',
                            'new_mobile', 'is_deleted', 'is_email_verified']


class AdminVenueOwnerSAVESerializer(serializers.ModelSerializer):
    class Meta:
        model = VenueOwner
        fields = '__all__'


class AdminVenueDailyAvailibilitySerializer(serializers.ModelSerializer):
    class Meta:
        model = VenueDailyTiming
        fields = '__all__'


class VenueOwnerEditInfoSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(
        required=False, help_text="ID of the venue owner")
    Telephone_number = serializers.JSONField(
        required=False, help_text='{"mobile":["847123654","35135654"]}')
    iban = serializers.CharField(required=False)
    business_name = serializers.CharField(required=False)
    bank = serializers.CharField(required=False)
    acc_holder_first_name = serializers.CharField(required=False)
    acc_holder_last_name = serializers.CharField(required=False)
    acc_type = serializers.CharField(required=False)
    bank_identification_no = serializers.CharField(required=False)
    contact_person = serializers.JSONField(
        required=False,
        help_text='[{"first_name":"Test","last_name":"asd","telephone":1234567,"email":"email"},{"first_name":"Test","last_name":"asd","telephone":1234567,"email":"email"}]')
    timing = serializers.JSONField(
        required=False, help_text='[{"day":1, "start_time":12:00:00, "is_active":1, "end_time":1:00:00}]')
    vat_exemption_order = serializers.IntegerField(
        required=False, help_text="vat_exemption_order")
    vat = serializers.IntegerField(
        required=False, help_text="vat")

    # Venue Details
    logo = serializers.ImageField(required=False, help_text="Venue Logo")
    name = serializers.CharField(required=False, help_text="Venue Name")
    street = serializers.CharField(required=False, help_text="Venue street")
    pincode = serializers.CharField(required=False, help_text="Venue pincode")
    mobile = serializers.CharField(required=False, help_text="Mobile number")

    venue_number = serializers.CharField(
        required=False, help_text="Venue number")
    buisness_vat = serializers.CharField(
        required=False, help_text="Venue buisness vat")
    tax_offices = serializers.CharField(
        required=False, help_text="Venue buisness vat")

    working_hours = serializers.CharField(
        required=False, help_text="working hours")
    user_code = serializers.CharField(
        required=False)

    def validate_id(self, value):
        if VenueOwner.objects.filter(id=value).exists():
            return value
        else:
            raise serializers.FieldDoesNotExist("No Such Record")

    # def validate_mobile(self, value):
    #     if len(value) < 10:
    #         raise serializers.ValidationError(
    #             "Mobile Number must be Filled in.")
    #     else:
    #         return value

    def validate_timing(self, value):
        for items in value:
            if items['day']:
                if not (items['start_time'] and items['end_time']):
                    raise serializers.ValidationError(
                        "Start time and End time must be filled in.")
                else:
                    return value

    @transaction.atomic
    def create(self, validated_data):
        '''
        venue owner
        '''
        # print(validated_data)
        print(self.context['id'])

        vobj = VenueOwner.objects.get(id=self.context['id'])
        validated_data['user'] = vobj.user.id
        validated_data['email'] = vobj.email
        print(validated_data['email'])
        print(validated_data['user'])

        print(validated_data)
        if "area" in validated_data:
            validated_data['area'] = validated_data['area'].id
        if "city" in validated_data:
            validated_data['city'] = validated_data['city'].id
        if "state" in validated_data:
            validated_data['state'] = validated_data['state'].id
        if "country" in validated_data:
            validated_data['country'] = validated_data['country'].id
        if "payment_method" in validated_data:
            validated_data['payment_method'] = validated_data['payment_method'].id

        serialized_data = AdminVenueOwnerSAVESerializer(instance=VenueOwner.objects.get(
            id=validated_data['id']), data=validated_data)
        if serialized_data.is_valid():
            venueowner_data = serialized_data.save()
        else:
            raise serializers.ValidationError(
                handle_errors(serialized_data.errors))

        '''
        Venue
        '''

        # validated_data.pop('street')
        # validated_data.pop('venue_number')
        # validated_data.pop('pincode')
        # validated_data.pop('country')
        # validated_data.pop('city')
        # validated_data.pop('state')
        # validated_data.pop('area')

        # validated_data['created_by'] = validated_data['user']
        # if Venue.objects.filter(created_by=validated_data['user'], is_deleted=False).exists():
        #     venue_serializer = VenueProfileSerializer(instance=Venue.objects.filter(
        #         created_by=validated_data['user'], is_deleted=False).first(), data=validated_data)
        # else:
        #     venue_serializer = VenueProfileSerializer(data=validated_data)

        # if venue_serializer.is_valid():
        #     venue_data = venue_serializer.save()
        # else:
        #     raise serializers.ValidationError(
        #         handle_errors(venue_serializer.errors))

        if "street" in validated_data:
            validated_data.pop('street')

        if "venue_number" in validated_data:
            validated_data.pop('venue_number')

        if "pincode" in validated_data:
            validated_data.pop('pincode')

        validated_data.pop('country')
        validated_data.pop('city')
        validated_data.pop('state')
        validated_data.pop('area')

        validated_data['created_by'] = validated_data['user']

        if Venue.objects.filter(created_by=validated_data['user'], is_deleted=False).exists():
            venue_data = Venue.objects.filter(
                created_by=validated_data['user'], is_deleted=False).first()
            if "name" in validated_data:
                venue_data.name = validated_data['name']
            if "business_name" in validated_data:
                venue_data.business_name = validated_data['business_name']
            if "buisness_vat" in validated_data:
                venue_data.buisness_vat = validated_data['buisness_vat']
            if "tax_offices" in validated_data:
                venue_data.tax_offices = validated_data['tax_offices']
            if "logo" in validated_data:
                venue_data.logo = validated_data['logo']
            venue_data.save()

        else:
            venue_data = Venue.objects.create(created_by=vobj.user)
            if "name" in validated_data:
                venue_data.name = validated_data['name']
            if "business_name" in validated_data:
                venue_data.business_name = validated_data['business_name']
            if "buisness_vat" in validated_data:
                venue_data.buisness_vat = validated_data['buisness_vat']
            if "tax_offices" in validated_data:
                venue_data.tax_offices = validated_data['tax_offices']
            if "logo" in validated_data:
                venue_data.logo = validated_data['logo']
            venue_data.save()

        '''
        buisness name
        
        '''

        if "business_name" in validated_data:
            venue_id = venue_data.id
            Venue.objects.filter(id=venue_id).update(
                business_name=validated_data['business_name'])

        '''
        timing
        '''

        if "timing" in validated_data:
            for i in validated_data['timing']:
                i['venue'] = venue_data.id
            timing_serializer = AdminVenueDailyAvailibilitySerializer(
                data=validated_data['timing'], many=True)
            print(timing_serializer)
            if timing_serializer.is_valid():
                VenueDailyTiming.objects.filter(venue=venue_data.id).delete()
                timing_data = timing_serializer.save()

            else:
                print(timing_serializer.errors)
                raise serializers.ValidationError(
                    handle_errors(timing_serializer.errors))

        '''
        Contact Info
        '''

        serialized_data = ContactInfoProfileSerializer(data=validated_data)
        if serialized_data.is_valid():
            if len(serialized_data.validated_data) > 0:
                contact_obj = VenueOwner.objects.filter(
                    id=validated_data['id'])
                contact_contentType = ContentType.objects.get_for_model(
                    VenueOwner)
                ContactInfo.objects.filter(
                    content_type__pk=contact_contentType.id, object_id__in=contact_obj).delete()
                serialized_data.validated_data['content_object'] = venueowner_data
                contact_info_data = serialized_data.save()

        else:
            raise serializers.ValidationError(
                handle_errors(serialized_data.errors))

        '''
        Contact Numbers
        '''

        if "Telephone_number" in validated_data:
            TelephoneNumbers.objects.filter(
                contact_id=contact_info_data).delete()
            telephone_number = []

            for i in validated_data['Telephone_number']['mobile']:
                if i == "":
                    pass
                else:
                    telephone_number.append(i)

            contact_numbers = [TelephoneNumbers(contact_id=contact_info_data,
                                                contact_number=i
                                                ) for i in telephone_number]

            TelephoneNumbers.objects.bulk_create(contact_numbers)

        '''
        Contact Person
        '''

        if "contact_person" in validated_data:
            contact_persons = []

            contact_contentType = ContentType.objects.get_for_model(VenueOwner)
            contact_obj = VenueOwner.objects.filter(
                id=validated_data['id'])

            if Contact_Person.objects.filter(content_type__pk=contact_contentType.id,
                                             object_id__in=contact_obj).exists():
                Contact_Person.objects.filter(
                    content_type__pk=contact_contentType.id, object_id__in=contact_obj).delete()

            for i in validated_data['contact_person']:
                if i['first_name'] == "" and i['last_name'] == "":
                    pass
                else:
                    contact_persons.append(i)

            serialized_data = ContactPersonSerializer(
                data=contact_persons, many=True)
            if serialized_data.is_valid():
                for i in serialized_data.validated_data:
                    i['content_object'] = venueowner_data
                serialized_data.save()

        '''
        Bank Info
        '''

        if "bank" in validated_data:
            serialized_data = BankInfoProfileSerializer(data=validated_data)
            BankInfo.objects.filter(
                object_id=venueowner_data.id).update(is_deleted=True)
            if serialized_data.is_valid():
                serialized_data.validated_data['content_object'] = venueowner_data
                serialized_data.save()
            else:
                raise serializers.ValidationError(
                    handle_errors(serialized_data.errors))

        return True

    class Meta:
        model = VenueOwner
        fields = '__all__'
        extra_kwargs = {
            'email': {"read_only": True},
            'new_email': {"read_only": True},
            'user': {"read_only": True}
        }


class VenuePhotoAndAllServicesSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(
        required=False, help_text="ID of the venue owner")
    services = serializers.JSONField(
        required=False, write_only=True,
        help_text='[{"venue_service":1,"price_per_hour":50},{"venue_service":2,"price_per_hour":60}]')
    all_services = serializers.JSONField(required=False, help_text="[1,2,3]")
    images = serializers.ListField(help_text="Array of images", required=False)
    latitude = serializers.FloatField(required=False)
    longitude = serializers.FloatField(required=False)
    google_address = serializers.CharField(required=False)
    place_id = serializers.CharField(required=False)
    professional_allowed = serializers.JSONField(
        required=False, help_text="[1,2,3]")
    all_sports = serializers.JSONField(required=False, help_text="[1,2,3]")
    covid19Measure_description = serializers.JSONField(
        required=False, help_text="[1,2,3]")
    main_image = serializers.ImageField(required=False)
    street = serializers.CharField(required=False)
    state = serializers.IntegerField(
        required=False, help_text="state")
    area = serializers.IntegerField(
        required=False, help_text="area")
    city = serializers.IntegerField(
        required=False, help_text="city")
    country = serializers.IntegerField(
        required=False, help_text="country")
    venue_number = serializers.CharField(required=False)
    pincode = serializers.CharField(required=False)
    name = serializers.CharField(required=False)

    def validate_id(self, value):
        if VenueOwner.objects.filter(id=value).exists():
            return value
        else:
            raise serializers.FieldDoesNotExist("No Such Record")

    def validate_services(self, data):

        for i in range(len(data)):
            if VenueService.objects.filter(id=data[i]['venue_service']).exists():
                data[i]['venue_service'] = VenueService.objects.get(
                    id=data[i]['venue_service'])
            else:
                raise serializers.FieldDoesNotExist(
                    "No record with ID " + str(data[i]['venue_service']))
        return data

    @transaction.atomic
    def create(self, validated_data):

        vobj = VenueOwner.objects.get(id=self.context['id'])
        validated_data['user'] = vobj.user.id
        validated_data['email'] = vobj.email

        venue = Venue.objects.filter(created_by=validated_data['user']).first()

        latitude = validated_data.get("latitude", None)
        longitude = validated_data.get("longitude", None)
        google_address = validated_data.get("google_address", "")
        place_id = validated_data.get("place_id", "")
        sports = validated_data.get("all_sports", "")
        professional_allowed = validated_data.get("professional_allowed", "")

        # Venue Services
        all_service = validated_data.get("all_services", "")
        covid19Measure_description = validated_data.get(
            "covid19Measure_description", "")

        # main_image = validated_data.get("main_image", "")

        # Update lat long
        venue.latitude = latitude
        venue.longitude = longitude
        venue.google_address = google_address
        venue.place_id = place_id
        venue.description = validated_data['description']
        venue.street = validated_data['street']
        venue.area = Area.objects.get(
            id=validated_data['area'])
        venue.city = City.objects.get(
            id=validated_data['city'])
        venue.state = State.objects.get(
            id=validated_data['state'])
        venue.country = Country.objects.get(
            id=validated_data['country'])
        venue.pincode = validated_data['pincode']
        venue.venue_number = validated_data['venue_number']
        venue.name = validated_data['name']

        if 'main_image' in validated_data:
            venue.main_image = validated_data["main_image"]

        venue.professional_allowed.set(professional_allowed)
        venue.all_sports.set(sports)
        venue.covid19Measure_description.set(covid19Measure_description)

        # venue sevices
        venue.all_services.set(all_service)

        venue.save()

        if 'images' in validated_data:
            # VenueImage.objects.filter(venue=venue).delete()
            obj = [VenueImage(
                created_by=vobj.user,
                venue=venue,
                image=image
            ) for image in validated_data['images']]

            VenueImage.objects.bulk_create(obj)

        """
        Venue Service Cost
        """

        attributes = []

        venue_obj = Venue.objects.filter(
            created_by=validated_data['user']).first()

        venue_service_cost_object = VenueServiceCost.objects.filter(
            venue=venue_obj.id)
        venue_service_cost_object.delete()

        if 'services' in validated_data:
            attributes.append("services")
            services = validated_data['services']
            validated_data.pop('services')

        validated_data['venue'] = venue_obj

        if 'services' in attributes:
            venue_Service_cost = [VenueServiceCost(
                **i,
                venue=venue_obj
            ) for i in services]

            VenueServiceCost.objects.bulk_create(
                venue_Service_cost)

        return True

    class Meta:
        model = VenueImage
        fields = '__all__'
        read_only_fields = ['venue', 'image', 'created_by']


class VenueOwnerListViewSerialzier(serializers.Serializer):
    """
    Venue Owner Lists
    """

    venue_owner_id = serializers.IntegerField(required=True)
    buisness_vat = serializers.CharField(
        required=False, help_text="Venue buisness vat")
    page = serializers.IntegerField(
        required=False, default=1, help_text="Page number")

    def validate_venue_owner_id(self, value):
        if VenueOwner.objects.filter(id=value).exists():
            return value
        else:
            raise serializers.FieldDoesNotExist("No Such Record")


class VenueOwnerCRMSerialzier(serializers.Serializer):
    """
    Venue Owner Lists
    """
    venue_owner_id = serializers.IntegerField(required=True)
    from_date = serializers.DateField(required=False)
    to_date = serializers.DateField(required=False)

    def validate_venue_owner_id(self, value):
        if VenueOwner.objects.filter(id=value).exists():
            return value
        else:
            raise serializers.FieldDoesNotExist("No Such Record")


class ContactInfoGETSerializer(serializers.ModelSerializer):
    """
    Getting Contact Info of Venue Owner
    """
    telephone = serializers.SerializerMethodField()

    class Meta:
        model = ContactInfo
        fields = "__all__"

    def get_telephone(self, obj):
        data = TelephoneNumbers.objects.filter(contact_id=obj.id)
        serializers = TelephoneNumbersSerializers(data, many=True)
        return serializers.data


class ApprovalsSerializer(serializers.Serializer):
    REGISTERATION_TYPE = (
        ('A', 'Advertiser'),
        ('P', 'Professional'),
        ('V', 'Venue Owner'),
    )
    user_type = serializers.ChoiceField(
        required=False, choices=REGISTERATION_TYPE, help_text="V - Venue Owner, P - Professional, A - Advertiser")
    sport_type = serializers.CharField(required=False, help_text="Sports Type")
    service_type = serializers.CharField(
        required=False, help_text="Service Type")
    key_word = serializers.CharField(
        required=False, help_text="Name / Email / Phone")
    venue_name = serializers.CharField(required=False, help_text="Venue Name")
    location = serializers.CharField(
        required=False, help_text="City")
    ordering = serializers.CharField(
        required=False, help_text="Ascending / Descending")
    mode = serializers.CharField(
        required=False, help_text="Approved, Declined, In Progress, New, Complete")
    assortment = serializers.CharField(
        required=False, help_text="RegisterationTime-Descending / RegisterationTime-Ascending")
    page = serializers.IntegerField(
        required=False, default=1, help_text="Page number")


class ChangeApproveStatusSerializer(serializers.Serializer):
    mode = serializers.CharField(
        required=False, help_text="Mode to change to || Approved, Declined, Deleted")
    id = serializers.IntegerField(required=True, help_text="Id of the form")

    # def validate_id(self,value):
    #     if OnBoardAPI.objects.filter(id=value, is_deleted=False).exists():
    #         return value
    #     else:
    #         raise serializers.FieldDoesNotExist(" No Such Record ")


class GetApprovalsByIDSerializer(serializers.Serializer):
    id = serializers.IntegerField(required=True, help_text="ID")


class GetFieldAttributesSerializer(serializers.ModelSerializer):
    class Meta:
        model = FieldAttribute
        fields = "__all__"


class GetVenueFieldSerializer(serializers.ModelSerializer):
    field_type = serializers.CharField(
        source="field_type.name", required=False)
    floor_type = serializers.CharField(
        source="field_type.name", required=False)
    field_attributes = GetFieldAttributesSerializer(many=True, required=False)

    class Meta:
        model = VenueField
        fields = "__all__"


class GetCampaignByIDSerializer(serializers.ModelSerializer):
    sport = serializers.CharField(source="sport.name", required=False)

    class Meta:
        model = CampaignPart1
        fields = '__all__'


# class VenueAddFieldSerializer(serializers.ModelSerializer):
#     venue = serializers.PrimaryKeyRelatedField(
#         queryset=Venue.objects.all(), help_text="Venue ID")
#     field_attributes = serializers.JSONField(
#         required=False, help_text='[1,2,3,4]')
#     images = serializers.ListField(
#         required=False, help_text="list of images to save")

#     class Meta:
#         model = VenueField
#         fields = "__all__"


# class VenueEditFieldSerializer(serializers.ModelSerializer):
#     id = serializers.CharField(
#         required=False, help_text="Id of field to update")
#     field_attributes = serializers.JSONField(
#         required=False, help_text='[1,2,3,4]')
#     images = serializers.ListField(
#         required=False, help_text="list of images to save")

#     def validate_field_attributes(self, value):
#         data = []
#         for i in value:
#             if FieldAttribute.objects.filter(id=i).exists():
#                 data.append(i)
#             else:
#                 raise serializers.ValidationError("No Record Found")
#         return data

#     class Meta:
#         model = VenueField
#         fields = "__all__"


class VenueAddFieldSerializer(serializers.ModelSerializer):
    venueowner = serializers.PrimaryKeyRelatedField(
        queryset=VenueOwner.objects.all(), help_text="VenueOwner ID")
    field_services = serializers.JSONField(
        required=False, help_text='[1,2,3,4]')
    images = serializers.ListField(
        required=False, help_text="list of images to save")

    class Meta:
        model = VenueField
        fields = "__all__"


class VenueEditFieldSerializer(serializers.ModelSerializer):
    id = serializers.CharField(
        required=False, help_text="Id of field to update")
    field_services = serializers.JSONField(
        required=False, help_text='[1,2,3,4]')
    images = serializers.ListField(
        required=False, help_text="list of images to save")

    def create(self, validated_data):

        field_obj = VenueField.objects.get(id=validated_data['id'])
        if "main_image" in validated_data:
            field_obj.main_image = validated_data['main_image']

        if "availability_in_use" in validated_data:
            field_obj.availability_in_use = validated_data['availability_in_use']
        else:
            field_obj.availability_in_use = None

        if "mixed_gender" in validated_data:
            field_obj.mixed_gender = validated_data['mixed_gender']
        else:
            field_obj.mixed_gender = None

        field_obj.name = validated_data['name']
        field_obj.description = validated_data['description']
        field_obj.sport = validated_data['sport']
        field_obj.field_type = validated_data['field_type']
        field_obj.floor_type = validated_data['floor_type']
        field_obj.facility_type = validated_data['facility_type']
        field_obj.lights = validated_data['lights']
        field_obj.save()

        field_type_obj = FieldTypeSerializer(field_obj.field_type)
        facility_type_obj = FacilitysGetSerializer(field_obj.facility_type)
        floor_type_obj = FloorTypeSerializer(field_obj.floor_type)
        lights_obj = LightsGetSerializer(field_obj.lights)
        sport_obj = SportAdminSerializer(field_obj.sport)
        availability_obj = AvailabilityInUseSerializer(
            field_obj.availability_in_use)
        mixed_gender_obj = MixedGenderSerializer(field_obj.mixed_gender)

        return_data = {
            "id": field_obj.id,
            "description": field_obj.description,
            "field_name": field_obj.name,
            "field_type": field_type_obj.data,
            "floor_type": floor_type_obj.data,
            "seat_capacity": field_obj.seat_capacity,
            "lights": lights_obj.data,
            "facility_type": facility_type_obj.data,
            "facility_type_id": field_obj.facility_type.id,
            "sport": sport_obj.data,
            "mixed_gender": mixed_gender_obj.data,
            "availability": availability_obj.data
        }
        main_image_class = VenueFieldSerializerVer1(
            field_obj, many=False)
        return_data['main_image'] = main_image_class.data

        if "field_services" in validated_data:
            VenueFieldService.objects.filter(venue_field=field_obj.id).delete()
            venue_fieldService = [VenueFieldService(
                venue_service=VenueService.objects.get(id=i),
                venue_field=field_obj,
                created_by=field_obj.created_by
            ) for i in validated_data['field_services']]
            service_obj = VenueFieldService.objects.bulk_create(
                venue_fieldService)
            services_class = VenueFieldServicePreviewSerializer1(
                service_obj, many=True)
            return_data['services'] = services_class.data

        else:
            VenueFieldService.objects.filter(venue_field=field_obj.id).delete()

        if "images" in validated_data:
            venueField_image = [VenueFieldImage(
                image=i,
                venue_field=field_obj,
                created_by=field_obj.created_by
            )
                for i in validated_data['images']]
            image_obj = VenueFieldImage.objects.bulk_create(venueField_image)
            images_class = VenueFieldImageSerializer(image_obj, many=True)
            return_data['images'] = images_class.data

        else:
            image_obj = VenueFieldImage.objects.filter(
                venue_field=field_obj.id)
            images_class = VenueFieldImageSerializer(image_obj, many=True)
            return_data['images'] = images_class.data

        return return_data

    class Meta:
        model = VenueField
        fields = "__all__"


class VenueOwnerAddContactInfoSerializer(serializers.ModelSerializer):
    content_object = serializers.PrimaryKeyRelatedField(
        queryset=VenueOwner.objects.all(), help_text="Venue Owner ID")
    Telephone_number = serializers.JSONField(
        required=False, help_text='{"mobile":["847123654","35135654"]}')

    def create(self, validated_data):
        telephone_number = None
        if "Telephone_number" in validated_data:
            telephone_number = validated_data['Telephone_number']
            validated_data.pop("Telephone_number")

        contactinfo_obj = ContactInfo.objects.create(**validated_data)

        if telephone_number is not None:
            TelephoneNumbers.objects.filter(
                contact_id=contactinfo_obj).delete()
            contact_numbers = [TelephoneNumbers(contact_id=contactinfo_obj,
                                                contact_number=i
                                                ) for i in telephone_number['mobile']]

            TelephoneNumbers.objects.bulk_create(contact_numbers)

        return contactinfo_obj

    class Meta:
        model = ContactInfo
        fields = "__all__"


class AdminGetMessageSerialzer(serializers.Serializer):
    # STATUS_CHOICE = (
    #     ("True", "True"),
    #     ("False", "False"),
    # )
    id = serializers.PrimaryKeyRelatedField(
        queryset=VenueOwner.objects.all(), required=True, help_text="ID of venue owner")
    MessageType = serializers.CharField(
        max_length=200, help_text='Inbox / Sentbox', required=False)
    from_date = serializers.DateField(required=False, help_text="YYYY-MM-DD")
    to_date = serializers.DateField(required=False, help_text="YYYY-MM-DD")
    is_deleted = serializers.BooleanField(
        required=False, default=False, help_text="deleted messages")
    is_read = serializers.BooleanField(
        required=False, help_text=" read/unread messages")
    # is_read = serializers.ChoiceField(
    #     choices=STATUS_CHOICE, required=False, help_text="is_read = True/False")
    page = serializers.IntegerField(
        required=False, default=1, help_text="Page number")


class AdminComposeMessageSerialier(serializers.ModelSerializer):
    recipient = serializers.CharField(required=False)
    sender = serializers.CharField(required=False)

    def validate_recipient(self, value):
        recipient = User.objects.get(username=value)
        return recipient

    def validate_sender(self, value):
        sender = User.objects.get(username=value)
        return sender

    class Meta:
        model = MessagesInbox
        fields = ('recipient', 'sender', 'message')


class VenueFieldApprovalSerializer(serializers.ModelSerializer):
    field_type = serializers.CharField(
        source="field_type.name", required=False)
    floor_type = serializers.CharField(
        source="floor_type.name", required=False)

    class Meta:
        model = VenueField
        fields = '__all__'


class ApprovalFieldStatusSerializer(serializers.Serializer):
    STATUS_CHOICES = (
        ('A', 'Accepted'),
        ('R', 'Rejected'),
        ('O', 'On Hold')
    )
    status = serializers.ChoiceField(
        choices=STATUS_CHOICES, required=True, help_text="O for On hold / A for accepted / R rejected")
    page = serializers.IntegerField(
        required=False, default=1, help_text="page number")
    field_type = serializers.IntegerField(
        required=False, help_text="fieldtype")
    floor_type = serializers.IntegerField(
        required=False, help_text="floortype")


class ApprovalStatusSerializer(serializers.Serializer):
    STATUS_CHOICES = (
        ('A', 'Accepted'),
        ('R', 'Rejected'),
        ('O', 'On Hold')
    )
    status = serializers.ChoiceField(
        choices=STATUS_CHOICES, required=True, help_text="O for On hold / A for accepted / R rejected")
    page = serializers.IntegerField(
        required=False, default=1, help_text="page number")


class ApprovalCampaignStatusSerializer(serializers.Serializer):
    STATUS_CHOICES = (
        ('A', 'Accepted'),
        ('R', 'Rejected'),
        ('O', 'On Hold')
    )
    status = serializers.ChoiceField(
        choices=STATUS_CHOICES, required=True, help_text="O for On hold / A for accepted / R rejected")
    page = serializers.IntegerField(
        required=False, default=1, help_text="page number")
    campaign_kind = serializers.CharField(
        required=False, help_text="sms / email / banner")
    sport = serializers.CharField(required=False, write_only=True)
    sport_name = serializers.CharField(
        required=False, read_only=True, source="sport.name")
    assortment = serializers.CharField(
        required=False, write_only=True, help_text="Assortment start_date / end_date")


class CampaignStatusSerializer(serializers.ModelSerializer):
    STATUS_CHOICES = (
        ('A', 'Accepted'),
        ('R', 'Rejected'),
        ('O', 'On Hold')
    )
    status = serializers.ChoiceField(
        choices=STATUS_CHOICES, required=True, help_text="O for On hold / A for accepted / R rejected")


class CampaignApprovalSerializer(serializers.ModelSerializer):
    class Meta:
        model = CampaignPart1
        fields = '__all__'


class VenueChangeStatusSerialzier(serializers.Serializer):
    id = serializers.PrimaryKeyRelatedField(
        queryset=VenueField.objects.all(), required=True)
    status = serializers.CharField(
        required=True, help_text="Approved / Declined")


class CampaignChangeStatusSerialzier(serializers.Serializer):
    id = serializers.PrimaryKeyRelatedField(
        queryset=CampaignPart1.objects.all(), required=True)
    status = serializers.CharField(
        required=True, help_text="Approved / Declined")


class GetMessageIDSerializer(serializers.Serializer):
    id = serializers.PrimaryKeyRelatedField(
        queryset=MessagesInbox.objects.all(), required=True)


class DeleteMessageSerializer(serializers.Serializer):
    id = serializers.PrimaryKeyRelatedField(
        queryset=MessagesInbox.objects.all(), required=True, help_text="Message ID")


class GetMessageDetailsSerializer(serializers.ModelSerializer):
    recipient = serializers.CharField(source="recipient.username")
    sender = serializers.CharField(source="sender.username")

    class Meta:
        model = MessagesInbox
        fields = '__all__'


class VenueOwnerDeleteContactInfoSerializer(serializers.ModelSerializer):
    id = serializers.CharField(help_text="Contact Info ID")

    def validate_id(self, value):
        if Contact_Person.objects.filter(id=value).exists():
            return value
        else:
            raise serializers.FieldDoesNotExist("No Record found with this ID")

    class Meta:
        model = ContactInfo
        fields = ["id"]


class ProfessionalSerivcesSerializer(serializers.ModelSerializer):
    status = serializers.CharField(required=False, default="O")
    page = serializers.IntegerField(
        required=False, default=1, help_text="Page Number", write_only=True)
    service = ProfessionSerializer(
        required=False, read_only=True, many=True)

    class Meta:
        model = Professional
        fields = ['id', 'first_name', 'last_name',
                  'email', 'status', 'page', 'service']


class ProfessionalChangeStatusSerialzier(serializers.Serializer):
    id = serializers.PrimaryKeyRelatedField(
        queryset=Professional.objects.all(), required=True)
    status = serializers.CharField(
        required=True, help_text="Approved / Declined")


class GetProfessionalByIDSerializer(serializers.Serializer):
    id = serializers.IntegerField(
        help_text="ID of the professional", required=True)


class ProfessionalServicePhotoSerializer(serializers.ModelSerializer):
    class Meta:
        model = ProfessionalServicePhoto
        fields = ['id', 'main_image']


class ProfessionalStepServiceDataSerializer(serializers.ModelSerializer):
    profession = serializers.CharField(
        source="profession.name", required=False)

    class Meta:
        model = ProfessionalStepServiceData
        fields = ['id', 'profession', 'service_description']


class ProfessionalCertificateSerializer(serializers.ModelSerializer):
    class Meta:
        model = ProfessionalCertificate
        fields = ['certificate_name', 'certificate']


class ProfessionalStepSkillSerializer(serializers.ModelSerializer):
    skill_name = serializers.CharField(
        required=False, source="skill_name.name")

    class Meta:
        model = ProfessionalStepSkill
        fields = ['skill_name', 'id']


class AdminPackagesGetSerializer(serializers.ModelSerializer):
    sport_relivance = serializers.CharField(
        required=False, help_text="Sport ID")
    sport_name = serializers.CharField(
        required=False, read_only=True, source="sport.name")
    created_at_from = serializers.DateField(
        required=False, help_text="YYYY-MM-DD")
    created_at_to = serializers.DateField(
        required=False, help_text="YYYY-MM-DD")
    keyword = serializers.CharField(
        required=False, help_text="Search Text box")
    page = serializers.IntegerField(
        required=False, default=1, help_text="Page number. Default = 1")
    package_type = serializers.CharField(
        required=False, help_text="Subscription/Lessons/Bookings Package")

    status_name = serializers.CharField(
        required=False, write_only=True, help_text="PENDING/APPROVED/DECLINED")

    venue_name = serializers.SerializerMethodField('venue_id')

    def venue_id(self, obj):
        venue_obj = Venue.objects.filter(created_by=obj.created_by).first()

        return venue_obj.name

    class Meta:
        model = Packages
        fields = '__all__'
        read_only_fields = ['package_image', 'package_description',
                            'number_sessions', 'session_duration', 'package_quantity', 'package_duration', 'level',
                            'price', 'name']


class PackageChangeStatusSerialzier(serializers.Serializer):
    id = serializers.PrimaryKeyRelatedField(
        queryset=Packages.objects.all(), required=True)
    status = serializers.CharField(
        required=True, help_text="Approved / Declined")


class DeletePackageSerializer(serializers.Serializer):
    id = serializers.PrimaryKeyRelatedField(
        queryset=Packages.objects.all(), required=True, help_text="Package ID")


class DeleteFieldImageSerializer(serializers.Serializer):
    id = serializers.IntegerField(required=True)


class VenueOwnerAddContactPersonSerializer(serializers.ModelSerializer):
    venueowner = serializers.IntegerField(
        required=True, help_text="Venue Owner ID")
    contact_person = serializers.JSONField(
        required=False,
        help_text='[{"first_name":"Test","last_name":"asd","telephone":1234567,"email":"email"},{"first_name":"Test","last_name":"asd","telephone":1234567,"email":"email"}]')

    def create(self, validated_data):
        if "contact_person" in validated_data:
            contact_persons = []
            for i in validated_data['contact_person']:
                if i['first_name'] == "" and i['last_name'] == "":
                    pass
                else:
                    contact_persons.append(i)

            serialized_data = ContactPersonSerializer(
                data=contact_persons, many=True)
            if serialized_data.is_valid():
                for i in serialized_data.validated_data:
                    i['content_object'] = VenueOwner.objects.get(
                        id=validated_data['venueowner'])
                serialized_data.save()

        return True

    class Meta:
        model = Contact_Person
        fields = "__all__"


class VenueOwnerStatisticsSerializer(serializers.Serializer):
    """
    Venue Owner Lists
    """
    venue_owner_id = serializers.IntegerField(required=True)
    from_date = serializers.DateField(required=False)
    to_date = serializers.DateField(required=False)
    field = serializers.IntegerField(required=False)

    def validate_venue_owner_id(self, value):
        if VenueOwner.objects.filter(id=value).exists():
            return value
        else:
            raise serializers.FieldDoesNotExist("No Such Record")


class VenueOwnerEditContactPerson(serializers.Serializer):
    id = serializers.PrimaryKeyRelatedField(
        queryset=Contact_Person.objects.all(), required=True)
    first_name = serializers.CharField(required=True)
    last_name = serializers.CharField(required=True)
    email = serializers.CharField(required=True)
    telephone = serializers.CharField(required=True)


class VenueOwnerIDSerializer(serializers.Serializer):
    """
    Venue Owner Lists
    """
    venue_owner_id = serializers.IntegerField(required=True)

    def validate_venue_owner_id(self, value):
        if VenueOwner.objects.filter(id=value).exists():
            return value
        else:
            raise serializers.FieldDoesNotExist("No Such Record")


class VenueOwnerVenueFieldSerializer(serializers.ModelSerializer):
    class Meta:
        model = VenueField
        fields = '__all__'


class VenueOwnerListSerializer(serializers.ModelSerializer):
    country = CountrySerializer(read_only=True)
    city = CityGetSerializer(read_only=True)
    state = StateGetSerializer(read_only=True)
    area = AreaGetSerializer(read_only=True)

    class Meta:
        model = VenueOwner
        fields = "__all__"


class ProfessionalListSerializer(serializers.ModelSerializer):
    country = CountrySerializer(read_only=True)
    city = CityGetSerializer(read_only=True)
    state = StateGetSerializer(read_only=True)
    area = AreaGetSerializer(read_only=True)
    service = ProfessionSerializer(read_only=True, many=True)

    class Meta:
        model = Professional
        fields = "__all__"


class AdvertiserListSerializer(serializers.ModelSerializer):
    country = CountrySerializer(read_only=True)
    city = CityGetSerializer(read_only=True)
    state = StateGetSerializer(read_only=True)
    area = AreaGetSerializer(read_only=True)

    class Meta:
        model = Advertiser
        fields = "__all__"


class VenueSerializerForList(serializers.ModelSerializer):
    class Meta:
        model = Venue
        fields = "__all__"
        # fields = ['business_name','buisness_vat']


class VenueOwnerSerializerForFilteredList(serializers.Serializer):
    business_vat = serializers.CharField(required=False)
    business_name_or_venue_name = serializers.CharField(required=False)
    all_sports = serializers.CharField(required=False)
    area = serializers.JSONField(
        required=False, help_text='list of area=[1,2,3]')
    page = serializers.JSONField(
        required=False, default=1, help_text="Page number")


class VenueOwnerIDGETSerializer(serializers.Serializer):
    id = serializers.IntegerField(
        required=False, help_text="ID of the venue owner")

    def validate_id(self, value):
        if VenueOwner.objects.filter(id=value).exists():
            return value
        else:
            raise serializers.FieldDoesNotExist("No Such Record")


class VenuePhotosGetSerializer(serializers.ModelSerializer):
    venue = serializers.CharField(source="venue.name", read_only=True)

    class Meta:
        model = VenueImage
        fields = '__all__'


class VenueAllServicesGetSerializer(serializers.ModelSerializer):
    created_by = serializers.CharField(
        source="created_by.username", read_only=True)
    all_sports = SportSerializer(read_only=True, many=True)
    all_services = VenueServiceSerializer1(read_only=True, many=True)
    allowed_entry = VenuePermissionSerializer(read_only=True, many=True)
    professional_allowed = ProfessionSerializer(read_only=True, many=True)

    country = CountrySerializer(read_only=True)
    city = CityGetSerializer(read_only=True)
    state = StateGetSerializer(read_only=True)
    area = AreaGetSerializer(read_only=True)

    class Meta:
        model = Venue
        fields = '__all__'


class OnBoardIdSerializer(serializers.Serializer):
    id = serializers.CharField(required=True)


class OnBoardAPISerializer(serializers.ModelSerializer):
    class Meta:
        model = OnBoardAPI
        fields = "__all__"


class DeleteVenueImageSerializer(serializers.Serializer):
    id = serializers.JSONField(required=True, help_text='[1,2]')


class RegisterSerializer(serializers.ModelSerializer):

    def create(self, validated_data):
        user = User.objects.create(
            username=validated_data['username'],
            email=validated_data['email'],
            first_name=validated_data['first_name'],
            last_name=validated_data['last_name'],
            is_active=False
        )
        user.save()
        return user

    class Meta:
        model = User
        fields = ('username', 'email', 'first_name', 'last_name', 'is_active')


class AdminVenueOwnerPOSTSerializer(serializers.ModelSerializer):
    Telephone_number = serializers.JSONField(
        required=False, help_text='{"mobile":["847123654","35135654"]}')
    iban = serializers.CharField(required=False)
    business_name = serializers.CharField(required=False)
    bank = serializers.CharField(required=False)
    acc_holder_first_name = serializers.CharField(required=False)
    acc_holder_last_name = serializers.CharField(required=False)
    acc_type = serializers.CharField(required=False)
    bank_identification_no = serializers.CharField(required=False)
    contact_person = serializers.JSONField(
        required=False,
        help_text='[{"first_name":"Test","last_name":"asd","telephone":1234567,"email":"email"},{"first_name":"Test","last_name":"asd","telephone":1234567,"email":"email"}]')
    timing = serializers.JSONField(
        required=False, help_text='[{"day":1, "start_time":12:00:00, "is_active":1, "end_time":1:00:00}]')

    # Venue Details
    logo = serializers.ImageField(required=False, help_text="Venue Logo")
    name = serializers.CharField(required=False, help_text="Venue Name")
    street = serializers.CharField(required=False, help_text="Venue street")
    pincode = serializers.CharField(required=False, help_text="Venue pincode")
    mobile = serializers.CharField(required=False, help_text="Mobile number")
    email = serializers.CharField(required=False, help_text="Email ID")

    venue_number = serializers.CharField(
        required=False, help_text="Venue number")
    buisness_vat = serializers.CharField(
        required=False, help_text="Venue buisness vat")
    tax_offices = serializers.CharField(
        required=False, help_text="tax offices")

    working_hours = serializers.CharField(
        required=False, help_text="working hours")

    state = serializers.IntegerField(
        required=False, help_text="state")
    area = serializers.IntegerField(
        required=False, help_text="area")
    city = serializers.IntegerField(
        required=False, help_text="city")
    country = serializers.IntegerField(
        required=False, help_text="country")
    latitude = serializers.DecimalField(
        required=False, max_digits=9, decimal_places=6, help_text="Latitude")
    longitude = serializers.DecimalField(
        required=False, max_digits=9, decimal_places=6, help_text="Longitude")

    def validate_mobile(self, value):
        if len(value) < 10:
            raise serializers.ValidationError(
                "Mobile Number must be Filled in.")
        else:
            return value

    def validate_timing(self, value):
        for items in value:
            if items['day']:
                if not (items['start_time'] and items['end_time']):
                    raise serializers.ValidationError(
                        "Start time and End time must be filled in.")
                else:
                    return value

    def validate_business_name(self, value):
        if Venue.objects.filter(business_name=value).exists():
            raise serializers.ValidationError("business name already exists")
        else:
            return value

    @transaction.atomic
    def create(self, validated_data):
        '''
        user
        '''

        if User.objects.filter(username=validated_data['email'],
                               email=validated_data['email']).exists():
            raise serializers.ValidationError("user already exists")
        else:
            Userdata = {}
            Userdata['username'] = validated_data['email']
            Userdata['email'] = validated_data['email']
            Userdata['first_name'] = validated_data['first_name']
            Userdata['last_name'] = validated_data['last_name']
            # Userdata['is_active'] = False

            user_ser = RegisterSerializer(data=Userdata)
            if user_ser.is_valid():
                user_ser.save()
                user = User.objects.get(username=Userdata['username'])
                validated_data['user'] = user.id
            else:
                raise serializers.ValidationError(
                    handle_errors(user_ser.errors))

            my_group = Group.objects.get(name='Venue Owner')
            my_group.user_set.add(user)
        # Convert UUID format to a Python string.
        random = str(uuid.uuid4())

        # Make all characters uppercase.
        random = random.upper()

        # Remove the UUID '-'.
        random = random.replace("-", "")

        # Return the random string.
        user_code = random[0:6]
        validated_data['user_code'] = user_code

        '''
       onboard api 
       '''
        validated_data['created_by'] = validated_data['user']
        validated_data['status'] = 'A'
        validated_data['registeration_type'] = 'V'
        onboard_serializer = OnboardSerializer(data=validated_data)

        if onboard_serializer.is_valid():
            onboard_data = onboard_serializer.save()
        else:
            raise serializers.ValidationError(
                handle_errors(onboard_serializer.errors))

        '''
       Send email to user
       '''
        images = Template_Images.objects.filter().values('template_image')
        logo = images[0]
        logo2 = images[1]

        token_obj = RegisterPrivateKey.objects.create(
            user=user)
        business_name_for_msg = onboard_data.business_name
        registeration_type = "field"
        verify_redirect = "{}{}".format(
            "https://sportsbookingnew.worldindia.com/#",
            "/" + registeration_type + "/create-profile/set-password/" + str(token_obj.key))
        msg = business_name_for_msg + " has been approved as a Venue Owner" \
                                      ". Your profile is under development. You can logon for first time to the platform changing your password."
        send_emal_onboard_accept(
            onboard_data.first_name, verify_redirect, onboard_data.email, msg, logo, logo2)

        '''
        venue owner
        '''

        serialized_data = VenueOwnerOnboardSerializer(data=validated_data)
        if serialized_data.is_valid():
            venueowner_data = serialized_data.save()
        else:
            raise serializers.ValidationError(
                handle_errors(serialized_data.errors))

        '''
        Venue
        '''
        validated_data.pop('street')
        validated_data.pop('venue_number')
        validated_data.pop('pincode')
        validated_data.pop('country')
        validated_data.pop('city')
        validated_data.pop('state')
        validated_data.pop('area')
        validated_data['created_by'] = validated_data['user']

        venue_serializer = VenueProfileSerializer(data=validated_data)

        if venue_serializer.is_valid():
            venue_data = venue_serializer.save()
        else:
            raise serializers.ValidationError(
                handle_errors(venue_serializer.errors))

        '''
       timing
       '''

        if "timing" in validated_data:
            for i in validated_data['timing']:
                i['venue'] = venue_data.id
            timing_serializer = AdminVenueDailyAvailibilitySerializer(
                data=validated_data['timing'], many=True)

            if timing_serializer.is_valid():
                VenueDailyTiming.objects.filter(venue=venue_data.id).delete()
                timing_data = timing_serializer.save()

            else:
                print('timing_serializer.errors--', timing_serializer.errors)
                raise serializers.ValidationError(
                    handle_errors(timing_serializer.errors))

            '''
           Contact Info
           '''
            serialized_data = ContactInfoProfileSerializer(data=validated_data)
            if serialized_data.is_valid():
                if len(serialized_data.validated_data) > 0:
                    contact_obj = VenueOwner.objects.filter(
                        user=validated_data['user'])
                    contact_contentType = ContentType.objects.get_for_model(
                        VenueOwner)
                    ContactInfo.objects.filter(
                        content_type__pk=contact_contentType.id, object_id__in=contact_obj).delete()
                    serialized_data.validated_data['content_object'] = venueowner_data
                    contact_info_data = serialized_data.save()

            else:
                raise serializers.ValidationError(
                    handle_errors(serialized_data.errors))

            '''
            Contact Numbers
            '''
            if "Telephone_number" in validated_data:
                TelephoneNumbers.objects.filter(
                    contact_id=contact_info_data).delete()
                telephone_number = []

                for i in validated_data['Telephone_number']['mobile']:
                    if i == "":
                        pass
                    else:
                        telephone_number.append(i)

                contact_numbers = [TelephoneNumbers(contact_id=contact_info_data,
                                                    contact_number=i
                                                    ) for i in telephone_number]

                TelephoneNumbers.objects.bulk_create(contact_numbers)

            '''
           Contact Person
           '''
            if "contact_person" in validated_data:
                contact_persons = []
                # print('validated-----contat persomn',
                #       validated_data['contact_person'])

                contact_contentType = ContentType.objects.get_for_model(Venue)
                contact_obj = VenueOwner.objects.filter(
                    user=validated_data['user'])

                if Contact_Person.objects.filter(content_type__pk=contact_contentType.id,
                                                 object_id__in=contact_obj).exists():
                    Contact_Person.objects.filter(
                        content_type__pk=contact_contentType.id, object_id__in=contact_obj).delete()

                for i in validated_data['contact_person']:
                    if i['first_name'] == "" and i['last_name'] == "":
                        pass
                    else:
                        contact_persons.append(i)

                serialized_data = ContactPersonSerializer(
                    data=contact_persons, many=True)
                if serialized_data.is_valid():
                    for i in serialized_data.validated_data:
                        i['content_object'] = venueowner_data
                    serialized_data.save()

            '''
            Bank Info
            '''
            if "bank" in validated_data:
                serialized_data = BankInfoProfileSerializer(
                    data=validated_data)
                BankInfo.objects.filter(
                    object_id=venueowner_data.id).update(is_deleted=True)
                if serialized_data.is_valid():
                    serialized_data.validated_data['content_object'] = venueowner_data
                    serialized_data.save()
                else:
                    raise serializers.ValidationError(
                        handle_errors(serialized_data.errors))
            return venueowner_data.id

    class Meta:
        model = VenueOwner
        fields = '__all__'
        extra_kwargs = {
            'email': {"read_only": True},
            'user_code': {"read_only": True},
            'new_email': {"read_only": True},
            'user': {"read_only": True}
        }


class AdminCreateProfessionalPOSTSerializer(serializers.ModelSerializer):
    Telephone_number = serializers.JSONField(
        required=False, help_text='{"mobile":["847123654","35135654"]}')
    iban = serializers.CharField(required=False)
    bank = serializers.CharField(required=False)

    acc_holder_first_name = serializers.CharField(required=False)
    acc_holder_last_name = serializers.CharField(required=False)
    acc_type = serializers.CharField(required=False)
    bank_identification_no = serializers.CharField(required=False)
    iban_certificate = models.FileField(
        upload_to='pdfs/', null=True, blank=True, help_text='IBAN CERTIFICATE')

    contact_person = serializers.JSONField(
        required=False, help_text='[{"first_name":"Test","last_name":"asd","telephone":1234567,"email":"email"},{"first_name":"Test","last_name":"asd","telephone":1234567,"email":"email"}]')
    mobile = serializers.CharField(required=False, help_text="Mobile number")
    email = serializers.CharField(required=False, help_text="Email ID")

    timing = serializers.JSONField(
        required=False, help_text='[{"day":1,"start_time":12:00:00,"is_active":1,"end_time":1:00:00}]')

    ServedAreas = serializers.JSONField(
        required=False, help_text='[{"latitude":00.00, "longitude":00.00, "google_address":"FFF", "place_id":"ppp"}]')
    service = serializers.JSONField(required=False, help_text="[1,2,3]")
    served_areas = serializers.JSONField(required=False, help_text="[1,2,3]")
    payment_method = serializers.IntegerField(
        required=False, help_text="payment method id")
    state = serializers.IntegerField(
        required=False, help_text="state")
    area = serializers.IntegerField(
        required=False, help_text="area")
    city = serializers.IntegerField(
        required=False, help_text="city")
    country = serializers.IntegerField(
        required=False, help_text="country")

    def validate_mobile(self, value):
        if Professional.objects.filter(mobile=value, is_deleted=False).exists():
            # raise serializers.ValidationError("Mobile number is already linked")
            pass

        if len(value) < 10:
            raise serializers.ValidationError(
                "Mobile Number must be filled in.")
        else:
            return value

    @transaction.atomic
    def create(self, validated_data):
        '''
        user
        '''

        if User.objects.filter(username=validated_data['email'],
                               email=validated_data['email']).exists():
            raise serializers.ValidationError("user already exists")
        else:
            Userdata = {}
            Userdata['username'] = validated_data['email']
            Userdata['email'] = validated_data['email']
            Userdata['first_name'] = validated_data['first_name']
            Userdata['last_name'] = validated_data['last_name']
            # Userdata['is_active'] = False

            user_ser = RegisterSerializer(data=Userdata)
            if user_ser.is_valid():
                user_ser.save()
                user = User.objects.get(username=Userdata['username'])
                validated_data['user'] = user.id
            else:
                raise serializers.ValidationError(
                    handle_errors(user_ser.errors))

            my_group = Group.objects.get(name='Professional')
            my_group.user_set.add(user)
        # Convert UUID format to a Python string.
        random = str(uuid.uuid4())

        # Make all characters uppercase.
        random = random.upper()

        # Remove the UUID '-'.
        random = random.replace("-", "")

        # Return the random string.
        user_code = random[0:6]
        validated_data['user_code'] = user_code

        '''
       onboard api 
       '''
        validated_data['created_by'] = validated_data['user']
        validated_data['status'] = 'A'
        validated_data['registeration_type'] = 'P'
        onboard_serializer = OnboardSerializer(data=validated_data)

        if onboard_serializer.is_valid():
            onboard_data = onboard_serializer.save()
        else:
            raise serializers.ValidationError(
                handle_errors(onboard_serializer.errors))

        '''
       Send email to user
       '''

        images = Template_Images.objects.filter().values('template_image')
        logo = images[0]
        logo2 = images[1]

        token_obj = RegisterPrivateKey.objects.create(
            user=user)
        business_name_for_msg = onboard_data.business_name
        registeration_type = "professional"
        verify_redirect = "{}{}".format(
            "https://sportsbookingnew.worldindia.com/#",
            "/" + registeration_type + "/create-profile/set-password/" + str(token_obj.key))
        msg = business_name_for_msg + " has been approved as a Professional" \
                                      ". Your profile is under development. You can logon for first time to the platform changing your password."
        send_emal_onboard_accept(
            onboard_data.first_name, verify_redirect, onboard_data.email, msg, logo, logo2)

        '''
        Profession
        '''

        serialized_data = ProfessionalOnboardSerializer(data=validated_data)

        if serialized_data.is_valid():
            professional_data = serialized_data.save()
            professional_data.served_areas.set(validated_data['served_areas'])
            professional_data.save()
        else:
            raise serializers.ValidationError(
                handle_errors(serialized_data.errors))

        '''
        multiple profession service data
        '''

        for pr in validated_data['service']:
            print(pr)
        if ProfessionalStepServiceData.objects.filter(user=validated_data['user']).exists():
            step_service_data = ProfessionalStepServiceData.objects.get(
                user=validated_data['user'])
            step_service_data.professions.set(validated_data['service'])
            step_service_data.save()

            for pr2 in validated_data['service']:
                if ProfessionalStepSkill.objects.filter(~Q(skil__profession=pr2), created_by=validated_data['user']).exists():
                    ProfessionalStepSkill.objects.filter(
                        ~Q(skil__profession=pr2), created_by=validated_data['user']).delete()

        else:
            dataobj = ProfessionalStepServiceData.objects.create(
                user=user)
            dataobj.professions.set(validated_data['service'])
            dataobj.save()

        '''
        Contact Info
        '''

        serialized_data = ContactInfoProfileSerializer(
            data=validated_data, context={"user": validated_data['user']})
        if serialized_data.is_valid():
            contact_obj = Professional.objects.filter(
                user=validated_data['user'])
            contact_contentType = ContentType.objects.get_for_model(
                Professional)
            ContactInfo.objects.filter(
                content_type__pk=contact_contentType.id, object_id__in=contact_obj).delete()
            serialized_data.validated_data['content_object'] = professional_data
            contact_info_data = serialized_data.save()
        else:

            raise serializers.ValidationError(
                handle_errors(serialized_data.errors))

        '''
        Timing
        '''
        if "timing" in validated_data:

            for i in validated_data['timing']:
                i['professional'] = professional_data.id
            timing_serializer = ProfessionalDailyAvailibilitySerializer(
                data=validated_data['timing'], many=True)

            if timing_serializer.is_valid():
                ProfessionalDailyTiming.objects.filter(
                    professional=professional_data.id).delete()
                timing_data = timing_serializer.save()
            else:
                print(timing_serializer.errors)
                raise serializers.ValidationError(
                    handle_errors(timing_serializer.errors))

        '''
        ServedArea
        '''
        if "ServedAreas" in validated_data:
            for i in validated_data['ServedAreas']:
                # ProfessionalServedArea.objects.filter(professional_id__pk=professional_data.id,professional_id__in=professional_data)
                i['professional'] = professional_data.id
            servearea_serializer = ServedAreaSerializer(
                data=validated_data['ServedAreas'], many=True)

            if servearea_serializer.is_valid():
                ProfessionalServedArea.objects.filter(
                    professional=professional_data.id).delete()
                servarea_data = servearea_serializer.save()
            else:
                print(servearea_serializer.errors)
                raise serializers.ValidationError(
                    handle_errors(servearea_serializer.errors))

        '''
        Contact Person
        '''
        if "contact_person" in validated_data:
            contact_persons = []
            Contact_Person.objects.filter(
                content_type__pk=contact_contentType.id, object_id__in=contact_obj).delete()
            # Contact_Person.objects.filter(content_type__pk=contact_contentType.id,object_id__in=contact_obj).all()

            for i in validated_data['contact_person']:
                if i['first_name'] == "" and i['last_name'] == "":
                    pass
                else:
                    contact_persons.append(i)

            serialized_data = ContactPersonSerializer(
                data=contact_persons, many=True)

            if serialized_data.is_valid():
                for i in serialized_data.validated_data:
                    i['content_object'] = professional_data
                serialized_data.save()
            else:
                raise serializers.ValidationError(
                    handle_errors(serialized_data.errors))

        '''
        Contact Numbers
        '''
        if "Telephone_number" in validated_data:
            TelephoneNumbers.objects.filter(
                contact_id=contact_info_data).delete()
            contact_numbers = [TelephoneNumbers(contact_id=contact_info_data,
                                                contact_number=i
                                                ) for i in validated_data['Telephone_number']['mobile']]

            TelephoneNumbers.objects.bulk_create(contact_numbers)

        '''
        Bank Info
        '''
        if 'bank' in validated_data:
            BankInfo.objects.filter(
                content_type__pk=contact_contentType.id, object_id__in=contact_obj).delete()
            serialized_data = BankInfoProfileSerializer(data=validated_data)
            if serialized_data.is_valid():
                serialized_data.validated_data['content_object'] = professional_data
                serialized_data.save()
            else:
                raise serializers.ValidationError(
                    handle_errors(serialized_data.errors))
        return professional_data.id

    class Meta:
        model = Professional
        fields = '__all__'
        extra_kwargs = {
            'email': {"read_only": True},
            'user_code': {"read_only": True},
            'new_email': {"read_only": True},
            'user': {"read_only": True}
        }


class AdminCreateProfileAdvertiserPOSTSerializers(serializers.ModelSerializer):
    Telephone_number = serializers.JSONField(
        required=False, help_text='{"mobile":["847123654","35135654"]}')
    iban = serializers.CharField(required=False)
    bank = serializers.CharField(required=False)
    acc_holder_first_name = serializers.CharField(required=False)
    acc_holder_last_name = serializers.CharField(required=False)
    contact_person = serializers.JSONField(
        required=False,
        help_text='[{"first_name":"Test","last_name":"asd","telephone":1234567,"email":"email"},{"first_name":"Test","last_name":"asd","telephone":1234567,"email":"email"}]')
    mobile = serializers.CharField(required=False, help_text="Mobile number")
    email = serializers.CharField(required=False, help_text="Email ID")
    timing = serializers.JSONField(
        required=False, help_text='[{"day":1,"start_time":12:00:00,"is_active":1,"end_time":1:00:00}]')
    state = serializers.IntegerField(
        required=False, help_text="state")
    area = serializers.IntegerField(
        required=False, help_text="area")
    city = serializers.IntegerField(
        required=False, help_text="city")
    country = serializers.IntegerField(
        required=False, help_text="country")

    def validate_mobile(self, value):
        if len(value) < 10:
            raise serializers.ValidationError("Mobile number is Invalid")

    @transaction.atomic
    def create(self, validated_data):
        '''
        user
        '''

        if User.objects.filter(username=validated_data['email'],
                               email=validated_data['email']).exists():
            raise serializers.ValidationError("user already exists")
        else:
            Userdata = {}
            Userdata['username'] = validated_data['email']
            Userdata['email'] = validated_data['email']
            Userdata['first_name'] = validated_data['first_name']
            Userdata['last_name'] = validated_data['last_name']
            # Userdata['is_active'] = False

            user_ser = RegisterSerializer(data=Userdata)
            if user_ser.is_valid():
                user_ser.save()
                user = User.objects.get(username=Userdata['username'])
                validated_data['user'] = user.id
            else:
                raise serializers.ValidationError(
                    handle_errors(user_ser.errors))

            my_group = Group.objects.get(name='Advertiser')
            my_group.user_set.add(user)
        # Convert UUID format to a Python string.
        random = str(uuid.uuid4())

        # Make all characters uppercase.
        random = random.upper()

        # Remove the UUID '-'.
        random = random.replace("-", "")

        # Return the random string.
        user_code = random[0:6]
        validated_data['user_code'] = user_code

        '''
       onboard api 
       '''
        validated_data['created_by'] = validated_data['user']
        validated_data['status'] = 'A'
        validated_data['registeration_type'] = 'A'
        onboard_serializer = OnboardSerializer(data=validated_data)

        if onboard_serializer.is_valid():
            onboard_data = onboard_serializer.save()
        else:
            raise serializers.ValidationError(
                handle_errors(onboard_serializer.errors))

        '''
       Send email to user
       '''
        images = Template_Images.objects.filter().values('template_image')
        logo = images[0]
        logo2 = images[1]

        token_obj = RegisterPrivateKey.objects.create(
            user=user)
        business_name_for_msg = onboard_data.business_name
        registeration_type = "advertiser"
        verify_redirect = "{}{}".format(
            "https://sportsbookingnew.worldindia.com/#",
            "/" + registeration_type + "/create-profile/set-password/" + str(token_obj.key))
        msg = business_name_for_msg + " has been approved as a Advertiser" \
                                      ". Your profile is under development. You can logon for first time to the platform changing your password."
        send_emal_onboard_accept(
            onboard_data.first_name, verify_redirect, onboard_data.email, msg, logo, logo2)

        '''
        Advertiser
        '''

        # validated_data['email'] = self.context['user'].email
        # validated_data['user'] = self.context['user'].id
        serialized_data = AdvertiserOnboardSerializer(data=validated_data)
        if serialized_data.is_valid():
            advertiser_data = serialized_data.save()
            print(advertiser_data)
        else:
            raise serializers.ValidationError(
                handle_errors(serialized_data.errors))

        '''
        Contact Info
        '''
        serialized_data = ContactInfoProfileSerializer(data=validated_data)
        if serialized_data.is_valid():
            if len(serialized_data.validated_data) > 0:
                contact_obj = Advertiser.objects.filter(
                    user=validated_data['user'])
                contact_contentType = ContentType.objects.get_for_model(
                    Advertiser)
                # ContactInfo.objects.filter(content_type__pk=contact_contentType.id,object_id__in=contact_obj).delete()
                serialized_data.validated_data['content_object'] = advertiser_data
                contact_info_data = serialized_data.save()
        else:
            raise serializers.ValidationError(
                handle_errors(serialized_data.errors))

        '''
        Contact Numbers
        '''
        if "Telephone_number" in validated_data:
            contact_numbers = [TelephoneNumbers(contact_id=contact_info_data,
                                                contact_number=i
                                                ) for i in validated_data['Telephone_number']['mobile']]
            TelephoneNumbers.objects.bulk_create(contact_numbers)
            # return 'success'

        '''
        Timing
        '''
        if "timing" in validated_data:
            timing = []
            for i in validated_data['timing']:
                timing.append(i)
                i['advertiser'] = advertiser_data.id
            timing_serializer = AdvertiserDailyAvailibilitySerializer(
                data=timing, many=True)
            if timing_serializer.is_valid():
                AdvertiserDailyTiming.objects.filter(
                    advertiser=advertiser_data.id).delete()
                timing_data = timing_serializer.save()
            else:
                print(timing_serializer.errors)
                raise serializers.ValidationError(
                    handle_errors(timing_serializer.errors))

        '''
        Contact Person
        '''

        if "contact_person" in validated_data:
            contact_persons = []
            Contact_Person.objects.filter(
                content_type__pk=contact_contentType.id, object_id__in=contact_obj).delete()

            for i in validated_data['contact_person']:
                if i['first_name'] == "" and i['last_name'] == "":
                    pass
                else:
                    contact_persons.append(i)

            serialized_data = ContactPersonSerializer(
                data=contact_persons, many=True)
            if serialized_data.is_valid():
                for i in serialized_data.validated_data:
                    i['content_object'] = advertiser_data
                    serialized_data.save()
            else:
                raise serializers.ValidationError(
                    handle_errors(serialized_data.errors))

        '''
        Bank Info
        '''
        if "bank" in validated_data:
            serialized_data = BankInfoProfileSerializer(data=validated_data)
            if serialized_data.is_valid():
                serialized_data.validated_data['content_object'] = advertiser_data
                serialized_data.save()
            else:
                raise serializers.ValidationError(
                    handle_errors(serialized_data.errors))
        return advertiser_data.id

    class Meta:
        model = Advertiser
        fields = '__all__'
        extra_kwargs = {
            'email': {'read_only': True},
            'user_code': {"read_only": True},
            'new_email': {"read_only": True},
            'user': {'read_only': True}
        }


class AdminPackagesPostSerializer(serializers.ModelSerializer):
    venueowner_id = serializers.IntegerField(
        required=False, help_text="ID of the venue owner")
    package_id = serializers.IntegerField(required=False)
    level = serializers.JSONField(required=False, help_text="Level IDs[1,2,3]")

    def validate_venueowner_id(self, value):
        if VenueOwner.objects.filter(id=value).exists():
            return value
        else:
            raise serializers.FieldDoesNotExist("No Such Record")

    class Meta:
        model = Packages
        fields = '__all__'


class AdminPackagesGetSerializer(serializers.ModelSerializer):
    venueowner_id = serializers.IntegerField(
        required=False, help_text="ID of the venue owner")
    sport_relivance = serializers.CharField(
        required=False, help_text="Sport ID")
    sport_name = serializers.CharField(
        required=False, read_only=True, source="sport.name")
    sport_id = serializers.CharField(
        required=False, read_only=True, source="sport_relivance.id")
    keyword = serializers.CharField(
        required=False, help_text="Search Text box")
    page = serializers.IntegerField(
        required=False, default=1, help_text="Page number. Default = 1")
    package_type = serializers.CharField(
        required=False, )
    package_type_id = serializers.CharField(
        required=False, read_only=True, source="package_type.id")
    package_duration = serializers.CharField(required=False)
    package_duration_id = serializers.CharField(
        required=False, read_only=True, source="package_duration.id")

    status_name = serializers.CharField(
        required=False, write_only=True, help_text="UNDER APPROVAL/ON GOING/WAITING/REJECTED")
    package_availabiity = serializers.NullBooleanField(
        required=False, help_text="Package Availability")

    def validate_venueowner_id(self, value):
        if VenueOwner.objects.filter(id=value).exists():
            return value
        else:
            raise serializers.FieldDoesNotExist("No Such Record")

    class Meta:
        model = Packages
        fields = '__all__'
        read_only_fields = ['package_image', 'package_description',
                            'number_sessions', 'session_duration', 'package_quantity', 'level', 'price', 'name']


class AdminProfileServiceSerializer(serializers.ModelSerializer):
    venueowner_id = serializers.IntegerField(required=False)
    all_services = serializers.JSONField(required=False, help_text="[1,2,3]")
    covid19Measure_description = serializers.JSONField(
        required=False, write_only=True, help_text="covid19Measure_description ID-[1,2,3]")

    def validate_all_services(self, value):
        for item in value:
            if not VenueService.objects.filter(id=item).exists():
                raise serializers.FieldDoesNotExist(
                    "No Record with given ID in venue services")
        return value

    def validate_covid19Measure_description(self, value):
        for item in value:
            if not Covid19Measures.objects.filter(id=item).exists():
                raise serializers.FieldDoesNotExist(
                    "No Record with given ID in Covid19Measures")
        return value

    class Meta:
        model = Venue
        fields = '__all__'
        read_only_fields = ['name', 'logo', 'main_image', 'business_name', 'professional_allowed', 'all_sports',
                            'allowed_entry', 'description', 'street', 'pincode', 'country', 'city',
                            'state', 'area', 'latitude', 'longitude', 'google_address', 'place_id', 'venue_number',
                            'buisness_vat', 'tax_offices', 'working_hours', 'is_deleted', 'created_by', 'created_at',
                            'updated_at']


class AdminPaidServiceSerializer(serializers.ModelSerializer):
    venueowner_id = serializers.IntegerField(required=False)
    venue_service = serializers.JSONField(
        required=False, write_only=True, help_text="venue services ID-[1,2,3]")

    class Meta:
        model = VenueServiceCost
        fields = '__all__'
        read_only_fields = ['venue', 'price_per_hour']


class AdminVenueFieldPOSTServicesSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(
        required=True, help_text="ID of the venue owner")
    venue_field = serializers.IntegerField(
        required=True, help_text="Venue Field ID")
    services = serializers.JSONField(
        required=False, write_only=True,
        help_text='[{"venue_service":1,"price_per_hour":50},{"venue_service":2,"price_per_hour":60}]')

    def validate_id(self, value):
        if VenueOwner.objects.filter(id=value).exists():
            return value
        else:
            raise serializers.FieldDoesNotExist("No Such Record")

    def validate_services(self, data):

        for i in range(len(data)):
            if VenueService.objects.filter(id=data[i]['venue_service']).exists():
                data[i]['venue_service'] = VenueService.objects.get(
                    id=data[i]['venue_service'])
            else:
                raise serializers.FieldDoesNotExist(
                    "No record with ID " + str(data[i]['venue_service']))
        return data

    def create(self, validated_data):

        # print(self.context['id'])

        venue_owner_obj = VenueOwner.objects.get(id=self.context['id'])
        validated_data['user'] = venue_owner_obj.user

        # print(validated_data['user'])

        attributes = []

        venue_field_obj = VenueField.objects.filter(
            id=validated_data['venue_field']).first()

        venue_field_service_object = VenueFieldService.objects.filter(
            venue_field=validated_data['venue_field'])
        venue_field_service_object.delete()

        if 'services' in validated_data:
            attributes.append("services")
            services = validated_data['services']
            validated_data.pop('services')

        validated_data['venue_field'] = venue_field_obj

        return_data = {}

        if 'services' in attributes:
            venue_fieldService = [VenueFieldService(
                **i,
                venue_field=venue_field_obj,
                created_by=validated_data['user']
            ) for i in services]

            service_obj = VenueFieldService.objects.bulk_create(
                venue_fieldService)
            services_class = VenueFieldServicePreviewSerializer(
                service_obj, many=True)
            return_data['VenueFieldService'] = services_class.data

        return return_data

    class Meta:
        model = VenueFieldService
        fields = '__all__'
        read_only_fields = ['venue_service',
                            'price_per_hour', 'is_deleted', 'created_by']


class IdSerializer(serializers.Serializer):
    id = serializers.IntegerField(required=True, help_text="Id of venue owner")

    def validate_id(self, value):
        if VenueOwner.objects.filter(id=value).exists():
            return value
        else:
            raise serializers.FieldDoesNotExist("No Such Record")


class AdminVenuePOSTServicesSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(required=True, help_text="Id of venue owner")
    services = serializers.JSONField(
        required=True, write_only=True,
        help_text='[{"venue_service":1,"price_per_hour":50},{"venue_service":2,"price_per_hour":60}]')

    def validate_id(self, value):
        if VenueOwner.objects.filter(id=value).exists():
            return value
        else:
            raise serializers.FieldDoesNotExist("No Such Record")

    def validate_services(self, data):

        for i in range(len(data)):
            if VenueService.objects.filter(id=data[i]['venue_service']).exists():
                data[i]['venue_service'] = VenueService.objects.get(
                    id=data[i]['venue_service'])
            else:
                raise serializers.FieldDoesNotExist(
                    "No record with ID " + str(data[i]['venue_service']))
        return data

    def create(self, validated_data):

        # print(self.context['id'])

        venue_owner_obj = VenueOwner.objects.get(id=self.context['id'])
        validated_data['user'] = venue_owner_obj.user

        attributes = []

        venue_obj = Venue.objects.filter(
            created_by=validated_data['user']).first()

        venue_service_cost_object = VenueServiceCost.objects.filter(
            venue=venue_obj.id)
        venue_service_cost_object.delete()

        if 'services' in validated_data:
            attributes.append("services")
            services = validated_data['services']
            validated_data.pop('services')

        validated_data['venue'] = venue_obj

        return_data = {}

        if 'services' in attributes:
            venue_Service_cost = [VenueServiceCost(
                **i,
                venue=venue_obj
            ) for i in services]

            service_obj = VenueServiceCost.objects.bulk_create(
                venue_Service_cost)
            services_class = VenueSAVESerializer(
                service_obj, many=True)
            return_data['VenueService'] = services_class.data

        return return_data

    class Meta:
        model = VenueServiceCost
        fields = '__all__'
        read_only_fields = ['venue', 'venue_service',
                            'price_per_hour', 'is_deleted', 'created_by']


class AdminVenueFieldPaidServiceSerializer(serializers.ModelSerializer):
    venueowner_id = serializers.IntegerField(required=True,
                                             help_text="ID of the venue owner")
    venue_field = serializers.IntegerField(
        required=True, help_text="Venue Field ID")

    def validate_venueowner_id(self, value):
        if VenueOwner.objects.filter(id=value).exists():
            return value
        else:
            raise serializers.FieldDoesNotExist("No Such Record")

    class Meta:
        model = VenueFieldService
        fields = '__all__'
        read_only_fields = ['is_deleted', 'price_per_hour',
                            'venue_service', 'created_by']


class AdminVenueFieldGETSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(required=True,
                                  help_text="ID of the venue owner")

    def validate_id(self, value):
        if VenueOwner.objects.filter(id=value).exists():
            return value
        else:
            raise serializers.FieldDoesNotExist("No Such Record")

    class Meta:
        model = VenueField
        fields = '__all__'
        read_only_fields = ['venue', 'venue_service',
                            'price_per_hour', 'is_deleted', 'created_by', 'name', 'main_image', 'field_attributes',
                            'field_type', 'sport', 'seat_capacity', 'floor_type', 'status', 'facility_type', 'lights',
                            'description']


class DocumentSerializer(serializers.ModelSerializer):
    Uploaded_by = serializers.CharField(
        source='created_by.username', read_only=True)

    class Meta:
        model = Document
        fields = "__all__"


class DocumentDeleteSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(required=True)

    class Meta:
        model = Document
        fields = "__all__"
        extra_kwargs = {
            'document_name': {'required': False},
        }


class AdminScheduleTemplateGETLISTSerializer(serializers.ModelSerializer):
    venueowner_id = serializers.IntegerField(
        required=False, help_text="ID of the venue owner")
    assortment = serializers.CharField(required=False, help_text="Field Name")
    page = serializers.IntegerField(
        required=False, default=1, help_text="Page number")
    keyword = serializers.CharField(
        required=False, write_only=True, help_text="Keyword")
    in_use = serializers.SerializerMethodField('is_in_use')
    filter_in_use = serializers.CharField(
        required=False, write_only=True, help_text="yes/no")

    def validate_venueowner_id(self, value):
        if VenueOwner.objects.filter(id=value).exists():
            return value
        else:
            raise serializers.FieldDoesNotExist("No Such Record")

    def is_in_use(self, obj):
        if Schedule.objects.filter(template__id=obj.id).exists():
            return "yes"
        else:
            return "no"

    class Meta:
        model = ScheduleTemplateStep1
        fields = '__all__'
        read_only_fields = ['template_Name', 'interval',
                            'gap_period', 'plan_range', 'day', 'is_available']


class AdminScheduleTemplateLISTDELETESerializer(serializers.Serializer):
    id = serializers.IntegerField(
        required=False, help_text="ID of the venue owner")
    template_id = serializers.IntegerField(
        required=False, help_text="ID of the TemplateList")

    def validate_id(self, value):
        if VenueOwner.objects.filter(id=value).exists():
            return value
        else:
            raise serializers.FieldDoesNotExist("No Such Record")

    def validate_template_id(self, value):
        if ScheduleTemplateStep1.objects.filter(id=value,
                                                created_by=VenueOwner.objects.get(id=self.context['id']).user).exists():
            return value
        else:
            raise serializers.FieldDoesNotExist("No Record Found")

    @transaction.atomic
    def update(self, instance, validated_data):
        instance.is_deleted = True
        instance.save()
        ScheduleTemplateSlot.objects.filter(template=instance).delete()
        return instance


class AdminScheduleTemplateStep1POSTSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(
        required=False, help_text="ID of the venue owner")
    slots = serializers.JSONField(
        required=False,
        help_text='each slot [{ "start_hour": "10:00", "end_hour": "11:00","day":1,"is_available":True }, { "start_hour": "11:00","end_hour": "12:00","day":1,"is_available":True}]')

    def validate_id(self, value):
        if VenueOwner.objects.filter(id=value).exists():
            return value
        else:
            raise serializers.FieldDoesNotExist("No Such Record")

    def validate_Template_Name(self, value, ):
        if ScheduleTemplateStep1.objects.filter(Template_Name=value,
                                                created_by=VenueOwner.objects.get(id=self.context['id'].user)).exists():
            raise serializers.ImproperlyConfigured(
                "Template Name already exists")
        else:
            return value

    @transaction.atomic
    def create(self, validated_data, **kwargs):

        vobj = VenueOwner.objects.get(id=self.context['id'])
        print(vobj)

        slots = validated_data['slots']
        validated_data.pop('slots')
        validated_data.pop('id')

        schedule_template = ScheduleTemplateStep1(
            **validated_data, created_by=vobj.user)
        schedule_template.save()

        data = [ScheduleTemplateSlot(
            **i,
            template=schedule_template
        ) for i in slots]
        ScheduleTemplateSlot.objects.bulk_create(data)
        return "Success"

    class Meta:
        model = ScheduleTemplateStep1
        fields = '__all__'


class AdminScheduleTemplate1GETSerializer(serializers.ModelSerializer):
    venueowner_id = serializers.IntegerField(
        required=False, help_text="ID of the venue owner")
    id = serializers.IntegerField(
        required=False, help_text="Id of the schedule Template")

    def validate_venueowner_id(self, value):
        if VenueOwner.objects.filter(id=value).exists():
            return value
        else:
            raise serializers.FieldDoesNotExist("No Such Record")

    def validate_Template_Name(self, value):
        if ScheduleTemplateStep1.objects.filter(Template_Name=value, created_by=VenueOwner.objects.get(
                id=self.context['venueowner_id']).user).exists():
            return value
        else:
            raise serializers.FieldDoesNotExist("No Record with TemplateName")

    def validate_id(self, value):
        if ScheduleTemplateStep1.objects.filter(id=value, created_by=VenueOwner.objects.get(
                id=self.context['venueowner_id']).user).exists():
            return value
        else:
            raise serializers.FieldDoesNotExist("No Record with ID")

    def __init__(self, *args, **kwargs):
        super(AdminScheduleTemplate1GETSerializer,
              self).__init__(*args, **kwargs)
        for field in self.fields:
            if field != 'Template_Name' and field != 'id' and field != 'venueowner_id':
                self.fields[field].read_only = True

    class Meta:
        model = ScheduleTemplateStep1
        fields = '__all__'


class AdminScheduleTemplateSlotsSerialzier(serializers.ModelSerializer):
    class Meta:
        model = ScheduleTemplateSlot
        fields = '__all__'


class AdminScheduleTemplateStep1UPDATESerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(
        required=False, help_text="ID of the venue owner")
    template_id = serializers.IntegerField(
        required=False, help_text='Insert ID to update')
    # created_by = serializers.IntegerField(required=False)
    slots = serializers.JSONField(
        required=False,
        help_text='each slot [{ "start_hour": "10:00", "end_hour": "11:00","day":1,"is_available":True }, { "start_hour": "11:00","end_hour": "12:00","day":1,"is_available":True}]')

    def validate_id(self, value):
        if VenueOwner.objects.filter(id=value).exists():
            return value
        else:
            raise serializers.FieldDoesNotExist("No Such Record")

    class Meta:
        model = ScheduleTemplateStep1
        fields = '__all__'


class AdminScheduleTemplateLISTWithoutPaginationSerializer(serializers.ModelSerializer):
    venueowner_id = serializers.IntegerField(
        required=False, help_text="ID of the venue owner")
    assortment = serializers.CharField(required=False, help_text="Field Name")
    keyword = serializers.CharField(
        required=False, write_only=True, help_text="Keyword")

    def validate_venueowner_id(self, value):
        if VenueOwner.objects.filter(id=value).exists():
            return value
        else:
            raise serializers.FieldDoesNotExist("No Such Record")

    class Meta:
        model = ScheduleTemplateStep1
        fields = '__all__'
        read_only_fields = ['template_Name', 'interval',
                            'gap_period', 'plan_range', 'day', 'is_available']


class AdminScheduleLISTSerializer(serializers.ModelSerializer):
    venueowner_id = serializers.IntegerField(
        required=False, help_text="ID of the venue owner")
    assortment = serializers.CharField(
        required=False, help_text="start_date / ")
    start_date = serializers.CharField(
        required=False, help_text="starting date")
    end_date = serializers.CharField(
        required=False, help_text="ending date")
    active = serializers.CharField(
        required=False, help_text="Inuse - Yes / No")
    keyword = serializers.CharField(
        required=False, help_text="Search keywords")
    page = serializers.IntegerField(
        required=False, default=1, help_text="Page number")

    def validate_venueowner_id(self, value):
        if VenueOwner.objects.filter(id=value).exists():
            return value
        else:
            raise serializers.FieldDoesNotExist("No Such Record")

    class Meta:
        model = Schedule
        fields = '__all__'
        read_only_fields = ['schedule_name', 'start_date', 'template',
                            'pricelist', 'start_date', 'end_date', 'created_by']


class AdminScheduleLISTDELETESerializer(serializers.Serializer):
    id = serializers.IntegerField(
        required=False, help_text="ID of the venue owner")
    schedule_id = serializers.IntegerField(
        required=False, help_text="ID of the Schedule")

    def validate_id(self, value):
        if VenueOwner.objects.filter(id=value).exists():
            return value
        else:
            raise serializers.FieldDoesNotExist("No Such Record")

    def validate_schedule_id(self, value):
        if Schedule.objects.filter(id=value, created_by=VenueOwner.objects.get(id=self.context['id']).user).exists():
            return value
        else:
            raise serializers.FieldDoesNotExist("No Record Found")

    @transaction.atomic
    def update(self, instance, validated_data):
        instance.is_deleted = True
        instance.save()
        # ScheduleTemplateSlot.objects.filter(template=instance).delete()
        return instance


class AdminScheduleGETSerializer(serializers.Serializer):
    venueowner_id = serializers.IntegerField(
        required=False, help_text="ID of the venue owner")
    Template_Id = serializers.IntegerField(required=False)
    Pricelist_Id = serializers.IntegerField(required=False)
    start_date = serializers.DateField(required=False)
    id = serializers.IntegerField(
        required=False, help_text="While editing Schedule")
    time_slot_price = serializers.JSONField(
        required=False, help_text='[{"start_hour":"13:00:00","end_hour":"15:00:00","cost":30}]')

    def validate_venueowner_id(self, value):
        if VenueOwner.objects.filter(id=value).exists():
            return value
        else:
            raise serializers.FieldDoesNotExist("No Such Record")

    def validate_Template_Id(self, value):
        if ScheduleTemplateStep1.objects.filter(id=value, created_by=VenueOwner.objects.get(
                id=self.context['venueowner_id']).user).exists():
            return value
        else:
            raise serializers.FieldDoesNotExist("No Template with ID")

    def validate_Pricelist_Id(self, value):
        if PriceListStep1.objects.filter(id=value).exists():
            return value
        else:
            raise serializers.FieldDoesNotExist("No Pricelist with ID")

    def validate_id(self, value):
        if Schedule.objects.filter(id=value,
                                   created_by=VenueOwner.objects.get(id=self.context['venueowner_id']).user).exists():
            return value
        else:
            raise serializers.FieldDoesNotExist("No Schedule with ID")


class AdminSchedulePOSTSerializer(serializers.ModelSerializer):
    venueowner_id = serializers.IntegerField(
        required=False, help_text="ID of the venue owner")
    id = serializers.IntegerField(required=False, help_text="Id if edit")
    slots = serializers.JSONField(
        required=True,
        help_text='[{"start_hour":"13:00:00","end_hour":"15:00:00","date":"2020-08-12","day":"Tuesday","is_available":"True","cost":30}]')
    confirmation = serializers.JSONField(
        required=False, help_text='[{"id":2},{"id":3}]')
    end_date = serializers.CharField(required=False, help_text='Do not send')
    time_slot_price = serializers.JSONField(
        required=False, help_text='[{"start_hour":"13:00:00","end_hour":"15:00:00","cost":30}]')

    def validate_venueowner_id(self, value):
        if VenueOwner.objects.filter(id=value).exists():
            return value
        else:
            raise serializers.FieldDoesNotExist("No Such Record")

    @transaction.atomic
    def create(self, validated_data):
        print(validated_data)
        print(self.context['venueowner_id'])

        venue_owner_obj = VenueOwner.objects.get(
            id=self.context['venueowner_id'])
        # validated_data['user'] = venue_owner_obj.user
        print(venue_owner_obj.user)

        slots = validated_data['slots']
        validated_data.pop("slots")
        if 'time_slot_price' in validated_data:
            time_slots = validated_data['time_slot_price']
            validated_data.pop("time_slot_price")
        else:
            time_slots = []
        schedule = Schedule(**validated_data, created_by=venue_owner_obj.user)
        schedule.save()
        email_cron_obj = Schedule_Email_Cron.objects.create(
            schedule_id=schedule)
        email_cron_obj.schedule_end_date = schedule.end_date
        email_cron_obj.run_date = datetime.datetime.strptime(email_cron_obj.schedule_end_date, "%Y-%m-%d").date() - \
            datetime.timedelta(days=5)
        email_cron_obj.save()

        for i in slots:
            i['day'] = DayofWeek.objects.get(dayofweek=i['day'])

        slots = [Schedule_Slot(
            **i,
            schedule_id=schedule
        ) for i in slots]

        if len(time_slots) > 0:
            tslots = [Time_Slot_Price(**j, schedule_id=schedule)
                      for j in time_slots]
            Time_Slot_Price.objects.bulk_create(tslots)

        Schedule_Slot.objects.bulk_create(slots)
        return True

    class Meta:
        model = Schedule
        fields = "__all__"
        extra_kwargs = {
            "schedule_name": {
                "required": False
            },
            "field": {
                "required": False
            }
        }


class AdminScheduleEditGetSerializer(serializers.Serializer):
    venueowner_id = serializers.IntegerField(
        required=False, help_text="ID of the venue owner")
    id = serializers.IntegerField(required=True)

    def validate_venueowner_id(self, value):
        if VenueOwner.objects.filter(id=value).exists():
            return value
        else:
            raise serializers.FieldDoesNotExist("No Such Record")


class AdminScheduleEditPOSTSerializer(serializers.ModelSerializer):
    venueowner_id = serializers.IntegerField(
        required=False, help_text="ID of the venue owner")
    id = serializers.IntegerField(required=True, help_text="Id if edit")
    slots = serializers.JSONField(
        required=True,
        help_text='[{"start_hour":"13:00:00","end_hour":"15:00:00","date":"2020-08-12","day":"Tuesday","is_available":"True","cost":30}]')

    def validate_venueowner_id(self, value):
        if VenueOwner.objects.filter(id=value).exists():
            return value
        else:
            raise serializers.FieldDoesNotExist("No Such Record")

    @transaction.atomic
    def create(self, validated_data):
        # print(validated_data)
        # print(self.context['venueowner_id'])

        venue_owner_obj = VenueOwner.objects.get(
            id=self.context['venueowner_id'])

        slots = validated_data['slots']
        validated_data.pop("slots")
        schedule = Schedule.objects.get(
            id=validated_data['id'], created_by=venue_owner_obj.user)
        '''schedule = Schedule(**validated_data, created_by=self.context['user'])
        schedule.save()'''
        Schedule_Slot.objects.filter(schedule_id=schedule).delete()
        for i in slots:
            i['day'] = DayofWeek.objects.get(dayofweek=i['day'])

        slots = [Schedule_Slot(
            **i,
            schedule_id=schedule
        ) for i in slots]

        Schedule_Slot.objects.bulk_create(slots)
        return True

    class Meta:
        model = Schedule
        fields = "__all__"


class AdminScheduleEditPOST2Serializer(serializers.ModelSerializer):
    venueowner_id = serializers.IntegerField(
        required=False, help_text="ID of the venue owner")
    id = serializers.IntegerField(required=True, help_text="Id if edit")
    slots = serializers.JSONField(
        required=True,
        help_text='[{"start_hour":"13:00:00","end_hour":"15:00:00","date":"2020-08-12","day":"Tuesday","is_available":"True","cost":30}]')
    time_slot_price = serializers.JSONField(
        required=False, help_text='[{"start_hour":"13:00:00","end_hour":"15:00:00","cost":30}]')

    def validate_venueowner_id(self, value):
        if VenueOwner.objects.filter(id=value).exists():
            return value
        else:
            raise serializers.FieldDoesNotExist("No Such Record")

    def create(self, validated_data):
        print(validated_data)
        print(self.context['venueowner_id'])

        venue_owner_obj = VenueOwner.objects.get(
            id=self.context['venueowner_id'])

        slots = validated_data['slots']
        validated_data.pop("slots")
        if 'time_slot_price' in validated_data:
            time_slots = validated_data['time_slot_price']
            validated_data.pop("time_slot_price")
        else:
            time_slots = []

        schedule = Schedule.objects.get(
            id=validated_data['id'], created_by=venue_owner_obj.user)
        schedule.schedule_name = validated_data['schedule_name']
        schedule.template = validated_data['template']
        schedule.field = validated_data['field']
        schedule.start_date = validated_data['start_date']
        schedule.end_date = validated_data['end_date']
        schedule.save()
        # Email_cron
        email_cron_obj = Schedule_Email_Cron.objects.filter(
            schedule_id=schedule).first()
        print(email_cron_obj)
        email_cron_obj.schedule_end_date = schedule.end_date
        email_cron_obj.run_date = email_cron_obj.schedule_end_date - \
            datetime.timedelta(days=5)
        email_cron_obj.save()

        Schedule_Slot.objects.filter(schedule_id=schedule).delete()
        Time_Slot_Price.objects.filter(schedule_id=schedule).delete()
        for i in slots:
            i['day'] = DayofWeek.objects.get(dayofweek=i['day'])

        slots = [Schedule_Slot(
            **i,
            schedule_id=schedule
        ) for i in slots]

        Schedule_Slot.objects.bulk_create(slots)
        if len(time_slots) > 0:
            tslots = [Time_Slot_Price(**j, schedule_id=schedule)
                      for j in time_slots]
            Time_Slot_Price.objects.bulk_create(tslots)

        return True

    class Meta:
        model = Schedule
        fields = "__all__"


class AdminScheduleGetFieldSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(
        required=False, help_text="ID of the venue owner")

    def validate_id(self, value):
        if VenueOwner.objects.filter(id=value).exists():
            return value
        else:
            raise serializers.FieldDoesNotExist("No Such Record")

    class Meta:
        model = VenueField
        fields = '__all__'
        # fields = ['id', 'name']
        read_only_fields = ['venue', 'venue_service',
                            'price_per_hour', 'is_deleted', 'created_by', 'name', 'main_image', 'field_attributes',
                            'field_type', 'sport', 'seat_capacity', 'floor_type', 'status', 'facility_type', 'lights',
                            'description']


class AdminCampaignPart1SerializerGET(serializers.ModelSerializer):
    venueowner_id = serializers.IntegerField(
        required=False, help_text="ID of the venue owner")
    created_by = serializers.CharField(
        source='created_by.username', read_only=True)
    id = serializers.IntegerField(required=False)

    def validate_venueowner_id(self, value):
        if VenueOwner.objects.filter(id=value).exists():
            return value
        else:
            raise serializers.FieldDoesNotExist("No Such Record")

    def validate_id(self, value):
        if id:
            if CampaignPart1.objects.filter(id=value, created_by=VenueOwner.objects.get(
                    id=self.context['venueowner_id']).user).exists():
                return value
            else:
                return serializers.ValidationError("No Such id")

    class Meta:
        model = CampaignPart1
        fields = '__all__'
        extra_kwargs = {
            "name": {
                "required": False
            }
        }


class AdminCampaignPart1POSTSerializer(serializers.ModelSerializer):
    venueowner_id = serializers.IntegerField(
        required=False, help_text="ID of the venue owner")
    id = serializers.IntegerField(required=False)

    def validate_venueowner_id(self, value):
        if VenueOwner.objects.filter(id=value).exists():
            return value
        else:
            raise serializers.FieldDoesNotExist("No Such Record")

    def create(self, validated_data):
        if validated_data['email'] == True or validated_data['sms'] == True:
            validated_data['status'] = 'A'
        else:
            validated_data['status'] = 'O'

        validated_data['sport'] = []
        data = CampaignPart1.objects.create(name=validated_data['name'], email=validated_data['email'],
                                            sms=validated_data['sms'], banner=validated_data['banner'],
                                            paid_ads=validated_data['paid_ads'], comment=validated_data.get(
            'comment', None), status=validated_data['status'], created_by=validated_data['created_by'])
        # data.sport.set(validated_data['sport'])
        # data.save()
        return data

    class Meta:
        model = CampaignPart1
        fields = '__all__'
        extra_kwargs = {
            "name": {
                "required": False
            }
        }


class AdminCampaignPart2SerializerGET(serializers.ModelSerializer):
    venueowner_id = serializers.IntegerField(
        required=False, help_text="ID of the venue owner")
    created_by = serializers.CharField(
        source='created_by.username', read_only=True)
    id = serializers.IntegerField(required=False)

    def validate_venueowner_id(self, value):
        if VenueOwner.objects.filter(id=value).exists():
            return value
        else:
            raise serializers.FieldDoesNotExist("No Such Record")

    def validate_id(self, value):
        if value:
            if CampaignPart2.objects.filter(id=value, created_by=VenueOwner.objects.get(
                    id=self.context['venueowner_id']).user).exists():
                return value
            else:
                return serializers.ValidationError("No Such ID")

    def __init__(self, *args, **kwargs):
        super(AdminCampaignPart2SerializerGET, self).__init__(*args, **kwargs)
        for field in self.fields:
            if field == 'id':
                self.fields[field].read_only = False
            elif field == 'campaign1':
                self.fields[field].read_only = False
            elif field == 'venueowner_id':
                self.fields[field].read_only = False
            else:
                self.fields[field].read_only = True

    class Meta:
        model = CampaignPart2
        fields = '__all__'
        # fields = ('id', 'notification',   'campaign1',
        #           'created_by', 'created_at', 'is_deleted', 'user_notification', 'sports', 'Areas', 'professional_service', 'venueowner_id')


class AdminCampaignPart2POSTSerializer(serializers.ModelSerializer):
    venueowner_id = serializers.IntegerField(
        required=False, help_text="ID of the venue owner")
    campaign1 = AdminCampaignPart1SerializerGET(read_only=True)
    part1ID = serializers.IntegerField(write_only=True)
    id = serializers.IntegerField(required=False)
    user_notification = serializers.JSONField(
        write_only=True, required=False, help_text="[1,2,3,4]")

    def validate_venueowner_id(self, value):
        if VenueOwner.objects.filter(id=value).exists():
            return value
        else:
            raise serializers.FieldDoesNotExist("No Such Record")

    def create(self, validated_data):
        print(validated_data)
        if 'sports' in validated_data:
            validated_data.pop('sports')
        if 'Areas' in validated_data:
            validated_data.pop('Areas')
        if 'professional_service' in validated_data:
            validated_data.pop('professional_service')
        validated_data['campaign1'] = CampaignPart1.objects.get(
            id=validated_data['part1ID'])
        validated_data.pop('part1ID')
        validated_data.pop('venueowner_id')
        if 'user_notification' in validated_data:
            tp = []
            for i in validated_data['user_notification']:
                player_obj = Player.objects.filter(id=int(i))
                if player_obj.exists():
                    tp.append(player_obj.last().user.id)
                # validated_data['user_notification'] = player_obj.last().user
                else:
                    raise serializers.ValidationError("No Such User ")
            validated_data.pop('user_notification')
            data = CampaignPart2.objects.create(**validated_data)
            data.user_notification.set(tp)
            data.save()
            return data
        else:
            data = CampaignPart2.objects.create(**validated_data)
            return data

        # data = CampaignPart2.objects.create(**validated_data)
        # print(data)
        # data.professional_service.set(prof_serv)
        # return data

    class Meta:
        model = CampaignPart2
        fields = '__all__'


class AdminGetPlayerListSerializer(serializers.ModelSerializer):
    venueowner_id = serializers.IntegerField(
        required=False, help_text="ID of the venue owner")
    city = serializers.CharField(
        required=False, help_text="city", write_only=True)
    street = serializers.CharField(
        required=False, help_text="street", write_only=True)

    def validate_venueowner_id(self, value):
        if VenueOwner.objects.filter(id=value).exists():
            return value
        else:
            raise serializers.FieldDoesNotExist("No Such Record")

    class Meta:
        model = Player
        fields = ['id', 'first_name', 'last_name',
                  'city', 'street', 'venueowner_id']


class AdminCampaignPart3GETEmailSerializer(serializers.ModelSerializer):
    venueowner_id = serializers.IntegerField(
        required=False, help_text="ID of the venue owner")
    id = serializers.IntegerField(required=False)

    def validate_venueowner_id(self, value):
        if VenueOwner.objects.filter(id=value).exists():
            return value
        else:
            raise serializers.FieldDoesNotExist("No Such Record")

    def validate_id(self, value):
        if value:
            if CampaignPart3.objects.filter(id=value, created_by=VenueOwner.objects.get(
                    id=self.context['venueowner_id']).user).exists():
                return value
            else:
                serializers.ValidationError("No Record Found")

    def __init__(self, *args, **kwargs):
        super(AdminCampaignPart3GETEmailSerializer,
              self).__init__(*args, **kwargs)
        for field in self.fields:
            if field == 'id':
                self.fields[field].read_only = False
            elif field == 'campaign1':
                self.fields[field].read_only = False
                self.fields[field].required = False
            elif field == 'venueowner_id':
                self.fields[field].read_only = False
            else:
                self.fields[field].read_only = True

    class Meta:
        model = CampaignPart3
        fields = ('venueowner_id', 'id', 'emailBody', 'email_logo', 'campaign1',
                  'email_background', 'created_by', 'created_at',)


class AdminCampaignPart3POSTEmailSerializer(serializers.ModelSerializer):
    venueowner_id = serializers.IntegerField(
        required=False, help_text="ID of the venue owner")
    id = serializers.IntegerField(required=False)
    part1ID = serializers.IntegerField(required=False, write_only=True)
    copy = serializers.IntegerField(required=False, write_only=True)

    def validate_venueowner_id(self, value):
        if VenueOwner.objects.filter(id=value).exists():
            return value
        else:
            raise serializers.FieldDoesNotExist("No Such Record")

    def create(self, validated_data):
        validated_data['campaign1'] = CampaignPart1.objects.get(
            id=validated_data['part1ID'])
        validated_data.pop('part1ID')
        validated_data.pop('venueowner_id')

        if "copy" in validated_data:
            camp3obj = CampaignPart3.objects.get(
                campaign1__id=validated_data["copy"])
            validated_data["email_logo"] = camp3obj.email_logo
            validated_data["email_background"] = camp3obj.email_background
            validated_data.pop('copy')
        data = CampaignPart3.objects.create(**validated_data)
        return data

    class Meta:
        model = CampaignPart3
        fields = ('venueowner_id', 'id', 'emailBody', 'email_logo', 'part1ID',
                  'email_background', 'email_url', 'created_by', 'created_at', 'copy', 'subject_line')


class AdminCampaignPart3GETSmsSerializer(serializers.ModelSerializer):
    venueowner_id = serializers.IntegerField(
        required=False, help_text="ID of the venue owner")
    id = serializers.IntegerField(required=False)

    def validate_venueowner_id(self, value):
        if VenueOwner.objects.filter(id=value).exists():
            return value
        else:
            raise serializers.FieldDoesNotExist("No Such Record")

    def validate_id(self, value):
        if value:
            if CampaignPart3.objects.filter(id=value, created_by=VenueOwner.objects.get(
                    id=self.context['venueowner_id']).user).exists():
                return value
            else:
                serializers.ValidationError("No Record Found")

    def __init__(self, *args, **kwargs):
        super(AdminCampaignPart3GETSmsSerializer,
              self).__init__(*args, **kwargs)
        for field in self.fields:
            if field == 'id':
                self.fields[field].read_only = False
            elif field == 'campaign1':
                self.fields[field].read_only = False
                self.fields[field].required = False
            elif field == 'venueowner_id':
                self.fields[field].read_only = False
            else:
                self.fields[field].read_only = True

    class Meta:
        model = CampaignPart3
        fields = ('venueowner_id', 'id', 'sms_text', 'campaign1',
                  'created_by', 'created_at')


class AdminCampaignPart3POSTSmsSerializer(serializers.ModelSerializer):
    venueowner_id = serializers.IntegerField(
        required=False, help_text="ID of the venue owner")
    id = serializers.IntegerField(required=False)
    part1ID = serializers.IntegerField(required=False, write_only=True)

    def validate_venueowner_id(self, value):
        if VenueOwner.objects.filter(id=value).exists():
            return value
        else:
            raise serializers.FieldDoesNotExist("No Such Record")

    def create(self, validated_data):
        validated_data['campaign1'] = CampaignPart1.objects.get(
            id=validated_data['part1ID'])
        validated_data.pop('part1ID')
        validated_data.pop('venueowner_id')
        data = CampaignPart3.objects.create(**validated_data)
        return data

    class Meta:
        model = CampaignPart3
        fields = ('venueowner_id', 'id', 'sms_text', 'part1ID',
                  'sms_url', 'created_by', 'created_at')


class AdminCampaignPart3BannerGETSerializer(serializers.ModelSerializer):
    venueowner_id = serializers.IntegerField(
        required=False, help_text="ID of the venue owner")
    id = serializers.IntegerField(required=False)

    def validate_venueowner_id(self, value):
        if VenueOwner.objects.filter(id=value).exists():
            return value
        else:
            raise serializers.FieldDoesNotExist("No Such Record")

    def validate_id(self, value):
        if value:
            if CampaignPart3.objects.filter(id=value, created_by=VenueOwner.objects.get(
                    id=self.context['venueowner_id']).user).exists():
                return value
            else:
                serializers.ValidationError("No Record Found")

    def __init__(self, *args, **kwargs):
        super(AdminCampaignPart3BannerGETSerializer,
              self).__init__(*args, **kwargs)
        for field in self.fields:
            if field == 'id':
                self.fields[field].read_only = False
            elif field == 'campaign1':
                self.fields[field].read_only = False
                self.fields[field].required = False
            elif field == 'venueowner_id':
                self.fields[field].read_only = False
            else:
                self.fields[field].read_only = True

    class Meta:
        model = CampaignPart3
        fields = ('venueowner_id', 'id', 'banner_text', 'campaign1', 'banner_logo',
                  'banner_background', 'created_by', 'created_at')


class AdminCampaignPart3BannerPOSTSerializer(serializers.ModelSerializer):
    venueowner_id = serializers.IntegerField(
        required=False, help_text="ID of the venue owner")
    id = serializers.IntegerField(required=False)
    part1ID = serializers.IntegerField(required=False, write_only=True)

    def validate_venueowner_id(self, value):
        if VenueOwner.objects.filter(id=value).exists():
            return value
        else:
            raise serializers.FieldDoesNotExist("No Such Record")

    def create(self, validated_data):
        validated_data['campaign1'] = CampaignPart1.objects.get(
            id=validated_data['part1ID'])
        validated_data.pop('part1ID')
        validated_data.pop('venueowner_id')
        data = CampaignPart3.objects.create(**validated_data)
        return data

    class Meta:
        model = CampaignPart3
        fields = ('venueowner_id', 'id', 'banner_text', 'part1ID', 'banner_url',
                  'banner_logo', 'banner_background', 'created_by', 'created_at')


class AdminCampaignPart4GETSerializer(serializers.ModelSerializer):
    venueowner_id = serializers.IntegerField(
        required=False, help_text="ID of the venue owner")
    id = serializers.IntegerField(required=False)

    def validate_venueowner_id(self, value):
        if VenueOwner.objects.filter(id=value).exists():
            return value
        else:
            raise serializers.FieldDoesNotExist("No Such Record")

    def __init__(self, *args, **kwargs):
        super(AdminCampaignPart4GETSerializer, self).__init__(*args, **kwargs)
        for field in self.fields:
            if field == 'id':
                self.fields[field].read_only = False
            elif field == 'campaign1':
                self.fields[field].read_only = False
                self.fields[field].required = False
            elif field == 'venueowner_id':
                self.fields[field].read_only = False
            else:
                self.fields[field].read_only = True

    class Meta:
        model = CampaignPart4
        fields = '__all__'


class AdminCampaignPart4POSTSerializer(serializers.ModelSerializer):
    # campaign1 = CampaignPart1SerializerGET(read_only=True)
    venueowner_id = serializers.IntegerField(
        required=False, help_text="ID of the venue owner")
    id = serializers.IntegerField(required=False)
    part1ID = serializers.IntegerField(required=False, write_only=True)
    notification_time = serializers.JSONField(
        required=False, write_only=True,
        help_text='[{"date":YYYY-MM-DD, "time":HH:MM},{"date":YYYY-MM-DD, "time":HH:MM}]')
    start_date = serializers.DateField(
        required=False, write_only=True, help_text="Date range start date")
    end_date = serializers.DateField(
        required=False, write_only=True, help_text="Date range end date")
    fields = serializers.PrimaryKeyRelatedField(
        queryset=VenueField.objects.all(), required=False, help_text="VenueField ID")

    def validate_venueowner_id(self, value):
        if VenueOwner.objects.filter(id=value).exists():
            return value
        else:
            raise serializers.FieldDoesNotExist("No Such Record")

    def create(self, validated_data):
        campaign1_object = CampaignPart1.objects.get(
            id=validated_data['part1ID'])

        if "fields" in validated_data:
            campaign1_object.field = validated_data.get('fields', None)
            campaign1_object.save()
            validated_data.pop("fields")

        if "start_date" in validated_data:
            campaign1_object.start_date = validated_data['start_date']
            campaign1_object.save()
            validated_data.pop("start_date")

        if "end_date" in validated_data:
            campaign1_object.end_date = validated_data['end_date']
            campaign1_object.save()
            validated_data.pop("end_date")

        if "notification_time" in validated_data:
            notification_time = validated_data['notification_time']
            notification_obj = [CampaignNotificationTime(
                campaign1=campaign1_object,
                date=i['date'],
                time=i['time']
            ) for i in notification_time]
            validated_data.pop("notification_time")
            CampaignNotificationTime.objects.bulk_create(notification_obj)
        validated_data['campaign1'] = CampaignPart1.objects.get(
            id=validated_data['part1ID'])
        validated_data.pop('part1ID')
        validated_data.pop('venueowner_id')
        data = CampaignPart4.objects.create(**validated_data)

        return data

    class Meta:
        model = CampaignPart4
        fields = '__all__'
        read_only_fields = ['campaign1']


class AdminCampaignSummarySerializer(serializers.Serializer):
    venueowner_id = serializers.IntegerField(
        required=False, help_text="ID of the venue owner")
    id = serializers.PrimaryKeyRelatedField(
        queryset=CampaignPart1.objects.all(), help_text="Campaign Part 1")

    def validate_venueowner_id(self, value):
        if VenueOwner.objects.filter(id=value).exists():
            return value
        else:
            raise serializers.FieldDoesNotExist("No Such Record")


class AdminCampaignLISTSerializer(serializers.ModelSerializer):
    venueowner_id = serializers.IntegerField(
        required=False, help_text="ID of the venue owner")
    campaign_kind = serializers.CharField(
        required=False, help_text="email / sms / banner/ paid_ads")
    keyword = serializers.CharField(
        required=False, help_text="Search Texr box")
    page = serializers.IntegerField(
        required=False, default=1, help_text="Page number. Default = 1")
    # sport = serializers.CharField(required=False, write_only=True)
    # sport_name = serializers.CharField(
    #     required=False, read_only=True, source="sport.name")
    assortment = serializers.CharField(
        required=False, write_only=True, help_text="Assortment start_date / end_date")
    status = serializers.CharField(
        required=False, write_only=True, help_text="UNDER APPROVAL/ON GOING/WAITING/REJECTED")
    status_to_send = serializers.SerializerMethodField('status_decider')
    field = serializers.CharField(required=False, write_only=True)
    field_name = serializers.CharField(
        required=False, read_only=True, source="field.name")

    def validate_venueowner_id(self, value):
        if VenueOwner.objects.filter(id=value).exists():
            return value
        else:
            raise serializers.FieldDoesNotExist("No Such Record")

    def status_decider(self, obj):
        d = CampaignPart1.objects.get(id=obj.id)
        if d.status == "O":
            return "UNDER APPROVAL"
        elif d.status == "R":
            return "REJECTED"
        elif d.status == "A":
            if d.start_date is None:
                return "WAITING"
            else:
                if d.start_date >= timezone.now().date():
                    return "WAITING"
                else:
                    return "ON GOING"

    class Meta:
        model = CampaignPart1
        fields = '__all__'
        read_only_fields = ['name', 'email', 'sms',
                            'banner', 'paid_ads']


class AdminCampaignDELETESerializer(serializers.Serializer):
    venueowner_id = serializers.IntegerField(
        required=False, help_text="ID of the venue owner")
    id = serializers.CharField(
        required=True, help_text="ID of campaign step 1")

    def validate_venueowner_id(self, value):
        if VenueOwner.objects.filter(id=value).exists():
            return value
        else:
            raise serializers.FieldDoesNotExist("No Such Record")

    def validate_id(self, value):
        if CampaignPart1.objects.filter(id=value,
                                        created_by=VenueOwner.objects.get(
                                            id=self.context['venueowner_id']).user,
                                        is_deleted=False).exists():
            return CampaignPart1.objects.filter(id=value, created_by=VenueOwner.objects.get(
                id=self.context['venueowner_id']).user, is_deleted=False).first()
        else:
            serializers.ValidationError("No such record")


class AdminglobalSerializer(serializers.Serializer):
    venueowner_id = serializers.IntegerField(
        required=False, help_text="ID of the venue owner")
    id = serializers.IntegerField(required=False)

    def validate_venueowner_id(self, value):
        if VenueOwner.objects.filter(id=value).exists():
            return value
        else:
            raise serializers.FieldDoesNotExist("No Such Record")

    def validate_id(self, value):
        if CampaignPart1.objects.filter(id=value, created_by=VenueOwner.objects.get(
                id=self.context['venueowner_id']).user).exists():
            return value
        else:
            raise serializers.ValidationError("No Such Record")


class AdminPlayerAreaSerializer(serializers.Serializer):
    venueowner_id = serializers.IntegerField(
        required=False, help_text="ID of the venue owner")
    area_list = serializers.JSONField(required=False, help_text="area")
    sports_list = serializers.JSONField(required=False, help_text="[1,2,3]")

    def validate_venueowner_id(self, value):
        if VenueOwner.objects.filter(id=value).exists():
            return value
        else:
            raise serializers.FieldDoesNotExist("No Such Record")


class AdminGetAllVenueAPIViewSerializer(serializers.ModelSerializer):
    class Meta:
        model = Venue
        fields = ['id', 'name']


class BookingCancellationApprovalFilterSerializer(serializers.Serializer):
    venue = serializers.CharField(required=False, help_text='venue id')
    cancellation_reason = serializers.CharField(required=False)
    start_date = serializers.DateField(required=False)
    end_date = serializers.DateField(required=False)
    status = serializers.CharField(required=False, write_only=True)
    ordering = serializers.CharField(
        required=False, help_text='DATE FROM-DESCENDING or DATE FROM-ASCENDING')
    page = serializers.JSONField(
        required=False, default=1, help_text="Page number")


class BookingCancelGetSerializer(serializers.ModelSerializer):
    id = serializers.CharField(required=True)

    class Meta:
        model = RequestBookingCancellation
        fields = "__all__"
        read_only_fields = ['cancellation_reason', 'venue_field', 'venue_field', 'venue', 'start_time', 'end_time',
                            'start_date',
                            'end_date', 'status', 'comments', 'created_at', 'created_by']


class BookingCancelChangeApproveStatusSerializer(serializers.Serializer):
    mode = serializers.CharField(
        required=False, help_text="Mode to change to || Approve, Rejected, New")
    id = serializers.IntegerField(
        required=True, help_text="Booking Cancel Request ID")


class RequestBookingCancellationVenueOwnerSerializer(serializers.ModelSerializer):
    created_by = VenueOwnerOnboardSerializer()

    class Meta:
        model = RequestBookingCancellation
        fields = "__all__"


class CancellationReasonSerializer(serializers.ModelSerializer):
    class Meta:
        model = BookingCancellation
        fields = "__all__"


class DeleteCancellationReasonSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(
        required=True, help_text="Id of Cancellation Reason")
    cancellation_reason = serializers.CharField(read_only=True)

    class Meta:
        model = BookingCancellation
        fields = "__all__"


class SaveEditCancellationReasonSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(
        required=False, help_text="Id of Cancellation Reason")
    cancellation_reason = serializers.CharField(required=False)

    class Meta:
        model = BookingCancellation
        fields = "__all__"


class EditCancellationPolicySerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(
        required=False, help_text="Id of Cancellation Policy")
    charge_rate = serializers.CharField(required=False)

    class Meta:
        model = CancellationPolicy
        fields = "__all__"
        read_only_fields = ['start_time', 'end_time']


class SaveCancellationPolicySerializer(serializers.ModelSerializer):
    start_time = serializers.IntegerField(required=True)
    end_time = serializers.IntegerField(required=True)

    class Meta:
        model = CancellationPolicy
        fields = "__all__"


class DeleteCancellationPolicySerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(
        required=True, help_text="Id of Cancellation Policy")

    class Meta:
        model = CancellationPolicy
        fields = "__all__"
        read_only_fields = ['charge_rate', 'start_time', 'end_time']


class CancellationPolicyGetAllSerializer(serializers.ModelSerializer):
    page = serializers.IntegerField(
        required=False, default=1, help_text="Page number")

    class Meta:
        model = CancellationPolicy
        fields = '__all__'
        read_only_fields = ['charge_rate', 'start_time', 'end_time']


class CancellationPolicyGetAllWithoutPageSerializer(serializers.ModelSerializer):
    class Meta:
        model = CancellationPolicy
        fields = '__all__'


class CollaborationTypesGetAllSerializer(serializers.ModelSerializer):
    class Meta:
        model = CollaborationTypes
        fields = '__all__'


class DeleteCollaborationTypesSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(
        required=True, help_text="Id of CancellationPolicy")

    class Meta:
        model = CollaborationTypes
        fields = "__all__"
        read_only_fields = ['collaboration_name']


class SaveEditCollaborationTypesSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(
        required=False, help_text="Id of CancellationPolicy")
    collaboration_name = serializers.CharField(required=False)

    class Meta:
        model = CollaborationTypes
        fields = "__all__"


class SaveEditCovid19MeasuresSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(
        required=False, help_text="Id of Covid19Measures")
    description = serializers.CharField(required=False)

    class Meta:
        model = Covid19Measures
        fields = "__all__"


class Covid19MeasuresGetAllSerializer(serializers.ModelSerializer):
    page = serializers.IntegerField(
        required=False, default=1, help_text="Page number")

    class Meta:
        model = Covid19Measures
        fields = '__all__'
        read_only_fields = ['description']


class DeleteCovid19MeasuresSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(
        required=True, help_text="Id of Covid19Measures")

    class Meta:
        model = Covid19Measures
        fields = "__all__"
        read_only_fields = ['description']


class AdminBookingCancellationRequestPostSerializer(serializers.ModelSerializer):
    venueowner_id = serializers.IntegerField(
        required=False, help_text="ID of the venue owner")
    status = serializers.CharField(read_only=True)
    start_time = serializers.TimeField(required=True)
    end_time = serializers.TimeField(required=True)
    start_date = serializers.DateField(required=True)
    end_date = serializers.DateField(required=True)
    venue_field = serializers.JSONField(required=False, help_text="[1,2]")

    def validate_venueowner_id(self, value):
        if VenueOwner.objects.filter(id=value).exists():
            return value
        else:
            raise serializers.FieldDoesNotExist("No Such Record")

    class Meta:
        model = RequestBookingCancellation
        fields = '__all__'
        read_only_fields = ['created_by']


class AdminBookingCancellationFilterSerializer(serializers.Serializer):
    venueowner_id = serializers.IntegerField(
        required=False, help_text="ID of the venue owner")
    venue_field = serializers.JSONField(required=False, help_text="[1,2]")
    cancellation_reason = serializers.CharField(required=False)
    start_date = serializers.DateField(required=False)
    end_date = serializers.DateField(required=False)
    status = serializers.CharField(required=False, write_only=True)
    page = serializers.JSONField(
        required=False, default=1, help_text="Page number")

    def validate_venueowner_id(self, value):
        if VenueOwner.objects.filter(id=value).exists():
            return value
        else:
            raise serializers.FieldDoesNotExist("No Such Record")


class AdminBookingCancellationRequestEditSerializer(serializers.ModelSerializer):
    venueowner_id = serializers.IntegerField(
        required=False, help_text="ID of the venue owner")
    id = serializers.IntegerField(
        required=True, help_text="ID of BookingCancellationRequest")
    status = serializers.CharField(required=True, help_text="N/A/R")

    # start_time = serializers.TimeField(required=True)
    # end_time = serializers.TimeField(required=True)
    # start_date = serializers.DateField(required=True)
    # end_date = serializers.DateField(required=True)

    def validate_venueowner_id(self, value):
        if VenueOwner.objects.filter(id=value).exists():
            return value
        else:
            raise serializers.FieldDoesNotExist("No Such Record")

    class Meta:
        model = RequestBookingCancellation
        fields = '__all__'
        read_only_fields = ['created_by', 'start_time', 'end_time', 'start_date',
                            'end_date', 'cancellation_reason', 'venue_field', 'comments']
        # extra_kwargs = {
        #     'cancellation_reason': {'required': True},
        #     'venue_field': {'required': True},
        #     'comments': {'required': True},
        # }


class ProfessinalsSerializerForFilteredList(serializers.Serializer):
    business_vat = serializers.CharField(required=False)
    business_name = serializers.CharField(required=False)
    area = serializers.JSONField(
        required=False, help_text='list of area=[1,2,3]')
    service = serializers.JSONField(
        required=False, help_text='list of Profession services =[1,2,3]')
    page = serializers.IntegerField(
        required=False, default=1, help_text="Page number")


class AdvertiserSerializerForFilteredList(serializers.Serializer):
    business_vat = serializers.CharField(required=False)
    business_name = serializers.CharField(required=False)
    area = serializers.JSONField(
        required=False, help_text='list of area=[1,2,3]')
    # service = serializers.JSONField(
    #     required=False, help_text='list of Profession services =[1,2,3]')
    page = serializers.JSONField(
        required=False, default=1, help_text="Page number")


class AdminProfessionalGETSerializer(serializers.ModelSerializer):
    professional_id = serializers.IntegerField(
        required=False, help_text="ID of the professional")
    user = serializers.CharField(
        required=False, read_only=True, source='user.username')
    service = ProfessionSerializer(
        required=False, read_only=True, many=True)
    served_areas = AreaGetSerializer(
        required=False, read_only=True, many=True)
    payment_method = PaymentMethodSerializer(
        required=False, read_only=True)
    country = CountrySerializer(read_only=True)
    city = CityGetSerializer(read_only=True)
    state = StateGetSerializer(read_only=True)
    area = AreaGetSerializer(read_only=True)

    def validate_professional_id(self, value):
        if Professional.objects.filter(id=value).exists():
            return value
        else:
            raise serializers.ValidationError("No Such Record")

    class Meta:
        model = Professional
        fields = '__all__'
        read_only_fields = ['user', 'first_name', 'middle_name', 'last_name', 'email', 'new_email', 'gender', 'mobile', 'new_mobile', 'telephone', 'telephone2', 'address1', 'address2',
                            'business_name', 'TaxOffices', 'buisness_vat', 'latitude', 'longitude', 'google_address', 'place_id', 'status', 'pincode', 'image', 'dob', 'is_deleted', 'is_email_verified', 'user_code', 'street', 'number', 'payment_method']
        extra_kwargs = {
            'email': {'required': False, 'help_text': 'Email'},
            'user': {'required': False, 'help_text': 'User'},
        }


class AdminProfessionalPOSTSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(
        required=False, help_text="ID of the professional")
    Telephone_number = serializers.JSONField(
        required=False, help_text='{"mobile":["847123654","35135654"]}')
    iban = serializers.CharField(required=False)
    bank = serializers.CharField(required=False)

    acc_holder_first_name = serializers.CharField(required=False)
    acc_holder_last_name = serializers.CharField(required=False)
    acc_type = serializers.CharField(required=False)
    bank_identification_no = serializers.CharField(required=False)
    iban_certificate = models.FileField(
        upload_to='pdfs/', null=True, blank=True, help_text='IBAN CERTIFICATE')

    contact_person = serializers.JSONField(
        required=False, help_text='[{"first_name":"Test","last_name":"asd","telephone":1234567,"email":"email"},{"first_name":"Test","last_name":"asd","telephone":1234567,"email":"email"}]')
    mobile = serializers.CharField(required=False, help_text="Mobile number")

    timing = serializers.JSONField(
        required=False, help_text='[{"day":1,"start_time":12:00:00,"is_active":1,"end_time":1:00:00}]')

    ServedAreas = serializers.JSONField(
        required=False, help_text='[{"latitude":00.00, "longitude":00.00, "google_address":"FFF", "place_id":"ppp"}]')
    service = serializers.JSONField(required=False, help_text="[1,2,3]")
    served_areas = serializers.JSONField(required=False, help_text="[1,2,3]")
    payment_method = serializers.IntegerField(
        required=False, help_text="payment method id")
    state = serializers.IntegerField(
        required=False, help_text="state")
    area = serializers.IntegerField(
        required=False, help_text="area")
    city = serializers.IntegerField(
        required=False, help_text="city")
    country = serializers.IntegerField(
        required=False, help_text="country")

    def validate_id(self, value):
        if Professional.objects.filter(id=value).exists():
            return value
        else:
            raise serializers.ValidationError("No Such Record")

    def validate_mobile(self, value):
        if Professional.objects.filter(mobile=value, is_deleted=False).exists():
            # raise serializers.ValidationError("Mobile number is already linked")
            pass

        if len(value) < 10:
            raise serializers.ValidationError(
                "Mobile Number must be filled in.")
        else:
            return value

    @transaction.atomic
    def create(self, validated_data):
        '''
        Profession
        '''

        print(self.context['id'])

        pobj = Professional.objects.get(id=self.context['id'])
        validated_data['user'] = pobj.user.id
        validated_data['email'] = pobj.email
        print(validated_data['email'])
        print(validated_data['user'])

        print(validated_data)

        # validated_data['user'] = self.context['user'].id
        # validated_data['email'] = self.context['user'].email
        serialized_data = ProfessionalSAVESerializer(instance=Professional.objects.filter(
            user=validated_data['user']).first(), data=validated_data)

        if serialized_data.is_valid():
            professional_data = serialized_data.save()
            professional_data.served_areas.set(validated_data['served_areas'])
            professional_data.save()
        else:
            raise serializers.ValidationError(
                handle_errors(serialized_data.errors))

        '''
        multiple profession service data
        '''

        for pr in validated_data['service']:
            print(pr)
        if ProfessionalStepServiceData.objects.filter(user=validated_data['user']).exists():
            step_service_data = ProfessionalStepServiceData.objects.get(
                user=validated_data['user'])
            step_service_data.professions.set(validated_data['service'])
            step_service_data.save()

            for pr2 in validated_data['service']:
                if ProfessionalStepSkill.objects.filter(~Q(skil__profession=pr2), created_by=validated_data['user']).exists():
                    ProfessionalStepSkill.objects.filter(
                        ~Q(skil__profession=pr2), created_by=validated_data['user']).delete()

        else:
            dataobj = ProfessionalStepServiceData.objects.create(
                user=pobj.user)
            dataobj.professions.set(validated_data['service'])
            dataobj.save()

        '''
        Contact Info
        '''

        serialized_data = ContactInfoProfileSerializer(
            data=validated_data)
        if serialized_data.is_valid():
            contact_obj = Professional.objects.filter(
                user=validated_data['user'])
            contact_contentType = ContentType.objects.get_for_model(
                Professional)
            ContactInfo.objects.filter(
                content_type__pk=contact_contentType.id, object_id__in=contact_obj).delete()
            serialized_data.validated_data['content_object'] = professional_data
            contact_info_data = serialized_data.save()
        else:

            raise serializers.ValidationError(
                handle_errors(serialized_data.errors))

        '''
        Timing
        '''
        if "timing" in validated_data:

            for i in validated_data['timing']:
                i['professional'] = professional_data.id
            timing_serializer = ProfessionalDailyAvailibilitySerializer(
                data=validated_data['timing'], many=True)

            if timing_serializer.is_valid():
                ProfessionalDailyTiming.objects.filter(
                    professional=professional_data.id).delete()
                timing_data = timing_serializer.save()
            else:
                print(timing_serializer.errors)
                raise serializers.ValidationError(
                    handle_errors(timing_serializer.errors))

        '''
        ServedArea
        '''
        if "ServedAreas" in validated_data:
            for i in validated_data['ServedAreas']:
                # ProfessionalServedArea.objects.filter(professional_id__pk=professional_data.id,professional_id__in=professional_data)
                i['professional'] = professional_data.id
            servearea_serializer = ServedAreaSerializer(
                data=validated_data['ServedAreas'], many=True)

            if servearea_serializer.is_valid():
                ProfessionalServedArea.objects.filter(
                    professional=professional_data.id).delete()
                servarea_data = servearea_serializer.save()
            else:
                print(servearea_serializer.errors)
                raise serializers.ValidationError(
                    handle_errors(servearea_serializer.errors))

        '''
        Contact Person
        '''
        if "contact_person" in validated_data:
            contact_persons = []
            Contact_Person.objects.filter(
                content_type__pk=contact_contentType.id, object_id__in=contact_obj).delete()
            # Contact_Person.objects.filter(content_type__pk=contact_contentType.id,object_id__in=contact_obj).all()

            for i in validated_data['contact_person']:
                if i['first_name'] == "" and i['last_name'] == "":
                    pass
                else:
                    contact_persons.append(i)

            serialized_data = ContactPersonSerializer(
                data=contact_persons, many=True)

            if serialized_data.is_valid():
                for i in serialized_data.validated_data:
                    i['content_object'] = professional_data
                serialized_data.save()
            else:
                raise serializers.ValidationError(
                    handle_errors(serialized_data.errors))

        '''
        Contact Numbers
        '''
        if "Telephone_number" in validated_data:
            TelephoneNumbers.objects.filter(
                contact_id=contact_info_data).delete()
            contact_numbers = [TelephoneNumbers(contact_id=contact_info_data,
                                                contact_number=i
                                                ) for i in validated_data['Telephone_number']['mobile']]

            TelephoneNumbers.objects.bulk_create(contact_numbers)

        '''
        Bank Info
        '''
        if 'bank' in validated_data:
            BankInfo.objects.filter(
                content_type__pk=contact_contentType.id, object_id__in=contact_obj).delete()
            serialized_data = BankInfoProfileSerializer(data=validated_data)
            if serialized_data.is_valid():
                serialized_data.validated_data['content_object'] = professional_data
                serialized_data.save()
            else:
                raise serializers.ValidationError(
                    handle_errors(serialized_data.errors))
        return professional_data.id

    class Meta:
        model = Professional
        fields = '__all__'
        extra_kwargs = {
            'email': {"read_only": True},
            'new_email': {"read_only": True},
            'user': {"read_only": True}
        }


################## SMS SETUP ################

class SmsSetupFeeSerializer(serializers.ModelSerializer):
    measurement_units = MeasurementUnitsWithoutPageSerializer(
        required=False, read_only=True)
    vat = VatWithoutPageSerializer(required=False, read_only=True)

    class Meta:
        model = SmsSetupFee
        fields = '__all__'


class SmsSetupFeePostSerializer(serializers.ModelSerializer):

    # def create(self, validated_data):
    #     data = SmsSetupFee.objects.create(**validated_data)
    #     return data

    class Meta:
        model = SmsSetupFee
        fields = '__all__'


class SmsPackagesSerializer(serializers.ModelSerializer):
    page = serializers.IntegerField(
        required=False, default=1, help_text="Page number")
    measurement_units = MeasurementUnitsWithoutPageSerializer(
        required=False, read_only=True)
    vat = VatWithoutPageSerializer(required=False, read_only=True)
    code = serializers.CharField(required=False, source="code.code")

    class Meta:
        model = SmsPackages
        fields = '__all__'
        read_only_fields = ['code', 'description',
                            'sms_qty_from', 'sms_qty_to', 'price', 'is_deleted']


class SmsPackagesPOSTSerializer(serializers.ModelSerializer):
    code = serializers.CharField(required=True, help_text="Code")

    def validate_code(self, value):
        if UniqueCodes.objects.filter(code=value).exists():
            raise serializers.ValidationError("Unique Code already exists")
        return UniqueCodes.objects.create(code=value)

    class Meta:
        model = SmsPackages
        fields = '__all__'


class SmsPackagesEDITSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(
        required=False, help_text="Id of SMS Package")

    class Meta:
        model = SmsPackages
        fields = '__all__'
        read_only_fields = ['code']


class DeleteSmsPackagesSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(
        required=True, help_text="Id of SMS Package")

    class Meta:
        model = SmsPackages
        fields = ['id']


class SingleSmsPackagesSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(
        required=False, help_text="Id of SMS Package")
    measurement_units = MeasurementUnitsWithoutPageSerializer(
        required=False, read_only=True)
    vat = VatWithoutPageSerializer(required=False, read_only=True)
    code = serializers.CharField(required=False, source="code.code")

    class Meta:
        model = SmsPackages
        fields = '__all__'
        read_only_fields = ['code', 'description',
                            'sms_qty_from', 'sms_qty_to', 'price', 'is_deleted']


############ EMAIL SETUP ##############


class EmailSetupFeeSerializer(serializers.ModelSerializer):
    measurement_units = MeasurementUnitsWithoutPageSerializer(
        required=False, read_only=True)
    vat = VatWithoutPageSerializer(required=False, read_only=True)

    class Meta:
        model = EmailSetupFee
        fields = '__all__'


class EmailSetupFeePostSerializer(serializers.ModelSerializer):

    class Meta:
        model = EmailSetupFee
        fields = '__all__'


class EmailPackagesSerializer(serializers.ModelSerializer):
    page = serializers.IntegerField(
        required=False, default=1, help_text="Page number")
    measurement_units = MeasurementUnitsWithoutPageSerializer(
        required=False, read_only=True)
    vat = VatWithoutPageSerializer(required=False, read_only=True)

    code = serializers.CharField(required=False, source="code.code")

    class Meta:
        model = EmailPackages
        fields = '__all__'
        read_only_fields = ['code', 'description',
                            'email_qty_from', 'email_qty_to', 'price', 'is_deleted']


class EmailPackagesPOSTSerializer(serializers.ModelSerializer):
    code = serializers.CharField(required=True, help_text="Code")

    def validate_code(self, value):
        if UniqueCode.objects.filter(code=value).exists():
            raise serializers.ValidationError("Code Already Exists")
        return UniqueCode.objects.create(code=value)

    class Meta:
        model = EmailPackages
        fields = '__all__'


class EmailPackagesEDITSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(
        required=False, help_text="Id of EMAIL Package")

    class Meta:
        model = EmailPackages
        fields = '__all__'
        read_only_fields = ['code']


class DeleteEmailPackagesSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(
        required=True, help_text="Id of EMAIL Package")

    class Meta:
        model = EmailPackages
        fields = ['id']


class SingleEmailPackagesSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(
        required=False, help_text="Id of EMAIL Package")
    measurement_units = MeasurementUnitsWithoutPageSerializer(
        required=False, read_only=True)
    vat = VatWithoutPageSerializer(required=False, read_only=True)
    code = serializers.CharField(required=False, source="code.code")

    class Meta:
        model = EmailPackages
        fields = '__all__'
        read_only_fields = ['code', 'description',
                            'email_qty_from', 'email_qty_to', 'price', 'is_deleted']


################### BANNER SETUP #######################
class BannerSetupFeeSerializer(serializers.ModelSerializer):
    measurement_units = MeasurementUnitsWithoutPageSerializer(
        required=False, read_only=True)
    vat = VatWithoutPageSerializer(required=False, read_only=True)
    code = serializers.CharField(required=False, source="code.code")

    class Meta:
        model = BannerSetupFee
        fields = '__all__'


class BannerSetupFeePostSerializer(serializers.ModelSerializer):

    code = serializers.CharField(required=False, help_text="Code")

    def validate_code(self, value):
        if UniqueCode.objects.filter(code=value).exists():
            raise serializers.ValidationError("Code already in use")
        return UniqueCode.objects.create(code=value)

    class Meta:
        model = BannerSetupFee
        fields = '__all__'


#################### PAID ADS SETUP #####################

class PaidAdsSetupFeeSerializer(serializers.ModelSerializer):
    measurement_units = MeasurementUnitsWithoutPageSerializer(
        required=False, read_only=True)
    vat = VatWithoutPageSerializer(required=False, read_only=True)
    code = serializers.CharField(required=False, source="code.code")

    class Meta:
        model = PaidAdsSetupFee
        fields = '__all__'


class PaidAdsSetupFeePostSerializer(serializers.ModelSerializer):
    code = serializers.CharField(required=False, help_text="Code")

    def validate_code(self, value):
        if UniqueCode.objects.filter(code=value).exists():
            raise serializers.ValidationError("Code already in use")
        return UniqueCode.objects.create(code=value)

    class Meta:
        model = PaidAdsSetupFee
        fields = '__all__'


class PaidAdsViewsPackagesSerializer(serializers.ModelSerializer):
    page = serializers.IntegerField(
        required=False, default=1, help_text="Page number")
    measurement_units = MeasurementUnitsWithoutPageSerializer(
        required=False, read_only=True)
    vat = VatWithoutPageSerializer(required=False, read_only=True)

    class Meta:
        model = PaidAdsViewsPackages
        fields = '__all__'
        read_only_fields = ['code', 'description',
                            'view_qty_from', 'view_qty_to', 'price', 'is_deleted', 'low_limit_notification']


class PaidAdsViewsPackagesPOSTSerializer(serializers.ModelSerializer):

    class Meta:
        model = PaidAdsViewsPackages
        fields = '__all__'


class PaidAdsViewsPackagesEDITSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(
        required=False, help_text="Id of PaidAds Views Package")

    class Meta:
        model = PaidAdsViewsPackages
        fields = '__all__'


class DeletePaidAdsViewsPackagesSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(
        required=True, help_text="Id of PaidAds Views Package")

    class Meta:
        model = PaidAdsViewsPackages
        fields = ['id']


class SinglePaidAdsViewsPackagesSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(
        required=False, help_text="Id of PaidAds Views Package")
    measurement_units = MeasurementUnitsWithoutPageSerializer(
        required=False, read_only=True)
    vat = VatWithoutPageSerializer(required=False, read_only=True)

    class Meta:
        model = PaidAdsViewsPackages
        fields = '__all__'
        read_only_fields = ['code', 'description',
                            'view_qty_from', 'view_qty_to', 'price', 'is_deleted', 'low_limit_notification']


class AdminCampaignPart2RecipientSerializer(serializers.ModelSerializer):

    class Meta:
        model = CampaignPart2
        fields = ['notification', 'user_notification']
        read_only_fields = ['campaign1', 'created_by',
                            'created_at', 'is_deleted', 'professional_service', 'sports', 'Areas']


class CampaginVenueFieldSerializerNameSerializer(serializers.ModelSerializer):
    #venue_service = VenueServiceSerializerForServices(read_only=True)

    class Meta:
        model = VenueField
        fields = ['id', 'name']


class AdminCampaignPart4DiscountSerializer(serializers.ModelSerializer):
    views = CampaignPaidAdsViewsPackagesEDITSerializer(
        required=False, read_only=True)
    discount_applied_field = CampaginVenueFieldSerializerNameSerializer(
        required=False, read_only=True)

    class Meta:
        model = CampaignPart4
        fields = '__all__'


class AdminNewPackagesGetAndFiltersSerializer(serializers.ModelSerializer):
    sport_relivance = serializers.CharField(
        required=False, help_text="Sport ID")
    sport_name = serializers.CharField(
        required=False, read_only=True, source="sport.name")
    sport_id = serializers.CharField(
        required=False, read_only=True, source="sport_relivance.id")
    keyword = serializers.CharField(
        required=False, help_text="Search Text box")
    page = serializers.IntegerField(
        required=False, default=1, help_text="Page number. Default = 1")
    package_type = serializers.CharField(
        required=False, )
    package_type_id = serializers.CharField(
        required=False, read_only=True, source="package_type.id")
    package_duration = serializers.CharField(required=False, read_only=True,)
    package_duration_id = serializers.CharField(
        required=False, read_only=True, source="package_duration.id")
    creation_date_from = serializers.DateField(required=False)
    creation_date_to = serializers.DateField(required=False)
    status_name = serializers.CharField(
        required=True, write_only=True, help_text="NEW/APPROVED/REJECTED")
    venue_name = serializers.SerializerMethodField('getVenueName')

    def getVenueName(self, obj):
        print(obj)
        # print(obj.created_by)
        if Venue.objects.filter(created_by=obj.created_by).exists():
            venue_obj = Venue.objects.filter(created_by=obj.created_by).last()
            if venue_obj.name is not None:
                return venue_obj.name
            else:
                return "-"
        else:
            return "-"

    class Meta:
        model = Packages
        fields = '__all__'
        read_only_fields = ['package_image', 'package_description',
                            'number_sessions', 'session_duration', 'package_quantity', 'level', 'price', 'name',  'is_deleted', 'package_duration', 'status', 'created_by']


class CreateOrderSerializer(serializers.Serializer):
    camp_id = serializers.IntegerField(required=True)


class TransactionSerializer(serializers.Serializer):
    orderCode = serializers.CharField(required=True)
    transactionId = serializers.CharField(required=True)


class AdminServiceDataCustomSerializer(serializers.Serializer):
    id = serializers.IntegerField(
        required=True, help_text="Id of professional")
    # image part
    Mainimage = serializers.ListField(
        help_text='send array of photos', required=False)
    #created_by = serializers.CharField(required=False)
    # Step service part
    #profession = serializers.CharField(help_text="id of the profession")
    professions = serializers.JSONField(
        required=False, help_text="[1,2,3]")
    serviceDescription = serializers.CharField(required=False,)

    def validate_id(self, value):

        if Professional.objects.filter(id=self.context['id']).exists():
            print(value)
            return value
        else:
            raise serializers.ValidationError("No Such Record")

    # def validate_created_by(self, value):
    #     print(value)
    #     print((self.context['id']).user)
    #     if Professional.objects.filter(user=self.context['id']).user.exists():
    #         return value
    #     else:
    #         raise serializers.ValidationError("No User Founnd")

    def validate_professions(self, value):
        tp = []
        for p in value:
            print("outside: ", p)
            if Profession.objects.filter(id=p).exists():
                print("inside: ", p)
                tp.append(p)
                pass
            else:
                raise serializers.ValidationError("No Profession Found")
        return tuple(tp)

    def create(self, validated_data, **kwargs):
        # print(validated_data)
        # print(self.context['id'])

        Professional_obj = Professional.objects.get(
            id=self.context['id'])
        validated_data['created_by'] = Professional_obj.user

        '''validated_data['profession'] = Profession.objects.get(
            id=validated_data['profession'])'''

        if 'Mainimage' in validated_data:
            photo = [ProfessionalServicePhoto(
                main_image=i,
                created_by=validated_data['created_by']
            )for i in validated_data['Mainimage']]

            ProfessionalServicePhoto.objects.bulk_create(photo)
        prof = validated_data['professions']
        print(prof)
        validated_data.pop('professions')
        user_professional = Professional.objects.get(
            user=validated_data['created_by'])
        user_professional.service.set(prof)
        user_professional.save()
        ProfessionalStepServiceData.objects.filter(
            user=validated_data['created_by']).delete()
        data = ProfessionalStepServiceData.objects.create(
            user=validated_data['created_by'], service_description=validated_data['serviceDescription'])
        data.professions.set(prof)
        return "Success"


class AdminCertificatesGetSerializer(serializers.Serializer):
    id = serializers.IntegerField(
        required=True, help_text="Id of professional")


class AdminCertificatesPostSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(
        required=True, help_text="Id of professional")
    uploaded_by = serializers.CharField(
        source='created_by.username', read_only=True)
    certificate_file = serializers.ListField(required=False, write_only=True)
    file_data = serializers.JSONField(
        required=False, help_text='[{"doc_name1":"file_name1","user_input":"test"},{"doc_name2":"file_name2","user_input":"test"}]', write_only=True)

    def create(self, validated_data, format=None):
        try:
            # print(validated_data)
            # print(self.context['id'])

            Professional_obj = Professional.objects.get(
                id=self.context['id'])
            certificate = [ProfessionalCertificate(
                certificate=i,
                certificate_name=next(
                    item for item in validated_data['file_data'] if item["doc_name1"] == i.name)['user_input'],
                created_by=Professional_obj.user
            ) for i in validated_data['certificate_file']]
            certificate = ProfessionalCertificate.objects.bulk_create(
                certificate)
            return True

        except Exception as e:
            raise serializers.ValidationError(
                "Error Occureed in serializer "+str(e))

    class Meta:
        model = ProfessionalCertificate
        fields = "__all__"
        read_only_fields = ['is_deleted', 'certificate', 'certificate_name']


class AdminDeleteCertificateSerializer(serializers.Serializer):
    prof_id = serializers.IntegerField(
        required=True, help_text="Id of professional")
    id = serializers.IntegerField(required=True, help_text="Id of certificate")


class AdminRequestSkillsSerializer(serializers.ModelSerializer):
    prof_id = serializers.IntegerField(
        required=False, help_text="Id of professional")

    class Meta:
        model = Skill
        fields = '__all__'


class AdminStepSkillsSerializer(serializers.Serializer):
    prof_id = serializers.IntegerField(
        required=True, help_text="Id of professional")
    skill_name = serializers.CharField(
        required=False, help_text="1,2,3")
    # profession_id = serializers.IntegerField(required=False, write_only=True)

    # def validate_profession_id(self, value):
    #     profession_obj = Profession.objects.filter(id=value)
    #     if profession_obj.exists():
    #         return profession_obj.last()
    #     raise serializers.ValidationError("Invalid Profession")

    def validate_skill_name(self, value):
        skill_name = [Skill.objects.get(id=i)
                      for i in value.split(',')]
        return skill_name

    def create(self, validated_data, **kwargs):
        try:
            created_by = Professional.objects.get(id=self.context['prof_id'])
            skills_obj = [ProfessionalStepSkill(
                skil=i,
                created_by=created_by.user
            ) for i in validated_data['skill_name']]
            data = ProfessionalStepSkill.objects.bulk_create(skills_obj)
            return "Success"

        except Exception as e:
            raise serializers.ValidationError(
                "Error Occureed in serializer "+str(e))
        # return data


class ProfessionalRequestServiceSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(required=False)
    buisness_name = serializers.CharField(
        required=False, read_only=True, source="professional.business_name")
    buisness_name_search = serializers.CharField(
        required=False, write_only=True)
    first_name = serializers.CharField(
        required=False, read_only=True, source="professional.first_name")
    profession_name = serializers.CharField(
        required=False, read_only=True, source="profession.professionName")

    class Meta:
        model = ProfessionalRequestService
        fields = '__all__'


class ProfessionalRequestSkillSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(required=False)
    buisness_name = serializers.CharField(
        required=False, read_only=True, source="professional.business_name")
    buisness_name_search = serializers.CharField(
        required=False, write_only=True)
    first_name = serializers.CharField(
        required=False, read_only=True, source="professional.first_name")
    profession_name = serializers.CharField(
        required=False, read_only=True, source="profession.professionName")

    class Meta:
        model = ProfessionalRequestSkill
        fields = '__all__'


########### Package Types ##############
class PackageTypesSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(
        required=False, help_text="Id to edit package types")

    class Meta:
        model = PackageTypes
        fields = '__all__'


class PackageTypesGetSerializer(serializers.ModelSerializer):
    class Meta:
        model = PackageTypes
        fields = '__all__'


class DeletePackageTypesSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(
        required=True, help_text="Id of Package Types")

    class Meta:
        model = PackageTypes
        fields = ['id']


class SinglePackageTypesSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(
        required=False, help_text="Id of package types")

    class Meta:
        model = PackageTypes
        fields = '__all__'
        read_only_fields = ['description']


########### Package Duration ##############
class PackageDurationsSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(
        required=False, help_text="Id to edit Package Durations")

    class Meta:
        model = PackageDurations
        fields = '__all__'


class PackageDurationsGetSerializer(serializers.ModelSerializer):
    class Meta:
        model = PackageDurations
        fields = '__all__'


class DeletePackageDurationsSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(
        required=True, help_text="Id of Package Durations")

    class Meta:
        model = PackageDurations
        fields = ['id']


class SinglePackageDurationsSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(
        required=False, help_text="Id of Package Durations")

    class Meta:
        model = PackageDurations
        fields = '__all__'
        read_only_fields = ['description']


########### Sports ##############
class SportsPostSerializer(serializers.ModelSerializer):
    catagory = serializers.CharField(
        required=False, help_text="For Catagory send : Sports")

    class Meta:
        model = Sport
        fields = '__all__'
        read_only_fields = ['play_type', 'show_on_homepage', 'parent_sport']


class AllSportsGetSerializer(serializers.ModelSerializer):
    catagory = serializers.CharField(
        required=True, help_text="For Catagory send : Sports")

    class Meta:
        model = Sport
        fields = '__all__'
        read_only_fields = ['play_type',
                            'show_on_homepage', 'parent_sport', 'name']


class SportsEditSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(
        required=False, help_text="Id to edit sport")

    class Meta:
        model = Sport
        fields = '__all__'
        read_only_fields = ['play_type',
                            'show_on_homepage', 'parent_sport', 'catagory']


class DeleteSportSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(
        required=True, help_text="Id of Sport")

    class Meta:
        model = Sport
        fields = ['id']


class SingleSportSerializer(serializers.Serializer):
    id = serializers.IntegerField(
        required=False, help_text="Id of Sport")


class SingleSportGetSerializer(serializers.ModelSerializer):

    class Meta:
        model = Sport
        fields = ['id', 'name', 'catagory']


########### Winter Sports #########

class WinterSportsPostSerializer(serializers.ModelSerializer):
    catagory = serializers.CharField(
        required=False, help_text="For Catagory send : Winter Sports")

    class Meta:
        model = Sport
        fields = '__all__'
        read_only_fields = ['play_type', 'show_on_homepage', 'parent_sport']


class AllWinterSportsGetSerializer(serializers.ModelSerializer):
    catagory = serializers.CharField(
        required=True, help_text="For Catagory send : Winter Sports")

    class Meta:
        model = Sport
        fields = '__all__'
        read_only_fields = ['play_type',
                            'show_on_homepage', 'parent_sport', 'name']

########### Summer Sports #########


class SummerSportsPostSerializer(serializers.ModelSerializer):
    catagory = serializers.CharField(
        required=False, help_text="For Catagory send : Summer Sports")

    class Meta:
        model = Sport
        fields = '__all__'
        read_only_fields = ['play_type', 'show_on_homepage', 'parent_sport']


class AllSummerSportsGetSerializer(serializers.ModelSerializer):
    catagory = serializers.CharField(
        required=True, help_text="For Catagory send : Summer Sports")

    class Meta:
        model = Sport
        fields = '__all__'
        read_only_fields = ['play_type',
                            'show_on_homepage', 'parent_sport', 'name']


########### All Season Sports #########
class AllSeasonsSportsPostSerializer(serializers.ModelSerializer):
    catagory = serializers.CharField(
        required=False, help_text="For Catagory send : All Seasons Sports")

    class Meta:
        model = Sport
        fields = '__all__'
        read_only_fields = ['play_type', 'show_on_homepage', 'parent_sport']


class AllSeasonsSportsGetSerializer(serializers.ModelSerializer):
    catagory = serializers.CharField(
        required=True, help_text="For Catagory send : All Seasons Sports")

    class Meta:
        model = Sport
        fields = '__all__'
        read_only_fields = ['play_type',
                            'show_on_homepage', 'parent_sport', 'name']


########### Field Type #########
class FieldTypesPostSerializer(serializers.ModelSerializer):
    sport = serializers.JSONField(
        required=False, help_text="[1,2,3]")

    def create(self, validated_data):
        data = FieldType.objects.create(name=validated_data['name'])
        data.sport.set(validated_data['sport'])
        data.save()
        return data

    class Meta:
        model = FieldType
        fields = '__all__'


class AllFieldTypesGetSerializer(serializers.ModelSerializer):
    sport = SportSerializer(read_only=True, many=True)

    class Meta:
        model = FieldType
        fields = '__all__'


class FieldTypeEditSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(
        required=False, help_text="Id to edit FieldType")
    sport = serializers.JSONField(
        required=False, help_text="[1,2,3]")

    class Meta:
        model = FieldType
        fields = '__all__'


class DeleteFieldTypeSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(
        required=True, help_text="Id of FieldType")

    class Meta:
        model = FieldType
        fields = ['id']


class SingleFieldTypeSerializer(serializers.Serializer):
    id = serializers.IntegerField(
        required=False, help_text="Id of FieldType")


########### Floor Type #########
class FloorTypesPostSerializer(serializers.ModelSerializer):
    sport = serializers.JSONField(
        required=False, help_text="[1,2,3]")

    def create(self, validated_data):
        data = FloorType.objects.create(name=validated_data['name'])
        data.sport.set(validated_data['sport'])
        data.save()
        return data

    class Meta:
        model = FloorType
        fields = '__all__'


class AllFloorTypesGetSerializer(serializers.ModelSerializer):
    sport = SportSerializer(read_only=True, many=True)

    class Meta:
        model = FloorType
        fields = '__all__'


class FloorTypeEditSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(
        required=False, help_text="Id to edit FloorType")
    sport = serializers.JSONField(
        required=False, help_text="[1,2,3]")

    class Meta:
        model = FloorType
        fields = '__all__'


class DeleteFloorTypeSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(
        required=True, help_text="Id of FloorType")

    class Meta:
        model = FloorType
        fields = ['id']


class SingleFloorTypeSerializer(serializers.Serializer):
    id = serializers.IntegerField(
        required=False, help_text="Id of FloorType")


########### Facility Type #########
class FacilityTypesPostSerializer(serializers.ModelSerializer):
    sport = serializers.JSONField(
        required=False, help_text="[1,2,3]")

    def create(self, validated_data):
        data = FacilityType.objects.create(name=validated_data['name'])
        data.sport.set(validated_data['sport'])
        data.save()
        return data

    class Meta:
        model = FacilityType
        fields = '__all__'


class AllFacilityTypesGetSerializer(serializers.ModelSerializer):
    sport = SportSerializer(read_only=True, many=True)

    class Meta:
        model = FacilityType
        fields = '__all__'


class FacilityTypeEditSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(
        required=False, help_text="Id to edit FacilityType")
    sport = serializers.JSONField(
        required=False, help_text="[1,2,3]")

    class Meta:
        model = FacilityType
        fields = '__all__'


class DeleteFacilityTypeSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(
        required=True, help_text="Id of FacilityType")

    class Meta:
        model = FacilityType
        fields = ['id']


class SingleFacilityTypeSerializer(serializers.Serializer):
    id = serializers.IntegerField(
        required=False, help_text="Id of FacilityType")


########### Lights #########

class LightsPostSerializer(serializers.ModelSerializer):
    sport = serializers.JSONField(
        required=False, help_text="[1,2,3]")

    def create(self, validated_data):
        data = Lights.objects.create(name=validated_data['name'])
        data.sport.set(validated_data['sport'])
        data.save()
        return data

    class Meta:
        model = Lights
        fields = '__all__'


class AllLightsGetSerializer(serializers.ModelSerializer):
    sport = SportSerializer(read_only=True, many=True)

    class Meta:
        model = Lights
        fields = '__all__'


########### AVAILABILITY_IN_USE #########

class AVAILABILITY_IN_USEPostSerializer(serializers.ModelSerializer):
    sport = serializers.JSONField(
        required=False, help_text="[1,2,3]")

    def create(self, validated_data):
        data = AVAILABILITY_IN_USE.objects.create(name=validated_data['name'])
        data.sport.set(validated_data['sport'])
        data.save()
        return data

    class Meta:
        model = AVAILABILITY_IN_USE
        fields = '__all__'


class AllAVAILABILITY_IN_USEGetSerializer(serializers.ModelSerializer):
    sport = SportSerializer(read_only=True, many=True)

    class Meta:
        model = AVAILABILITY_IN_USE
        fields = '__all__'
