from utils.mixins import PlatformAPICallMixin
from rest_framework import status, generics
from rest_framework.permissions import IsAuthenticated
from rest_framework.authentication import TokenAuthentication
from rest_framework.response import Response

from static_data.serializers import PlayerLevelSerializer, TeamLevelSerializer
from static_data.functionalities.request import get_levels, insert_levels, delete_levels, update_levels



def response_message(add_api_call, request, result, **kwargs):
    response = dict(result=result, **kwargs)
    add_api_call(request, response)
    return response


def get_level(request, serializer_class, add_api_call, format=None):
    try:
        level_obj = get_levels(serializer_class)
        response = response_message(add_api_call, request, result=1, data=level_obj)
        return Response(response, status=status.HTTP_200_OK)
    except Exception as e:
        response = response_message(add_api_call, request, 0, errors=list(str(e)))
        return Response(response, status=status.HTTP_200_OK)

def post_level(request, serializer_class, add_api_call, format=None):
    try:
        insert_levels(serializer_class, request.data)
        response = response_message(add_api_call, request, result=1, message=["Sucessfully Inserted"])
        return Response(response, status=status.HTTP_200_OK)
    except RuntimeError as re:
        response = response_message(add_api_call, request, 0, errors=list(str(re)))
        return Response(response, status=status.HTTP_200_OK)
    except Exception as e:
        response = response_message(add_api_call, request, 0, errors=list(str(e)))
        return Response(response, status=status.HTTP_200_OK)

def delete_level(request, serializer_class, add_api_call, format=None):
    try:
        assert request.GET.get("id", False), "No Id Provided"
        id = request.GET.get("id", False)
        delete_levels(serializer_class, id)
        response = response_message(add_api_call, request, result=1, message=["Successfully Deleted"])
        return Response(response, status=status.HTTP_200_OK)
    except AssertionError as ae:
        response = response_message(add_api_call, request, 0, errors=list(str(ae)))
        return Response(response, status=status.HTTP_200_OK)
    except Exception as e:
        response = response_message(add_api_call, request, 0, errors=list(str(e)))
        return Response(response, status=status.HTTP_200_OK)


def patch_level(request, serializer_class, add_api_call, format=None):
    try:
        assert request.GET.get("id", False), "No Id Provided"
        id = request.GET.get("id", False)
        update_levels(serializer_class, request.data, id)
        response = response_message(add_api_call, request, result=1, message=["Successfully Updated"])
        return Response(response, status=status.HTTP_200_OK)
    except AssertionError as ae:
        response = response_message(add_api_call, request, 0, errors=list(str(ae)))
        return Response(response, status=status.HTTP_200_OK)
    except Exception as e:
        response = response_message(add_api_call, request, 0, errors=list(str(e)))
        return Response(response, status=status.HTTP_200_OK)



class PlayerLevelAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    # authentication_classes = (TokenAuthentication,)
    # permission_classes = (IsAuthenticated,)
    serializer_class = PlayerLevelSerializer

    def get(self, request, format=None):
        """
        Player Level Get Api View
        """
        return get_level(request, self.serializer_class, self.add_api_call)

    
    def post(self, request, format=None):
        """ 
        Player Level API View
        """ 
        return post_level(request, self.serializer_class, self.add_api_call)

        
    def delete(self, request, format=None):
        """
        Player Level Delete API View
        """
        return delete_level(request, self.serializer_class, self.add_api_call)


    def patch(self, request, format=None):
        """
        Player Level Delete API View
        """
        return patch_level(request, self.serializer_class, self.add_api_call)



class TeamLevelAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    # authentication_classes = (TokenAuthentication,)
    # permission_classes = (IsAuthenticated,)
    serializer_class = TeamLevelSerializer

    def get(self, request, format=None):
        """
        Team Level Get Api View
        """
        return get_level(request, self.serializer_class, self.add_api_call)

    
    def post(self, request, format=None):
        """ 
        Team Level API View
        """ 
        return post_level(request, self.serializer_class, self.add_api_call)

        
    def delete(self, request, format=None):
        """
        Team Level Delete API View
        """
        return delete_level(request, self.serializer_class, self.add_api_call)


    def patch(self, request, format=None):
        """
        Team Level Delete API View
        """
        return patch_level(request, self.serializer_class, self.add_api_call)