import pytz
import json
from django.http import HttpResponse
from django.template import Template, Context
from django.template.loader import get_template
from django.db import transaction
from django.urls import reverse

from django.contrib.auth import login as django_login
import datetime
from rest_framework.authtoken.models import Token
from rest_framework import status, generics
from rest_framework.response import Response
from rest_framework.authentication import TokenAuthentication
from rest_framework.permissions import IsAuthenticated, DjangoModelPermissions
from utils.mixins import PlatformAPICallMixin
from utils.api_handle_errors import handle_errors
from admin_app.serializers import *
from professionals.serializers import *
from professionals.models import *
from reviews.serializers import ReviewGETSerializer
from reviews.models import Review
from utils.decorator import check_permission
from commonAPI.serializers import ContactPersonSerializer
from commonAPI.models import Contact_Person
from advertisers.models import *
from advertisers.serializers import *
from static_data.functionalities.request import save_skill_service
from professionals.serializers import (
    ProfessionalDocumentSerializer, ProfessionalCertificateSerializer, ProfessionalPOSTSerializer, ServiceDataCustomSerializer,
    StepSkillsSerializer,CertificatesSerializer, ProfessionalGETSerializer, ServicePhotosSerializer, GetServiceSkillSerializer)

class AdminGetProfessionalAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    '''
    Get Professional Data
    '''
    authentication_classes = (TokenAuthentication,)
    #permission_classes = (DjangoModelPermissions,)
    serializer_class = AdminProfessionalGETSerializer
    # dailySerializer_class = DailyTimingGlobalSerializer
    dailySerializer_class = ProfessionalDailyAvailibilitySerializer
    servedarea_class = ServedAreaSerializer

    queryset = Professional.objects.none()

    # Get Professional
    def post(self, request, format=None):

        response = {}
        responsedata = []
        professionaldata = []

        ProfessionalInfo = Professional.objects.filter(
            id=request.data['professional_id'])
        print(ProfessionalInfo)

        serverd_area = ProfessionalServedArea.objects.filter(
            professional_id=ProfessionalInfo.first(), is_deleted=False)
        serverd_area_serializer = ServedAreaSerializer(serverd_area, many=True)

        professional_data = ContentType.objects.get_for_model(Professional)

        working_hours = ProfessionalDailyTiming.objects.filter(
            professional_id=ProfessionalInfo.first())

        working_hours_serializer = ProfessionalDailyAvailibilitySerializer(
            working_hours, many=True)

        content_type = ContentType.objects.get_for_model(Professional)
        contact_person = Contact_Person.objects.filter(
            content_type__pk=content_type.id, object_id__in=ProfessionalInfo)

        contact_person_serializer = ContactPersonSerializer(
            contact_person, many=True)

        # Contact Info
        contact_info = ContactInfo.objects.filter(
            content_type__pk=content_type.id, object_id__in=ProfessionalInfo)
        contact_info_serializer = ContactInfoGETSerializer(
            contact_info, many=True)

        # Bank Info
        bank_obj = BankInfo.objects.filter(
            created_by=ProfessionalInfo[0].user, is_deleted=False)
        # bank_obj = BankInfo.objects.filter(
        #     content_type__pk=content_type.id, object_id__in=ProfessionalInfo)
        bank_serializer = BankInfoProfileSerializer(bank_obj, many=True)

        serializer = self.serializer_class(ProfessionalInfo.first())
        professionaldata = [{"professional": serializer.data}]

        responsedata = [{"contact_person": contact_person_serializer.data}, {
            "working_hours": working_hours_serializer.data}, {"serverd_area": serverd_area_serializer.data}, {"contact_info": contact_info_serializer.data}, {"bank_info": bank_serializer.data}]

        responsedata = professionaldata + responsedata

        response['result'] = 1
        response['data'] = responsedata
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


class AdminPOSTProfessionalAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    '''
    Update Professional Data
    '''
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = AdminProfessionalPOSTSerializer
    # servedarea_class = ServedAreaSerializer
    queryset = Professional.objects.none()

    def post(self, request, format=None):
        print(request)
        response = {}
        responsedata = []

        ProfessionalInfo = Professional.objects.get(
            id=request.data['id'])
        print(ProfessionalInfo)

        serializers = self.serializer_class(
            data=request.data, context={"id": request.data['id']})
        # serializers = self.serializer_class(
        #     data=request.data, context={"user": request.user})
        # print(serializers)
        if serializers.is_valid():
            data = serializers.save()
            # print(data)
            response['result'] = 1
            responsedata.append("Successfully Updated")
            response['errors'] = responsedata
            response['ID'] = data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializers.errors)
            response['result'] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


# Create Profile Professional

class AdminCreatePOSTProfessionalAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    '''
    Create New Professional 
    '''
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = AdminCreateProfessionalPOSTSerializer
    # servedarea_class = ServedAreaSerializer
    queryset = Professional.objects.none()

    def post(self, request, format=None):
        print(request)
        response = {}
        responsedata = []
        serializers = self.serializer_class(
            data=request.data)
        # print(serializers)
        if serializers.is_valid():
            data = serializers.save()
            # print(data)
            response['result'] = 1
            responsedata.append("Successfully Inserted")
            response['data'] = responsedata
            response['ID'] = data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializers.errors)
            response['result'] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


# Admin Stepper

class AdminServiceInfoDataAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    '''
    Add Service Image and Profession Data
    '''
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    queryset = Professional.objects.none()
    serializer_class = AdminServiceDataCustomSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []
        request.data._mutable = True
        #request.data['created_by'] = str(request.user)
        request.data['created_by'] = Professional.objects.get(
            id=request.data['id']).user
        print(request.data)
        serializer = self.serializer_class(data=request.data, context={
                                           "id": request.data['id']})
        if serializer.is_valid():
            serializer.save()
            responsedata.append("Successfully Created")[{"first_name":"Test","last_name":"asd","telephone":1234567,"email":"email"},{"first_name":"Test","last_name":"asd","telephone":1234567,"email":"email"}]


            response = handle_errors(serializer.errors)
            response['result'] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


# Stepper Certificate apis


class AdminCertificateAPIView(PlatformAPICallMixin, generics.GenericAPIView):

    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    queryset = Professional.objects.none()
    serializer_class = AdminCertificatesGetSerializer
    # GET Certificate

    def post(self, request, format=None):
        '''
        Get Certificates
        '''
        response = {}
        responsedata = []
        if Professional.objects.filter(id=request.data['id']).exists():
            professional_obj = Professional.objects.get(
                id=request.data['id'])
            certificates = ProfessionalCertificate.objects.filter(
                created_by=professional_obj.user, is_deleted=False)
            serializer = CertificatesSerializer(certificates, many=True)
        else:
            response['result'] = 0
            response['error'] = "No user found for given id"
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        response['result'] = 1
        response['data'] = serializer.data
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


class AdminPostCertificateAPIView(PlatformAPICallMixin, generics.GenericAPIView):

    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    queryset = Professional.objects.none()
    serializer_class = AdminCertificatesPostSerializer
    # Post Certificate

    def post(self, request, format=None):
        '''
        Upload Certificate
        '''
        response = {}
        responsedata = []
        request.data._mutable = True
        # request.data['created_by'] = Professional.objects.get(
        #     id=request.data['id'])
        serializer = self.serializer_class(
            data=request.data, context={"id": request.data['id']})
        if serializer.is_valid():
            serializer.save()
            responsedata.append("Successfully Uploaded")
            response['result'] = 1
            response['data'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response['result'] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class AdminCertificateDeleteAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    '''
    delete Certificates
    '''
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    queryset = Professional.objects.none()
    serializer_class = AdminDeleteCertificateSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []
        serializers = self.serializer_class(
            data=request.data)
        if serializers.is_valid():
            if Professional.objects.filter(id=serializers.validated_data['prof_id']).exists():
                professional_obj = Professional.objects.get(
                    id=serializers.validated_data['prof_id'])
                if ProfessionalCertificate.objects.filter(id=serializers.validated_data['id'], created_by=professional_obj.user).exists():
                    ProfessionalCertificate.objects.get(
                        id=serializers.validated_data['id'], created_by=professional_obj.user).delete()
                else:
                    response['result'] = 0
                    response['errors'] = "No record found"
                    self.add_api_call(request, response)
                    return Response(response, status=status.HTTP_200_OK)
            else:
                response['result'] = 0
                response['errors'] = "No user found for given id"
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)

            response['result'] = 1
            responsedata.append("Successfully Deleted")
            response['data'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializers.errors)
            response['result'] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class AdminRequestSkillAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    '''
    Add Skill
    '''
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = AdminRequestSkillsSerializer
    # Get All Skills

    @transaction.atomic
    def post(self, request, format=None):
        response = {}
        responsedata = []
        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            professional_obj = Professional.objects.get(
                id=serializers.validated_data['prof_id'])
            serializers.validated_data.pop('prof_id')

            ProfessionalRequestSkill.objects.create(
                skill_name=serializers.validated_data['skill_name'], profession=serializers.validated_data['profession'], professional=professional_obj, status_change_by=request.user, status="ACCEPTED")
            serializers.save()

            # Send response
            response['result'] = 1
            response['Message'] = "Successfully Inserted"
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class AdminSkillsSelectAPIView(PlatformAPICallMixin, generics.GenericAPIView):

    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    queryset = Professional.objects.none()
    serializer_class = AdminStepSkillsSerializer

    def post(self, request, format=None):
        '''Add Skills'''
        response = {}
        responsedata = []
        data = {}
        request.data._mutable = True
        request.data['created_by'] = Professional.objects.get(
            id=request.data['prof_id']).user
        serializer = self.serializer_class(
            data=request.data, context={'prof_id': request.data['prof_id']})
        if serializer.is_valid():
            professional_obj = Professional.objects.get(
                id=serializer.validated_data['prof_id'])
            serializer.validated_data.pop('prof_id')
            # if "profession_id" in serializer.validated_data:
            #     ProfessionalStepSkill.objects.filter(
            #         skil__profession=serializer.validated_data['profession_id'], created_by=professional_obj.user).delete()
            #     serializer.validated_data.pop("profession_id")
            # else:
            ProfessionalStepSkill.objects.filter(
                created_by=professional_obj.user).delete()
            data = serializer.save()
            response['result'] = 1
            response['message'] = "Successfully Updated"
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class ProfessionalRequestServiceAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    # authentication_classes = (TokenAuthentication,)
    # permission_classes = (IsAuthenticated,)
    serializer_class = ProfessionalRequestServiceSerializer

    def get(self, request, format=None):
        """ 
        ProfessionalRequestServiceAPIView
        """
        response = {}
        responsedata = []

        request_service = self.serializer_class.Meta.model.objects.filter(
            status="PENDING")
        request_class = self.serializer_class(request_service, many=True)

        response["result"] = 1
        response["data"] = request_class.data
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)
    
    def post(self, request, format=None):
        response = {}
        serializer = self.serializer_class(data=request.data)
        if serializer.is_valid():
            if "buisness_name_search" in serializer.validated_data:
                serializer.validated_data['professional__business_name__icontains'] = serializer.validated_data.pop("buisness_name_search")
            request_service = self.serializer_class.Meta.model.objects.filter(**serializer.validated_data)
            request_class = self.serializer_class(request_service, many=True)
            response["result"] = 1
            response["data"] = request_class.data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response['result'] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

    def patch(self, request, format=None):
        """
        Professional Request Service
        """
        try:
            response = {}
            service_obj = self.serializer_class.Meta.model.objects.filter(
                id=request.data.get("id"))
            assert service_obj.exists(), "Incorrect Services"

            serializers = self.serializer_class(
                instance=service_obj.last(), data=request.data)
            if serializers.is_valid():
                serializers.validated_data['status_change_by'] = request.user
                serializers.save()
                save_skill_service(
                    "service", serializers.validated_data['status'], service_obj.last())
                response["result"] = 1
                response["message"] = ["Successful"]
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)
            else:
                response = handle_errors(serializers.errors)
                response['result'] = 0
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)
        except AssertionError as ae:
            response["result"] = 0
            response["errors"] = [str(ae)]
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class ProfessionalRequestSkillsAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = ProfessionalRequestSkillSerializer

    def get(self, request, format=None):
        """ 
        ProfessionalRequestServiceAPIView
        """
        response = {}
        responsedata = []

        request_service = self.serializer_class.Meta.model.objects.filter(
            status="PENDING")
        request_class = self.serializer_class(request_service, many=True)

        response["result"] = 1
        response["data"] = request_class.data
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)
    
    def post(self, request, format=None):
        response = {}
        serializer = self.serializer_class(data=request.data)
        if serializer.is_valid():
            if "buisness_name_search" in serializer.validated_data:
                serializer.validated_data['professional__business_name__icontains'] = serializer.validated_data.pop("buisness_name_search")
            request_service = self.serializer_class.Meta.model.objects.filter(**serializer.validated_data)
            request_class = self.serializer_class(request_service, many=True)
            response["result"] = 1
            response["data"] = request_class.data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializers.errors)
            response['result'] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

    def patch(self, request, format=None):
        """
        Professional Request Skil
        """
        try:
            response = {}
            assert request.data.get("id", False), "No Id Provided"
            skill_obj = self.serializer_class.Meta.model.objects.filter(
                id=request.data.get("id"))
            assert skill_obj.exists(), "Incorrect Skills"

            serializers = self.serializer_class(
                instance=skill_obj.last(), data=request.data)
            if serializers.is_valid():
                serializers.validated_data['status_change_by'] = request.user
                serializers.save()
                save_skill_service(
                    "skill", serializers.validated_data['status'], skill_obj.last())
                response["result"] = 1
                response["message"] = ["Successful"]
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)
            else:
                response = handle_errors(serializers.errors)
                response['result'] = 0
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)
        except AssertionError as ae:
            response["result"] = 0
            response["errors"] = [str(ae)]
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)




from rest_framework.mixins import CreateModelMixin, RetrieveModelMixin, ListModelMixin, DestroyModelMixin
class ProfessionalDegreesAPIView(CreateModelMixin, RetrieveModelMixin,DestroyModelMixin, PlatformAPICallMixin, generics.GenericAPIView):
    # authentication_classes = (TokenAuthentication,)
    # permission_classes = (IsAuthenticated,)
    queryset = ProfessionalDocument.objects.all()
    serializer_class = ProfessionalDocumentSerializer

    def create(self,request, *args, **kwargs):
        super().create(request, *args, **kwargs)
        response = {"message":"Successfully Created", "result":1}
        return Response(response, status=status.HTTP_201_CREATED)
        
    def post(self, request, *args, **kwargs):
        self.user = request.user
        return self.create(request, *args, **kwargs)
    
    def list_data(self,request, *args, **kwargs):
        professional_id = request.GET.get("professional", False)
        professional = Professional.objects.filter(id=professional_id)
        assert professional.exists(), "Invalid Professional Id"
        professional = professional.last()
        queryset = self.serializer_class.Meta.model.objects.filter(created_by=professional.user)

        page = self.paginate_queryset(queryset)
        if page is not None:
            serializer = self.get_serializer(page, many=True)
            return self.get_paginated_response(serializer.data)

        serializer = self.get_serializer(queryset, many=True)

        response = {"data":serializer.data, "result":1}
        return Response(response, status=status.HTTP_201_CREATED)

    
    def get(self, request, *args, **kwargs):
        return self.list_data(request, *args, **kwargs)

    def destroy(self, request, *args, **kwargs):
        document_id = request.GET.get("pk", False)
        queryset = self.serializer_class.Meta.model.objects.filter(pk=document_id)
        instance = queryset
        instance.delete()
        response = {"message":"Successfully Deleted", "result":1}
        return Response(response, status=status.HTTP_201_CREATED)
    
    def delete(self, request, *args, **kwargs):
        return self.destroy(request, *args, **kwargs)


class ProfessionalCertificateAPIView(CreateModelMixin, RetrieveModelMixin,DestroyModelMixin, PlatformAPICallMixin, generics.GenericAPIView):
    # authentication_classes = (TokenAuthentication,)
    # permission_classes = (IsAuthenticated,)
    queryset = ProfessionalCertificate.objects.all()
    serializer_class = ProfessionalCertificateSerializer

    def create(self,request, *args, **kwargs):
        super().create(request, *args, **kwargs)
        response = {"message":"Successfully Created", "result":1}
        return Response(response, status=status.HTTP_201_CREATED)
        
    def post(self, request, *args, **kwargs):
        self.user = request.user
        return self.create(request, *args, **kwargs)
    
    def list_data(self,request, *args, **kwargs):
        professional_id = request.GET.get("professional", False)
        professional = Professional.objects.filter(id=professional_id)
        assert professional.exists(), "Invalid Professional Id"
        professional = professional.last()
        queryset = self.serializer_class.Meta.model.objects.filter(created_by=professional.user)

        page = self.paginate_queryset(queryset)
        if page is not None:
            serializer = self.get_serializer(page, many=True)
            return self.get_paginated_response(serializer.data)

        serializer = self.get_serializer(queryset, many=True)

        response = {"data":serializer.data, "result":1}
        return Response(response, status=status.HTTP_201_CREATED)

    
    def get(self, request, *args, **kwargs):
        return self.list_data(request, *args, **kwargs)

    def destroy(self, request, *args, **kwargs):
        document_id = request.GET.get("pk", False)
        queryset = self.serializer_class.Meta.model.objects.filter(pk=document_id)
        instance = queryset
        instance.delete()
        response = {"message":"Successfully Deleted", "result":1}
        return Response(response, status=status.HTTP_201_CREATED)
    
    def delete(self, request, *args, **kwargs):
        return self.destroy(request, *args, **kwargs)


class ProfessionalProfileAPIView(CreateModelMixin, RetrieveModelMixin, PlatformAPICallMixin, generics.GenericAPIView):
    queryset = Professional.objects.all()
    serializer_class = ProfessionalPOSTSerializer
    serializer_get_class = ProfessionalGETSerializer
    def create(self,request, *args, **kwargs):
        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            professional_id = serializers.save()
            response = {"message":"Successfully Created", "result":1, "professional_id":professional_id}
            return Response(response, status=status.HTTP_201_CREATED)
        else:
            response = handle_errors(serializers.errors)
            response['result'] = 0
            self.add_api_call(request, response)
            return Response(response,status=status.HTTP_200_OK)

    def retrieve(self, request, *args, **kwargs):
        pk = request.GET.get("professional_id", False)
        assert pk, "Professional_id Instance ID"
        instance = Professional.objects.filter(id=pk)
        assert instance.exists(), "Invalid Instance ID"
        instance = instance.last()
        serializer = self.serializer_get_class(instance)
        response = {"data":serializer.data, "result":1}
        return Response(response, status=status.HTTP_201_CREATED)

    @transaction.atomic
    def post(self, request, *args, **kwargs):
        # Check for django User
        print(request.data['email'])
        django_user = User.objects.filter(username=request.data['email'])
        print(django_user)
        assert not django_user.exists(), "Email address already registered"
        django_user = User.objects.create(username=request.data['email'], email=request.data['email'])
        request.data._mutable=True
        print(django_user.id)
        request.data['user'] = django_user.id
        request.data._mutable=False
        return self.create(request, *args, **kwargs)
    
    def get(self, request, *args, **kwargs):
        return self.retrieve(request)





class ProfessionalServiceInfoProfileAPIView(CreateModelMixin,PlatformAPICallMixin, generics.GenericAPIView):
    serializer_class = ServiceDataCustomSerializer
    serializer_get_class = ServiceDataPhotoSerializer

    def create(self,request, *args, **kwargs):
        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            professional_id = serializers.save()
            response = {"message":"Successfully Created", "result":1}
            return Response(response, status=status.HTTP_201_CREATED)
        else:
            response = handle_errors(serializers.errors)
            response['result'] = 0
            self.add_api_call(request, response)
            return Response(response,status=status.HTTP_200_OK)


    @transaction.atomic
    def post(self, request, *args, **kwargs):
        # Check for django User
        return self.create(request, *args, **kwargs)

    def retrieve(self, request, *args, **kwargs):
        pk = request.GET.get("professional_id", False)
        assert pk, "Professional_id Instance ID"
        professional_instance = Professional.objects.filter(id=pk)
        assert professional_instance.exists(), "Invalid Instance ID"
        professional_instance = professional_instance.last()
        instance = self.serializer_get_class.Meta.model.objects.filter(user=professional_instance.user)
        assert instance.exists(), "Invalid Instance ID"
        instance = instance.last()
        serializer = self.serializer_get_class(instance)
        response = {"data":serializer.data, "result":1}
        return Response(response, status=status.HTTP_201_CREATED)

    def get(self, request, *args, **kwargs):
        return self.retrieve(request)




class ProfessionalYourSkillsProfileAPIView(CreateModelMixin,PlatformAPICallMixin, generics.GenericAPIView):
    serializer_class = StepSkillsSerializer
    serializer_get_class = GetServiceSkillSerializer

    def create(self,request, *args, **kwargs):
        serializers = self.serializer_class(data=request.data, context={"user":self.user})
        if serializers.is_valid():
            if "profession_id":
                ProfessionalStepSkill.objects.filter(
                    skil__profession=serializers.validated_data['profession_id'], created_by=self.user).delete()
                serializers.validated_data.pop("profession_id")
            else:
                ProfessionalStepSkill.objects.filter(
                    created_by=request.user).delete()
            professional_id = serializers.save()
            response = {"message":"Successfully Created", "result":1}
            return Response(response, status=status.HTTP_201_CREATED)
        else:
            response = handle_errors(serializers.errors)
            response['result'] = 0
            self.add_api_call(request, response)
            return Response(response,status=status.HTTP_200_OK)


    @transaction.atomic
    def post(self, request, *args, **kwargs):
        # Check for django User
        django_user = User.objects.filter(username=request.data['created_by'])
        print(django_user)
        assert django_user.exists(), "No Such User Found"
        self.user = request.data['created_by']
        self.user = django_user.last()
        return self.create(request, *args, **kwargs)
    

    def retrieve(self, request, *args, **kwargs):
        pk = request.GET.get("professional_id", False)
        assert pk, "Professional_id Instance ID"
        professional_instance = Professional.objects.filter(id=pk)
        assert professional_instance.exists(), "Invalid Instance ID"
        professional_instance = professional_instance.last()
        instance = self.serializer_get_class.Meta.model.objects.filter(created_by=professional_instance.user)
        assert instance.exists(), "Invalid Instance ID"
        instance = instance.last()
        serializer = self.serializer_get_class(instance)
        response = {"data":serializer.data, "result":1}
        return Response(response, status=status.HTTP_201_CREATED)

    def get(self, request, *args, **kwargs):
        return self.retrieve(request)


class ProfessionalYourCertificatesAPIView(CreateModelMixin, PlatformAPICallMixin, generics.GenericAPIView):
    serializer_class = CertificatesSerializer

    def create(self,request, *args, **kwargs):
        serializers = self.serializer_class(data=request.data, context={"user":self.user})
        if serializers.is_valid():
            serializers.validated_data.pop("professional_email")
            professional_id = serializers.save()
            response = {"message":"Successfully Created", "result":1}
            return Response(response, status=status.HTTP_201_CREATED)
        else:
            response = handle_errors(serializers.errors)
            response['result'] = 0
            self.add_api_call(request, response)
            return Response(response,status=status.HTTP_200_OK)


    @transaction.atomic
    def post(self, request, *args, **kwargs):
        # Check for django User
        django_user = User.objects.filter(username=request.data['professional_email'])
        print(django_user)
        assert django_user.exists(), "No Such User Found"
        self.user = django_user.last()
        return self.create(request, *args, **kwargs)
    

from professionals.serializers import ProfessionalContractSerializer




class ProfessionalConListView(CreateModelMixin, RetrieveModelMixin, ListModelMixin,DestroyModelMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    queryset = ProfessionalContract.objects.all()
    serializer_class = ProfessionalContractSerializer

    def create(self,request, *args, **kwargs):
        super().create(request, *args, **kwargs)
        response = {"message":"Successfully Created", "result":1}
        return Response(response, status=status.HTTP_201_CREATED)
        
    def post(self, request, *args, **kwargs):
        return self.create(request, *args, **kwargs)
    
    def get(self, request, *args, **kwargs):
        return self.list(request, *args, **kwargs)



class ProfessionalConDetailView(RetrieveModelMixin,DestroyModelMixin, generics.GenericAPIView):
    queryset = ProfessionalContract.objects.all()
    serializer_class = ProfessionalContractSerializer

    def destroy(self, request, *args, **kwargs):
        super().destroy(request, *args, **kwargs)
        response = {"message":"Successfully Deleted", "result":1}
        return Response(response, status=status.HTTP_201_CREATED)

    
    def get(self, request, *args, **kwargs):
        return self.retrieve(request, *args, **kwargs)

    def delete(self, request, *args, **kwargs):
        return self.destroy(request, *args, **kwargs)