import pytz
import json
from django.http import HttpResponse
from django.template import Template, Context
from django.template.loader import get_template
from django.db import transaction
from django.contrib.contenttypes.models import ContentType
from django.urls import reverse
from django.contrib.auth.models import Group
from django.contrib.auth import login as django_login
import datetime
import calendar
from django.utils import timezone
from rest_framework.authtoken.models import Token
from rest_framework import fields, status, generics
from rest_framework.response import Response
from rest_framework.authentication import TokenAuthentication
from rest_framework.permissions import IsAuthenticated
from utils.mixins import *
from utils.api_handle_errors import handle_errors

from admin_app.serializers import *

from venue_owners.models import *
from venue_owners.serializers import *
from venues.models import *
from venues.serializers import *
from players.models import BookMatch
from commonAPI.models import *
from commonAPI.serializers import OnBoardAPIGETSerializer, OnBoardAPIPOSTSerializer
from django.db.models import Q, Avg
from reviews.models import Review, VenueReviewAndRating
from reviews.serializers import ReviewGETSerializer
from commonAPI.models import Contact_Person
from utils.mail import send_emal_onboard_accept, send_mail_registeration, field_approval_email, \
    in_progress_email_verification, onboard_Complete_mail, onboard_delete_or_declined_mail
from commonAPI.serializers import ContactPersonSerializer, BankInfoProfileSerializer
from field_booking.settings import DOMAIN_NAME
from platform_central.models import RegisterPrivateKey
from message.serializers import MessagesInboxSerializer
from message.models import MessagesInbox, Notification
from advertisers.models import Advertiser
from static_data.models import *
from static_data.serializers import *
from message.serializers import NotificationsSerializer
from campaigns.serializers import CampaignPart4GETSerializer, Campaign3Get, CampaignPart2SerializerGET
from campaigns.models import CampaignPart1, CampaignPart2, CampaignPart3, CampaignPart4
from professionals.models import Professional


class AdminScheduleListGETAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = AdminScheduleLISTSerializer

    def post(self, request, format=None):
        try:
            response = {}
            responsedata = []
            serializer = self.serializer_class(data=request.data)
            if serializer.is_valid():

                vobj = VenueOwner.objects.get(
                    id=serializer.validated_data['venueowner_id'])
                # print(vobj.user.id)

                # Get page number
                start_count = (serializer.validated_data['page'] - 1) * 10
                end_count = start_count + 10
                serializer.validated_data.pop('page')

                serializer.validated_data['created_by'] = vobj.user.id

                serializer.validated_data.pop('venueowner_id')

                if "assortment" in request.data:
                    assortment = serializer.validated_data['assortment']
                    serializer.validated_data.pop('assortment')

                    if "keyword" in serializer.validated_data:
                        serializer.validated_data['schedule_name__icontains'] = serializer.validated_data['keyword']
                        serializer.validated_data.pop("keyword")

                    # Check for in use
                    if "active" in serializer.validated_data:
                        if serializer.validated_data['active'] == 'No':
                            serializer.validated_data['start_date__gt'] = timezone.now(
                            ).date()
                        else:
                            serializer.validated_data['start_date__lte'] = timezone.now(
                            ).date()
                        serializer.validated_data.pop('active')

                    data = Schedule.objects.filter(**serializer.validated_data).order_by(
                        "-created_at").order_by(assortment)[start_count:end_count]
                    total_count = Schedule.objects.filter(
                        **serializer.validated_data).order_by(assortment).count()

                else:

                    if "keyword" in serializer.validated_data:
                        serializer.validated_data['schedule_name__icontains'] = serializer.validated_data['keyword']
                        serializer.validated_data.pop("keyword")

                     # Check for in use
                    if "active" in serializer.validated_data:
                        if serializer.validated_data['active'] == 'No':
                            serializer.validated_data['start_date__gt'] = timezone.now(
                            ).date()
                        else:
                            serializer.validated_data['start_date__lte'] = timezone.now(
                            ).date()
                        serializer.validated_data.pop('active')

                    data = Schedule.objects.filter(
                        **serializer.validated_data).order_by("-created_at")[start_count:end_count]
                    total_count = Schedule.objects.filter(
                        **serializer.validated_data).count()
                serializer = self.serializer_class(data, many=True)

                for count, i in enumerate(serializer.data):
                    if data[count].start_date > timezone.now().date():
                        i['in_use'] = False
                    else:
                        i['end_date'] = data[count].end_date
                        i['current date'] = timezone.now().date()
                        i['in_use'] = True

                response["result"] = 1
                response['data'] = serializer.data
                response['total_count'] = total_count
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)
            else:
                response = handle_errors(serializer.errors)
                response['result'] = 0
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_400_BAD_REQUEST)
        except Exception as e:
            response["result"] = 0
            responsedata.append(str(e))
            response['errors'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class AdminScheduleListDELETEAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = AdminScheduleLISTDELETESerializer

    def post(self, request, format=None):
        try:
            response = {}
            responsedata = []
            v_owner_id = VenueOwner.objects.get(
                id=request.data['id'])
            print(v_owner_id.user)

            serializer = self.serializer_class(instance=Schedule.objects.get(
                id=request.data['schedule_id']), data=request.data, context={"id": request.data['id']})
            if serializer.is_valid():
                # Check if schedule is running
                schedule_obj = Schedule.objects.get(
                    id=serializer.validated_data['schedule_id'])
                # Get current time
                current_datetime = timezone.now().date()

                if schedule_obj.start_date <= current_datetime:
                    response["result"] = 0
                    responsedata.append("Schedule is already active")
                    response['errors'] = responsedata
                    self.add_api_call(request, response)
                    return Response(response, status=status.HTTP_200_OK)

                seconds_difference = schedule_obj.created_at.date() - current_datetime
                print(type(seconds_difference))
                print(seconds_difference.total_seconds())
                param_obj = Schedule_Parameter.objects.filter().first()
                if seconds_difference.total_seconds() <= param_obj.schedule_delete*86400:
                    response["result"] = 0
                    responsedata.append(
                        "Cannot delete a schedule until "+str(param_obj.schedule_delete)+" days after its creation")
                    response['errors'] = responsedata
                    self.add_api_call(request, response)
                    return Response(response, status=status.HTTP_200_OK)

                serializer.save()
                response["result"] = 1
                responsedata.append("Successfully Deleted")
                response['data'] = responsedata
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)
            else:
                response = handle_errors(serializer.errors)
                response['result'] = 0
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_400_BAD_REQUEST)
        except Exception as e:
            response["result"] = 0
            responsedata.append(str(e))
            response['errors'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class AdminScheduleGETTableAPIView(PlatformAPICallMixin, generics.GenericAPIView):

    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = AdminScheduleGETSerializer
    templateSerializer_class = ScheduleTemplateScheduleSerializer
    SlotSerializer_class = ScheduleTemplateSlotsSerialzier
    pricelistSerializer_class = PriceList2ScheduleSerializer
    schedule_slots_class = ScheduleSlotSerializer

    def post(self, request, format=None):
        try:
            response = {}
            responsedata = []
            v_owner = VenueOwner.objects.get(
                id=request.data['venueowner_id'])
            print(v_owner.user)

            serializers = self.serializer_class(
                data=request.data, context={"venueowner_id": request.data['venueowner_id']})
            if serializers.is_valid():

                if "id" in serializers.validated_data:
                    schedule_obj = Schedule.objects.get(
                        id=serializers.validated_data['id'], created_by=v_owner.user)
                    scheudle_slots = Schedule_Slot.objects.filter(
                        schedule_id=schedule_obj, is_deleted=False)
                    template_obj = schedule_obj.template

                    start_hour = template_obj.start_hour
                    end_hour = template_obj.end_hour

                    plan_range = 1 if template_obj.plan_range == "D" else 7 if template_obj.plan_range == "W" else 31

                    response_data = []
                    token_1 = 0
                    while start_hour < end_hour:

                        data = {}
                        token = 0
                        start_date = schedule_obj.start_date
                        end_date = schedule_obj.end_date
                        break_down = 0

                        while start_date < end_date:
                            if (datetime.datetime.combine(datetime.date.today(), start_hour) + datetime.timedelta(minutes=template_obj.interval) + datetime.timedelta(minutes=template_obj.gap_period)).time() > end_hour:
                                break
                            scheudle_slots = Schedule_Slot.objects.filter(
                                schedule_id=schedule_obj.id, date=start_date, start_hour=start_hour, is_deleted=False).first()
                            scheudle_slots_class = self.schedule_slots_class(
                                scheudle_slots)
                            data[token] = scheudle_slots_class.data
                            start_date = start_date + \
                                datetime.timedelta(days=1)
                            token = token + 1

                            if break_down > 500:
                                break
                            else:
                                break_down = break_down + 1

                        start_hour = (datetime.datetime.combine(datetime.date.today(), start_hour) + datetime.timedelta(
                            minutes=template_obj.interval) + datetime.timedelta(minutes=template_obj.gap_period)).time()
                        if len(data) == 0:
                            pass
                        else:
                            response_data.append(data)
                        token_1 = token_1 + 1

                    response['results'] = 1
                    response['data'] = response_data
                    response['plan_range'] = plan_range
                    # response['schedule_data'] = scheudle_slots_class.data
                    self.add_api_call(request, response)
                    return Response(response, status=status.HTTP_200_OK)

                time_slots_price = serializers.validated_data['time_slot_price']
                template = ScheduleTemplateStep1.objects.get(
                    id=serializers.validated_data['Template_Id'], created_by=v_owner.user)
                plan_range = 1 if template.plan_range == "D" else 7 if template.plan_range == "W" else 31
                start_date = serializers.validated_data['start_date']
                end_date = start_date + datetime.timedelta(days=plan_range)

                start_hour = template.start_hour
                end_hour = template.end_hour

                datalist = []
                break_down = 0

                if plan_range == 7:
                    date_day_array = []
                    for d in range(0, 7):
                        wd = {}
                        wd['date'] = start_date+datetime.timedelta(days=d)
                        wd['day'] = calendar.day_name[wd['date'].weekday()]
                        date_day_array.append(wd)
                    workinghours = VenueDailyTiming.objects.filter(
                        venue__created_by=v_owner.user)
                    '''days_array=[]
                    for w in range(0,workinghours.count()):
                        days_array.append(workinghours[w].day)

                    print(days_array)'''
                    print(date_day_array)
                    slots = ScheduleTemplateSlot.objects.filter(
                        template=template.id)
                    slotsSerializer = self.SlotSerializer_class(
                        slots, many=True)
                    if len(slotsSerializer.data) > 0:
                        for index, i in enumerate(slotsSerializer.data):
                            data = {}
                            dayname = ""
                            if i['day'] == 0:
                                dayname = "Monday"
                            if i['day'] == 1:
                                dayname = "Tuesday"
                            if i['day'] == 2:
                                dayname = "Wednesday"
                            if i['day'] == 3:
                                dayname = "Thursday"
                            if i['day'] == 4:
                                dayname = "Friday"
                            if i['day'] == 5:
                                dayname = "Saturday"
                            if i['day'] == 6:
                                dayname = "Sunday"

                            for rec in date_day_array:
                                if rec['day'] == dayname:
                                    current_date = rec['date']

                            data['date'] = current_date
                            data['day'] = dayname
                            data['is_available'] = i['is_available']
                            data['start_hour'] = i['start_hour']
                            data['end_hour'] = i['end_hour']
                            which_has_max_weightage = []
                            for cnt in range(0, len(time_slots_price)):
                                sh = datetime.datetime.strptime(
                                    data['start_hour'], '%H:%M:%S')
                                eh = datetime.datetime.strptime(
                                    data['end_hour'], '%H:%M:%S')

                                dt = {}
                                count_of_weight = 0
                                while sh <= eh:
                                    if datetime.datetime.strptime(time_slots_price[cnt]['start_hour'], '%H:%M:%S') < sh and sh < datetime.datetime.strptime(time_slots_price[cnt]['end_hour'], '%H:%M:%S'):
                                        count_of_weight = count_of_weight+1
                                    sh = sh+datetime.timedelta(seconds=60)
                                dt["weight"] = count_of_weight
                                dt["slot_sh"] = data["start_hour"]
                                dt["slot_eh"] = data["end_hour"]
                                dt["duration"] = time_slots_price[cnt]
                                which_has_max_weightage.append(dt)

                            # print(which_has_max_weightage)
                            winner = 0
                            for z in range(0, len(which_has_max_weightage)):
                                if which_has_max_weightage[z]['weight'] > which_has_max_weightage[winner]['weight']:
                                    winner = z
                            if which_has_max_weightage[winner]['weight'] == 0:
                                data['cost'] = 0
                            else:
                                data['cost'] = time_slots_price[winner]['cost']
                            datalist.append(data)

                else:
                    while start_hour < end_hour:

                        if (datetime.datetime.combine(datetime.date.today(), start_hour) + datetime.timedelta(minutes=template.interval)).time() > end_hour:
                            break

                        slots = ScheduleTemplateSlot.objects.filter(
                            template=template.id, start_hour=start_hour)
                        slotsSerializer = self.SlotSerializer_class(
                            slots, many=True)

                        if len(slotsSerializer.data) == 0:
                            # start_hour = slots[0].end_hour + datetime.timedelta(minutes=template.gap_period)
                            break

                        token = 0
                        main_obj = {}

                        for index, i in enumerate(slotsSerializer.data):
                            data = {}
                            current_date = start_date + \
                                datetime.timedelta(days=i['day'])
                            data['date'] = current_date
                            data['day'] = calendar.day_name[current_date.weekday()]
                            data['is_available'] = i['is_available']
                            data['start_hour'] = i['start_hour']
                            data['end_hour'] = i['end_hour']
                            which_has_max_weightage = []
                            for cnt in range(0, len(time_slots_price)):
                                sh = datetime.datetime.strptime(
                                    data['start_hour'], '%H:%M:%S')
                                eh = datetime.datetime.strptime(
                                    data['end_hour'], '%H:%M:%S')

                                dt = {}
                                count_of_weight = 0
                                while sh <= eh:
                                    if datetime.datetime.strptime(time_slots_price[cnt]['start_hour'], '%H:%M:%S') < sh and sh < datetime.datetime.strptime(time_slots_price[cnt]['end_hour'], '%H:%M:%S'):
                                        count_of_weight = count_of_weight+1
                                    sh = sh+datetime.timedelta(seconds=60)
                                dt["weight"] = count_of_weight
                                dt["slot_sh"] = data["start_hour"]
                                dt["slot_eh"] = data["end_hour"]
                                dt["duration"] = time_slots_price[cnt]
                                which_has_max_weightage.append(dt)

                            # print(which_has_max_weightage)
                            winner = 0
                            for z in range(0, len(which_has_max_weightage)):
                                if which_has_max_weightage[z]['weight'] > which_has_max_weightage[winner]['weight']:
                                    winner = z
                            if which_has_max_weightage[winner]['weight'] == 0:
                                data['cost'] = 0
                            else:
                                data['cost'] = time_slots_price[winner]['cost']
                            main_obj[token] = data
                            token = token + 1

                        datalist.append(main_obj)
                        start_hour = (datetime.datetime.combine(datetime.date.today(
                        ), slots[0].end_hour) + datetime.timedelta(minutes=template.gap_period)).time()

                        if break_down > 500:
                            break
                        else:
                            break_down = break_down + 1

                responsedata.append(datalist)
                response['results'] = 1
                response['data'] = datalist
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)
            else:
                response = handle_errors(serializers.errors)
                response['result'] = 0
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_400_BAD_REQUEST)
        except Exception as e:
            response["result"] = 0
            responsedata.append(str(e))
            response['errors'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class AdminSchedulePOSTTableAPIView(PlatformAPICallMixin, generics.GenericAPIView):

    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = AdminSchedulePOSTSerializer

    SlotSerializer_class = Schedule_SlotSerializer
    templateSerializer_class = ScheduleTemplateScheduleSerializer
    pricelistSerializer_class = PriceList2ScheduleSerializer

    def post(self, request, format=None):
        try:
            response = {}
            responsedata = []
            v_owner = VenueOwner.objects.get(
                id=request.data['venueowner_id'])
            print(v_owner.user)

            serializers = self.serializer_class(
                data=request.data, context={"venueowner_id": request.data['venueowner_id']})
            if serializers.is_valid():
                serializers.validated_data.pop('venueowner_id')
                if "id" in serializers.validated_data:
                    # Check if id exists for slot update
                    if Schedule.objects.filter(id=serializers.validated_data['id'], created_by=v_owner.user, is_deleted=False).exists():
                        schedule_obj = Schedule.objects.get(
                            id=serializers.validated_data['id'], created_by=v_owner.user, is_deleted=False)
                        # UPdate Slots
                        schedule_slot_obj = Schedule_Slot.objects.filter(
                            schedule_id=schedule_obj, is_deleted=False).update(is_deleted=True)

                        for i in serializers.validated_data['slots']:
                            i['day'] = DayofWeek.objects.get(id=int(i['day']))

                        slots = [Schedule_Slot(
                            **i,
                            schedule_id=schedule_obj
                        ) for i in serializers.validated_data['slots']]

                        Schedule_Slot.objects.bulk_create(slots)

                        response['result'] = 1
                        response['data'] = ["Successfully Updated"]
                        self.add_api_call(request, response)
                        return Response(response, status=status.HTTP_200_OK)

                    else:
                        response['result'] = 0
                        response['errors'] = ["No Such Schedule Exists"]
                        self.add_api_call(request, response)
                        return Response(response, status=status.HTTP_200_OK)

                else:
                    if "schedule_name" not in serializers.validated_data or "field" not in serializers.validated_data:
                        response['result'] = 0
                        response['errors'] = [
                            "No Schedule Name and/or Field sent"]
                        self.add_api_call(request, response)
                        return Response(response, status=status.HTTP_200_OK)

                if 'confirmation' in serializers.validated_data:
                    for i in serializers.validated_data['confirmation']:
                        delete_slot = Schedule_Slot.objects.get(id=i['id'])
                        delete_slot.is_deleted = True
                        delete_slot.save()

                days = 1 if serializers.validated_data[
                    'template'].plan_range == "D" else 7 if serializers.validated_data['template'].plan_range == "W" else 31

                end_date = serializers.validated_data['end_date']
                schedule = Schedule.objects.filter(Q(start_date__range=(serializers.validated_data['start_date'], end_date), created_by=v_owner.user, is_deleted=False, field=serializers.validated_data['field']) | Q(
                    end_date__range=(serializers.validated_data['start_date'], end_date), created_by=v_owner.user, is_deleted=False, field=serializers.validated_data['field']))
                if (len(schedule) > 0):
                    reserved_slots = []
                    ignore_slots = []
                    for i in serializers.validated_data['slots']:
                        for j in schedule:
                            # scheduleSlot = Schedule_Slot.objects.filter(date = i['date'],start_hour__lte=i['start_hour'],schedule_id=j,is_deleted=False).first()
                            scheduleSlot = Schedule_Slot.objects.filter(Q(start_hour=i['start_hour'], end_hour=i['end_hour'], schedule_id=j, is_deleted=False) | Q(
                                start_hour__lte=i['start_hour'], end_hour__gt=i['start_hour'], schedule_id=j, is_deleted=False) | Q(start_hour__lt=i['end_hour'], end_hour__gt=i['end_hour'], schedule_id=j, is_deleted=False)).first()
                            slot_serializer = self.SlotSerializer_class(
                                scheduleSlot)
                            if scheduleSlot is not None:
                                if 'confirmation' in serializers.validated_data:
                                    ignore_slots.append(slot_serializer.data)
                                else:
                                    reserved_slots.append(slot_serializer.data)
                            else:
                                serializers.validated_data['end_date'] = end_date
                                serializers.save()
                                response['result'] = 1
                                response['data'] = ["Successful"]
                                self.add_api_call(request, response)
                                return Response(response, status=status.HTTP_200_OK)

                    if 'confirmation' in serializers.validated_data:
                        for i in ignore_slots:
                            serializers.validated_data['slots'] = [x for x in serializers.validated_data['slots'] if str(x['start_hour']) != str(
                                i['start_hour']) or str(x['end_hour']) != str(i['end_hour']) or str(x['date']) != str(i['date'])]
                        serializers.validated_data.pop('confirmation')
                        serializers.save()
                        response['result'] = 1
                        response['data'] = ["Successful"]
                        self.add_api_call(request, response)
                        return Response(response, status=status.HTTP_200_OK)
                    else:
                        response['result'] = 2
                        response['message'] = ["Time slot already reserved"]
                        response['data'] = reserved_slots
                        self.add_api_call(request, response)
                        return Response(response, status=status.HTTP_200_OK)
                else:
                    serializers.validated_data['end_date'] = end_date
                    serializers.save()
                    response['result'] = 1
                    response['data'] = ["Successful"]
                    self.add_api_call(request, response)
                    return Response(response, status=status.HTTP_200_OK)
            else:
                response = handle_errors(serializers.errors)
                response['result'] = 0
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)
        except Exception as e:
            response["result"] = 0
            responsedata.append(str(e))
            response['errors'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class AdminGetSchedule1(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = AdminScheduleEditGetSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []
        v_owner = VenueOwner.objects.get(
            id=request.data['venueowner_id'])
        print(v_owner.user)

        #user_obj = request.user
        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            if Schedule.objects.filter(id=serializers.validated_data['id'], created_by=v_owner.user, is_deleted=False).exists():
                data = []
                schedule_data = Schedule.objects.get(
                    id=serializers.validated_data['id'])
                start_date = schedule_data.start_date
                end_date = schedule_data.end_date
                current_date = start_date
                while current_date <= end_date:
                    inner_data = {}
                    slots_for_date = Schedule_Slot.objects.filter(
                        schedule_id=schedule_data.id, date=current_date, is_deleted=False)
                    if slots_for_date.count() > 0:
                        inner_data['date_of_day'] = current_date.strftime(
                            "%d/%m/%Y")
                        inner_data['day_name'] = calendar.day_name[current_date.weekday()]
                        slots_list = []
                        for i in range(0, slots_for_date.count()):
                            obj = {}
                            obj['date'] = slots_for_date[i].date
                            obj['day'] = slots_for_date[i].day.dayofweek
                            obj['start_hour'] = slots_for_date[i].start_hour
                            obj['end_hour'] = slots_for_date[i].end_hour
                            obj['cost'] = slots_for_date[i].cost
                            obj['is_available'] = slots_for_date[i].is_available
                            slots_list.append(obj)
                        inner_data['slots'] = slots_list
                    else:
                        inner_data['date_of_day'] = current_date.strftime(
                            "%d/%m/%Y")
                        inner_data['day_name'] = calendar.day_name[current_date.weekday()]
                        inner_data['slots'] = []

                    data.append(inner_data)
                    current_date = current_date+datetime.timedelta(days=1)

                time_slots_prices = Time_Slot_Price.objects.filter(
                    schedule_id=schedule_data)
                sl_list = []
                for z in range(0, time_slots_prices.count()):
                    o = {}
                    o['start_hour'] = time_slots_prices[z].start_hour
                    o['end_hour'] = time_slots_prices[z].end_hour
                    o['cost'] = time_slots_prices[z].cost
                    sl_list.append(o)

                dd = (datetime.datetime.now(datetime.timezone.utc) -
                      schedule_data.created_at).days
                schedule_details = {}
                schedule_details['name'] = schedule_data.schedule_name
                schedule_details['field'] = schedule_data.field.id
                schedule_details['field_name'] = schedule_data.field.name
                schedule_details['schedule_template'] = schedule_data.template.id
                schedule_details['schedule_template_name'] = schedule_data.template.template_name

                schedule_details['start_date'] = schedule_data.start_date.strftime(
                    "%d-%m-%Y")
                schedule_details['end_date'] = schedule_data.end_date.strftime(
                    "%d-%m-%Y")
                schedule_details['days_from_creation'] = dd
                # schedule_details['days_from_creation']=11
                schedule_details['plan_range'] = schedule_data.template.plan_range
                schedule_details['prices'] = sl_list
                response['result'] = 1
                response['data'] = data
                l = []
                l.append(schedule_details)
                print(l)
                print(schedule_details)
                response['schedule_details'] = l
                # response['pricelist']=pricelist

                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)
            else:
                response['result'] = 0
                response['error'] = ["No such schedule exist"]
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)

        else:
            response['result'] = 0
            response['error'] = ["Schedule Id is not valid"]
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class AdminEditScheduleOnlyHours(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = AdminScheduleEditPOSTSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []
        user_obj = request.user
        serializers = self.serializer_class(
            data=request.data, context={"venueowner_id": request.data['venueowner_id']})
        if serializers.is_valid():
            serializers.save()
            response['result'] = 1
            response['message'] = ["Schedule Edited Successfully"]
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response['result'] = 0
            response['error'] = ["Schedule Id is not valid"]
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class AdminEditScheduleAll(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = AdminScheduleEditPOST2Serializer

    def post(self, request, format=None):
        response = {}
        responsedata = []
        v_owner = VenueOwner.objects.get(
            id=request.data['venueowner_id'])
        print(v_owner.user)
        serializers = self.serializer_class(
            data=request.data, context={"venueowner_id": request.data['venueowner_id']})
        if serializers.is_valid():
            if Schedule.objects.filter(id=serializers.validated_data['id'], created_by=v_owner.user).exists():
                schedule = Schedule.objects.filter(Q(start_date__range=(serializers.validated_data['start_date'], serializers.validated_data['end_date']), created_by=v_owner.user, is_deleted=False, field=serializers.validated_data['field']) | Q(
                    end_date__range=(serializers.validated_data['start_date'], serializers.validated_data['end_date']), created_by=v_owner.user, is_deleted=False, field=serializers.validated_data['field']))
                if (len(schedule) > 1):
                    response['result'] = 0
                    response['message'] = [
                        "Some schedule is already applied on the selected field for the particular date durations"]
                    self.add_api_call(request, response)
                    return Response(response, status=status.HTTP_200_OK)
                elif (len(schedule) == 1):
                    if schedule[0].id == serializers.validated_data['id']:
                        serializers.save()
                        response['result'] = 1
                        response['message'] = ["Successfully Edited schedule"]
                        self.add_api_call(request, response)
                        return Response(response, status=status.HTTP_200_OK)
                    else:
                        response['result'] = 0
                        response['message'] = [
                            "Some schedule is already applied on the selected field for the particular date durations"]
                        self.add_api_call(request, response)
                        return Response(response, status=status.HTTP_200_OK)

                else:
                    serializers.save()
                    response['result'] = 1
                    response['message'] = ["Successfully Edited schedule"]
                    self.add_api_call(request, response)
                    return Response(response, status=status.HTTP_200_OK)
            else:
                response['result'] = 0
                response['message'] = ["No such schedule found"]
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)
        else:
            response['result'] = 0
            response['message'] = ["Schedule Id is not valid"]
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class AdminGetFieldsAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = AdminScheduleGetFieldSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []

        # get user

        v_owner = VenueOwner.objects.get(
            id=request.data['id'])
        print(v_owner.user)
        # Get fields of user
        venue_obj = Venue.objects.filter(created_by=v_owner.user).last()
        # fields = VenueField.objects.filter(created_by=user_obj, venue=venue_obj, is_deleted=False)
        serializer = self.serializer_class(request.data)

        fields = VenueField.objects.filter(
            created_by=v_owner.user, is_deleted=False)
        print(fields)

        fields_class = self.serializer_class(fields, many=True)

        response['result'] = 1
        response['data'] = fields_class.data
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)
