import uuid

import pytz
import json
from django.http import HttpResponse
from django.template import Template, Context
from django.template.loader import get_template
from django.db import transaction
from django.contrib.contenttypes.models import ContentType
from django.urls import reverse
from django.contrib.auth.models import Group
from django.contrib.auth import login as django_login
import datetime
from django.utils import timezone
from rest_framework.authtoken.models import Token
from rest_framework import status, generics
from rest_framework.response import Response
from rest_framework.authentication import TokenAuthentication
from rest_framework.permissions import IsAuthenticated
from utils.mixins import *
from utils.api_handle_errors import handle_errors
from admin_app.serializers import *
from venue_owners.models import *
from venue_owners.serializers import *
from venues.models import *
from venues.serializers import *
from players.models import BookMatch
from django.db.models import Q, Avg
from field_booking.settings import DOMAIN_NAME
from platform_central.models import RegisterPrivateKey
from static_data.models import *
from static_data.serializers import *


class SportsPostAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = SportsPostSerializer

    def post(self, request, format=None):
        '''
        Add Sports
        '''
        responsedata = []
        response = {}

        if Sport.objects.filter(name=request.data['name'], catagory=request.data['catagory']).exists():
            response["result"] = 0
            response['data'] = 'Sport already exists'
            return Response(response, status=status.HTTP_200_OK)
        else:
            serializer = self.serializer_class(data=request.data)
            message = "Created Successfully"

        if serializer.is_valid():
            serializer.save()
            response["result"] = 1
            response['data'] = [message]
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class AdminSportsGetAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    """
    Get all Sports
    """
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = AllSportsGetSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []
        if Sport.objects.filter(catagory=request.data['catagory']).exists():
            data = Sport.objects.filter(catagory=request.data['catagory'])
            serializer = self.serializer_class(data, many=True)
        else:
            response["result"] = 0
            response['data'] = 'No record found'
            return Response(response, status=status.HTTP_200_OK)
        response['result'] = 1
        response['data'] = serializer.data
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


class SportsEditAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = SportsEditSerializer

    def post(self, request, format=None):
        '''
        Update Sport
        '''
        responsedata = []
        response = {}

        if Sport.objects.filter(id=request.data['id']).exists():
            serializer = self.serializer_class(instance=Sport.objects.filter(
                id=request.data['id']).first(), data=request.data)
            message = "Updated Successfully"
        else:
            response["result"] = 0
            response['data'] = 'No record found'
            return Response(response, status=status.HTTP_200_OK)

        if serializer.is_valid():
            serializer.save()
            response["result"] = 1
            response['data'] = [message]
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class SportDeleteAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = DeleteSportSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []

        if not Sport.objects.filter(id=request.data['id']).exists():
            response['results'] = 0
            response['error'] = "Not a valid ID"
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        serializers = self.serializer_class(data=request.data)

        if serializers.is_valid():
            Sport_obj = Sport.objects.get(
                id=serializers.validated_data['id'])
            Sport_obj.delete()

            response['results'] = 1
            response['data'] = "Successfully Deleted"
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        else:
            response['results'] = 0
            response = handle_errors(serializers.errors)
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class GetSingleSportAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = SingleSportSerializer

    def post(self, request, format=None):
        '''
        Get Sport by Id
        '''
        response = {}
        responsedata = []

        serializer = self.serializer_class(data=request.data)
        if serializer.is_valid():

            if Sport.objects.filter(id=serializer.validated_data["id"]).exists():
                Sport_obj = Sport.objects.get(
                    id=serializer.validated_data["id"])
            else:
                response["result"] = 0
                response['error'] = "Invalid Sport id"
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)

            serializer = SingleSportGetSerializer(Sport_obj)

            response["result"] = 1
            response['data'] = serializer.data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


# winter Sports Apis


class WinterSportsPostAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = WinterSportsPostSerializer

    def post(self, request, format=None):
        '''
        Add Winter Sports
        '''
        responsedata = []
        response = {}

        if Sport.objects.filter(name=request.data['name'], catagory=request.data['catagory']).exists():
            response["result"] = 0
            response['data'] = 'Sport already exists'
            return Response(response, status=status.HTTP_200_OK)
        else:
            serializer = self.serializer_class(data=request.data)
            message = "Created Successfully"

        if serializer.is_valid():
            serializer.save()
            response["result"] = 1
            response['data'] = [message]
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class WinterSportsGetAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    """
    Get all Winter Sports
    """
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = AllWinterSportsGetSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []
        if Sport.objects.filter(catagory=request.data['catagory']).exists():
            data = Sport.objects.filter(catagory=request.data['catagory'])
            serializer = self.serializer_class(data, many=True)
        else:
            response["result"] = 0
            response['data'] = 'No record found'
            return Response(response, status=status.HTTP_200_OK)
        response['result'] = 1
        response['data'] = serializer.data
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


# Summer Sports Apis


class SummerSportsPostAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = SummerSportsPostSerializer

    def post(self, request, format=None):
        '''
        Add Summer Sports
        '''
        responsedata = []
        response = {}

        if Sport.objects.filter(name=request.data['name'], catagory=request.data['catagory']).exists():
            response["result"] = 0
            response['data'] = 'Sport already exists'
            return Response(response, status=status.HTTP_200_OK)
        else:
            serializer = self.serializer_class(data=request.data)
            message = "Created Successfully"

        if serializer.is_valid():
            serializer.save()
            response["result"] = 1
            response['data'] = [message]
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class SummerSportsGetAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    """
    Get all Summer Sports
    """
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = AllSummerSportsGetSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []
        if Sport.objects.filter(catagory=request.data['catagory']).exists():
            data = Sport.objects.filter(catagory=request.data['catagory'])
            serializer = self.serializer_class(data, many=True)
        else:
            response["result"] = 0
            response['data'] = 'No record found'
            return Response(response, status=status.HTTP_200_OK)
        response['result'] = 1
        response['data'] = serializer.data
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


# All Seasons Sports Apis

class AllSeasonsSportsPostAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = AllSeasonsSportsPostSerializer

    def post(self, request, format=None):
        '''
        Add All Seasons Sports
        '''
        responsedata = []
        response = {}

        if Sport.objects.filter(name=request.data['name'], catagory=request.data['catagory']).exists():
            response["result"] = 0
            response['data'] = 'Sport already exists'
            return Response(response, status=status.HTTP_200_OK)
        else:
            serializer = self.serializer_class(data=request.data)
            message = "Created Successfully"

        if serializer.is_valid():
            serializer.save()
            response["result"] = 1
            response['data'] = [message]
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class AllSeasonsSportsGetAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    """
    Get All Seasons Sports
    """
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = AllSeasonsSportsGetSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []
        if Sport.objects.filter(catagory=request.data['catagory']).exists():
            data = Sport.objects.filter(catagory=request.data['catagory'])
            serializer = self.serializer_class(data, many=True)
        else:
            response["result"] = 0
            response['data'] = 'No record found'
            return Response(response, status=status.HTTP_200_OK)
        response['result'] = 1
        response['data'] = serializer.data
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


# FieldTypes Apis

class FieldTypesPostAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = FieldTypesPostSerializer

    def post(self, request, format=None):
        '''
        Add FieldTypes
        '''
        responsedata = []
        response = {}

        if FieldType.objects.filter(name=request.data['name']).exists():
            response["result"] = 0
            response['data'] = 'FieldType already exists'
            return Response(response, status=status.HTTP_200_OK)
        else:
            serializer = self.serializer_class(data=request.data)
            message = "Created Successfully"

        if serializer.is_valid():
            serializer.save()
            response["result"] = 1
            response['data'] = [message]
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class AllFieldTypesGetAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    """
    Get All FieldTypes
    """
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = AllFieldTypesGetSerializer

    def get(self, request, format=None):
        response = {}
        responsedata = []

        data = FieldType.objects.all()
        serializer = self.serializer_class(data, many=True)

        response['result'] = 1
        response['data'] = serializer.data
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


class FieldTypeEditAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = FieldTypeEditSerializer

    def post(self, request, format=None):
        '''
        Update FieldType
        '''
        responsedata = []
        response = {}

        if FieldType.objects.filter(id=request.data['id']).exists():
            serializer = self.serializer_class(instance=FieldType.objects.filter(
                id=request.data['id']).first(), data=request.data)
            message = "Updated Successfully"
        else:
            response["result"] = 0
            response['data'] = 'No record found'
            return Response(response, status=status.HTTP_200_OK)

        if serializer.is_valid():
            serializer.save()
            response["result"] = 1
            response['data'] = [message]
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class FieldTypeDeleteAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = DeleteFieldTypeSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []

        if not FieldType.objects.filter(id=request.data['id']).exists():
            response['results'] = 0
            response['error'] = "Not a valid ID"
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        serializers = self.serializer_class(data=request.data)

        if serializers.is_valid():
            FieldType_obj = FieldType.objects.get(
                id=serializers.validated_data['id'])
            FieldType_obj.delete()

            response['results'] = 1
            response['data'] = "Successfully Deleted"
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        else:
            response['results'] = 0
            response = handle_errors(serializers.errors)
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class GetSingleFieldTypeAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = SingleFieldTypeSerializer

    def post(self, request, format=None):
        '''
        Get FieldType by Id
        '''
        response = {}
        responsedata = []

        serializer = self.serializer_class(data=request.data)
        if serializer.is_valid():

            if FieldType.objects.filter(id=serializer.validated_data["id"]).exists():
                FieldType_obj = FieldType.objects.get(
                    id=serializer.validated_data["id"])
            else:
                response["result"] = 0
                response['error'] = "Invalid FieldType id"
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)

            serializer = AllFieldTypesGetSerializer(FieldType_obj)

            response["result"] = 1
            response['data'] = serializer.data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


# FloorTypes Apis

class FloorTypesPostAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = FloorTypesPostSerializer

    def post(self, request, format=None):
        '''
        Add FloorType
        '''
        responsedata = []
        response = {}

        if FloorType.objects.filter(name=request.data['name']).exists():
            response["result"] = 0
            response['data'] = 'FloorType already exists'
            return Response(response, status=status.HTTP_200_OK)
        else:
            serializer = self.serializer_class(data=request.data)
            message = "Created Successfully"

        if serializer.is_valid():
            serializer.save()
            response["result"] = 1
            response['data'] = [message]
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class AllFloorTypesGetAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    """
    Get All FloorType
    """
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = AllFloorTypesGetSerializer

    def get(self, request, format=None):
        response = {}
        responsedata = []

        data = FloorType.objects.all()
        serializer = self.serializer_class(data, many=True)
        response['result'] = 1
        response['data'] = serializer.data
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


class FloorTypeEditAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = FloorTypeEditSerializer

    def post(self, request, format=None):
        '''
        Update FloorType
        '''
        responsedata = []
        response = {}

        if FloorType.objects.filter(id=request.data['id']).exists():
            serializer = self.serializer_class(instance=FloorType.objects.filter(
                id=request.data['id']).first(), data=request.data)
            message = "Updated Successfully"
        else:
            response["result"] = 0
            response['data'] = 'No record found'
            return Response(response, status=status.HTTP_200_OK)

        if serializer.is_valid():
            serializer.save()
            response["result"] = 1
            response['data'] = [message]
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class FloorTypeDeleteAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = DeleteFloorTypeSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []

        if not FloorType.objects.filter(id=request.data['id']).exists():
            response['results'] = 0
            response['error'] = "Not a valid ID"
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        serializers = self.serializer_class(data=request.data)

        if serializers.is_valid():
            FloorType_obj = FloorType.objects.get(
                id=serializers.validated_data['id'])
            FloorType_obj.delete()

            response['results'] = 1
            response['data'] = "Successfully Deleted"
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        else:
            response['results'] = 0
            response = handle_errors(serializers.errors)
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class GetSingleFloorTypeAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = SingleFloorTypeSerializer

    def post(self, request, format=None):
        '''
        Get FloorType by Id
        '''
        response = {}
        responsedata = []

        serializer = self.serializer_class(data=request.data)
        if serializer.is_valid():

            if FloorType.objects.filter(id=serializer.validated_data["id"]).exists():
                FloorType_obj = FloorType.objects.get(
                    id=serializer.validated_data["id"])
            else:
                response["result"] = 0
                response['error'] = "Invalid FloorType id"
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)

            serializer = AllFloorTypesGetSerializer(FloorType_obj)

            response["result"] = 1
            response['data'] = serializer.data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


# FacilityTypes Apis

class FacilityTypesPostAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = FacilityTypesPostSerializer

    def post(self, request, format=None):
        '''
        Add Facility Type
        '''
        responsedata = []
        response = {}

        if FacilityType.objects.filter(name=request.data['name']).exists():
            response["result"] = 0
            response['data'] = 'FacilityType already exists'
            return Response(response, status=status.HTTP_200_OK)
        else:
            serializer = self.serializer_class(data=request.data)
            message = "Created Successfully"

        if serializer.is_valid():
            serializer.save()
            response["result"] = 1
            response['data'] = [message]
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class AllFacilityTypesGetAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    """
    Get All Facility Type
    """
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = AllFacilityTypesGetSerializer

    def get(self, request, format=None):
        response = {}
        responsedata = []

        data = FacilityType.objects.all()
        serializer = self.serializer_class(data, many=True)
        response['result'] = 1
        response['data'] = serializer.data
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


class FacilityTypeEditAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = FacilityTypeEditSerializer

    def post(self, request, format=None):
        '''
        Update Facility Type
        '''
        responsedata = []
        response = {}

        if FacilityType.objects.filter(id=request.data['id']).exists():
            serializer = self.serializer_class(instance=FacilityType.objects.filter(
                id=request.data['id']).first(), data=request.data)
            message = "Updated Successfully"
        else:
            response["result"] = 0
            response['data'] = 'No record found'
            return Response(response, status=status.HTTP_200_OK)

        if serializer.is_valid():
            serializer.save()
            response["result"] = 1
            response['data'] = [message]
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class FacilityTypeDeleteAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = DeleteFacilityTypeSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []

        if not FacilityType.objects.filter(id=request.data['id']).exists():
            response['results'] = 0
            response['error'] = "Not a valid ID"
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        serializers = self.serializer_class(data=request.data)

        if serializers.is_valid():
            FacilityType_obj = FacilityType.objects.get(
                id=serializers.validated_data['id'])
            FacilityType_obj.delete()

            response['results'] = 1
            response['data'] = "Successfully Deleted"
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        else:
            response['results'] = 0
            response = handle_errors(serializers.errors)
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class GetSingleFacilityTypeAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = SingleFacilityTypeSerializer

    def post(self, request, format=None):
        '''
        Get Facility Type by Id
        '''
        response = {}
        responsedata = []

        serializer = self.serializer_class(data=request.data)
        if serializer.is_valid():

            if FacilityType.objects.filter(id=serializer.validated_data["id"]).exists():
                FacilityType_obj = FacilityType.objects.get(
                    id=serializer.validated_data["id"])
            else:
                response["result"] = 0
                response['error'] = "Invalid Facility Type id"
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)

            serializer = AllFacilityTypesGetSerializer(FacilityType_obj)

            response["result"] = 1
            response['data'] = serializer.data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


# Lights Apis

class LightsPostAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = LightsPostSerializer

    def post(self, request, format=None):
        '''
        Add Lights
        '''
        responsedata = []
        response = {}

        if Lights.objects.filter(name=request.data['name']).exists():
            response["result"] = 0
            response['data'] = 'Lights already exists'
            return Response(response, status=status.HTTP_200_OK)
        else:
            serializer = self.serializer_class(data=request.data)
            message = "Created Successfully"

        if serializer.is_valid():
            serializer.save()
            response["result"] = 1
            response['data'] = [message]
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class AllLightsGetAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    """
    Get All Lights
    """
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = AllLightsGetSerializer

    def get(self, request, format=None):
        response = {}
        responsedata = []

        data = Lights.objects.all()
        serializer = self.serializer_class(data, many=True)

        response['result'] = 1
        response['data'] = serializer.data
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


# AVAILABILITY_IN_USE Apis

class AVAILABILITY_IN_USEPostAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = AVAILABILITY_IN_USEPostSerializer

    def post(self, request, format=None):
        '''
        Add AVAILABILITY_IN_USE
        '''
        responsedata = []
        response = {}

        if AVAILABILITY_IN_USE.objects.filter(name=request.data['name']).exists():
            response["result"] = 0
            response['data'] = 'already exists'
            return Response(response, status=status.HTTP_200_OK)
        else:
            serializer = self.serializer_class(data=request.data)
            message = "Created Successfully"

        if serializer.is_valid():
            serializer.save()
            response["result"] = 1
            response['data'] = [message]
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class AllAVAILABILITY_IN_USEGetAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    """
    Get All AVAILABILITY_IN_USE
    """
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = AllAVAILABILITY_IN_USEGetSerializer

    def get(self, request, format=None):
        response = {}
        responsedata = []

        data = AVAILABILITY_IN_USE.objects.all()
        serializer = self.serializer_class(data, many=True)

        response['result'] = 1
        response['data'] = serializer.data
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)
