import uuid

import pytz
import json
from django.http import HttpResponse
from django.template import Template, Context
from django.template.loader import get_template
from django.db import transaction
from django.contrib.contenttypes.models import ContentType
from django.urls import reverse
from django.contrib.auth.models import Group
from django.contrib.auth import login as django_login
import datetime
from django.utils import timezone
from rest_framework.authtoken.models import Token
from rest_framework import status, generics
from rest_framework.response import Response
from rest_framework.authentication import TokenAuthentication
from rest_framework.permissions import IsAuthenticated
from utils.mixins import *
from utils.api_handle_errors import handle_errors

from admin_app.serializers import *

from venue_owners.models import *
from venue_owners.serializers import *
from venues.models import *
from venues.serializers import *
from players.models import BookMatch
from commonAPI.models import *
from commonAPI.serializers import OnBoardAPIGETSerializer, OnBoardAPIPOSTSerializer
from django.db.models import Q, Avg
from reviews.models import Review, VenueReviewAndRating
from reviews.serializers import ReviewGETSerializer
from commonAPI.models import Contact_Person
from utils.mail import send_emal_onboard_accept, send_mail_registeration, field_approval_email, \
    in_progress_email_verification, onboard_Complete_mail, onboard_delete_or_declined_mail
from commonAPI.serializers import ContactPersonSerializer, BankInfoProfileSerializer
from field_booking.settings import DOMAIN_NAME
from platform_central.models import RegisterPrivateKey
from message.serializers import MessagesInboxSerializer
from message.models import MessagesInbox, Notification
from advertisers.models import Advertiser
from static_data.models import *
from static_data.serializers import *
from message.serializers import NotificationsSerializer
from campaigns.serializers import CampaignPart4GETSerializer, Campaign3Get, CampaignPart2SerializerGET
from campaigns.models import CampaignPart1, CampaignPart2, CampaignPart3, CampaignPart4
from professionals.models import Professional

from utils.mail import booking_cancel_request_approve_mail, venue_owner_booking_cancel_Complete_mail


class VenueOwnerAutoFillAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    '''
    Venue Owner Auto Fill API View
    '''

    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = VenueOwnerAutoFillSerializer

    venue_owner_class = VenueOwnerSerializer

    @transaction.atomic
    def post(self, request, format=None):
        response = {}
        responsedata = []

        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():

            # Searching venue owner with key word
            data = VenueOwner.objects.filter(Q(email__icontains=serializers.validated_data['key_word']) | Q(
                first_name__icontains=serializers.validated_data['key_word']) | Q(
                last_name__icontains=serializers.validated_data['key_word']))
            venue_owner_data = self.venue_owner_class(data, many=True)

            # Sending the response
            response['result'] = 1
            response['data'] = venue_owner_data.data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        # Serializer Errors
        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class VenueOwnerListInfoAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    '''
    Venue Owner List Info API View
    '''

    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = VenueOwnerListViewSerialzier

    venue_owner_class = VenueOwnerSerializer

    @transaction.atomic
    def post(self, request, format=None):
        response = {}
        responsedata = []

        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():

            # Searching venue owner with id
            data = VenueOwner.objects.filter(
                id=serializers.validated_data["venue_owner_id"])
            venue_owner_data = self.venue_owner_class(data.first())

            # Getting Review and Ratings
            rated_to_type = ContentType.objects.get_for_model(VenueOwner)
            reviews_count = Review.objects.filter(
                rated_to_type__pk=rated_to_type.id, rated_to_id__in=data).count()
            ratings = Review.objects.filter(
                rated_to_type__pk=rated_to_type.id, rated_to_id__in=data).aggregate(Avg("rating"))

            venue_info = Venue.objects.filter(created_by=data.first().user)
            reviews_count = VenueReviewAndRating.objects.filter(
                venue_id__in=venue_info).count()
            ratings = VenueReviewAndRating.objects.filter(
                venue_id__in=venue_info).aggregate(Avg("rating"))

            extra_data = [{"reviews_count": reviews_count}]
            extra_data.append(ratings)

            # Get Banking Information
            venue_info = Venue.objects.filter(
                created_by=data.first().user).first()
            venue_class = VenueProfileSerializer(venue_info)

            # Get Contact Person
            contact_person = Contact_Person.objects.filter(
                content_type__pk=rated_to_type.id, object_id__in=data)
            contact_person_class = ContactPersonSerializer(
                contact_person, many=True)

            # Putting Vat and address in venue owner class data
            venue_owner_info = {}
            venue_owner_info['vat'] = venue_class.data['buisness_vat']
            venue_owner_info['city'] = venue_class.data['city']
            venue_owner_info['pincode'] = venue_class.data['pincode']

            # Sending the response
            response['result'] = 1
            responsedata.append(venue_owner_data.data)
            responsedata.append(venue_owner_info)
            responsedata.append({"contact_person": contact_person_class.data})
            response['data'] = responsedata + extra_data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        # Serializer Errors
        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class VenueOwnerCRMAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    '''
    Venue Owner CRM API View
    '''

    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = VenueOwnerCRMSerialzier

    def post(self, request, format=None):
        response = {}
        responsedata = []
        extra_args = {}

        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():

            # Venue Owner
            venue_owner = VenueOwner.objects.get(
                id=serializers.validated_data['venue_owner_id'])
            # print(venue_owner.area)

            # Venues
            venues = Venue.objects.filter(created_by=venue_owner.user).first()

            venues_class = VenueProfileSerializer(venues)

            # Venue Fields
            if "from_date" in serializers.validated_data:
                extra_args['created_at__gte'] = serializers.validated_data['from_date']
                extra_args['created_at__lte'] = serializers.validated_data['to_date']

            venue_fields = VenueField.objects.filter(
                venue=venues, is_deleted=False, **extra_args)
            venue_field_counts = len(venue_fields)

            # Booking Counts
            future_bookings = 0
            total_bookings = 0
            player_list = []

            for i in venue_fields:
                match_data = BookMatch.objects.filter(
                    venue_field=i, **extra_args)
                # print(match_data)
                total_bookings = total_bookings + len(match_data)
                for j in match_data:
                    schedule_slot = j.schedule_slot

                    # Check for player in bookings table
                    player_obj = j.created_by
                    # print(player_obj)

                    # Check for unique player
                    if player_obj not in player_list:
                        player_list.append(player_obj)
                        print(player_list)

                    # Check for future dates
                    if schedule_slot.date > timezone.now().date():
                        future_bookings = future_bookings + 1

            ####graph 1 logic start####

            graph1_player_data = {}
            for j in range(1, 13):
                that_month_data = BookMatch.objects.filter(
                    venue_field__created_by=venue_owner.user, schedule_slot__date__month=j)
                if that_month_data.count() > 0:
                    unique_player_ids = []
                    for k in that_month_data:
                        if k.created_by.user.id not in unique_player_ids:
                            unique_player_ids.append(k.created_by.user.id)

                    graph1_player_data[j] = len(unique_player_ids)

                else:
                    graph1_player_data[j] = 0

            graph1_professional_data = {}
            profdata = Professional.objects.filter(area=venue_owner.area)
            graph1_professional_data[1] = 0
            graph1_professional_data[2] = 0
            graph1_professional_data[3] = 0
            graph1_professional_data[4] = 0
            graph1_professional_data[5] = 0
            graph1_professional_data[6] = 0
            graph1_professional_data[7] = 0
            graph1_professional_data[8] = 0
            graph1_professional_data[9] = 0
            graph1_professional_data[10] = 0
            graph1_professional_data[11] = 0
            graph1_professional_data[12] = 0
            if profdata.count() > 0:
                for l in profdata:
                    graph1_professional_data[l.created_at.month] = graph1_professional_data[l.created_at.month] + 1

            gplayerlist = []
            for x in graph1_player_data:
                gplayerlist.append(graph1_player_data[x])

            gprofessionallist = []
            for y in graph1_professional_data:
                gprofessionallist.append(graph1_professional_data[y])

            graph1_obj = {"player_data": gplayerlist,
                          "professional_data": gprofessionallist}
            ####graph 1 logic end####

            ####graph 2 logic start####
            graph2_fields_data = {}
            for j in range(1, 13):
                datafield = BookMatch.objects.filter(
                    venue_field__created_by=venue_owner.user, schedule_slot__date__month=j)
                if datafield.count() > 0:
                    unique_field_id = []
                    for q in datafield:
                        if q.venue_field.id not in unique_field_id:
                            unique_field_id.append(q.venue_field.id)
                    graph2_fields_data[j] = len(unique_field_id)
                else:
                    graph2_fields_data[j] = 0

            gfieldlist = []
            for x in graph2_fields_data:
                gfieldlist.append(graph2_fields_data[x])

            graph2_object = {"field_data": gfieldlist}

            ####graph 3 logic start####
            graph3_booking_data = {}
            graph3_future_booking_data = {}
            for j in range(1, 13):
                that_month_data3 = BookMatch.objects.filter(
                    venue_field__created_by=venue_owner.user, schedule_slot__date__month=j)
                if that_month_data3.count() > 0:
                    graph3_booking_data[j] = that_month_data3.count()
                    c = 0
                    for m in that_month_data3:
                        schedule_slot = m.schedule_slot
                        if schedule_slot.date > timezone.now().date():
                            c = c + 1
                    graph3_future_booking_data[j] = c
                else:
                    graph3_booking_data[j] = 0
                    graph3_future_booking_data[j] = 0

            gbookinglist = []
            for x in graph3_booking_data:
                gbookinglist.append(graph3_booking_data[x])

            gfuturebookinglist = []
            for y in graph3_future_booking_data:
                gfuturebookinglist.append(graph3_future_booking_data[y])

            graph3_obj = {"booking_data": gbookinglist,
                          "future_booking": gfuturebookinglist}
            ####graph 3 logic end####

            # Check for professional
            professional_obj = Professional.objects.filter(
                area=venue_owner.area)

            # Check for venue service
            venueService_obj = VenueServiceCost.objects.filter(
                venue__created_by=venue_owner.user)

            # News Letter Sent  ( Yet to work on)
            newsletter_count = 4

            # ADS SMS sent
            ad_sms = 10

            # ADS Banner
            banner = 2

            # ADS Banner
            banner = 2

            # Services
            services = venueService_obj.count()

            # Professional
            professional = professional_obj.count()

            # Player
            player = len(player_list)

            # Consolidating The Data
            data = {"logo": venues_class.data['logo'], "venue_id": venues_class.data['id'],
                    "professional_count": professional, "services_count": services, "player_count": player,
                    "newsletter_count": newsletter_count,
                    "banner_count": banner, "ad_sms_count": ad_sms, "baner_count": banner,
                    "total_booking": total_bookings, "future_booking": future_bookings,
                    "venue_field_counts": venue_field_counts, "graph1": graph1_obj, "graph2": graph2_object,
                    "graph3": graph3_obj}

            # Sending Response
            response["result"] = 1
            response['data'] = data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        # Serializer Errors
        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class VenueOwnerInfoAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    '''
    Venue Owner Info API View
    '''

    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = VenueOwnerListViewSerialzier
    contact_person_serializer = ContactPersonSerializer
    venue_owner_class = VenueOwnerSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []

        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            # GET Venue Owner Information
            venue_owner = VenueOwner.objects.filter(
                id=serializers.validated_data['venue_owner_id'])
            venue_owner_serializer = self.venue_owner_class(
                venue_owner, many=True)

            # Get Venue Info
            venue_obj = Venue.objects.filter(
                created_by=venue_owner[0].user).first()
            venue_class = VenueProfileSerializer(venue_obj)

            availability = VenueDailyTiming.objects.filter(
                venue=venue_obj, is_active=True, is_deleted=False)
            timing = VenueTimingSerializer(availability, many=True)

            data_content_type = ContentType.objects.get_for_model(VenueOwner)
            # content_type = ContentType.objects.get_for_model(Venue)
            contact_info = ContactInfo.objects.filter(
                content_type__pk=data_content_type.id, object_id__in=venue_owner)
            contact_info_serializer = ContactInfoGETSerializer(
                contact_info, many=True)
            contact_person = Contact_Person.objects.filter(
                content_type__pk=data_content_type.id, object_id__in=venue_owner)
            contact_person_serializer = ContactPersonSerializer(
                contact_person, many=True)

            # Bank Info
            bank_obj = BankInfo.objects.filter(
                created_by=venue_owner[0].user, is_deleted=False)
            # bank_obj = BankInfo.objects.filter(
            #     content_type__pk=data_content_type.id, object_id__in=venue_owner)
            bank_serializer = BankInfoProfileSerializer(bank_obj, many=True)

            # print(bank_serializer.data)

            # Consolidating The Data
            data = {"venue_owner": venue_owner_serializer.data,
                    "venue_id": venue_class.data['id'], "contact_info": contact_info_serializer.data,
                    "venue_info": venue_class.data, "contact_person": contact_person_serializer.data,
                    "working_hour": timing.data, "bank_info": bank_serializer.data}

            # Sending Response
            response["result"] = 1
            response['data'] = data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        # Serializer Errors
        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class VenueOwnerEditInfoAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    '''
    Venue Owner EDIT Info API View
    '''

    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = VenueOwnerEditInfoSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []

        # serializers = self.serializer_class(
        #     instance=VenueOwner.objects.get(id=request.data['id']), data=request.data)
        serializers = self.serializer_class(
            data=request.data, context={"id": request.data['id']})
        if serializers.is_valid():
            # Saving the serializer
            serializers.save()
            # Sending Response
            response["result"] = 1
            response['data'] = "Success"
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        # Serializer Errors
        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class VenueOwnerFieldInfoGetAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    '''
    Get Field Info API View
    '''

    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = VenueOwnerListViewSerialzier

    venue_field_class = VenueFieldGETAdminSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []

        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            # get page number
            start_count = serializers.validated_data['page'] - 1
            start_count = start_count * 10
            end_count = start_count + 10

            # GET Venue Owner Information
            venue_owner = VenueOwner.objects.get(
                id=serializers.validated_data['venue_owner_id'])

            # Venues
            venues = Venue.objects.filter(created_by=venue_owner.user).last()

            # Venue Fields
            total_count = VenueField.objects.filter(
                venue=venues, is_deleted=False).count()
            venue_fields = VenueField.objects.filter(venue=venues, is_deleted=False)[
                start_count:end_count]
            venue_fields_serializer = self.venue_field_class(
                venue_fields, many=True)

            # Sending Response
            response["result"] = 1
            response['data'] = venue_fields_serializer.data
            response['total_count'] = total_count
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        # Serializer Errors
        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class ApprovalsAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    '''
    Get Approvals API View
    '''

    # authentication_classes = (TokenAuthentication,)
    # permission_classes = (IsAuthenticated,)

    serializer_class = ApprovalsSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []

        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            # get page
            start_count = (serializers.validated_data['page'] - 1) * 10
            end_count = start_count + 10
            serializers.validated_data.pop("page")

            extra_args = {}

            # Filters
            # User Type
            if "user_type" in serializers.validated_data:
                extra_args['registeration_type'] = serializers.validated_data['user_type']

                if serializers.validated_data['user_type'] == 'V':
                    if "sport_type" in serializers.validated_data:
                        extra_args['sports'] = serializers.validated_data['sport_type']

                if serializers.validated_data['user_type'] == 'P':
                    if "service_type" in serializers.validated_data:
                        extra_args['service'] = serializers.validated_data['service_type']

                # Sport Type   ....To be changed
            if "sport_type" in serializers.validated_data:
                serializers.validated_data['sport_type'] = serializers.validated_data['sport_type']
                # Service Type
            if "service_type" in serializers.validated_data:
                serializers.validated_data['service_type'] = serializers.validated_data['service_type']
                # Venue Name
            if "venue_name" in serializers.validated_data:
                serializers.validated_data['venue_name'] = serializers.validated_data['venue_name']

            if "location" in serializers.validated_data:
                extra_args['city'] = serializers.validated_data['location']

                # Mode
            if "mode" in serializers.validated_data:
                # if serializers.validated_data['mode'] == "Pending":
                #     extra_args['status'] = "O"
                if serializers.validated_data['mode'] == "Approve":
                    extra_args['status'] = "A"
                if serializers.validated_data['mode'] == "Declined":
                    extra_args['status'] = "R"
                if serializers.validated_data['mode'] == "In Progress":
                    extra_args['status'] = "I"
                if serializers.validated_data['mode'] == "New":
                    extra_args['status'] = "N"
                if serializers.validated_data['mode'] == "Complete":
                    extra_args['status'] = "C"

            # Get Data
            # Key Words
            if "key_word" in serializers.validated_data:
                if "assortment" in serializers.validated_data:
                    if serializers.validated_data['assortment'] == "RegisterationTime-Descending":
                        onboard_data = OnBoardAPI.objects.filter(
                            Q(email__icontains=serializers.validated_data['key_word'], **extra_args) | Q(
                                first_name__icontains=serializers.validated_data['key_word'], **extra_args) | Q(
                                last_name__icontains=serializers.validated_data['key_word'], **extra_args) | Q(
                                mobile__contains=serializers.validated_data['key_word'], **extra_args) | Q(
                                business_name__icontains=serializers.validated_data['key_word'],
                                **extra_args)).order_by(
                            "-created_at")
                    else:
                        onboard_data = OnBoardAPI.objects.filter(
                            Q(email__icontains=serializers.validated_data['key_word'], **extra_args) | Q(
                                first_name__icontains=serializers.validated_data['key_word'], **extra_args) | Q(
                                last_name__icontains=serializers.validated_data['key_word'], **extra_args) | Q(
                                mobile__contains=serializers.validated_data['key_word'], **extra_args) | Q(
                                business_name__icontains=serializers.validated_data['key_word'],
                                **extra_args)).order_by(
                            "created_at")
                else:
                    onboard_data = OnBoardAPI.objects.filter(
                        Q(email__icontains=serializers.validated_data['key_word'], **extra_args) | Q(
                            first_name__icontains=serializers.validated_data['key_word'], **extra_args) | Q(
                            last_name__icontains=serializers.validated_data['key_word'], **extra_args) | Q(
                            mobile__contains=serializers.validated_data['key_word'], **extra_args) | Q(
                            business_name__icontains=serializers.validated_data['key_word'], **extra_args))
            else:
                if "assortment" in serializers.validated_data:
                    if serializers.validated_data['assortment'] == "RegisterationTime-Descending":
                        onboard_data = OnBoardAPI.objects.filter(
                            is_deleted=False, **extra_args).order_by("-created_at")
                    else:
                        onboard_data = OnBoardAPI.objects.filter(
                            is_deleted=False, **extra_args).order_by("created_at")
                else:
                    # added it for latest record
                    onboard_data = OnBoardAPI.objects.filter(
                        is_deleted=False, **extra_args).order_by("-created_at")

                # Serializer
            total_count = onboard_data.count()
            onboard_data = onboard_data[start_count:end_count]
            onboard_serializer = OnBoardAPIGETSerializer(
                onboard_data, many=True)

            # Sending Response
            response["result"] = 1
            response['data'] = onboard_serializer.data
            response['total_count'] = total_count
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        # Serializer Errors
        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class ApprovalsByIDAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    '''
    Get Approvval by id
    '''
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = GetApprovalsByIDSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []

        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            approval_obj = OnBoardAPI.objects.get(
                id=serializers.validated_data['id'])
            approval_class = OnBoardAPIGETSerializer(approval_obj)
            # Sending Response
            response["result"] = 1
            response['data'] = approval_class.data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class VenueFieldByIDAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    '''
    Get Venue Field by id
    '''
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = GetApprovalsByIDSerializer

    venue_field_image_serializer = VenueFieldImageSerializer
    venue_field_service_serializer = VenueFieldServiceSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []

        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            approval_obj = VenueField.objects.get(
                id=serializers.validated_data['id'])
            approval_class = GetVenueFieldSerializer(approval_obj)

            # venue Field images
            field_images = VenueFieldImage.objects.filter(
                venue_field=approval_obj, is_deleted=False)
            serializer_field_images = self.venue_field_image_serializer(
                field_images, many=True)

            # VenueFieldService
            venue_field_services = VenueFieldService.objects.filter(
                venue_field=approval_obj, is_deleted=False)
            serializer_field_services = self.venue_field_service_serializer(
                venue_field_services, many=True)

            # Venue Availability
            availability = VenueDailyTiming.objects.filter(
                venue=approval_obj.venue, is_active=True, is_deleted=False)
            timing = VenueTimingSerializer(availability, many=True)

            data = {"field_data": approval_class.data, "field_images": serializer_field_images.data,
                    "venue_services": serializer_field_services.data, "timing": timing.data}

            # Sending Response
            response["result"] = 1
            response['data'] = data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class CampaignByIDAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    '''
    Get Campaign by id
    '''
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = GetApprovalsByIDSerializer

    Campaign2serializer_class = CampaignPart2SerializerGET
    Campaign3serializer_class = Campaign3Get
    Campaign4serializer_class = CampaignPart4GETSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []

        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            approval_obj = CampaignPart1.objects.get(
                id=serializers.validated_data['id'])
            approval_class = GetCampaignByIDSerializer(approval_obj)
            Campaign2 = CampaignPart2.objects.filter(
                campaign1=approval_obj, is_deleted=False)
            Campaign3 = CampaignPart3.objects.filter(
                campaign1=approval_obj, is_deleted=False)
            Campaign4 = CampaignPart4.objects.filter(
                campaign1=approval_obj, is_deleted=False)

            serializers2 = self.Campaign2serializer_class(Campaign2, many=True)
            serializers3 = self.Campaign3serializer_class(Campaign3, many=True)
            serializers4 = self.Campaign4serializer_class(Campaign4, many=True)

            data = {"0": serializers2.data,
                    "1": serializers3.data, "2": serializers4.data}
            # Sending Response
            response["result"] = 1
            response['data'] = approval_class.data
            response['all_data'] = data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class ChangeApprovalAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    '''
    Changing Onboard Form Status
    '''

    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = ChangeApproveStatusSerializer

    @transaction.atomic
    def post(self, request, format=None):
        response = {}
        responsedata = []

        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            # Get Form
            onboard_form = OnBoardAPI.objects.get(
                id=serializers.validated_data['id'], is_deleted=False)

            city = City.objects.get(id=onboard_form.city.id)
            country = Country.objects.get(id=onboard_form.country.id)
            area = Area.objects.get(id=onboard_form.area.id)
            state = State.objects.get(id=onboard_form.state.id)

            service_dict = onboard_form.service.all()
            # print(service_dict)
            service_id = []
            for i in range(service_dict.count()):
                service_id.append(service_dict[i].id)

            images = Template_Images.objects.filter().values('template_image')
            logo = images[0]
            logo2 = images[1]

            # If Progress
            if serializers.validated_data['mode'] == "In Progress":
                onboard_form.status = "I"
                onboard_form.save()
                message = "Status in Progress"

                # Create Table on respective user type
                if serializers.validated_data['mode'] == "In Progress":
                    # 333
                    whitelist = ['lanoisseforptest1@gmail.com',
                                 'lanoisseforptest2@gmail.com', 'lanoisseforptest3@gmail.com']

                    if onboard_form.email in whitelist:
                        User.objects.filter(
                            username=onboard_form.email).delete()

                    # 3

                    if User.objects.filter(username=onboard_form.email, email=onboard_form.email).exists():
                        onboard_form.status = "R"
                        onboard_form.save()

                        # Send User Exists response
                        response["result"] = 0
                        responsedata.append(
                            "Username already Exists. Claim Rejected")
                        response['errors'] = responsedata
                        self.add_api_call(request, response)
                        return Response(response, status=status.HTTP_200_OK)

                    # Send Email for approve
                    else:
                        # Create User Object
                        user = User.objects.create(username=onboard_form.email, email=onboard_form.email,
                                                   first_name=onboard_form.first_name, last_name=onboard_form.last_name,
                                                   is_active=False)
                        # Convert UUID format to a Python string.
                        random = str(uuid.uuid4())

                        # Make all characters uppercase.
                        random = random.upper()

                        # Remove the UUID '-'.
                        random = random.replace("-", "")

                        # Return the random string.
                        user_code = random[0:6]
                        onboard_form.user_code = user_code

                        # Create Private Token
                        if RegisterPrivateKey.objects.filter(user=user).exists():
                            # Send Token Exists response
                            response["result"] = 0
                            responsedata.append("Token Already Exists")
                            response['errors'] = responsedata
                            self.add_api_call(request, response)
                            return Response(response, status=status.HTTP_200_OK)

                        else:
                            token_obj = RegisterPrivateKey.objects.create(
                                user=user)

                        # Send Email
                        # registeration_type = "professional" if onboard_form.registeration_type == "P" else "advertiser" if onboard_form.registeration_type == "A" else "field"
                        msg = "The request has been received and somebody will contact you soon."
                        in_progress_email_verification(
                            onboard_form.first_name, msg, onboard_form.email, logo, logo2)

                    onboard_form.__dict__['created_by'] = user.id
                    onboard_form.__dict__['user'] = user.id

                    if onboard_form.registeration_type == "A":
                        advertiser_class = AdvertiserOnboardSerializer(
                            data=onboard_form.__dict__)
                        if advertiser_class.is_valid():
                            adv_obj = advertiser_class.save()
                            adv_obj.country = country
                            adv_obj.state = state
                            adv_obj.city = city
                            adv_obj.area = area
                            adv_obj.save()
                        else:
                            response = handle_errors(serializers.errors)
                            response["result"] = 0
                            self.add_api_call(request, response)
                            return Response(response, status=status.HTTP_400_BAD_REQUEST)
                        my_group = Group.objects.get(name='Advertiser')

                    if onboard_form.registeration_type == "V":
                        venue_owner_class = VenueOwnerOnboardSerializer(
                            data=onboard_form.__dict__)
                        venue_class = VenueOnboardSerializer(
                            data=onboard_form.__dict__)
                        if venue_class.is_valid() and venue_owner_class.is_valid():
                            venue_owner_obj = venue_owner_class.save()
                            venue_obj = venue_class.save()
                            venue_obj.created_by = venue_owner_obj.user
                            venue_obj.save()
                            venue_owner_obj.country = country
                            venue_owner_obj.state = state
                            venue_owner_obj.city = city
                            venue_owner_obj.area = area
                            venue_owner_obj.save()
                            print(venue_owner_obj)
                            response['venue_owner_Id'] = venue_owner_obj.id

                        else:
                            response = handle_errors(
                                venue_owner_class.errors or venue_class.errors)
                            response["result"] = 0
                            self.add_api_call(request, response)
                            return Response(response, status=status.HTTP_400_BAD_REQUEST)
                        my_group = Group.objects.get(name='Venue Owner')

                    elif onboard_form.registeration_type == "P":
                        # onboard = onboard_form.__dict__
                        # onboard["service"] = service_dict[0].__dict__
                        professional_class = ProfessionalOnboardSerializer(
                            data=onboard_form.__dict__)
                        # professional_class = ProfessionalOnboardSerializer(
                        #     data=onboard)
                        # print(onboard)
                        if professional_class.is_valid():
                            professional_obj = professional_class.save()
                            professional_obj.country = country
                            professional_obj.state = state
                            professional_obj.city = city
                            professional_obj.area = area
                            professional_obj.service.set(service_id)
                            professional_obj.save()
                        else:
                            response = handle_errors(professional_class.errors)
                            response["result"] = 0
                            self.add_api_call(request, response)
                            return Response(response, status=status.HTTP_400_BAD_REQUEST)
                        my_group = Group.objects.get(name='Professional')

                    my_group.user_set.add(user)

            elif serializers.validated_data['mode'] == "Approve":
                onboard_form.status = "A"
                onboard_form.save()
                message = "Successfully Approved"
                user_obj = User.objects.get(
                    username=onboard_form.email, email=onboard_form.email)
                token_obj = RegisterPrivateKey.objects.get(user=user_obj)
                business_name_for_msg = onboard_form.business_name
                registeration_type = "professional" if onboard_form.registeration_type == "P" else "advertiser" if onboard_form.registeration_type == "A" else "field"
                verify_redirect = "{}{}".format(
                    "https://sportsbookingnew.worldindia.com/#",
                    "/" + registeration_type + "/create-profile/set-password/" + str(token_obj.key))
                msg = business_name_for_msg + " has been approved as " + registeration_type + \
                    ". Your profile is under development. You can logon for first time to the platform changing your password."
                send_emal_onboard_accept(
                    onboard_form.first_name, verify_redirect, onboard_form.email, msg, logo, logo2)

            elif serializers.validated_data['mode'] == "Complete":
                onboard_form.status = "C"
                onboard_form.save()
                message = "Successfully Completed"
                business_name_for_msg = onboard_form.business_name

                if onboard_form.registeration_type == 'P':
                    registeration_type = "professional"
                    verify_redirect = "{}".format(
                        "https://sportsbookingnew.worldindia.com/#/professional")
                    msg = business_name_for_msg + \
                        " is available for search by users on the platform and you will start receiving email/notifications for bookings."
                    onboard_Complete_mail(
                        onboard_form.first_name, verify_redirect, onboard_form.email, msg, logo, logo2)

                elif onboard_form.registeration_type == 'A':
                    registeration_type = "advertiser"
                    verify_redirect = "{}".format(
                        "https://sportsbookingnew.worldindia.com/#/advertiser")
                    msg = business_name_for_msg + \
                        " is available for search by users on the platform and you will start receiving email/notifications for bookings."
                    onboard_Complete_mail(
                        onboard_form.first_name, verify_redirect, onboard_form.email, msg, logo, logo2)

                else:
                    registeration_type = "venue owner"
                    verify_redirect = "{}".format(
                        "https://sportsbookingnew.worldindia.com/#/field")
                    msg = business_name_for_msg + \
                        " is available for search by users on the platform and you will start receiving email/notifications for bookings."
                    onboard_Complete_mail(
                        onboard_form.first_name, verify_redirect, onboard_form.email, msg, logo, logo2)

            # If Declined
            elif serializers.validated_data['mode'] == "Declined":
                onboard_form.status = "R"
                onboard_form.save()
                message = "Successfully Declined"
                business_name_for_msg = onboard_form.business_name
                msg = business_name_for_msg + \
                    " request has been rejected and you will receive an additional email on rejection reasons."
                onboard_delete_or_declined_mail(
                    onboard_form.first_name, onboard_form.email, msg, logo, logo2)

            # If Deleted
            elif serializers.validated_data['mode'] == "Deleted":
                onboard_form.status = "R"
                onboard_form.is_deleted = True
                onboard_form.save()
                message = "Successfully Deleted"
                business_name_for_msg = onboard_form.business_name
                msg = business_name_for_msg + \
                    " request has been rejected and you will receive an additional email on rejection reasons."
                onboard_delete_or_declined_mail(
                    onboard_form.first_name, onboard_form.email, msg, logo, logo2)

            # Sending Response
            response["result"] = 1
            responsedata.append(message)
            response['data'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        # Serializer Errors
        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class AddFieldAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = VenueAddFieldSerializer

    def post(self, request, format=None):
        """
        Add Field
        """
        response = {}
        responsedata = []

        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            images_list = []
            services = None

            if "images" in serializers.validated_data:
                images_list = serializers.validated_data['images']
                serializers.validated_data.pop("images")

            if "field_services" in serializers.validated_data:
                services = serializers.validated_data['field_services']
                serializers.validated_data.pop("field_services")

            serializers.validated_data['created_by'] = serializers.validated_data['venueowner'].user
            serializers.validated_data['venue'] = Venue.objects.filter(
                created_by=serializers.validated_data['created_by']).first()
            serializers.validated_data.pop("venueowner")

            field_obj = serializers.save()

            field_type_obj = FieldTypeSerializer(field_obj.field_type)
            facility_type_obj = FacilitysGetSerializer(field_obj.facility_type)
            floor_type_obj = FloorTypeSerializer(field_obj.floor_type)
            lights_obj = LightsGetSerializer(field_obj.lights)
            sport_obj = SportAdminSerializer(field_obj.sport)
            availability_obj = AvailabilityInUseSerializer(
                field_obj.availability_in_use)
            mixed_gender_obj = MixedGenderSerializer(field_obj.mixed_gender)

            return_data = {
                "id": field_obj.id,
                "description": field_obj.description,
                "field_name": field_obj.name,
                "field_type": field_type_obj.data,
                "floor_type": floor_type_obj.data,
                "seat_capacity": field_obj.seat_capacity,
                "lights": lights_obj.data,
                "facility_type": facility_type_obj.data,
                "facility_type_id": field_obj.facility_type.id,
                "sport": sport_obj.data,
                "mixed_gender": mixed_gender_obj.data,
                "availability": availability_obj.data
            }

            main_image_class = VenueFieldSerializerVer1(
                field_obj, many=False)
            return_data['main_image'] = main_image_class.data

            if services is not None:
                field_service_object = [VenueFieldService(
                    venue_service=VenueService.objects.get(id=i),
                    venue_field=field_obj
                ) for i in services]
                service_obj = VenueFieldService.objects.bulk_create(
                    field_service_object)
                services_class = VenueFieldServicePreviewSerializer1(
                    service_obj, many=True)
                return_data['services'] = services_class.data

            if len(images_list) > 0:
                venueField_image = [VenueFieldImage(
                    image=i,
                    venue_field=field_obj,
                    created_by=field_obj.created_by
                )
                    for i in images_list]
                image_obj = VenueFieldImage.objects.bulk_create(
                    venueField_image)

                sub_image_obj = VenueFieldImage.objects.filter(
                    venue_field=field_obj)
                images_class = VenueFieldImageSerializer(
                    sub_image_obj, many=True)
                return_data['images'] = images_class.data

            # Sending Response
            response["result"] = 1
            responsedata.append("Successfully Inserted")
            response['msg'] = responsedata
            response['data'] = return_data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        # Serializer Errors
        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class UpdateFieldAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = VenueEditFieldSerializer

    def post(self, request, format=None):
        """
        Edit Field 
        """
        response = {}
        responsedata = []

        if VenueField.objects.filter(id=request.data['id'], is_deleted=False).exists():
            pass
        else:
            response["result"] = 0
            responsedata.append("No Record Found")
            response['errors'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        serializer = self.serializer_class(data=request.data)
        if serializer.is_valid():
            field_obj = serializer.save()

            return_data = {
                "id": field_obj['id'],
                "description": field_obj['description'],
                "field_name": field_obj['field_name'],
                "field_type": field_obj['field_type'],
                "floor_type": field_obj['floor_type'],
                "seat_capacity": field_obj['seat_capacity'],
                "lights": field_obj['lights'],
                "facility_type": field_obj['facility_type'],
                "facility_type_id": field_obj['facility_type'],
                "sport": field_obj['sport'],
                "images": field_obj['images'],
                "main_image": field_obj['main_image'],
                "images": field_obj['images'],
                "main_image": field_obj['main_image'],
                "mixed_gender": field_obj['mixed_gender'],
                "availability": field_obj['availability']
            }

            if "services" in serializer.validated_data:
                VenueFieldService.objects.filter(
                    created_by=request.user, venue_field=field_obj).delete()
                venue_fieldService = [VenueFieldService(
                    **i,
                    venue_field=field_obj,
                    created_by=request.user
                ) for i in serializer.validated_data['services']]
                service_obj = VenueFieldService.objects.bulk_create(
                    venue_fieldService)
                services_class = VenueFieldServicePreviewSerializer1(
                    service_obj, many=True)
                return_data['services'] = services_class.data

            # Sending Response
            response["result"] = 1
            responsedata.append("Successfully Updated")
            response['errors'] = responsedata
            response['data'] = field_obj
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        # Serializer Errors
        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


# class UpdateFieldAPIView(PlatformAPICallMixin, generics.GenericAPIView):
#     authentication_classes = (TokenAuthentication,)
#     permission_classes = (IsAuthenticated,)
#     serializer_class = VenueEditFieldSerializer

#     def post(self, request, format=None):
#         """
#         Edit Field
#         """
#         response = {}
#         responsedata = []

#         if VenueField.objects.filter(id=request.data['id'], is_deleted=False).exists():
#             instance = VenueField.objects.get(
#                 id=request.data['id'], is_deleted=False)
#         else:
#             response["result"] = 0
#             responsedata.append("No Record Found")
#             response['errors'] = responsedata
#             self.add_api_call(request, response)
#             return Response(response, status=status.HTTP_200_OK)

#         serializers = self.serializer_class(
#             instance=instance, data=request.data)
#         if serializers.is_valid():
#             services = None
#             images_list = []

#             if "images" in serializers.validated_data:
#                 images_list = images_list + \
#                     serializers.validated_data['images']
#                 serializers.validated_data.pop("images")

#             if "field_attributes" in serializers.validated_data:
#                 services = serializers.validated_data['field_attributes']
#                 serializers.validated_data.pop("field_attributes")

#             field_obj = serializers.save()

#             venueField_image = [VenueFieldImage(
#                 image=i,
#                 venue_field=field_obj,
#                 created_by=field_obj.created_by
#             )
#                 for i in images_list]

#             image_obj = VenueFieldImage.objects.bulk_create(venueField_image)

#             # Save services
#             if services is not None:
#                 VenueFieldService.objects.filter(
#                     venue_field=field_obj).delete()
#                 field_service_object = [VenueFieldService(
#                     venue_service=VenueService.objects.get(id=i),
#                     venue_field=field_obj
#                 ) for i in services]
#                 VenueFieldService.objects.bulk_create(field_service_object)

#             venueField_image = [VenueFieldImage(
#                 image=i,
#                 venue_field=field_obj,
#                 created_by=field_obj.created_by
#             )
#                 for i in images_list]
#             image_obj = VenueFieldImage.objects.bulk_create(venueField_image)

#             # Sending Response
#             response["result"] = 1
#             responsedata.append("Successfully Updated")
#             response['errors'] = responsedata
#             self.add_api_call(request, response)
#             return Response(response, status=status.HTTP_200_OK)

#         # Serializer Errors
#         else:
#             response = handle_errors(serializers.errors)
#             response["result"] = 0
#             self.add_api_call(request, response)
#             return Response(response, status=status.HTTP_400_BAD_REQUEST)


# class AddContactInfoAPIView(PlatformAPICallMixin, generics.GenericAPIView):
#     authentication_classes = (TokenAuthentication,)
#     permission_classes = (IsAuthenticated,)
#     serializer_class = VenueOwnerAddContactInfoSerializer

#     def post(self, request, format=None):
#         """
#         Add Contact Info
#         """
#         response = {}
#         responsedata = []

#         serializers = self.serializer_class(data=request.data)
#         if serializers.is_valid():

#             contact_obj = ContactInfo.objects.filter(
#                 id=serializers.validated_data['content_object'].id)
#             contact_contentType = ContentType.objects.get_for_model(VenueOwner)
#             ContactInfo.objects.filter(
#                 content_type__pk=contact_contentType.id, object_id__in=contact_obj).delete()

#             contactinfo_obj = serializers.save()

#             # Sending Response
#             response["result"] = 1
#             responsedata.append("Successfully Inserted")
#             response['errors'] = responsedata
#             self.add_api_call(request, response)
#             return Response(response, status=status.HTTP_200_OK)

#         # Serializer Errors
#         else:
#             response = handle_errors(serializers.errors)
#             response["result"] = 0
#             self.add_api_call(request, response)
#             return Response(response, status=status.HTTP_400_BAD_REQUEST)


class AddContactInfoAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = VenueOwnerAddContactPersonSerializer

    def post(self, request, format=None):
        """
        Add Contact Info
        """
        response = {}
        responsedata = []

        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            contactinfo_obj = serializers.save()
            # Sending Response
            response["result"] = 1
            responsedata.append("Successfully Inserted")
            response['errors'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class DeleteContactInfoAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = VenueOwnerDeleteContactInfoSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []

        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():

            # contact_obj = ContactInfo.objects.filter(
            #     id=serializers.validated_data['id']).first()
            contact_contentType = ContentType.objects.get_for_model(VenueOwner)
            # TelephoneNumbers.objects.filter(contact_id=contact_obj).delete()
            Contact_Person.objects.filter(
                content_type__pk=contact_contentType.id, id=serializers.validated_data['id']).delete()

            # Sending Response
            response["result"] = 1
            responsedata.append("Successfully Deleted")
            response['errors'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        # Serializer Errors
        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class GetMessageAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    # authentication_classes = (TokenAuthentication,)
    # permission_classes = (IsAuthenticated,)
    serializer_class = AdminGetMessageSerialzer

    def post(self, request, format=None):
        response = {}
        responsedata = []
        extra_data = {}

        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            # Get page number
            start_count = (serializers.validated_data['page'] - 1) * 10
            end_count = start_count + 10
            serializers.validated_data.pop("page")

            # Get Venue owner
            venue_owner_object = serializers.validated_data['id']

            # Get user object
            user_object = venue_owner_object.user

            # Get message of the venue owner
            if serializers.data['MessageType'] == 'Inbox':

                if 'from_date' in serializers.validated_data:
                    extra_data["updated_at__gte"] = serializers.validated_data['from_date']
                if 'to_date' in serializers.validated_data:
                    extra_data['updated_at__lte'] = serializers.validated_data['to_date']
                if 'is_read' in serializers.validated_data:
                    extra_data['is_read'] = serializers.validated_data['is_read']
                    print(serializers.validated_data['is_read'])

                message_obj = MessagesInbox.objects.filter(
                    recipient=user_object, is_deleted=serializers.data[
                        'is_deleted'],
                    **extra_data).order_by("-created_at")

            elif serializers.data['MessageType'] == 'Sentbox':
                message_obj = MessagesInbox.objects.filter(
                    sender=user_object,  is_deleted=serializers.data[
                        'is_deleted'], ).order_by("-created_at")

            else:
                response["result"] = 0
                responsedata.append("Incorrect Type")
                response['errors'] = responsedata
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)

            total_count = message_obj.count()
            message_obj = message_obj[start_count:end_count]
            serializer = MessagesInboxSerializer(message_obj, many=True)
            response['result'] = 1
            response['data'] = serializer.data
            response['total_count'] = total_count
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class AdminComposeMessageAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    # authentication_classes = (TokenAuthentication,)
    # permission_classes = (IsAuthenticated,)
    serializer_class = AdminComposeMessageSerialier

    def post(self, request, format=None):
        response = {}
        responsedata = []

        serializer = self.serializer_class(data=request.data)
        if serializer.is_valid():
            if "recipient" not in serializer.validated_data:
                response["result"] = 0
                responsedata.append("Please enter valid Recipient username")
                response['errors'] = responsedata
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)

            if User.objects.filter(username=serializer.validated_data['sender']).exists():
                getID = serializer.save()
                message = MessagesInbox.objects.get(id=getID.id)

                # Get Message Icons
                notification_obj = NotificationIcon.objects.get(name="Message")

                # Notifications
                data = {"category": "Message", "button_text": "Reply", "icon": notification_obj,
                        "subject": "You have a new Message from " +
                                   str(request.user), "recipient": message.recipient, "content_object": message,
                        'body': serializer.validated_data['message']}

                data = Notification(**data)
                data.save()
                # Notification Ends

                response["result"] = 1
                response['errors'] = ["Successfully sent"]
                return Response(response, status=status.HTTP_200_OK)
            else:
                response["result"] = 0
                responsedata.append("No Authorized User")
                response['errors'] = responsedata
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)

        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            return Response(response, status=status.HTTP_200_OK)


class VenueApprovalsAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    # authentication_classes = (TokenAuthentication,)
    # permission_classes = (IsAuthenticated,)
    serializer_class = VenueFieldApprovalSerializer

    def get(self, request, format=None):
        response = {}
        responsedata = []

        # Get Pending Venues
        fields_obj = VenueField.objects.filter(status="O", is_deleted=False)
        field_class = VenueFieldApprovalSerializer(fields_obj, many=True)

        response['result'] = 1
        response['data'] = field_class.data
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


class VenueStatusApprovalAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    # authentication_classes = (TokenAuthentication,)
    # permission_classes = (IsAuthenticated,)
    serializer_class = ApprovalFieldStatusSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []

        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            # get page
            start_count = (serializers.validated_data['page'] - 1) * 10
            end_count = start_count + 10
            serializers.validated_data.pop("page")

            # Get Pending Venues
            fields_obj = VenueField.objects.filter(
                **serializers.validated_data)
            total_count = fields_obj.count()
            fields_obj = fields_obj[start_count:end_count]
            field_class = VenueFieldApprovalSerializer(fields_obj, many=True)

            response['result'] = 1
            response['data'] = field_class.data
            response['total_count'] = total_count
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            return Response(response, status=status.HTTP_200_OK)


class CampaignStatusApprovalAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    # authentication_classes = (TokenAuthentication,)
    # permission_classes = (IsAuthenticated,)
    serializer_class = ApprovalCampaignStatusSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []

        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            # get page
            start_count = (serializers.validated_data['page'] - 1) * 10
            end_count = start_count + 10
            serializers.validated_data.pop("page")

            # Get Filters
            if "start_date" in serializers.validated_data:
                serializers.validated_data['start_date__gte'] = serializers.validated_data["start_date"]
                serializers.validated_data.pop("start_date")

            if "end_date" in serializers.validated_data:
                serializers.validated_data['end_date__lte'] = serializers.validated_data["end_date"]
                serializers.validated_data.pop("end_date")

            if "campaign_kind" in serializers.validated_data:
                if serializers.validated_data['campaign_kind'] == "email":
                    serializers.validated_data['email'] = True
                elif serializers.validated_data['campaign_kind'] == "sms":
                    serializers.validated_data['sms'] = True
                elif serializers.validated_data['campaign_kind'] == "banner":
                    serializers.validated_data['banner'] = True
                serializers.validated_data.pop("campaign_kind")

            if "keyword" in serializers.validated_data:
                serializers.validated_data['name__icontains'] = serializers.validated_data['keyword']
                serializers.validated_data.pop("keyword")

            # Get Pending Venues
            if "assortment" in serializers.validated_data:
                assortment = serializers.validated_data['assortment']
                serializers.validated_data.pop("assortment")
                campaign_obj = CampaignPart1.objects.filter(
                    **serializers.validated_data).order_by("-" + assortment)
            else:
                campaign_obj = CampaignPart1.objects.filter(
                    **serializers.validated_data).order_by("-created_at")

            total_count = campaign_obj.count()
            campaign_obj = campaign_obj[start_count:end_count]
            campaign_class = CampaignApprovalSerializer(
                campaign_obj, many=True)

            response['result'] = 1
            response['data'] = campaign_class.data
            response['total_count'] = total_count
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            return Response(response, status=status.HTTP_200_OK)


class VenueChangeStatusAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    # authentication_classes = (TokenAuthentication,)
    # permission_classes = (IsAuthenticated,)
    serializer_class = VenueChangeStatusSerialzier

    def post(self, request, format=None):
        response = {}
        responsedata = []

        serializers = self.serializer_class(data=request.data)

        if serializers.is_valid():

            if serializers.validated_data['status'] == "Approved":
                serializers.validated_data['id'].status = "A"
                serializers.validated_data['id'].save()
                message = "Successfully approved"
                field_approval_email(
                    serializers.validated_data['id'].created_by.username)

            elif serializers.validated_data['status'] == "Declined":
                serializers.validated_data['id'].status = "R"
                serializers.validated_data['id'].is_deleted = True
                serializers.validated_data['id'].save()
                message = "Successfully Rejected"

            response['result'] = 1
            response['errors'] = message
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class GetMessageDetailsAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    # authentication_classes = (TokenAuthentication,)
    # permission_classes = (IsAuthenticated,)
    serializer_class = GetMessageIDSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []

        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            message_object = serializers.validated_data['id']
            message_object.is_read = True
            message_object.save()
            message_class = GetMessageDetailsSerializer(message_object)

            response['result'] = 1
            response['data'] = message_class.data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class DeleteMessageAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    # authentication_classes = (TokenAuthentication,)
    # permission_classes = (IsAuthenticated,)
    serializer_class = DeleteMessageSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []

        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            # Get message
            message_obj = serializers.validated_data['id']
            message_obj.is_deleted = True
            message_obj.save()
            # Send response
            response['result'] = 1
            response['error'] = "Successfully Deleted"
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class CampaignApprovalsAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    # authentication_classes = (TokenAuthentication,)
    # permission_classes = (IsAuthenticated,)
    serializer_class = CampaignApprovalSerializer

    def get(self, request, format=None):
        response = {}
        responsedata = []

        # Get Pending Camapign
        campaign_obj = CampaignPart1.objects.filter(
            status="O", is_deleted=False)
        campaign_class = CampaignApprovalSerializer(campaign_obj, many=True)

        response['result'] = 1
        response['data'] = campaign_class.data
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


class CampaignChangeStatusAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    # authentication_classes = (TokenAuthentication,)
    # permission_classes = (IsAuthenticated,)
    serializer_class = CampaignChangeStatusSerialzier

    def post(self, request, format=None):
        response = {}
        responsedata = []

        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            if serializers.validated_data['status'] == "Approved":
                serializers.validated_data['id'].status = "A"
                serializers.validated_data['id'].save()
                message = "Successfully approved"

            elif serializers.validated_data['status'] == "Declined":
                serializers.validated_data['id'].status = "R"
                serializers.validated_data['id'].save()
                message = "Successfully Rejected"

            response['result'] = 1
            response['errors'] = message
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class GetProfessionalServicesAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    # authentication_classes = (TokenAuthentication,)
    # permission_classes = (IsAuthenticated,)
    serializer_class = ProfessionalSerivcesSerializer

    def post(self, request, format=None):
        """
        Get Professional Services 
        """
        response = {}
        responsedata = []

        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            # get page
            start_count = (serializers.validated_data['page'] - 1) * 10
            end_count = start_count + 10
            serializers.validated_data.pop("page")
            professional_obj = Professional.objects.filter(
                **serializers.validated_data, is_deleted=False)
            total_count = professional_obj.count()
            professional_obj = professional_obj[start_count: end_count]
            professional_class = self.serializer_class(
                professional_obj, many=True)
            response['result'] = 1
            response['data'] = professional_class.data
            response['total_count'] = total_count
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class ProfessionalByIDAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    '''
    Get Professional by id
    '''
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = GetProfessionalByIDSerializer

    service_photo_serializer_class = ProfessionalServicePhotoSerializer
    service_data_serializer_class = ProfessionalStepServiceDataSerializer
    certificate_serializer_class = ProfessionalCertificateSerializer
    step_skills_serializer_class = ProfessionalStepSkillSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []

        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            professional_obj = Professional.objects.get(
                id=serializers.validated_data['id'])
            professional_class = ProfessionalSerivcesSerializer(
                professional_obj)

            service_photo = ProfessionalServicePhoto.objects.filter(
                created_by=professional_obj.user, is_deleted=False)

            service_data = ProfessionalStepServiceData.objects.filter(
                user=professional_obj.user, is_deleted=False)

            certificate = ProfessionalCertificate.objects.filter(
                created_by=professional_obj.user, is_deleted=False)

            step_skills = ProfessionalStepSkill.objects.filter(
                created_by=professional_obj.user, is_deleted=False)

            serializers2 = self.service_photo_serializer_class(
                service_photo, many=True)
            serializers3 = self.service_data_serializer_class(
                service_data, many=True)
            serializers4 = self.certificate_serializer_class(
                certificate, many=True)
            serializers5 = self.step_skills_serializer_class(
                step_skills, many=True)

            data = {"0": serializers2.data,
                    "1": serializers3.data, "2": serializers4.data}

            # Sending Response
            response["result"] = 1
            response['data'] = professional_class.data
            response['all_data'] = data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class ProfessionalChangeStatusAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    '''
    Change professional Status
    '''
    serializer_class = ProfessionalChangeStatusSerialzier

    def post(self, request, foramt=None):
        response = {}
        responsedata = []

        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            if serializers.validated_data['status'] == "Approved":
                serializers.validated_data['id'].status = "A"
                serializers.validated_data['id'].save()
                message = "Successfully approved"

            elif serializers.validated_data['status'] == "Declined":
                serializers.validated_data['id'].status = "R"
                serializers.validated_data['id'].save()
                message = "Successfully Rejected"

            response['result'] = 1
            response['errors'] = message
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class AdminPackagesGETAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    # authentication_classes = (TokenAuthentication,)
    # permission_classes = (IsAuthenticated,)
    serializer_class = AdminPackagesGetSerializer

    def post(self, request, format=None):
        '''
        Get All Packages
        '''
        response = {}
        responsedata = []
        # request.data._mutable = True

        serializer = self.serializer_class(data=request.data)
        serializer1 = self.serializer_class(data=request.data)
        if serializer.is_valid() and serializer1.is_valid():

            pagination = serializer.validated_data['page']
            serializer.validated_data.pop("page")
            serializer1.validated_data.pop("page")

            # Get Filters
            if "created_at_from" in serializer.validated_data:
                serializer.validated_data['created_at__gte'] = serializer.validated_data["created_at_from"]
                serializer.validated_data.pop("created_at_from")
                serializer1.validated_data['created_at__gte'] = serializer1.validated_data["created_at_from"]
                serializer1.validated_data.pop("created_at_from")

            if "created_at_to" in serializer.validated_data:
                serializer.validated_data['created_at__lte'] = serializer.validated_data["created_at_to"]
                serializer.validated_data.pop("created_at_to")
                serializer1.validated_data['created_at__lte'] = serializer1.validated_data["created_at_to"]
                serializer1.validated_data.pop("created_at_to")

            if "sport_relivance" in serializer.validated_data:
                serializer.validated_data['sport_relivance__in'] = serializer.validated_data["sport_relivance"]
                serializer.validated_data.pop("sport_relivance")
                serializer1.validated_data['sport_relivance__in'] = serializer1.validated_data["sport_relivance"]
                serializer1.validated_data.pop("sport_relivance")

            if "package_type" in serializer.validated_data:
                if serializer.validated_data['package_type'] == "Subscription":
                    serializer.validated_data['package_type__in'] = 'S'
                    serializer1.validated_data['package_type__in'] = 'S'

                elif serializer.validated_data['package_type'] == "Lessons":
                    serializer.validated_data['package_type__in'] = 'L'
                    serializer1.validated_data['package_type__in'] = 'L'

                elif serializer.validated_data['package_type'] == "Bookings Package":
                    serializer.validated_data['package_type__in'] = 'B'
                    serializer1.validated_data['package_type__in'] = 'B'
                serializer.validated_data.pop("package_type")
                serializer1.validated_data.pop("package_type")

            if "status_name" in serializer.validated_data:

                if serializer.validated_data["status_name"] == "PENDING":
                    serializer.validated_data['status_name'] = "O"
                    serializer1.validated_data['status_name'] = "O"

                elif serializer.validated_data["status_name"] == "APPROVED":
                    serializer.validated_data['status_name'] = "A"
                    serializer1.validated_data['status_name'] = "O"

                elif serializer.validated_data["status_name"] == "DECLINED":
                    serializer.validated_data['status_name'] = "R"
                    serializer1.validated_data['status_name'] = "O"

            if "keyword" in serializer.validated_data:
                serializer.validated_data['name__icontains'] = serializer.validated_data['keyword']
                a = serializer.validated_data['keyword']
                serializer.validated_data.pop("keyword")
                venues = Venue.objects.filter(name__icontains=a)
                if venues.count() > 0:
                    list_of_user_ids = []
                    for i in venues:
                        list_of_user_ids.append(i.created_by.id)
                    if len(list_of_user_ids) > 0:
                        serializer1.validated_data['created_by__in'] = list_of_user_ids
                    serializer1.validated_data.pop("keyword")
                else:
                    serializer1.validated_data.pop("keyword")

            start_data = (pagination - 1) * 10
            end_data = start_data + 10

            # print(serializer.validated_data)
            # print(serializer1.validated_data)
            if "created_by__in" in serializer1.validated_data:
                package_data = Packages.objects.filter(
                    Q(**serializer.validated_data) | Q(**serializer1.validated_data))
            else:
                package_data = Packages.objects.filter(
                    **serializer.validated_data)

            package_count = package_data.count()
            package_data = package_data[start_data:end_data]

            serializer = self.serializer_class(package_data, many=True)

            response["result"] = 1
            response['data'] = serializer.data
            response['total_page'] = package_count
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class PackageChangeStatusAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    # authentication_classes = (TokenAuthentication,)
    # permission_classes = (IsAuthenticated,)
    serializer_class = PackageChangeStatusSerialzier

    def post(self, request, format=None):
        response = {}
        responsedata = []

        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            if serializers.validated_data['status'] == "Approved":
                serializers.validated_data['id'].status = "A"
                serializers.validated_data['id'].save()
                message = "Successfully approved"

            elif serializers.validated_data['status'] == "Declined":
                serializers.validated_data['id'].status = "R"
                serializers.validated_data['id'].save()
                message = "Successfully Rejected"

            response['result'] = 1
            response['data'] = message
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class DeletePackageAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    # authentication_classes = (TokenAuthentication,)
    # permission_classes = (IsAuthenticated,)
    serializer_class = DeletePackageSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []

        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            # Get message
            package_obj = serializers.validated_data['id']
            package_obj.is_deleted = True
            package_obj.save()
            # Send response
            response['result'] = 1
            response['data'] = "Successfully Deleted"
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class DeleteFieldImage(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = DeleteFieldImageSerializer

    def post(self, request, format=None):
        """
        Delete Field Image
        """
        response = {}
        responsedata = []

        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            message = ""
            if VenueFieldImage.objects.filter(id=serializers.validated_data['id']).exists():
                VenueFieldImage.objects.filter(
                    id=serializers.validated_data['id']).delete()
                message = "FIELD IMAGE DELETED!"
            else:
                message = "NO RECORD FOUND"
            response["result"] = 1
            response['message'] = message
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class VenueOwnerStatisticsAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    '''
    Venue Owner Statistics API View
    '''

    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = VenueOwnerStatisticsSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []
        extra_args = {}

        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():

            # Venue Owner
            venue_owner = VenueOwner.objects.get(
                id=serializers.validated_data['venue_owner_id'])
            # print(venue_owner.area)

            # Venues
            venues = Venue.objects.filter(created_by=venue_owner.user).first()

            venues_class = VenueProfileSerializer(venues)

            # Venue Fields
            if "from_date" in serializers.validated_data:
                extra_args['schedule_slot__date__gte'] = serializers.validated_data['from_date']

            if "to_date" in serializers.validated_data:
                extra_args['schedule_slot__date__lte'] = serializers.validated_data['to_date']

            venue_fields = VenueField.objects.filter(
                venue=venues, is_deleted=False)

            # Check for no of venue fields
            venue_field_counts = len(venue_fields)
            print(venue_field_counts)

            field_list = []

            if "field" in serializers.validated_data:
                particular_field = VenueField.objects.get(
                    id=serializers.validated_data['field'])
                d = {}
                d['field_name'] = particular_field.name
                bookmatch_obj = BookMatch.objects.filter(
                    venue_field=particular_field.id, **extra_args)
                book_cost = 0
                for k in bookmatch_obj:
                    book_cost = book_cost + k.total_cost
                d['field_revenue'] = book_cost
                field_list.append(d)

            else:
                if venue_field_counts > 5:
                    venue_field_range = venue_fields[0:5]
                    for l in venue_field_range:
                        my_data = {}

                        my_data['field_name'] = l.name

                        bookmatch_obj = BookMatch.objects.filter(
                            venue_field=l.id, **extra_args)

                        book_cost = 0
                        for k in bookmatch_obj:
                            book_cost = book_cost + k.total_cost
                        my_data['field_revenue'] = book_cost
                        field_list.append(my_data)

                else:
                    for l in venue_fields:
                        my_data = {}

                        my_data['field_name'] = l.name

                        bookmatch_obj = BookMatch.objects.filter(
                            venue_field=l.id, **extra_args)

                        book_cost = 0
                        for k in bookmatch_obj:
                            book_cost = book_cost + k.total_cost
                        my_data['field_revenue'] = book_cost
                        field_list.append(my_data)

            # check for field_list

            venue_field_list = field_list

            # Booking Counts
            total_bookings_cost = 0
            player_list = []

            for i in venue_fields:
                match_data = BookMatch.objects.filter(
                    venue_field=i, **extra_args)
                print(match_data)

                for j in match_data:
                    schedule_slot = j.schedule_slot

                    # Check for player in bookings table
                    player_obj = j.created_by

                    # Check for player in bookings table
                    total_bookings_cost = total_bookings_cost + j.total_cost

                    # Check for unique player
                    if player_obj not in player_list:
                        player_list.append(player_obj)
                        print(player_list)

            # Check for professional
            professional_obj = Professional.objects.filter(
                area=venue_owner.area)

            # Check for venueowner advertisement
            advertisement_obj = CampaignPart1.objects.filter(
                created_by=venue_owner.user, paid_ads=True)

            views_cost = 0
            for m in advertisement_obj:
                part4_obj = CampaignPart4.objects.filter(
                    campaign1=m.id).first()
                # print(part4_obj.views.price)
                views_cost = views_cost + part4_obj.views.price

            advertisement_cost = views_cost
            # Player
            player = len(player_list)

            # booking_cost
            booking_cost = total_bookings_cost

            # Professional
            professional = professional_obj.count()

            # Advertisement
            advertisement = advertisement_obj.count()

            # Consolidating The Data
            data = {"venue_id": venues_class.data['id'], "player_count": player, "booking_cost": booking_cost,
                    "venue_field_counts": venue_field_counts, "venue_field_lists": venue_field_list,
                    "professional_count": professional,
                    "advertisement_count": advertisement, "advertisement_cost": advertisement_cost}

            # Sending Response
            response["result"] = 1
            response['data'] = data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        # Serializer Errors
        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class VenueOwnerContactPersonEditInfoAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    '''
    Venue Owner Contact person EDIT Info API View
    '''
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = VenueOwnerEditContactPerson

    def post(self, request, format=None):
        response = {}
        responsedata = []

        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():

            cp = Contact_Person.objects.get(
                id=serializers.validated_data['id'].id)
            cp.first_name = serializers.validated_data['first_name']
            cp.last_name = serializers.validated_data['last_name']
            cp.email = serializers.validated_data['email']
            cp.telephone = serializers.validated_data['telephone']

            cp.save()
            # Sending Response
            response["result"] = 1
            response['Data'] = "Updated Successfully"
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        # Serializer Errors
        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class VenueOwnerVenueFieldAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    '''
    Venue Owner VenueField API View
    '''

    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = VenueOwnerIDSerializer
    venue_field_serializer_class = VenueOwnerVenueFieldSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []
        extra_args = {}

        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():

            # Venue Owner
            venue_owner = VenueOwner.objects.get(
                id=serializers.validated_data['venue_owner_id'])
            # print(venue_owner.area)

            # Venues
            venues = Venue.objects.filter(created_by=venue_owner.user).first()

            venue_fields = VenueField.objects.filter(
                venue=venues, is_deleted=False)
            print(venue_fields)

            serialized_data = self.venue_field_serializer_class(
                venue_fields, many=True)
            # Sending Response
            response["result"] = 1
            response['data'] = serialized_data.data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        # Serializer Errors
        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class VenueOwnerListAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    '''
    Venue Owner List API View
    '''

    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = VenueOwnerSerializerForFilteredList

    venue_class = VenueOwnerSerializer

    @transaction.atomic
    def post(self, request, format=None):
        response = {}
        responsedata = []
        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            # get page number
            start_count = serializers.validated_data['page'] - 1
            start_count = start_count * 10
            end_count = start_count + 10

            extra_args = {}
            extra_args_owner = {}

            if "business_vat" in serializers.validated_data:
                extra_args['buisness_vat__icontains'] = serializers.validated_data['business_vat']

            if "all_sports" in serializers.validated_data:
                if Sport.objects.filter(id=serializers.validated_data["all_sports"]).exists():
                    extra_args['all_sports'] = Sport.objects.get(
                        id=serializers.validated_data['all_sports'])
                    #extra_args['all_sports'] = serializers.validated_data['all_sports']

            if "business_name_or_venue_name" in serializers.validated_data:
                venue_obj_vat = Venue.objects.filter(
                    (Q(name__icontains=serializers.validated_data['business_name_or_venue_name']) | Q(
                        business_name__icontains=serializers.validated_data['business_name_or_venue_name'])),
                    **extra_args).order_by("-created_at")

            else:
                venue_obj_vat = Venue.objects.filter(
                    is_deleted=False, **extra_args).order_by("-created_at")

            venue_serializer = VenueSerializerForList(venue_obj_vat, many=True)
            i = 0
            venue_owner_list = []
            for item in venue_serializer.data:
                venue_owner_info = {}
                if "area" in serializers.validated_data:
                    venue_owner_filter = VenueOwner.objects.filter(user=item['created_by'],
                                                                   area__id__in=serializers.validated_data['area'])
                else:
                    venue_owner_filter = VenueOwner.objects.filter(
                        user=item['created_by'])

                venue_owner_list.append(venue_owner_filter)

                venue_owner_serializer = VenueOwnerListSerializer(
                    venue_owner_filter, many=True)
                venue_owner_info['venue_owner'] = venue_owner_serializer.data
                venue_owner_info['venue'] = item

                # Get Ratings and Review

                reviews_count = VenueReviewAndRating.objects.filter(venue=item['id']
                                                                    ).count()
                ratings = VenueReviewAndRating.objects.filter(venue=item['id']
                                                              ).aggregate(Avg("rating"))
                venue_owner_info['reviews_count'] = reviews_count
                venue_owner_info['ratings'] = ratings

                i += 1

                # Get Contact Person
                for venue_owner_item in venue_owner_serializer.data:
                    rated_to_type = ContentType.objects.get_for_model(
                        VenueOwner)
                    contact_person = Contact_Person.objects.filter(
                        content_type__pk=rated_to_type.id, object_id=venue_owner_item['id'])

                    contact_person_class = ContactPersonSerializer(
                        contact_person, many=True)
                    venue_owner_info['contact_person'] = contact_person_class.data
                    responsedata.append(venue_owner_info)

            total_count = i
            responsedata = responsedata[start_count:end_count]
            response['result'] = 1
            response['data'] = responsedata
            response['total_count'] = total_count
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        # Serializer Errors
        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class VenueOwnerIdAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    '''
    Venue Owner Id from OnboardAPI
    '''

    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = OnBoardIdSerializer

    @transaction.atomic
    def post(self, request, format=None):
        response = {}
        responsedata = []
        print(request.data)
        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            onboardapiobj = OnBoardAPI.objects.get(
                id=serializers.validated_data['id'])
            onboard_serializers = OnBoardAPISerializer(onboardapiobj)

            if VenueOwner.objects.filter(
                    email=onboard_serializers.data['email']).exists():
                venue_owner_obj = VenueOwner.objects.get(
                    email=onboard_serializers.data['email'])
                venue_owner_obj_serializer = VenueOwnerSAVESerializer(
                    venue_owner_obj)
            else:
                response["result"] = 1
                response['error'] = 'Venue owner ID not exist for given On board ID, Please verify email address.'
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)

            # Sending Response
            response["result"] = 1
            response['venue_owner_Id'] = venue_owner_obj_serializer.data['id']
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        # Serializer Errors
        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class VenuePhotoAndAllServicesAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = VenuePhotoAndAllServicesSerializer

    def post(self, request, format=None):
        try:
            response = {}
            responsedata = []
            serializers = self.serializer_class(
                data=request.data, context={"id": request.data['id']})
            if serializers.is_valid():
                serializers.save()
            else:
                response = handle_errors(serializers.errors)
                response["result"] = 0
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_400_BAD_REQUEST)
            response["result"] = 1
            responsedata.append("Successfully Inserted")
            response['data'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        except Exception as e:
            response["result"] = 0
            responsedata.append(str(e))
            response['errors'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class VenueGetPhotoAndAllServicesAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = VenueOwnerIDGETSerializer
    venuePhoto_class = VenuePhotosGetSerializer
    venue_class = VenueAllServicesGetSerializer

    def post(self, request, format=None):

        try:
            response = {}
            responsedata = []
            serializers = self.serializer_class(data=request.data)

            if serializers.is_valid():

                vobj = VenueOwner.objects.get(
                    id=serializers.validated_data['id'])

                serializers.validated_data['user'] = vobj.user.id

                # print(serializers.validated_data)

                venue_data = Venue.objects.filter(
                    created_by=serializers.validated_data['user']).first()

                image_data = VenueImage.objects.filter(
                    venue=venue_data, is_deleted=False)

                serializers_venue = self.venue_class(venue_data, many=False)
                serializers_image = self.venuePhoto_class(
                    image_data, many=True)

                venue_services_cost = VenueServiceCost.objects.filter(
                    venue=venue_data)

                venue_service_cost_serializer = VenueServicesCostSerializer(
                    venue_services_cost, many=True)

                response['result'] = 1
                responsedata.append({'venue_profile': serializers_venue.data})
                responsedata.append(
                    {'venue_service_cost': venue_service_cost_serializer.data})
                responsedata.append({'venue_images': serializers_image.data})

                response['data'] = responsedata
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)
            else:
                response = handle_errors(serializers.errors)
                response["result"] = 0
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_400_BAD_REQUEST)
        except Exception as e:
            response["result"] = 0
            responsedata.append(str(e))
            response['errors'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class DeleteVenueImage(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = DeleteVenueImageSerializer

    def post(self, request, format=None):
        """
        Delete Venue Image
        """
        response = {}
        responsedata = []

        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            message = ""
            if VenueImage.objects.filter(id__in=serializers.validated_data['id']).exists():
                VenueImage.objects.filter(
                    id__in=serializers.validated_data['id']).delete()
                message = "VENUE IMAGE DELETED!"
            else:
                message = "NO RECORD FOUND"
            response["result"] = 1
            response['message'] = message
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class AdminVenueOwnerPOSTAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = AdminVenueOwnerPOSTSerializer

    def post(self, request, format=None):
        '''
        Create New Venue Owner
        '''
        try:
            response = {}
            responsedata = []
            errors = []
            request.data._mutable = True

            serializers = self.serializer_class(
                data=request.data)
            if serializers.is_valid():
                data = serializers.save()
                response["result"] = 1
                responsedata.append("Successfully Inserted")
                response["errors"] = errors
                response['ID'] = data
                response['data'] = responsedata

                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)
            else:
                response["result"] = 1
                response = handle_errors(serializers.errors)
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_400_BAD_REQUEST)

        except Exception as e:
            response["result"] = 0
            errors.append(str(e))
            response['errors'] = errors
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class AdminPackagesPostAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = AdminPackagesPostSerializer

    def post(self, request, format=None):
        '''
        Add and Update Packages
        '''
        responsedata = []
        response = {}

        vobj = VenueOwner.objects.get(id=request.data['venueowner_id'])

        request.data._mutable = True
        request.data['created_by'] = vobj.user.id
        # (request.data['created_by'])

        if 'package_id' in request.data:
            if Packages.objects.filter(id=request.data['package_id'], is_deleted=False).exists():
                serializer = self.serializer_class(instance=Packages.objects.filter(
                    id=request.data['package_id'], is_deleted=False).first(), data=request.data, )
            message = "Updated Successfully"
        else:
            serializer = self.serializer_class(data=request.data, )
            message = "Created Successfully"

        if serializer.is_valid():
            print(serializer.validated_data)
            serializer.validated_data.pop('venueowner_id')
            serializer.save()
            response["result"] = 1
            response['data'] = [message]
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class AdminPackagesGETAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = AdminPackagesGetSerializer

    def post(self, request, format=None):
        '''
        Get Packages
        '''
        response = {}
        responsedata = []

        vobj = VenueOwner.objects.get(id=request.data['venueowner_id'])
        print('vobj------------', vobj)

        request.data._mutable = True
        request.data['created_by'] = vobj.user.id
        print(request.data['created_by'])

        serializer = self.serializer_class(data=request.data)
        if serializer.is_valid():

            pagination = serializer.validated_data['page']
            serializer.validated_data.pop("page")

            # if "sport_relivance" in serializer.validated_data:
            #     serializer.validated_data['sport_relivance__in'] = serializer.validated_data["sport_relivance"]
            #     serializer.validated_data.pop("sport_relivance")

            # if "package_type" in serializer.validated_data:
            #     if serializer.validated_data['package_type'] == "Subscription":
            #         serializer.validated_data['package_type__in'] = 'S'

            #     elif serializer.validated_data['package_type'] == "Lessons":
            #         serializer.validated_data['package_type__in'] = 'L'

            #     elif serializer.validated_data['package_type'] == "Bookings Package":
            #         serializer.validated_data['package_type__in'] = 'B'
            #     serializer.validated_data.pop("package_type")

            if "status_name" in serializer.validated_data:
                if serializer.validated_data["status_name"] == "UNDER APPROVAL":
                    serializer.validated_data['status'] = "O"

                elif serializer.validated_data["status_name"] == "ON GOING":
                    serializer.validated_data['status'] = "A"

                elif serializer.validated_data["status_name"] == "WAITING":
                    serializer.validated_data['status'] = "A"

                elif serializer.validated_data["status_name"] == "REJECTED":
                    serializer.validated_data['status'] = "R"
                serializer.validated_data.pop("status_name")

            if "keyword" in serializer.validated_data:
                serializer.validated_data['name__icontains'] = serializer.validated_data['keyword']
                serializer.validated_data.pop("keyword")

            # if "package_duration" in serializer.validated_data:
            #     serializer.validated_data['package_duration__in'] = [
            #         serializer.validated_data["package_duration"]]
            #     serializer.validated_data.pop("package_duration")

            if "package_availabiity" in serializer.validated_data:
                print('---------if----------')
                package_availabiity = serializer.validated_data.pop(
                    "package_availabiity")
            else:
                package_availabiity = None

            start_data = (pagination - 1) * 10
            end_data = start_data + 10

            # print(serializer.validated_data)
            serializer.validated_data.pop('venueowner_id')
            package_data = Packages.objects.filter(
                **serializer.validated_data).order_by("-created_at")

            # package_availabiity
            for item in package_data:
                if package_availabiity == True:
                    if item.package_quantity > 0:
                        pass
                    else:
                        package_data = package_data.exclude(id=item.id)
                elif package_availabiity == False:
                    if item.package_quantity == 0:
                        pass
                    else:
                        package_data = package_data.exclude(id=item.id)
                else:
                    pass

            package_count = package_data.count()
            package_data = package_data[start_data:end_data]

            serializer = self.serializer_class(package_data, many=True)

            response["result"] = 1
            response['data'] = serializer.data
            response['total_page'] = package_count
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class AdminProfileServiceSerializerAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = AdminProfileServiceSerializer

    def post(self, request, format=None):
        '''
        Edit Venue Services
        '''
        try:
            responsedata = []
            response = {}
            request.data._mutable = True
            venue_owner = VenueOwner.objects.get(
                id=request.data['venueowner_id'])
            if Venue.objects.filter(created_by=venue_owner.user).exists():
                serializer = self.serializer_class(instance=Venue.objects.filter(
                    created_by=venue_owner.user, is_deleted=False).first(), data=request.data)
                message = "Updated Successfully"

                if serializer.is_valid():
                    serializer.validated_data['created_by'] = venue_owner.user
                    serializer.save()
                    response["result"] = 1
                    response['message'] = ["Updated Successfully"]
                    return Response(response, status=status.HTTP_200_OK)
                else:
                    response = handle_errors(serializer.errors)
                    response["result"] = 0
                    return Response(response, status=status.HTTP_400_BAD_REQUEST)
            else:
                raise Exception("No Such Venue")

        except Exception as e:
            response["result"] = 0
            responsedata.append(str(e))
            response['errors'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class AdminPaidServicesPOSTAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = AdminPaidServiceSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []
        venue_owner = VenueOwner.objects.get(id=request.data['venueowner_id'])
        venue = Venue.objects.filter(created_by=venue_owner.user).first()
        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            venue_service_cost_object = [VenueServiceCost(
                venue_service=VenueService.objects.filter(id=i).last(),
                price_per_hour="0",
                venue=venue,
            )
                for i in serializers.validated_data['venue_service']]

            VenueServiceCost.objects.bulk_create(
                venue_service_cost_object)
            response["result"] = 1
            response['message'] = "Added Successfully"
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class AdminVenueFieldPOSTServicesSerializerAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = AdminVenueFieldPOSTServicesSerializer

    def post(self, request, format=None):
        '''
        Post request for Updating the field into VenueFieldService

        '''
        responsedata = []
        response = {}
        v_owner_id = VenueOwner.objects.get(id=request.data['id'])

        request.data._mutable = True
        request.data['created_by'] = v_owner_id.user.id

        serializer = self.serializer_class(
            data=request.data, context={"id": request.data['id']})
        message = "Inserted Successfully"

        # save data
        if serializer.is_valid():
            field_obj = serializer.save()
            return_data = field_obj

            responsedata.append(message)
            response['results'] = responsedata
            response["result"] = 1
            response['data'] = return_data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class AdminVenueOwnerPaidServiceGETAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = IdSerializer
    serializer_class_1 = VenueOwnerPaidServiceCostSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []
        venue_service = []
        venue_owner = VenueOwner.objects.get(
            id=request.data['id'], is_deleted=False)
        venue = Venue.objects.filter(created_by=venue_owner.user).first()
        venue_service_cost_object = VenueServiceCost.objects.filter(
            venue=venue.id)
        serializers = self.serializer_class_1(
            venue_service_cost_object, many=True)
        print(serializers.data)

        for item in serializers.data:
            data = {}
            data['venue_paid_service'] = item['venue_service']
            data['price_per_hr'] = item['price_per_hour']
            responsedata.append(data)

        response["result"] = 1
        response['data'] = responsedata
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


class AdminVenuePOSTServicesSerializerAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = AdminVenuePOSTServicesSerializer

    def post(self, request, format=None):
        '''
        Post request for Updating the VenueService

        '''
        responsedata = []
        response = {}
        v_owner_id = VenueOwner.objects.get(id=request.data['id'])

        request.data._mutable = True
        request.data['created_by'] = v_owner_id.user.id

        venue = Venue.objects.filter(created_by=v_owner_id.user).first()

        serializer = self.serializer_class(
            data=request.data, context={"id": request.data['id']})
        message = "Inserted Successfully"

        # save data
        if serializer.is_valid():
            field_obj = serializer.save()
            return_data = field_obj

            responsedata.append(message)
            response['results'] = responsedata
            response["result"] = 1
            response['data'] = return_data
            response['data'] = return_data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class AdminVenueFieldPaidServiceGETAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = AdminVenueFieldPaidServiceSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []
        serializer = self.serializer_class(request.data)

        venue_owner = VenueOwner.objects.get(
            id=serializer.data['venueowner_id'])
        print(venue_owner)
        venue_field_object = VenueFieldService.objects.filter(
            venue_field=serializer.data['venue_field'], created_by=venue_owner.user)
        print(venue_field_object)
        serializers = VenueFieldPaidServiceSerializer1(
            venue_field_object, many=True)

        for item in serializers.data:
            data = {}
            data['venue_paid_service'] = item['venue_service']
            data['price_per_hr'] = item['price_per_hour']
            responsedata.append(data)

        response['result'] = 1
        response['data'] = responsedata

        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


class AdminVenueFieldGETInfoAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = AdminVenueFieldGETSerializer

    # Get Field Info

    def post(self, request, format=None):
        '''
        POST request for getting the field
        '''
        try:
            responsedata = []
            response = {}
            venue_owner = VenueOwner.objects.get(
                id=request.data['id'], is_deleted=False)
            print(venue_owner)
            venue = Venue.objects.filter(created_by=venue_owner.user).first()
            print(venue)
            serializers = self.serializer_class(data=request.data)
            if serializers.is_valid():

                FieldInfo = VenueField.objects.filter(
                    venue=venue.id, is_deleted=False)
                print(FieldInfo)

            else:
                response = handle_errors(serializers.errors)
                response["result"] = 0
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_400_BAD_REQUEST)

            serializers = self.serializer_class(FieldInfo, many=True)

            response['result'] = 1
            response['data'] = serializers.data
            # response['total_count'] = total_count
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        except Exception as e:
            response["result"] = 0
            responsedata.append(str(e))
            response['errors'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_401_UNAUTHORIZED)


class DocumentAddAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    '''
    Upload Documents
    '''
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = IdSerializer
    serializer_class_1 = DocumentSerializer

    def post(self, request, format=None):
        response = {}
        errors = []
        request.data._mutable = True
        request.data['created_by'] = VenueOwner.objects.get(
            id=request.data['id']).user.id
        print(request.data)
        serializer = self.serializer_class_1(data=request.data)
        if serializer.is_valid():
            serializer.save()
            response['result'] = 1
            errors.append("Successfully Inserted")
            response['errors'] = errors
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response['result'] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class DocumentDeleteAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    '''
    Delete Documents
    '''
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = DocumentDeleteSerializer

    def post(self, request, format=None):
        responsedata = []
        response = {}
        serializer = self.serializer_class(data=request.data)
        if serializer.is_valid():
            if Document.objects.filter(id=serializer.data['id'], is_deleted=False).exists():
                data = Document.objects.filter(
                    id=serializer.data['id'], is_deleted=False).first()
                data.is_deleted = True
                data.save(update_fields=['is_deleted'])
                responsedata.append("Successfully deleted")
                response['result'] = 0
                response['errors'] = responsedata
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)
            else:
                responsedata.append("No Record Found")
                response['result'] = 1
                response['errors'] = responsedata
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors())
            response['result'] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class DocumentListAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    '''
    List Documents
    '''
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = IdSerializer
    serializer_class_1 = DocumentSerializer

    def post(self, request, format=None):
        response = {}
        errors = []
        serializer = self.serializer_class(data=request.data)
        if serializer.is_valid():
            documentqs = Document.objects.filter(created_by=VenueOwner.objects.get(
                id=serializer.validated_data['id']).user, is_deleted=False)
            serializer1 = DocumentSerializer(documentqs, many=True)
            response['result'] = 1
            response['data'] = serializer1.data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response['result'] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class DeleteVenueFieldIdAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = DeleteFieldSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []

        if not VenueField.objects.filter(id=request.data['id']).exists():
            response['results'] = 0
            response['error'] = "Not a valid Field"
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        serializers = self.serializer_class(data=request.data)

        if serializers.is_valid():
            VenueField.objects.filter(
                id=serializers.validated_data['id']).delete()

            response['results'] = 1
            response['error'] = "Successfully Deleted"
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        else:
            response['results'] = 0
            response = handle_errors(serializers.errors)
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class BookingCancellationApprovalFilterAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    '''
    Booking cancellation filter API View
    '''

    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = BookingCancellationApprovalFilterSerializer

    @transaction.atomic
    def post(self, request, format=None):
        response = {}
        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            # get page number
            start_count = serializers.validated_data['page'] - 1
            start_count = start_count * 10
            end_count = start_count + 10
            extra_args = {}
            response_data = {}

            if "cancellation_reason" in serializers.validated_data:
                extra_args['cancellation_reason'] = serializers.validated_data['cancellation_reason']

            if "start_date" in serializers.validated_data:
                extra_args['start_date'] = serializers.validated_data['start_date']

            if "end_date" in serializers.validated_data:
                extra_args['end_date'] = serializers.validated_data['end_date']

            if "status" in serializers.validated_data:
                extra_args['status'] = serializers.validated_data['status']

            if "venue" in serializers.validated_data:
                venue_obj = Venue.objects.get(
                    id=serializers.validated_data['venue'])
                extra_args['created_by'] = venue_obj.created_by

            if "ordering" in serializers.validated_data:
                if serializers.validated_data['ordering'] == "DATE FROM-DESCENDING":
                    book_cancel_obj = RequestBookingCancellation.objects.filter(
                        **extra_args).order_by("-start_date")
                else:
                    book_cancel_obj = RequestBookingCancellation.objects.filter(
                        **extra_args).order_by("start_date")
            else:
                book_cancel_obj = RequestBookingCancellation.objects.filter(
                    **extra_args).order_by("-start_date")

            total_count = book_cancel_obj.count()
            book_cancel_obj = book_cancel_obj[start_count:end_count]
            booking_cancel_serializer = BookingCancellationRequestAllSerializer(
                book_cancel_obj, many=True)

            data = book_cancel_obj[start_count:end_count]
            response['result'] = 1
            response['data'] = booking_cancel_serializer.data
            response['total_count'] = total_count
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        # Serializer Errors
        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class AdminGetAllVenueAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    """
    Get all venue
    """
    serializer_class = AdminGetAllVenueAPIViewSerializer

    def get(self, request, format=None):
        response = {}
        responsedata = []
        data = Venue.objects.all()
        serializer = self.serializer_class(data, many=True)
        response['result'] = 1
        response['data'] = serializer.data
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


class BookingCancelGetAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    """
    Get all venue
    """
    serializer_class = BookingCancelGetSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []
        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            data = RequestBookingCancellation.objects.get(
                id=serializers.validated_data['id'])
            serializer = BookingCancellationRequestAllSerializer(data)
            response['result'] = 1
            response['data'] = serializer.data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class BookingCancelChangeApproveStatusAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    '''
    Changing Booking Cancel Form Status
    '''

    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = BookingCancelChangeApproveStatusSerializer

    @transaction.atomic
    def post(self, request, format=None):
        response = {}
        responsedata = []

        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            # Get Form
            request_booking_form = RequestBookingCancellation.objects.get(
                id=serializers.validated_data['id'])
            req_booking_cancel_obj_serializer = RequestBookingCancellationVenueOwnerSerializer(
                request_booking_form)

            # If Progress
            if serializers.validated_data['mode'] == "New":
                request_booking_form.status = "N"
                request_booking_form.save()
                message = "Status is New"

            elif serializers.validated_data['mode'] == "Approve":
                request_booking_form.status = "A"
                request_booking_form.save()
                message = "Successfully Approved"

                images = Template_Images.objects.filter().values('template_image')
                logo = images[0]
                logo2 = images[1]

                venue_owner_obj = request_booking_form.created_by

                booking_cancel_venuefield = request_booking_form.venue_field.all()

                venue_field_list = []
                for i in booking_cancel_venuefield:
                    venue_field_list.append(i.name)
                print(str(", ".join(venue_field_list)))
                # str(venue_field_list)[1:-1]
                if venue_field_list is not None:
                    msg = "Your bookings for " + str(", ".join(venue_field_list)) + " field are cancelled from " + \
                        str(request_booking_form.start_date) + "," + str(request_booking_form.start_time) + " to " \
                        + str(request_booking_form.end_date) + "," + str(request_booking_form.end_time) + \
                        ". All amounts are released. All your clients are informed by email and sms."
                else:
                    msg = "Your bookings for all fields are cancelled from " + \
                        str(request_booking_form.start_date) + "," + str(request_booking_form.start_time) + " to " \
                        + str(request_booking_form.end_date) + "," + str(request_booking_form.end_time) + \
                        ". All amounts are released. All your clients are informed by email and sms."

                booking_cancel_request_approve_mail(
                    venue_owner_obj.first_name, venue_owner_obj.email, msg, logo, logo2)

                for j in booking_cancel_venuefield:
                    bookmatch_obj = BookMatch.objects.filter(
                        venue_field=j.id)

                extra_args = {}
                extra_args['date__gte'] = request_booking_form.start_date
                extra_args['date__lte'] = request_booking_form.end_date

                sl = Schedule_Slot.objects.filter(start_hour=request_booking_form.start_time,
                                                  end_hour=request_booking_form.end_time,
                                                  **extra_args)
                print(sl)
                for item in sl:
                    bookmatch_obj = BookMatch.objects.filter(
                        schedule_slot=item)
                    print(bookmatch_obj)
                    # if len(bookmatch_obj) > 0:
                    for k in bookmatch_obj:
                        player_obj = Player.objects.get(
                            id=k.created_by.id)
                        player_obj_first_name = player_obj.first_name
                        player_obj_email = player_obj.email
                        k.delete()
                    cancel_reason = request_booking_form.cancellation_reason
                    msg = "Your booking is cancelled due to " + str(
                        cancel_reason) + " . You are not charged for the booking." \
                                         " We are sorry for the inconvenience."
                    booking_cancel_request_approve_mail(
                        player_obj_first_name, player_obj_email, msg, logo, logo2)

            # If Rejected
            elif serializers.validated_data['mode'] == "Rejected":
                request_booking_form.status = "R"
                request_booking_form.save()
                message = "Successfully Rejected"

            # Sending Response
            response["result"] = 1
            responsedata.append(message)
            response['data'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        # Serializer Errors
        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class DeleteCancellationReasonAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = DeleteCancellationReasonSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []

        if not BookingCancellation.objects.filter(id=request.data['id']).exists():
            response['results'] = 0
            response['error'] = "Not a valid ID"
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        serializers = self.serializer_class(data=request.data)

        if serializers.is_valid():
            BookingCancellation.objects.filter(
                id=serializers.validated_data['id']).delete()

            response['results'] = 1
            response['error'] = "Successfully Deleted"
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        else:
            response['results'] = 0
            response = handle_errors(serializers.errors)
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class GetSingleCancellationReasonAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = DeleteCancellationReasonSerializer

    def post(self, request, format=None):
        '''
        Get Cancellation Reason by Id
        '''
        response = {}
        responsedata = []

        serializer = self.serializer_class(data=request.data)
        if serializer.is_valid():

            if "id" in serializer.validated_data:
                if BookingCancellation.objects.filter(id=serializer.validated_data["id"]).exists():
                    cancellation_reason_id = BookingCancellation.objects.get(
                        id=serializer.validated_data["id"])
                else:
                    response["result"] = 0
                    response['data'] = "Invalid Cancellation Reason id"
                    self.add_api_call(request, response)
                    return Response(response, status=status.HTTP_200_OK)

            serializer = self.serializer_class(cancellation_reason_id)

            response["result"] = 1
            response['data'] = serializer.data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class CancellationReasonSaveEditInfoAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = SaveEditCancellationReasonSerializer

    def post(self, request, format=None):
        '''
        Add and Update BookingCancellation
        '''
        responsedata = []
        response = {}

        if 'id' in request.data:
            if BookingCancellation.objects.filter(id=request.data['id']).exists():
                serializer = self.serializer_class(instance=BookingCancellation.objects.get(
                    id=request.data['id']), data=request.data)
                message = "Updated Successfully"
            else:
                response["result"] = 0
                response['data'] = "Invalid Cancellation Reason id"
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)
        else:
            serializer = self.serializer_class(data=request.data)
            message = "Created Successfully"

        if serializer.is_valid():
            print(serializer.validated_data)
            serializer.save()
            response["result"] = 1
            response['data'] = [message]
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class CancellationPolicyEditInfoAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = EditCancellationPolicySerializer

    def post(self, request, format=None):
        '''
        Update CancellationPolicy
        '''
        responsedata = []
        response = {}

        if CancellationPolicy.objects.filter(id=request.data['id']).exists():
            serializer = self.serializer_class(instance=CancellationPolicy.objects.get(
                id=request.data['id']), data=request.data)
            message = "Updated Successfully"
        else:
            response["result"] = 0
            response['data'] = "Invalid Cancellation Policy id"
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        if serializer.is_valid():
            print(serializer.validated_data)
            serializer.save()
            response["result"] = 1
            response['data'] = [message]
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class CancellationPolicySaveInfoAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = SaveCancellationPolicySerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []

        all_data = CancellationPolicy.objects.all()
        for item in all_data:
            if int(request.data['start_time']) in range(item.start_time + 1, item.end_time) \
                    or int(request.data['end_time']) in range(item.start_time + 1, item.end_time):
                response["result"] = 0
                response['data'] = "Time Interval Already in Use"
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)

        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            CancellationPolicy_obj = serializers.save()
            # Sending Response
            response["result"] = 1
            responsedata.append("Successfully Inserted")
            response['data'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class DeleteCancellationPolicyAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = DeleteCancellationPolicySerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []

        if not CancellationPolicy.objects.filter(id=request.data['id']).exists():
            response['results'] = 0
            response['error'] = "Not a valid ID"
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        serializers = self.serializer_class(data=request.data)

        if serializers.is_valid():
            CancellationPolicy.objects.filter(
                id=serializers.validated_data['id']).delete()

            response['results'] = 1
            response['error'] = "Successfully Deleted"
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        else:
            response['results'] = 0
            response = handle_errors(serializers.errors)
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class CancellationPolicyGetAllAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    """
    Get All data from CancellationPolicy table
    """
    serializer_class = CancellationPolicyGetAllSerializer

    def post(self, request, format=None):
        # get page number
        if 'page' in request.data:
            start_count = int(request.data['page']) - 1
        else:
            start_count = 0
        start_count = start_count * 10
        end_count = start_count + 10
        response = {}
        responsedata = []
        data = CancellationPolicy.objects.all()[start_count:end_count]
        total_count = CancellationPolicy.objects.all().count()
        serializer = self.serializer_class(data, many=True)
        response['result'] = 1
        response['data'] = serializer.data
        response['total_count'] = total_count
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


class CancellationPolicyWithoutPaginationGetAllAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    """
    Get All data from CancellationPolicy table without pagination
    """
    serializer_class = CancellationPolicyGetAllWithoutPageSerializer

    def get(self, request, format=None):
        response = {}
        responsedata = []
        data = CancellationPolicy.objects.all()
        serializer = self.serializer_class(data, many=True)
        response['result'] = 1
        response['data'] = serializer.data
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


class GetSingleCancellationPolicyAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = DeleteCancellationPolicySerializer

    def post(self, request, format=None):
        '''
        Get Cancellation Policy by Id
        '''
        response = {}
        responsedata = []

        serializer = self.serializer_class(data=request.data)
        if serializer.is_valid():

            if "id" in serializer.validated_data:
                if CancellationPolicy.objects.filter(id=serializer.validated_data["id"]).exists():
                    CancellationPolicy_id = CancellationPolicy.objects.get(
                        id=serializer.validated_data["id"])
                else:
                    response["result"] = 0
                    response['data'] = "Invalid Cancellation Policy id"
                    self.add_api_call(request, response)
                    return Response(response, status=status.HTTP_200_OK)

            serializer = self.serializer_class(CancellationPolicy_id)

            response["result"] = 1
            response['data'] = serializer.data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class CollaborationTypesGetAllAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    """
    Get All data from CollaborationTypes table
    """
    serializer_class = CollaborationTypesGetAllSerializer

    def get(self, request, format=None):
        response = {}
        responsedata = []
        data = CollaborationTypes.objects.all()
        serializer = self.serializer_class(data, many=True)
        response['result'] = 1
        response['data'] = serializer.data
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


class DeleteCollaborationTypesAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = DeleteCollaborationTypesSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []

        if not CollaborationTypes.objects.filter(id=request.data['id']).exists():
            response['results'] = 0
            response['error'] = "Not a valid ID"
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        serializers = self.serializer_class(data=request.data)

        if serializers.is_valid():
            CollaborationTypes.objects.filter(
                id=serializers.validated_data['id']).delete()

            response['results'] = 1
            response['error'] = "Successfully Deleted"
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        else:
            response['results'] = 0
            response = handle_errors(serializers.errors)
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class CollaborationTypesSaveEditInfoAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = SaveEditCollaborationTypesSerializer

    def post(self, request, format=None):
        '''
        Add and Update CollaborationTypes
        '''
        responsedata = []
        response = {}

        if 'id' in request.data:
            if CollaborationTypes.objects.filter(id=request.data['id']).exists():
                serializer = self.serializer_class(instance=CollaborationTypes.objects.get(
                    id=request.data['id']), data=request.data)
                message = "Updated Successfully"
            else:
                response["result"] = 0
                response['data'] = "Invalid Cancellation Reason id"
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)
        else:
            serializer = self.serializer_class(data=request.data)
            message = "Created Successfully"

        if serializer.is_valid():
            print(serializer.validated_data)
            serializer.save()
            response["result"] = 1
            response['data'] = [message]
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class GetSingleCollaborationTypesAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = DeleteCollaborationTypesSerializer

    def post(self, request, format=None):
        '''
        Get CollaborationTypes by Id
        '''
        response = {}
        responsedata = []

        serializer = self.serializer_class(data=request.data)
        if serializer.is_valid():

            if "id" in serializer.validated_data:
                if CollaborationTypes.objects.filter(id=serializer.validated_data["id"]).exists():
                    CollaborationTypes_id = CollaborationTypes.objects.get(
                        id=serializer.validated_data["id"])
                else:
                    response["result"] = 0
                    response['data'] = "Invalid Cancellation Types id"
                    self.add_api_call(request, response)
                    return Response(response, status=status.HTTP_200_OK)

            serializer = self.serializer_class(CollaborationTypes_id)

            response["result"] = 1
            response['data'] = serializer.data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class Covid19MeasuresSaveEditInfoAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = SaveEditCovid19MeasuresSerializer

    def post(self, request, format=None):
        '''
        Add and Update Covid19Measures
        '''
        responsedata = []
        response = {}

        if 'id' in request.data:
            if Covid19Measures.objects.filter(id=request.data['id']).exists():
                serializer = self.serializer_class(instance=Covid19Measures.objects.get(
                    id=request.data['id']), data=request.data)
                message = "Updated Successfully"
            else:
                response["result"] = 0
                response['data'] = "Invalid Covid19Measures id"
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)
        else:
            serializer = self.serializer_class(data=request.data)
            message = "Created Successfully"

        if serializer.is_valid():
            print(serializer.validated_data)
            serializer.save()
            response["result"] = 1
            response['data'] = [message]
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class Covid19MeasuresGetAllAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    """
    Get All data from CancellationPolicy table
    """
    serializer_class = Covid19MeasuresGetAllSerializer

    def post(self, request, format=None):
        # get page number
        if 'page' in request.data:
            start_count = int(request.data['page']) - 1
        else:
            start_count = 0
        start_count = start_count * 10
        end_count = start_count + 10
        response = {}
        responsedata = []
        data = Covid19Measures.objects.all()[start_count:end_count]
        total_count = Covid19Measures.objects.all().count()
        serializer = self.serializer_class(data, many=True)
        response['result'] = 1
        response['data'] = serializer.data
        response['total_count'] = total_count
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


class Covid19MeasuresGetAllWithoutPageAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    """
    Get All data from CollaborationTypes table
    """
    serializer_class = SaveEditCovid19MeasuresSerializer

    def get(self, request, format=None):
        response = {}
        responsedata = []
        data = Covid19Measures.objects.all()
        serializer = self.serializer_class(data, many=True)
        response['result'] = 1
        response['data'] = serializer.data
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


class DeleteCovid19MeasuresAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = DeleteCovid19MeasuresSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []

        if not Covid19Measures.objects.filter(id=request.data['id']).exists():
            response['results'] = 0
            response['error'] = "Not a valid ID"
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        serializers = self.serializer_class(data=request.data)

        if serializers.is_valid():
            Covid19Measures.objects.filter(
                id=serializers.validated_data['id']).delete()

            response['results'] = 1
            response['error'] = "Successfully Deleted"
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        else:
            response['results'] = 0
            response = handle_errors(serializers.errors)
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class AdminBookingCancellationRequestPOSTAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = AdminBookingCancellationRequestPostSerializer

    def post(self, request, format=None):
        '''
        Admin BookingCancellationRequest post API
        '''
        try:
            response = {}
            responsedata = []
            request.data._mutable = True
            serializers = self.serializer_class(data=request.data)
            v_owner_id = VenueOwner.objects.get(
                id=request.data['venueowner_id'])

            msg = "We received your bookings cancellation request.It will be processed shortly."
            venue_owner_booking_cancel_Complete_mail(
                v_owner_id.first_name, v_owner_id.email, msg)
            if serializers.is_valid():
                serializers.validated_data.pop('venueowner_id')
                serializers.validated_data['created_by'] = v_owner_id.user
                data = serializers.save()
                response["result"] = 1
                responsedata.append("Successfully Inserted")
                response["data"] = responsedata
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)
            else:
                response["result"] = 1
                response = handle_errors(serializers.errors)
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_400_BAD_REQUEST)
        except Exception as e:
            response["result"] = 0
            responsedata.append(str(e))
            response['errors'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class AdminBookingCancellationFilterAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    '''
    Admin Booking cancellation filter API View
    '''

    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = AdminBookingCancellationFilterSerializer

    @transaction.atomic
    def post(self, request, format=None):
        response = {}
        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():

            vobj = VenueOwner.objects.get(
                id=serializers.validated_data['venueowner_id'])
            # print(vobj.user.id)

            # get page number
            start_count = serializers.validated_data['page'] - 1
            start_count = start_count * 10
            end_count = start_count + 10
            serializers.validated_data['created_by'] = vobj.user.id

            serializers.validated_data.pop('venueowner_id')
            extra_args = {}

            if "cancellation_reason" in serializers.validated_data:
                extra_args['cancellation_reason'] = serializers.validated_data['cancellation_reason']

            if "start_date" in serializers.validated_data:
                extra_args['start_date'] = serializers.validated_data['start_date']

            if "end_date" in serializers.validated_data:
                extra_args['end_date'] = serializers.validated_data['end_date']

            if "status" in serializers.validated_data:
                extra_args['status'] = serializers.validated_data['status']

            if "venue_field" in serializers.validated_data:
                extra_args['venue_field__in'] = serializers.validated_data['venue_field']

                book_cancel_obj = RequestBookingCancellation.objects.filter(created_by=vobj.user,
                                                                            **extra_args).distinct().order_by("-created_at")
            else:
                book_cancel_obj = RequestBookingCancellation.objects.filter(created_by=vobj.user,
                                                                            **extra_args).order_by("-created_at")

            total_count = book_cancel_obj.count()
            book_cancel_obj = book_cancel_obj[start_count:end_count]
            booking_cancel_serializer = BookingCancellationRequestAllSerializer(
                book_cancel_obj, many=True)

            data = book_cancel_obj[start_count:end_count]
            response['result'] = 1
            response['data'] = booking_cancel_serializer.data
            response['total_count'] = total_count
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        # Serializer Errors
        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class AdminBookingCancellationRequestEditAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = AdminBookingCancellationRequestEditSerializer

    def post(self, request, format=None):
        '''
        Admin Booking cancellation Edit API View
        '''
        response = {}
        responsedata = []
        v_owner = VenueOwner.objects.get(
            id=request.data['venueowner_id'])
        # print(v_owner.user)
        request.data._mutable = True
        request.data['created_by'] = v_owner.user.id

        if RequestBookingCancellation.objects.filter(id=request.data['id'], created_by=v_owner.user, ).exists():
            serializers = self.serializer_class(instance=RequestBookingCancellation.objects.filter(
                id=request.data['id'], created_by=v_owner.user, ).first(), data=request.data)
            message = "Successfully Updated"
        else:
            response["result"] = 0
            responsedata.append("No Such Record to update")
            response['errors'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        if serializers.is_valid():
            serializers.validated_data.pop('venueowner_id')
            save_data = serializers.save()
            response["result"] = 1
            responsedata.append(message)
            response['data'] = responsedata
            response['id'] = save_data.id
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializers.errors)
            response['result'] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class ProfessionalsListAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    '''
    Professionals List API View
    '''

    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = ProfessinalsSerializerForFilteredList

    @transaction.atomic
    def post(self, request, format=None):
        response = {}
        responsedata = []
        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            # get page number
            start_count = serializers.validated_data['page'] - 1
            start_count = start_count * 10
            end_count = start_count + 10

            extra_args = {}

            if "business_vat" in serializers.validated_data:
                extra_args['buisness_vat__icontains'] = serializers.validated_data['business_vat']

            if "business_name" in serializers.validated_data:
                extra_args['business_name__icontains'] = serializers.validated_data['business_name']

            if "area" in serializers.validated_data and "service" in serializers.validated_data:
                prof_filter = Professional.objects.filter(is_deleted=False,
                                                          area__id__in=serializers.validated_data['area'],
                                                          service__id__in=serializers.validated_data['service'],
                                                          **extra_args).order_by("-created_at")
            elif "area" in serializers.validated_data:
                prof_filter = Professional.objects.filter(is_deleted=False,
                                                          area__id__in=serializers.validated_data['area'],
                                                          **extra_args).order_by("-created_at")

            elif "service" in serializers.validated_data:
                prof_filter = Professional.objects.filter(is_deleted=False,
                                                          service__in=serializers.validated_data['service'],
                                                          **extra_args).order_by("-created_at")
            else:
                prof_filter = Professional.objects.filter(is_deleted=False,
                                                          **extra_args).order_by("-created_at")
            #     print('-------else---')
            # print('prof_filter before---', prof_filter)
            prof_filter = (prof_filter)
            # print('prof_filter after---', prof_filter)

            prof_serializer = ProfessionalListSerializer(
                prof_filter, many=True)
            # prof_info['professional'] = prof_serializer.data
            # responsedata.append(prof_info)

            # Get Contact Person
            contact_info = {}
            for prof_item in prof_serializer.data:
                prof_info = {}
                rated_to_type = ContentType.objects.get_for_model(
                    Professional)
                contact_person = Contact_Person.objects.filter(
                    content_type__pk=rated_to_type.id, object_id=prof_item['id'])
                contact_person = set(contact_person)

                contact_person_class = ContactPersonSerializer(
                    contact_person, many=True)
                prof_info['professional'] = prof_item

                prof_info['contact_person'] = contact_person_class.data
                responsedata.append(prof_info)

            alldata = responsedata[start_count:end_count]
            response['result'] = 1
            response['data'] = alldata
            response['total_count'] = len(responsedata)
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        # Serializer Errors
        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class AdvertiserListAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    '''
    Advertiser List API View
    '''

    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = AdvertiserSerializerForFilteredList

    @transaction.atomic
    def post(self, request, format=None):
        response = {}
        responsedata = []
        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            # get page number
            start_count = serializers.validated_data['page'] - 1
            start_count = start_count * 10
            end_count = start_count + 10

            extra_args = {}

            if "business_vat" in serializers.validated_data:
                extra_args['buisness_vat'] = serializers.validated_data['business_vat']

            if "business_name" in serializers.validated_data:
                extra_args['business_name'] = serializers.validated_data['business_name']

            if "area" in serializers.validated_data:
                adv_filter = Advertiser.objects.filter(is_deleted=False,
                                                       area__id__in=serializers.validated_data['area'],
                                                       **extra_args)
            else:
                adv_filter = Advertiser.objects.filter(is_deleted=False,
                                                       **extra_args)
            adv_filter = set(adv_filter)

            adv_serializer = AdvertiserListSerializer(
                adv_filter, many=True)

            # Get Contact Person
            for adv_item in adv_serializer.data:
                adv_info = {}
                rated_to_type = ContentType.objects.get_for_model(
                    Professional)
                contact_person = Contact_Person.objects.filter(
                    content_type__pk=rated_to_type.id, object_id=adv_item['id'])

                contact_person_class = ContactPersonSerializer(
                    contact_person, many=True)
                adv_info['Advertiser'] = adv_item
                adv_info['contact_person'] = contact_person_class.data
                responsedata.append(adv_info)

            alldata = responsedata[start_count:end_count]
            response['result'] = 1
            response['data'] = alldata
            response['total_count'] = len(responsedata)
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        # Serializer Errors
        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class AdminNewPackagesGetAndFiltersAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    # authentication_classes = (TokenAuthentication,)
    # permission_classes = (IsAuthenticated,)
    serializer_class = AdminNewPackagesGetAndFiltersSerializer

    def post(self, request, format=None):
        '''
        Get Packages
        '''
        response = {}
        responsedata = []

        serializer = self.serializer_class(data=request.data)
        serializer1 = self.serializer_class(data=request.data)
        if serializer.is_valid() and serializer1.is_valid():

            pagination = serializer.validated_data['page']
            serializer.validated_data.pop("page")
            serializer1.validated_data.pop("page")

            # if "sport_relivance" in serializer.validated_data:
            #     serializer.validated_data['sport_relivance__in'] = serializer.validated_data["sport_relivance"]
            #     serializer.validated_data.pop("sport_relivance")

            if "creation_date_from" in serializer.validated_data:
                serializer.validated_data['created_at__gte'] = serializer.validated_data["creation_date_from"]
                serializer.validated_data.pop("creation_date_from")
                serializer1.validated_data['created_at__gte'] = serializer1.validated_data["creation_date_from"]
                serializer1.validated_data.pop("creation_date_from")

            if "creation_date_to" in serializer.validated_data:
                serializer.validated_data['created_at__lte'] = serializer.validated_data["creation_date_to"]
                serializer.validated_data.pop("creation_date_to")
                serializer1.validated_data['created_at__lte'] = serializer1.validated_data["creation_date_to"]
                serializer1.validated_data.pop("creation_date_to")

            # if "package_type" in serializer.validated_data:
            #     if serializer.validated_data['package_type'] == "Subscription":
            #         serializer.validated_data['package_type__in'] = 'S'
            #         serializer1.validated_data['package_type__in'] = 'S'

            #     elif serializer.validated_data['package_type'] == "Lessons":
            #         serializer.validated_data['package_type__in'] = 'L'
            #         serializer1.validated_data['package_type__in'] = 'L'

            #     elif serializer.validated_data['package_type'] == "Bookings Package":
            #         serializer.validated_data['package_type__in'] = 'B'
            #         serializer1.validated_data['package_type__in'] = 'B'
            #     serializer.validated_data.pop("package_type")
            #     serializer1.validated_data.pop("package_type")

            if "status_name" in serializer.validated_data:
                if serializer.validated_data["status_name"] == "NEW":
                    serializer.validated_data['status'] = "O"
                    serializer1.validated_data['status'] = "O"

                elif serializer.validated_data["status_name"] == "APPROVED":
                    serializer.validated_data['status'] = "A"
                    serializer1.validated_data['status'] = "A"

                elif serializer.validated_data["status_name"] == "REJECTED":
                    serializer.validated_data['status'] = "R"
                    serializer1.validated_data['status'] = "R"
                serializer.validated_data.pop("status_name")
                serializer1.validated_data.pop("status_name")

            if "keyword" in serializer.validated_data:
                serializer.validated_data['name__icontains'] = serializer.validated_data['keyword']
                a = serializer.validated_data['keyword']
                serializer.validated_data.pop("keyword")
                venues = Venue.objects.filter(name__icontains=a)
                if venues.count() > 0:
                    list_of_user_ids = []
                    for i in venues:
                        list_of_user_ids.append(i.created_by.id)
                    if len(list_of_user_ids) > 0:
                        serializer1.validated_data['created_by__in'] = list_of_user_ids
                    serializer1.validated_data.pop("keyword")
                else:
                    serializer1.validated_data.pop("keyword")

            start_data = (pagination - 1) * 10
            end_data = start_data + 10

            # print(serializer.validated_data)
            # print(serializer1.validated_data)
            if "created_by__in" in serializer1.validated_data:
                package_data = Packages.objects.filter(
                    Q(**serializer.validated_data) | Q(**serializer1.validated_data)).order_by(
                    "-created_at")
            else:
                package_data = Packages.objects.filter(
                    **serializer.validated_data).order_by(
                    "-created_at")

            package_count = package_data.count()
            package_data = package_data[start_data:end_data]

            serialized = self.serializer_class(package_data, many=True)

            response["result"] = 1
            response['data'] = serialized.data
            response['total_page'] = package_count
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class PackageTypesPostAndEditAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = PackageTypesSerializer

    def post(self, request, format=None):
        '''
        Add and Update Package Types
        '''
        responsedata = []
        response = {}

        if 'id' in request.data:
            if PackageTypes.objects.filter(id=request.data['id']).exists():
                serializer = self.serializer_class(instance=PackageTypes.objects.filter(
                    id=request.data['id']).first(), data=request.data)
            message = "Updated Successfully"
        else:
            serializer = self.serializer_class(data=request.data)
            message = "Created Successfully"

        if serializer.is_valid():
            serializer.save()
            response["result"] = 1
            response['data'] = [message]
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class PackageTypesGetAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    """
    Get all package types
    """
    serializer_class = PackageTypesGetSerializer

    def get(self, request, format=None):
        response = {}
        responsedata = []
        data = PackageTypes.objects.all()
        serializer = self.serializer_class(data, many=True)
        response['result'] = 1
        response['data'] = serializer.data
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


class PackageTypesDeleteAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = DeletePackageTypesSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []

        if not PackageTypes.objects.filter(id=request.data['id']).exists():
            response['results'] = 0
            response['error'] = "Not a valid ID"
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        serializers = self.serializer_class(data=request.data)

        if serializers.is_valid():
            PackageTypes_obj = PackageTypes.objects.get(
                id=serializers.validated_data['id'])
            PackageTypes_obj.delete()

            response['results'] = 1
            response['data'] = "Successfully Deleted"
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        else:
            response['results'] = 0
            response = handle_errors(serializers.errors)
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class GetSinglePackageTypesAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = SinglePackageTypesSerializer

    def post(self, request, format=None):
        '''
        Get Package Types  by Id
        '''
        response = {}
        responsedata = []

        serializer = self.serializer_class(data=request.data)
        if serializer.is_valid():

            if PackageTypes.objects.filter(id=serializer.validated_data["id"]).exists():
                PackageTypes_obj = PackageTypes.objects.get(
                    id=serializer.validated_data["id"])
            else:
                response["result"] = 0
                response['error'] = "Invalid Package Type id"
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)

            serializer = self.serializer_class(PackageTypes_obj)

            response["result"] = 1
            response['data'] = serializer.data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class PackageDurationsPostAndEditAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = PackageDurationsSerializer

    def post(self, request, format=None):
        '''
        Add and Update Package Duration
        '''
        responsedata = []
        response = {}

        if 'id' in request.data:
            if PackageDurations.objects.filter(id=request.data['id']).exists():
                serializer = self.serializer_class(instance=PackageDurations.objects.filter(
                    id=request.data['id']).first(), data=request.data)
            message = "Updated Successfully"
        else:
            serializer = self.serializer_class(data=request.data)
            message = "Created Successfully"

        if serializer.is_valid():
            serializer.save()
            response["result"] = 1
            response['data'] = [message]
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class PackageDurationsGetAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    """
    Get all Package Durations
    """
    serializer_class = PackageDurationsGetSerializer

    def get(self, request, format=None):
        response = {}
        responsedata = []
        data = PackageDurations.objects.all()
        serializer = self.serializer_class(data, many=True)
        response['result'] = 1
        response['data'] = serializer.data
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


class PackageDurationsDeleteAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = DeletePackageDurationsSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []

        if not PackageDurations.objects.filter(id=request.data['id']).exists():
            response['results'] = 0
            response['error'] = "Not a valid ID"
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        serializers = self.serializer_class(data=request.data)

        if serializers.is_valid():
            PackageDurations_obj = PackageDurations.objects.get(
                id=serializers.validated_data['id'])
            PackageDurations_obj.delete()

            response['results'] = 1
            response['data'] = "Successfully Deleted"
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        else:
            response['results'] = 0
            response = handle_errors(serializers.errors)
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class GetSinglePackageDurationsAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = SinglePackageDurationsSerializer

    def post(self, request, format=None):
        '''
        Get Package Durations  by Id
        '''
        response = {}
        responsedata = []

        serializer = self.serializer_class(data=request.data)
        if serializer.is_valid():

            if PackageDurations.objects.filter(id=serializer.validated_data["id"]).exists():
                PackageDurations_obj = PackageDurations.objects.get(
                    id=serializer.validated_data["id"])
            else:
                response["result"] = 0
                response['error'] = "Invalid Package Type id"
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)

            serializer = self.serializer_class(PackageDurations_obj)

            response["result"] = 1
            response['data'] = serializer.data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)
