from django.db import models

from django.core.validators import MaxValueValidator, MinValueValidator

from django.contrib.auth.models import User
from storages.backends.s3boto import S3BotoStorage
from django.core.files.images import get_image_dimensions
from django.core.exceptions import ValidationError

from static_data.models import *


def validate_image(image):
    w, h = get_image_dimensions(image)
    if w != 400 and h != 400:
        raise ValidationError("Image is not of correct Dimensions")


class Advertiser(models.Model):
    """ Advertiser """
    GENDER_CHOICES = (
        ('M', 'Male'),
        ('F', 'Female'),
        ('O', 'Other')
    )

    user = models.ForeignKey(
        User, on_delete=models.CASCADE, blank=True, null=True)

    first_name = models.CharField(
        max_length=100, null=True, help_text="user first name")
    middle_name = models.CharField(
        max_length=100, null=True, help_text="user middle name", blank=True)
    last_name = models.CharField(
        max_length=100, null=True, help_text="user last name", blank=True)

    email = models.CharField(max_length=100, null=True,
                             blank=True, help_text="user email")
    new_email = models.CharField(
        max_length=100, null=True, blank=True, help_text="user new email")
    business_name = models.CharField(
        max_length=100, null=True, blank=True, help_text="user buisness name")

    buisness_type = models.CharField(
        max_length=100, null=True, blank=True, help_text="user buisness Type")
    buisness_vat = models.CharField(
        max_length=100, null=True, blank=True, help_text="user buisness vat")
    buisness_tax_office = models.CharField(
        max_length=100, null=True, blank=True, help_text="user buisness Tax office")
    # working_hours			= models.CharField(max_length=100, null=True, blank=True, help_text="user buisness working hours")

    # mobile     = models.CharField(max_length=10, null=True, help_text="user mobile number", unique=True)
    mobile = models.CharField(max_length=15, null=True,
                              blank=True, help_text="user mobile number")
    # new_mobile = models.CharField(max_length=10, null=True, help_text="user new mobile number", unique=True,blank=True)
    new_mobile = models.CharField(
        max_length=10, null=True, help_text="user new mobile number", blank=True)
    telephone = models.CharField(max_length=15, null=True, blank=True)
    telephone2 = models.CharField(max_length=15, null=True, blank=True)

    gender = models.CharField(max_length=1, null=True, choices=GENDER_CHOICES,
                              help_text="M - Male, F - Female, O - Other")
    address1 = models.CharField(max_length=150, blank=True, null=True,
                                help_text="Apartment, Suite, Unit, building, floor,etc.")
    address2 = models.CharField(
        max_length=150, blank=True, null=True, help_text="Street & Number, P.O.box c/o")
    profile = models.TextField(blank=True, null=True)
    user_code = models.CharField(max_length=6, null=True, blank=True)
    city = models.ForeignKey(
        City, on_delete=models.CASCADE, blank=True, null=True)
    country = models.ForeignKey(
        Country, on_delete=models.CASCADE, blank=True, null=True)
    area = models.ForeignKey(
        Area, on_delete=models.CASCADE, blank=True, null=True)
    #city = models.CharField(max_length=50, blank=True,null=True, help_text="city")
    state = models.ForeignKey(
        State, on_delete=models.SET_NULL, blank=True, null=True)
    #country = models.CharField(max_length=50, blank=True, null=True, help_text="country")
    pincode = models.CharField(
        max_length=10, blank=True, null=True, help_text="code")
    image = models.ImageField(upload_to="advertiser_images", blank=True,
                              null=True, help_text="Profile image", validators=[validate_image])
    dob = models.DateField(
        blank=True, null=True, help_text='date of birth format YYYY-MM-DD', verbose_name='date of birth')

    is_deleted = models.BooleanField(
        default=False, help_text="delete/restore user")
    is_inactive = models.BooleanField(
        default=False, help_text="activate/deactivate user")

    # created_by = models.ForeignKey('self', on_delete=models.SET_NULL, null=True, blank=True)

    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)
    updated_at = models.DateTimeField(auto_now=True, blank=True, null=True)

    def __str__(self):
        return "{}-{}".format(self.user.username, self.user.email)

    class Meta:
        unique_together = ('user', 'email')
        verbose_name_plural = "Advertiser Users"
        db_table = 'advertiser_user'


class AdvertiserDailyTiming(models.Model):
    """
    Weekly Timing of Advertiser
    """
    advertiser = models.ForeignKey(
        Advertiser, on_delete=models.CASCADE, blank=True, null=True)
    day = models.ForeignKey(
        DayofWeek, on_delete=models.SET_NULL, blank=True, null=True, help_text="Day")
    start_time = models.TimeField(
        blank=True, null=True, help_text="Start time")
    end_time = models.TimeField(blank=True, null=True, help_text="Start time")
    is_active = models.BooleanField(blank=True, null=True)
    is_deleted = models.BooleanField(default=False, null=True, blank=True)
    created_at = models.DateTimeField(auto_now=True, blank=True, null=True)

    def __str__(self):
        return "{}-{}-{}-{}".format(self.advertiser, self.day, self.start_time, self.end_time)
