from rest_framework import serializers
from django.contrib.auth.models import User, Group
from rest_framework.authtoken.models import Token
from django.contrib.contenttypes.models import ContentType

from utils.api_handle_errors import handle_errors
from datetime import date
from static_data.serializers import *
from .models import *
from commonAPI.models import TelephoneNumbers, ContactInfo, Contact_Person
from commonAPI.serializers import ContactInfoProfileSerializer, BankInfoProfileSerializer, ContactPersonSerializer
from django.db import transaction


class AdvertiserSerializers(serializers.ModelSerializer):
    user = serializers.CharField(source="user.username", read_only=True)
    country = CountrySerializer(read_only=True)
    city = CityGetSerializer(read_only=True)
    state = StateGetSerializer(read_only=True)
    area = AreaGetSerializer(read_only=True)

    class Meta:
        model = Advertiser
        fields = '__all__'


class AdvertiserSAVESerializer(serializers.ModelSerializer):
    class Meta:
        model = Advertiser
        fields = '__all__'


class AdvertiserDailyAvailibilitySerializer(serializers.ModelSerializer):
    class Meta:
        model = AdvertiserDailyTiming
        fields = '__all__'


class AdvertiserPOSTSerializers(serializers.ModelSerializer):
    Telephone_number = serializers.JSONField(
        required=False, help_text='{"mobile":["847123654","35135654"]}')
    iban = serializers.CharField(required=False)
    bank = serializers.CharField(required=False)
    acc_holder_first_name = serializers.CharField(required=False)
    contact_person = serializers.JSONField(
        required=False,
        help_text='[{"first_name":"Test","last_name":"asd","telephone":1234567,"email":"email"},{"first_name":"Test","last_name":"asd","telephone":1234567,"email":"email"}]')
    acc_holder_last_name = serializers.CharField(required=False)
    mobile = serializers.CharField(required=False, help_text="Mobile number")
    timing = serializers.JSONField(
        required=False, help_text='[{"day":1,"start_time":12:00:00,"is_active":1,"end_time":1:00:00}]')
    state = serializers.IntegerField(
        required=False, help_text="state")
    area = serializers.IntegerField(
        required=False, help_text="area")
    city = serializers.IntegerField(
        required=False, help_text="city")
    country = serializers.IntegerField(
        required=False, help_text="country")

    def validate_mobile(self, value):
        if len(value) < 10:
            raise serializers.ValidationError("Mobile number is Invalid")

    @transaction.atomic
    def create(self, validated_data):
        '''
        Advertiser
        '''

        validated_data['email'] = self.context['user'].email
        validated_data['user'] = self.context['user'].id
        serialized_data = AdvertiserSAVESerializer(instance=Advertiser.objects.get(
            user=self.context['user']), data=validated_data)
        if serialized_data.is_valid():
            advertiser_data = serialized_data.save()
        else:
            raise serializers.ValidationError(
                handle_errors(serialized_data.errors))

        '''
        Contact Info
        '''
        serialized_data = ContactInfoProfileSerializer(data=validated_data)
        if serialized_data.is_valid():
            if len(serialized_data.validated_data) > 0:
                contact_obj = Advertiser.objects.filter(
                    user=self.context['user'])
                contact_contentType = ContentType.objects.get_for_model(
                    Advertiser)
                # ContactInfo.objects.filter(content_type__pk=contact_contentType.id,object_id__in=contact_obj).delete()
                serialized_data.validated_data['content_object'] = advertiser_data
                contact_info_data = serialized_data.save()
        else:
            raise serializers.ValidationError(
                handle_errors(serialized_data.errors))

        '''
        Contact Numbers
        '''
        if "Telephone_number" in validated_data:
            contact_numbers = [TelephoneNumbers(contact_id=contact_info_data,
                                                contact_number=i
                                                ) for i in validated_data['Telephone_number']['mobile']]
            TelephoneNumbers.objects.bulk_create(contact_numbers)
            return 'success'

        '''
        Timing
        '''
        if "timing" in validated_data:
            timing = []
            for i in validated_data['timing']:
                timing.append(i)
                i['advertiser'] = advertiser_data.id
            timing_serializer = AdvertiserDailyAvailibilitySerializer(
                data=timing, many=True)
            if timing_serializer.is_valid():
                AdvertiserDailyTiming.objects.filter(
                    advertiser=advertiser_data.id).delete()
                timing_data = timing_serializer.save()
            else:
                print(timing_serializer.errors)
                raise serializers.ValidationError(
                    handle_errors(timing_serializer.errors))

        '''
        Contact Person
        '''

        if "contact_person" in validated_data:
            contact_persons = []
            Contact_Person.objects.filter(
                content_type__pk=contact_contentType.id, object_id__in=contact_obj).delete()

            for i in validated_data['contact_person']:
                if i['first_name'] == "" and i['last_name'] == "":
                    pass
                else:
                    contact_persons.append(i)

            serialized_data = ContactPersonSerializer(
                data=contact_persons, many=True)
            if serialized_data.is_valid():
                for i in serialized_data.validated_data:
                    i['content_object'] = advertiser_data
                    serialized_data.save()
            else:
                raise serializers.ValidationError(
                    handle_errors(serialized_data.errors))

        '''
        Bank Info
        '''
        if "bank" in validated_data:
            serialized_data = BankInfoProfileSerializer(data=validated_data)
            if serialized_data.is_valid():
                serialized_data.validated_data['content_object'] = advertiser_data
                serialized_data.save()
            else:
                raise serializers.ValidationError(
                    handle_errors(serialized_data.errors))
        return advertiser_data.id

    class Meta:
        model = Advertiser
        fields = '__all__'
        extra_kwargs = {
            'email': {'read_only': True},
            'user': {'read_only': True}
        }
