import pytz
import json
from django.http import HttpResponse
from django.template import Template, Context
from django.template.loader import get_template
from django.db import transaction
from django.urls import reverse
from django.contrib.contenttypes.models import ContentType
from django.contrib.auth import login as django_login
import datetime
from rest_framework.authtoken.models import Token
from rest_framework import status, generics
from rest_framework.response import Response
from rest_framework.authentication import TokenAuthentication
from rest_framework.permissions import IsAuthenticated
from utils.mixins import *
from utils.api_handle_errors import handle_errors
from campaigns.serializers import *
from advertisers.models import *
from advertisers.serializers import *
from commonAPI.serializers import ContactPersonSerializer
from commonAPI.models import Contact_Person


class AdvertiserProfileAPIView(PlatformAPICallMixin,generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = AdvertiserSerializers
    # dailySerializer_class  = AdvertiserDailyAvailibilitySerializer

    def get(self,request,format=None):
        response = {}
        responsedata = []
        data = Advertiser.objects.filter(user=request.user)
        serializers = self.serializer_class(data,many=True)

        content_type =  ContentType.objects.get_for_model(Advertiser)

        working_hours = AdvertiserDailyTiming.objects.filter(advertiser_id__in=data)
        working_hours_serializer = AdvertiserDailyAvailibilitySerializer(working_hours,many=True)

        content_type =  ContentType.objects.get_for_model(Advertiser)
        contact_person = Contact_Person.objects.filter(content_type__pk=content_type.id,object_id__in=data)
        contact_person_serializer = ContactPersonSerializer(contact_person,many=True)

        responsedata = [{"contact_person":contact_person_serializer.data},{"working_hours":working_hours_serializer.data}]
        responsedata = serializers.data + responsedata

        response["result"] = 1
        response['data'] = responsedata
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)



class AdvertiserPOSTProfileAPIView(PlatformAPICallMixin,generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = AdvertiserPOSTSerializers
    dailySerializer_class  = AdvertiserDailyAvailibilitySerializer
    

    def post(self,request,format=None):

        response = {}
        responsedata = []
        user = request.user
        serializers = self.serializer_class(data=request.data,context={"user":request.user})
        if serializers.is_valid():
            data = serializers.save()
            responsedata.append("Successfully Inserted")
            response["result"] = 1
            response['errors'] = responsedata
            response['ID'] = data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)
