from django.db import models

from django.core.validators import MaxValueValidator, MinValueValidator

from django.contrib.auth.models import User
from storages.backends.s3boto import S3BotoStorage
from django.core.files.images import get_image_dimensions
from django.core.exceptions import ValidationError
from static_data.models import *
from venue_owners.models import VenueOwner
from venues.models import VenueField
# Create your models here.


def validate_email_background(image):
    w, h = get_image_dimensions(image)
    print(w)
    if w != 600 and h != 800:
        raise ValidationError("Image is not of correct Dimensions")


def validate_banner_background(image):
    w, h = get_image_dimensions(image)
    if w != 500 and h != 250:
        raise ValidationError("Image is not of correct Dimensions")


def validate_logo(image):
    w, h = get_image_dimensions(image)
    if w != 400 and h != 400:
        raise ValidationError("Image is not of correct Dimensions")


class S3CustomStorage(S3BotoStorage):
    def _normalize_name(self, name):
        """
        S3 Storage function for storing files
        """
        return name


class CampaignPart1(models.Model):
    """
    Campaign Step 1 Data
    """
    STATUS_CHOICES = (
        ('A', 'Accepted'),
        ('R', 'Rejected'),
        ('O', 'On Hold')
    )
    PAYMENT_STATUS_CHOICES = (
        ('F', 'Failed'),
        ('S', 'Success'),
    )
    name = models.CharField(max_length=100, blank=False)
    email = models.BooleanField(default=False)
    sms = models.BooleanField(default=False)
    banner = models.BooleanField(default=False)
    paid_ads = models.BooleanField(default=False)
    #sport = models.ManyToManyField(Sport, blank=True, null=True)
    field = models.ForeignKey(
        VenueField, on_delete=models.CASCADE, blank=True, null=True)
    start_date = models.DateField(
        blank=True, null=True, help_text="Start Date")
    end_date = models.DateField(blank=True, null=True, help_text="End Date")
    status = models.CharField(
        max_length=10, default=STATUS_CHOICES[2][0], choices=STATUS_CHOICES, blank=True, null=True, help_text="Is document Verified")
    comment = models.TextField(blank=True, null=True)
    order_code = models.CharField(max_length=100, blank=True, null=True)
    payment_status = models.CharField(
        max_length=10, default=PAYMENT_STATUS_CHOICES[0][0], choices=PAYMENT_STATUS_CHOICES, blank=True, null=True, help_text="Payment Status")
    completed = models.BooleanField(default=False)
    cost = models.DecimalField(
        max_digits=19, decimal_places=2, blank=True, null=True, default=0.0)
    created_by = models.ForeignKey(
        User, on_delete=models.CASCADE, blank=True, null=True)
    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)
    is_deleted = models.BooleanField(default=False)

    def __str__(self):
        return "{}-{}-{}".format(self.name, self.created_by, self.start_date)

    class Meta:
        verbose_name_plural = "Campaign Part1"
        db_table = 'campaign_part1'


class CampaignPart2(models.Model):
    """
    Campaign Step 2 Data
    """

    campaign1 = models.ForeignKey(
        CampaignPart1, on_delete=models.CASCADE, blank=False, null=False)
    notification = models.CharField(
        max_length=4, null=True, help_text="A - All, V - venue owners, P - Players, F -Professional")
    user_notification = models.ManyToManyField(
        User, blank=True, related_name="Personalized_message")
    created_by = models.ForeignKey(
        User, on_delete=models.CASCADE, blank=True, null=True, related_name="Created_by")
    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)
    is_deleted = models.BooleanField(default=False)
    #latitude = models.FloatField(blank=True, null=True)
    #longitude = models.FloatField(blank=True, null=True)
    sports = models.ManyToManyField(Sport, blank=True)
    Areas = models.ManyToManyField(Area, blank=True)
    professional_service = models.ManyToManyField(
        Profession, blank=True)

    def __str__(self):
        return "{}-{}-{}".format(self.campaign1, self.notification, self.created_by)

    class Meta:
        verbose_name_plural = "Campaign Part2"
        db_table = 'campaign_part2'

class CampaignBannerQuoteRequest(models.Model):
    """
    Campagin Banner Reequest a quote
    """
    campaign1 = models.ForeignKey(
        CampaignPart1, on_delete=models.CASCADE, blank=False, null=False)
    start_date = models.DateField( blank=True, null=True, help_text="Start Date")
    end_date = models.DateField( blank=True, null=True, help_text="End Date")
    created_by = models.ForeignKey(
        User, on_delete=models.CASCADE, blank=True, null=True)
    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)
    is_deleted = models.BooleanField(default=False)


    def __str__(self):
        return "{}-{}-{}".format(self.campaign1, self.start_date, self.end_date)

    class Meta:
        verbose_name_plural = "Campaign Banner Requests"
        db_table = 'campaign_banner_request'

class CampaignPart3(models.Model):
    """
    Campaign step 3 Data
    """
    campaign1 = models.ForeignKey(
        CampaignPart1, on_delete=models.CASCADE, blank=False, null=False)
    emailBody = models.TextField(max_length=600, null=False, blank=True)
    email_logo = models.ImageField(
        upload_to="Campaign", blank=True, null=True, validators=[validate_logo])
    email_background = models.ImageField(
        upload_to="Campaign", blank=True, null=True, validators=[validate_email_background])
    email_url = models.TextField(
        max_length=800, null=True, blank=True, help_text="3rd party link")
    sms_text = models.TextField(max_length=80, null=False, blank=True)
    sms_url = models.TextField(
        max_length=800, null=True, blank=True, help_text="3rd party link")
    banner_text = models.TextField(max_length=30, null=False, blank=True)
    banner_background = models.ImageField(
        upload_to="Campaign", blank=True, null=True, validators=[validate_banner_background])
    banner_logo = models.ImageField(
        upload_to="Campaign", blank=True, null=True, validators=[validate_logo])
    banner_url = models.TextField(
        max_length=800, null=True, blank=True, help_text="3rd party link")
    subject_line = models.TextField(max_length=600, null=False, blank=True)
    created_by = models.ForeignKey(
        User, on_delete=models.CASCADE, blank=True, null=True)
    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)
    is_deleted = models.BooleanField(default=False)

    def __str__(self):
        return "{}-{}-{}".format(self.campaign1, self.emailBody, self.emailBody)

    class Meta:
        verbose_name_plural = "Campaign Part3"
        db_table = 'campaign_part3'


class ViewsPackage(models.Model):

    cost = models.IntegerField(blank=True, null=True, help_text="cost")
    start_view = models.IntegerField(blank=True, null=True)
    end_view = models.IntegerField(blank=True, null=True)
    is_deleted = models.BooleanField(
        default=False)

    def __str__(self):
        return "{}-{}-{}-{}".format(self.cost, self.start_view, self.end_view, self.is_deleted)

    class Meta:
        verbose_name_plural = "Views Package"
        db_table = "views_package"


class ClicksPackage(models.Model):

    cost = models.IntegerField(blank=True, null=True, help_text="cost")
    start_click = models.IntegerField(blank=True, null=True)
    end_click = models.IntegerField(blank=True, null=True)
    is_deleted = models.BooleanField(
        default=False)

    def __str__(self):
        return "{}-{}-{}-{}".format(self.cost, self.start_click, self.end_click, self.is_deleted)

    class Meta:
        verbose_name_plural = "Clicks Package"
        db_table = "clicks_package"


class PaidAdsViewsPackages(models.Model):
    """
    PaidAdsViews Packages
    """
    code = models.CharField(max_length=5, blank=True,
                             null=True, help_text="Code")
    description = models.CharField(
        max_length=100, blank=True, null=True, help_text="Description")
    view_qty_from = models.IntegerField(blank=True, null=True)
    view_qty_to = models.IntegerField(blank=True, null=True)
    measurement_units = models.ForeignKey(
        MeasurementUnits, on_delete=models.CASCADE, blank=True, null=True, help_text="measurement units id")
    price = models.DecimalField(
        max_digits=19, decimal_places=2, blank=True, null=True, default=0.0)
    vat = models.ForeignKey(
        Vat, on_delete=models.CASCADE, blank=True, null=True, help_text="vat id")
    low_limit_notification = models.IntegerField(blank=True, null=True)
    is_deleted = models.BooleanField(
        default=False, blank=True, null=True, help_text="If the record is deleted")
    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)

    def __str__(self):
        return "{}-{}".format(self.pk, self.code)

    class Meta:
        verbose_name_plural = 'PAIDADS Views Packages'
        db_table = 'paid_ads_views_package'


class CampaignPart4(models.Model):
    """
    Campaign Step 4 Data
    """
    CAMPAIGN_PURPOSE = (
        ('INFORMATIVE', 'INFORMATIVE'),
        ('DISCOUNT', 'DISCOUNT')
    )
    campaign1 = models.ForeignKey(
        CampaignPart1, on_delete=models.CASCADE, blank=False, null=False)
    campaign_purpose = models.CharField(choices=CAMPAIGN_PURPOSE, blank=True, null=True, max_length=100, help_text="Campaign Purpose")
    views = models.ForeignKey(
        PaidAdsViewsPackages, on_delete=models.CASCADE, blank=True, null=True)
    clicks = models.ForeignKey(
        ClicksPackage, on_delete=models.CASCADE, blank=True, null=True)
    discount = models.IntegerField(null=True, help_text="discount percentage")
    discount_applied_field = models.ForeignKey(VenueField, on_delete=models.SET_NULL, blank=True, null=True)
    discount_code = models.CharField(
        max_length=100, blank=True, null=True, help_text="Discount coupon code")
    created_by = models.ForeignKey(
        User, on_delete=models.CASCADE, blank=True, null=True)
    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)
    is_deleted = models.BooleanField(default=False)

    def __str__(self):
        return "{}-{}-{}".format(self.campaign1, self.discount, self.created_by)

    class Meta:
        verbose_name_plural = "Campaign Part4"
        db_table = 'campaign_part4'


class CampaignNotificationTime(models.Model):
    campaign1 = models.ForeignKey(
        CampaignPart1, on_delete=models.CASCADE, blank=False, null=False, help_text="Campaign 1 ID")
    date = models.DateField(blank=True, null=True, help_text="Date YYYY-MM-DD")
    time = models.TimeField(blank=True, null=True, help_text="Time HH:MM")

    def __str__(self):
        return "{}-{}-{}".format(self.campaign1, self.date, self.time)

    class Meta:
        verbose_name_plural = "Campaign Notifications"
        db_table = "campaign_notification"


class UniqueCode(models.Model):
    code = models.CharField(max_length=6, blank=True,
                            null=True, unique=True, help_text="Code")
    is_deleted = models.BooleanField(
        default=False, blank=True, null=True, help_text="If the record is deleted")
    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)

    def __str__(self):
        return "{}-{}".format(self.pk, self.code)

    class Meta:
        verbose_name_plural = 'Unique Codes'
        db_table = 'unique_code'

class SmsSetupFee(models.Model):
    """
    Sms Setup Fee
    """
    code = models.CharField(max_length=5, blank=True,
                            null=True, help_text="Code")
    description = models.CharField(
        max_length=100, blank=True, null=True, help_text="Description")
    measurement_units = models.ForeignKey(
        MeasurementUnits, on_delete=models.CASCADE, blank=True, null=True)
    price = models.DecimalField(
        max_digits=19, decimal_places=2, blank=True, null=True, default=0.0)
    vat = models.ForeignKey(
        Vat, on_delete=models.CASCADE, blank=True, null=True)
    is_deleted = models.BooleanField(
        default=False, blank=True, null=True, help_text="If the record is deleted")
    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)

    def __str__(self):
        return "{}-{}".format(self.pk, self.code)

    class Meta:
        verbose_name_plural = 'SMS Setup Fee'
        db_table = 'sms_setup_fee'


class SmsPackages(models.Model):
    """
    Sms Packages
    """
    code = models.ForeignKey(UniqueCode, on_delete=models.CASCADE, blank=True,
                            null=True, help_text="Code")
    description = models.CharField(
        max_length=100, blank=True, null=True, help_text="Description")
    sms_qty_from = models.IntegerField(blank=True, null=True)
    sms_qty_to = models.IntegerField(blank=True, null=True)
    measurement_units = models.ForeignKey(
        MeasurementUnits, on_delete=models.CASCADE, blank=True, null=True, help_text="measurement units id")
    price = models.DecimalField(
        max_digits=19, decimal_places=2, blank=True, null=True, default=0.0)
    vat = models.ForeignKey(
        Vat, on_delete=models.CASCADE, blank=True, null=True, help_text="vat id")
    is_deleted = models.BooleanField(
        default=False, blank=True, null=True, help_text="If the record is deleted")
    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)

    def __str__(self):
        return "{}-{}".format(self.pk, self.code)

    class Meta:
        verbose_name_plural = 'SMS Packages'
        db_table = 'sms_package'


class EmailSetupFee(models.Model):
    """
    Email Setup Fee
    """
    code = models.CharField(max_length=5, blank=True,
                            null=True, help_text="Code")
    description = models.CharField(
        max_length=100, blank=True, null=True, help_text="Description")
    measurement_units = models.ForeignKey(
        MeasurementUnits, on_delete=models.CASCADE, blank=True, null=True)
    price = models.DecimalField(
        max_digits=19, decimal_places=2, blank=True, null=True, default=0.0)
    vat = models.ForeignKey(
        Vat, on_delete=models.CASCADE, blank=True, null=True)
    is_deleted = models.BooleanField(
        default=False, blank=True, null=True, help_text="If the record is deleted")
    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)

    def __str__(self):
        return "{}-{}".format(self.pk, self.code)

    class Meta:
        verbose_name_plural = 'EMAIL Setup Fee'
        db_table = 'email_setup_fee'


class EmailPackages(models.Model):
    """
    Email Packages
    """
    code = models.ForeignKey(UniqueCode, on_delete=models.CASCADE, blank=True,
                            null=True, help_text="Code")
    description = models.CharField(
        max_length=100, blank=True, null=True, help_text="Description")
    email_qty_from = models.IntegerField(blank=True, null=True)
    email_qty_to = models.IntegerField(blank=True, null=True)
    measurement_units = models.ForeignKey(
        MeasurementUnits, on_delete=models.CASCADE, blank=True, null=True, help_text="measurement units id")
    price = models.DecimalField(
        max_digits=19, decimal_places=2, blank=True, null=True, default=0.0)
    vat = models.ForeignKey(
        Vat, on_delete=models.CASCADE, blank=True, null=True, help_text="vat id")
    is_deleted = models.BooleanField(
        default=False, blank=True, null=True, help_text="If the record is deleted")
    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)

    def __str__(self):
        return "{}-{}".format(self.pk, self.code)

    class Meta:
        verbose_name_plural = 'EMAIL Packages'
        db_table = 'email_package'


class BannerSetupFee(models.Model):
    """
    Banner Setup Fee
    """
    code = models.ForeignKey(UniqueCode, on_delete=models.CASCADE, blank=True,
                            null=True, help_text="Code")
    description = models.CharField(
        max_length=100, blank=True, null=True, help_text="Description")
    measurement_units = models.ForeignKey(
        MeasurementUnits, on_delete=models.CASCADE, blank=True, null=True)
    price = models.DecimalField(
        max_digits=19, decimal_places=2, blank=True, null=True, default=0.0)
    vat = models.ForeignKey(
        Vat, on_delete=models.CASCADE, blank=True, null=True)
    is_deleted = models.BooleanField(
        default=False, blank=True, null=True, help_text="If the record is deleted")
    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)

    def __str__(self):
        return "{}-{}".format(self.pk, self.code)

    class Meta:
        verbose_name_plural = 'BANNER Setup Fee'
        db_table = 'banner_setup_fee'


class PaidAdsSetupFee(models.Model):
    """
    PaidAds Setup Fee
    """
    code = models.ForeignKey(UniqueCode, on_delete=models.CASCADE, blank=True,
                            null=True, help_text="Code")
    description = models.CharField(
        max_length=100, blank=True, null=True, help_text="Description")
    measurement_units = models.ForeignKey(
        MeasurementUnits, on_delete=models.CASCADE, blank=True, null=True)
    price = models.DecimalField(
        max_digits=19, decimal_places=2, blank=True, null=True, default=0.0)
    vat = models.ForeignKey(
        Vat, on_delete=models.CASCADE, blank=True, null=True)
    is_deleted = models.BooleanField(
        default=False, blank=True, null=True, help_text="If the record is deleted")
    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)

    def __str__(self):
        return "{}-{}".format(self.pk, self.code)

    class Meta:
        verbose_name_plural = 'PAIDADS Setup Fee'
        db_table = 'paid_ads_setup_fee'
