from rest_framework import serializers
from django.contrib.auth.models import User, Group
from rest_framework.authtoken.models import Token

from datetime import date
from django.utils import timezone
from .models import *
from players.models import Player
from static_data.models import *
from venues.models import VenueField
from professionals.models import Professional
from venue_owners.models import VenueOwner
from advertisers.models import Advertiser
from static_data.serializers import *


class ChargeTokenSerializer(serializers.Serializer):
    amount = serializers.IntegerField(required=False)
    access_token = serializers.CharField(read_only=False)
    charge_token = serializers.CharField(read_only=False)


class CampaignPart1SerializerGET(serializers.ModelSerializer):
    created_by = serializers.CharField(
        source='created_by.username', read_only=True)
    id = serializers.IntegerField(required=False)
    name = serializers.CharField(required=False)

    def validate_id(self, value):
        if id:
            if CampaignPart1.objects.filter(id=value).exists():
                return value
            else:
                return serializers.ValidationError("No Such Form")

    class Meta:
        model = CampaignPart1
        fields = '__all__'


class CampaignPart1POSTSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(required=False)

    def create(self, validated_data):
        if validated_data['email'] == True or validated_data['sms'] == True:
            validated_data['status'] = 'A'
        else:
            validated_data['status'] = 'O'

        validated_data['sport'] = []
        data = CampaignPart1.objects.create(name=validated_data['name'], email=validated_data['email'], sms=validated_data['sms'], banner=validated_data['banner'], paid_ads=validated_data['paid_ads'], comment=validated_data.get(
            'comment', None), status=validated_data['status'], created_by=validated_data['created_by'])
        # data.sport.set(validated_data['sport'])
        # data.save()
        return data

    class Meta:
        model = CampaignPart1
        fields = '__all__'
        extra_kwargs = {
            "name": {
                "required": False
            }
        }


class CampaignPart2POSTSerializer(serializers.ModelSerializer):

    campaign1 = CampaignPart1SerializerGET(read_only=True)
    part1ID = serializers.IntegerField(write_only=True)
    id = serializers.IntegerField(required=False)
    user_notification = serializers.JSONField(
        write_only=True, required=False, help_text="[1,2,3,4]")

    def create(self, validated_data):
        professional_service = False
        sports = False
        areas = False
        print(validated_data)
        if 'sports' in validated_data:
            sports = validated_data.pop('sports')
        if 'Areas' in validated_data:
            areas = validated_data.pop('Areas')
        if 'professional_service' in validated_data:
            professional_service = validated_data.pop('professional_service')
        validated_data['campaign1'] = CampaignPart1.objects.get(
            id=validated_data['part1ID'])
        validated_data.pop('part1ID')

        if 'user_notification' in validated_data:
            tp = []
            for i in validated_data['user_notification']:
                player_obj = Player.objects.filter(id=int(i))
                if player_obj.exists():
                    tp.append(player_obj.last().user.id)
            #validated_data['user_notification'] = player_obj.last().user
                else:
                    raise serializers.ValidationError("No Such User ")
            validated_data.pop('user_notification')
            data = CampaignPart2.objects.create(**validated_data)
            data.user_notification.set(tp)
            data.save()
        else:
            data = CampaignPart2.objects.create(**validated_data)
        
        if sports:
            data.sports.add(*sports)
        if professional_service:
            data.professional_service.add(*professional_service)
        if areas:
            data.Areas.add(*areas)

        data.save()
        return data

        # data = CampaignPart2.objects.create(**validated_data)
        # print(data)
        # data.professional_service.set(prof_serv)
        # return data

    class Meta:
        model = CampaignPart2
        fields = '__all__'


class CampaignPart2SerializerGET(serializers.ModelSerializer):
    created_by = serializers.CharField(
        source='created_by.username', read_only=True)
    id = serializers.IntegerField(required=False)

    def validate_id(self, value):
        if id:
            if CampaignPart2.objects.filter(id=value).exists():
                return value
            else:
                return serializers.ValidationError("No Such Form")

    def __init__(self, *args, **kwargs):
        super(CampaignPart2SerializerGET, self).__init__(*args, **kwargs)
        for field in self.fields:
            if field == 'id':
                self.fields[field].read_only = False
            elif field == 'campaign1':
                self.fields[field].read_only = False
            else:
                self.fields[field].read_only = True

    class Meta:
        model = CampaignPart2
        fields = '__all__'


class GetPlayerListSerializer(serializers.ModelSerializer):
    city = serializers.CharField(
        required=False, help_text="city", write_only=True)
    street = serializers.CharField(
        required=False, help_text="city", write_only=True)

    class Meta:
        model = Player
        fields = ['id', 'first_name', 'last_name', 'city', 'street']


class CampaignPart3GETEmailSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(required=False)

    def validate_id(self, value):
        if value:
            if CampaignPart3.objects.filter(id=value).exists():
                return value
            else:
                serializers.ValidationError("No Record Found")

    def __init__(self, *args, **kwargs):
        super(CampaignPart3GETEmailSerializer, self).__init__(*args, **kwargs)
        for field in self.fields:
            if field == 'id':
                self.fields[field].read_only = False
            elif field == 'campaign1':
                self.fields[field].read_only = False
                self.fields[field].required = False
            else:
                self.fields[field].read_only = True

    class Meta:
        model = CampaignPart3
        fields = ('emailBody', 'id', 'email_logo', 'campaign1',
                  'email_background', 'created_by', 'created_at')


class CampaignPart3POSTEmailSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(required=False)
    part1ID = serializers.IntegerField(required=False, write_only=True)
    copy = serializers.IntegerField(required=False, write_only=True)

    def create(self, validated_data):
        validated_data['campaign1'] = CampaignPart1.objects.get(
            id=validated_data['part1ID'])
        validated_data.pop('part1ID')

        if "copy" in validated_data:
            camp3obj = CampaignPart3.objects.get(
                campaign1__id=validated_data["copy"])
            validated_data["email_logo"] = camp3obj.email_logo
            validated_data["email_background"] = camp3obj.email_background
            validated_data.pop('copy')
        data = CampaignPart3.objects.create(**validated_data)
        return data

    class Meta:
        model = CampaignPart3
        fields = ('emailBody', 'id', 'email_logo', 'part1ID',
                  'email_background', 'email_url', 'created_by', 'created_at', 'copy', 'subject_line')


class CampaignPart3POSTSmsSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(required=False)
    part1ID = serializers.IntegerField(required=False, write_only=True)

    def create(self, validated_data):
        validated_data['campaign1'] = CampaignPart1.objects.get(
            id=validated_data['part1ID'])
        validated_data.pop('part1ID')
        data = CampaignPart3.objects.create(**validated_data)
        return data

    class Meta:
        model = CampaignPart3
        fields = ('sms_text', 'id', 'part1ID',
                  'sms_url', 'created_by', 'created_at')


class CampaignPart3GETSmsSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(required=False)

    def validate_id(self, value):
        if value:
            if CampaignPart3.objects.filter(id=value).exists():
                return value
            else:
                serializers.ValidationError("No Record Found")

    def __init__(self, *args, **kwargs):
        super(CampaignPart3GETSmsSerializer, self).__init__(*args, **kwargs)
        for field in self.fields:
            if field == 'id':
                self.fields[field].read_only = False
            elif field == 'campaign1':
                self.fields[field].read_only = False
                self.fields[field].required = False
            else:
                self.fields[field].read_only = True

    class Meta:
        model = CampaignPart3
        fields = ('sms_text', 'id', 'campaign1', 'created_by', 'created_at')


class CampaignPart3BannerGETSerializer(serializers.ModelSerializer):

    id = serializers.IntegerField(required=False)

    def validate_id(self, value):
        if value:
            if CampaignPart3.objects.filter(id=value).exists():
                return value
            else:
                serializers.ValidationError("No Record Found")

    def __init__(self, *args, **kwargs):
        super(CampaignPart3BannerGETSerializer, self).__init__(*args, **kwargs)
        for field in self.fields:
            if field == 'id':
                self.fields[field].read_only = False
            elif field == 'campaign1':
                self.fields[field].read_only = False
                self.fields[field].required = False
            else:
                self.fields[field].read_only = True

    class Meta:
        model = CampaignPart3
        fields = ('banner_text', 'id', 'campaign1', 'banner_logo',
                  'banner_background', 'created_by', 'created_at')


class CampaignPart3BannerPOSTSerializer(serializers.ModelSerializer):

    id = serializers.IntegerField(required=False)
    part1ID = serializers.IntegerField(required=False, write_only=True)

    def create(self, validated_data):
        validated_data['campaign1'] = CampaignPart1.objects.get(
            id=validated_data['part1ID'])
        validated_data.pop('part1ID')
        data = CampaignPart3.objects.create(**validated_data)
        return data

    class Meta:
        model = CampaignPart3
        fields = ('banner_text', 'id', 'part1ID', 'banner_url',
                  'banner_logo', 'banner_background', 'created_by', 'created_at')


class CampaignPart4POSTSerializer(serializers.ModelSerializer):
    #campaign1 = CampaignPart1SerializerGET(read_only=True)
    id = serializers.IntegerField(required=False)
    part1ID = serializers.IntegerField(required=False, write_only=True)
    notification_time = serializers.JSONField(
        required=False, write_only=True, help_text='[{"date":YYYY-MM-DD, "time":HH:MM},{"date":YYYY-MM-DD, "time":HH:MM}]')
    start_date = serializers.DateField(
        required=False, write_only=True, help_text="Date range start date")
    end_date = serializers.DateField(
        required=False, write_only=True, help_text="Date range end date")
    fields = serializers.PrimaryKeyRelatedField(
        queryset=VenueField.objects.all(), required=False, help_text="VenueField ID")

    def create(self, validated_data):
        campaign1_object = CampaignPart1.objects.get(
            id=validated_data['part1ID'])

        if "fields" in validated_data:
            campaign1_object.field = validated_data.get('fields', None)
            campaign1_object.save()
            validated_data.pop("fields")

        if "start_date" in validated_data:
            campaign1_object.start_date = validated_data['start_date']
            campaign1_object.save()
            validated_data.pop("start_date")

        if "end_date" in validated_data:
            campaign1_object.end_date = validated_data['end_date']
            campaign1_object.save()
            validated_data.pop("end_date")

        if "notification_time" in validated_data:
            notification_time = validated_data['notification_time']
            notification_obj = [CampaignNotificationTime(
                campaign1=campaign1_object,
                date=i['date'],
                time=i['time']
            ) for i in notification_time]
            validated_data.pop("notification_time")
            CampaignNotificationTime.objects.bulk_create(notification_obj)
        validated_data['campaign1'] = CampaignPart1.objects.get(
            id=validated_data['part1ID'])
        validated_data.pop('part1ID')
        data = CampaignPart4.objects.create(**validated_data)

        return data

    class Meta:
        model = CampaignPart4
        fields = '__all__'
        read_only_fields = ['campaign1']


class CampaignPart4GETSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(required=False)

    def __init__(self, *args, **kwargs):
        super(CampaignPart4GETSerializer, self).__init__(*args, **kwargs)
        for field in self.fields:
            if field == 'id':
                self.fields[field].read_only = False
            elif field == 'campaign1':
                self.fields[field].read_only = False
                self.fields[field].required = False
            else:
                self.fields[field].read_only = True

    class Meta:
        model = CampaignPart4
        fields = '__all__'


class globalSerializer(serializers.Serializer):
    id = serializers.IntegerField(required=False)

    def validate_id(self, value):
        if CampaignPart1.objects.filter(id=value).exists():
            return value
        else:
            raise serializers.ValidationError("No Such Record")


class Campaign3Get(serializers.ModelSerializer):
    created_by = serializers.CharField(source="created_by.username")

    class Meta:
        model = CampaignPart3
        fields = '__all__'


class CampaignSummarySerializer(serializers.Serializer):
    id = serializers.PrimaryKeyRelatedField(
        queryset=CampaignPart1.objects.all(), help_text="Campaign Part 1")


class CampaignNotificationSummarySerializer(serializers.ModelSerializer):

    class Meta:
        model = CampaignNotificationTime
        fields = ['date', 'time']
        read_only_fields = ['campaign1']


class CampaignPart2RecipientSerializer(serializers.ModelSerializer):

    class Meta:
        model = CampaignPart2
        fields = ['notification']
        read_only_fields = ['campaign1', 'user_notification', 'created_by',
                            'created_at', 'is_deleted', 'professional_service', 'sports', 'Areas']


class CampaignPart3SmsANDEmailSerializer(serializers.ModelSerializer):

    class Meta:
        model = CampaignPart3
        fields = ['emailBody', 'email_logo',
                  'email_background', 'email_url', 'sms_text', 'sms_url', 'created_by', 'created_at']


class CampaignPaidAdsViewsPackagesEDITSerializer(serializers.ModelSerializer):
    measurement_units = MeasurementUnitsWithoutPageSerializer(
        required=False, read_only=True)
    vat = VatWithoutPageSerializer(required=False, read_only=True)

    class Meta:
        model = PaidAdsViewsPackages
        fields = '__all__'


class CampaignPart4DiscountSerializer(serializers.ModelSerializer):
    views = CampaignPaidAdsViewsPackagesEDITSerializer(
        required=False, read_only=True)

    class Meta:
        model = CampaignPart4
        fields = ['discount', 'views', 'clicks']


class GetCityAndCountrySerializer(serializers.ModelSerializer):
    city = serializers.JSONField(required=False, help_text="city")
    #country = serializers.JSONField(required=False, help_text="country")

    class Meta:
        model = User
        fields = ['id', 'first_name', 'last_name', 'city']
        read_only_fields = ['id', 'first_name', 'last_name']


class AllUserCitySerializer(serializers.ModelSerializer):

    class Meta:
        model = User
        fields = '__all__'


class PlayerCityAndCountrySerializer(serializers.ModelSerializer):

    class Meta:
        model = Player
        fields = ['city', 'country']


class VenueOwnerCityAndCountrySerializer(serializers.ModelSerializer):

    class Meta:
        model = VenueOwner
        fields = ['city', 'country']


class ProfessionalCityAndCountrySerializer(serializers.ModelSerializer):

    class Meta:
        model = Professional
        fields = ['city', 'country']


class AdvertiserCityAndCountrySerializer(serializers.ModelSerializer):

    class Meta:
        model = Advertiser
        fields = ['city', 'country']


class ViewPackageSerializer(serializers.ModelSerializer):

    class Meta:
        model = ViewsPackage
        fields = '__all__'


class ClickPackageSerializer(serializers.ModelSerializer):

    class Meta:
        model = ClicksPackage
        fields = '__all__'

################ LIST VIEW #######################


class CampaignLISTSerializer(serializers.ModelSerializer):
    campaign_kind = serializers.CharField(
        required=False, help_text="email / sms / banner/ paid_ads")
    keyword = serializers.CharField(
        required=False, help_text="Search Texr box")
    page = serializers.IntegerField(
        required=False, default=1, help_text="Page number. Default = 1")
    # sport = serializers.CharField(required=False, write_only=True)
    # sport_name = serializers.CharField(
    #     required=False, read_only=True, source="sport.name")
    assortment = serializers.CharField(
        required=False, write_only=True, help_text="Assortment start_date / end_date")
    status = serializers.CharField(
        required=False, write_only=True, help_text="UNDER APPROVAL/ON GOING/WAITING/REJECTED")
    status_to_send = serializers.SerializerMethodField('status_decider')
    field = serializers.CharField(required=False, write_only=True)
    field_name = serializers.CharField(
        required=False, read_only=True, source="field.name")
    campaign_type_string = serializers.SerializerMethodField('_get_campaign_type_string')

    def status_decider(self, obj):
        d = CampaignPart1.objects.get(id=obj.id)
        if d.status == "O":
            return "UNDER APPROVAL"
        elif d.status == "R":
            return "REJECTED"
        elif d.status == "A":
            if d.start_date is None:
                return "WAITING"
            else:
                print(timezone.now().date())
                print(d.start_date)
                if d.start_date >= timezone.now().date():
                    return "WAITING"
                else:
                    return "ON GOING"

    def _get_campaign_type_string(self, obj):
        return_str = list()
        if obj.email == True:
            return_str.append("EMAIL")
        if obj.sms == True:
            return_str.append("SMS")
        if obj.banner == True:
            return_str.append("BANNER")
        if obj.paid_ads == True:
            return_str.append("PAID ADS")
        if len(return_str) > 0:
            return ",".join(return_str)
        return ""

    class Meta:
        model = CampaignPart1
        fields = '__all__'
        read_only_fields = ['name', 'email', 'sms',
                            'banner', 'paid_ads']


class CampaignDELETESerializer(serializers.Serializer):
    id = serializers.CharField(
        required=True, help_text="ID of campaign step 1")

    def validate_id(self, value):
        if CampaignPart1.objects.filter(id=value, is_deleted=False).exists():
            return CampaignPart1.objects.filter(id=value, is_deleted=False).first()
        else:
            serializers.ValidationError("No such record")


class VenueFieldsSerializer(serializers.ModelSerializer):
    class Meta:
        model = VenueField
        fields = ['id', 'name']


class PlayerAreaSerializer(serializers.Serializer):
    area_list = serializers.JSONField(required=False, help_text="area")
    sports_list = serializers.JSONField(required=False, help_text="[1,2,3]")

class CampaignBannerQuoteRequestSerializer(serializers.ModelSerializer):
    class Meta:
        model = CampaignBannerQuoteRequest
        fields = '__all__'


class CampaignPart2POSTSerializerForAdvertiser(serializers.ModelSerializer):

    campaign1 = CampaignPart1SerializerGET(read_only=True)
    part1ID = serializers.IntegerField(write_only=True)
    id = serializers.IntegerField(required=False)
    sports_list = serializers.JSONField(
        write_only=True, required=False, help_text="[1,2,3,4]")
    area_list = serializers.JSONField(
        write_only=True, required=False, help_text="['area1','area2']")
    professions_list = serializers.JSONField(
        write_only=True, required=False, help_text="[1,2,3,4]")

    def create(self, validated_data):
        print(validated_data)
        area_id_list = []
        sport_list = []
        plist = []
        validated_data.pop('user_notification')
        if 'area_list' in validated_data:
            for a in validated_data['area_list']:
                area_obj = Area.objects.get(name=a)
                area_id_list.append(area_obj.id)
            validated_data.pop('area_list')
            validated_data.pop('Areas')
        else:
            validated_data.pop('Areas')

        if 'sports_list' in validated_data:
            for s in validated_data['sports_list']:
                sport_obj = Sport.objects.get(id=s)
                sport_list.append(sport_obj.id)
            validated_data.pop('sports')
            validated_data.pop('sports_list')
        else:
            validated_data.pop('sports')

        if 'professions_list' in validated_data:
            for p in validated_data['professions_list']:
                pobj = Profession.objects.get(id=p)
                plist.append(pobj.id)
            validated_data.pop('professional_service')
            validated_data.pop('professions_list')
        else:
            validated_data.pop('professional_service')

        validated_data['campaign1'] = CampaignPart1.objects.get(
            id=validated_data['part1ID'])
        validated_data.pop('part1ID')
        print(validated_data)

        data = CampaignPart2.objects.create(**validated_data)
        if len(area_id_list) > 0:
            data.Areas.set(area_id_list)
        if len(sport_list) > 0:
            data.sports.set(sport_list)
        if len(plist) > 0:
            data.professional_service.set(plist)
        return data

    class Meta:
        model = CampaignPart2
        fields = '__all__'


class CreateOrderSerializer(serializers.Serializer):
    camp_id = serializers.IntegerField(required=True)


class TransactionSerializer(serializers.Serializer):
    orderCode = serializers.CharField(required=True)
    transactionId = serializers.CharField(required=True)


class CampaignPart4PaidAdsViewsPackagesSerializer(serializers.ModelSerializer):
    measurement_units = MeasurementUnitsWithoutPageSerializer(
        required=False, read_only=True)
    vat = VatWithoutPageSerializer(required=False, read_only=True)

    class Meta:
        model = PaidAdsViewsPackages
        fields = '__all__'
        read_only_fields = ['code', 'description',
                            'view_qty_from', 'view_qty_to', 'price', 'is_deleted', 'low_limit_notification']
