import pytz
import json
from django.http import HttpResponse
from django.template import Template, Context
from django.template.loader import get_template
from django.db import transaction
from django.urls import reverse

from django.contrib.auth import login as django_login
import datetime
from rest_framework.authtoken.models import Token
from rest_framework import status, generics
from rest_framework.response import Response
from rest_framework.authentication import TokenAuthentication
from rest_framework.permissions import IsAuthenticated
from utils.mixins import *
from utils.api_handle_errors import handle_errors
from campaigns.serializers import *


class CampaignPart1GETAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    '''
    GET Campaign1 value
    '''
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = CampaignPart1SerializerGET

    def post(self, request, format=None):
        '''
        Campaign Part1 Profile GET
        '''
        response = {}
        responsedata = []
        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            if 'id' in serializers.validated_data:
                OwnerInfo = CampaignPart1.objects.filter(
                    id=serializers.validated_data['id'], is_deleted=False)
            else:
                OwnerInfo = CampaignPart1.objects.filter(
                    created_by=request.user, is_deleted=False)
        else:
            response['result'] = 0
            response = handle_errors(serializers.errors)
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)
        serializer = self.serializer_class(OwnerInfo, many=True)
        response['result'] = 1
        response['data'] = serializer.data
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


class CampaignPart1POSTAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = CampaignPart1POSTSerializer
    # Get Campaign Part1

    # Add Campaign Part1
    def post(self, request, format=None):
        '''
        Campaign Part1 Profile Update/Create
        '''

        response = {}
        responsedata = []
        request.data._mutable = True
        request.data['created_by'] = User.objects.get(
            username=str(request.user)).id
        if 'id' in request.data:
            if CampaignPart1.objects.filter(id=request.data['id'], is_deleted=False).exists():
                serializers = self.serializer_class(instance=CampaignPart1.objects.filter(
                    id=request.data['id'], is_deleted=False).first(), data=request.data)
                message = "Successfully Updated"
            else:
                responsedata.append("No Such Record to update")
                response['result'] = 0
                response['errors'] = responsedata
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)
        else:
            serializers = self.serializer_class(data=request.data)
            message = 'Successfully Inserted'

        if serializers.is_valid():
            if 'id' in request.data:
                # Calculate number of days between Start date and end date
                campaign_part1_obj = CampaignPart1.objects.filter(
                    id=request.data['id'], is_deleted=False).first()
                difference_days = campaign_part1_obj.end_date - campaign_part1_obj.start_date

                serializers.validated_data['completed'] = True
                # Get New End Date
                serializers.validated_data['end_date'] = serializers.validated_data['start_date'] + \
                    datetime.timedelta(days=difference_days.days)

            # check for date
            # if serializers.validated_data['start_date'] > serializers.validated_data['end_date']:
            #     responsedata.append("Incorrect Dates")
            #     response['result'] = 0
            #     response['errors'] = responsedata
            #     self.add_api_call(request, response)
            #     return Response(response, status=status.HTTP_200_OK)

            save_data = serializers.save()
            response["result"] = 1
            responsedata.append(message)
            response['errors'] = responsedata
            response['id'] = save_data.id
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializers.errors)
            response['result'] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)
