import pytz
import json
from django.http import HttpResponse
from django.template import Template, Context
from django.template.loader import get_template
from django.db import transaction
from django.urls import reverse

from django.contrib.auth import login as django_login
import datetime
from rest_framework.authtoken.models import Token
from rest_framework import status, generics
from rest_framework.response import Response
from rest_framework.authentication import TokenAuthentication
from rest_framework.permissions import IsAuthenticated
from utils.mixins import *
from utils.api_handle_errors import handle_errors
from campaigns.serializers import *
from players.models import Player


class CampaignPart2GETAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = CampaignPart2SerializerGET
    # Get Campaign Part1

    def post(self, request, format=None):
        '''
        Campaign Part2 Profile GET
        '''

        response = {}
        responsedata = []
        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            if 'id' in serializers.validated_data:
                data = CampaignPart2.objects.filter(
                    id=serializers.validated_data['id'], is_deleted=False)
            elif 'campaign1' in serializers.validated_data:
                data = CampaignPart2.objects.filter(
                    campaign1=serializers.validated_data['campaign1'], is_deleted=False)

            else:
                data = CampaignPart2.objects.filter(
                    created_by=request.user, is_deleted=False)
        else:
            response['result'] = 0
            response = handle_errors(serializers.errors)
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)
        serializer = self.serializer_class(data, many=True)
        response['result'] = 1
        response['data'] = serializer.data
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


class CampaignPart2POSTAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = CampaignPart2POSTSerializer

    # Add Campaign Part2
    def post(self, request, format=None):
        response = {}
        responsedata = []
        request.data._mutable = True
        request.data['created_by'] = User.objects.get(
            username=str(request.user)).id
        if 'id' in request.data:
            if CampaignPart2.objects.filter(id=request.data['id'], is_deleted=False).exists():
                serializers = self.serializer_class(instance=CampaignPart2.objects.filter(
                    id=request.data['id'], is_deleted=False).first(), data=request.data)
                message = "Successfully Updated"
            else:
                response["result"] = 0
                responsedata.append("No Such Record to update")
                response['errors'] = responsedata
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)
        else:
            serializers = self.serializer_class(data=request.data)
            message = 'Successfully Inserted'
        if serializers.is_valid():
            save_data = serializers.save()
            response["result"] = 1
            responsedata.append(message)
            response['errors'] = responsedata
            response['id'] = save_data.id
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializers.errors)
            response['result'] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class GetPlayerListAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = GetPlayerListSerializer

    def post(self, request, format=None):
        response = {}
        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            # Get players list
            player_object = Player.objects.filter(
                **serializers.validated_data, is_deleted=False)
            player_class = self.serializer_class(player_object, many=True)
            response["result"] = 1
            response['data'] = player_class.data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        else:
            response = handle_errors(serializers.errors)
            response['result'] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class CampaignPart2POSTAPIViewAdvertiser(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = CampaignPart2POSTSerializerForAdvertiser

    # Add Campaign Part2 Advertiser
    def post(self, request, format=None):
        response = {}
        responsedata = []
        request.data._mutable = True
        request.data['created_by'] = User.objects.get(
            username=str(request.user)).id
        if 'id' in request.data:
            if CampaignPart2.objects.filter(id=request.data['id'], is_deleted=False).exists():
                serializers = self.serializer_class(instance=CampaignPart2.objects.filter(
                    id=request.data['id'], is_deleted=False).first(), data=request.data)
                message = "Successfully Updated"
            else:
                response["result"] = 0
                responsedata.append("No Such Record to update")
                response['errors'] = responsedata
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)
        else:
            serializers = self.serializer_class(data=request.data)
            message = 'Successfully Inserted'
        if serializers.is_valid():
            save_data = serializers.save()
            response["result"] = 1
            responsedata.append(message)
            response['errors'] = responsedata
            response['id'] = save_data.id
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializers.errors)
            response['result'] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)



class CampaignBannerQuoteRequestAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = CampaignBannerQuoteRequestSerializer
    
    def post(self, request, format=None):
        """ 
        CampaignBannerQuoteRequestAPIView
        """ 
        response = {}
        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            serializers.validated_data['created_by'] = request.user
            serializers.save()
            response["result"] = 1
            response["message"] = ["Successfully Inserted"]
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializers.errors)
            response['result'] = 0
            self.add_api_call(request, response)
            return Response(response,status=status.HTTP_200_OK)


    def get(self, request, format=None):
        """ 
        Campaign Banner Get Request
        """ 
        response = {}
        campaign_banner_id = request.GET.get("id", False)
        try:
            if not campaign_banner_id:
                raise Exception("No ID Found")
        except Exception as e:
            response["result"] = 0
            response["errors"] = [str(e)]
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        campaign_banner = self.serializer_class.Meta.model.objects.filter(id=campaign_banner_id, created_by=request.user)
        if campaign_banner.exists():
            campaign_banner = campaign_banner.last()
        else:
            response["result"] = 0
            response["errors"] = ["No Banner Found"]
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)        
        
        campaign_banner_class = self.serializer_class(campaign_banner, many=True)
        response["result"] = 1
        response["data"] = campaign_banner_class.data
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)

    def patch(self, request, format=None):
        """ 
        Campaign Banner Patch Request
        """ 
        response = {}
        campaign_banner_id = request.GET.get("id", False)
        try:
            if not campaign_banner_id:
                raise Exception("No ID Found")
        except Exception as e:
            response["result"] = 0
            response["errors"] = [str(e)]
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        
        campaign_banner = self.serializer_class.Meta.model.objects.filter(id=campaign_banner_id, created_by=request.user)
        if campaign_banner.exists():
            campaign_banner = campaign_banner.last()
        else:
            response["result"] = 0
            response["errors"] = ["No Banner Found"]
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)    

        serializers = self.serializer_class(instance=campaign_banner,data=request.data)
        if serializers.is_valid():
            serializers.save()
            response["result"] = 1
            response["message"] = ["Successfully updated"]
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializers.errors)
            response['result'] = 0
            self.add_api_call(request, response)
            return Response(response,status=status.HTTP_200_OK)