import pytz
import json
from django.http import HttpResponse
from django.template import Template, Context
from django.template.loader import get_template
from django.db import transaction
from django.urls import reverse

from django.contrib.auth import login as django_login
import datetime
from rest_framework.authtoken.models import Token
from rest_framework import status, generics
from rest_framework.response import Response
from rest_framework.authentication import TokenAuthentication
from rest_framework.permissions import IsAuthenticated
from utils.mixins import *
from utils.api_handle_errors import handle_errors
from campaigns.serializers import *
from task.models import Task

class CampaignPart3GETEmailAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = CampaignPart3GETEmailSerializer
    # Get Campaign Part33

    def post(self, request, format=None):
        '''
        Campaign Part3 Email GET
        '''

        response = {}
        responsedata = []
        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            if 'id' in serializers.validated_data:
                data = CampaignPart3.objects.filter(
                        id=serializers.validated_data['id'], is_deleted=False)
            elif 'campaign1' in serializers.validated_data:
                data = CampaignPart3.objects.filter(
                    campaign1=serializers.validated_data['campaign1'], is_deleted=False)
            else:
                data = CampaignPart3.objects.filter(
                    created_by=request.user, is_deleted=False)
        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)
        serializer = self.serializer_class(data, many=True)
        response['result'] = 1
        response['data'] = serializer.data
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)




class CampaignPart3POSTEmailAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = CampaignPart3POSTEmailSerializer

    # Add Campaign Part3
    def post(self, request, format=None):

            response = {}
            responsedata = []
            request.data._mutable = True
            request.data['created_by'] = User.objects.get(
                username=str(request.user)).id
            if 'id' in request.data:

                if CampaignPart3.objects.filter(id=request.data['id'], is_deleted=False).exists():
                    serializers = self.serializer_class(instance=CampaignPart3.objects.filter(
                        id=request.data['id'], is_deleted=False).first(), data=request.data)
                    message = "Successfully Updated"
                else:
                    response["result"] = 0
                    responsedata.append("No Such Record to update")
                    response['errors'] = responsedata
                    self.add_api_call(request, response)
                    return Response(response, status=status.HTTP_200_OK)

            elif CampaignPart3.objects.filter(campaign1=request.data['part1ID'], is_deleted=False).exists():
                serializers = self.serializer_class(instance=CampaignPart3.objects.filter(
                    campaign1=request.data['part1ID'], is_deleted=False).first(),data=request.data)
                message = 'Successfully Updated'
            else:
                serializers = self.serializer_class(data=request.data)
                message = 'Successfully Inserted'
            if serializers.is_valid():
                save_data = serializers.save()
                # set task
                campaign1 = save_data.campaign1
                Task.objects.create(name=campaign1.name, command="campaign_mail", argument_list=campaign1.id, task_type="Campaign Mail", scheduled_time=campaign1.start_date)
                response["result"] = 1
                responsedata.append(message)
                response['errors'] = responsedata
                response['id'] = save_data.id
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)
            else:
                response = handle_errors(serializers.errors)
                response['result'] = 0
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)


class CampaignPart3GETSMSAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = CampaignPart3GETSmsSerializer
    # Get Campaign Part33

    def post(self, request, format=None):
        '''
        Campaign Part3 SMS GET
        '''

        response = {}
        responsedata = []
        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            if 'id' in serializers.validated_data:
                data = CampaignPart3.objects.filter(
                        id=serializers.validated_data['id'], is_deleted=False)
            elif 'campaign1' in serializers.validated_data:
                data = CampaignPart3.objects.filter(
                    campaign1=serializers.validated_data['campaign1'], is_deleted=False)
            else:
                data = CampaignPart3.objects.filter(
                    created_by=request.user, is_deleted=False)
        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)
        serializer = self.serializer_class(data, many=True)
        response['result'] = 1
        response['data'] = serializer.data
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)



class CampaignPart3POSTSMSAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = CampaignPart3POSTSmsSerializer
    # ADD / update SMS

    def post(self, request, format=None):
        '''
        Campaign Part3 ADD / update SMS
        '''

        response = {}
        responsedata = []
        request.data._mutable = True
        request.data['created_by'] = User.objects.get(
            username=str(request.user)).id
        if 'id' in request.data:
            if CampaignPart3.objects.filter(id=request.data['id'], is_deleted=False).exists():
                serializers = self.serializer_class(instance=CampaignPart3.objects.filter(
                    id=request.data['id'], is_deleted=False).first(), data=request.data)
                message = "Successfully Updated"
            else:
                response["result"] = 0
                responsedata.append("No Such Record to update")
                response['errors'] = responsedata
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)
                
        elif CampaignPart3.objects.filter(campaign1=request.data['part1ID'], is_deleted=False).exists():
            serializers = self.serializer_class(instance=CampaignPart3.objects.filter(
                campaign1=request.data['part1ID'], is_deleted=False).first(),data=request.data)
            message = 'Successfully Updated'
        else:
            serializers = self.serializer_class(data=request.data)
            message = 'Successfully Inserted'
        if serializers.is_valid():
            save_data = serializers.save()
            response["result"] = 1
            responsedata.append(message)
            response['errors'] = responsedata
            response['id'] = save_data.id
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializers.errors)
            response['result'] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)



class CampaignPart3GETBannerAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = CampaignPart3BannerGETSerializer
    # Get Campaign Part33

    def post(self, request, format=None):
        '''
        Campaign Part3 Email GET
        '''

        response = {}
        responsedata = []
        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            if 'id' in serializers.validated_data:
                data = CampaignPart3.objects.filter(
                    id=serializers.validated_data['id'], is_deleted=False)
            elif 'campaign1' in serializers.validated_data:
                data = CampaignPart3.objects.filter(
                    campaign1=serializers.validated_data['campaign1'], is_deleted=False)
            else:
                data = CampaignPart3.objects.filter(
                    created_by=request.user, is_deleted=False)
        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)
        serializer = self.serializer_class(data, many=True)
        response['result'] = 1
        response['data'] = serializer.data
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)




class CampaignPart3POSTBannerAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = CampaignPart3BannerPOSTSerializer
    # Get Campaign Part33

    def post(self, request, format=None):
        '''
        Banner Add/Edit
        '''

        response = {}
        responsedata = []
        request.data._mutable = True
        request.data['created_by'] = User.objects.get(
            username=str(request.user)).id
        if 'id' in request.data:
            if CampaignPart3.objects.filter(id=request.data['id'], is_deleted=False).exists():
                serializers = self.serializer_class(instance=CampaignPart3.objects.filter(
                    id=request.data['id'], is_deleted=False).first(), data=request.data)
                message = "Successfully Updated"
            else:
                response["result"] = 0
                responsedata.append("No Such Record to update")
                response['errors'] = responsedata
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)

        elif CampaignPart3.objects.filter(campaign1=request.data['part1ID'], is_deleted=False).exists():
            serializers = self.serializer_class(instance=CampaignPart3.objects.filter(
                campaign1=request.data['part1ID'], is_deleted=False).first(),data=request.data)
            message = 'Successfully Updated'
        else:
            serializers = self.serializer_class(data=request.data)
            message = 'Successfully Inserted'
        if serializers.is_valid():
            save_data = serializers.save()
            response["result"] = 1
            responsedata.append(message)
            response['errors'] = responsedata
            response['id'] = save_data.id
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializers.errors)
            response['result'] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
