import pytz
import json
from django.http import HttpResponse
from django.template import Template, Context
from django.template.loader import get_template
from django.db import transaction
from django.urls import reverse

from django.contrib.auth import login as django_login
import datetime
from rest_framework.authtoken.models import Token
from rest_framework import status, generics
from rest_framework.response import Response
from rest_framework.authentication import TokenAuthentication
from rest_framework.permissions import IsAuthenticated
from utils.mixins import *
from utils.api_handle_errors import handle_errors
from campaigns.serializers import *


class CampaignPart4GETAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = CampaignPart4GETSerializer
    # Get Campaign Part 4

    def post(self, request, format=None):
        '''
        Campaign Part4 GET
        '''

        response = {}
        responsedata = []
        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            if 'id' in serializers.validated_data:
                data = CampaignPart4.objects.filter(
                    id=serializers.validated_data['id'], is_deleted=False)
            elif 'campaign1' in serializers.validated_data:
                data = CampaignPart4.objects.filter(
                    campaign1=serializers.validated_data['campaign1'], is_deleted=False)
            else:
                data = CampaignPart4.objects.filter(
                    created_by=request.user, is_deleted=False)
        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)
        serializer = self.serializer_class(data, many=True)
        response['result'] = 1
        response['data'] = serializer.data
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


class CampaignPart4POSTAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = CampaignPart4POSTSerializer

    # Add Campaign Part4
    def post(self, request, format=None):
        '''
        Add Update Campaign Part4
        '''

        response = {}
        responsedata = []
        request.data._mutable = True
        request.data['created_by'] =  request.user.id
        if 'id' in request.data:
            campaign_part4 = CampaignPart4.objects.filter(id=request.data['id'], is_deleted=False)
            if campaign_part4.exists():
                serializers = self.serializer_class(instance=campaign_part4.last(), data=request.data)
                message = "Successfully Updated"
            else:
                response["result"] = 0
                responsedata.append("No Such Record to update")
                response['errors'] = responsedata
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)

        elif CampaignPart4.objects.filter(campaign1=request.data['part1ID'], is_deleted=False).exists():
            # serializers = self.serializer_class(instance=CampaignPart4.objects.filter(
            #     campaign1=request.data['part1ID'], is_deleted=False).first(), data=request.data)
            campaign_obj = CampaignPart4.objects.get(
                campaign1=request.data['part1ID'], is_deleted=False)
            campaign_obj.delete()
            campaign_Notification_obj = CampaignNotificationTime.objects.filter(
                campaign1=request.data['part1ID'])
            campaign_Notification_obj.delete()
            # campaign_obj.save()
            serializers = self.serializer_class(data=request.data)
            message = 'Successfully Updated'
        else:
            serializers = self.serializer_class(data=request.data)
            message = 'Successfully Inserted'
        if serializers.is_valid():
            save_data = serializers.save()
            campaign1 = save_data.campaign1
            print(campaign1)
            if campaign1.banner == True:
                campaign1.completed = True
                campaign1.save()
            response["result"] = 1
            responsedata.append(message)
            response['data'] = responsedata
            response['id'] = save_data.id
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializers.errors)
            response['result'] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class CampaignPart4PaidAdsViewsPackagesGetAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    """
    PaidAds Views Packages
    """
    serializer_class = CampaignPart4PaidAdsViewsPackagesSerializer

    def get(self, request, format=None):
        response = {}
        responsedata = []
        data = PaidAdsViewsPackages.objects.all()
        serializer = self.serializer_class(data, many=True)
        response['result'] = 1
        response['data'] = serializer.data
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)
