import pytz
import json
from django.http import HttpResponse
from django.template import Template, Context
from django.template.loader import get_template
from django.db import transaction
from django.urls import reverse

from django.contrib.auth import login as django_login
import datetime
from rest_framework.authtoken.models import Token
from rest_framework import status, generics
from rest_framework.response import Response
from rest_framework.authentication import TokenAuthentication
from rest_framework.permissions import IsAuthenticated
from utils.mixins import *
from utils.api_handle_errors import handle_errors
from campaigns.serializers import CampaignSummarySerializer, CampaignNotificationSummarySerializer, CampaignPart3SmsANDEmailSerializer, CampaignPart4DiscountSerializer, CampaignPart2RecipientSerializer
from campaigns.models import *
from venues.models import VenueField


class CampaignSummaryAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = CampaignSummarySerializer

    notification_serializer_class = CampaignNotificationSummarySerializer
    sms_email_serializer_class = CampaignPart3SmsANDEmailSerializer
    discount_serializer_class = CampaignPart4DiscountSerializer
    recipient_serializer_class = CampaignPart2RecipientSerializer
    #date_serializer_class = CampaignDateSummarySerializer
    #time_serializer_class = CampaignTimeSummarySerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []
        serializers = self.serializer_class(data=request.data)

        if serializers.is_valid():
            # Check if campign exists
            if CampaignPart1.objects.filter(id=serializers.validated_data['id'].id, created_by=request.user).exists():
                campaign1_obj = serializers.validated_data['id']
                #campaign_class = CampaignSummarySerializer(campaign1_obj)
                campaign2_obj = CampaignPart2.objects.filter(
                    campaign1=campaign1_obj, is_deleted=False)
                campaign3_sms_email_obj = CampaignPart3.objects.filter(
                    campaign1=campaign1_obj, is_deleted=False)
                campaign4_obj = CampaignPart4.objects.filter(
                    campaign1=campaign1_obj, is_deleted=False)
                campaign_notification_obj = CampaignNotificationTime.objects.filter(
                    campaign1=campaign1_obj)

            # Campaign Type
            campaign_type = ""

            if campaign1_obj.email == True:
                campaign_type = campaign_type + "Email "
            if campaign1_obj.sms == True:
                campaign_type = campaign_type + "SMS "
            if campaign1_obj.banner == True:
                campaign_type = campaign_type + "Banner "
            if campaign1_obj.paid_ads == True:
                campaign_type = campaign_type + "Paid_ads "

            campaign_type = campaign_type.strip()
            campaign_type = campaign_type.replace(" ", '-')

            # Get Duration
            if campaign1_obj.end_date == None or campaign1_obj.start_date == None:
                duration = 0
            else:
                duration = campaign1_obj.end_date - campaign1_obj.start_date
                duration = duration.days

            # Data Consolidation
            # sports_data = ""
            # if campaign1_obj.sport is not None:
            #     sports_data = campaign1_obj.sport.name

            field_data = ""
            if campaign1_obj.field is not None:
                field_data = campaign1_obj.field.name

            email_setup_fee = 0
            email_pack_fee = 0
            sms_setup_fee = 0
            sms_pack_fee = 0
            views_setup_fee = 0
            views_pack_fee = 0

            setup_fee = 0
            pack_fee = 0
            total_cost = 0
            if campaign1_obj.email:
                EmailSetupFee_obj = EmailSetupFee.objects.all().first()
                price = int(EmailSetupFee_obj.price)
                vat = int(EmailSetupFee_obj.vat.vat_code)

                setup_fee = setup_fee + price + (price * (vat)/100)
                #setup_fee_decimal = '{:.2f}'.format(setup_fee)

                # PART2 Object
                print(campaign2_obj)
                for i in campaign2_obj:
                    print('count', (i.user_notification).count())
                    email_recipient_count = (i.user_notification).count()

                # Package
                EmailPackages_obj = EmailPackages.objects.all()

                for i in EmailPackages_obj:
                    edata = []
                    qty_from = i.email_qty_from
                    qty_to = i.email_qty_to
                    if email_recipient_count >= qty_from and email_recipient_count <= qty_to:
                        edata.append(i)

                        eprice = int(edata[0].price)
                        evat = int(edata[0].vat.vat_code)
                        pack_fee = pack_fee + eprice + (eprice * (evat)/100)
                        # print('e-packfee', pack_fee)
                        # print('e-setupfee', setup_fee)

                        total_cost = total_cost + setup_fee + pack_fee
                        # print('e-', total_cost)

                email_setup_fee = email_setup_fee + setup_fee
                email_pack_fee = email_pack_fee + pack_fee

            if campaign1_obj.sms:
                setup_fee = 0
                pack_fee = 0

                SmsSetupFee_obj = SmsSetupFee.objects.all().first()
                price = int(SmsSetupFee_obj.price)
                vat = int(SmsSetupFee_obj.vat.vat_code)

                setup_fee = setup_fee + price + (price * (vat)/100)

                # PART2 Object
                print(campaign2_obj)
                for i in campaign2_obj:
                    # print(i.user_notification)
                    recipient_count = (i.user_notification).count()

                # Package
                SmsPackages_obj = SmsPackages.objects.all()

                for i in SmsPackages_obj:
                    edata = []
                    qty_from = i.sms_qty_from
                    qty_to = i.sms_qty_to
                    if recipient_count >= qty_from and recipient_count <= qty_to:
                        edata.append(i)

                        sprice = int(edata[0].price)
                        svat = int(edata[0].vat.vat_code)
                        pack_fee = pack_fee + sprice + (sprice * (svat)/100)

                        total_cost = total_cost + setup_fee + pack_fee
                        print('s', total_cost)
                sms_setup_fee = sms_setup_fee + setup_fee
                sms_pack_fee = sms_pack_fee + pack_fee

            # Paid Ads

            paid_ads_setup_fee = 0
            paid_ads_pack_fee = 0
            if campaign1_obj.paid_ads:
                PaidAdsSetupFee_obj = PaidAdsSetupFee.objects.all().first()
                price = int(PaidAdsSetupFee_obj.price)
                vat = int(PaidAdsSetupFee_obj.vat.vat_code)

                paid_ads_setup_fee = paid_ads_setup_fee + \
                    price + (price * (vat)/100)
                print('paid ads', paid_ads_setup_fee)
                #setup_fee_decimal = '{:.2f}'.format(setup_fee)

                # PART4 Object
                print('part-4', campaign4_obj)
                #print('part-4', campaign4_obj.views)

                for i in campaign4_obj:
                    print('views price', (i.views.price))
                    print('views vat code', (i.views.vat.vat_code))

                    pprice = int(i.views.price)
                    pvat = int(i.views.vat.vat_code)

                    paid_ads_pack_fee = paid_ads_pack_fee + \
                        pprice + (pprice * (pvat)/100)

                    total_cost = total_cost + \
                        paid_ads_setup_fee + paid_ads_pack_fee

                    print('paid ads pack fee', paid_ads_pack_fee)

                    print('ptotal', total_cost)
                views_setup_fee = views_setup_fee + paid_ads_setup_fee
                views_pack_fee = views_pack_fee + paid_ads_pack_fee

            campaign1_obj.cost = total_cost
            campaign1_obj.save()

            if campaign1_obj.paid_ads == True:
                campaign1_obj.completed = True
                campaign1_obj.save()

            data = {"campaign_type": campaign_type,  "start_date": campaign1_obj.start_date,
                    "duration": duration, "field": field_data, "sms_setup_fee": sms_setup_fee, "sms_package_fee": sms_pack_fee, "email_setup_fee": email_setup_fee, "email_package_fee": email_pack_fee, "views_setup_fee": views_setup_fee, "views_pack_fee": views_pack_fee, "total_fee": total_cost}  # "start_time": campaign1_obj.start_date.time()
            # campaign 2
            recipient_serializer_data = self.recipient_serializer_class(
                campaign2_obj, many=True)

            # campaign 3
            sms_email_serializer_data = self.sms_email_serializer_class(
                campaign3_sms_email_obj, many=True)

            # campaign 4
            discount_serializer_data = self.discount_serializer_class(
                campaign4_obj, many=True)

            # Campaign_notification_time
            serializer_data = self.notification_serializer_class(
                campaign_notification_obj, many=True)

            data['recipients'] = recipient_serializer_data.data

            data['sms_email_design'] = sms_email_serializer_data.data

            data['discount_percentage'] = discount_serializer_data.data

            data['campaign_notification'] = serializer_data.data

            response["result"] = 1
            responsedata.append(data)
            response['data'] = responsedata
            #response['data'] = campaign_class.data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        else:
            response = handle_errors(serializers.errors)
            response['result'] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

            return Response(response, status=status.HTTP_200_OK)
