import pytz
import json
from django.http import HttpResponse
from django.template import Template, Context
from django.template.loader import get_template
from django.db import transaction
from django.urls import reverse
import requests
from django.contrib.auth import login as django_login
from django.contrib.auth.models import User, Group
import datetime
from rest_framework.authtoken.models import Token
from rest_framework import status, generics
from rest_framework.response import Response
from rest_framework.authentication import TokenAuthentication
from rest_framework.permissions import IsAuthenticated
from utils.mixins import *
from utils.api_handle_errors import handle_errors
from campaigns.serializers import *
from players.models import *
from professionals.models import Professional
from venue_owners.models import VenueOwner
from advertisers.models import Advertiser
from static_data.models import *
from campaigns.models import *
from commonAPI.models import *
from decimal import *


class GlobalCampaignAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = globalSerializer
    Campaign1serializer_class = CampaignPart1SerializerGET
    Campaign2serializer_class = CampaignPart2SerializerGET
    Campaign3serializer_class = Campaign3Get
    Campaign4serializer_class = CampaignPart4GETSerializer

    def post(self, request, format=None):
        '''
        Campaign Global GET
        '''

        response = {}
        responsedata = []
        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            if 'id' in serializers.validated_data:
                Campaign1 = CampaignPart1.objects.filter(
                    id=serializers.validated_data['id'], is_deleted=False)
                Campaign2 = CampaignPart2.objects.filter(
                    campaign1=serializers.validated_data['id'], is_deleted=False)
                Campaign3 = CampaignPart3.objects.filter(
                    campaign1=serializers.validated_data['id'], is_deleted=False)
                Campaign4 = CampaignPart4.objects.filter(
                    campaign1=serializers.validated_data['id'], is_deleted=False)
            else:
                Campaign1 = CampaignPart1.objects.filter(
                    created_by=request.user, is_deleted=False)
                Campaign2 = CampaignPart2.objects.filter(
                    created_by=request.user, is_deleted=False)
                Campaign3 = CampaignPart3.objects.filter(
                    created_by=request.user, is_deleted=False)
                Campaign4 = CampaignPart4.objects.filter(
                    created_by=request.user, is_deleted=False)
        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)
        serializers1 = self.Campaign1serializer_class(Campaign1, many=True)
        serializers2 = self.Campaign2serializer_class(Campaign2, many=True)
        serializers3 = self.Campaign3serializer_class(Campaign3, many=True)
        serializers4 = self.Campaign4serializer_class(Campaign4, many=True)
        response['result'] = 1
        responsedata.append({"Campaign1": serializers1.data})
        responsedata.append({"Campaign2": serializers2.data})
        responsedata.append({"Campaign3": serializers3.data})
        responsedata.append({"Campaign4": serializers4.data})
        response['data'] = responsedata
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


class PaymentGatewayApiView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = globalSerializer

    def post(self, request, format=None):
        '''
        Payment Gateway  POST
        '''

        response = {}
        responsedata = []
        access_token = 'cXd2cG04ZHU1N3M3anpxc2sxdTYzY2Q5NnEwOHZnYXZndHZjdzljNTRwMWc4LmFwcHMudml2YXBheW1lbnRzLmNvbTpmNW1xeWhZcTZiaTJuNzFlUmowYnFvTzN6N3pEeE0='
        url = 'https://demo-accounts.vivapayments.com/connect/token'
        payload = {'grant_type': 'client_credentials'}
        result = requests.post(url,
                               headers={'Content-Type': 'application/x-www-form-urlencoded',
                                        'Authorization': 'Basic {}'.format(access_token), 'Accept': 'application/json'}, data=payload)

        response['result'] = 1
        responsedata.append(result.json())
        response['data'] = responsedata
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


class ChargePaymentGatewayApiView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = ChargeTokenSerializer

    def post(self, request, format=None):
        '''
        Payment Gateway  POST
        '''
        response = {}
        responsedata = []
        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            access_token = serializers.validated_data['access_token']
            charge_token = serializers.validated_data['charge_token']
            amount = serializers.validated_data['amount']
            print('Bearer {}'.format(access_token))
            url1 = "https://demo-api.vivapayments.com/acquiring/v1/cards/tokens"
            url2 = "https://demo-api.vivapayments.com/acquiring/v1/cards/chargetokens"
            url3 = "https://demo-api.vivapayments.com/nativecheckout/v2/transactions"
            payload1 = {'chargetoken': charge_token}
            result1 = requests.get(url1,
                                   headers={'Content-Type': 'application/x-www-form-urlencoded',
                                            'Authorization': 'Bearer {}'.format(access_token), 'Accept': 'application/json'}, params=payload1)
            r1_json = result1.json()

            token = r1_json['token']

            payload2 = {'token': token}
            result2 = requests.get(url2,
                                   headers={'Content-Type': 'application/x-www-form-urlencoded',
                                            'Authorization': 'Bearer {}'.format(access_token), 'Accept': 'application/json'}, params=payload2)
            r2_json = result2.json()
            token_charge = r2_json['chargeToken']
            #amount = amount*100
            payload3 = {
                "amount": amount,
                "preauth": false,
                "sourceCode": "1325",
                "chargeToken": token_charge,
                "installments": 1,
                "merchantTrns": "Merchant transaction reference",
                "customerTrns": "Sample transaction ",
                "currencyCode": 978,
                "customer": {
                              "email": "native@vivawallet.com",
                              "phone": "409167937459",
                              "fullname": "Abhishek pawar",
                              "requestLang": "en",
                              "countryCode": "GB"
                },
            }
            result3 = requests.get(url3,
                                   headers={'Content-Type': 'application/x-www-form-urlencoded',
                                            'Authorization': 'Bearer {}'.format(access_token), 'Accept': 'application/json'}, data=payload3)
            r3_json = result3.json()
            #r3 = r3_json[0]
            token = r3_json['transactionId']
            response['result'] = 1
            responsedata.append(result3.json())
            response['data'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class GetPlayerByCityCountryAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = GetCityAndCountrySerializer
    new_serializer_class = AllUserCitySerializer

    def post(self, request, format=None):
        """
        Get Player by City and Country
        """
        response = {}
        responsedata = []
        serializers = self.serializer_class(
            data=request.data)
        if serializers.is_valid():

            user_ids = []
            myid = request.user.id

            if 'city' in serializers.validated_data:
                player_obj = Player.objects.filter(
                    city__in=serializers.validated_data['city'], is_deleted=False)
                player_count = player_obj.count()
                if player_count > 0:
                    for p in range(0, player_count):
                        if player_obj[p].user.id != myid:
                            user_ids.append(player_obj[p].user.id)

                venueowner_obj = VenueOwner.objects.filter(
                    city__in=serializers.validated_data['city'], is_deleted=False)
                venueowner_count = venueowner_obj.count()
                if venueowner_count > 0:
                    for v in range(0, venueowner_count):
                        if venueowner_obj[v].user.id != myid:
                            user_ids.append(venueowner_obj[v].user.id)

                professional_obj = Professional.objects.filter(
                    city__in=serializers.validated_data['city'], is_deleted=False)
                professional_count = professional_obj.count()
                if professional_count > 0:
                    for pr in range(0, professional_count):
                        if professional_obj[pr].user.id != myid:
                            user_ids.append(professional_obj[pr].user.id)

                advertiser_obj = Advertiser.objects.filter(
                    city__in=serializers.validated_data['city'], is_deleted=False)
                advertiser_count = advertiser_obj.count()
                if advertiser_count > 0:
                    for a in range(0, advertiser_count):
                        if advertiser_obj[a].user.id != myid:
                            user_ids.append(advertiser_obj[a].user.id)
                            print("h1")

            if len(user_ids) > 0:
                all_users = User.objects.filter(id__in=user_ids)
                # serialized_data=self.serializer_class(all_users,many=True)
                l = []
                for z in range(0, all_users.count()):
                    o = {}
                    o['first_name'] = all_users[z].first_name

                    o['last_name'] = all_users[z].last_name

                    o['id'] = all_users[z].id
                    l.append(o)
                response["result"] = 1
                #response["data"] = serialized_data.data
                response["data"] = l
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)
            else:
                response["result"] = 1
                response["data"] = []
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)

        else:
            response = handle_errors(serializers.errors)
            response['result'] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class AllUsersCityCountryAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    '''
    Get city and country from all users
    '''
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    player_serializer_class = PlayerCityAndCountrySerializer
    professional_serializer_class = ProfessionalCityAndCountrySerializer
    venue_owner_serializer_class = VenueOwnerCityAndCountrySerializer
    advertiser_serializer_class = AdvertiserCityAndCountrySerializer
    # get sports by groups

    def get(self, request, format=None):
        response = {}
        responsedata = []
        player_obj = Player.objects.filter(is_deleted=False)
        venueowner_obj = VenueOwner.objects.filter(is_deleted=False)
        professional_obj = Professional.objects.filter(is_deleted=False)
        Advertiser_obj = Advertiser.objects.filter(is_deleted=False)

        serializers1 = self.player_serializer_class(player_obj, many=True)
        serializers2 = self.venue_owner_serializer_class(
            venueowner_obj, many=True)
        serializers3 = self.professional_serializer_class(
            professional_obj, many=True)
        serializers4 = self.advertiser_serializer_class(
            Advertiser_obj, many=True)

        data = {"player_city_country": serializers1.data, "venue_owner_city_country": serializers2.data,
                "professional_city_country": serializers3.data, "advertiser_city_country": serializers4.data}

        response['result'] = 1
        response['data'] = data
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


class ViewsPackageAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    """
    Views Package
    """
    serializer_class = ViewPackageSerializer

    def get(self, request, format=None):
        response = {}
        responsedata = []
        data = ViewsPackage.objects.all()
        serializer = self.serializer_class(data, many=True)
        response['result'] = 1
        response['data'] = serializer.data
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


class ClicksPackageAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    """
    Clicks Package
    """
    serializer_class = ClickPackageSerializer

    def get(self, request, format=None):
        response = {}
        responsedata = []
        data = ClicksPackage.objects.all()
        serializer = self.serializer_class(data, many=True)
        response['result'] = 1
        response['data'] = serializer.data
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


class GeoFilterApi(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = PlayerAreaSerializer

    def post(self, request, format=None):
        '''
        Area serializer
        '''
        response = {}
        responsedata = []
        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            ar = []
            area_ids = []
            sports = []
            pids = []
            if 'area_list' in serializers.validated_data:
                ar = serializers.validated_data['area_list']

            if 'sports_list' in serializers.validated_data:
                sports = serializers.validated_data['sports_list']
                ppobj = PlayerSkills.objects.filter(sport__id__in=sports)
                for j in range(0, ppobj.count()):
                    pids.append(ppobj[j].player.id)
                pids = list(set(pids))

            aobj = Area.objects.filter(name__in=ar)
            for i in range(0, aobj.count()):
                area_ids.append(aobj[i].id)

            player_data = []
            if len(pids) > 0 and len(area_ids) > 0:
                pobj = Player.objects.filter(
                    area__id__in=area_ids, id__in=pids)
                for a in range(0, pobj.count()):
                    if BookMatch.objects.filter(created_by=pobj[a].id, venue_field__created_by=request.user).exists():
                        dt = {}
                        dt['id'] = pobj[a].id
                        dt['firstname'] = pobj[a].first_name
                        dt['lastname'] = pobj[a].last_name
                        dt['nickname'] = pobj[a].nick_name
                        player_data.append(dt)
            elif len(pids) == 0 and len(area_ids) > 0:
                pobj = Player.objects.filter(area__id__in=area_ids)
                for a in range(0, pobj.count()):
                    if BookMatch.objects.filter(created_by=pobj[a].id, venue_field__created_by=request.user).exists():
                        dt = {}
                        dt['id'] = pobj[a].id
                        dt['firstname'] = pobj[a].first_name
                        dt['lastname'] = pobj[a].last_name
                        dt['nickname'] = pobj[a].nick_name
                        player_data.append(dt)
            elif len(pids) > 0 and len(area_ids) == 0:
                pobj = Player.objects.filter(id__in=pids)
                for a in range(0, pobj.count()):
                    if BookMatch.objects.filter(created_by=pobj[a].id, venue_field__created_by=request.user).exists():
                        dt = {}
                        dt['id'] = pobj[a].id
                        dt['firstname'] = pobj[a].first_name
                        dt['lastname'] = pobj[a].last_name
                        dt['nickname'] = pobj[a].nick_name
                        player_data.append(dt)
            else:
                print("no data")

            response['result'] = 1
            response['data'] = player_data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class CreateOrderPaymentApiView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = CreateOrderSerializer

    def post(self, request, format=None):
        '''
        Payment Gateway  POST
        '''
        response = {}
        responsedata = []
        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            camp_id = serializers.validated_data['camp_id']
            url_of_viva = "https://demo.vivapayments.com/api/orders"
            if CampaignPart1.objects.filter(id=camp_id, is_deleted=False).exists():
                camp_obj = CampaignPart1.objects.get(
                    id=camp_id, is_deleted=False)
                # print(Decimal(camp_obj.cost))
                value = Decimal(camp_obj.cost)
                # print(value*100)

                amount = 0
                if camp_obj.email:
                    amount = amount+(int(value*100))
                elif camp_obj.sms:
                    amount = amount+(int(value*100))
                elif camp_obj.sms and camp_obj.email:
                    amount = amount+(int(value*100))
                print(amount)

                user_details = None
                if VenueOwner.objects.filter(user=camp_obj.created_by).exists():
                    user_details = VenueOwner.objects.get(
                        user=camp_obj.created_by)
                else:
                    user_details = Advertiser.objects.filter(
                        user=camp_obj.created_by)
                payload = {
                    "amount": amount,
                    "email": camp_obj.created_by.username,
                    "fullName": user_details.first_name+" "+user_details.last_name,
                    "customerTrns": "PAYMENT FOR CAMPAIGN",
                    "phone": user_details.mobile,
                    "requestLang": "en-US",
                    "sourceCode": "1325",
                    "disableCash": "false",
                    "disableExactAmount": "false",
                    "allowRecurring": "false",
                    "isPreAuth": "false",
                    "maxInstallments": 1,
                    "paymentTimeOut": 86400,
                    "merchantTrns": "PAYMENT FOR CAMPAIGN",
                }
                access_token = "NWZlZTg4YjAtYzUyNy00MGJmLTkzOTAtYjkzMjM4NzMyNTI4OnxacDIuXw=="
                result_of_order = requests.post(url_of_viva,
                                                headers={'Content-Type': 'application/x-www-form-urlencoded',
                                                         'Authorization': 'Basic {}'.format(access_token), 'Accept': 'application/json'}, data=payload)
                print(result_of_order.json()['OrderCode'])

                camp_obj.order_code = result_of_order.json()['OrderCode']
                camp_obj.save()

                t = Transaction.objects.create(order_code=result_of_order.json(
                )['OrderCode'], payment_done_for="C", created_by=camp_obj.created_by)
                t.save()
                response['result'] = 1
                responsedata.append(result_of_order.json()['OrderCode'])
                response['data'] = responsedata
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)
            else:
                response['result'] = 0
                responsedata.append("NO SUCH CAMPAIGN FOUND")
                response['data'] = responsedata
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)

        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class TransactionCheck(PlatformAPICallMixin, generics.GenericAPIView):
    serializer_class = TransactionSerializer

    def post(self, request, format=None):
        '''
        Transaction success  POST
        '''
        response = {}
        responsedata = []
        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            order_code = serializers.validated_data['orderCode']
            t_id = serializers.validated_data['transactionId']
            access_token = "NWZlZTg4YjAtYzUyNy00MGJmLTkzOTAtYjkzMjM4NzMyNTI4OnxacDIuXw=="
            url_of_viva = "https://demo.vivapayments.com/api/transactions/"+t_id
            result_of_order = requests.get(url_of_viva,
                                           headers={'Content-Type': 'application/x-www-form-urlencoded',
                                                    'Authorization': 'Basic {}'.format(access_token), 'Accept': 'application/json'})
            print(result_of_order.json())
            message = ""
            u = ""
            if result_of_order.json()['Success']:
                t = Transaction.objects.get(order_code=order_code)
                t.transaction_id = t_id
                t.payment_status = "S"
                t.save()
                camp_obj = CampaignPart1.objects.get(order_code=order_code)
                camp_obj.payment_status = "S"
                camp_obj.completed = True
                camp_obj.save()
                if VenueOwner.objects.filter(user=camp_obj.created_by).exists():
                    u = "V"
                else:
                    u = "A"
                message = "Your Payment Has Been Successful! Your Campaign is in process"
            else:
                t = Transaction.objects.get(order_code=order_code)
                t.transaction_id = t_id
                t.payment_status = "F"
                t.save()
                camp_obj = CampaignPart1.objects.get(order_code=order_code)
                camp_obj.payment_status = "F"
                camp_obj.save()
                if VenueOwner.objects.filter(user=camp_obj.created_by).exists():
                    u = "V"
                else:
                    u = "A"
                message = "Sorry your payment has failed,campaign is not created."
            response['result'] = 1
            responsedata.append(message)
            response['data'] = responsedata
            response['user'] = [u]
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)
