import pytz
import json
from django.http import HttpResponse
from django.template import Template, Context
from django.template.loader import get_template
from django.db import transaction
from django.urls import reverse

from django.contrib.auth import login as django_login
import datetime
from rest_framework.authtoken.models import Token
from rest_framework import status, generics
from rest_framework.response import Response
from rest_framework.authentication import TokenAuthentication
from rest_framework.permissions import IsAuthenticated
from utils.mixins import *
from utils.api_handle_errors import handle_errors
from campaigns.serializers import *
from django.utils import timezone
from venues.models import VenueField

# Apply pagination


class CampaignListLISTGET(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = CampaignLISTSerializer

    def post(self, request, format=None):

        response = {}
        responsedata = []
        request.data._mutable = True
        request.data['created_by'] = User.objects.get(
            username=str(request.user)).id

        serializer = self.serializer_class(data=request.data)
        if serializer.is_valid():

            pagination = serializer.validated_data['page']
            serializer.validated_data.pop("page")
            serializer.validated_data['completed'] = True

            # if "start_date" in serializer.validated_data:
            #     serializer.validated_data['start_date__gte'] = serializer.validated_data["start_date"]
            #     serializer.validated_data.pop("start_date")

            # if "end_date" in serializer.validated_data:
            #     serializer.validated_data['end_date__lte'] = serializer.validated_data["end_date"]
            #     serializer.validated_data.pop("end_date")

            if "campaign_kind" in serializer.validated_data:
                if serializer.validated_data['campaign_kind'] == "email":
                    serializer.validated_data['email'] = True
                elif serializer.validated_data['campaign_kind'] == "sms":
                    serializer.validated_data['sms'] = True
                elif serializer.validated_data['campaign_kind'] == "banner":
                    serializer.validated_data['banner'] = True
                elif serializer.validated_data['campaign_kind'] == "paid_ads":
                    serializer.validated_data['paid_ads'] = True
                serializer.validated_data.pop("campaign_kind")

            if "status" in serializer.validated_data:
                if serializer.validated_data["status"] == "UNDER APPROVAL":
                    serializer.validated_data['status'] = "O"
                elif serializer.validated_data["status"] == "ON GOING":
                    serializer.validated_data["start_date__lte"] = timezone.now(
                    )
                    serializer.validated_data['status'] = "A"
                elif serializer.validated_data["status"] == "WAITING":
                    serializer.validated_data["start_date__gte"] = timezone.now(
                    )
                    serializer.validated_data['status'] = "A"
                elif serializer.validated_data["status"] == "REJECTED":
                    serializer.validated_data['status'] = "R"

            if "keyword" in serializer.validated_data:
                serializer.validated_data['name__icontains'] = serializer.validated_data['keyword']
                serializer.validated_data.pop("keyword")

            if "start_date" in serializer.validated_data and "end_date" in serializer.validated_data:
                serializer.validated_data['start_date__gte'] = serializer.validated_data.pop(
                    "start_date")
                serializer.validated_data['end_date__lte'] = serializer.validated_data.pop(
                    "end_date")

            start_data = (pagination - 1) * 10
            end_data = start_data + 10

            if "assortment" in serializer.validated_data:
                assortment = serializer.validated_data['assortment']
                serializer.validated_data.pop("assortment")
                campaign_data = CampaignPart1.objects.filter(
                    **serializer.validated_data).order_by('-'+assortment)

            else:
                campaign_data = CampaignPart1.objects.filter(
                    **serializer.validated_data).order_by("-created_at")
            campaign_count = campaign_data.count()
            campaign_data = campaign_data[start_data:end_data]

            serializer = self.serializer_class(campaign_data, many=True)

            response["result"] = 1
            response['data'] = serializer.data
            response['total_page'] = campaign_count
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class CampaignListLISTDELETE(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = CampaignDELETESerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []
        serializer = self.serializer_class(data=request.data)
        if serializer.is_valid():
            # Check whether it is started or not
            print(serializer.validated_data['id'].start_date)
            print(timezone.now())
            if serializer.validated_data['id'].start_date < timezone.now():
                response["result"] = 0
                responsedata.append("This Campaign Cannot be deleted")
                response['errors'] = responsedata
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)

            serializer.validated_data['id'].is_deleted = True
            serializer.validated_data['id'].save()

            response["result"] = 1
            responsedata.append("Sucessfully deleted")
            response['errors'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)
