from django.db import models

from django.core.validators import MaxValueValidator, MinValueValidator

from django.contrib.auth.models import User
from storages.backends.s3boto import S3BotoStorage

from static_data.models import *
from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from salesman.models import *


# Create your models here.


class S3CustomStorage(S3BotoStorage):
    def _normalize_name(self, name):
        """
        S3 Storage function for storing files
        """
        return name


class Document(models.Model):
    """ Documents details reside here """
    document_name = models.CharField(max_length=50)
    # document        = models.CharField(max_length=19,blank = True, null = True)
    document = models.FileField(
        upload_to="professional_Documents", blank=True, null=True)
    is_deleted = models.BooleanField(
        default=False, help_text="activate/deactivate user")
    created_by = models.ForeignKey(
        User, on_delete=models.CASCADE, blank=True, null=True)
    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)
    updated_at = models.DateTimeField(auto_now=True, blank=True, null=True)

    def __str__(self):
        return "{}-{}-{}".format(self.document_name, self.created_by, self.document)

    class Meta:
        verbose_name_plural = "Documents"
        db_table = 'Documents'


class BankInfo(models.Model):
    """ Bank Account Information """
    iban = models.CharField(max_length=100, help_text="Bank Account Number")
    certificate_name = models.CharField(
        max_length=100, help_text="Certificates", blank=True, null=True)

    bank = models.CharField(max_length=100, help_text="Bank Name")
    acc_holder_first_name = models.CharField(
        max_length=100, help_text="Bank Account Holder First Name")

    acc_holder_last_name = models.CharField(
        max_length=100, help_text="Bank Account Holder Last Name", blank=True, null=True)
    acc_type = models.CharField(
        max_length=100, null=True, blank=True, help_text="Account Type")
    bank_identification_no = models.CharField(
        max_length=100, null=True, blank=True, help_text="Bank BIC/Sift Code")
    iban_certificate = models.FileField(
        upload_to='pdfs/', null=True, blank=True, help_text='IBAN CERTIFICATE')

    is_deleted = models.BooleanField(
        default=False, help_text="Active/Deleted Bank")

    content_type = models.ForeignKey(
        ContentType, on_delete=models.CASCADE, blank=True, null=True)
    object_id = models.PositiveIntegerField(blank=True, null=True)
    content_object = GenericForeignKey('content_type', 'object_id')

    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)
    created_by = models.ForeignKey(
        User, on_delete=models.CASCADE, blank=True, null=True)

    def __str__(self):
        return "{}".format(self.bank)

    class Meta:
        verbose_name_plural = "Bank Info"
        db_table = 'bank_info'


class Contact_Person(models.Model):
    """
    Profiles Contact Persons
    """
    first_name = models.CharField(
        max_length=100, null=True, blank=True, help_text="Contact Person first name")
    last_name = models.CharField(
        max_length=100, null=True, blank=True, help_text="user last name")
    email = models.CharField(max_length=50, blank=True, null=True)

    telephone = models.CharField(max_length=15, null=True, blank=True)
    is_deleted = models.BooleanField(
        default=False, help_text="activate/deactivate user")

    content_type = models.ForeignKey(
        ContentType, on_delete=models.CASCADE, blank=True, null=True)
    object_id = models.PositiveIntegerField(blank=True, null=True)
    content_object = GenericForeignKey('content_type', 'object_id')

    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)

    def __str__(self):
        return "{}-{}".format(self.first_name, self.last_name)

    class Meta:
        verbose_name_plural = "ContactPerson"
        db_table = 'common_contactPerson'


class ContactInfo(models.Model):
    """  Contact Info """
    GENDER_CHOICES = (
        ('M', 'Male'),
        ('F', 'Female'),
        ('O', 'Other')
    )
    first_name = models.CharField(max_length=50, blank=True, null=True)
    last_name = models.CharField(max_length=50, blank=True, null=True)
    mobile = models.CharField(max_length=20, blank=True, null=True)
    email = models.CharField(max_length=50, blank=True, null=True)
    gender = models.CharField(max_length=1, null=True, choices=GENDER_CHOICES,
                              help_text="M - Male, F - Female, O - Other")
    is_deleted = models.BooleanField(default=False)

    content_type = models.ForeignKey(
        ContentType, on_delete=models.CASCADE, blank=True, null=True)
    object_id = models.PositiveIntegerField(blank=True, null=True)
    content_object = GenericForeignKey('content_type', 'object_id')

    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)
    updated_at = models.DateTimeField(auto_now=True, blank=True, null=True)

    def __str__(self):
        return "{}-{}".format(self.first_name, self.email)

    class Meta:
        verbose_name_plural = "Contact Info"
        db_table = 'contactInfo'


class TelephoneNumbers(models.Model):
    """
    All Telephone Numbers
    """
    contact_id = models.ForeignKey(
        ContactInfo, on_delete=models.CASCADE, blank=True, null=True)
    contact_number = models.CharField(max_length=15, null=True, blank=True)

    def __str__(self):
        return "{}-{}".format(self.contact_id, self.contact_number)

    class Meta:
        verbose_name_plural = "Telephone Numbers"
        db_table = "TelephoneNumber"


class OnBoardAPI(models.Model):
    """
    On Board API Form
    """
    REGISTERATION_TYPE = (
        ('A', 'Advertiser'),
        ('P', 'Professional'),
        ('V', 'Venue Owner'),
    )
    GENDER_CHOICES = (
        ('M', 'Male'),
        ('F', 'Female'),
        ('O', 'Other')
    )
    STATUS_CHOICES = (
        ('I', 'In Progress'),
        ('R', 'Rejected'),
        ('N', 'New'),
        ('A', 'Approve'),
        ('C', 'Complete')
    )
    first_name = models.CharField(max_length=50, blank=True, null=True)
    last_name = models.CharField(max_length=50, blank=True, null=True)
    gender = models.CharField(max_length=1, null=True, choices=GENDER_CHOICES,
                              help_text="M - Male, F - Female, O - Other")
    registeration_type = models.CharField(
        max_length=1, null=True, choices=REGISTERATION_TYPE,
        help_text="V - Venue Owner, P - Professional, A - Advertiser")
    email = models.CharField(max_length=50, blank=True, null=True)
    telephone = models.CharField(max_length=50, blank=True, null=True)
    telephone2 = models.CharField(max_length=50, blank=True, null=True)
    mobile = models.CharField(max_length=20, blank=True, null=True)
    business_name = models.CharField(max_length=50, blank=True, null=True)
    address = models.TextField(max_length=100, blank=True, null=True)
    field_type = models.CharField(max_length=100, blank=True, null=True)
    buisness_type = models.CharField(max_length=100, blank=True, null=True)
    sports = models.ManyToManyField(Sport, blank=True)
    # buisnessVat         = models.CharField(max_length=50, blank=True, null=True)
    # city = models.CharField(max_length=100, blank=True, null=True)
    # country = models.CharField(max_length=100, blank=True, null=True)
    city = models.ForeignKey(
        City, on_delete=models.CASCADE, blank=True, null=True)
    country = models.ForeignKey(
        Country, on_delete=models.CASCADE, blank=True, null=True)
    state = models.ForeignKey(
        State, on_delete=models.CASCADE, blank=True, null=True)
    area = models.ForeignKey(
        Area, on_delete=models.CASCADE, blank=True, null=True)
    # service             = models.CharField(max_length=100, blank=True, null=True)
    service = models.ManyToManyField(Profession, blank=True)
    total_field = models.CharField(max_length=50, blank=True, null=True)
    buisness_vat = models.CharField(max_length=50, blank=True, null=True)
    comments = models.CharField(max_length=5000, blank=True, null=True)
    is_deleted = models.BooleanField(default=False)
    status = models.CharField(
        max_length=10, default=STATUS_CHOICES[2][0], choices=STATUS_CHOICES, blank=True, null=True,
        help_text="Is document Verified")
    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)
    created_by = models.ForeignKey(
        User, on_delete=models.CASCADE, blank=True, null=True)

    def __str__(self):
        return "{}-{}-{}".format(self.first_name, self.email, self.created_by)

    class Meta:
        verbose_name_plural = "Onboard API"
        db_table = 'onboard'


class ProfileSetting(models.Model):
    """
    Profile Settings
    """
    Notification_CHOICES = (
        ('M', 'Messages'),
        ('F', 'Join Requests'),
        ('S', 'Schedule'),
        ('P', 'Platform Messages'),
        ('O', 'Offers')
    )
    notification_name = models.CharField(max_length=1, null=True, choices=Notification_CHOICES,
                                         help_text="M - Message, J - Join Requests, S - Schedule, P - Platform Messages, O - Offers")

    is_active = models.BooleanField(default=False)
    from_time = models.TimeField(blank=True, null=True)

    to_time = models.TimeField(blank=True, null=True)
    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)

    content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE)
    object_id = models.PositiveIntegerField()
    content_object = GenericForeignKey('content_type', 'object_id')

    def __str__(self):
        return "{}-{}".format(self.notification_name, self.is_active)

    class Meta:
        verbose_name_plural = "Profile Settings"
        db_table = 'profile_Setting'


class ContractTerms(models.Model):
    """
    Contract Terms
    """

    commission_fee = models.TextField(blank=True, null=True)
    payments = models.TextField(blank=True, null=True)
    # collaboration_name = models.ForeignKey(
    #     CollaborationTypes, on_delete=models.CASCADE, blank=True, null=True)
    collaboration_name = models.CharField(max_length=50, blank=True, null=True)
    # contract_terms_document = models.ForeignKey(ContractTermsDocument, on_delete=models.CASCADE, null=True, blank=True)
    contract_terms_document = models.FileField(
        upload_to="Contract_Terms_Documents", blank=True, null=True)
    cancellation_policy = models.ManyToManyField(
        CancellationPolicy, blank=True)
    salesman = models.ForeignKey(
        Salesman, on_delete=models.CASCADE, blank=True, null=True)
    supervisor = models.CharField(max_length=50, blank=True, null=True)
    created_for = models.ForeignKey(
        User, on_delete=models.CASCADE, blank=True, null=True)

    def __str__(self):
        return "{}-{}-{}-{}".format(self.id, self.collaboration_name, self.cancellation_policy, self.contract_terms_document)

    class Meta:
        verbose_name_plural = "Contract Terms"
        db_table = 'Contract_Terms'


class Platform_Update(models.Model):
    """
    Platform_Updates
    """

    email = models.CharField(max_length=50, blank=True, null=True)
    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)

    def __str__(self):
        return "{}".format(self.email)

    class Meta:
        verbose_name_plural = "Platform Updates"
        db_table = 'platform_updates'


class Transaction(models.Model):
    """
    Transactions
    """

    PAYMENT_STATUS_CHOICES = (
        ('F', 'Failed'),
        ('S', 'Success'),
    )
    PAYMENT_FOR_CHOICES = (
        ('C', 'Campaign'),
        ('B', 'Booking'),
    )

    transaction_id = models.CharField(
        max_length=100, blank=True, null=True, help_text="id of transaction")
    order_code = models.CharField(max_length=100, blank=True, null=True)
    payment_status = models.CharField(
        max_length=10, default=PAYMENT_STATUS_CHOICES[0][0], choices=PAYMENT_STATUS_CHOICES, blank=True, null=True, help_text="Payment Status-F=Failed,S=Success")
    payment_done_for = models.CharField(
        max_length=10, choices=PAYMENT_FOR_CHOICES, blank=True, null=True, help_text="C= Campaign,B=Booking")
    created_by = models.ForeignKey(
        User, on_delete=models.CASCADE, blank=True, null=True)
    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)

    def __str__(self):
        return "{}-{}-{}".format(self.transaction_id, self.payment_status, self.payment_done_for)

    class Meta:
        verbose_name_plural = "Transactions"
        db_table = 'transaction'
