from rest_framework import serializers
from django.contrib.auth.models import User, Group
from rest_framework.authtoken.models import Token

from datetime import date
from venue_owners.models import VenueOwner
from django.contrib.auth.password_validation import validate_password

from players.models import Player

from django.contrib.auth import authenticate
from professionals.models import Professional
from advertisers.models import Advertiser
from static_data.models import Profession
from static_data.serializers import *
from .models import *
from salesman.serializers import *


class UserSerializer(serializers.ModelSerializer):
    class Meta:
        model = User
        fields = '__all__'


class CityApiserializer(serializers.Serializer):
    country = serializers.CharField(
        required=False, max_length=100, min_length=8)
    keyword = serializers.CharField(
        required=False, max_length=100, min_length=8)


class CountryApiserializer(serializers.Serializer):
    keyword = serializers.CharField(
        required=False, max_length=100, min_length=8)


class ChangePassword(serializers.Serializer):
    password = serializers.CharField(
        required=False, max_length=100, min_length=8)

    def save(self, **kwargs):
        password = self.validated_data['password']
        print(kwargs)
        user = User.objects.get(username=kwargs['user'])
        user.set_password(password)
        user.save()
        return user


class DocumentSerializer(serializers.ModelSerializer):
    Uploaded_by = serializers.CharField(
        source='created_by.username', read_only=True)

    class Meta:
        model = Document
        fields = "__all__"


class DocumentDeleteSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(required=True)

    class Meta:
        model = Document
        fields = "__all__"
        extra_kwargs = {
            'document_name': {'required': False},
        }


class BankInfoSerializer(serializers.ModelSerializer):
    mode = serializers.CharField(
        required=True, help_text="Advertiser,Professional,Venue Owner,Player", write_only=True)
    field_id = serializers.CharField(
        required=False, help_text="ID for specific")

    def __init__(self, *args, **kwargs):
        super(BankInfoSerializer, self).__init__(*args, **kwargs)
        for field in self.fields:
            if field == 'mode' or field == "field_id":
                self.fields[field].read_only = False
            else:
                self.fields[field].read_only = True

    def validate_field_id(self, value):
        if BankInfo.objects.filter(id=value).exists():
            return value
        else:
            raise FileNotFoundError("No Data found")

    class Meta:
        model = BankInfo
        fields = "__all__"


class BankInfoEDITSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(write_only=True, required=True)

    class Meta:
        model = BankInfo
        fields = "__all__"
        extra_kwargs = {
            'iban': {
                'required': False
            },
            'acc_holder_last_name': {
                'required': False
            },
            'acc_holder_first_name': {
                'required': False
            },
            'bank': {
                'required': False
            },
            'acc_type': {
                'required': False
            },
            'bank_identification_no': {
                'required': False
            },
            'iban_certificate': {
                'required': False
            }
        }


class BankInfoPOSTSerializer(serializers.Serializer):
    data = serializers.JSONField(
        required=False,
        help_text='[{"Iban": "1234","Bank": "State","acc_holder_first_name": "Yatin","acc_holder_last_name": "Pandit" }]')
    mode = serializers.CharField(
        required=True, help_text="Advertiser,Professional,Venue Owner,Player", write_only=True)

    def create(self, validated_data, **kwargs):
        try:
            data_obj = VenueOwner.objects.filter(user=self.context['user']).first() if validated_data[
                                                                                           'mode'] == "Venue Owner" else Player.objects.filter(
                user=self.context['user']).first(
            ) if validated_data['mode'] == "Player" else Professional.objects.filter(
                user=self.context['user']).first() if validated_data[
                                                          'mode'] == "Professional" else Advertiser.objects.filter(
                user=self.context['user']).first()
            print(data_obj)

            data = [BankInfo(
                **i,
                content_object=data_obj
            )
                for i in validated_data['data']
            ]
            print("hii")
            BankInfo.objects.bulk_create(data)
            return 'Success'
        except Exception as e:
            raise serializers.FieldDoesNotExist(str(e))


class BankDeleteSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(required=True)

    class Meta:
        model = BankInfo
        fields = ("id",)
        extra_kwargs = {
            'iban': {'required': False},
            'Bank': {'required': False},
            'acc_holder_last_name': {'required': False},
            'acc_holder_first_name': {'required': False},
            'acc_type': {'required': False},
            'bank_identification_no': {'required': False},
            'iban_certificate': {'required': False}
        }


# Change Password Serialziers =======================

class ChangePasswordSerializers(serializers.Serializer):
    username = serializers.CharField(
        required=True, help_text="Username of the user")
    password = serializers.CharField(
        required=True, help_text="Password of the user")
    key = serializers.CharField(required=True, help_text="Key")


# Contact INFORMATION ===============================>

class TelephoneNumbersSerializers(serializers.ModelSerializer):
    class Meta:
        model = TelephoneNumbers
        fields = ['contact_number']


class UserContactSerializer(serializers.ModelSerializer):
    """
    User Serializer for Contact Info
    """

    class Meta:
        model = User
        fields = ['first_name', 'last_name']


class ContactInfoSerializer(serializers.ModelSerializer):
    mode = serializers.CharField(
        required=True, help_text="Advertiser,Professional,Venue Owner,Player", write_only=True)
    telephone_numbers = serializers.JSONField(write_only=True, required=False)
    telephone = serializers.CharField(required=False, help_text="mode telephone_number")
    telephone2 = serializers.CharField(required=False, help_text="mode telephone_number2")

    def update(self, instance, validated_data):
        instance.first_name = validated_data.get(
            'first_name', instance.first_name)
        instance.last_name = validated_data.get(
            'last_name', instance.last_name)
        instance.mobile = validated_data.get('mobile', instance.mobile)
        instance.email = validated_data.get('email', instance.email)
        instance.gender = validated_data.get('gender', instance.gender)

        instance.save()

        # Save User Info
        user = UserContactSerializer(
            instance=self.context['user'], data=validated_data)
        if user.is_valid():
            user.save()

        telephone = validated_data['telephone']
        telephone2 = validated_data['telephone2']
        validated_data.pop('telephone')
        validated_data.pop('telephone2')

        if validated_data['mode'] == "Professional":
            prof = Professional.objects.get(user=self.context['user'])
            print('prof------------------', prof)
            prof.first_name = instance.first_name
            prof.last_name = instance.last_name
            prof.gender = instance.gender
            prof.telephone = telephone
            prof.telephone2 = telephone2
            prof.save()

        elif validated_data['mode'] == "Advertiser":
            adv = Advertiser.objects.get(user=self.context['user'])
            print(adv)
            adv.first_name = instance.first_name
            adv.last_name = instance.last_name
            adv.gender = instance.gender
            adv.telephone = telephone
            adv.telephone2 = telephone2
            adv.save()

        elif validated_data['mode'] == "Player":
            playerr = Player.objects.get(user=self.context['user'])
            print(playerr)
            playerr.first_name = instance.first_name
            playerr.last_name = instance.last_name
            playerr.gender = instance.gender
            playerr.telephone = telephone
            playerr.telephone2 = telephone2
            playerr.save()

        else:
            vo = VenueOwner.objects.get(user=self.context['user'])
            print(vo)
            vo.first_name = instance.first_name
            vo.last_name = instance.last_name
            vo.gender = instance.gender
            vo.telephone = telephone
            vo.telephone2 = telephone2
            vo.save()

        if 'telephone_numbers' in validated_data:
            telephone_numbers = []
            for i in validated_data['telephone_numbers']:
                if i == "" or i is None:
                    pass
                else:
                    telephone_numbers.append(i)

            TelephoneNumbers.objects.filter(contact_id=instance.id).delete()
            data = [TelephoneNumbers(
                contact_id=instance,
                contact_number=i
            ) for i in telephone_numbers]
            TelephoneNumbers.objects.bulk_create(data)
        return "Success"

    def create(self, validated_data):
        telephone_numbers = None

        validated_data['content_object'] = VenueOwner.objects.get(user=self.context['user'], is_deleted=False) if \
            validated_data['mode'] == "Venue Owner" else Player.objects.get(
            user=self.context['user'], is_deleted=False) if validated_data[
                                                                'mode'] == "Player" else Professional.objects.get(
            user=self.context['user'], is_deleted=False) if validated_data[
                                                                'mode'] == "Professional" else Advertiser.objects.get(
            user=self.context['user'], is_deleted=False)
        validated_data.pop("mode")

        if "telephone_numbers" in validated_data:
            telephone_numbers = []

            for i in validated_data['telephone_numbers']:
                if i == "" or i is None:
                    pass
                else:
                    telephone_numbers.append(i)

            validated_data.pop("telephone_numbers")

        validated_data.pop('telephone')
        validated_data.pop('telephone2')

        contact_info = ContactInfo.objects.create(**validated_data)

        if telephone_numbers:
            TelephoneNumbers.objects.filter(
                contact_id=contact_info.id).delete()
            data = [TelephoneNumbers(
                contact_id=contact_info,
                contact_number=i
            ) for i in telephone_numbers]
            TelephoneNumbers.objects.bulk_create(data)

        # Save User Info
        user = UserContactSerializer(
            instance=self.context['user'], data=validated_data)
        if user.is_valid():
            user.save()

        if validated_data['mode'] == "Professional":
            prof = Professional.objects.get(user=self.context['user'])
            print(prof)
            prof.first_name = instance.first_name
            prof.last_name = instance.last_name
            prof.gender = instance.gender
            prof.save()

        elif validated_data['mode'] == "Advertiser":
            adv = Advertiser.objects.get(user=self.context['user'])
            print(adv)
            adv.first_name = instance.first_name
            adv.last_name = instance.last_name
            adv.gender = instance.gender
            adv.save()

        elif validated_data['mode'] == "Player":
            playerr = Player.objects.get(user=self.context['user'])
            print(playerr)
            playerr.first_name = instance.first_name
            playerr.last_name = instance.last_name
            playerr.gender = instance.gender
            playerr.save()

        else:
            vo = VenueOwner.objects.get(user=self.context['user'])
            print(vo)
            vo.first_name = instance.first_name
            vo.last_name = instance.last_name
            vo.gender = instance.gender
            vo.save()

        return contact_info

    class Meta:
        model = ContactInfo
        fields = "__all__"
        # fields  = ['first_name','last_name', 'email','telephone']

        read_only_fields = ['is_deleted']


class ContactInfoGETSerializer(serializers.ModelSerializer):
    mode = serializers.CharField(
        required=True, help_text="Advertiser,Professional,Venue Owner,Player", write_only=True)
    telephone = serializers.SerializerMethodField()
    # telephone1 = serializers.SerializerMethodField()
    field_id = serializers.CharField(
        required=False, help_text="ID for specific")

    def validate_id(self, value):
        if ContactInfo.objects.filter(id=value).exists():
            return value
        else:
            raise FileNotFoundError("No Data found")

    def __init__(self, *args, **kwargs):
        super(ContactInfoGETSerializer, self).__init__(*args, **kwargs)
        for field in self.fields:
            if field == 'mode' or field == "field_id":
                self.fields[field].read_only = False
            else:
                self.fields[field].read_only = True

    class Meta:
        model = ContactInfo
        fields = "__all__"

    def get_telephone(self, obj):
        data = TelephoneNumbers.objects.filter(
            contact_id=self.context['contact_id'])
        serializers = TelephoneNumbersSerializers(data, many=True)
        return serializers.data


class ContactDeleteSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(required=True)

    def validate_id(self, value):
        if ContactInfo.objects.filter(id=value, is_deleted=False).exists():
            return value
        else:
            raise serializers.FieldDoesNotExist("No Such Record Exists")

    class Meta:
        model = ContactInfo
        fields = ("id",)


# Contact person #####################3
class VenueOwnerContactInfoSerializer(serializers.ModelSerializer):
    class Meta:
        model = VenueOwner
        fields = ['telephone', 'telephone2']


class ProfessionalContactInfoSerializer(serializers.ModelSerializer):
    class Meta:
        model = Professional
        fields = ['telephone', 'telephone2']


class AdvertiserContactInfoSerializer(serializers.ModelSerializer):
    class Meta:
        model = Advertiser
        fields = ['telephone', 'telephone2']


class PlayerContactInfoSerializer(serializers.ModelSerializer):
    class Meta:
        model = Player
        fields = ['telephone', 'telephone2']


class ContactPersonPostSerializer(serializers.Serializer):
    contact_person = serializers.JSONField(
        required=False,
        help_text='[{"first_name":"rita","last_name":"dhage","email":"ritadhage@gmail.com","telephone": "134567" }]')
    mode = serializers.CharField(
        required=True, help_text="Advertiser,Professional,Venue Owner,Player", write_only=True)

    def create(self, validated_data, **kwargs):
        data_obj = VenueOwner.objects.filter(user=self.context['user']).first() if validated_data[
                                                                                       'mode'] == "Venue Owner" else Player.objects.filter(
            user=self.context['user']).first(
        ) if validated_data['mode'] == "Player" else Professional.objects.filter(user=self.context['user']).first() if \
            validated_data['mode'] == "Professional" else Advertiser.objects.filter(user=self.context['user']).first()
        print(data_obj)
        delete_obj = VenueOwner.objects.filter(user=self.context['user']) if validated_data[
                                                                                 'mode'] == "Venue Owner" else Player.objects.filter(
            user=self.context['user']) if validated_data['mode'] == "Player" else Professional.objects.filter(
            user=self.context['user']) if validated_data['mode'] == "Professional" else Advertiser.objects.filter(
            user=self.context['user'])

        delete_contentType = ContentType.objects.get_for_model(VenueOwner) if validated_data[
                                                                                  'mode'] == "Venue Owner" else ContentType.objects.get_for_model(
            Player) if validated_data['mode'] == "Player" else ContentType.objects.get_for_model(Professional) if \
            validated_data['mode'] == "Professional" else ContentType.objects.get_for_model(Advertiser)

        Contact_Person.objects.filter(
            content_type__pk=delete_contentType.id, object_id__in=delete_obj).delete()

        contact_person = [Contact_Person(
            first_name=i['first_name'],
            last_name=i['last_name'],
            email=i['email'],
            telephone=i['telephone'],
            content_object=data_obj
        )
            for i in validated_data['contact_person']
        ]
        Contact_Person.objects.bulk_create(contact_person)
        return contact_person


class ContactPersonSerializer(serializers.ModelSerializer):
    class Meta:
        model = Contact_Person
        fields = "__all__"
        # fields = ["first_name","last_name", "email","telephone"]
        # exclude = ["id","is_deleted","object_id","created_at","content_type"]


########  ONBOARD #####################

class OnBoardAPIGETSerializer(serializers.ModelSerializer):
    created_by = serializers.CharField(
        source='created_by.username', required=False)
    service = ProfessionSerializer(
        required=False, read_only=True, many=True)
    sports = SportSerializer(required=False, read_only=True, many=True)
    country = CountrySerializer(read_only=True)
    city = CityGetSerializer(read_only=True)
    state = StateGetSerializer(read_only=True)
    area = AreaGetSerializer(read_only=True)

    class Meta:
        model = OnBoardAPI
        fields = '__all__'


class OnBoardAPIPOSTSerializer(serializers.ModelSerializer):
    created_by = serializers.CharField(required=False)
    id = serializers.IntegerField(required=False)
    service = serializers.JSONField(required=False, help_text="[1,2,3]")
    sports = serializers.JSONField(required=False, help_text="[1,2,3]")
    #telephone = serializers.CharField(required=True)
    #telephone2 = serializers.CharField(required=True)
    #mobile = serializers.CharField(required=True)

    def validate_created_by(self, value):
        created_by = User.objects.get(username=value)
        return created_by

    class Meta:
        model = OnBoardAPI
        fields = '__all__'
        read_only_fields = ['status']


########  Profile Settings #####################


class ProfileSettingGETSerializer(serializers.ModelSerializer):
    mode = serializers.CharField(
        required=True, help_text="Advertiser,Professional,Venue Owner,Player", write_only=True)

    def __init__(self, *args, **kwargs):
        super(ProfileSettingGETSerializer, self).__init__(*args, **kwargs)
        for field in self.fields:
            if field == 'mode':
                self.fields[field].read_only = False
            else:
                self.fields[field].read_only = True

    class Meta:
        model = ProfileSetting
        fields = '__all__'


class ProfileSettingPOSTSerializer(serializers.ModelSerializer):
    '''
    [{"notification_name":"M","is_active":"True","from_date":"date"}]
    [{"notification_name":"M","is_active":"True","from_date":"2020-08-11T13:09:24.383554+05:30"},
    {"notification_name":"O","is_active":"True","from_date":"2020-08-11T13:09:24.383554+05:30"}]
    '''
    notification_data = serializers.JSONField(
        required=False, help_text="Send list of all the data")
    password = serializers.CharField(required=False)
    new_password = serializers.CharField(required=False)
    email = serializers.CharField(required=False)
    new_email = serializers.CharField(required=False)
    mode = serializers.CharField(
        required=True, help_text="Advertiser,Professional,Venue Owner,Player")

    def validate(self, obj):
        if 'new_email' in obj:
            if "email" not in obj:
                raise serializers.FieldDoesNotExist(
                    "No Previous Email Provided")

            if obj['mode'] == "Venue Owner":
                if VenueOwner.objects.filter(email=obj['email'], user=self.context['user']).exists():
                    pass
                else:
                    raise serializers.FieldDoesNotExist(
                        "Previous Email does not match")

            if obj['mode'] == "Professional":
                if Professional.objects.filter(email=obj['email'], user=self.context['user']).exists():
                    pass
                else:
                    raise serializers.FieldDoesNotExist(
                        "Previous Email does not match")

            if obj['mode'] == "Advertiser":
                if Advertiser.objects.filter(email=obj['email'], user=self.context['user']).exists():
                    pass
                else:
                    raise serializers.FieldDoesNotExist(
                        "Previous Email does not match")

            if obj['mode'] == "Player":
                if Player.objects.filter(email=obj['email'], user=self.context['user']).exists():
                    pass
                else:
                    raise serializers.FieldDoesNotExist(
                        "Previous Email does not match")

        return obj

    def create(self, validated_data, **kwargs):
        data = []

        if "new_email" in validated_data:
            venue_owner_obj = VenueOwner.objects.get(email=validated_data['email'], is_deleted=False) if validated_data[
                                                                                                             'mode'] == "Venue Owner" else Player.objects.get(
                email=validated_data['email'], is_deleted=False) if validated_data[
                                                                        'mode'] == "Player" else Professional.objects.get(
                email=validated_data['email'], is_deleted=False) if validated_data[
                                                                        'mode'] == "Professional" else Advertiser.objects.get(
                email=validated_data['email'], is_deleted=False)
            print(validated_data)
            venue_owner_obj.new_email = validated_data['new_email']
            venue_owner_obj.save()
            print("hello")

        if 'password' in validated_data:
            user = User.objects.get(username=str(self.context['user']))
            user.set_password(validated_data['new_password'])
            user.save()

        delete_obj = VenueOwner.objects.filter(user=self.context['user']) if validated_data[
                                                                                 'mode'] == "Venue Owner" else Player.objects.filter(
            user=self.context['user']) if validated_data['mode'] == "Player" else Professional.objects.filter(
            user=self.context['user']) if validated_data['mode'] == "Professional" else Advertiser.objects.filter(
            user=self.context['user'])

        delete_contentType = ContentType.objects.get_for_model(VenueOwner) if validated_data[
                                                                                  'mode'] == "Venue Owner" else ContentType.objects.get_for_model(
            Player) if validated_data['mode'] == "Player" else ContentType.objects.get_for_model(Professional) if \
            validated_data['mode'] == "Professional" else ContentType.objects.get_for_model(Advertiser)

        ProfileSetting.objects.filter(
            content_type__pk=delete_contentType.id, object_id__in=delete_obj).delete()

        data = [ProfileSetting(
            **i,
            content_object=VenueOwner.objects.get(user=self.context['user']) if validated_data['mode'] == "Venue Owner"
            else Player.objects.get(user=self.context['user']) if validated_data['mode'] == "Player"
            else Professional.objects.get(user=self.context['user']) if validated_data['mode'] == "Professional"
            else Advertiser.objects.get(user=self.context['user'])
        ) for i in validated_data['notification_data']]

        ProfileSetting.objects.bulk_create(data)
        return "Success"

    class Meta:
        model = ProfileSetting
        fields = ['notification_data', 'password',
                  'email', 'mode', 'new_email', 'new_password']


class UpdateUserSerializer(serializers.ModelSerializer):
    email = serializers.EmailField(required=True)
    mode = serializers.CharField(
        required=True, help_text="Advertiser,Professional,Venue Owner,Player")

    def validate_email(self, value):
        user = self.context['request'].user
        if User.objects.exclude(pk=user.pk).filter(email=value).exists():
            raise serializers.ValidationError(
                {"email": "This email is already in use."})
        return value

    def create(self, validated_data):

        if "email" in validated_data:
            email = self.context['request'].user.email
            new_email = validated_data['email']
            if validated_data['mode'] == "Venue Owner":
                data_obj = VenueOwner.objects.get(
                    user=self.context['request'].user)
            elif validated_data['mode'] == "Professional":
                data_obj = Professional.objects.get(
                    user=self.context['request'].user)
            elif validated_data['mode'] == "Advertiser":
                data_obj = Advertiser.objects.get(
                    user=self.context['request'].user)
            else:
                data_obj = Player.objects.get(
                    user=self.context['request'].user)

            data_obj.new_email = new_email
            data_obj.save()
            # user_obj = User.objects.get(email=email)
            # user_obj.email = new_email
            # user_obj.save()
        return data_obj

    class Meta:
        model = User
        fields = ['email', 'first_name', 'mode']


######### PRofile serializers ###################


class ContactInfoProfileSerializer(serializers.ModelSerializer):

    def create(self, validated_data):
        content_obj = validated_data['content_object']
        validated_data.pop("content_object")
        obj, created = ContactInfo.objects.update_or_create(
            **validated_data, defaults={'email': content_obj.email})
        obj.content_object = content_obj
        obj.save()
        return obj

    class Meta:
        model = ContactInfo
        fields = '__all__'


class BankInfoProfileSerializer(serializers.ModelSerializer):
    class Meta:
        model = BankInfo
        fields = '__all__'

        #######Contract Terms#########


class SaveContractTermsSerializer(serializers.ModelSerializer):
    user_id = serializers.IntegerField(required=False)
    mode = serializers.CharField(
        required=True, help_text="Advertiser,VenueOwner,Professional")
    cancellation_policy = serializers.JSONField(
        write_only=True, required=False, help_text="[1,2,3]")

    def validate_cancellation_policy(self, value):
        for item in value:
            if not CancellationPolicy.objects.filter(id=item).exists():
                raise serializers.FieldDoesNotExist(
                    "No Record with given ID in cancellation_policy")
        return value

    class Meta:
        model = ContractTerms
        fields = '__all__'
        read_only_fields = ['created_for']


class DeleteContractTermDocumentSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(
        required=True, help_text="Id of Contract Term")

    class Meta:
        model = ContractTerms
        fields = "__all__"
        read_only_fields = ['commission_fee', 'payments', 'collaboration_name', 'contract_terms_document',
                            'cancellation_policy', 'salesman', 'supervisor', 'created_for']


class CancellationPolicyGetAllWithoutPageSerializer1(serializers.ModelSerializer):
    class Meta:
        model = CancellationPolicy
        fields = '__all__'


class GetSingleContractTermsSerializer(serializers.Serializer):
    user_id = serializers.IntegerField(required=False)
    mode = serializers.CharField(
        required=True, help_text="Advertiser,VenueOwner,Professional")


class GetSingleContractTermsSerializer1(serializers.ModelSerializer):
    salesman = SalesmanSerializer(read_only=True)
    cancellation_policy = CancellationPolicyGetAllWithoutPageSerializer1(
        read_only=True, many=True)

    class Meta:
        model = ContractTerms
        fields = "__all__"


class EditContractTermsSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(
        required=False, help_text="Id of Contract Term")
    cancellation_policy = serializers.JSONField(
        write_only=True, required=False, help_text="[1,2,3]")

    class Meta:
        model = ContractTerms
        fields = "__all__"
        read_only_fields = ['created_for']


class PlatformUpdatePostSerializer(serializers.ModelSerializer):
    class Meta:
        model = Platform_Update
        fields = '__all__'
