import pytz
import json
from django.http import HttpResponse
from django.template import Template, Context
from django.template.loader import get_template
from django.db import transaction
from django.urls import reverse

from django.contrib.auth import login as django_login
import datetime
from rest_framework.authtoken.models import Token
from rest_framework import status, generics
from rest_framework.response import Response
from rest_framework.authentication import TokenAuthentication
from rest_framework.permissions import IsAuthenticated
from utils.mixins import PlatformAPICallMixin
from utils.api_handle_errors import handle_errors

from commonAPI.serializers import *
from commonAPI.models import *
from venues.models import Venue


class BankAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    '''
    Bank Information
    '''
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = BankInfoSerializer

    def post(self, request, format=None):
        '''
        Get bank Information
        '''
        response = {}
        responsedata = []
        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():

            if 'field_id' in serializers.validated_data:
                data = BankInfo.objects.filter(
                    id=serializers.validated_data['field_id'])
            else:
                data_obj = VenueOwner.objects.filter(user=request.user) if serializers.validated_data['mode'] == "Venue Owner" else Player.objects.filter(
                    user=request.user) if serializers.validated_data['mode'] == "Player" else Professional.objects.filter(user=request.user) if serializers.validated_data['mode'] == "Professional" else Advertiser.objects.filter(user=request.user)

                data_contentType = ContentType.objects.get_for_model(VenueOwner) if serializers.validated_data['mode'] == "Venue Owner" else ContentType.objects.get_for_model(
                    Player) if serializers.validated_data['mode'] == "Player" else ContentType.objects.get_for_model(Professional) if serializers.validated_data['mode'] == "Professional" else ContentType.objects.get_for_model(Advertiser)

                data = BankInfo.objects.filter(
                    content_type__pk=data_contentType.id, object_id__in=data_obj, is_deleted=False)

                data_documents = Document.objects.filter(
                    created_by=request.user)
        serializer = self.serializer_class(data, many=True)
        response['result'] = 1
        response['data'] = serializer.data
        response['documents'] = data_documents.values()
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


class EditBankAPIView (PlatformAPICallMixin, generics.GenericAPIView):
    '''
    Edit Bank Information
    '''
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = BankInfoEDITSerializer

    def post(self, request, format=None):
        '''update Bank
        '''
        response = {}
        responsedata = []
        request.data._mutable = True
        request.data['created_by'] = User.objects.get(username=request.user).id
        serializer = self.serializer_class(data=request.data)
        if serializer.is_valid():
            if BankInfo.objects.filter(id=serializer.validated_data['id'], is_deleted=False).exists():
                serializer = self.serializer_class(instance=BankInfo.objects.filter(
                    id=serializer.validated_data['id'], is_deleted=False).first(), data=request.data)
                if serializer.is_valid():
                    serializer.save()
                    responsedata.append(
                        "Bank Information Updated Successfully")
                    response['result'] = 1
                    response['errors'] = responsedata
                    self.add_api_call(request, response)
                    return Response(response, status=status.HTTP_200_OK)
                else:
                    response = handle_errors(serializer.errors)
                    response['result'] = 0
                    self.add_api_call(request, response)
                    return Response(response, status=status.HTTP_200_OK)
            else:
                response['data'] = "No Data Found"
                response['result'] = 0
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response['result'] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class AddBankAPIView (PlatformAPICallMixin, generics.GenericAPIView):
    '''
    Add Bank Information
    '''
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = BankInfoPOSTSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []
        serializers = self.serializer_class(
            data=request.data, context={'user': request.user})
        if serializers.is_valid():
            serializers.save()
            responsedata.append("Added Successfully")
            response['result'] = 1
            response['errors'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializers.errors)
            response['result'] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class BankDeleteAPIView (PlatformAPICallMixin, generics.GenericAPIView):
    '''
    Bank Delete Information
    '''
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = BankDeleteSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []
        serializer = self.serializer_class(data=request.data)
        if serializer.is_valid():
            if BankInfo.objects.filter(id=serializer.data['id'], is_deleted=False).exists():
                data = BankInfo.objects.filter(
                    id=serializer.data['id'], is_deleted=False).first()
                data.is_deleted = True
                data.save(update_fields=['is_deleted'])
                response['result'] = 1
                responsedata.append("Successfully Deleted")
                response['errors'] = responsedata
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)
            else:
                response['result'] = 0
                responsedata.append("No Record Found")
                response['errors'] = responsedata
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_204_NO_CONTENT)
        else:
            response = handle_errors(serializer.errors)
            response['result'] = 1
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
