import pytz
import json
from django.http import HttpResponse
from django.template import Template, Context
from django.contrib.auth.models import User, Group
from django.template.loader import get_template
from django.db import transaction
from django.urls import reverse
from django.contrib.auth import login, authenticate, logout

from django.contrib.auth import login as django_login
import datetime

from rest_framework.authtoken.models import Token
from rest_framework import status, generics
from rest_framework.response import Response
from rest_framework.authentication import TokenAuthentication
from rest_framework.permissions import IsAuthenticated
import requests
from utils.mixins import PlatformAPICallMixin
from utils.api_handle_errors import handle_errors
from django.utils import timezone
from utils.mail import platform_thank_you_email
from commonAPI.serializers import *
from commonAPI.models import *
from platform_auth.serializers import GroupSerializer
from platform_central.models import RegisterPrivateKey

from commonAPI.models import OnBoardAPI
from platform_auth.serializers import OnBoardAPISerializer


class SettingAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    '''
    Change Password // Settings
    '''
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = ChangePassword

    def post(self, request, format=None):
        response = {}
        errors = []
        serializers = ChangePassword(data=request.data)
        if serializers.is_valid():
            serializers.save(user=str(request.user))
            response["result"] = 1
            errors.append("Successfully Changed the password")
            response["errors"] = errors
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response["result"] = 1
            response = handle_errors(serializers.errors)
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class DocumentAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    '''
    Upload Documents
    '''
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = DocumentSerializer

    def post(self, request, format=None):
        response = {}
        errors = []
        request.data._mutable = True
        request.data['created_by'] = User.objects.get(username=request.user).id
        print(request.data)
        serializer = self.serializer_class(data=request.data)
        if serializer.is_valid():

            serializer.save()
            response['result'] = 1
            errors.append("Successfully Inserted")
            response['errors'] = errors
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response['result'] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

    def get(self, request, format=None):
        response = {}
        errors = []
        data = Document.objects.filter(
            created_by=request.user, is_deleted=False)
        serializer = self.serializer_class(data, many=True)
        response['result'] = 1
        response['data'] = serializer.data
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


class DocumentDeleteAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    '''
    Delete Documents
    '''
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = DocumentDeleteSerializer

    def post(self, request, format=None):
        responsedata = []
        response = {}
        serializer = self.serializer_class(data=request.data)
        if serializer.is_valid():
            if Document.objects.filter(id=serializer.data['id'], created_by=request.user, is_deleted=False).exists():
                data = Document.objects.filter(id=serializer.data['id'], created_by=request.user,
                                               is_deleted=False).first()
                data.is_deleted = True
                data.save(update_fields=['is_deleted'])
                responsedata.append("Successfully deleted")
                response['result'] = 0
                response['errors'] = responsedata
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)
            else:
                responsedata.append("No Record Found")
                response['result'] = 1
                response['errors'] = responsedata
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors())
            response['result'] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class ChangePasswordAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    """
    Change Password after admin approval
    """
    serializer_class = ChangePasswordSerializers

    def post(self, request, format=None):
        response = {}
        responsedata = []
        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            if User.objects.filter(email=serializers.validated_data['username'], is_active=False).exists():
                user = User.objects.get(
                    email=serializers.validated_data['username'], is_active=False)

                # Check Token
                if RegisterPrivateKey.objects.filter(key=serializers.validated_data['key'], user=user).exists():
                    user.set_password(serializers.validated_data['password'])
                    user.is_active = True
                    user.save()

                    registeration_key = RegisterPrivateKey.objects.get(
                        key=serializers.validated_data['key'], user=user)
                    registeration_key.is_active = False
                    registeration_key.activated_at = timezone.now()
                    registeration_key.save()

                else:
                    responsedata.append("No Valid Token Found")
                    response['result'] = 0
                    response['errors'] = responsedata
                    self.add_api_call(request, response)
                    return Response(response, status=status.HTTP_200_OK)

                onboard_obj = OnBoardAPI.objects.filter(
                    email=user.username).last()
                onboard_serializer = OnBoardAPISerializer(onboard_obj)
                if onboard_serializer.data['status'] == 'A':
                    flag = True
                    response['flag'] = flag
                elif onboard_serializer.data['status'] == 'C':
                    flag = False
                    response['flag'] = flag
            else:
                responsedata.append("The account is already active")
                response['result'] = 0
                response['errors'] = responsedata
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)

            # login
            # login(request,user)
            login(request, user, backend='django.contrib.auth.backends.ModelBackend')
            token = Token.objects.update_or_create(user=user)

            # Get Groups
            groups = Group.objects.filter(user=user)
            group_class = GroupSerializer(groups, many=True)

            responsedata.append("Password Changed Successfully")
            response['result'] = 1
            response['errors'] = responsedata
            response['name'] = user.first_name
            response['user_type'] = group_class.data
            response['token'] = str(token[0])
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializers.errors())
            response['result'] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class ContractTermsSaveInfoAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = SaveContractTermsSerializer

    def post(self, request, format=None):
        try:
            response = {}
            responsedata = []
            request.data._mutable = True
            print(request.data)
            if request.data['mode'] == "Advertiser":
                user_obj = Advertiser.objects.get(id=request.data['user_id'])
            elif request.data['mode'] == "VenueOwner":
                user_obj = VenueOwner.objects.get(id=request.data['user_id'])
            elif request.data['mode'] == "Professional":
                user_obj = Professional.objects.get(id=request.data['user_id'])

            a = request.data.pop('user_id')
            serializers = self.serializer_class(data=request.data)
            if serializers.is_valid():
                serializers.validated_data.pop('mode')
                serializers.validated_data['created_for'] = user_obj.user

                ContractTerms_obj = serializers.save()
                # Sending Response
                response["result"] = 1
                responsedata.append("Successfully Inserted")
                response['data'] = responsedata
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)
            else:
                response = handle_errors(serializers.errors)
                response["result"] = 0
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_400_BAD_REQUEST)
        except Exception as e:
            response["result"] = 0
            responsedata.append(str(e))
            response['errors'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class DeleteContractTermDocumentAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = DeleteContractTermDocumentSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []

        if not ContractTerms.objects.filter(id=request.data['id']).exists():
            response['results'] = 0
            response['error'] = "Not a valid ID"
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        serializers = self.serializer_class(data=request.data)

        if serializers.is_valid():
            ContractTerms_obj = ContractTerms.objects.get(
                id=serializers.validated_data['id'])
            if ContractTerms_obj.contract_terms_document:
                print('ContractTerms_obj before delete---', ContractTerms_obj)
                a = ContractTerms_obj.contract_terms_document.delete()
                ContractTerms_obj.save()
            else:
                response['results'] = 1
                response['error'] = "File Not Found"
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)

            response['results'] = 1
            response['data'] = "Successfully Deleted"
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        else:
            response['results'] = 0
            response = handle_errors(serializers.errors)
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class GetSingleContractTermsAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = GetSingleContractTermsSerializer

    def post(self, request, format=None):
        '''
        Get Contract term by mode Id
        '''
        response = {}
        responsedata = []
        request.data._mutable = True
        print(request.data)

        if request.data['mode'] == "Advertiser":
            user_obj = Advertiser.objects.get(id=request.data['user_id'])
        elif request.data['mode'] == "VenueOwner":
            user_obj = VenueOwner.objects.get(id=request.data['user_id'])
        elif request.data['mode'] == "Professional":
            user_obj = Professional.objects.get(id=request.data['user_id'])

        a = request.data.pop('user_id')
        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            serializers.validated_data.pop('mode')
            if ContractTerms.objects.filter(created_for=user_obj.user).exists():
                contractTerm_obj = ContractTerms.objects.filter(
                    created_for=user_obj.user)
                serializer = GetSingleContractTermsSerializer1(
                    contractTerm_obj, many=True)
                doc_list = []
                list = []
                for i in contractTerm_obj:
                    for j in serializer.data:
                        dict = {}
                        contract_terms_document_sep = str(j['contract_terms_document']).replace(
                            str(j['contract_terms_document']), str(i.contract_terms_document))
                        doc_list.append(contract_terms_document_sep)
                        dict['ContractTerms'] = j
                        dict['documents'] = contract_terms_document_sep if contract_terms_document_sep == '' else \
                            contract_terms_document_sep.split('/')[1]
                        list.append(dict)
                        break

            else:
                response["result"] = 0
                response['data'] = "Invalid contract term id"
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)

            response["result"] = 1
            response['data'] = list
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class ContractTermsEditInfoAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = EditContractTermsSerializer

    def post(self, request, format=None):
        '''
        Update ContractTerms
        '''
        responsedata = []
        response = {}

        if ContractTerms.objects.filter(id=request.data['id']).exists():
            serializer = self.serializer_class(instance=ContractTerms.objects.get(
                id=request.data['id']), data=request.data)
            message = "Updated Successfully"
        else:
            response["result"] = 0
            response['data'] = "Invalid ContractTerms id"
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        if serializer.is_valid():
            print(serializer.validated_data)
            serializer.save()
            response["result"] = 1
            response['data'] = [message]
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class GetAllContractTermsVenueOwnerAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = GetSingleContractTermsSerializer1

    def get(self, request, format=None):
        '''
        Get Contract term 
        '''
        response = {}
        responsedata = []
        #venuowner_user = {"user": request.user}
        #user_obj = VenueOwner.objects.get(user=venuowner_user['user'])
        if ContractTerms.objects.filter(created_for=request.user).exists():
            contractTerm_obj = ContractTerms.objects.filter(
                created_for=request.user)
            serializer = self.serializer_class(contractTerm_obj, many=True)
            response["result"] = 1
            response['data'] = serializer.data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response["result"] = 0
            response['data'] = "No Contract Terms available"
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class PlatformUpdatePostAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    # authentication_classes = (TokenAuthentication,)
    # permission_classes = (IsAuthenticated,)
    serializer_class = PlatformUpdatePostSerializer

    def post(self, request, format=None):
        '''
        Add Platform updates
        '''
        responsedata = []
        response = {}

        # if Platform_Update.objects.filter(email=request.data['email']).exists():
        #     message = "Email already exists"
        #     response["result"] = 0
        #     response['data'] = [message]
        #     self.add_api_call(request, response)
        #     return Response(response, status=status.HTTP_200_OK)

        # else:
        #     serializer = self.serializer_class(data=request.data)
        #     message = "Created Successfully"
        serializer = self.serializer_class(data=request.data)
        if serializer.is_valid():

            url = "https://a.klaviyo.com/api/v2/list/SkNNVP/members"
            querystring = {"api_key": "pk_35e92df2b14fa6852c81d573044eafde7f"}
            payload = {"profiles": [
                {"email": serializer.validated_data['email']}]}
            headers = {
                "Accept": "application/json",
                "Content-Type": "application/json"
            }
            responseklaviyo = requests.request(
                "POST", url, json=payload, headers=headers, params=querystring)

            # platform_thank_you_email(serializer.validated_data['email'])
            # serializer.save()
            response["result"] = 1
            response['data'] = 'Success'
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)
