import pytz
import json
from django.http import HttpResponse
from django.template import Template, Context
from django.template.loader import get_template
from django.db import transaction
from django.urls import reverse

from django.contrib.auth import login as django_login
import datetime
from rest_framework.authtoken.models import Token
from rest_framework import status, generics
from rest_framework.response import Response
from rest_framework.authentication import TokenAuthentication
from rest_framework.permissions import IsAuthenticated
from utils.mixins import PlatformAPICallMixin
from utils.api_handle_errors import handle_errors

from commonAPI.serializers import *
from commonAPI.models import *

from venue_owners.models import VenueOwner
from players.models import Player
from professionals.models import Professional
from advertisers.models import Advertiser
import requests


class CountryApiView(PlatformAPICallMixin, generics.GenericAPIView):
    serializer_class = CountryApiserializer

    def post(self, request, format=None):
        '''
        CountryApiView  POST
        '''
        response = {}
        responsedata = []
        country_data = []
        keyword = request.data['keyword']

        url = "https://shivammathur.com/countrycity/countries"
        result = requests.get(url)
        r = result.json()
        for country in r:
            if keyword.lower() in country.lower():
                country_data.append(country)

        response['result'] = 1
        responsedata.append(country_data)
        response['data'] = responsedata
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


class CityApiView(PlatformAPICallMixin, generics.GenericAPIView):
    serializer_class = CityApiserializer

    def post(self, request, format=None):
        '''
        CityApiView  POST
        '''
        response = {}
        responsedata = []
        city_data = []
        keyword = request.data['keyword']
        country = request.data['country']
        url = "https://shivammathur.com/countrycity/cities/{}".format(country.lower())
        result = requests.get(url)
        res = result.json()
        for city in res:
            if keyword.lower() in city.lower():
                city_data.append(city)

        response['result'] = 1
        responsedata.append(city_data)
        response['data'] = responsedata
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


class ContactGETAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = ContactInfoGETSerializer
    contact_person_serializer = ContactPersonSerializer

    # Get Field Info
    def post(self, request, format=None):
        '''
        GET Contact Information
        '''

        responsedata = []
        response = {}
        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            if 'field_id' in serializers.validated_data:
                FieldInfo = ContactInfo.objects.filter(id=serializers.validated_data['field_id']).first()
            else:
                data_obj = VenueOwner.objects.filter(user=request.user) if serializers.validated_data[
                                                                               'mode'] == "Venue Owner" else Player.objects.filter(
                    user=request.user) if serializers.validated_data[
                                              'mode'] == "Player" else Professional.objects.filter(user=request.user) if \
                    serializers.validated_data['mode'] == "Professional" else Advertiser.objects.filter(
                    user=request.user)
                print('data_obj--->>', data_obj)

                data_contentType = ContentType.objects.get_for_model(VenueOwner) if serializers.validated_data[
                                                                                        'mode'] == "Venue Owner" else ContentType.objects.get_for_model(
                    Player) if serializers.validated_data['mode'] == "Player" else ContentType.objects.get_for_model(
                    Professional) if serializers.validated_data[
                                         'mode'] == "Professional" else ContentType.objects.get_for_model(Advertiser)

                print('data_contentType--->>', data_contentType)
                print('data_contentType--->>', data_contentType.id)

                FieldInfo = ContactInfo.objects.filter(content_type__pk=data_contentType.id, object_id__in=data_obj,
                                                       is_deleted=False).first()
                print('FieldInfo 1---->>', FieldInfo)

                content_type = ContentType.objects.get_for_model(Professional)
                contact_person = Contact_Person.objects.filter(content_type__pk=data_contentType.id,
                                                               object_id__in=data_obj)
                contact_person_serializer = ContactPersonSerializer(contact_person, many=True)
                if serializers.validated_data['mode'] == "Venue Owner":
                    telephone_serializer = VenueOwnerContactInfoSerializer(data_obj,many=True)
                elif serializers.validated_data['mode'] == "Professional":
                    print('---prof---')
                    telephone_serializer = ProfessionalContactInfoSerializer(data_obj, many=True)
                elif serializers.validated_data['mode'] == "Player":
                    print('---player---')
                    telephone_serializer = PlayerContactInfoSerializer(data_obj, many=True)
                elif serializers.validated_data['mode'] == "Advertiser":
                    print('---advr---')
                    telephone_serializer = AdvertiserContactInfoSerializer(data_obj, many=True)

            if FieldInfo is None:
                response['result'] = 1
                response['data'] = responsedata
                # response["contact_person"] = contact_person_serializer.data
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)

            serializers = self.serializer_class(FieldInfo, many=False, context={'contact_id': FieldInfo.id})

            response['result'] = 1
            response['data'] = serializers.data
            response["Telephone_Details"] = telephone_serializer.data
            response["contact_person"] = contact_person_serializer.data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializers.errors)
            response['result'] = 0
            return Response(response, status=status.HTTP_200_OK)


class ContactAddEditAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = ContactInfoSerializer

    def post(self, request, format=None):
        '''
        Post request for Updating the Contact Information field
        
        '''
        responsedata = []
        response = {}
        request.data._mutable = True
        request.data['created_by'] = User.objects.get(username=request.user).id

        data_obj = VenueOwner.objects.filter(user=request.user) if request.data[
                                                                       'mode'] == "Venue Owner" else Player.objects.filter(
            user=request.user) if request.data['mode'] == "Player" else Professional.objects.filter(
            user=request.user) if request.data['mode'] == "Professional" else Advertiser.objects.filter(
            user=request.user)
        data_contentType = ContentType.objects.get_for_model(VenueOwner) if request.data[
                                                                                'mode'] == "Venue Owner" else ContentType.objects.get_for_model(
            Player) if request.data['mode'] == "Player" else ContentType.objects.get_for_model(Professional) if \
            request.data['mode'] == "Professional" else ContentType.objects.get_for_model(Advertiser)

        if ContactInfo.objects.filter(content_type__pk=data_contentType.id, object_id__in=data_obj,
                                      is_deleted=False).exists():
            serializer = self.serializer_class(
                instance=ContactInfo.objects.filter(content_type__pk=data_contentType.id, object_id__in=data_obj,
                                                    is_deleted=False).first(), data=request.data,
                context={"user": request.user})
            message = "Updated Successfully"
        else:
            serializer = self.serializer_class(data=request.data, context={"user": request.user})
            message = "Created Successfully"

        if serializer.is_valid():
            data = serializer.save()
            # Get Name
            user_obj = request.user

            response['errors'] = ["Updated Successfully"]
            response["result"] = 1
            response["first_name"] = user_obj.first_name
            return Response(response, status=status.HTTP_200_OK)
        else:
            response['result'] = 0
            response = handle_errors(serializer.errors)
            return Response(response, status=status.HTTP_200_OK)

    '''
    Add contact person  Information
    '''


class AddContactPersonAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = ContactPersonPostSerializer

    def post(self, request, format=None):

        responsedata = []
        response = {}
        request.data._mutable = True
        request.data['created_by'] = User.objects.get(username=request.user).id

        data_obj = VenueOwner.objects.filter(user=request.user) if request.data[
                                                                       'mode'] == "Venue Owner" else Player.objects.filter(
            user=request.user) if request.data['mode'] == "Player" else Professional.objects.filter(
            user=request.user) if request.data['mode'] == "Professional" else Advertiser.objects.filter(
            user=request.user)
        data_contentType = ContentType.objects.get_for_model(VenueOwner) if request.data[
                                                                                'mode'] == "Venue Owner" else ContentType.objects.get_for_model(
            Player) if request.data['mode'] == "Player" else ContentType.objects.get_for_model(Professional) if \
            request.data['mode'] == "Professional" else ContentType.objects.get_for_model(Advertiser)
        serializer = self.serializer_class(data=request.data, context={"user": request.user})

        if serializer.is_valid():
            data = serializer.save()
            print(data)
            responsedata.append("Added Successfully")
            response['result'] = 1
            response['errors'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response['result'] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class DeleteContactAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = ContactDeleteSerializer

    def post(self, request, format=None):

        response = {}
        responsedata = []
        serializer = self.serializer_class(data=request.data)
        if serializer.is_valid():
            data = ContactInfo.objects.filter(id=serializer.data['id'], is_deleted=False).first()
            data.is_deleted = True
            data.save(update_fields=['is_deleted'])
            response['result'] = 1
            responsedata.append("Successfully Deleted")
            response['errors'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response['result'] = 1
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
