import pytz
import json
from django.http import HttpResponse
from django.template import Template, Context
from django.template.loader import get_template
from django.db import transaction
from django.urls import reverse

from django.contrib.auth import login as django_login
import datetime
from rest_framework.authtoken.models import Token
from rest_framework import status, generics
from rest_framework.response import Response
from rest_framework.authentication import TokenAuthentication
from rest_framework.permissions import IsAuthenticated
from utils.mixins import PlatformAPICallMixin
from utils.api_handle_errors import handle_errors
from utils.mail import thank_you_email
from commonAPI.serializers import *
from commonAPI.models import OnBoardAPI, User
from django.db.models import Q
from static_data.serializers import ProfessionSerializer
from static_data.models import Profession

from professionals.models import Professional
from advertisers.models import Advertiser
from venues.models import Venue


class OnboardAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = OnBoardAPIGETSerializer

    # Get ALL Onboard Form
    def get(self, request, format=None):
        '''
        GET Onboard Registeration form
        '''

        responsedata = []
        response = {}
        FieldInfo = OnBoardAPI.objects.filter(
            is_deleted=False, created_by=User.objects.get(username=request.user))

        serializers = self.serializer_class(FieldInfo, many=True)
        resposedata = serializers.data
        response['result'] = 1
        response['data'] = resposedata
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


class OnboardUpdateAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    serializer_class = OnBoardAPIPOSTSerializer

    def post(self, request, format=None):
        '''
        ADD / UPDATE Specific User Registeration Form
        '''

        responsedata = []
        response = {}
        if 'id' in request.data:
            data = OnBoardAPI.objects.filter(id=request.data['id']).first()
            if data:
                serializers = self.serializer_class(
                    instance=data, data=request.data)
                if serializers.is_valid():

                    all_data = OnBoardAPI.objects.all()
                    professional_data = Professional.objects.all()
                    advertiser_data = Advertiser.objects.all()
                    venue_data = Venue.objects.all()

                    business_name_list = []
                    for item in all_data:
                        business_name_list.append(item.business_name)

                    for item in professional_data:
                        business_name_list.append(item.business_name)

                    for item in advertiser_data:
                        business_name_list.append(item.business_name)

                    for item in venue_data:
                        business_name_list.append(item.business_name)

                    # if serializers.validated_data['business_name'] in business_name_list:
                    #     response["result"] = 0
                    #     response["error"] = "Business name already exist"
                    #     self.add_api_call(request, response)
                    #     return Response(response, status=status.HTTP_200_OK)
                    #
                    # else:
                        # serializers.validated_data['created_by'] = request.user
                    serializers.save()
                    response['result'] = 1
                    responsedata.append("Updated Successfully")
                    response['data'] = responsedata
                    self.add_api_call(request, response)
                    return Response(response, status=status.HTTP_200_OK)
                else:
                    response = handle_errors(serializers.errors)
                    response["result"] = 0
                    self.add_api_call(request, response)
                    return Response(response, status=status.HTTP_400_BAD_REQUEST)
            else:
                response["result"] = 0
                responsedata.append("No such Record")
                response['errors'] = responsedata
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)
        else:
            if User.objects.filter(username=request.data['email']).exists():

                # 333
                whitelist = ['lanoisseforptest1@gmail.com',
                             'lanoisseforptest2@gmail.com', 'lanoisseforptest3@gmail.com']

                if request.data['email'] in whitelist:
                    pass
                else:

                    # 3

                    response["result"] = 0
                    responsedata.append(
                        "This email address is already registered")
                    response['errors'] = responsedata
                    self.add_api_call(request, response)
                    return Response(response, status=status.HTTP_200_OK)

            serializers = self.serializer_class(data=request.data)
            if serializers.is_valid():

                all_data = OnBoardAPI.objects.all()
                professional_data = Professional.objects.all()
                advertiser_data = Advertiser.objects.all()
                venue_data = Venue.objects.all()

                images = Template_Images.objects.filter().values('template_image')
                logo = images[0]
                logo2 = images[1]

                business_name_list = []
                for item in all_data:
                    business_name_list.append(item.business_name)

                for item in professional_data:
                    business_name_list.append(item.business_name)

                for item in advertiser_data:
                    business_name_list.append(item.business_name)

                for item in venue_data:
                    business_name_list.append(item.business_name)

                # if serializers.validated_data['business_name'] in business_name_list:
                #     response["result"] = 0
                #     response["error"] = "Business name already exist"
                #     self.add_api_call(request, response)
                #     return Response(response, status=status.HTTP_200_OK)
                #
                # else:
                #     # serializers.validated_data['created_by'] = request.user
                serializers.save()

                # send email
                thank_you_email(
                    request.data['first_name'], request.data['email'], logo, logo2)

                response['result'] = 1
                responsedata.append("Added Successfully")
                response['data'] = responsedata
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)
            else:
                response = handle_errors(serializers.errors)
                response["result"] = 0
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_400_BAD_REQUEST)
