import pytz
import json
from django.http import HttpResponse
from django.template import Template, Context
from django.template.loader import get_template
from django.db import transaction
from django.urls import reverse
from django.contrib.auth import password_validation
from django.http import Http404
from django.utils.http import urlsafe_base64_encode, urlsafe_base64_decode
from django.contrib.sites.shortcuts import get_current_site
from django.contrib.auth import login as django_login
from django.utils.encoding import force_bytes
import datetime
from rest_framework.authtoken.models import Token
from rest_framework import status, generics
from rest_framework.response import Response
from rest_framework.authentication import TokenAuthentication
from rest_framework.permissions import IsAuthenticated
from utils.mixins import PlatformAPICallMixin
from utils.api_handle_errors import handle_errors
from utils.mail import email_verification
from commonAPI.serializers import *
from commonAPI.models import *
from django.contrib.auth.models import User
from django.core.mail import send_mail

from platform_central.models import UserToken
# from field_booking.functions import send_sms
from field_booking.auth_backend import EmailBackend
from field_booking.settings import DOMAIN_NAME


from venue_owners.models import VenueOwner
from players.models import Player
from professionals.models import Professional
from django.contrib.auth import authenticate
from advertisers.models import Advertiser


class ProfileSettingAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = ProfileSettingGETSerializer

    def post(self, request, format=None):
        """
        User Type Profile Settings
        """

        response = {}
        responsedata = []

        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():

            # Content Type Data
            data_obj = VenueOwner.objects.filter(user=request.user) if serializers.validated_data['mode'] == "Venue Owner" else Player.objects.filter(
                user=request.user) if serializers.validated_data['mode'] == "Player" else Professional.objects.filter(user=request.user) if serializers.validated_data['mode'] == "Professional" else Advertiser.objects.filter(user=request.user)
            data_contentType = ContentType.objects.get_for_model(VenueOwner) if serializers.validated_data['mode'] == "Venue Owner" else ContentType.objects.get_for_model(
                Player) if serializers.validated_data['mode'] == "Player" else ContentType.objects.get_for_model(Professional) if serializers.validated_data['mode'] == "Professional" else ContentType.objects.get_for_model(Advertiser)
            data = ProfileSetting.objects.filter(
                content_type__pk=data_contentType.id, object_id__in=data_obj)

            serializer = self.serializer_class(data, many=True)

            # Get email
            try:
                email_obj = data_obj.email
                #password_obj = data_obj.password
            except:
                email_obj = request.user.email
                #password_obj = request.user.password

            response['result'] = 1
            response['data'] = serializer.data
            response['email'] = email_obj
            #response['password'] = password_obj
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        else:
            response = handle_errors(serializers.errors)
            response['result'] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class ProfileSettingPOSTAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    '''
    [{"notification_name":"M","is_active":"True","from_time":"date"}]
    [{"notification_name":"M","is_active":"True","from_time":"13:09:24","to_time":"15:00:00"gitla},
    {"notification_name":"O","is_active":"True","from_time":"13:09:24","to_time":"15:00:00"}]
    '''
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = ProfileSettingPOSTSerializer

    def post(self, request, format=None):

        response = {}
        responsedata = []
        serializers = self.serializer_class(
            data=request.data, context={'user': request.user})
        if serializers.is_valid():
            if "new_password" in serializers.validated_data:
                if "password" not in serializers.validated_data:
                    response['result'] = 0
                    response['errors'] = ["No Old Password Provided"]
                    self.add_api_call(request, response)
                    return Response(response, status=status.HTTP_200_OK)

                user = authenticate(
                    username=request.user.username, password=serializers.validated_data['password'])

                if user is None:
                    response['result'] = 0
                    response['errors'] = ["Old Password is invalid"]
                    self.add_api_call(request, response)
                    return Response(response, status=status.HTTP_200_OK)

            serializers.save()

            if 'new_email' in serializers.validated_data:
                email_verification_token = UserToken.objects.create(
                    user=request.user, purpose='EMAIL CHANGE')
                message = 'Verification Link Has Been Send on {} Email Address to verify email id'.format(
                    serializers.validated_data['new_email'])
                verify_redirect = "{}{}".format(DOMAIN_NAME, reverse('Email_change_token', kwargs={
                                                "mode": serializers.validated_data['mode'], "keytoken": email_verification_token}))

                email_verification(
                    request.user.first_name, verify_redirect, serializers.validated_data['new_email'])

                response['data'] = message
                response['result'] = 1
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)

            response['result'] = 1
            response['data'] = "Success"
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializers.errors)
            response['result'] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class UpdateProfileView(PlatformAPICallMixin, generics.GenericAPIView):

    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    queryset = User.objects.all()
    serializer_class = UpdateUserSerializer

    def post(self, request, format=None):
        response = {}
        response['result'] = 0
        serializers = self.serializer_class(
            data=request.data, context={'request': request})
        if serializers.is_valid():
            if 'email' in serializers.validated_data:
                serializers.save()
                response['data'] = 'Verification Link Has Been Send on {} Email Address to verify email id'.format(
                    serializers.validated_data['email'])
                email_verification_token = UserToken.objects.create(
                    user=request.user, purpose='EMAIL CHANGE')
                verify_redirect = "https://sportsbookingnew.worldindia.com/#/user-success/{}/{}".format(
                    serializers.validated_data['mode'], str(email_verification_token))

                email_verification(
                    request.user.first_name, verify_redirect, serializers.validated_data['email'])

                self.add_api_call(request, response)
                response['result'] = 1
                response['mode'] = str(serializers.validated_data['mode'])
                response['keytoken'] = str(email_verification_token)
                response['url'] = str(reverse('Email_change_token', kwargs={
                    "mode": serializers.validated_data['mode'], "keytoken": email_verification_token}))
                return Response(response, status=status.HTTP_200_OK)
        else:
            response['errors'] = ["Email not valid ."]
            self.add_api_call(request, response)
            response['result'] = 0
            return Response(response, status=status.HTTP_400_BAD_REQUEST)

        response = handle_errors(serializers.errors)
        response['result'] = 0
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_400_BAD_REQUEST)
