from django.db import models

from django.core.validators import MaxValueValidator, MinValueValidator

from django.contrib.auth.models import User
from storages.backends.s3boto import S3BotoStorage

from static_data.models import UserType


class S3CustomStorage(S3BotoStorage):
    def _normalize_name(self, name):
        """
        S3 Storage function for storing files
        """
        return name


class Carousel(models.Model):
    """
    Carousel
    """
    user_type = models.ForeignKey(
        UserType, on_delete=models.SET_NULL, null=True, blank=True, help_text="User Type")

    image = models.ImageField(upload_to="homepage",
                              blank=True, null=True, help_text="Image")
    heading = models.CharField(
        max_length=100, blank=True, null=True, help_text="Heading")

    description = models.TextField(
        blank=True, null=True, help_text="Description")
    link = models.CharField(max_length=100, blank=True,
                            null=True, help_text="Link")

    is_deleted = models.BooleanField(default=False)

    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)
    updated_at = models.DateTimeField(auto_now=True, blank=True, null=True)

    def __str__(self):
        return "{}-{}-{}".format(self.user_type, self.heading, self.link)

    class Meta:
        verbose_name_plural = "Carousel"
        db_table = 'carousel'


class Advertisement(models.Model):
    """
    Advertisements in Homepage
    """
    IMAGE_TYPE = (
        ("BANNER", "Banner"),
        ("ADVERTISEMENT", "Advertisement")
    )

    user_type = models.ForeignKey(
        UserType, on_delete=models.SET_NULL, null=True, blank=True, help_text="User Type")
    image = models.ImageField(upload_to="homepage",
                              blank=True, null=True, help_text="Ad Image")
    link = models.CharField(max_length=100, blank=True,
                            null=True, help_text="Link")

    image_type = models.CharField(
        choices=IMAGE_TYPE, max_length=30, help_text="Banner / Advertisement")

    is_deleted = models.BooleanField(default=False)

    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)
    updated_at = models.DateTimeField(auto_now=True, blank=True, null=True)

    def __str__(self):
        return "{}".format(self.user_type)

    class Meta:
        verbose_name_plural = "Advertisement"
        db_table = 'advertisement'


class Block(models.Model):
    """
    3rd Blocks in Homepage
    """
    user_type = models.ForeignKey(
        UserType, on_delete=models.SET_NULL, null=True, blank=True, help_text="User Type")
    icon = models.ImageField(upload_to="homepage",
                             blank=True, null=True, help_text="Ad Image")

    heading = models.CharField(
        max_length=100, blank=True, null=True, help_text="Heading")
    description = models.TextField(
        blank=True, null=True, help_text="Description")

    link = models.CharField(max_length=100, blank=True,
                            null=True, help_text="Link")
    is_deleted = models.BooleanField(default=False)

    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)
    updated_at = models.DateTimeField(auto_now=True, blank=True, null=True)

    def __str__(self):
        return "{}-{}-{}".format(self.user_type, self.heading, self.link)

    class Meta:
        verbose_name_plural = "Block 3"
        db_table = 'homepage_block'


class Visitor_Home(models.Model):
    """
    Visitor
    """
    user_type = models.ForeignKey(
        UserType, on_delete=models.SET_NULL, null=True, blank=True, help_text="User Type")

    image = models.ImageField(upload_to="homepage",
                              blank=True, null=True, help_text="Image")
    heading = models.CharField(
        max_length=100, blank=True, null=True, help_text="Heading")

    description = models.TextField(
        blank=True, null=True, help_text="Description")
    link = models.CharField(max_length=100, blank=True,
                            null=True, help_text="Link")

    background_colour = models.CharField(
        max_length=100, blank=True, null=True, help_text="background colour")
    n_icon = models.FileField(
        upload_to='n_icons/', null=True, blank=True, help_text='n_icons')
    fontColour = models.CharField(
        max_length=100, blank=True, null=True, help_text="font colour")

    is_deleted = models.BooleanField(default=False)

    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)
    updated_at = models.DateTimeField(auto_now=True, blank=True, null=True)

    def __str__(self):
        return "{}-{}-{}".format(self.user_type, self.heading, self.link)

    class Meta:
        verbose_name_plural = "Visitor_Home"
        db_table = 'visitor_home'


class Partner_Sponser_Logo(models.Model):
    """
    Partner_Sponser_Logo
    """

    logo = models.ImageField(upload_to="partner",
                             blank=True, null=True, help_text="Sponser Logo")

    text = models.CharField(
        max_length=100, blank=True, null=True, help_text="Logo Description")

    link = models.CharField(max_length=100, blank=True,
                            null=True, help_text="Link")
    is_deleted = models.BooleanField(default=False)

    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)
    updated_at = models.DateTimeField(auto_now=True, blank=True, null=True)

    def __str__(self):
        return "{}-{}".format(self.logo, self.link)

    class Meta:
        verbose_name_plural = "Partner Sponser Logo"
        db_table = 'partner_sponser_logo'
