import json
from django.http import HttpResponse
from django.template import Template, Context
from django.db import transaction
from django.urls import reverse
from django.contrib.contenttypes.models import ContentType

from django.contrib.auth import login as django_login
import datetime
from rest_framework.authtoken.models import Token
from rest_framework import status, generics

from rest_framework.response import Response
from rest_framework.authentication import TokenAuthentication
from rest_framework.permissions import IsAuthenticated

from utils.mixins import PlatformAPICallMixin
from utils.api_handle_errors import handle_errors

from static_data.models import UserType
from homepage.serializers import HomepageSerializer, AdvertisementSerializer, BlockSerializer, CarouselSerializer, Visitor_HomeSerializer, Partner_Sponser_LogoSerializer
from homepage.models import Carousel, Advertisement, Block, Visitor_Home, Partner_Sponser_Logo


class HomepageGetAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    serializer_class = HomepageSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []

        serialiers = self.serializer_class(data=request.data)
        if serialiers.is_valid():

            # Check for token
            if "token" in serialiers.validated_data:
                # Get User Type
                if Token.objects.filter(key=serialiers.validated_data['token']).exists():
                    user_type_obj = UserType.objects.get(
                        name=serialiers.validated_data['user_type'])
                else:
                    user_type_obj = UserType.objects.get(name="Visitor")
            else:
                user_type_obj = UserType.objects.get(name="Visitor")

            # Get Carousel
            carousel_obj = Carousel.objects.filter(
                user_type=user_type_obj, is_deleted=False)
            carousel_class = CarouselSerializer(carousel_obj, many=True)

            # Get Advertisement
            advertisement_obj = Advertisement.objects.filter(
                user_type=user_type_obj, image_type="Advertisement", is_deleted=False)
            advertisement_class = AdvertisementSerializer(
                advertisement_obj, many=True)

            # # Get Banner
            # banner_obj = Advertisement.objects.filter(user_type=user_type_obj, image_type="Banner", is_deleted=False)
            # banner_class = AdvertisementSerializer(advertisement_obj, many=True)

            # Get Block
            block_obj = Block.objects.filter(
                user_type=user_type_obj, is_deleted=False)
            block_class = BlockSerializer(block_obj, many=True)

            Visitor_Home_obj = Visitor_Home.objects.filter(
                user_type=user_type_obj, is_deleted=False)
            Visitor_Home_class = Visitor_HomeSerializer(
                Visitor_Home_obj, many=True)

            # Sending Response
            response["result"] = 1
            response['carousel'] = carousel_class.data
            response['advertisement'] = advertisement_class.data
            # response['banner'] = banner_class.data
            response['block'] = block_class.data
            response['visitor'] = Visitor_Home_class.data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        # Serializer Errors
        else:
            response = handle_errors(serialiers.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class Partner_Sponser_LogoAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    """
    Partner_Sponser_Logo
    """
    serializer_class = Partner_Sponser_LogoSerializer

    def get(self, request, format=None):
        response = {}
        responsedata = []
        data = Partner_Sponser_Logo.objects.all()

        serializer = self.serializer_class(data, many=True)
        for j in serializer.data:
            # print(j['text'])
            text_obj = j['text']
            if text_obj is not None:
                j['text'] = text_obj
            else:
                j['text'] = ""
        response['result'] = 1
        response['data'] = serializer.data
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)
