from django.db import models

from django.core.validators import MaxValueValidator, MinValueValidator

from django.contrib.auth.models import User
from storages.backends.s3boto import S3BotoStorage


from static_data.models import UserType, State

class InternalUser(models.Model): # InternalSystem
	""" Internal System User reside here """
	GENDER_CHOICES = (
		('M', 'Male'),
		('F', 'Female'),
		('O', 'Other')
	)

	user        = models.ForeignKey(User, on_delete=models.CASCADE, blank=True, null=True)
	user_type   = models.ForeignKey(UserType, on_delete=models.SET_NULL, blank=True, null=True)

	first_name  = models.CharField(max_length=100, null=True, help_text="user first name")
	middle_name = models.CharField(max_length=100, null=True, help_text="user middle name", blank=True)
	last_name   = models.CharField(max_length=100, null=True, help_text="user last name", blank=True)

	email      = models.CharField(max_length=100, null=True, help_text="user email")
	new_email  = models.CharField(max_length=100, null=True, blank=True, help_text="user new email")
	mobile     = models.CharField(max_length=10, null=True, help_text="user mobile number", unique=True)
	gender     = models.CharField(max_length=1, null=True, choices=GENDER_CHOICES, help_text="M - Male, F - Female, O - Other")
	address1   = models.CharField(max_length=150, blank=True, null=True, help_text="Apartment, Suite, Unit, building, floor,etc.")
	address2   = models.CharField(max_length=150, blank=True, null=True, help_text="Street & Number, P.O.box c/o")
	profile    = models.TextField(blank=True, null=True)

	city       = models.CharField(max_length=50, blank=True, null=True, help_text="city")
	state      = models.ForeignKey(State, on_delete=models.SET_NULL, blank=True, null=True)
	pincode    = models.CharField(max_length=10, blank=True, null=True, help_text="code")
	image      = models.ImageField(upload_to = "internal_user_images", blank = True, null = True, help_text="Profile image")
	dob        = models.DateField(blank=True, null=True, help_text='date of birth format YYYY-MM-DD', verbose_name='date of birth')



	is_deleted = models.BooleanField(default = False, help_text="delete/restore user")
	is_inactive  = models.BooleanField(default = False, help_text="activate/deactivate user")

	created_by = models.ForeignKey('self', on_delete=models.SET_NULL, null=True, blank=True)

	created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)
	updated_at = models.DateTimeField(auto_now=True, blank=True, null=True)

	def __str__(self):
		return "{}-{}".format(self.user.username, self.user.email)

	class Meta:
		unique_together = ('user', 'email')
		verbose_name_plural = "Internal System Users"
		db_table = 'internal_system_user'