from .models import *
from django.core.exceptions import ValidationError, FieldDoesNotExist


def CreateInvoiceDocumentData():
    document_data = InvoiceDocument.objects.filter(is_deleted=False, is_active=False)
    if document_data.exists():
        document_data = document_data.last()
        return document_data
    else:
        return InvoiceDocument.objects.create(is_deleted=False)
        
def CreateInvoiceItems(validated_data):
    invoice_item = InvoiceItems.objects.create(**validated_data)
    InvoiceDocument.objects.filter(id=validated_data['invoice_document']).update(is_active=True)
    return invoice_item


def GetInvoiceCategories(self,start_page:int=0, end_page:int=10, **kwargs) -> object:
    """
    Get the Invoice Category List
    Params: self, start_page, end_page, **kwargs
    Returns: Serialized Invoice Category object and Total count
    """
    invoice_category_obj = InvoiceCategories.objects.filter(**kwargs)
    total_count = invoice_category_obj.count()
    paginated_invoice_category_obj = invoice_category_obj [ start_page : end_page ]
    return self.serializer_class(paginated_invoice_category_obj, many=True).data, total_count


def GetListofData(self, start_page:int=0, end_page:int=10, **kwargs) -> object:
    """
    Get All Invoice List Data
    Params: self, start_page, end_page, **kwargs
    Returns: Serialized Invoice Category Object and Total Count 
    """
    data_obj = self.serializer_class.Meta.model.objects.filter(**kwargs)
    total_count = data_obj.count()
    paginated_data_obj = data_obj [ start_page : end_page ]
    if id in kwargs:
        return self.detailed_serializer_class(paginated_data_obj, many=True).data, total_count
    else:
        return self.serializer_class(paginated_data_obj, many=True).data, total_count




def GetDetailedData(self,data_obj=None, **kwargs) -> object:
    """
    Get All Invoice Detailed Data
    Params: self, **kwargs
    Returns: Serialized Invoice Category Object  
    """
    if not data_obj:
        data_obj = self.serializer_class.Meta.model.objects.filter(**kwargs)
        if data_obj.exists():
            data_obj = data_obj.last()
        else:
            raise FieldDoesNotExist("No Value found against the ID")
    return self.detailed_serializer_class(data_obj, many=False).data




def UpdateInvoiceDocument(validated_data:object, instance) -> int:
    """
    Purpose: Updates the Invoice Document
    Args: Validated data
    Returns: Updated Invoice Document Id
    """
    for (key, value) in validated_data.items():
        setattr(instance, key, value)
    instance.save()
    return instance.id


def InsertInvoiceItem(validated_data:list, document_id:int) -> bool:
    """
    Purpose: Inserts Invoice Item with their invoice document id
    Args: validated_data: list Validated_daa of invoice item 
          document_id: Document Foreign Key
    Returns Boolean
    """
    InvoiceItems.objects.filter(invoice_document=document_id).delete()
    invoice_item_obj = [InvoiceItems(**i, invoice_document=document_id) for i in validated_data]
    InvoiceItems.objects.bulk_create(invoice_item_obj)
    return True
    


def InsertInvoiceDocument(self) -> int:
    """
    Purpose: Creates the Invoice Document
    Args: self: self of a function with validated_Data in it
    Returns: Created Invoice Document Id
    """
    try:
        validated_data = self.validated_data
        items = False
        print(validated_data)
        if validated_data.get("item", False):
            items = validated_data.pop('item')

        invoice_document_instance = self.Meta.model.objects.create(**validated_data)
        if items:
            InsertInvoiceItem(items, invoice_document_instance)
        return invoice_document_instance.id
    except Exception as e:
        raise Exception(str(e))



def CheckIfUsedByInvoice(obj, model) -> bool:
    if model == InvoiceCategories:
        if InvoiceItems.objects.filter(**obj, is_deleted=False).exists():
            return True
        return False
    if InvoiceDocument.objects.filter(**obj,is_deleted=False).exists():
        return True
    return False


def GetAllComments(serializers, invoice_document:int)->object:
    """
    Purpose: Get All comments based on invoice document
    Args: invoice_document: Valid ID of innvoice Documment
    Returns: Comments of all invoice_document
    """
    invoice_document = InvoiceDocument.objects.get(id=invoice_document)
    comments_obj = InvoiceComment.objects.filter(invoice_document=invoice_document, is_deleted=False)
    comments_class = serializers(comments_obj, many=True)
    return comments_class.data