import requests
import pytz
import json
from django.http import HttpResponse
from django.template import Template, Context
from django.template.loader import get_template
from django.db import transaction
from django.urls import reverse
from django.core.exceptions import ValidationError, FieldDoesNotExist
from django.contrib.auth import login as django_login
import datetime
from rest_framework.authtoken.models import Token
from rest_framework import status, generics
from rest_framework.response import Response
from rest_framework.authentication import TokenAuthentication
from rest_framework.permissions import IsAuthenticated
from utils.mixins import PlatformAPICallMixin
from utils.api_handle_errors import handle_errors
from utils.pdf_generator import *
from invoicing.serializers import *
from static_data.models import *
from invoicing.functionalities import GetInvoiceCategories
from venue_owners.models import VenueOwner
from professionals.models import Professional
from advertisers.models import Advertiser
from players.models import Player
from venues.models import Venue
from rest_framework import serializers as drf_serializer


class InvoiceCategoryAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    # authentication_classes = (TokenAuthentication,)
    # permission_classes = (IsAuthenticated,)
    serializer_class = InvoiceCategoriesSerializer

    def delete(self, request, method, mode, format=None):
        """
        Delete Invoice Category with filters
        """
        response = {}
        if not request.GET.get("id", False):
            response["result"] = 0
            response["errors"] = ["No Id"]
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        category_obj = InvoiceCategories.objects.filter(mode=mode, id=request.GET.get("id"))
        if category_obj.exists():
            category_obj = category_obj.last()
        else:
            response["result"] = 0
            response["errors"] = ["Incorrect ID"]
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        # Check if in category or not
        category_exist_invoice = CheckIfUsedByInvoice({"invoice_category":category_obj}, InvoiceCategories)
        if category_exist_invoice:
            response["result"] = 0
            response["errors"] = ["This Category is in use currently"]
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        category_obj.is_deleted = True
        category_obj.save()
        response["result"] = 1
        response["message"] = ["Successfully Deleted"]
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)

    def post(self, request, method, mode, format=None):
        """ 
        Get Add Invoice Category with filters
        """ 
        try:
            response = {}

            if request.data.get("id", False):
                category_obj = self.serializer_class.Meta.model.objects.filter(id=request.data['id'], is_deleted=False)
                if category_obj.exists():
                    message = "Suuccessfully Updated"
                    serializers = self.serializer_class(instance=category_obj.last(),data=request.data)
                else:
                    response["result"] = 0
                    response["errors"] = ["Incorrect ID"]
                    self.add_api_call(request, response)
                    return Response(response, status=status.HTTP_200_OK)
            else:
                message = "Successfully Inserted"
                serializers = self.serializer_class(data=request.data)
            if serializers.is_valid():
                if method == "get":
                    serializers.validated_data.pop("measurement_unit", False)
                    serializers.validated_data.pop("price", False)
                    serializers.validated_data.pop("vat_category", False)
                    serializers.validated_data.pop("mode", False)
                    serializers.validated_data.pop("id", False)
                    serializers.validated_data.pop("created_at", False)
                    serializers.validated_data['is_deleted'] = False

                    offset = int(request.GET.get("offset",1))

                    assert offset != 0, "Offset Cannot be 0"
                    start_page = (offset - 1) * 10
                    end_page = start_page + 10

                    invoice_category_class, total_count = GetInvoiceCategories(self,start_page=start_page, end_page=end_page, **serializers.validated_data, mode=mode)
                    response["result"] = 1
                    response["data"] = invoice_category_class
                    response['total_count'] = total_count
                    self.add_api_call(request, response)
                    return Response(response, status=status.HTTP_200_OK)

                elif method == "post":
                    serializers.validated_data.pop("id", False)
                    serializers.validated_data['mode'] = mode
                    serializers.save()
                    response["result"] = 1
                    response["message"] = message
                    self.add_api_call(request, response)
                    return Response(response, status=status.HTTP_200_OK)

            else:
                response = handle_errors(serializers.errors)
                response['result'] = 0
                self.add_api_call(request, response)
                return Response(response,status=status.HTTP_200_OK)
        
        except ValueError:
            response["result"] = 0
            response["errors"] = ["Incorrect Value Provided"]
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        except Exception as e:
            response["result"] = 0
            response["errors"] = [str(e)]
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class InvoiceDocumentAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    # authentication_classes = (TokenAuthentication,)
    # permission_classes = (IsAuthenticated,)
    serializer_class = InvoiceDocumentSerializer
    detailed_serializer_class = InvoiceDocumentDetailedSerializer

    @transaction.atomic
    def post(self, request, method, format=None):
        """
        Add Edit Invoice Category with filters
        """
        try:
            response = {}
            if request.data.get("id", False):
                message = "Successfully Updated"
                invoice_document = InvoiceDocument.objects.filter(id=request.data.get("id"), is_deleted=False)
                if invoice_document.exists():
                    invoice_document = invoice_document.last()
                else:
                    raise ValueError("Invalid Invoice Document ID")
                serializers = self.serializer_class(instance=invoice_document, data=request.data)
            else:
                message = "Successfully Created"
                serializers = self.serializer_class(data=request.data)
            if serializers.is_valid():

                if method == "post":
                    serializers.validated_data.pop("date_from",False)
                    serializers.validated_data.pop("date_to",False)
                    serializers.validated_data.pop("number_from",False)
                    serializers.validated_data.pop("number_to",False)
                    obj_id = serializers.save()
                    response["result"] = 1
                    response["message"] = [message]
                    response['id'] = obj_id
                    self.add_api_call(request, response)
                    return Response(response, status=status.HTTP_200_OK)
            
                if method == "get":
                    if serializers.instance != None:
                        serializers.validated_data.pop("date_from",False)
                        serializers.validated_data.pop("date_to",False)
                        serializers.validated_data.pop("number_from",False)
                        serializers.validated_data.pop("number_to",False)
                        invoice_document = GetDetailedData(self, data_obj=serializers.instance)
                    else:
                        if serializers.validated_data.get("date_from",False) and serializers.validated_data.get("date_to",False):
                            serializers.validated_data['date__gte'] = serializers.validated_data.pop("date_from",False)
                            serializers.validated_data['date__lt'] = serializers.validated_data.pop("date_to",False)
                        if serializers.validated_data.get('number_from', False) and serializers.validated_data.get('number_to', False):
                            serializers.validated_data['number__gte'] = serializers.validated_data.pop("number_from",False)
                            serializers.validated_data['number__lte'] = serializers.validated_data.pop("number_to",False)

                        offset = int(request.GET.get("offset",1))
                        assert offset != "0", "Offset Cannot be 0"
                        start_page = (offset - 1) * 10
                        end_page = start_page + 10
                        invoice_document, total_count = GetListofData(self, start_page=start_page, end_page=end_page, **serializers.validated_data)
                        response['total_count'] = total_count
                    response["result"] = 1
                    response["data"] = invoice_document
                    self.add_api_call(request, response)
                    return Response(response, status=status.HTTP_200_OK)
            else:
                response = handle_errors(serializers.errors)
                response['result'] = 0
                self.add_api_call(request, response)
                return Response(response,status=status.HTTP_200_OK)

        except ValueError as e:
            response["result"] = 0
            response["errors"] = [str(e)]
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        except Exception as e:
            response["result"] = 0
            response["errors"] = [str(e)]
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

    def delete(self, request,method, format=None):
        try:
            response = {}
            invoice_document_id = request.GET.get("id",False)
            if not invoice_document_id:
                raise ValidationError("No Id Defined")
            invoice_document_obj = InvoiceDocument.objects.filter(id=invoice_document_id)
            if invoice_document_obj.exists():
                invoice_document_obj = invoice_document_obj.last()
            else:
                raise FieldDoesNotExist("No Document with Id found")
            invoice_document_obj.is_deleted = True
            invoice_document_obj.save()
            response["result"] = 1
            response["message"] = ["Invoice Document Successfully Deleted"]
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        
        except (FieldDoesNotExist, ValidationError) as ce:
            response["result"] = 0
            response["errors"] = [str(ce)]
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        except Exception as e:
            response["result"] = 0
            response["errors"] = ["Something went wrong"]
            response["description"] = [str(e)]
            self.add_api_call(request, response)
            response.pop("description")
            return Response(response, status=status.HTTP_200_OK)



class InvoiceCommentAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    # authentication_classes = (TokenAuthentication,)
    # permission_classes = (IsAuthenticated,)
    serializer_class = InvoiceCommentSerializer
    
    def post(self, request, format=None):
        """ 
        Invoice Comment Add Get Delete 
        """ 
        response = {}
        responsedata = []
        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            serializers.save()
            response["result"] = 1
            response["message"] = ["Successfully Inserted"]
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializers.errors)
            response['result'] = 0
            self.add_api_call(request, response)
            return Response(response,status=status.HTTP_200_OK)
    
    def get(self, request, format=None):
        """
        Get Invoice Comments
        """
        response = {}
        responsedata = []
        try:
            invoice_document_id = request.GET.get("document_id",False)
            assert invoice_document_id, "No valid ID sent"
            invoice_document_check = CheckIfUsedByInvoice({"id":invoice_document_id}, "Check for invoice")
            assert invoice_document_check, "No Valid ID Sent"
            comments = GetAllComments(self.serializer_class, invoice_document_id)
            response["result"] = 1
            response["data"] = comments
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        except AssertionError as ce:
            response["result"] = 0
            response["errors"] = [str(ce)]
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
    

    def delete(self, request, format=None):
        """
        To delete Invoice
        """
        response = {}
        responsedata = []
        try:
            comment_id = request.GET.get("id",False)
            assert comment_id, "No valid ID sent"
            comment_obj = InvoiceComment.objects.filter(id=comment_id)
            if comment_obj.exists():
                comment_obj = comment_obj.last()
            else:
                raise AssertionError("Invalid ID")

            comment_obj.is_deleted = True
            comment_obj.save()
            response["result"] = 1
            response["message"] = "Successfully Deleted"
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        except AssertionError as ce:
            response["result"] = 0
            response["errors"] = [str(ce)]
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class GetInvoiceCategoryFromCodeAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    # authentication_classes = (TokenAuthentication,)
    # permission_classes = (IsAuthenticated,)
    serializer_class = GetInvoiceCategoryFromCodeSerializer
    
    def get(self, request, code, format=None):
        """ 
        Get Invoice Category From Code
        """ 
        response = {}

        #  get object 
        invoice_category_obj = InvoiceCategories.objects.filter(code=code, is_deleted=False)
        if invoice_category_obj.exists():
            invoice_category_obj = invoice_category_obj.last()
        else:
            response["result"] = 0
            response["errors"] = ["No Invoice Category Found"]
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)        
        
        invoice_category_class = self.serializer_class(invoice_category_obj, many=False)
        response["result"] = 1
        response["data"] = invoice_category_class.data
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)





class GeneratePdfAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    # authentication_classes = (TokenAuthentication,)
    # permission_classes = (IsAuthenticated,)
    serializer_class = InvoiceDocumentDetailedSerializer
    detailed_serializer_class = InvoiceDocumentDetailedSerializer
    
    def get(self, request, format=None):
        """ 
        GeneratePdfAPIView
        """
        try: 
            response = {}
            invoice_document_id = request.GET.get("id", False)
            assert invoice_document_id,"No Id Passed"
            invoice_data = GetDetailedData(self, id=invoice_document_id)
            # Convert To Pdf
            pdf_bytes = createInvoice(invoice_data)
            
            # Insert to Invoice Document Data
            invoice_obj = InvoiceDocument.objects.get(id=invoice_data['id'])
            invoice_obj.pdf = pdf_bytes
            invoice_obj.save()

            invoice_class = self.serializer_class(invoice_obj).data

            response["result"] = 1
            response["data"] = {"invoice":invoice_class['pdf']}
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        except (AssertionError,FieldDoesNotExist) as ce:
            response["result"] = 0
            response["errors"] = [str(ce)]
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
            

class GetUserDatafromUserCode(PlatformAPICallMixin, generics.GenericAPIView):
    # authentication_classes = (TokenAuthentication,)
    # permission_classes = (IsAuthenticated,)
    serializer_class = GetUserDataFromUserCodeSerializer
    
    def get(self, request, user_code, format=None):
        """ 
        GetUserDatafromUserCode
        """ 
        response = {}
        responsedata = []

        # Check for Venue Owner
        dict_filter = {"user_code":user_code, "is_deleted":False}
        venue_owner = VenueOwner.objects.filter(**dict_filter)
        professional = Professional.objects.filter(**dict_filter)
        advertiser = Advertiser.objects.filter(**dict_filter)

        if venue_owner.exists():
            venue_owner = venue_owner.last()
            self.serializer_class.Meta.model = VenueOwner
            obj_class = self.serializer_class(venue_owner)
        elif professional.exists():
            professional = professional.last()
            self.serializer_class.Meta.model = Professional
            obj_class = self.serializer_class(professional)
        elif advertiser.exists():
            advertiser = advertiser.last()
            self.serializer_class.Meta.model = Advertiser
            obj_class = self.serializer_class(advertiser)
        else:
            response["result"] = 0
            response["errors"] = ["No User found from User Code"]
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        response["result"] = 1
        response["data"] = obj_class.data
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


class GetInvoiceCategoryAutoCompleteAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    # authentication_classes = (TokenAuthentication,)
    # permission_classes = (IsAuthenticated,)
    serializer_class = GetInvoiceCategoryAutoComplete

    
    def get(self, request,method, data, format=None):
        """ 
        Comments
        method = [user_code, buisness_name, buisness_vat]
        """ 
        response = {}


        if method == "category_code":
            dict_filter = {"code__icontains":data, "is_deleted":False}
            obj_data = self.serializer_class.Meta.model.objects.filter(**dict_filter)

        elif method == "description":
            dict_filter = {"description__icontains":data, "is_deleted":False}
            obj_data = self.serializer_class.Meta.model.objects.filter(**dict_filter)
            self.serializer_class.Meta.fields.append("description")


        invoice_category_class = self.serializer_class(obj_data, many=True)
        response["result"] = 1
        response["data"] = invoice_category_class.data
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)
        



class GetUserDataAutoCompleteAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    # authentication_classes = (TokenAuthentication,)
    # permission_classes = (IsAuthenticated,)
    serializer_class = GetDataAutoCompleteSerializer

    
    def get(self, request,method, data, format=None):
        """ 
        Comments
        method = [user_code, buisness_name, buisness_vat]
        """ 
        response = {}


        if method == "user_code":
            dict_filter = {"user_code__icontains":data, "is_deleted":False}
            obj_data = VenueOwner.objects.filter(**dict_filter)
            self.serializer_class.Meta.model = VenueOwner
            self.serializer_class.Meta.fields = ['obj_id','user_code']

        elif method == "buisness_name":
            dict_filter = {"business_name__icontains":data, "is_deleted":False}
            obj_data = Venue.objects.filter(**dict_filter)
            self.serializer_class.Meta.model = Venue
            self.serializer_class.Meta.fields = ['obj_id','business_name','created_by']

        elif method == "buisness_vat":
            dict_filter = {"buisness_vat__icontains":data, "is_deleted":False}
            obj_data = Venue.objects.filter(**dict_filter)
            self.serializer_class.Meta.model = Venue
            self.serializer_class.Meta.fields = ['obj_id','buisness_vat','created_by']


        venue_owner_class = self.serializer_class(obj_data, many=True)
        response["result"] = 1
        response["data"] = venue_owner_class.data
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)
        