from django.db import models

from django.core.validators import MaxValueValidator, MinValueValidator
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes.fields import GenericForeignKey

from django.contrib.auth.models import User
from storages.backends.s3boto import S3BotoStorage

from static_data.models import State, NotificationIcon

class S3CustomStorage(S3BotoStorage):
    def _normalize_name(self, name):
        """
        S3 Storage function for storing files
        """
        return name

class MessagesInbox(models.Model):
    """ Venue Owner Messages """
    message             = models.TextField(max_length=10000, null=True, help_text="Message Body")
    sender              = models.ForeignKey(User, on_delete=models.CASCADE,related_name="sender")
    recipient           = models.ForeignKey(User, on_delete=models.CASCADE,related_name="receiver")
    sent_at             = models.DateTimeField(auto_now_add=True, blank=True, null=True)
    is_read             = models.BooleanField(default = False, help_text="Unread/Read")
    # last_email          = models.CharField(max_length=100, null=True, blank=True, help_text="Email Replied On")
    # last_email_date     =  models.DateTimeField(auto_now_add=True, blank=True, null=True)
    is_deleted          = models.BooleanField(default = False, help_text="activate/deactivate user")
    created_at          = models.DateTimeField(auto_now_add=True, blank=True, null=True)
    updated_at          = models.DateField(auto_now=True, blank=True, null=True)

    def __str__(self):
        return "{}-{}-{}".format(self.sender, self.sent_at, self.message)

    class Meta:
        verbose_name_plural = "Message Inbox"
        db_table = 'message_storage'

class Notification(models.Model):
    category        = models.TextField(max_length=50, null=True, help_text="Notification Category")
    subject         = models.TextField(max_length=100, null=True, help_text="Notification Header")
    body            = models.TextField(max_length=100, null=True, help_text="Notification Body")
    icon            = models.ForeignKey(NotificationIcon,on_delete=models.SET_NULL ,blank=True, null=True, help_text="Icon")
    button_text     = models.CharField(max_length=100, blank=True, null=True, help_text="Icon")
    created_at      = models.DateTimeField(auto_now_add=True, blank=True, null=True)
    is_read         = models.BooleanField(default = False, help_text="Unread/Read")
    recipient       = models.ForeignKey(User, on_delete=models.CASCADE,related_name="NotificationReciver")
    content_type    = models.ForeignKey(ContentType, on_delete=models.CASCADE)
    object_id       = models.PositiveIntegerField()
    content_object  = GenericForeignKey('content_type', 'object_id')

    def __str__(self):
        return "{}-{}-{}".format(self.content_type, self.subject, self.object_id)   

    class Meta:
        verbose_name_plural = "Notifications"
        db_table = 'notifications'