from rest_framework import serializers
from .models import *
from venue_owners.models import VenueOwner
from players.models import Player
from professionals.models import Professional
from advertisers.models import Advertiser


class MessagesDeleteSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(required=True)
    MessageType = serializers.CharField(
        required=True, max_length=200, help_text='Inbox / Sentbox')

    class Meta:
        model = MessagesInbox
        fields = ('id', 'MessageType')
        extra_kwargs = {
            'sender': {'required': False},
            'recipient': {'required': False}
        }


class MessagesInboxSerializer(serializers.ModelSerializer):
    MessageType = serializers.CharField(
        max_length=200, help_text='Inbox / Sentbox', required=False)
    from_date = serializers.DateField(required=False, help_text="YYYY-MM-DD")
    to_date = serializers.DateField(required=False, help_text="YYYY-MM-DD")
    sender = serializers.CharField(source="sender.first_name", read_only=True)
    page = serializers.IntegerField(
        default=1, required=False, help_text="Page number for pagination")
    recipient = serializers.CharField(
        source="recipient.first_name", read_only=True)

    class Meta:
        model = MessagesInbox
        # fields = ('MessageType','is_read','is_deleted','from_date','to_date','message')
        fields = '__all__'
        read_only_fields = ['message', 'sender', 'recipient', 'sent_at']
        extra_kwargs = {
            'sender': {'required': False},
            'recipient': {'required': False},
            'is_deleted': {'required': False},
            'is_read': {'required': False},
            'message': {'read_only': True}
        }


class ComposeMessageSerialier(serializers.ModelSerializer):
    recipient = serializers.CharField(required=False)
    sender = serializers.CharField(required=False)

    def validate_recipient(self, value):
        recipient = User.objects.get(username=value)
        return recipient

    def validate_sender(self, value):
        sender = User.objects.get(username=value)
        return sender

    class Meta:
        model = MessagesInbox
        fields = ('recipient', 'sender', 'message')


class NotificationsSerializer(serializers.ModelSerializer):
    icon_link = serializers.FileField(
        source="icon.icons", read_only=True, required=False)
    page = serializers.IntegerField(
        default=1, required=False, help_text="Page Number")

    class Meta:
        model = Notification
        fields = '__all__'


class NotificationPaginationSerializer(serializers.Serializer):
    page = serializers.IntegerField(
        default=1, required=False, help_text="Page Number")


class ForwardMessageSerializer(serializers.ModelSerializer):
    class Meta:
        model = MessagesInbox
        fields = ['message']


class ReplyMessageSerializer(serializers.ModelSerializer):
    sender = serializers.CharField(source="sender.username")

    class Meta:
        model = MessagesInbox
        fields = ['message', 'sender']


class GetMessageActionSerializer(serializers.Serializer):
    action = serializers.CharField(required=True, help_text="Forward / Reply")
    id = serializers.PrimaryKeyRelatedField(
        queryset=MessagesInbox.objects.all(), required=False, help_text="Id of message")


class GetMessageSerializer(serializers.Serializer):
    id = serializers.PrimaryKeyRelatedField(
        queryset=MessagesInbox.objects.all(), required=False, help_text="Id of message")


class GetSingleMessageSerializer(serializers.ModelSerializer):
    sender = serializers.CharField(source="sender.username")

    class Meta:
        model = MessagesInbox
        fields = ['message', 'sender']


# GET NAmE AND EmAILS


class GetEmailSerializer(serializers.Serializer):
    key_word = serializers.CharField(
        required=True, help_text="Send first name, last name and email address")


class VenueOwnerSerializer(serializers.ModelSerializer):
    class Meta:
        model = VenueOwner
        fields = ['email', 'first_name', 'last_name']


class PlayerSerializer(serializers.ModelSerializer):
    class Meta:
        model = Player
        fields = ['email', 'first_name', 'last_name']


class ProfessionalSerializer(serializers.ModelSerializer):
    class Meta:
        model = Professional
        fields = ['email', 'first_name', 'last_name']


class AdvertiserSerializer(serializers.ModelSerializer):
    class Meta:
        model = Advertiser
        fields = ['email', 'first_name', 'last_name']
