from rest_framework import serializers
from django.contrib.auth.models import User, Group
from rest_framework.authtoken.models import Token

from datetime import date

from .models import *
from platform_central.models import UserFcmObject, UserToken, UserOtp
from players.models import Player
from venue_owners.models import VenueOwner
from professionals.models import Professional
from advertisers.models import Advertiser
from static_data.models import *
from commonAPI.models import OnBoardAPI


class GroupSerializer(serializers.ModelSerializer):
    class Meta:
        model = Group
        fields = ['name']


class RegisterationSerializer(serializers.ModelSerializer):
    GENDER_CHOICE = (
        ('M', 'Male'),
        ('F', 'Female'),
        ('O', 'Other')
    )
    email = serializers.CharField(required=True, help_text="Email Address")
    dob = serializers.DateField(required=False)
    gender = serializers.ChoiceField(
        choices=GENDER_CHOICE, help_text="M=Male,F=Female,O=Other", required=True)
    mobile = serializers.CharField(
        max_length=15, required=True, help_text="user mobile number")
    state = serializers.IntegerField(
        required=False, help_text="state")
    area = serializers.IntegerField(
        required=False, help_text="area")
    city = serializers.IntegerField(
        required=False, help_text="city")
    country = serializers.IntegerField(
        required=False, help_text="country")

    def create(self, validated_data):
        try:
            data = User.objects.create(username=validated_data['email'], first_name=validated_data['first_name'],
                                       last_name=validated_data['last_name'], email=validated_data['email'], is_active=False)
            data.set_password(validated_data['password'])
            data.save()
            player = Player.objects.update_or_create(user=data, defaults={"dob": validated_data['dob'], "gender": validated_data['gender'], "first_name": validated_data[
                                                     'first_name'], "last_name": validated_data['last_name'], "email": validated_data['email'], "mobile": validated_data['mobile'], "city": City.objects.get(
                id=validated_data['city']), "country": Country.objects.get(
                id=validated_data['country']), "state": State.objects.get(
                id=validated_data['state']), "area": Area.objects.get(
                id=validated_data['area'])})
            player_group = Group.objects.get(name="Player")
            data.groups.add(player_group)
            return data
        except Exception as e:
            raise serializers.ValidationError(str(e))

    class Meta:
        model = User
        fields = "__all__"
        read_only_fields = ['is_staff', 'username', 'last_login', 'is_superuser',
                            'is_active', 'date_joined', 'groups', 'user_permissions']


class ResendEmailSerializer(serializers.Serializer):
    email = serializers.CharField(required=True, help_text="Email Address")

    def validate_email(self, value):
        if User.objects.filter(email=value, is_active=False).exists():
            if UserToken.objects.filter(user=User.objects.get(email=value, is_active=False), is_active=True).exists():
                return User.objects.get(email=value, is_active=False)
            else:
                raise serializers.FieldDoesNotExist("No Token for this email")
        else:
            raise serializers.FieldDoesNotExist("Not a valid Email")


class ValidateEmailSerializers(serializers.Serializer):
    token = serializers.CharField(required=True)

    def validate_token(self, value):
        if UserToken.objects.filter(key=value).first().exists():
            return value
        else:
            raise serializers.FieldDoesNotExist("Not a valid Token")


class RegisterationPrivateKeySerializer(serializers.Serializer):
    key = serializers.CharField(required=True, help_text="Key")


############# Reset Password ##############

class ResetPasswordSerializers(serializers.Serializer):
    email_address = serializers.CharField(required=False)

    def validate_email_address(self, value):
        if User.objects.filter(email=value).exists():
            return value
        else:
            raise serializers.FieldDoesNotExist(
                "Email address is not registered")


class ConfirmTokenPasswordSerializer(serializers.Serializer):
    """
    Serializers for Confirming the token for forgot password
    """
    token = serializers.CharField(required=True, help_text="Token")


class VerifyPasswordTokenSerializer(serializers.Serializer):
    Token = serializers.CharField(required=True)
    password = serializers.CharField(required=True)

    def validate_Token(self, value):
        if UserToken.objects.filter(key=value).exists():
            token_obj = UserToken.objects.get(key=value)
            if token_obj.is_active == False:
                raise serializers.FieldDoesNotExist("Token is already used")
            else:
                return token_obj
        else:
            raise serializers.FieldDoesNotExist("Not a valid Token")


############## LOGIN ####################################


class LoginSerializers(serializers.Serializer):
    username = serializers.CharField(required=True)
    password = serializers.CharField(required=True)


############ Change Mobile ######################

class MobileChangeSerializer(serializers.Serializer):
    mobile_number = serializers.CharField(required=True)
    mode = serializers.CharField(
        required=True, help_text="Venue Owner, Professional, Player, Advertiser")

    def validate(self, obj):
        if obj['mode'] == "Venue Owner":
            if VenueOwner.objects.filter(mobile=obj['mobile_number']).exists():
                raise serializers.FieldDoesNotExist(
                    "Number is already registered")
        elif obj['mode'] == "Professional":
            if Professional.objects.filter(mobile=obj['mobile_number']).exists():
                raise serializers.FieldDoesNotExist(
                    "Number is already registered")
        elif obj['mode'] == "Player":
            if Player.objects.filter(mobile=obj['mobile_number']).exists():
                raise serializers.FieldDoesNotExist(
                    "Number is already registered")
        elif obj['mode'] == "Advertiser":
            if Advertiser.objects.filter(mobile=obj['mobile_number']).exists():
                raise serializers.FieldDoesNotExist(
                    "Number is already registered")
        return obj


class MobileChangeVerifySerializer(serializers.Serializer):
    otp = serializers.CharField(required=True)
    mode = serializers.CharField(
        required=True, help_text="Venue Owner, Professional, Player, Advertiser")

    def validate_otp(self, value):
        if UserOtp.objects.filter(user=self.context['user'], otp=value, is_active=True).exists():
            return value
        else:
            raise serializers.FieldDoesNotExist("Incorrect OTP")


class VerfiyUserSerializer(serializers.Serializer):
    token = serializers.CharField(required=True, help_text="Token")
    # user_type = serializers.CharField(required=True, help_text="Admin / Professional / Player / Advertiser")


class OnBoardAPISerializer(serializers.ModelSerializer):
    class Meta:
        model = OnBoardAPI
        fields = '__all__'
