import pytz

from django.http import HttpResponse, HttpResponseRedirect
from django.template import Template, Context
from django.template.loader import get_template
from django.db import transaction
from django.urls import reverse

from django.contrib.auth import login, authenticate, logout

import datetime

from rest_framework.authtoken.models import Token
from rest_framework import status, generics
from rest_framework.response import Response
from rest_framework.authentication import TokenAuthentication
from rest_framework.permissions import IsAuthenticated

from utils.mixins import *
from commonAPI.models import ContactInfo
from utils.api_handle_errors import handle_errors
from utils.mail import email_verification
from players.serializers import *
from platform_central.models import UserToken
# from field_booking.functions import send_sms
from field_booking.auth_backend import EmailBackend
from field_booking.settings import DOMAIN_NAME, TOKEN_EXPIRED_AFTER_HOURS
from datetime import timedelta
from platform_auth.serializers import *

from venue_owners.models import VenueOwner
from professionals.models import Professional
from advertisers.models import Advertiser
from players.models import Player

"""
REGISTERATION_TYPE = (
        ('A','Advertiser'),
        ('P','Professional'),
        ('V','Venue Owner'),
    )
"""


def is_token_expired(token):
    min_age = timezone.now() - timedelta(
        hours=TOKEN_EXPIRED_AFTER_HOURS)
    expired = token.created_at < min_age
    return expired


class ConfirmEmailTokenAPIView(PlatformAPICallMixin, generics.GenericAPIView):

    @transaction.atomic
    def get(self, request, mode, keytoken, format=None):
        response = {}
        responsedata = []
        # Taking Token
        token_data = UserToken.objects.filter(key=keytoken).first()
        expired = is_token_expired(token_data)
        if expired:
            token_data.delete()
            UserToken.objects.create(user=token.user)
            raise AuthenticationFailed("Token has expired")

        if token_data.is_active == True:
            if mode == "Venue Owner":
                data = VenueOwner.objects.get(
                    user=token_data.user, is_deleted=False)
            elif mode == "Professional":
                data = Professional.objects.get(
                    user=token_data.user, is_deleted=False)
            elif mode == "Advertiser":
                data = Advertiser.objects.get(
                    user=token_data.user, is_deleted=False)
            elif mode == "Player":
                data = Player.objects.get(
                    user=token_data.user, is_deleted=False)

            if not data.new_email:
                # Changing Email Address from User Type Profile
                new_email = token_data.user.email
                data.email = new_email
                data.new_email = ""
                data.save()
            else:
                new_email = data.new_email
                data.email = new_email
                data.new_email = ""
                data.save()

            # Changing User Email Address
            user_data = token_data.user
            user_data.email = new_email
            user_data.username = new_email
            user_data.save()

            # changing contact info email
            data_obj = VenueOwner.objects.filter(user=token_data.user) if mode == "Venue Owner" else Player.objects.filter(
                user=token_data.user) if mode == "Player" else Professional.objects.filter(user=token_data.user) if mode == "Professional" else Advertiser.objects.filter(user=token_data.user)
            data_contentType = ContentType.objects.get_for_model(VenueOwner) if mode == "Venue Owner" else ContentType.objects.get_for_model(
                Player) if mode == "Player" else ContentType.objects.get_for_model(Professional) if mode == "Professional" else ContentType.objects.get_for_model(Advertiser)
            if ContactInfo.objects.filter(content_type__pk=data_contentType.id, object_id__in=data_obj, is_deleted=False).exists():
                contact_info_object = ContactInfo.objects.get(
                    content_type__pk=data_contentType.id, object_id__in=data_obj, is_deleted=False)
                contact_info_object.email = new_email
                contact_info_object.save()

            # Change all  user Groups
            user_group = user_data.groups.all()
            for i in user_group:
                if i.name == "Professional":
                    user_type_email_data = Professional.objects.get(
                        user=token_data.user, is_deleted=False)
                if i.name == "Player":
                    user_type_email_data = Player.objects.get(
                        user=token_data.user, is_deleted=False)
                if i.name == "Venue Owner":
                    user_type_email_data = VenueOwner.objects.get(
                        user=token_data.user, is_deleted=False)
                if i.name == "Advertiser":
                    user_type_email_data = Advertiser.objects.get(
                        user=token_data.user, is_deleted=False)
                user_type_email_data.email = new_email
                user_type_email_data.username = new_email
                user_type_email_data.save()

            # Changing Token Active Status
            token_data.is_active = False
            token_data.last_used = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")
            token_data.save()

            message = "Successfully Saved"

            response['result'] = 1
            responsedata.append(message)
            response['errors'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        else:
            response['result'] = 0
            responsedata.append("Token Already used")
            response['errors'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)
