import pytz

from django.http import HttpResponse, HttpResponseRedirect
from django.template import Template, Context
from django.template.loader import get_template
from django.db import transaction
from django.urls import reverse

from django.contrib.auth.models import User, Group
from django.contrib.auth import login, authenticate, logout

import datetime

from rest_framework.authtoken.models import Token
from rest_framework import status, generics
from rest_framework.response import Response
from rest_framework.authentication import TokenAuthentication
from rest_framework.permissions import IsAuthenticated

from utils.mixins import *

from utils.api_handle_errors import handle_errors
from utils.mail import email_verification
from players.serializers import *
from platform_central.models import UserToken
# from field_booking.functions import send_sms
from field_booking.auth_backend import EmailBackend
from field_booking.settings import DOMAIN_NAME
from message.models import Notification
from platform_auth.serializers import *

from commonAPI.models import OnBoardAPI


class LoginAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    serializer_class = LoginSerializers

    def post(self, request, format=None):

        response = {}
        responsedata = []
        serializer = self.serializer_class(data=request.data)
        if serializer.is_valid():
            if not User.objects.filter(username=serializer.validated_data['username'], is_active=True).exists():
                response["result"] = 0
                response["errors"] = ["Incorrect Username of Password"]
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)

            if User.objects.get(username=serializer.validated_data['username']).is_active == False:
                response['result'] = 0
                responsedata.append(
                    "Please activate your account from the link  sent to your email")
                response['errors'] = responsedata
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_400_BAD_REQUEST)

            user = authenticate(
                request, username=serializer.validated_data['username'], password=serializer.validated_data['password'])

            if user is not None:
                login(request, user)
                token = Token.objects.update_or_create(user=user)
            else:
                response['result'] = 0
                responsedata.append("Incorrect Username or password")
                response['errors'] = responsedata
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_400_BAD_REQUEST)

            if user.username != "admin@gmail.com":
                onboard_obj = OnBoardAPI.objects.filter(
                    email=user.username).last()
                onboard_serializer = OnBoardAPISerializer(onboard_obj)
                if onboard_serializer.data['status'] == 'A':
                    flag = True
                    response['flag'] = flag
                elif onboard_serializer.data['status'] == 'C':
                    flag = False
                    response['flag'] = flag

            # Get Groups
            groups = Group.objects.filter(user=user)
            group_class = GroupSerializer(groups, many=True)

            # Get Notificatio Count
            notification_count = Notification.objects.filter(
                recipient=user, is_read=False).count()

            response['token'] = str(token[0])
            response['first_name'] = str(user.first_name)
            response['last_name'] = str(user.last_name)
            response['notification_count'] = str(notification_count)
            response['user_type'] = group_class.data
            response['result'] = 1
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        else:
            response = handle_errors(serializer.errors)
            response['result'] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class LogoutAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)

    def get(self, request, format=None):

        response = {}
        responsedata = []
        if Token.objects.filter(user=request.user).exists():
            Token.objects.get(user=request.user).delete()
        logout(request)
        response['message'] = "Logged Out"
        response['result'] = 1
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


class VerifyUserAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    serializer_class = VerfiyUserSerializer

    def post(self, request, format=None):
        """
        User Verification
        """

        response = {}
        responsedata = []
        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():

            # Check if token exists
            if Token.objects.filter(key=serializers.validated_data['token']).exists():
                token_obj = Token.objects.get(
                    key=serializers.validated_data['token'])
            else:
                response['result'] = 0
                responsedata.append("No Such Token Found")
                response['errors'] = responsedata
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_400_BAD_REQUEST)

            # Getting User object
            user_obj = token_obj.user
            if user_obj.username != "admin@gmail.com":
                onboard_obj = OnBoardAPI.objects.filter(
                    email=user_obj.username).last()
                onboard_serializer = OnBoardAPISerializer(onboard_obj)
                if onboard_serializer.data['status'] == 'A':
                    flag = True
                    response['flag'] = flag
                elif onboard_serializer.data['status'] == 'C':
                    flag = False
                    response['flag'] = flag

            # Get Groups
            groups = Group.objects.filter(user=user_obj)
            group_class = GroupSerializer(groups, many=True)

            # Get Notificatio Count
            notification_count = Notification.objects.filter(
                recipient=user_obj, is_read=False).count()

            # Sending Response
            response["result"] = 1
            response['message'] = "Valid"
            response['token'] = str(token_obj.key)
            response['first_name'] = str(user_obj.first_name)
            response['notification_count'] = str(notification_count)
            response['last_name'] = str(user_obj.last_name)
            response['user_type'] = group_class.data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        # Serializer Error
        else:
            response = handle_errors(serializers.errors)
            response['result'] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)
