import pytz

from django.http import HttpResponse,HttpResponseRedirect
from django.template import Template, Context
from django.template.loader import get_template
from django.db import transaction
from django.urls import reverse

from django.contrib.auth import login,authenticate,logout
from django.utils import timezone
import datetime

from rest_framework.authtoken.models import Token
from rest_framework import status , generics
from rest_framework.response import Response
from rest_framework.authentication import TokenAuthentication
from rest_framework.permissions import IsAuthenticated

from utils.mixins import *

from utils.api_handle_errors import handle_errors
from utils.mail import email_verification
from players.serializers import *
from platform_central.models import UserOtp,SmsCentral

from field_booking.settings import DOMAIN_NAME

from platform_auth.serializers import *

from utils.mail import send_sms
from venue_owners.models import VenueOwner



class ChangeMobileAPIVIEW(PlatformAPICallMixin, generics.GenericAPIView):
    '''
    Change Mobile
    '''
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,) 
    serializer_class = MobileChangeSerializer
    def post(self, request, format=None):

        response = {}	
        responsedata = []
        serializer = self.serializer_class(data=request.data)
        if serializer.is_valid():
            if UserOtp.objects.filter(user=request.user,is_active=True).exists():
                user_otp = UserOtp.objects.filter(user=request.user,is_active=True).first()
                time_difference = timezone.now() - user_otp.updated_at

                if (time_difference.seconds/60)%60 < 1:
                    response['result'] = 0
                    responsedata.append("Please Try Again after 1 Minute")
                    response['errors'] = responsedata
                    self.add_api_call(request, response)
                    return Response(response, status=status.HTTP_400_BAD_REQUEST)

            mode_data = VenueOwner.objects.get(user=request.user) if serializer.validated_data['mode'] == 'Venue Owner' else Player.objects.get(user=request.user) if serializer.validated_data['mode'] == 'Player' else Professional.objects.get(user=request.user) if serializer.validated_data['mode'] == 'Professional' else Advertiser.objects.get(user=request.user)
            mode_data.new_mobile = serializer.validated_data['mobile_number']
            mode_data.save()
            user_otp = UserOtp.objects.update_or_create(user=request.user)
            send_sms(mobile=serializer.validated_data['mobile_number'],user_otp=user_otp[0])
            response['message'] = "OTP has been sent to your phone"
            response['result'] = 1
            self.add_api_call(request, response)
            return Response(response,status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response['result'] = 0
            self.add_api_call(request, response)
            return Response(response,status=status.HTTP_400_BAD_REQUEST)



class VerifyMobileAPIVIEW(PlatformAPICallMixin, generics.GenericAPIView):
    '''
    Verify Mobile
    '''
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,) 
    serializer_class = MobileChangeVerifySerializer

    def post(self, request, format=None):

        response = {}	
        responsedata = []
        serializer = self.serializer_class(data=request.data,context={"user":request.user})
        if serializer.is_valid():
            user_otp = UserOtp.objects.filter(user=request.user,otp=serializer.validated_data['otp'],is_active=True).first()
            if user_otp.valid_till < timezone.now():
                user_otp.delete()
                response['result'] = 0
                responsedata.append("OTP Expired")
                response['errors'] = responsedata
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_400_BAD_REQUEST)
            mode_data = VenueOwner.objects.get(user=request.user) if serializer.validated_data['mode'] == 'Venue Owner' else Player.objects.get(user=request.user) if serializer.validated_data['mode'] == 'Player' else Professional.objects.get(user=request.user) if serializer.validated_data['mode'] == 'Professional' else Advertiser.objects.get(user=request.user)
            
            mode_data.mobile = mode_data.new_mobile
            mode_data.new_mobile =  ""
            mode_data.save()
            user_otp.delete()

            response['message'] = "Phone Changed Successfully"
            response['result'] = 1
            self.add_api_call(request, response)
            return Response(response,status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response['result'] = 0
            self.add_api_call(request, response)
            return Response(response,status=status.HTTP_400_BAD_REQUEST)
