import pytz

from django.http import HttpResponse, HttpResponseRedirect
from django.db import transaction
from django.urls import reverse

from django.contrib.auth import login as django_login

import datetime

from rest_framework.authtoken.models import Token
from rest_framework import status, generics
from rest_framework.response import Response
from rest_framework.authentication import TokenAuthentication
from rest_framework.permissions import IsAuthenticated
from django.utils import timezone

from utils.mixins import *

from utils.api_handle_errors import handle_errors
from utils.mail import email_verification, send_mail_registeration
from players.serializers import *
from platform_central.models import UserToken, RegisterPrivateKey
# from field_booking.functions import send_sms
from field_booking.auth_backend import EmailBackend
from field_booking.settings import DOMAIN_NAME

from platform_auth.serializers import *


class WebRegistrationAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    serializer_class = RegisterationSerializer

    def post(self, request, format=None):

        response = {}
        responsedata = []
        serializer = self.serializer_class(data=request.data)
        if serializer.is_valid():
            serializer.validated_data['username'] = serializer.validated_data['email']
            if User.objects.filter(username=serializer.validated_data['email']).exists():
                response['message'] = "This email address is already registered"
                response['result'] = 0
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)

            user_object = serializer.save()

            email_verification_token = UserToken.objects.create(
                user=user_object, purpose='EMAIL VERIFICATION')
            message = 'Verification Link Has Been Send on {} Email Addrees to verify email id'.format(
                serializer.validated_data['email'])

            verify_redirect = "{}{}".format(
                "https://sportsbookingnew.worldindia.com/#/success/", email_verification_token)

            send_mail_registeration(
                serializer.validated_data['first_name'], verify_redirect, serializer.validated_data['email'])

            response['message'] = message
            response['result'] = 1
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response['result'] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class ResendEmailAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    serializer_class = ResendEmailSerializer
    """
	Resend Registeration Email Verification
	"""

    def post(self, request, format=None):

        response = {}
        responsedata = []
        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            user_token = UserToken.objects.filter(
                user=serializers.validated_data['email'], purpose='EMAIL VERIFICATION', is_active=True).first()
            time_difference = timezone.now() - user_token.updated_at

            if (time_difference.seconds/60) % 60 < 1:
                response['message'] = "Please Retry after 1 minute"
                response['result'] = 1
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)

            message = 'Verification Link Has Been Send on {} Email Addrees to verify email id'.format(
                serializers.validated_data['email'].email)
            verify_redirect = "{}{}".format(
                "https://sportsbookingnew.worldindia.com/#/success/", user_token)
            send_mail_registeration(
                serializers.validated_data['email'].first_name, verify_redirect, serializers.validated_data['email'].email)

            user_token.updated_at = timezone.now()
            user_token.save()

            response['message'] = message
            response['result'] = 1
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        else:
            response = handle_errors(serializers.errors)
            response['result'] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class EmailRegisterationValidateAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    serializer_class = RegisterationPrivateKeySerializer

    def post(self, request, format=None):

        response = {}
        responsedata = []

        serializers = self.serializer_class(data=request.data)

        if serializers.is_valid():
            if UserToken.objects.filter(key=serializers.validated_data['key']).exists():
                token_data = UserToken.objects.filter(
                    key=serializers.validated_data['key']).first()
            else:
                response['result'] = 0
                responsedata.append("No Such token")
                response['errors'] = responsedata
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)

            if token_data.is_active == False:
                response['result'] = 0
                responsedata.append("Token is already activated")
                response['errors'] = responsedata
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)

            user = token_data.user
            user.is_active = True
            user.save()
            token_data.is_active = False
            token_data.last_used = timezone.now()
            token_data.save()
            message = "Successfully activated"

            response['result'] = 1
            responsedata.append(message)
            response['errors'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        else:
            response = handle_errors(serializers.errors)
            response['result'] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class PrivateKeyVerifyAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    serializer_class = RegisterationPrivateKeySerializer
    """
	Verify Registeration Private Key 
	"""

    def post(self, request, format=None):
        response = {}
        responsedata = []

        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            # Check if key exists
            if RegisterPrivateKey.objects.filter(key=serializers.validated_data['key'], is_active=True).exists():
                # Get user object
                token_obj = RegisterPrivateKey.objects.get(
                    key=serializers.validated_data['key'], is_active=True)
                user_obj = token_obj.user
                # Send response
                response['result'] = 1
                responsedata.append("Key Found")
                response['data'] = responsedata
                response['email'] = user_obj.email
                response['first_name'] = user_obj.first_name
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)

            else:
                # Send response
                response['result'] = 0
                responsedata.append("No Key found")
                response['errors'] = responsedata
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_400_BAD_REQUEST)
        else:
            response = handle_errors(serializers.errors)
            response['result'] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)
