import pytz

from django.http import HttpResponse, HttpResponseRedirect
from django.template import Template, Context
from django.template.loader import get_template
from django.db import transaction
from django.urls import reverse

from django.contrib.auth import login as django_login

import datetime

from rest_framework.authtoken.models import Token
from rest_framework import status, generics
from rest_framework.response import Response
from rest_framework.authentication import TokenAuthentication
from rest_framework.permissions import IsAuthenticated

from utils.mixins import *

from utils.api_handle_errors import handle_errors
from utils.mail import email_verification, send_mail_for_reset_password
from players.serializers import *
from platform_central.models import UserToken
# from field_booking.functions import send_sms
from field_booking.auth_backend import EmailBackend
from field_booking.settings import DOMAIN_NAME

from platform_auth.serializers import *


class ResetPasswordMailAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    '''
    Forgot Password
    '''
    serializer_class = ResetPasswordSerializers

    def post(self, request, format=None):

        response = {}
        responsedata = []
        serializer = self.serializer_class(data=request.data)
        if serializer.is_valid():
            if User.objects.filter(username=serializer.validated_data['email_address'], is_active=True).exists():
                user = User.objects.get(
                    username=serializer.validated_data['email_address'])
            else:
                response['errors'] = ['No Such Record Exists']
                response['result'] = 0
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)

            email_verification_token = UserToken.objects.create(
                user=user, purpose='RESET PASSWORD')

            message = 'Verification Link Has Been Send to {} Email id.Click on it to reset your Password'.format(
                serializer.validated_data['email_address'])

            verify_redirect = "{}{}{}".format(
                "https://sportsbookingnew.worldindia.com/#/", "reset-Password/", email_verification_token)

            images = Template_Images.objects.filter().values('template_image')
            logo = images[0]
            logo2 = images[1]

            send_mail_for_reset_password(
                user.first_name, verify_redirect, serializer.validated_data['email_address'], logo, logo2)

            response['message'] = message
            response['result'] = 1
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        else:
            response = handle_errors(serializer.errors)
            response['result'] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class ConfirmTokenResetPasswordAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    """
    Confirming the token for forgot password
    """
    serializer_class = ConfirmTokenPasswordSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []

        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():

            if UserToken.objects.filter(key=serializers.validated_data['token'], is_active=True, purpose="RESET PASSWORD").exists():
                token_obj = UserToken.objects.select_related('user').get(
                    key=serializers.validated_data['token'], is_active=True, purpose="RESET PASSWORD")
            else:
                response['errors'] = ['Invalid token']
                response['result'] = 0
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)

            # Get User Email address
            email_address = token_obj.user.email

            # Send response
            response['email'] = email_address
            response['result'] = 1
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        else:
            response = handle_errors(serializers.errors)
            response['result'] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class ResetPasswordAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    '''
    Forgot Password
    '''
    serializer_class = VerifyPasswordTokenSerializer

    def post(self, request, format=None):

        response = {}
        responsedata = []
        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            user = serializers.validated_data['Token'].user
            user.set_password(serializers.validated_data['password'])
            user.save()
            serializers.validated_data['Token'].is_active = False
            serializers.validated_data['Token'].last_used = datetime.datetime.now(
            ).strftime("%Y-%m-%d %H:%M:%S")
            serializers.validated_data['Token'].save()
            message = "Successfully changed Password"
            response['message'] = message
            response['result'] = 1
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializers.errors)
            response['result'] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)
