from django.contrib import admin

from nested_admin import NestedModelAdmin

from .models import *


def custom_titled_filter(title):
    class Wrapper(admin.FieldListFilter):
        def __new__(cls, *args, **kwargs):
            instance = admin.FieldListFilter.create(*args, **kwargs)
            instance.title = title
            return instance
    return Wrapper


@admin.register(Player)
class PlayerAdmin(NestedModelAdmin):
    model = Player
    list_display = ['id', 'user', 'first_name', 'middle_name', 'last_name', 'email', 'new_email', 'mobile', 'gender',
                    'street', 'pincode', 'image', 'dob', 'is_deleted', 'is_email_verified', 'created_at', 'updated_at']

# admin.site.register(Player, PlayerAdmin)


@admin.register(PlayerSkills)
class PlayerSkillAdmin(NestedModelAdmin):
    model = PlayerSkills
    search_fields = ['Template_Name']
    list_display = ['id', 'player', 'sport', 'position', 'created_at']
    # list_display = ['id', 'player', 'sport', 'position', 'level', 'created_at']


@admin.register(PlayerAvailability)
class PlayerAvailabilityAdmin(NestedModelAdmin):
    model = PlayerAvailability
    search_fields = ['player']
    list_display = ['id', 'player', 'start_time', 'end_time']


admin.site.register(FriendList)
admin.site.register(PlayerNotifyContactCronScheduler)
admin.site.register(PlayerNotifyContactAvailabilityDates)
admin.site.register(NewsLetter)
admin.site.register(VenueListPlayer)


@admin.register(TeamDetails)
class TeamDetailsAdmin(NestedModelAdmin):
    model = TeamDetails
    list_display = ['id', 'team_name', 'sport',
                    'date_founded', 'country', 'created_by']


@admin.register(PlayerInterest)
class PlayerInterestAdmin(NestedModelAdmin):
    model = PlayerInterest
    list_display = ['id', 'player', 'interest_sports',
                    'created_at', 'is_deleted']


@admin.register(PlayerOpponentSearchInterest)
class PlayerOpponentSearchInterestAdmin(NestedModelAdmin):
    model = PlayerOpponentSearchInterest
    list_display = ['id', 'player', 'interested_for_opponent',
                    'created_at', 'is_deleted']


@admin.register(TeamContacts)
class TeamContactAdmin(NestedModelAdmin):
    model = TeamContacts
    list_display = ['id', 'team_details',
                    'first_name', 'last_name', 'mobile', 'email']


@admin.register(TeamCalendar)
class TeamCalendarAdmin(NestedModelAdmin):
    model = TeamCalendar
    list_display = ['id', 'team_details', 'day', 'start_time', 'end_time']


@admin.register(BookMatch)
class BookMatchAdmin(NestedModelAdmin):
    model = BookMatch
    list_display = ['id', 'venue_field', 'status', 'created_by',
                    'total_cost']  # 'schedule_slot','services_cost'


@admin.register(TeamMember)
class TeamMemberAdmin(NestedModelAdmin):
    model = TeamMember
    list_display = ['id', 'team_details', 'player', 'email', 'first_name',
                    'last_name', 'position', 'status', 'is_deleted', 'created_by', 'created_at']


@admin.register(PlayerInvitation)
class PlayerInvitationAdmin(NestedModelAdmin):
    model = PlayerInvitation
    list_display = ['id', 'recipient', 'category',
                    'status', 'sender', 'created_by']


@admin.register(VenuePaidServiceQuantity)
class VenuePaidServiceQuantityAdmin(NestedModelAdmin):
    model = VenuePaidServiceQuantity
    list_display = ['id', 'book_match', 'venue_service_cost',
                    'quantity', 'created_at', 'is_deleted']


@admin.register(VenueFieldServiceQuantity)
class VenueFieldServiceQuantityAdmin(NestedModelAdmin):
    model = VenueFieldServiceQuantity
    list_display = ['id', 'book_match', 'venue_field_service',
                    'quantity', 'created_at', 'is_deleted']


@admin.register(PlayerMyContactsPlatform)
class PlayerMyContactsPlatformAdmin(NestedModelAdmin):
    model = PlayerMyContactsPlatform
    list_display = ['id', 'email',
                    'status', 'created_at', 'is_deleted', ]
