import random
from django.utils import timezone
from rest_framework import serializers
from django.contrib.auth.models import User, Group
from rest_framework.authtoken.models import Token

from datetime import date
import datetime
import calendar

from .models import *
from platform_central.models import UserFcmObject
from commonAPI.models import TelephoneNumbers
from commonAPI.serializers import *
from utils.api_handle_errors import handle_errors
from django.db import transaction
from venues.models import Venue, VenueField, VenueDailyTiming
from venues.serializers import Covid19MeasuresSerializer, VenueFieldBookingMatchSerializer
from venues.models import Venue, VenueField, VenueDailyTiming, VenueRating
from static_data.models import Area, FieldType, Sport
from static_data.serializers import SportSerializer, VenueServiceSerializer
from venue_owners.serializers import Schedule_SlotSerializer, ScheduleSlotSerializer, AdminVenueApprovalSerializer
from django.utils import timezone


class NotifyContactsSerializer(serializers.Serializer):
    contacts = serializers.JSONField(required=True, help_text="Players ID")
    copy_on = serializers.JSONField(
        required=False, help_text='[1,2,3,4,5] id of days of week')
    repeat_until = serializers.DateField(
        required=False, help_text='YYYY-MM-DD')
    date = serializers.DateField(required=True, help_text="date")
    start_time = serializers.TimeField(help_text="HH:MM 24hrs")
    end_time = serializers.TimeField(help_text="HH:MM 24hrs")
    sport = serializers.PrimaryKeyRelatedField(queryset=Sport.objects.all())

    def validate_copy_on(self, value):
        data = list()
        for i in value:
            day_of_week = DayofWeek.objects.filter(id=i)
            if day_of_week.exists():
                data.append(day_of_week.last().dayofweek)
            else:
                raise serializers.ValidationError("Incorrect Day Entered")
        return data

    def validate_contacts(self, values):
        return_data = list()
        for value in values:
            contacts_obj = Player.objects.filter(id=value)
            if contacts_obj.exists():
                return_data.append(contacts_obj.last())
            else:
                raise serializers.ValidationError("Invalid Contact ID")
        return return_data

    def validate_sport(self, value):
        return value.name

    def __copy_on(self, copy_on, repeat_until, start_date=datetime.date.today()):
        dates = [start_date]
        days_to_repeat = [i for i in copy_on]
        end_date = repeat_until
        while start_date <= end_date:
            day = calendar.day_name[start_date.weekday()]
            if day.upper() in days_to_repeat:
                dates.append(start_date)
            start_date += datetime.timedelta(days=1)
        return dates

    def save(self):
        validated_data = self.validated_data
        repeat_unitl = validated_data.pop("repeat_until")
        copy_on = validated_data.pop("copy_on")
        start_date = validated_data.pop("date")
        dates = self.__copy_on(copy_on, repeat_unitl, start_date)
        print(dates)

        contacts = self.validated_data['contacts']

        for i in contacts:
            notify_contact = PlayerNotifyContactCronScheduler.objects.create(
                to_player=i, from_player=self.context['player'])
            contact_availability = [
                PlayerNotifyContactAvailabilityDates(
                    player_notify_contact=notify_contact,
                    date=date,
                    start_time=validated_data['start_time'],
                    day=calendar.day_name[date.weekday()],
                    end_time=validated_data['end_time'],
                    sport=validated_data['sport']
                ) for date in dates
            ]

            PlayerNotifyContactAvailabilityDates.objects.bulk_create(
                contact_availability)

        return True


class PlayerNotifyContactAvailabilityDatesSerializer(serializers.ModelSerializer):
    class Meta:
        model = PlayerNotifyContactAvailabilityDates
        fields = '__all__'


class SocialSerializer(serializers.Serializer):
    """
    Serializer which accepts an OAuth2 access token and provider.
    """
    provider = serializers.CharField(max_length=255, required=True)
    access_token = serializers.CharField(
        max_length=4096, required=True, trim_whitespace=True)


class UserSerializer(serializers.ModelSerializer):
    class Meta:
        model = User
        fields = ('pk', 'username', 'first_name',
                  'last_name', 'email', 'date_joined')


class WebUserLoginSerializer(serializers.ModelSerializer):
    fcm_id = serializers.CharField(max_length=200, required=False)
    device_type = serializers.CharField(max_length=200, required=False)
    device_id = serializers.CharField(max_length=200, required=False)

    class Meta:
        model = User
        fields = ('email', 'password', 'fcm_id', 'device_type', 'device_id')
        extra_kwargs = {
            'email': {'required': True, 'help_text': 'registered email id'},
            'fcm_id': {'required': False},
            'device_type': {'required': False},
            'device_id': {'required': False}
        }


class VerifyUserInputSerializer(serializers.ModelSerializer):
    app_version = serializers.CharField(
        max_length=100, required=False, help_text="app_version")

    class Meta:
        model = UserFcmObject
        fields = ('device_id', 'app_version', 'fcm_id', 'device_type')
        extra_kwargs = {
            'device_id': {'required': False}
        }


class LogoutSerializer(serializers.ModelSerializer):
    class Meta:
        model = Token
        fields = ()


class WebRegisterationInputSerializer(serializers.ModelSerializer):
    class Meta:
        model = Player
        fields = ('first_name', 'middle_name',
                  'last_name', 'email', 'gender', 'dob')
        extra_kwargs = {
            'first_name': {'required': True, 'help_text': 'first name'},
            'email': {'required': True, 'help_text': 'last name'},
            'gender': {'required': True, 'help_text': 'M/F/O'},
            'dob': {'required': True, 'help_text': 'date of birth'},
            'middle_name': {'required': False, 'help_text': 'middle name'},
            'last_name': {'required': False, 'help_text': 'last name'},
        }


############### PLAYER PROFILE #################################


class PlayerProfileSerializer(serializers.ModelSerializer):
    user = serializers.CharField(source="user.username", read_only=True)
    email = serializers.CharField(read_only=True)

    class Meta:
        model = Player
        fields = '__all__'


class ChangeProfilePicSerializer(serializers.ModelSerializer):
    class Meta:
        model = Player
        fields = ['image']


class PlayerSAVESerializer(serializers.ModelSerializer):
    class Meta:
        model = Player
        fields = '__all__'


class PlayerGETSerializer(serializers.ModelSerializer):
    class Meta:
        model = Player
        fields = ['id', 'first_name', 'middle_name', 'last_name',
                  'email', 'gender', 'city', 'pincode', 'street', 'dob', 'image']


class PlayerPersonalInfoSerializer(serializers.ModelSerializer):
    sport = SportSerializer(read_only=True)
    country = CountrySerializer(read_only=True)
    city = CitySerializer(read_only=True)
    state = StateGetSerializer(read_only=True)
    area = AreaGetSerializer(read_only=True)

    class Meta:
        model = Player
        fields = '__all__'
        read_only_fields = ['user', 'is_deleted', 'is_email_verified']
        extra_kwargs = {"email": {"required": False}}


class PlayerGetAllObjectFormatSerializer(serializers.ModelSerializer):
    country = CountrySerializer(read_only=True)
    city = CitySerializer(read_only=True)
    state = StateGetSerializer(read_only=True)
    area = AreaGetSerializer(read_only=True)

    class Meta:
        model = Player
        fields = '__all__'


class PlayerTeamInfoSerializer(serializers.ModelSerializer):
    team_level = TeamLevelSerializer(many=True)
    sport = SportSerializer()
    country = CountrySerializer()
    city = CitySerializer()
    state = StateGetSerializer()
    area = AreaGetSerializer()

    class Meta:
        model = TeamDetails
        fields = "__all__"


class AddPlayerInterestSerializer(serializers.ModelSerializer):
    interest_obj = serializers.JSONField(
        required=False, help_text='[1,2,3,4,5,6] Id of static sports')
    player_opponent_interest = serializers.BooleanField(
        required=False, help_text="True/ False")

    def validate_interest_obj(self, values):
        return_data = list()
        for value in values:
            sports_obj = Sport.objects.filter(id=value)
            if sports_obj.exists():
                return_data.append(sports_obj.last())
            else:
                raise serializers.ValidationError("Not A Valid Sport")
        return return_data

    def save(self):
        validated_data = self.validated_data
        player_interest_data = validated_data['interest_obj']
        player_opponent_interest = validated_data['player_opponent_interest']
        self.Meta.model.objects.filter(
            player=self.context['player']).update(is_deleted=True)
        player_interest = [PlayerInterest(
            interest_sports=i,
            player=self.context['player']
        ) for i in player_interest_data]
        self.Meta.model.objects.bulk_create(player_interest)
        self.Meta.opponent_interest.objects.update_or_create(player=self.context['player'], defaults={
                                                             "interested_for_opponent": player_opponent_interest})
        return True

    class Meta:
        model = PlayerInterest
        opponent_interest = PlayerOpponentSearchInterest
        fields = ['interest_obj', 'player_opponent_interest']


class GetPlayerInterestSerializer(serializers.ModelSerializer):
    interest_sports = serializers.CharField(
        required=False, source="interest_sports.name")

    class Meta:
        model = PlayerInterest
        fields = '__all__'


class TeamDetailsFindOpponentInfoSerializer(serializers.ModelSerializer):
    sport = SportSerializer()
    country = CountrySerializer()
    city = CitySerializer()
    state = StateGetSerializer()
    area = AreaGetSerializer()
    player = PlayerGETSerializer()
    team_level = TeamLevelSerializer(many=True)

    class Meta:
        model = TeamDetails
        fields = "__all__"


class PlayerPersonalInfoPOSTSerializer(serializers.ModelSerializer):
    class Meta:
        model = Player
        fields = '__all__'
        read_only_fields = ['user', 'is_deleted', 'is_email_verified']
        extra_kwargs = {"email": {"required": False}}


class PlayerUserEditSerializer(serializers.ModelSerializer):
    class Meta:
        model = User
        fields = ['first_name', 'last_name']


class PlayerProfilePOSTSerializer(serializers.ModelSerializer):
    Telephone_number = serializers.JSONField(
        required=False, help_text='{"mobile":["847123654","35135654"]}')
    iban = serializers.CharField(required=False)
    bank = serializers.CharField(required=True)
    acc_holder_first_name = serializers.CharField(required=True)
    acc_holder_last_name = serializers.CharField(required=False)

    @transaction.atomic
    def create(self, validated_data):
        '''
        Profession
        '''
        validated_data['user'] = self.context['user'].id
        serialized_data = PlayerSAVESerializer(data=validated_data)
        if serialized_data.is_valid():
            player_data = serialized_data.save()
        else:
            raise serializers.ValidationError(
                handle_errors(serialized_data.errors))

        '''
		Contact Info
		'''
        serialized_data = ContactInfoProfileSerializer(data=validated_data)
        if serialized_data.is_valid():
            serialized_data.validated_data['content_object'] = player_data
            contact_info_data = serialized_data.save()
        else:
            raise serializers.ValidationError(
                handle_errors(serialized_data.errors))

        '''
		Contact Numbers
		'''
        contact_numbers = [TelephoneNumbers(contact_id=contact_info_data,
                                            contact_number=i
                                            ) for i in validated_data['Telephone_number']['mobile']]

        TelephoneNumbers.objects.bulk_create(contact_numbers)

        '''
		Bank Info
		'''
        serialized_data = BankInfoProfileSerializer(data=validated_data)
        if serialized_data.is_valid():
            serialized_data.validated_data['content_object'] = player_data
            serialized_data.save()
        else:
            raise serializers.ValidationError(
                handle_errors(serialized_data.errors))

        return player_data.id

    class Meta:
        model = Player
        fields = '__all__'
        read_only_fields = ['user']
        extra_kwargs = {
            'email': {
                'required': False
            },
            "user": {
                'required': False
            }
        }


#################################################################

# Player Skills


class PlayerSkillGETSerializer(serializers.ModelSerializer):
    player_id = serializers.IntegerField(
        required=False, help_text="ID of the Player", write_only=True)
    player = serializers.CharField(source='player.first_name')
    sport = serializers.CharField(source="sport.name")
    position = serializers.CharField(source="position.position")

    def __init__(self, *args, **kwargs):
        super(PlayerSkillGETSerializer, self).__init__(*args, **kwargs)
        for field in self.fields:
            if field != 'player_id':
                self.fields[field].read_only = True

    def validate_player_id(self, value):
        if Player.objects.filter(id=value).exists():
            return value
        else:
            raise serializers.FieldDoesNotExist("No Record Found")

    class Meta:
        model = PlayerSkills
        fields = '__all__'


class PlayerSkillPOSTSerializer(serializers.Serializer):
    data = serializers.JSONField(
        required=False,
        help_text='[{"sport":1,"position":1,"level":4}, {"sport":1,"position":1,"level":4},{"sport":0,"position":0,"level":0}]')

    def create(self, validate_data):
        for count, i in enumerate(validate_data['data']):
            if i['sport'] == 0:
                validate_data['data'].remove(i)
            else:
                i['sport'] = Sport.objects.get(id=i.get("sport"))
                i['position'] = Position.objects.get(id=i.get("position"))
                i['level'] = PlayerLevel.objects.get(id=i.get("level"))

        PlayerSkills.objects.filter(
            created_by=self.context['created_by'], player=self.context['player']).delete()

        skills_obj = [PlayerSkills(
            **i,
            created_by=self.context['created_by'],
            player=self.context['player']
        ) for i in validate_data['data']]
        PlayerSkills.objects.bulk_create(skills_obj)
        return True


class PersonalSkillsDeleteSerializer(serializers.Serializer):
    id = serializers.IntegerField(
        required=False, help_text="ID of personal SKills")


# FIND OPPONENT ###################3


class FindOpponentSerializer(serializers.Serializer):
    keyword = serializers.CharField(
        required=False, help_text="Search")
    level = serializers.JSONField(required=False, help_text=[1, 2, 3, 4])
    sport = serializers.CharField(
        required=False, help_text="Sports PK")
    availability_date = serializers.DateField(
        required=False, help_text="Date of player availability")
    start_time = serializers.TimeField(
        required=False, help_text="Time of player availability")
    end_time = serializers.TimeField(
        required=False, help_text="Time of player availability")
    location = serializers.JSONField(required=False, help_text='[1,2,3]')

    def validate_sport(self, value):
        sport = Sport.objects.filter(id=int(value))
        if sport.exists():
            return sport.last()
        raise serializers.ValidationError("Nota valid sport")


class OpponentInvitationSerializer(serializers.Serializer):
    player_id = serializers.IntegerField(required=False, help_text="Player Id")


class PlayerSkillsOpponentSerializer(serializers.ModelSerializer):
    player = PlayerGETSerializer()
    sport = serializers.CharField(source="sport.name", required=False)
    level_filter = serializers.JSONField(
        required=False, help_text="[1,2,3,4]", write_only=True)

    class Meta:
        model = PlayerSkills
        fields = '__all__'


class GetPlayerSkillsIDSerializer(serializers.ModelSerializer):
    player_id = serializers.PrimaryKeyRelatedField(queryset=Player.objects.all(
    ), required=True, help_text="Player ID", write_only=True)
    sport = serializers.CharField(required=False, source="sport.name")
    position = serializers.CharField(
        required=False, source="position.position")

    class Meta:
        model = PlayerSkills
        fields = '__all__'


############### AVAILABILITY #################################


class FriendlistSerializer(serializers.ModelSerializer):
    contacts = PlayerGETSerializer(many=True)
    user = serializers.CharField(source="user.first_name")

    class Meta:
        model = FriendList
        fields = '__all__'


class PlayerAvailabilitySerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(required=False, help_text="ID to edit")
    copy_on = serializers.JSONField(
        required=False, help_text='[1,2,3,4,5] id of days of week')
    repeat_until = serializers.DateField(
        required=False, help_text='YYYY-MM-DD')

    def validate_id(self, value):
        if PlayerAvailability.objects.filter(id=value).exists:
            return value
        else:
            raise serializers.FieldDoesNotExist("No Record Found")

    def validate_copy_on(self, value):
        data = list()
        for i in value:
            day_of_week = DayofWeek.objects.filter(id=i)
            if day_of_week.exists():
                data.append(day_of_week.last())
            else:
                raise serializers.ValidationError("Incorrect Day Entered")
        return data

    def __copy_on(self, copy_on, repeat_until, start_date=date.today()):
        dates = [start_date]
        days_to_repeat = [i.dayofweek for i in copy_on]
        end_date = repeat_until
        while start_date <= end_date:
            day = calendar.day_name[start_date.weekday()]
            if day.upper() in days_to_repeat:
                dates.append(start_date)
            start_date += datetime.timedelta(days=1)
        return dates

    def create(self, validated_data):
        player_obj = Player.objects.filter(user=self.context['user'])
        if player_obj.exists():
            player_obj = player_obj.last()
        else:
            raise serializers.ValidationError("Invalid Player")

        repeat_unitl = validated_data.pop("repeat_until")
        copy_on = validated_data.pop("copy_on")
        start_date = validated_data.pop("date")
        dates = self.__copy_on(copy_on, repeat_unitl, start_date)

        data = [PlayerAvailability(
            **validated_data,
            player=player_obj,
            date=date)
            for date in dates
        ]

        self.Meta.model.objects.filter(date__in=dates, player=Player.objects.get(
            user=self.context['user'])).update(is_deleted=True)
        self.Meta.model.objects.bulk_create(data)
        return_data = self.Meta.model.objects.filter(
            date__in=dates, player=Player.objects.get(user=self.context['user']), is_deleted=False)
        return_class = PlayerAvailabilitySerializer(return_data, many=True)
        return return_class.data

    class Meta:
        model = PlayerAvailability
        fields = '__all__'


class GetFieldPriceSerializer(serializers.Serializer):
    field_id = serializers.IntegerField(required=False, help_text="Field iD")
    booking_date = serializers.DateField(required=False, help_text="Date")
    start_hour = serializers.TimeField(required=False, help_text="Time")
    end_hour = serializers.TimeField(required=False, help_text="Time")


class PlayerAvailabilityDeleteSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(required=False, help_text="ID to Delete")

    def validate_id(self, value):
        if PlayerAvailability.objects.filter(id=value, is_deleted=False).exists():
            return value
        else:
            raise serializers.FieldDoesNotExist("No Record Found")

    class Meta:
        model = PlayerAvailability
        fields = ['id']


############## NEWS LEtter ##########################


class NewsLetterSerializer(serializers.ModelSerializer):
    email = serializers.CharField(required=False, help_text="Email address")

    class Meta:
        model = NewsLetter
        fields = '__all__'


# TEAM CREATE  #############################333


class TeamDetailPOSTSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(
        required=False, help_text="Id if want to edit it")
    team_level = serializers.JSONField(required=False, help_text="[1,2,3]")
    state = serializers.IntegerField(
        required=False, help_text="state")
    area = serializers.IntegerField(
        required=False, help_text="area")
    city = serializers.IntegerField(
        required=False, help_text="city")
    country = serializers.IntegerField(
        required=False, help_text="country")

    def validate_country(self, value):
        country_obj = Country.objects.filter(id=value)
        if country_obj.exists():
            country_obj = country_obj.last()
        else:
            raise serializers.ValidationError("Invalid Country")

    def validate_state(self, value):
        state_obj = State.objects.filter(id=value)
        if state_obj.exists():
            return state_obj.last()
        else:
            raise serializers.ValidationError("Invalid State")

    def validate_city(self, value):
        city_obj = City.objects.filter(id=value)
        if city_obj.exists():
            return city_obj.last()
        else:
            raise serializers.ValidationError("Invalid City")

    def validate_area(self, value):
        area_obj = Area.objects.filter(id=value)
        if area_obj.exists():
            return area_obj.last()
        raise serializers.ValidationError("Invalid Area")

    def create(self, validated_data):
        validated_data['player'] = Player.objects.get(
            user=self.context['user'])
        validated_data['created_by'] = self.context['user']
        player_team_level = validated_data.pop("team_level", "")
        data = TeamDetails.objects.create(**validated_data)
        data.team_level.set(player_team_level)

        # Create team member with owner status
        TeamMember.objects.create(first_name=validated_data['player'].first_name,
                                  last_name=validated_data['player'].last_name,
                                  email=validated_data['player'].email,
                                  player=validated_data['player'], team_details=data, status="APPROVED", is_owner=True)

        return data

    class Meta:
        model = TeamDetails
        fields = "__all__"


class TeamMemberSendRequestSerializer(serializers.ModelSerializer):
    id = serializers.PrimaryKeyRelatedField(
        required=False, queryset=TeamMember.objects.all())

    def update(self, instance, validated_data):
        print(instance)
        print(type(validated_data))
        validated_data.pop('id')
        for key, value in validated_data.items():
            print(key)
            print(value)
            setattr(instance, key, value)
        instance.save()
        return True

    def create(self, validate_data):
        validate_data.pop("id")
        team_member, created = TeamMember.objects.update_or_create(
            email=validate_data['email'], team_details=validate_data['team_details'], defaults=validate_data)
        return team_member

    class Meta:
        model = TeamMember
        fields = "__all__"


class TeamMemberGetSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(
        required=True, help_text="Send only ID of Team Details")

    class Meta:
        model = TeamMember
        fields = "__all__"
        read_only_fields = ['player', 'team_details', 'email', 'first_name', 'last_name',
                            'position', 'status', 'is_deleted', 'is_owner', 'created_by', 'created_at']


class GetTeamInputSerializer(serializers.Serializer):
    STATUS_CHOICE = (
        ("Pending", "Pending"),
        ("Approved", "Approved"),
        ("Declined", "Declined")
    )
    team_detail_id = serializers.PrimaryKeyRelatedField(
        queryset=TeamDetails.objects.all(), required=False)
    team_name = serializers.CharField(required=False)
    sport = serializers.PrimaryKeyRelatedField(
        queryset=Sport.objects.all(), required=False)
    status = serializers.ChoiceField(choices=STATUS_CHOICE, required=False)


class GetTeamSerializer(serializers.ModelSerializer):
    sport = serializers.SerializerMethodField("_get_sport")
    player = serializers.SerializerMethodField("_get_player_count")

    def _get_player_count(self, obj):
        count_team_members = TeamMember.objects.filter(
            team_details=obj, is_deleted=False, status="Approved").count()
        return count_team_members

    def _get_sport(self, obj):
        try:
            sport_obj = obj.sport
            sport_name = sport_obj.name
            return sport_name+" TEAM"
        except:
            return ""

    class Meta:
        model = TeamDetails
        fields = "__all__"


class TeamMemberDeleteRequestSerialzier(serializers.Serializer):
    member_id = serializers.PrimaryKeyRelatedField(queryset=TeamMember.objects.filter(
        is_deleted=False), required=True, help_text="Player Id to delete")


class TeamContactsSerializer(serializers.ModelSerializer):
    data = serializers.JSONField(
        required=False, help_text='[{"first_name":"yatin","last_name":"pandit","mobile":"771596454","email":"aa@aa"}]')
    team_detail = serializers.IntegerField(
        required=True, help_text='id of previous Step', write_only=True)

    def validate_team_detail(self, value):
        if TeamDetails.objects.filter(id=value).exists():
            return TeamDetails.objects.get(id=value)
        else:
            raise serializers.ValidationError("No Such Record")

    def create(self, validated_data):
        TeamContacts.objects.filter(
            created_by=self.context, team_details=validated_data['team_detail']).delete()
        team_contact = [TeamContacts(**i, team_details=validated_data['team_detail'], created_by=self.context)
                        for i in validated_data['data']
                        ]
        TeamContacts.objects.bulk_create(team_contact)

        return True

    class Meta:
        model = TeamContacts
        fields = '__all__'
        read_only_fields = ['team_details', 'first_name',
                            'last_name', 'mobile', 'email', 'created_at']


class TeamCalendarSerializer(serializers.ModelSerializer):
    data = serializers.JSONField(
        required=True, help_text='[{"day":1,"start_time":"21:00","end_time":"22:00"}]', write_only=True)
    team_detail = serializers.IntegerField(required=True, write_only=True)
    day = serializers.CharField(source='day.dayofweek', read_only=True)

    def validate_team_detail(self, value):
        if TeamDetails.objects.filter(id=value).exists():
            return TeamDetails.objects.get(id=value)
        else:
            raise serializers.FieldDoesNotExist("No Such Record")

    def validate_data(self, value):
        for i in value:
            i['day'] = DayofWeek.objects.get(id=i['day'])
        return value

    def create(self, validated_data):
        TeamCalendar.objects.filter(
            team_details=validated_data['team_detail']).update(is_deleted=True)
        team_calendar = [TeamCalendar(**i, team_details=validated_data['team_detail'], created_by=self.context)
                         for i in validated_data['data']
                         ]
        TeamCalendar.objects.bulk_create(team_calendar)
        return True

    class Meta:
        model = TeamCalendar
        fields = '__all__'
        read_only_fields = ['start_time', 'end_time', 'created_at', 'day']


class TeamDetailGETSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(
        required=False, help_text="Send ID to get specifc data")
    created_by = serializers.CharField(
        source="created_by.first_name", read_only=True)

    def validate_id(self, value):
        if TeamDetails.objects.filter(id=value).exists():
            return value

    class Meta:
        model = TeamDetails
        fields = "__all__"
        read_only_fields = ['team_name', 'date_founded',
                            'team_image', 'sport', 'country', 'city']


class TeamRequestSerializer(serializers.ModelSerializer):
    team = serializers.CharField(
        source="team_details.team_name", required=False, read_only=True)
    position = serializers.CharField(
        source="position.position", required=False, read_only=True)
    page = serializers.IntegerField(
        default=1, write_only=True, help_text="For received")
    sent_page = serializers.IntegerField(
        default=1, write_only=True, help_text="For sent")

    class Meta:
        model = TeamMember
        fields = ['id', 'team', 'sent_page', 'player', 'page', 'email',
                  'first_name', 'last_name', 'position', 'status', 'created_at']
        read_only_fields = ['id', 'team', 'player', 'email',
                            'first_name', 'last_name', 'position', 'status', 'created_at']


class MyJoinsSerializer(serializers.ModelSerializer):
    STATUS_CHOICE = (
        ("Pending", "Pending"),
        ("Approved", "Approved"),
        ("Declined", "Declined")
    )
    created_by = serializers.CharField(
        source="created_by.first_name", required=False)
    page = serializers.IntegerField(
        default=1, write_only=True, help_text="For received")
    sent_page = serializers.IntegerField(
        default=1, write_only=True, help_text="For sent")
    status = serializers.ChoiceField(
        choices=STATUS_CHOICE, default=STATUS_CHOICE[0][0], required=False, help_text="Pending / Approved / Declined")

    class Meta:
        model = PlayerInvitation
        fields = ['id', 'recipient', 'category', 'status', 'sender',
                  'created_by', 'sent_page', 'page', 'created_at']
        read_only_fields = ['id', 'recipient', 'category',
                            'sender', 'created_by', 'created_at']


class ChangeJoinStatusSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(required=True, help_text="Id of the join")

    class Meta:
        model = PlayerInvitation
        fields = ['id', 'status']


class TeamSentRequestSerializer(serializers.ModelSerializer):
    class Meta:
        model = TeamDetails
        fields = ['id', 'team_name', 'sport',
                  'country', 'city', 'date_founded']


################# Search Venue ##############


class GetVenueSerializer(serializers.Serializer):
    ASSORTMENT_CHOICE = (
        ("Price Low", "Price Low"),
        ("Star Rating", "Star Rating"),
        ("Star and Price", "Star and Price"),
        ("Distance", "Distance")
    )

    keyword = serializers.CharField(
        required=False, help_text="Venue Name Keywords")
    all_sports = serializers.CharField(required=False)
    area = serializers.IntegerField(required=False)
    booking_date = serializers.DateField(required=False)
    start_time = serializers.TimeField(required=False)
    end_time = serializers.TimeField(required=False)
    star_ratings = serializers.JSONField(required=False, help_text='[1,2,3]')
    page = serializers.IntegerField(
        required=False, default=1, help_text="Page number")
    field_type = serializers.PrimaryKeyRelatedField(
        required=False, queryset=FieldType.objects.all(), help_text="Field type ID")
    assortment = serializers.ChoiceField(
        choices=ASSORTMENT_CHOICE, required=False, help_text="Top Sorting Filters")
    price_range = serializers.JSONField(
        required=False, help_text='["0 10", "10 20", "20 30"]')
    popular_filters = serializers.JSONField(
        required=False, help_text="['as','sa']")
    availability = serializers.BooleanField(
        required=False, default=True, help_text="To show unavailable Fields")
    location = serializers.JSONField(required=False, help_text='[1,2,3]')
    attributes = serializers.JSONField(
        required=False, help_text='["INDOOR","OUTDOOR","YES","NO"]')


class TeamDetailAllSerializer(serializers.ModelSerializer):
    class Meta:
        model = TeamDetails
        fields = '__all__'


class MyMatchesFilterSerializer(serializers.Serializer):
    MATCH_STATUS_CHOICE = (
        ("Upcoming", "Upcoming"),
        ("Completed", "Completed"),
        ("Cancelled", "Cancelled")
    )
    all_sports = serializers.CharField(required=False)
    venue = serializers.CharField(required=False)
    status = serializers.ChoiceField(
        choices=MATCH_STATUS_CHOICE, required=False, help_text="Upcoming / Completed / Cancelled")
    start_date = serializers.DateField(required=False, help_text='YYYY-MM-DD')
    end_date = serializers.DateField(required=False, help_text='YYYY-MM-DD')
    start_time = serializers.TimeField(
        required=False, help_text="Format = HH:MM:SS")
    end_time = serializers.TimeField(
        required=False, help_text="Format = HH:MM:SS")
    ordering = serializers.CharField(
        required=False, default='Descending', help_text="Ascending / Descending")
    page = serializers.IntegerField(
        required=False, default=1, help_text="Page number")


class BookMatchMyMatchesSerializer(serializers.ModelSerializer):
    sport = SportSerializer(read_only=True)
    venue_field = VenueFieldBookingMatchSerializer()
    my_team = TeamDetailAllSerializer()
    opponent_team = TeamDetailAllSerializer()
    schedule_slot = ScheduleSlotSerializer()

    class Meta:
        model = BookMatch
        fields = '__all__'


class VenueDetailsSerializer(serializers.Serializer):
    id = serializers.IntegerField(required=True)
    booking_date = serializers.DateField(required=True)

    def validate_id(self, value):
        if Venue.objects.filter(id=value, is_deleted=False).exists():
            return value
        else:
            raise serializers.FieldDoesNotExist("No record with ID")


class GetVenueScheduleSlotSerializer(serializers.ModelSerializer):
    class Meta:
        model = Schedule_Slot
        fields = ['id', 'start_hour', 'end_hour', 'cost']


class GetVenueInMapSerializer(serializers.ModelSerializer):
    all_sports = SportSerializer(required=False, many=True, read_only=True)
    all_services = VenueServiceSerializer(
        required=False, many=True, read_only=True)

    class Meta:
        model = Venue
        fields = '__all__'
        read_only_fields = ['name', 'logo', 'all_sports', 'allowed_entry', 'all_services', 'area', 'description',
                            'street',
                            'city', 'pincode', 'country', 'venue_number', 'venue_buisness_vat', 'tax_offices',
                            'working_hours', 'is_deleted']


class VenueProfileGETSerializer(serializers.ModelSerializer):
    all_sports = SportSerializer(read_only=True, many=True)
    all_services = VenueServiceSerializer(read_only=True, many=True)
    covid19Measure_description = Covid19MeasuresSerializer(
        read_only=True, many=True)

    class Meta:
        model = Venue
        fields = '__all__'


class VenueFieldDetailPageSerializer(serializers.ModelSerializer):
    floor_type = serializers.CharField(
        source="floor_type.name", required=False)
    field_type = serializers.CharField(
        source="field_type.name", required=False)

    class Meta:
        model = VenueField
        fields = '__all__'


class VenueTimingSerializer(serializers.ModelSerializer):
    day = serializers.CharField(source="day.dayofweek", required=False)

    class Meta:
        model = VenueDailyTiming
        fields = ['id', 'day', 'start_time', 'end_time']


class VenueDailyTimingSerializer(serializers.ModelSerializer):
    day = serializers.CharField(source="day.dayofweek", required=False)

    class Meta:
        model = VenueDailyTiming
        fields = "__all__"


##### FIELD DETAILS  ######


class VenueFieldDetailsSerializer(serializers.Serializer):
    id = serializers.IntegerField(required=True)
    booking_date = serializers.DateField(
        required=False, help_text="Booking Date")
    start_hour = serializers.TimeField(required=False, help_text="Time")
    end_hour = serializers.TimeField(required=False, help_text="Time")

    def validate_id(self, value):
        if VenueField.objects.filter(id=value).exists():
            return value
        else:
            raise serializers.FieldDoesNotExist("No record with ID")


# Book Match steps ################3

class GetPlayerFavVenuesSerializer(serializers.Serializer):
    page = serializers.IntegerField(
        required=False, default=0, help_text="Page Number of field. Default 0")
    booking_date = serializers.DateField(required=False, default=timezone.now(
    ).date(), help_text="Booking date. Default Current")
    id = serializers.IntegerField(required=False, help_text="ID")


class GetFieldAutoCompleteSerializer(serializers.ModelSerializer):
    keyword = serializers.CharField(
        required=True, help_text="Keyword", write_only=True)

    class Meta:
        model = VenueField
        fields = ['id', 'name', 'keyword']
        read_only_fields = ['id', 'name']


class VenueFieldBookMatchSerializer(serializers.ModelSerializer):
    class Meta:
        model = VenueField
        fields = "__all__"


class VenueFieldServiceBookMatchSerializer(serializers.ModelSerializer):
    class Meta:
        model = VenueFieldService
        fields = "__all__"


class BookMatchStepSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(required=False, help_text="2nd step")

    field_services = serializers.JSONField(
        required=False, help_text='[1, 2] Step2', write_only=True)
    available_slot = serializers.JSONField(
        required=False, write_only=True,
        help_text='Step1 {"id": 54,"start_hour": "13:00:00","end_hour": "15:00:00","date": "2020-08-12","is_available": true,"cost": 30,"is_deleted": false,"created_at": "2020-09-08T12:54:33.429856+05:30","schedule_id": 19,"day": 3}')

    field_id = serializers.IntegerField(
        required=False, help_text='1st step', write_only=True)
    venue_field = VenueFieldBookMatchSerializer(read_only=True, required=False)

    schedule_slot = Schedule_SlotSerializer(read_only=True)
    services = VenueFieldServiceBookMatchSerializer(read_only=True, many=True)

    @transaction.atomic
    def update(self, instance, validated_data):
        for i in validated_data['field_services']:
            service = VenueService.objects.get(id=i)
            instance.services.add(service)
            # instance.services_cost = instance.services_cost + service.price_per_hour
        instance.save()
        return instance

    def validate_field_id(self, value):
        if VenueField.objects.filter(id=value, is_deleted=False).exists():
            return VenueField.objects.get(id=value, is_deleted=False)
        else:
            raise serializers.FieldDoesNotExist("No Such Field")

    def create(self, validated_data):
        # data = BookMatch.objects.create(venue_field=validated_data['field_id'],schedule_slot=validated_data['available_slot'],created_by=Player.objects.get(user=self.context['user']),total_cost=validated_data['available_slot'].cost)
        data = BookMatch.objects.create(venue_field=validated_data['field_id'],
                                        schedule_slot=validated_data['available_slot'],
                                        total_cost=validated_data['available_slot'].cost,
                                        created_by=validated_data['created_by'])
        return data

    class Meta:
        model = BookMatch
        fields = "__all__"


class BookMatchCreatedBySerializer(serializers.Serializer):
    id = serializers.IntegerField(required=True, help_text="ID of the Booking")
    first_name = serializers.CharField(
        required=False, help_text="First name incase of guest checkout")
    last_name = serializers.CharField(
        required=False, help_text="Last name incase of guest checkout")
    email = serializers.CharField(
        required=False, help_text="Email incase of guest checkout / register / login")
    mobile = serializers.CharField(
        required=False, help_text="Mobile for register")
    gender = serializers.CharField(required=False, help_text="M / F")
    password = serializers.CharField(
        required=False, help_text="Email incase of guest checkout")
    dob = serializers.DateField(required=False, help_text="Date")


class GetServicesSerializers(serializers.Serializer):
    id = serializers.IntegerField(required=False)

    def validate_id(self, value):
        if BookMatch.objects.filter(id=value).exists():
            return value
        else:
            raise serializers.FieldDoesNotExist("No data Found")


class SendMatchRequestSerializer(serializers.Serializer):
    id = serializers.IntegerField(
        required=True, help_text="Id of Match booking", write_only=True)
    player_id = serializers.IntegerField(
        required=False, help_text="ID of player", write_only=True)
    player_email = serializers.CharField(
        required=False, help_text="Email of player", write_only=True)


class GetContactPlayerSerializer(serializers.ModelSerializer):
    class Meta:
        model = Player
        fields = ['id', 'first_name', 'last_name', 'image']


class PlayerSkillSerializerOpponent(serializers.ModelSerializer):
    sport = serializers.CharField(source="sport.name", required=False)
    position = serializers.CharField(
        source="position.position", required=False)

    class Meta:
        model = PlayerSkills
        fields = '__all__'


class PlayerFindOpponentSerializer(serializers.ModelSerializer):
    skills = serializers.SerializerMethodField("_get_skills")

    def _get_skills(self, obj):
        skills = PlayerSkills.objects.filter(player=obj.id, is_deleted=False)
        if skills.exists():
            skill_class = PlayerSkillSerializerOpponent(skills, many=True).data
            return skill_class
        return []

    class Meta:
        model = Player
        fields = '__all__'

# class BookaMatchVenueSerivicesSerializer(serializers.ModelSerializer):
# 	venue_service = serializers.CharField(source="venue_service.name", required=False)
# 	class Meta:
# 		model = VenueFieldService
# 		fields = ['id','venue_service','price_per_hour']


class BookaMatchVenueSerivicesSerializer(serializers.ModelSerializer):
    price_per_hour = serializers.CharField(
        default=random.randint(0, 20), required=False)

    class Meta:
        model = VenueService
        fields = ['id', 'name', 'price_per_hour']


class Step4Serializers(serializers.ModelSerializer):
    id = serializers.IntegerField(required=True, help_text="ID of Booking")
    venue_field = serializers.CharField(
        source="venue_field.name", required=False)
    venue = serializers.CharField(
        source="venue_field.venue.name", required=False)
    services = BookaMatchVenueSerivicesSerializer(many=True, required=False)
    booking_date = serializers.DateField(
        default=timezone.now().date(), required=False)

    class Meta:
        model = BookMatch
        fields = ['id', 'venue_field', 'schedule_slot', 'services',
                  'services_cost', 'total_cost', 'venue', 'booking_date']
        read_only_fields = ['venue_fields', 'services', 'services_cost',
                            'total_cost', 'venue', 'schedule_slot', 'booking_date']


# Find Team ##############################33


class FindTeamSerializer(serializers.ModelSerializer):
    city = serializers.CharField(
        required=False, help_text="City name string of the team")
    keyword = serializers.CharField(required=False, help_text="Name Keyword")
    opening = serializers.JSONField(required=False, help_text="Openinggs")
    page = serializers.IntegerField(
        default=1, required=False, help_text="Pagination")

    team_levels = serializers.SerializerMethodField(
        "_get_team_level", read_only=True)

    def validate_city(self, data):
        print("hi")
        city = City.objects.filter(name__icontains=data)
        return city

    def _get_team_level(self, obj):
        return TeamLevelSerializer(obj.team_level, many=True).data

    class Meta:
        model = TeamDetails
        fields = "__all__"


class FindATeamFilterSerializer(serializers.ModelSerializer):
    area = serializers.JSONField(
        required=False, help_text='list of area=[1,2,3]')
    team_name = serializers.CharField(required=False, help_text="team name")
    sport = serializers.CharField(required=False)
    player = serializers.BooleanField(
        required=False, default=False, help_text="To show looking for fields")
    opponent = serializers.BooleanField(
        required=False, default=False, help_text="To show opponents")
    opening = serializers.JSONField(
        required=False, help_text="List position id [1,2,3]")
    page = serializers.IntegerField(
        default=1, required=False, help_text="Pagination")

    class Meta:
        model = TeamDetails
        fields = "__all__"
        read_only_fields = ['player', 'country', 'state', 'city', 'region', 'team_level',
                            'date_founded', 'team_image', 'player_count', 'created_by', 'created_at']


class FindTeamMember(serializers.ModelSerializer):
    class Meta:
        model = TeamMember
        fields = "__all__"


class JoinTeamSerializer(serializers.Serializer):
    id = serializers.IntegerField(required=False, help_text="Team Details ID")


class MatchTeamSerializer(serializers.Serializer):
    id = serializers.IntegerField(required=False, help_text="Team Details ID")


class MatchTeamDetailsSerializer(serializers.Serializer):
    id = serializers.IntegerField(required=True, help_text="Team Details ID")


######################### JOINS ###############################

class JoinsByIDSerializer(serializers.Serializer):
    id = serializers.IntegerField(required=True, help_text="Joins ID")


class JoinsViewJoinSerializer(serializers.ModelSerializer):
    class Meta:
        model = PlayerInvitation
        fields = ['id', 'recipient', 'status', 'category', 'sender']


class TeamDetailViewJoinSerializer(serializers.ModelSerializer):
    class Meta:
        model = TeamDetails
        fields = ['id', 'team_name', 'sport', 'country',
                  'city', 'date_founded', 'team_image']


class ViewTeamDetailsSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(required=True, help_text="TeamDetails ID")
    sport = SportSerializer(read_only=True)
    country = CountrySerializer(read_only=True)
    city = CitySerializer(read_only=True)
    state = StateGetSerializer(read_only=True)
    area = AreaGetSerializer(read_only=True)
    player = PlayerGETSerializer(read_only=True)
    team_level = TeamLevelSerializer(many=True, read_only=True)

    class Meta:
        model = TeamDetails
        fields = '__all__'
        read_only_fields = ['id', 'player', 'team_name', 'sport', 'country',
                            'city', 'state', 'area', 'region', 'team_level',
                            'date_founded', 'team_image', 'player_count', 'created_by', 'created_at']


class TeamMemberViewJoinSerializer(serializers.ModelSerializer):
    team_details = TeamDetailViewJoinSerializer(required=False)

    class Meta:
        model = TeamMember
        fields = ['id', 'team_details', 'position',
                  'status', 'last_name', 'first_name', 'email']


class BookMatchViewJoinSerializer(serializers.ModelSerializer):
    class Meta:
        model = BookMatch
        fields = ['id', 'venue_field', 'schedule_slot', 'services']


class MyMatchesEditInfoSerializer(serializers.ModelSerializer):
    id = serializers.CharField(required=True, help_text='BookMatch id')

    class Meta:
        model = BookMatch
        fields = ['id', 'status']


class BookMatchPlayerVenueFieldSerializer(serializers.ModelSerializer):
    venue_field = VenueFieldBookingMatchSerializer()
    schedule_slot = ScheduleSlotSerializer()

    class Meta:
        model = BookMatch
        fields = ['id', 'venue_field', 'schedule_slot']


class WebsiteInvitationSerializer(serializers.Serializer):
    email = serializers.CharField(required=True, help_text="Email address")


class PlayerAutoCompleteSerializer(serializers.ModelSerializer):
    keyword = serializers.CharField(
        required=False, write_only=True, help_text="Keyword")

    class Meta:
        model = Player
        fields = ['id', 'email', 'first_name',
                  'last_name', 'keyword', 'mobile']
        read_only_fields = ['id', 'email', 'first_name', 'last_name', 'mobile']


class VenueCitySerializer(serializers.ModelSerializer):
    keyword = serializers.CharField(
        required=False, write_only=True, help_text="Keyword")

    class Meta:
        model = VenueField
        fields = ['id', 'name', 'sport', 'keyword']
        read_only_fields = ['id', 'name', 'sport']


class PlayerNotifyMyContactsSerializer(serializers.ModelSerializer):
    id = serializers.JSONField(required=True, help_text="[1,2,3]")

    class Meta:
        model = Player
        fields = '__all__'
        read_only_fields = ['user', 'first_name', 'middle_name', 'last_name',
                            'email', 'new_email', 'mobile', 'new_mobile', 'gender', 'country', 'city', 'pincode',
                            'street', 'dob', 'image', 'is_email_verified', 'created_at', 'updated_at']


class PlayerGetOnIDSerializer(serializers.ModelSerializer):
    id = serializers.CharField(required=True, help_text='player id')

    class Meta:
        model = Player
        fields = ['id']


class VenueListPlayerPostSerializer(serializers.ModelSerializer):
    id = serializers.PrimaryKeyRelatedField(
        queryset=Venue.objects.all(), help_text="venue")

    class Meta:
        model = VenueListPlayer
        fields = '__all__'
        read_only_fields = ['venue_list', 'created_by']


class VenueRatingSerializer(serializers.ModelSerializer):
    class Meta:
        model = VenueRating
        fields = '__all__'


class BookMatchNewPOSTSerializer(serializers.ModelSerializer):
    schedule_slot = serializers.JSONField(required=False, help_text="[1,2,3]")

    def create(self, validated_data):

        data = BookMatch.objects.create(venue_field=validated_data['venue_field'], date=validated_data['date'],
                                        created_by=validated_data['created_by'])
        data.schedule_slot.set(validated_data['schedule_slot'])
        data.save()
        return data

    class Meta:
        model = BookMatch
        fields = '__all__'
        read_only_fields = ['services_cost',  'total_cost',
                            'my_team', 'opponent_team', 'status', 'match_result', 'sport', 'created_at']


class BookmatchSAVESerializer(serializers.ModelSerializer):
    class Meta:
        model = BookMatch
        fields = '__all__'


class BookMatchNew2POSTSerializer(serializers.ModelSerializer):
    id = serializers.CharField(required=False, help_text='1st booking step id')
    services = serializers.JSONField(
        required=False, write_only=True, help_text='[{"venue_service_cost": 1,"quantity": 2},{"venue_service_cost": 2,"quantity": 3}]')
    venue_field_services = serializers.JSONField(
        required=False, help_text='[{"venue_field_service": 1,"quantity": 2},{"venue_field_service": 2,"quantity": 3}]')

    def create(self, validated_data):

        bookmatch_data = BookMatch.objects.filter(
            id=validated_data['id'])
        if bookmatch_data.exists():
            attributes = []
            f_attributes = []
            book_obj = bookmatch_data.last()
            print(book_obj)

            if 'services' in validated_data:
                attributes.append("services")
                services = validated_data['services']
                validated_data.pop('services')

            if 'services' in attributes:
                if VenuePaidServiceQuantity.objects.filter(book_match=book_obj).exists():
                    paid_quantity_obj = VenuePaidServiceQuantity.objects.filter(
                        book_match=book_obj)
                    paid_quantity_obj.delete()
                    venue_Service_cost = [VenuePaidServiceQuantity(
                        quantity=i['quantity'],
                        venue_service_cost=VenueServiceCost.objects.get(
                            id=i['venue_service_cost']),
                        book_match=book_obj
                    ) for i in services]

                    VenuePaidServiceQuantity.objects.bulk_create(
                        venue_Service_cost)
                else:
                    venue_Service_cost = [VenuePaidServiceQuantity(
                        quantity=i['quantity'],
                        venue_service_cost=VenueServiceCost.objects.get(
                            id=i['venue_service_cost']),
                        book_match=book_obj
                    ) for i in services]

                    VenuePaidServiceQuantity.objects.bulk_create(
                        venue_Service_cost)

            # Field
            if 'venue_field_services' in validated_data:
                f_attributes.append("venue_field_services")
                field_services = validated_data['venue_field_services']
                validated_data.pop('venue_field_services')

            if 'venue_field_services' in f_attributes:
                if VenueFieldServiceQuantity.objects.filter(book_match=book_obj).exists():
                    field_paid_quantity_obj = VenueFieldServiceQuantity.objects.filter(
                        book_match=book_obj)
                    field_paid_quantity_obj.delete()
                    venue_field_service_cost = [VenueFieldServiceQuantity(
                        quantity=j['quantity'],
                        venue_field_service=VenueFieldService.objects.get(
                            id=j['venue_field_service']),
                        book_match=book_obj
                    ) for j in field_services]

                    VenueFieldServiceQuantity.objects.bulk_create(
                        venue_field_service_cost)
                else:
                    venue_field_service_cost = [VenueFieldServiceQuantity(
                        quantity=j['quantity'],
                        venue_field_service=VenueFieldService.objects.get(
                            id=j['venue_field_service']),
                        book_match=book_obj
                    ) for j in field_services]

                    VenueFieldServiceQuantity.objects.bulk_create(
                        venue_field_service_cost)

            book_obj.save()
        else:
            raise serializers.ValidationError('No Record Found')

        return book_obj

    class Meta:
        model = BookMatch
        fields = '__all__'
        read_only_fields = ['services_cost', 'venue_field', 'date', 'total_cost',
                            'my_team', 'opponent_team', 'status', 'match_result', 'sport', 'created_at', 'schedule_slot', 'created_by']


class NewVenueBookmatchSerializer(serializers.ModelSerializer):

    class Meta:
        model = Venue
        fields = ['name']


class NewVenueFieldBookingMatchSerializer(serializers.ModelSerializer):
    venue = NewVenueBookmatchSerializer()
    field_type = FieldTypeSerializer(required=False, read_only=True)
    floor_type = FloorTypeSerializer(required=False, read_only=True)

    class Meta:
        model = VenueField
        fields = ['name', 'venue', 'main_image', 'field_type', 'floor_type']


class FieldServiecsBookMatchSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(
        required=False, help_text="Id of book match")
    # schedule_slot = ScheduleSlotSerializer(
    #     required=False, read_only=True, many=True)
    venue_field = NewVenueFieldBookingMatchSerializer(
        required=False, read_only=True)

    class Meta:
        model = BookMatch
        fields = '__all__'
        read_only_fields = ['services_cost', 'venue_field', 'date', 'total_cost',
                            'my_team', 'opponent_team', 'status', 'match_result', 'sport', 'created_at', 'schedule_slot', 'created_by']

# Paid Service Serializer


class NewVenueServicesCostSerializer(serializers.ModelSerializer):
    venue_service_name = serializers.CharField(source="venue_service.name")
    venue_service_logo = serializers.ImageField(source="venue_service.logo")

    class Meta:
        model = VenueServiceCost
        fields = ['id', 'venue_service_name', 'venue_service_logo']


class VenuePaidServiceQuantityerializer(serializers.ModelSerializer):
    #book_match = BookmatchSAVESerializer(required=False, read_only=True)
    venue_service_cost = VenueServicesCost1Serializer(
        required=False, read_only=True)

    class Meta:
        model = VenuePaidServiceQuantity
        fields = '__all__'

# Field Service


class FieldServiceSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(
        required=False, help_text="Id of venue_field")
    venue_service = VenueServiceSerializer(required=False, read_only=True)

    class Meta:
        model = VenueFieldService
        fields = '__all__'
        read_only_fields = ['price_per_hour', 'is_deleted',
                            'venue_field', 'venue_service', 'created_by']


# Booking Summery

class BookingSummeryIdSerializer(serializers.Serializer):
    id = serializers.IntegerField(
        required=False, help_text="Id of book match")


class BookingSummerySerializer(serializers.ModelSerializer):
    schedule_slot = ScheduleSlotSerializer(
        required=False, read_only=True, many=True)
    venue_field = NewVenueFieldBookingMatchSerializer(
        required=False, read_only=True)

    class Meta:
        model = BookMatch
        fields = ['venue_field', 'schedule_slot', 'date', ]


class VenueFieldServiceQuantitySerializer(serializers.ModelSerializer):
    #book_match = BookmatchSAVESerializer(required=False, read_only=True)
    venue_field_service = VenueServicePreview1Serializer(
        required=False, read_only=True)

    class Meta:
        model = VenueFieldServiceQuantity
        fields = '__all__'


# Edit


class VenuePaidServiceQuantityEDITSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(
        required=False, help_text="Id of Venue Paid Service")

    class Meta:
        model = VenuePaidServiceQuantity
        fields = ['id', 'quantity']


class VenueFieldServiceQuantityEDITSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(
        required=False, help_text="Id of VenueField Service ")

    class Meta:
        model = VenueFieldServiceQuantity
        fields = ['id', 'quantity']


# Delete

class DeleteVenuePaidServiceQuantitySerializer(serializers.Serializer):
    id = serializers.IntegerField(
        required=False, help_text="Id of Venue Paid Service")


class DeleteVenueFieldServiceQuantitySerializer(serializers.Serializer):
    id = serializers.IntegerField(
        required=False, help_text="Id of VenueField Service")


# Email
class SendEmailPlayerSerializer(serializers.Serializer):
    id = serializers.IntegerField(
        required=False, help_text="Id of Player")
    mode = serializers.CharField(
        required=False, help_text="T=Team and O=Oppenent")
    booking_id = serializers.IntegerField(
        required=False, help_text="Id of booking")


class SendEmailFriendSerializer(serializers.Serializer):
    email = serializers.CharField(
        required=False, help_text="Email of Friend")
    mode = serializers.CharField(
        required=False, help_text="T=Team and O=Oppenent")
    booking_id = serializers.IntegerField(
        required=False, help_text="Id of booking")


class GetScheduleSlotSerializer(serializers.Serializer):

    keyword = serializers.CharField(
        required=False, help_text="Venue Name Keywords")
    sport = serializers.CharField(required=False)
    date = serializers.DateField(required=True)
    start_time = serializers.TimeField(required=False)
    field_type = serializers.CharField(
        required=False, help_text="Field type ID/ANY")
    venue_list = serializers.JSONField(required=False, help_text="[1,2,3]")
    prices = serializers.JSONField(
        required=False, help_text="{'start':23,'end':25}")
    categories = serializers.JSONField(
        required=False, help_text='["INDOOR", "5 x 5"]')
    services = serializers.JSONField(required=False, help_text="[1,2,3]")


class GetPopUpFieldTypeSerializer(serializers.Serializer):

    venue_id = serializers.CharField(required=False)
    date = serializers.DateField(required=False)
    start_hour = serializers.TimeField(required=False)
    end_hour = serializers.TimeField(required=False)


class SportsSerializer(serializers.Serializer):
    id = serializers.IntegerField(
        required=False, help_text="Id of sports")


class PlayerMyProfileSerializer(serializers.ModelSerializer):

    class Meta:
        model = Player
        fields = ['first_name', 'last_name', 'nick_name', 'created_at']
        extra_kwargs = {"email": {"required": False}}


class PlayerTeamInfoListSerializer(serializers.ModelSerializer):
    team_level = TeamLevelSerializer(many=True)
    sport = SportSerializer()
    country = CountrySerializer()
    city = CitySerializer()
    state = StateGetSerializer()
    area = AreaGetSerializer()

    class Meta:
        model = TeamDetails
        fields = ['id', 'team_name', 'country', 'city', 'state', 'area',
                  'team_level', 'sport', 'team_image', 'player_count', 'player']


class TeamMemberPositionSerializer(serializers.ModelSerializer):
    position = PositionSerializer(required=False, read_only=True)

    class Meta:
        model = TeamMember
        fields = "__all__"


class PlayerNewPersonalInfoPOSTSerializer(serializers.ModelSerializer):
    class Meta:
        model = Player
        fields = ['first_name', 'last_name', 'nick_name', 'image']
        # read_only_fields = ['user', 'is_deleted', 'is_email_verified']
        extra_kwargs = {"email": {"required": False}}


class PlayerMyConatactsDataGetSerializer(serializers.Serializer):
    mode1 = serializers.CharField(
        required=False, help_text="M=Invaitation to me,O=Invaitation to others")
    mode2 = serializers.CharField(
        required=False, help_text="G=Game,T=Team and P =Platform")
    sport = serializers.CharField(required=False)
    status_name = serializers.CharField(
        required=False, help_text="Pending/Approved/Declined")
    date_from = serializers.DateField(required=False, help_text="date from")
    date_to = serializers.DateField(required=False, help_text="date to")
    page = serializers.IntegerField(
        default=1, required=False, help_text="Pagination")


class SendInviteEmailSerializer(serializers.Serializer):
    email = serializers.CharField(
        required=False, help_text="Email of Friend")


class TeamMemberChangeStatusSerialzier(serializers.Serializer):
    id = serializers.PrimaryKeyRelatedField(
        queryset=TeamMember.objects.all(), required=True)
    status = serializers.CharField(
        required=True, help_text="Approved / Declined")
