import pytz
import json
from django.http import HttpResponse
from django.template import Template, Context
from django.template.loader import get_template
from django.db import transaction
from django.urls import reverse

from django.contrib.auth import login as django_login
import datetime
from rest_framework.authtoken.models import Token
from rest_framework import status, generics
from rest_framework.response import Response
from rest_framework.authentication import TokenAuthentication
from rest_framework.permissions import IsAuthenticated
from utils.mixins import *
from utils.api_handle_errors import handle_errors
from static_data.models import DayofWeek
from players.serializers import *
from message.models import Notification
from players.models import Player, PlayerInvitation, PlayerSkills, TeamMember
from django.db.models import Q
from datetime import date


class FindOpponentAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = FindOpponentSerializer
    playerSkills_serializer = PlayerSkillsOpponentSerializer
    teamMembers_serializer = TeamMemberViewJoinSerializer
    find_opponent_serializer = PlayerFindOpponentSerializer

    def post(self, request, format=None):
        """
        Get Players For Find Opponents
        """
        response = {}
        responsedata = list()
        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            total_ids = list()
            player_skill_filters = dict()

            q = Q()

            if "keyword" in serializers.validated_data:
                q |= Q(
                    area__name__icontains=serializers.validated_data['keyword'])
                q |= Q(
                    city__name__icontains=serializers.validated_data['keyword'])
                q |= Q(
                    state__name__icontains=serializers.validated_data['keyword'])
                q |= Q(
                    country__name__icontains=serializers.validated_data['keyword'])
                serializers.validated_data.pop("keyword")

            # Check for sports
            if serializers.validated_data.get("sport", False):
                player_skill_filters['sport'] = serializers.validated_data['sport']

            # Check for Level
            if serializers.validated_data.get("level", False):
                player_skill_filters['level__in'] = serializers.validated_data['level']

            if serializers.validated_data.get("level", False) or serializers.validated_data.get("sport", False):
                player_skill_obj = PlayerSkills.objects.filter(
                    **player_skill_filters).values_list("player")
                serializers.validated_data['id__in'] = [
                    i for i in player_skill_obj]

            if serializers.validated_data.get("location", False):
                serializers.validated_data['city__in'] = serializers.validated_data.pop(
                    "location")

            opponent_list = Player.objects.filter(
                q, **serializers.validated_data)
            opponent_class = self.find_opponent_serializer(
                opponent_list, many=True)

            response["result"] = 1
            response["data"] = opponent_class.data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializers.errors)
            response['result'] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

    # def post(self, request, format=None):
    #     response = {}
    #     responsedata = []
    #     serializers = self.serializer_class(
    #         data=request.data, context={'user': request.user})
    #     if serializers.is_valid():
    #         if serializers.validated_data.get("sport", False)
    #         sport_type = Sport.objects.get(
    #             id=int(serializers.validated_data['sport']))
    #         if sport_type.play_type == 'I':
    #             if "level" in serializers.validated_data:
    #                 if serializers.validated_data['sport'] == '-1':
    #                     queries = [Q(level=value)
    #                                for value in serializers.validated_data['level']]
    #                 else:
    #                     sport_obj = Sport.objects.filter(
    #                         id=int(serializers.validated_data['sport']))
    #                     if sport_obj.exists():
    #                         sport_obj = sport_obj.last()
    #                     else:
    #                         response["result"] = 0
    #                         response["errors"] = ["No SUch Sport Found"]
    #                         self.add_api_call(request, response)
    #                         return Response(response, status=status.HTTP_200_OK)
    #                     queries = [Q(level=value, sport=sport_obj)
    #                                for value in serializers.validated_data['level']]

    #                 query = queries.pop()
    #                 for item in queries:
    #                     query |= item
    #                 data = PlayerSkills.objects.filter(query)
    #                 delete_data = []
    #                 if data.count() > 0:
    #                     for i in data:
    #                         if i.player in delete_data:
    #                             data = data.exclude(id=i.id)
    #                         else:
    #                             delete_data.append(i.player)

    #                     delete_data.clear()
    #             else:
    #                 if serializers.validated_data['sport'] == '-1':
    #                     data = PlayerSkills.objects.filter(
    #                         is_deleted=False).distinct()
    #                 else:
    #                     sport_obj = Sport.objects.filter(
    #                         id=serializers.validated_data['sport'])
    #                     if sport_obj.exists():
    #                         sport_obj = sport_obj.last()
    #                     else:
    #                         response["result"] = 0
    #                         response["errors"] = ["No SUch Sport Found"]
    #                         self.add_api_call(request, response)
    #                         return Response(response, status=status.HTTP_200_OK)

    #                     data = PlayerSkills.objects.filter(
    #                         is_deleted=False, sport=sport_obj).distinct()

    #                 delete_data = []
    #                 if data.count() > 0:
    #                     for i in data:
    #                         if i.player in delete_data:
    #                             data = data.exclude(id=i.id)
    #                         else:
    #                             delete_data.append(i.player)
    #                     delete_data.clear()

    #             if "location" in serializers.validated_data:
    #                 location = serializers.validated_data['location']
    #                 serializers.validated_data.pop("location")
    #                 for item in data:
    #                     if item.player.city.id in location:
    #                         pass
    #                     else:
    #                         data = data.exclude(id=item.id)

    #             if "availability_date" in serializers.validated_data:
    #                 availability_date = serializers.validated_data['availability_date']
    #                 serializers.validated_data.pop("availability_date")
    #                 list = []
    #                 for item in data:
    #                     list.append(str(item.created_at).split()[0])
    #                     if str(availability_date) in list:
    #                         pass
    #                     else:
    #                         data = data.exclude(id=item.id)
    #                     list.clear()

    #             if "start_time" in serializers.validated_data and "end_time" in serializers.validated_data:
    #                 start_time = serializers.validated_data['start_time']
    #                 serializers.validated_data.pop("start_time")
    #                 end_time = serializers.validated_data['end_time']
    #                 serializers.validated_data.pop("end_time")
    #                 list1 = []
    #                 for item in data:
    #                     list1.append(str(item.created_at).split()[1])
    #                     list2 = (str(item.created_at).split()[1].split('.')[0])

    #                     import datetime as dt
    #                     t2 = dt.datetime.strptime('17:30:00', '%H:%M:%S')
    #                     t3 = dt.datetime.strptime(str(item.created_at).split()[1].split('.')[0], '%H:%M:%S')
    #                     time_zero = dt.datetime.strptime('00:00:00', '%H:%M:%S')
    #                     list2 = (t3 - time_zero + t2).strftime('%I:%M:%S')

    #                     if str(start_time) <= str(list2) <= str(end_time):
    #                         pass
    #                     else:
    #                         data = data.exclude(id=item.id)

    #             serializer = self.playerSkills_serializer(data, many=True)
    #             data = serializer.data

    #             new_dict = {}
    #             for dict_ in serializer.data:
    #                 team_members_list = []
    #                 date_of_birth = dict_['player']['dob']
    #                 date_time_obj = datetime.datetime.strptime(date_of_birth, '%Y-%m-%d')
    #                 today = date.today()
    #                 age = today.year - date_time_obj.year - (
    #                         (today.month, today.day) < (date_time_obj.month, date_time_obj.day))
    #                 team_members = TeamMember.objects.filter(player=dict_['player']['id'])

    #                 for team_member_obj in team_members:
    #                     team_member_serializer = self.teamMembers_serializer(team_member_obj)
    #                     team_members_list.append(team_member_serializer.data)

    #                 dict_['team_member'] = team_members_list
    #                 dict_['age'] = age

    #         else:
    #             if "level" in serializers.validated_data:
    #                 level = serializers.validated_data['level']
    #                 serializers.validated_data.pop("level")
    #                 team_details_obj = TeamDetails.objects.filter(sport=serializers.validated_data['sport'],
    #                                                               team_level__in=level).exclude(created_by=request.user)

    #             else:
    #                 team_details_obj = TeamDetails.objects.filter(sport=serializers.validated_data['sport'],
    #                                                               ).exclude(created_by=request.user)
    #             team_detail_serializer = TeamDetailsFindOpponentInfoSerializer(team_details_obj, many=True)
    #             data = team_detail_serializer.data

    #         response["result"] = 1
    #         response['data'] = data
    #         self.add_api_call(request, response)
    #         return Response(response, status=status.HTTP_200_OK)
    #     else:
    #         response = handle_errors(serializers.errors)
    #         response["result"] = 0
    #         self.add_api_call(request, response)
    #         return Response(response, status=status.HTTP_400_BAD_REQUEST)


class GetPlayerSkillsIDAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = GetPlayerSkillsIDSerializer

    def post(self, request, format=None):
        """ 
        Get Player Skills by ID
        """
        response = {}
        responsedata = []

        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():

            # Get Skills of the players
            player_skills = PlayerSkills.objects.filter(
                player=serializers.validated_data['player_id'], is_deleted=False)
            player_class = self.serializer_class(player_skills, many=True)

            response["result"] = 1
            response["data"] = player_class.data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        else:
            response = handle_errors(serializers.errors)
            response['result'] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class InviteOpponentAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = OpponentInvitationSerializer

    def post(self, request, format=None):
        response = {}

        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            players_obj = Player.objects.filter(
                id=serializers.validated_data['player_id'], is_deleted=False)

            if players_obj.exists():
                players_obj = players_obj.last()
            else:
                response["result"] = 0
                response['errors'] = ['No Such Player Found']
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_400_BAD_REQUEST)

            # Get current user player
            current_player = Player.objects.filter(
                user=request.user, is_deleted=False)
            if current_player.exists():
                current_player = current_player.last()
            else:
                response["result"] = 0
                response['errors'] = ['You are not authorized']
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_400_BAD_REQUEST)

            player_invitation = PlayerInvitation.objects.create(
                created_by=current_player, recipient=players_obj.email, category="Find Opponent",
                sender=request.user.email)

            # Notification.objects.create(recipient=players_obj.user, category="Find Opponent",
            # subject="Oppenent Invitation", content_object=player_invitation, body="Thank you for your invitation", created_at=datetime.datetime.now())

            response["result"] = 1
            response['message'] = "Invitation Sent Successfully"
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)
