import pytz
import json
from django.http import HttpResponse
from django.template import Template, Context
from django.template.loader import get_template
from django.db import transaction
from django.urls import reverse

from django.contrib.auth import login as django_login
import datetime
from rest_framework.authtoken.models import Token
from rest_framework import status, generics
from rest_framework.response import Response
from rest_framework.authentication import TokenAuthentication
from rest_framework.permissions import IsAuthenticated
from utils.mixins import *
from utils.mail import Player_platform_invite_email
from utils.api_handle_errors import handle_errors
from static_data.models import DayofWeek, Position
from message.models import Notification
from players.serializers import *
from players.models import Player, TeamMember, TeamDetails, PlayerInvitation


class FindTeamAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = FindTeamSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []

        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            # Pagination
            start_page = (serializers.validated_data['page'] - 1) * 10
            end_page = start_page + 10
            serializers.validated_data.pop("page")

            # Keyword
            if "keyword" in serializers.validated_data:
                serializers.validated_data['team_name__icontains'] = serializers.validated_data['keyword']
                serializers.validated_data.pop("keyword")

            # Opening filter
            if "opening" in serializers.validated_data:
                serializers.validated_data.pop("opening")

            # City
            if "city" in serializers.validated_data:
                print(serializers.validated_data['city'])
                serializers.validated_data['city__in'] = serializers.validated_data['city']
                serializers.validated_data.pop("city")
            
            if "team_level" in serializers.validated_data:
                print(serializers.validated_data)
                if len(serializers.validated_data['team_level']) == 0:
                    serializers.validated_data.pop("team_level")
                else:
                    serializers.validated_data['team_level__in'] = serializers.validated_data.pop("team_level")

            team_obj = TeamDetails.objects.filter(**serializers.validated_data)
            total_count = team_obj.count()
            team_obj = team_obj[start_page: end_page]
            team_class = self.serializer_class(team_obj, many=True)

            # Number of players per team
            for i in team_class.data:
                player_data = TeamMember.objects.filter(
                    status="Approved", team_details=i['id'], is_deleted=False)
                team_member_class = FindTeamMember(player_data, many=True)
                i['total_players'] = player_data.count()
                i['team_player_details'] = team_member_class.data

            response['results'] = 1
            response['data'] = team_class.data
            response['total_count'] = total_count
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response['results'] = 0
            response = handle_errors(serializers.errors)
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class JoinTeamAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = JoinTeamSerializer

    def post(self, request, format=None):
        response = {}
        resonsedata = []

        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            # Get Player
            player_obj = Player.objects.filter(
                user=request.user, is_deleted=False)
            if player_obj.exists():
                player_obj = player_obj.last()
            else:
                response["result"] = 0
                response['errors'] = [
                    'You are not authorized to perform this action']
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)

            # Check for team details
            team_details_obj = TeamDetails.objects.filter(
                id=serializers.validated_data['id'])
            if team_details_obj.exists():
                team_details_obj = team_details_obj.last()
            else:
                response["result"] = 0
                response['errors'] = ['No Such Team Found']
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)

            # Check for number of players in Team
            player_count = TeamMember.objects.filter(
                team_details=team_details_obj, status="Approved", is_deleted=False).count()
            if player_count == 5:
                response["result"] = 0
                response['errors'] = ['Team already Full']
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)

            # Add Team Member
            team_member_request = TeamMember.objects.create(team_details=team_details_obj, player=player_obj,
                                                            email=player_obj.email, first_name=player_obj.first_name,
                                                            last_name=player_obj.last_name,
                                                            status="Pending")

            # Create Player invitation
            player_invitation = PlayerInvitation.objects.create(sender=player_obj.email,
                                                                recipient=team_details_obj.player.email,
                                                                status="Pending", category="Team Request",
                                                                content_object=team_member_request)

            Notification.objects.create(recipient=player_obj.user, category="Join Team", button_text="View Details",
                                        subject="Team Request", content_object=player_invitation,
                                        body=(
                                            f"Team {team_details_obj.team_name} has asked you to join them"),
                                        created_at=datetime.datetime.now())

            response['results'] = 1
            response['data'] = ['Invitation Sent Successfully']
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        else:
            response['results'] = 0
            response = handle_errors(serializers.errors)
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class MatchTeamRequestAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = MatchTeamSerializer

    def post(self, request, format=None):
        response = {}

        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            # Get Player
            player_obj = Player.objects.filter(
                user=request.user, is_deleted=False)
            if player_obj.exists():
                player_obj = player_obj.last()
            else:
                response["result"] = 0
                response['errors'] = [
                    'You are not authorized to perform this action']
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)

            # Check for team details
            team_details_obj = TeamDetails.objects.filter(
                id=serializers.validated_data['id'])
            if team_details_obj.exists():
                team_details_obj = team_details_obj.last()
            else:
                response["result"] = 0
                response['errors'] = ['No Such Team Found']
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)

            # Create Player invitation
            player_invitation = PlayerInvitation.objects.create(sender=player_obj.email,
                                                                recipient=team_details_obj.player.email,
                                                                status="Pending", category="Team Match")

            # Notification.objects.create(recipient=player_obj.user, category="Team Match",
            # subject="Team Match Request", content_object=player_invitation, body="You are invited for a team match")

            response['results'] = 1
            response['data'] = ['Invitation Sent Successfully']
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        else:
            response['results'] = 0
            response = handle_errors(serializers.errors)
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class FindATeamAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = FindATeamFilterSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []
        extra_args = {}

        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            # Pagination
            start_page = (serializers.validated_data['page'] - 1) * 10
            end_page = start_page + 10
            serializers.validated_data.pop("page")

            # team_name
            if "team_name" in serializers.validated_data:
                extra_args['team_name__icontains'] = serializers.validated_data['team_name']
                serializers.validated_data.pop("team_name")

            # Sport
            if "sport" in serializers.validated_data:
                extra_args['sport'] = serializers.validated_data['sport']
                serializers.validated_data.pop("sport")

            # Opponent
            if "opponent" in serializers.validated_data:
                serializers.validated_data.pop("opponent")

            # Area
            if "area" in serializers.validated_data:
                team_obj = TeamDetails.objects.filter(**extra_args,
                                                      area__id__in=serializers.validated_data['area']).exclude(
                    created_by=request.user.id)
            else:
                team_obj = TeamDetails.objects.filter(
                    **extra_args).exclude(created_by=request.user.id)

            # Player
            if "player" in serializers.validated_data:
                for item in team_obj:
                    if serializers.validated_data['player']:
                        player_data_count = TeamMember.objects.filter(
                            status="Approved", team_details=item.id, is_deleted=False).count()
                        looking_for = item.player_count - player_data_count
                        if looking_for > 0:
                            pass
                        else:
                            team_obj = team_obj.exclude(id=item.id)

            # Opening
            if "opening" in serializers.validated_data:
                opening = serializers.validated_data['opening']
                serializers.validated_data.pop("opening")

                for item in opening:
                    position = Position.objects.get(id=item)
                    if position.sport.id == int(extra_args['sport']):
                        team_member_obj = TeamMember.objects.filter(
                            position=item)
                        for team_member_ob in team_member_obj:
                            if team_member_ob.position != position:
                                pass
                            else:
                                team_detail = team_member_ob.team_details
                                team_obj = team_obj.exclude(id=team_detail.id)

            total_count = team_obj.count()
            team_obj = team_obj[start_page: end_page]
            team_class = TeamDetailsFindOpponentInfoSerializer(
                team_obj, many=True)
            # Number of players per team
            for i in team_class.data:
                player_data = TeamMember.objects.filter(
                    status="Approved", team_details=i['id'], is_deleted=False)
                players_member = player_data.count()
                team_member_class = FindTeamMember(player_data, many=True)
                i['total_players'] = player_data.count()
                i['looking_for'] = i['player_count'] - players_member
                i['team_member_details'] = team_member_class.data

            response['results'] = 1
            response['team_details'] = team_class.data
            response['total_count'] = total_count
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response['results'] = 0
            response = handle_errors(serializers.errors)
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class ViewTeamDetailsAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = ViewTeamDetailsSerializer

    def post(self, request, format=None):
        '''
        Get TeamDetails
        '''
        response = {}
        responsedata = []

        serializer = self.serializer_class(data=request.data)
        if serializer.is_valid():

            if "id" in serializer.validated_data:
                if TeamDetails.objects.filter(id=serializer.validated_data["id"]).exists():
                    TeamDetails_obj = TeamDetails.objects.get(
                        id=serializer.validated_data["id"])
                else:
                    response["result"] = 0
                    response['error'] = "Invalid TeamDetails id"
                    self.add_api_call(request, response)
                    return Response(response, status=status.HTTP_200_OK)

            team_class = ViewTeamDetailsSerializer(TeamDetails_obj).data
            player_data = TeamMember.objects.filter(
                status="Approved", team_details=team_class['id'], is_deleted=False)
            team_member_class = FindTeamMember(player_data, many=True)
            team_class['total_players'] = player_data.count()
            team_class['team_player_details'] = team_member_class.data

            response["result"] = 1
            response['data'] = team_class
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class MatchTeamAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = MatchTeamDetailsSerializer

    def post(self, request, format=None):
        response = {}

        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            # Get your team
            TeamDetails_obj = TeamDetails.objects.filter(
                created_by=request.user)
            if TeamDetails_obj.exists():
                TeamDetails_obj = TeamDetails_obj.last()
            else:
                response["result"] = 0
                response['errors'] = [
                    'You are not authorized team']
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)

            TeamDetails_serializer = ViewTeamDetailsSerializer(TeamDetails_obj)

            # get opponent team
            team_details_obj = TeamDetails.objects.filter(
                id=serializers.validated_data['id'])
            if team_details_obj.exists():
                team_details_obj = team_details_obj.last()
            else:
                response["result"] = 0
                response['errors'] = ['No Such Team Found']
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)

            team_details_obj_serializer = ViewTeamDetailsSerializer(
                team_details_obj)

            response['results'] = 1
            response['your_team'] = TeamDetails_serializer.data
            response['opponent_team'] = team_details_obj_serializer.data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        else:
            response['results'] = 0
            response = handle_errors(serializers.errors)
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class JoinTeamFindATeamAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = JoinTeamSerializer

    def post(self, request, format=None):
        response = {}
        resonsedata = []

        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            # Get Player
            player_obj = Player.objects.filter(
                user=request.user, is_deleted=False)
            if player_obj.exists():
                player_obj = player_obj.last()
            else:
                response["result"] = 0
                response['errors'] = [
                    'You are not authorized to perform this action']
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)

            # Check for team details
            team_details_obj = TeamDetails.objects.filter(
                id=serializers.validated_data['id'])
            if team_details_obj.exists():
                team_details_obj = team_details_obj.last()
            else:
                response["result"] = 0
                response['errors'] = ['No Such Team Found']
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)

            # Check for number of players in Team
            player_count = TeamMember.objects.filter(
                team_details=team_details_obj, status="Approved", is_deleted=False).count()
            print('team_details_obj.player_count----->>',
                  team_details_obj.player_count)
            if player_count == team_details_obj.player_count:
                response["result"] = 0
                response['errors'] = ['Team already Full']
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)

            # Add Team Member
            team_member_request = TeamMember.objects.create(team_details=team_details_obj, player=player_obj,
                                                            email=player_obj.email, first_name=player_obj.first_name,
                                                            last_name=player_obj.last_name,
                                                            status="Pending")

            # Create Player invitation
            player_invitation = PlayerInvitation.objects.create(sender=player_obj.email,
                                                                recipient=team_details_obj.player.email,
                                                                status="Pending", category="Team Request",
                                                                content_object=team_member_request)

            Notification.objects.create(recipient=player_obj.user, category="Join Team", button_text="View Details",
                                        subject="Team Request", content_object=player_invitation,
                                        body=(
                                            f"Team {team_details_obj.team_name} has asked you to join them"),
                                        created_at=datetime.datetime.now())

            response['results'] = 1
            response['data'] = ['Invitation Sent Successfully']
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        else:
            response['results'] = 0
            response = handle_errors(serializers.errors)
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class MyContactsGetAndFilterAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = PlayerMyConatactsDataGetSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []
        extra_args = {}

        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            # Pagination
            start_page = (serializers.validated_data['page'] - 1) * 10
            end_page = start_page + 10
            serializers.validated_data.pop("page")

            # Sport
            if "sport" in serializers.validated_data:
                if serializers.validated_data['mode1'] == "O" and serializers.validated_data['mode2'] == "P":
                    response["result"] = 0
                    response['errors'] = [
                        'No records available for sport in invitation to other platform']
                    self.add_api_call(request, response)
                    return Response(response, status=status.HTTP_200_OK)

                else:
                    extra_args['team_details__sport__id'] = serializers.validated_data['sport']
                    serializers.validated_data.pop("sport")

            if "status_name" in serializers.validated_data:
                if serializers.validated_data["status_name"] == "Pending":
                    extra_args['status'] = "Pending"

                elif serializers.validated_data["status_name"] == "Approved":
                    extra_args['status'] = "Approved"

                elif serializers.validated_data["status_name"] == "Declined":
                    extra_args['status'] = "Declined"

                serializers.validated_data.pop("status_name")

            if "date_from" in serializers.validated_data:
                extra_args['created_at__gte'] = serializers.validated_data['date_from']

            if "date_to" in serializers.validated_data:
                extra_args['created_at__lte'] = serializers.validated_data['date_to']

            if "mode1" in serializers.validated_data:
                # For invitation me and team data
                if serializers.validated_data['mode1'] == "M":
                    if "mode2" in serializers.validated_data:
                        if serializers.validated_data['mode2'] == "G":
                            pass
                        elif serializers.validated_data['mode2'] == "T":
                            print('MT')
                            if TeamMember.objects.filter(player=request.user.id, **extra_args,):
                                team_obj = TeamMember.objects.filter(
                                    player=request.user.id, **extra_args,)
                                print(team_obj)
                                if len(team_obj) > 0:
                                    for i in team_obj:
                                        data = {}
                                        data['id'] = i.id
                                        data['invitation_from'] = i.team_details.team_name
                                        data['sport'] = i.team_details.sport.name
                                        data['invitation_date'] = i.created_at.date()
                                        data['status'] = i.status

                                        responsedata.append(data)

                else:
                    # For Invitation other, team  and platform data
                    if "mode2" in serializers.validated_data:
                        if serializers.validated_data['mode2'] == "G":
                            pass
                        elif serializers.validated_data['mode2'] == "T":
                            print('T')
                            if TeamMember.objects.filter(created_by=request.user.id, **extra_args,).exists():
                                team_obj = TeamMember.objects.filter(
                                    created_by=request.user.id, **extra_args,)
                                if len(team_obj) > 0:

                                    for i in team_obj:
                                        data = {}
                                        data['id'] = i.id
                                        data['my_team'] = i.team_details.team_name
                                        data['invited_person'] = i.player.first_name + \
                                            " " + i.player.last_name
                                        data['sport'] = i.team_details.sport.name
                                        data['invitation_date'] = i.created_at.date()
                                        data['status'] = i.status

                                        responsedata.append(data)
                        elif serializers.validated_data['mode2'] == "P":

                            if PlayerMyContactsPlatform.objects.filter(created_by=request.user, **extra_args).exists():

                                platform_obj = PlayerMyContactsPlatform.objects.filter(
                                    created_by=request.user, **extra_args)
                                for j in platform_obj:
                                    data = {}
                                    data['invited_person_email'] = j.email
                                    data['invited_date'] = j.created_at.date()
                                    data['status'] = j.status
                                    responsedata.append(data)

            print(responsedata)
            total_count = len(responsedata)
            team_obj = responsedata[start_page: end_page]

            response['results'] = 1
            response['team_details'] = team_obj
            response['total_count'] = total_count
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response['results'] = 0
            response = handle_errors(serializers.errors)
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class MyContactsSendFriendInvitationEmailAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = SendInviteEmailSerializer

    def post(self, request, format=None):
        '''
        Send Mail For Invitation
        '''
        response = {}
        responsedata = []

        serializer = self.serializer_class(data=request.data)
        if serializer.is_valid():

            email_obj = serializer.validated_data["email"]

            user_obj = request.user
            images = Template_Images.objects.filter().values('template_image')
            logo = images[0]
            logo2 = images[1]

            platform_data = PlayerMyContactsPlatform.objects.create(
                email=email_obj, created_by=request.user)
            platform_data.save()

            msg = f'{user_obj.first_name} {user_obj.last_name} has invited you to join BookNMatch Platform.'

            link = "https://sportsbookingnew.worldindia.com/#/"

            Player_platform_invite_email(email_obj, msg, link, logo, logo2)

            response["result"] = 1
            response['data'] = "Successfully Invited"
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class TeamMemberChangeStatusAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = TeamMemberChangeStatusSerialzier

    def post(self, request, format=None):
        response = {}
        responsedata = []

        serializers = self.serializer_class(data=request.data)

        if serializers.is_valid():

            if serializers.validated_data['status'] == "Approved":
                serializers.validated_data['id'].status = "Approved"
                serializers.validated_data['id'].save()
                message = "Successfully Approved"

            elif serializers.validated_data['status'] == "Declined":
                serializers.validated_data['id'].status = "Declined"
                serializers.validated_data['id'].save()
                message = "Successfully Rejected"

            response['result'] = 1
            response['Data'] = message
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
