import pytz
import json
from django.http import HttpResponse
from django.template import Template, Context
from django.template.loader import get_template
from django.db import transaction
from django.urls import reverse

from django.contrib.auth import login as django_login
import datetime
from rest_framework.authtoken.models import Token
from rest_framework import status, generics
from rest_framework.response import Response
from rest_framework.authentication import TokenAuthentication
from rest_framework.permissions import IsAuthenticated
from utils.mixins import *
from utils.api_handle_errors import handle_errors
from static_data.models import DayofWeek
from players.serializers import *
from players.models import Player, PlayerSkills


class PlayerSkillsAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = PlayerSkillGETSerializer

    def post(self, request, format=None):
        '''
        GET Player Skills
        '''

        response = {}
        responsedata = []
        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            if 'player_id' in serializers.validated_data:
                data = PlayerSkills.objects.filter(
                    player=serializers.validated_data['player_id'], is_deleted=False)
            else:
                player = Player.objects.filter(
                    user=request.user, is_deleted=False)
                if player.exists():
                    player = player.last()
                else:
                    response["result"] = 0
                    response["errors"] = ["Not a valid player"]
                    self.add_api_call(request, response)
                    return Response(response, status=status.HTTP_200_OK)
                data = PlayerSkills.objects.filter(
                    player=player, is_deleted=False)
            serializers_data = self.serializer_class(data, many=True)
            response['result'] = 1
            response['data'] = serializers_data.data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response["result"] = 0
            response = handle_errors(serializers.errors)
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class PersonalSkillsEDITAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = PlayerSkillPOSTSerializer

    def post(self, request, format=None):
        '''
        UPDATE Player Skills
        '''
        response = {}
        responsedata = []
        serializers = self.serializer_class(data=request.data, context={
                                            "player": Player.objects.filter(user=request.user).last(), "created_by": request.user})
        if serializers.is_valid():
            serializers.save()
            response['result'] = 1
            responsedata.append("Successfully Updated")
            response['data'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response["result"] = 0
            response = handle_errors(serializers.errors)
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class PersonalSkillsDeleteAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = PersonalSkillsDeleteSerializer

    def post(self, request, format=None):
        """ 
        Deleting Players personal skills
        """
        response = {}
        responsedata = []

        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            personal_skill = PlayerSkills.objects.filter(
                id=serializers.validated_data['id'], created_by=request.user)
            if personal_skill.exists():
                personal_skill = personal_skill.last()
            else:
                response["result"] = 0
                response["errors"] = ["No Personal Skills Found"]
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)

            # Delete
            personal_skill.is_deleted = True
            personal_skill.save()

            # Send response
            response["result"] = 1
            response["message"] = ["Successfully Deleted"]
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializers.errors)
            response['result'] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
