from static_data.models import *
from venues.serializers import VenueProfileSerializer
from venues.models import Venue, VenueRating
from utils.mail import send_mail_new_join
from players.serializers import JoinsViewJoinSerializer, WebsiteInvitationSerializer, ChangeProfilePicSerializer, \
    PlayerAutoCompleteSerializer
from players.serializers import BookMatchViewJoinSerializer, TeamDetailViewJoinSerializer, TeamMemberViewJoinSerializer, \
    JoinsByIDSerializer, PlayerGetOnIDSerializer, PlayerPersonalInfoSerializer, PlayerTeamInfoSerializer
import pytz
import json
from django.http import HttpResponse
from django.template import Template, Context
from django.template.loader import get_template
from django.db import transaction
from django.urls import reverse

from django.db.models import Q
from django.contrib.auth import login as django_login
import datetime
from rest_framework.authtoken.models import Token
from rest_framework import status, generics
from rest_framework.response import Response
from rest_framework.authentication import TokenAuthentication
from rest_framework.permissions import IsAuthenticated
from utils.mixins import *
from utils.api_handle_errors import handle_errors
from players.models import *
from players.serializers import PlayerProfilePOSTSerializer, PlayerProfileSerializer, MyJoinsSerializer, \
    ChangeJoinStatusSerializer, VenueCitySerializer, VenueListPlayerPostSerializer, PlayerGetAllObjectFormatSerializer, \
AddPlayerInterestSerializer, GetPlayerInterestSerializer

from venues.serializers import VenueProfileSerializer
# from sportsfieldbooking_backend.field_booking.players.models import Player
# from sportsfieldbooking_backend.field_booking.utils.mixins import PlatformAPICallMixin


class PlayerProfileAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = PlayerProfileSerializer

    def get(self, request, format=None):
        response = {}
        responsedata = []
        data = Player.objects.filter(user=request.user)
        serializers = self.serializer_class(data, many=True)
        response["result"] = 1
        response['data'] = serializers.data
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


class ChangePlayerProfilePicAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = ChangeProfilePicSerializer

    def post(self, request, format=None):
        """ 
        Change Player Profile Pic 
        """
        response = {}
        responsedata = []
        # Player GET
        player_obj = Player.objects.filter(user=request.user, is_deleted=False)
        if player_obj.exists():
            player_obj = player_obj.last()
        else:
            response["result"] = 0
            response["errors"] = ["No Such player found"]
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        serializers = self.serializer_class(
            instance=player_obj, data=request.data)
        if serializers.is_valid():
            serializers.save()
            response["result"] = 1
            response["message"] = ["Successfully Updated"]
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializers.errors)
            response['result'] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class PlayerPOSTProfileAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = PlayerProfilePOSTSerializer

    def post(self, request, format=None):

        response = {}
        responsedata = []
        serializers = self.serializer_class(
            data=request.data, context={"user": request.user})
        if serializers.is_valid():
            data = serializers.save()
            responsedata.append("Successfully Inserted")
            response["result"] = 1
            response['errors'] = responsedata
            response['ID'] = data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class PlayerMyJoinsAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = MyJoinsSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []

        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():

            # Get Page Number for recived
            start_page = (serializers.validated_data['page'] - 1) * 5
            end_page = start_page + 5

            # Get Page Number for Sent
            sent_start_page = (serializers.validated_data['sent_page'] - 1) * 5
            sent_end_page = sent_start_page + 5

            # Get player object
            player_obj = Player.objects.filter(
                user=request.user, is_deleted=False).first()

            # Get Team Pending Request Object
            team_member_obj = PlayerInvitation.objects.filter(
                status=serializers.validated_data['status'], recipient=player_obj.email)[start_page:end_page]
            total_received_count = PlayerInvitation.objects.filter(
                status=serializers.validated_data['status'], recipient=player_obj.email).count()
            team_request_class = self.serializer_class(
                team_member_obj, many=True).data

            # Get Venue Field Name
            for i in team_request_class:
                if i['category'] == "Play Match":
                    invitation_obj = PlayerInvitation.objects.filter(
                        id=i['id']).last()
                    book_match = invitation_obj.content_object
                    if book_match != None:
                        i['field'] = book_match.venue_field.name
                        i['date'] = book_match.schedule_slot.date
                        i['created_at'] = str(
                            book_match.schedule_slot.date) + " " + book_match.schedule_slot.start_hour.strftime(
                            format="%H:%M") + " - " + book_match.schedule_slot.end_hour.strftime(format="%H:%M")
                    else:
                        i['field'] = ""

            # Get Sent Requests
            request_sent_obj = PlayerInvitation.objects.filter(
                created_by=player_obj, sender=player_obj.email)[sent_start_page:sent_end_page]
            request_count = PlayerInvitation.objects.filter(
                created_by=player_obj, sender=player_obj.email).count()
            request_class = self.serializer_class(
                request_sent_obj, many=True).data

            for i in request_class:
                if i['category'] == "Play Match":
                    invitation_obj = PlayerInvitation.objects.filter(
                        id=i['id']).last()
                    book_match = invitation_obj.content_object
                    if book_match != None:
                        i['field'] = book_match.venue_field.name
                        i['date'] = book_match.schedule_slot.date
                        i['created_at'] = str(
                            book_match.schedule_slot.date) + " " + book_match.schedule_slot.start_hour.strftime(
                            format="%H:%M") + " - " + book_match.schedule_slot.end_hour.strftime(format="%H:%M")
                    else:
                        i['field'] = ""
                else:
                    i['field'] = ""

            response["result"] = 1
            response['data'] = team_request_class
            response['request_received_total'] = total_received_count
            response['request_sent'] = request_class
            response['request_sent_total'] = request_count
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class ChangeJoinStatusAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = ChangeJoinStatusSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []

        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            # Get player object
            player_obj = Player.objects.filter(
                user=request.user, is_deleted=False)
            if player_obj.exists():
                player_obj = player_obj.last()
            else:
                response["result"] = 0
                response['errors'] = [
                    'You are not authorized to perform this action']
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)

            # get invitation
            invitation_object = PlayerInvitation.objects.filter(
                id=serializers.validated_data['id'])
            if invitation_object.exists():
                invitation_object = invitation_object.last()
            else:
                response["result"] = 0
                response['errors'] = ['No Such Invitation']
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)

            # Change status
            invitation_object.status = serializers.validated_data['status']
            invitation_object.save()

            # Check for different category
            if invitation_object.category == "Join Team":
                team_contact_obj = invitation_object.content_object
                team_contact_obj.status = serializers.validated_data['status']
                team_contact_obj.save()
            else:
                response["result"] = 0
                response['errors'] = ['No Such Invitation']
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)

            response["result"] = 1
            response['error'] = ['Transaction Successfully']
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class ViewJoinByIDAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = JoinsByIDSerializer

    team_member_class = TeamMemberViewJoinSerializer
    book_match_class = BookMatchViewJoinSerializer
    player_invitation_class = JoinsViewJoinSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []

        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():

            # Get Invitation by ID
            invitation_obj = PlayerInvitation.objects.filter(
                id=serializers.validated_data['id'])
            if invitation_obj.exists():
                invitation_obj = invitation_obj.last()
            else:
                response["result"] = 0
                response['errors'] = ['No Such Request exist']
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)

            # Serialize Invitation Object
            invitation_class = self.player_invitation_class(invitation_obj)

            # Check if content object
            if invitation_obj.content_object != None:
                content_obj = invitation_obj.content_object

                if invitation_obj.category == "Join Team":
                    content_obj_class = self.team_member_class(content_obj)

                elif invitation_obj.category == "Play Match":
                    content_obj_class = self.book_match_class(content_obj)

                #               elif invitation_obj.category == "Team Request":
                #                   content_obj_class = self.player_invitation_class(content_obj)

                extra_data = content_obj_class.data
            else:
                extra_data = []

            response["result"] = 0
            response['data'] = invitation_class.data
            response['extra'] = extra_data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class WebsiteInvitationAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = WebsiteInvitationSerializer

    def post(self, request, format=None):
        """ 
        Inviting Email address to the Website
        """
        response = {}

        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            # Send Email for the registeration
            images = Template_Images.objects.filter().values('template_image')
            logo = images[0]
            logo2 = images[1]

            send_mail_new_join(serializers.validated_data['email'],logo,logo2)

            response["result"] = 1
            response["message"] = ["Successfull"]
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializers.errors)
            response['result'] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class PlayerAutoCompleteAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = PlayerAutoCompleteSerializer

    def post(self, request, format=None):
        """ 
        Player Auto Complete for joins
        """
        response = {}

        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            player_obj = Player.objects.filter(
                email__icontains=serializers.validated_data['keyword'], is_deleted=False)
            player_class = self.serializer_class(player_obj, many=True)

            response["result"] = 1
            response["data"] = player_class.data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializers.errors)
            response['result'] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class GetVenueBasedOnCityAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = VenueCitySerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []

        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():

            venue_obj = Venue.objects.filter(
                city=City.objects.get(
                    id=serializers.validated_data["keyword"]), is_deleted=False)

            for i in venue_obj:
                venue_field_obj = VenueField.objects.filter(
                    venue=i, is_deleted=False)
                venue_field_class = self.serializer_class(
                    venue_field_obj, many=True).data
                if venue_field_obj.count() > 0:
                    responsedata = responsedata + venue_field_class

            # send response
            response['result'] = 1
            response['data'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        # Serializer Errors
        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class PlayerVenueListApiView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = VenueCitySerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []
        data = VenueListPlayer.objects.filter(created_by=request.user)

        id = data.values_list('venue_list_id', flat=True)
        id_ = id[::]

        data_rating = VenueRating.objects.filter(venue__in=id_)
        data_venue = Venue.objects.filter(id__in=id_)
        dict_ = {}
        new_dict = {}

        for new_key, new_items in enumerate(data_venue.values()):
            avg = 0
            avg_count = 0

            for key, items in enumerate(data_rating.values()):
                if items['venue_id'] == new_items['id']:
                    avg = avg + items['rating']
                    avg_count = avg_count + 1

            dict_ = new_items
            if avg_count > 0:
                dict_['rating'] = avg / avg_count
            else:
                dict_['rating'] = 0
            responsedata.append(dict_)

        response['result'] = 1
        response['data'] = responsedata
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


class PlayerGetRecentVenuesAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = VenueProfileSerializer
    
    def get(self, request, format=None):
        """ 
        PlayerGetRecentVenuesAPIView
        """ 
        try:
            response = {}
            responsedata = []
            # Check player
            player_obj = Player.objects.filter(user=request.user, is_deleted=False)
            if player_obj.exists():
                player_obj = player_obj.last()
            else:
                raise Exception("UnAuthorized")
            player_bookings = BookMatch.objects.filter(created_by=player_obj).values_list("venue_field").distinct()
            # Get Venues fields
            venue_field_obj = VenueField.objects.filter(id__in=player_bookings, is_deleted=False).values_list("venue").distinct()
            # Get Venue
            venue_obj = Venue.objects.filter(id__in=venue_field_obj, is_deleted=False)
            venue_class = self.serializer_class(venue_obj, many=True)
            response["result"] = 1
            response["data"] = venue_class.data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        except Exception as exec:
            response["result"] = 0
            response["errors"] = [exec]
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class PlayerVenueListPostApiView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = VenueListPlayerPostSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []
        request.data._mutable = True

        serializers = self.serializer_class(
            data=request.data, context={"user": request.user})
        if serializers.is_valid():
            if 'id' in serializers.validated_data:

                venue_id = serializers.validated_data['id']
                if VenueListPlayer.objects.filter(
                        venue_list=venue_id, created_by=request.user).exists():

                    response["result"] = 0
                    response['errors'] = ['Record already exists']
                    self.add_api_call(request, response)
                    return Response(response, status=status.HTTP_200_OK)

                else:
                    venue_list_obj = VenueListPlayer.objects.create(
                        venue_list=venue_id, created_by=request.user)

                    data = venue_list_obj.save()

                    responsedata.append("Successfully Inserted")
                    response["result"] = 1
                    response['data'] = responsedata
                    self.add_api_call(request, response)
                    return Response(response, status=status.HTTP_200_OK)
        else:
            response["result"] = 1
            response = handle_errors(serializers.errors)
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class PlayerAllGetAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    """
    Get All data from Player table
    """
    serializer_class = PlayerGetAllObjectFormatSerializer

    def get(self, request, format=None):
        response = {}
        responsedata = []
        data = Player.objects.all().exclude(user=request.user)
        serializer = self.serializer_class(data, many=True)
        response['result'] = 1
        response['data'] = serializer.data
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


class PlayerGetOnIDAPIView(generics.GenericAPIView, PlatformAPICallMixin):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = PlayerGetOnIDSerializer
    player_class = PlayerPersonalInfoSerializer
    team_serializer_class = PlayerTeamInfoSerializer

    def post(self, request, format=None):
        '''
        GET Player on id
        '''

        response = {}
        responsedata = []
        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            if Player.objects.filter(id=serializers.validated_data['id']).exists():
                player_obj = Player.objects.get(
                    id=serializers.validated_data['id'])
            else:
                response["result"] = 0
                response['error'] = "Invalid player id"
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)

            serializers_data = self.player_class(player_obj, many=False).data

            date_of_birth = player_obj.dob
            date_time_obj = datetime.datetime.strptime(
                str(date_of_birth), '%Y-%m-%d')
            today = datetime.date.today()
            age = today.year - date_time_obj.year - (
                (today.month, today.day) < (date_time_obj.month, date_time_obj.day))
            serializers_data['age'] = age

            team_data = TeamDetails.objects.filter(player=player_obj).last()
            team_class = self.team_serializer_class(team_data)
            team_class = team_class.data

            response['result'] = 1
            response['data'] = serializers_data
            response['team_data'] = team_class
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response["result"] = 0
            response = handle_errors(serializers.errors)
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class AddPlayerInterestAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = AddPlayerInterestSerializer
    get_serializers_class = GetPlayerInterestSerializer

    def __check_player_user(self, request):
        player_obj = Player.objects.filter(user=request.user, is_deleted=False)
        if player_obj.exists():
            player_obj = player_obj.last()
            return player_obj
        else:
            raise ValueError("UnAuthorized")

    def delete(self, request, id, format=None):
        """
        Delete Player INterest API view
        """
        response = {}
        try:
            player_obj = self.__check_player_user(request)
            #  check for the id
            player_interest_obj = self.serializer_class.Meta.model.objects.filter(
                player=player_obj, id=id)
            if player_interest_obj.exists():
                player_interest_obj = player_interest_obj.last()
            else:
                raise Exception("Invalid ID")

            player_interest_obj.is_deleted = True
            player_interest_obj.save()
            response["result"] = 1
            response["data"] = "Successfully Deleted"
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        except Exception as e:
            response["result"] = 0
            response["errors"] = [str(e)]
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

    def get(self, request, format=None):
        """
        Get Player Interest API View
        """
        response = {}
        try:
            player_obj = self.__check_player_user(request)
            sports_obj = self.serializer_class.Meta.model.objects.filter(
                player=player_obj, is_deleted=False)
            sports_class = self.get_serializers_class(sports_obj, many=True)
            response["result"] = 1
            response["data"] = sports_class.data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        except Exception as e:
            response["result"] = 0
            response["errors"] = [str(e)]
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

    def post(self, request, format=None):
        """ 
        Add Player Interest API View
        """
        response = {}
        responsedata = []
        try:
            player_obj = self.__check_player_user(request)
            serializers = self.serializer_class(
                data=request.data, context={"player": player_obj})
            if serializers.is_valid():
                serializers.save()
                response["result"] = 1
                response["message"] = ["Successfully Inserted"]
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)
            else:
                response = handle_errors(serializers.errors)
                response['result'] = 0
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)
        except Exception as e:
            response["result"] = 0
            response["errors"] = [str(e)]
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
