import pytz
import json

from django.db.models import Avg
from django.http import HttpResponse
from django.template import Template, Context
from django.template.loader import get_template
from django.db import transaction
from django.urls import reverse

from django.contrib.auth import login as django_login
import datetime
from rest_framework.authtoken.models import Token
from rest_framework import status, generics
from rest_framework.response import Response
from rest_framework.authentication import TokenAuthentication
from rest_framework.permissions import IsAuthenticated
from utils.mixins import *
from utils.api_handle_errors import handle_errors
from static_data.models import DayofWeek
from players.serializers import *
from players.models import Player

from reviews.models import VenueReviewAndRating


class ProfileGETAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = PlayerPersonalInfoSerializer

    def get(self, request, format=None):
        '''
        GET Player Profile
        '''

        response = {}
        responsedata = []
        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            player_obj = Player.objects.filter(user=request.user)
            if player_obj.exists():
                data = player_obj.last()
            else:
                response["result"] = 0
                response["errors"] = ["No Player Found"]
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)
            serializers_data = self.serializer_class(data, many=False)
            response['result'] = 1
            response['data'] = serializers_data.data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response["result"] = 0
            response = handle_errors(serializers.errors)
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class ProfileEDITAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = PlayerPersonalInfoPOSTSerializer

    user_class = PlayerUserEditSerializer

    def post(self, request, format=None):
        '''
        UPDATE Player Profile
        '''

        response = {}
        responsedata = []
        if Player.objects.filter(user=request.user, is_deleted=False).exists():
            serializers = self.serializer_class(
                instance=Player.objects.filter(user=request.user, is_deleted=False).last(), data=request.data)
        else:
            serializers = self.serializer_class(
                instance=Player.objects.filter(user=request.user, is_deleted=False).last(), data=request.data)
        if serializers.is_valid():
            serializers.validated_data['user'] = request.user
            data = serializers.save()

            user_serializers = self.user_class(
                instance=request.user, data=request.data)
            if user_serializers.is_valid():
                user_serializers.save()
            else:
                response["result"] = 0
                response = handle_errors(serializers.errors)
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_400_BAD_REQUEST)

            response['result'] = 1
            responsedata.append("Successfully Updated")
            response['data'] = responsedata
            response['first_name'] = data.first_name
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response["result"] = 0
            response = handle_errors(serializers.errors)
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class NewsLetterAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    # authentication_classes = (TokenAuthentication,)
    # permission_classes = (IsAuthenticated,)
    serializer_class = NewsLetterSerializer

    def get(self, request, format=None):
        response = NewsLetter.objects.all()
        serializers = self.serializer_class(response, many=True)
        response = {"data": serializers.data}
        return Response(response, status=status.HTTP_200_OK)

    def post(self, request, format=None):

        response = {}
        responsedata = []
        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            serializers.save()
            response['result'] = 1
            responsedata.append("Thank you for subscribing")
            response['error'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response["result"] = 0
            response = handle_errors(serializers.errors)
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class MyProfileGetAPIView(generics.GenericAPIView, PlatformAPICallMixin):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = PlayerPersonalInfoSerializer

    team_serializer_class = PlayerTeamInfoSerializer

    def get(self, request, format=None):
        '''
        GET Player My Profile & Teams
        '''

        response = {}
        responsedata = []
        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            player_obj = Player.objects.get(user=request.user)
            venue_book = BookMatch.objects.filter(created_by=player_obj)
            venue_book_serializer = BookMatchPlayerVenueFieldSerializer(
                venue_book, many=True)
            serializers_data = self.serializer_class(
                player_obj, many=False).data

            serializers_data['fav_position'] = "QuaterBack"
            serializers_data['endrosed'] = "0"
            date_of_birth = player_obj.dob
            date_time_obj = datetime.datetime.strptime(
                str(date_of_birth), '%Y-%m-%d')
            today = date.today()
            age = today.year - date_time_obj.year - (
                (today.month, today.day) < (date_time_obj.month, date_time_obj.day))
            serializers_data['age'] = age

            team_data = TeamDetails.objects.filter(player=player_obj).last()
            team_class = self.team_serializer_class(team_data)
            team_class = team_class.data
            team_class['members'] = TeamMember.objects.filter(
                team_details=team_data, status="Approved").count()
            team_class['position'] = "Owner"

            for venuebook in venue_book_serializer.data:
                venue_rating_list = {}
                venue_rating = VenueReviewAndRating.objects.filter(
                    venue=venuebook['venue_field']['venue']['id']).values(
                    'rating').aggregate(rating=Avg('rating'))

                venue_rating_list['venue'] = venuebook['venue_field']['venue']
                venue_rating_list['schedule_slot'] = venuebook['schedule_slot']
                venue_rating_list['venue_rating'] = venue_rating
                responsedata.append(venue_rating_list)

            response['result'] = 1
            response['data'] = serializers_data
            response['team_data'] = team_class
            #response['Venue_book_slot'] = venue_book_serializer.data
            response['Venue_book_slot'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response["result"] = 0
            response = handle_errors(serializers.errors)
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class PlayerMyProfileGETAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = PlayerMyProfileSerializer

    def get(self, request, format=None):
        '''
        GET Player Profile , Team List data
        '''

        response = {}
        responsedata = []

        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            player_obj = Player.objects.filter(user=request.user)
            if player_obj.exists():
                player_data = player_obj.last()
            else:
                response["result"] = 0
                response["errors"] = ["No Player Found"]
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)
            serializers_data = self.serializer_class(player_data, many=False)

            team_list = []
            if TeamDetails.objects.filter(player=player_data).exists():

                teamdetails_obj = TeamDetails.objects.filter(
                    player=player_data)
                serialized_team_data = PlayerTeamInfoListSerializer(
                    teamdetails_obj, many=True)

                for index, i in enumerate(serialized_team_data.data):
                    team_data = {}
                    id = i['id']
                    team_data['id'] = i['id']
                    #team_data['player'] = player_data.first_name
                    team_data['team_name'] = i['team_name']
                    team_data['team_image'] = i['team_image']
                    print(i['team_level'])
                    team_data['team_level'] = i['team_level']
                    team_data['player_count'] = i['player_count']
                    team_data['counrty'] = i['country']
                    team_data['state'] = i['state']
                    team_data['city'] = i['city']
                    team_data['area'] = i['area']

                    # To find Position
                    team_member_obj = TeamMember.objects.get(
                        player=player_data, team_details=id)
                    serialized_team_member_data = TeamMemberPositionSerializer(
                        team_member_obj)
                    print(serialized_team_member_data.data['position'])

                    team_data['position'] = serialized_team_member_data.data['position']

                    team_list.append(team_data)

            response['result'] = 1
            response['player_data'] = serializers_data.data
            response['team_details'] = team_list
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response["result"] = 0
            response = handle_errors(serializers.errors)
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class PlayerMyProfileEDITAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = PlayerNewPersonalInfoPOSTSerializer

    user_class = PlayerUserEditSerializer

    def post(self, request, format=None):
        '''
        UPDATE Player Profile
        '''

        response = {}
        responsedata = []
        if Player.objects.filter(user=request.user, is_deleted=False).exists():
            serializers = self.serializer_class(
                instance=Player.objects.filter(user=request.user, is_deleted=False).last(), data=request.data)
        else:
            serializers = self.serializer_class(
                instance=Player.objects.filter(user=request.user, is_deleted=False).last(), data=request.data)
        if serializers.is_valid():
            serializers.validated_data['user'] = request.user
            data = serializers.save()

            user_serializers = self.user_class(
                instance=request.user, data=request.data)
            if user_serializers.is_valid():
                user_serializers.save()
            else:
                response["result"] = 0
                response = handle_errors(serializers.errors)
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_400_BAD_REQUEST)

            response['result'] = 1
            responsedata.append("Successfully Updated")
            response['data'] = responsedata
            response['first_name'] = data.first_name
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response["result"] = 0
            response = handle_errors(serializers.errors)
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)
