import pytz
import json
from django.http import HttpResponse
from django.template import Template, Context
from django.template.loader import get_template
from django.db import transaction
from django.urls import reverse

from django.contrib.auth import login as django_login
import datetime
from django.db.models import Q, Avg

from rest_framework.authtoken.models import Token
from rest_framework import status, generics
from rest_framework.response import Response
from rest_framework.authentication import TokenAuthentication
from rest_framework.permissions import IsAuthenticated
from utils.mixins import *
from utils.api_handle_errors import handle_errors
from static_data.models import DayofWeek
from players.serializers import *
from players.models import Player, PlayerInvitation, TeamMember, TeamDetails
from message.models import Notification


class TeamDetailAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = TeamDetailPOSTSerializer

    def post(self, request, format=None):
        """
        Player Create Team Step 1
        """

        response = {}
        responsedata = []

        if 'id' in request.data:
            if request.data['id'] == "null":
                serializers = self.serializer_class(
                    data=request.data, context={"user": request.user})
                message = "Successfully Inserted"
            else:
                serializers = self.serializer_class(instance=TeamDetails.objects.get(
                    id=request.data['id']), data=request.data, context={"user": request.user})
                message = "Successfully Updated"
        else:
            serializers = self.serializer_class(
                data=request.data, context={"user": request.user})
            message = "Successfully Inserted"
        if serializers.is_valid():
            serializers.validated_data['created_by'] = request.user
            data = serializers.save()
            response['result'] = 1
            responsedata.append(message)
            response['errors'] = responsedata
            response['id'] = data.id
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class TeamContactAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = TeamContactsSerializer

    def post(self, request, format=None):
        """
        Player Create Team Step 2
        """

        response = {}
        responsedata = []
        serializers = self.serializer_class(
            data=request.data, context=request.user)
        if serializers.is_valid():
            serializers.save()
            response['result'] = 1
            responsedata.append('Successfully inserted')
            response['errors'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class TeamMembersAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    # authentication_classes = (TokenAuthentication,)
    # permission_classes = (IsAuthenticated,)
    serializer_class = TeamMemberSendRequestSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []

        player_obj = Player.objects.filter(user=request.user, is_deleted=False)
        if player_obj.exists():
            player_obj = player_obj.last()
        else:
            response["result"] = 0
            response["errors"] = ["Un Authorized"]
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        serializers = self.serializer_class(
            data=request.data, context=request.user)
        if serializers.is_valid():
            # Get Player User from email id
            if Player.objects.filter(email=serializers.validated_data['email'], is_deleted=False).exists():
                serializers.validated_data['player'] = Player.objects.get(
                    email=serializers.validated_data['email'], is_deleted=False)
            else:
                response['result'] = 0
                responsedata.append('No Player with this email exists')
                response['errors'] = responsedata
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)

            # save data
            member_invitation = serializers.save()

            # Check for team details
            if serializers.validated_data['team_details'].player != player_obj:
                response["result"] = 0
                response['errors'] = ['No Such Team Found']
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)

            team_details_obj = serializers.validated_data['team_details']

            # Send Invitation
            user_player = Player.objects.filter(
                user=request.user, is_deleted=False)
            if user_player.exists():
                user_player = user_player.last()
            else:
                raise serializers.ValidationError("Not Authorized")

            player_invitation = PlayerInvitation.objects.create(created_by=user_player, recipient=serializers.validated_data['email'],
                                                                sender=request.user.email, category="Join Team", content_object=member_invitation)

            Notification.objects.create(recipient=user_player.user, category="Join Team", button_text="View Details",
                                        subject="Team Request", content_object=player_invitation, body=(f"Team {team_details_obj.team_name} has asked you to join them"), created_at=datetime.datetime.now())

            # Get data
            team_members = TeamMember.objects.filter(
                team_details=member_invitation.team_details, is_deleted=False)
            team_members_class = self.serializer_class(team_members, many=True)

            # Send response
            response['result'] = 1
            responsedata.append('Successfully inserted')
            response['errors'] = responsedata
            response['data'] = team_members_class.data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        # Serializer Errors
        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)

    def delete(self, request, format=None):
        response = {}
        try:
            team_member_id = request.GET.get('id', False)
            print(team_member_id)
            if not team_member_id:
                raise Exception("Invalid Team Member")
            
            team_member_obj = TeamMember.objects.filter(id=team_member_id)
            if team_member_obj.exists():
                team_member_obj = team_member_obj.last()
                team_member_obj.is_deleted = True
                team_member_obj.save()
            else:
                raise Exception("Invalid Team Member")
            
            response["result"] = 1
            response["message"] = ["Successfully deleted"]
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        except Exception as e:
            response["result"] = 0
            response["errors"] = [str(e)]
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

    def patch(self, request, format=None):
        try:
            response = {}
            member_id = request.data.get("id", False)
            if not member_id:
                raise Exception("No Id Found")
            member_obj = TeamMember.objects.filter(id=member_id)
            if member_obj.exists():
                member_obj = member_obj.last()
            else:
                raise Exception("Invalid Team Member")
            serializers = self.serializer_class(instance=member_obj,data=request.data)
            if serializers.is_valid():
                serializers.save()
                response["result"] = 1
                response["message"] = ["Successfully Edited"]
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)
            else:
                response = handle_errors(serializers.errors)
                response['result'] = 0
                self.add_api_call(request, response)
                return Response(response,status=status.HTTP_200_OK)

        except Exception as e:
            response["result"] = 0
            response["errors"] = [str(e)]
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

class GetTeamMemberAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = TeamMemberGetSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []

        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            # Get Team Details
            team_details = TeamDetails.objects.filter(
                id=serializers.validated_data['id'])
            if team_details.exists():
                team_details = team_details.last()
            else:
                response['result'] = 0
                responsedata.append('No Record Found')
                response['errors'] = responsedata
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)

            # Get Team Member data
            team_members = TeamMember.objects.filter(
                team_details=team_details, is_deleted=False)
            team_members_class = self.serializer_class(team_members, many=True)

            # Send response
            response['result'] = 1
            responsedata.append('Successfully Found')
            response['errors'] = responsedata
            response['data'] = team_members_class.data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        # Serializer Errors
        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class GetTeamsApiView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = GetTeamInputSerializer
    # get_serializer_class = GetTeamSerializer
    
    def post(self, request, format=None):
        """ 
        Get All player Teams
        """ 
        try:
            response = {}
            responsedata = []
            player_obj = Player.objects.filter(user=request.user, is_deleted=False)
            if player_obj.exists():
                player_obj = player_obj.last()
            else:
                raise Exception("UnAuthorized")
            serializers = self.serializer_class(data=request.data)
            if serializers.is_valid():
                # Filters
                filters = {}
                if serializers.validated_data.get("status", False):
                    filters['status'] = serializers.validated_data.get("status", False)
                if serializers.validated_data.get("team_name", False):
                    filters["team_details__team_name"] = serializers.validated_data.get("team_name", False)
                if serializers.validated_data.get("sport", False):
                    filters["team_details__sport"] = serializers.validated_data.get("sport", False)
                if serializers.validated_data.get("team_detail_id", False):
                    filters["team_details"] = serializers.validated_data.get("team_detail_id", False)

                team_objs = TeamMember.objects.filter(player=player_obj, **filters).values_list("team_details")
                team_objs_list = [team_obj[0] for team_obj in team_objs]
                team_detail_queryset = TeamDetails.objects.filter(id__in=team_objs_list)

                team_objs_class = GetTeamSerializer(team_detail_queryset, many=True).data
                response["result"] = 1
                response["data"] = team_objs_class
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)
            else:
                response = handle_errors(serializers.errors)
                response['result'] = 0
                self.add_api_call(request, response)
                return Response(response,status=status.HTTP_200_OK)
        except Exception as exec:
            response["result"] = 0
            response["errors"] = [str(exec)]
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class DeleteTeamMembersAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = TeamMemberDeleteRequestSerialzier

    def post(self, request, format=None):
        response = {}
        responsedata = []

        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            # player team member object
            team_member_obj = serializers.validated_data['player_id']
            team_member_obj.is_deleted = True
            team_member_obj.save()

            # Send response
            response['result'] = 1
            responsedata.append('Successfully Deleted')
            response['errors'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        # Serializer Errors
        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class TeamCalendarAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = TeamCalendarSerializer

    def post(self, request, format=None):
        """
        Player Create Team Step 4
        """

        response = {}
        responsedata = []
        serializers = self.serializer_class(
            data=request.data, context=request.user)
        if serializers.is_valid():
            serializers.save()
            response['result'] = 1
            responsedata.append('Successfully inserted')
            response['errors'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)

    def get(self, request, format=None):
        """
        Player Get Team avaibality
        """
        try:
            response = {}
            responsedata = []
            # Get Player
            player_obj = Player.objects.filter(user=request.user, is_deleted=False)
            if player_obj.exists():
                player_obj = player_obj.last()
            else:
                raise Exception("Unauthorized")

            team_details = request.GET.get("team_detail_id", False)
            if not team_details:
                raise Exception("No Team Selected")

            team_detail_obj = TeamDetails.objects.filter(id=team_details)
            if team_detail_obj.exists():
                team_detail_obj = team_detail_obj.last()
            else:
                raise Exception("Incorrect Team")
            
            team_member_check = TeamMember.objects.filter(player=player_obj, is_deleted=False, team_details=team_detail_obj)
            if not team_member_check.exists():
                raise Exception("Invalid Team")

            team_detail_obj = self.serializer_class.Meta.model.objects.filter(team_details=team_detail_obj, is_deleted=False)
            team_detail_class = self.serializer_class(team_detail_obj, many=True)
            response["result"] = 1
            response["data"] = team_detail_class.data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        except Exception as e:
            response["result"] = 0
            response["errors"] = [str(e)]
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class GlobalTeamCreateGETAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = TeamDetailGETSerializer
    contact_Serializer_class = TeamContactsSerializer
    calendar_Serializer_class = TeamCalendarSerializer

    def post(self, request, format=None):
        '''
        Global Team Create Get. Send ID to get Specific
        '''

        response = {}
        responsedata = []
        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            team_detail = TeamDetails.objects.filter(
                id=serializers.validated_data['id']).first()
            team_contact = TeamContacts.objects.filter(
                team_details=team_detail)
            team_calendar = TeamCalendar.objects.filter(
                team_details=team_detail)

            team_detailSerializer = self.serializer_class(team_detail)
            team_contactSerializer = self.contact_Serializer_class(
                team_contact, many=True)
            team_calendarSerializer = self.calendar_Serializer_class(
                team_calendar, many=True)

            responsedata.append({"team_details": team_detailSerializer.data})
            responsedata.append({"team_contacts": team_contactSerializer.data})
            responsedata.append(
                {"team_calendar": team_calendarSerializer.data})

            response['result'] = 1
            response['data'] = responsedata
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)
