import pytz

from django.http import HttpResponse
from django.template import Template, Context
from django.template.loader import get_template
from django.db import transaction
from django.urls import reverse
from message.models import Notification
from django.contrib.auth import login as django_login
from platform_auth.serializers import *
import datetime

from rest_framework.authtoken.models import Token
from rest_framework import status , generics 
from rest_framework.response import Response
from rest_framework.authentication import TokenAuthentication
from rest_framework.permissions import IsAuthenticated
from requests.exceptions import HTTPError
 
from social_django.utils import load_strategy, load_backend
from social_core.backends.oauth import BaseOAuth2
from social_core.exceptions import MissingBackend, AuthTokenError, AuthForbidden
from utils.mixins import *

from utils.api_handle_errors import handle_errors
# from utils.mail import email_verification, email_change_verification, send_mail_for_reset_password
from players.serializers import *
# from platform_central.serializers import *
# from field_booking.functions import send_sms
from field_booking.auth_backend import EmailBackend
# from field_booking.settings import DOMAIN_NAME

from social_django.utils import load_strategy, load_backend
############################ -- Login -- ###############################

class WebLoginAPIView(PlatformAPICallMixin, generics.GenericAPIView):

	serializer_class = WebUserLoginSerializer

	@transaction.atomic
	def post(self, request, format=None):
		serializer = self.serializer_class(data=request.data)
		response   = {}

		if serializer.is_valid():

			player = Player.objects.filter(email=serializer.data['email'], is_deleted=False).last()
			if not player:
				response['result'] = 0
				response['errors'] = ['email does not exist']
				self.add_api_call(request, response)
				return Response(response, status=status.HTTP_200_OK)


			user = EmailBackend.authenticate(email=player.email, password=serializer.data["password"])
			if user is not None and user.is_active:
				django_login(request, user)


				data  = {}
				token = Token.objects.filter(user = user).last()
				if not token:
					token  = Token.objects.create(user=user)

				fcm_object = UserFcmObject.objects.filter(user=user).last()
				if not fcm_object:
					fcm_object = UserFcmObject(user=user)

				if 'device_id' in serializer.data:
					fcm_object.device_id   = serializer.data['device_id']
				if 'device_type' in serializer.data:
					fcm_object.device_type = serializer.data['device_type']
				if 'fcm_id' in serializer.data:
					fcm_object.fcm_id      = serializer.data['fcm_id']

				fcm_object.save()

				data["token"] = token.key
				data["user"]  = UserSerializer(user, many=False).data
				
				response["data"]   = data
				response["data"]["user"]["last_login"] = user.last_login.astimezone(pytz.timezone("Asia/Kolkata"))
				response["result"] = 1
				self.add_api_call(request, response)
				return Response(response, status=status.HTTP_200_OK)
			else:
				response["result"] = 0
				response["errors"] = ['User does not exist or password is incorrect']
				self.add_api_call(request, response)
				return Response(response, status=status.HTTP_200_OK)
		else:
			response           = handle_errors(serializer.errors)
			response["result"] = 0
			self.add_api_call(request, response)
			return Response(response, status=status.HTTP_200_OK)


############################ -- Verify User -- ###############################

class WebVerifyAPIView(PlatformAPICallMixin, generics.GenericAPIView):
	serializer_class       = VerifyUserInputSerializer

	@transaction.atomic
	def post(self, request, format=None):
		serializer = self.serializer_class(data = request.data)
		response   = {}

		if serializer.is_valid():
			
			if request.user.is_anonymous:
				# if user is anonymous then send empty data and result 2
				response['result']  = 2
				response['message'] = 'anonymous user'
				self.add_api_call(request, response)
				return Response(response, status=status.HTTP_200_OK)
			else:
				try:
					key = request.META['HTTP_AUTHORIZATION'].split(' ')[1].strip()
				except KeyError:
					response["result"] = 0
					response["errors"] = ["Unable to get Token in Header"]
					self.add_api_call(request, response)
					return Response(response, status=status.HTTP_200_OK)

			player = Player.objects.filter(user = request.user).last()
			if not player:
				response['result'] = 0
				response['errors'] = ['Incorrect user']
				self.add_api_call(request, response)
				return Response(response, status=status.HTTP_200_OK)
			
			fcm_object = UserFcmObject.objects.filter(user=request.user).last()
			if not fcm_object:
				fcm_object = UserFcmObject(user=request.user)

			if 'device_id' in serializer.data:
				fcm_object.device_id   = serializer.data['device_id']

			if 'device_type' in serializer.data:
				fcm_object.device_type = serializer.data['device_type']

			if 'fcm_id' in serializer.data:
				fcm_object.fcm_id      = serializer.data['fcm_id']

			fcm_object.save()

			user_serializer = UserSerializer(request.user, many=False).data

			response["data"] = {
				"token": key,
				"user": user_serializer,
			}
			response["result"] = 1
			self.add_api_call(request, response)
			return Response(response, status=status.HTTP_200_OK)
		else:
			response = handle_errors(serializer.errors)
			response["result"] = 0
			self.add_api_call(request, response)
			return Response(response, status=status.HTTP_200_OK)



##########################################--- Logout ---############################################

class WebLogoutAPIView(PlatformAPICallMixin, generics.GenericAPIView):
	authentication_classes = (TokenAuthentication,)
	permission_classes     = (IsAuthenticated,)
	serializer_class       = LogoutSerializer

	@transaction.atomic
	def post(self, request, format=None):
		serializer = self.serializer_class(data = request.data)
		response = {}

		if not Token.objects.filter(user = request.user).exists():
			response["result"] = 0
			response["errors"] = ["Incorrect Token"]
			self.add_api_call(request, response)
			return Response(response, status=status.HTTP_200_OK)
		else:
			Token.objects.get(user = request.user).delete()
			response["result"] = 1
			response["message"] = "You've been logged out"
			self.add_api_call(request, response)
			return Response(response, status=status.HTTP_200_OK)


class WebRegistrationAPIView(PlatformAPICallMixin, generics.GenericAPIView):
	serializer_class = WebRegisterationInputSerializer

	def register_user(self, request, serializer, user_object=None):
		""" add new user or update existing user records """
		response   = {}
		username   = serializer.data['email']
		first_name = serializer.data['first_name']
		gender     = serializer.data['gender']


		if not user_object:
			""" add new user records """
			user_object   = User(username=username, is_staff=False, is_active=False)
			user_object.save()
			platform_user = Player(user=user_object)
		else:
			""" fetch Player user from User models """
			platform_user = Player.objects.filter(user=user_object).last()

		user_object.first_name = first_name

		platform_user.email      = username
		platform_user.gender     = gender
		platform_user.first_name = first_name

		if 'last_name' in serializer.data:
			user_object.last_name   = serializer.data['last_name']
			platform_user.last_name = serializer.data['last_name']

			
		user_object.save()
		if 'email' in serializer.data:
			platform_user.email = serializer.data['email']

		platform_user.save()

		email_verification_token = UserToken.objects.create(user=user_object, purpose='EMAIL VERIFICATION')
		message = 'Verification Link Has Been Send on {} Email Addrees to verify email id'.format(serializer.data['email'])

		verify_redirect = "{}{}".format(DOMAIN_NAME, reverse('email_verify_token', args=[token.code]))
		email_verification(platform_user.user.first_name, verify_redirect, platform_user.user.email)
		response['message'] = "OTP has been sent to your phone and Verification Link has been send to email address to verify email id"
		response['result'] = 1
		return response


	@transaction.atomic
	def post(self, request, format=None):
		serializer = self.serializer_class(data = request.data)
		response = {}

		if serializer.is_valid():

			user_object = User.objects.filter(email=serializer.data['email']).last()
			if user_object:
				""" given email user already exist """
				if not user_object.is_active:
					""" email user already exist but not active (update record)  """
					response = self.register_user(request, serializer, user_object)
					self.add_api_call(request, response)
					return Response(response, status=status.HTTP_200_OK)
				else:
					""" email user already exist and active """
					response["result"] = 0
					response["errors"] = ["User already exists with email {}".format(serializer.data['email'])]
					self.add_api_call(request, response)
					return Response(response, status=status.HTTP_200_OK)
			else:
				""" given email user does not exist(create new user record) """
				response = self.register_user(request, serializer)
				self.add_api_call(request, response)
				return Response(response, status=status.HTTP_200_OK)

		else:
			response = handle_errors(serializer.errors)
			response["result"] = 0
			self.add_api_call(request, response)
			return Response(response, status=status.HTTP_200_OK)



#get auth token from django auth
class SocialLoginView(generics.GenericAPIView):
	"""Log in using google"""
	serializer_class = SocialSerializer
	#permission_classes = [permissions.AllowAny]
 
	def post(self, request):
		"""Authenticate user through the provider and access_token"""

		serializer = self.serializer_class(data=request.data)
		serializer.is_valid(raise_exception=True)
		provider = serializer.data.get('provider', None)
		strategy = load_strategy(request)

		try:
			backend = load_backend(strategy=strategy, name=provider,redirect_uri=None)
 
		except MissingBackend:
			return Response({'error': 'Please provide a valid provider'},
			status=status.HTTP_400_BAD_REQUEST)
		try:
			if isinstance(backend, BaseOAuth2):
				access_token = serializer.data.get('access_token')
			user = backend.do_auth(access_token)
		except HTTPError as error:
			return Response({
				"error": {
					"access_token": "Invalid token",
					"details": str(error)
				}
			}, status=status.HTTP_400_BAD_REQUEST)
		except AuthTokenError as error:
			return Response({
				"error": "Invalid credentials",
				"details": str(error)
			}, status=status.HTTP_400_BAD_REQUEST)
 
		try:
			authenticated_user = backend.do_auth(access_token, user=user)
		
		except HTTPError as error:
			return Response({
				"error":"invalid token",
				"details": str(error)
			}, status=status.HTTP_400_BAD_REQUEST)
		
		except AuthForbidden as error:
			return Response({
				"error":"invalid token",
				"details": str(error)
			}, status=status.HTTP_400_BAD_REQUEST)
 
		if authenticated_user and authenticated_user.is_active:
			#generate  token
			django_login(request, authenticated_user)
			token = Token.objects.update_or_create(user=authenticated_user)
			# Get Groups
			groups = Group.objects.filter(user = authenticated_user)
			group_class = GroupSerializer(groups, many=True)

			# Get Notificatio Count
			notification_count = Notification.objects.filter(recipient=authenticated_user, is_read=False).count()

			response = {}

			response['token'] = str(token[0])
			response['email'] = str(authenticated_user.email)
			response['first_name'] = str(authenticated_user.first_name)
			response['last_name'] = str(authenticated_user.last_name)
			response['notification_count'] = str(notification_count)
			response['user_type'] = group_class.data
			response['result'] = 1
			 
		  
			data= response
			return Response(status=status.HTTP_200_OK, data=response)
########################################## Forgot Password #####################################

# class ForgotPasswordAPIView(PlatformAPICallMixin, generics.GenericAPIView):
# 	serializer_class = ForgotPasswordSerializer

# 	@transaction.atomic
# 	def post(self, request, format=None):
# 		serializer = self.serializer_class(data = request.data)
# 		response   = {}

# 		if serializer.is_valid():
# 			internal_user = InternalUser.objects.filter(email = serializer.data['email']).last()
# 			if internal_user:
# 				email_token     = UserEmailToken.objects.create(internal_user=internal_user, purpose='RESET PASSWORD')
# 				# verify_redirect = "{}{}".format(DOMAIN_NAME, reverse('reset_password_token', args=[str(email_token)]))
# 				verify_redirect = "{}{}".format(DOMAIN_NAME, reverse('reset_password', args=[str(email_token.key)]))
# 				print('verify_redirect', verify_redirect) # http://domain_name/reset-password/token
# 				send_mail_for_reset_password(internal_user.user.first_name, verify_redirect, internal_user.email)
# 				response["result"]  = 1
# 				response['message'] = "Reset password link is send to your Email ID"
# 				return Response(response, status=status.HTTP_200_OK)
				
# 			else :
# 				response["result"] = 0
# 				response["errors"] = ["email is incorrect"]
# 				self.add_api_call(request, response)
# 				return Response(response, status=status.HTTP_200_OK)
# 		else:
# 			response = handle_errors(serializer.errors)
# 			response["result"] = 0
# 			self.add_api_call(request, response)
# 			return Response(response, status=status.HTTP_200_OK)


# class ResetPasswordAPIView(PlatformAPICallMixin, generics.GenericAPIView):
# 	serializer_class = ResetPasswordSerializer

# 	@transaction.atomic
# 	def post(self, request, format=None):
# 		serializer = self.serializer_class(data = request.data)
# 		response   = {}

# 		if serializer.is_valid():
# 			new_password        = serializer.data['new_password']
# 			retype_new_password = serializer.data['retype_new_password']

# 			if serializer.data['new_password'] == serializer.data['retype_new_password']:

# 				email_token = UserEmailToken.objects.filter(key = serializer.data['token']).last()
# 				if email_token:
# 					internal_user = email_token.internal_user
# 					internal_user.user.set_password(serializer.data['new_password'])
# 					internal_user.user.save()
# 					response["result"]  = 1
# 					response["message"] = "Your Password has been changed successfully"
# 					email_token.delete()
# 					return Response(response, status=status.HTTP_200_OK)
					
# 				else :
# 					response["result"] = 0
# 					response["errors"] = ["incorrect token"]
# 					self.add_api_call(request, response)
# 					return Response(response, status=status.HTTP_200_OK)
# 			else:
# 				response["result"] = 0
# 				response["errors"] = ["New Password and Re-enter New Password are not same"]
# 				return Response(response, status=status.HTTP_200_OK)

# 		else:
# 			response = handle_errors(serializer.errors)
# 			response["result"] = 0
# 			self.add_api_call(request, response)
# 			return Response(response, status=status.HTTP_200_OK)

########################################## Forgot Password END #####################################
