from django.urls import path, include
from .views import *

auth_urls = [
    path('web_login', WebLoginAPIView.as_view()),
    path('web_logout/', WebLogoutAPIView.as_view()),
    path('web_verify_user/', WebVerifyAPIView.as_view()),

]

player_urls = [
    path('personal_info/get', ProfileGETAPIView.as_view()),
    path('personal_info/post', ProfileEDITAPIView.as_view()),
    path('venue/get', PlayerVenueListApiView.as_view()),
    path('venue/get-recent', PlayerGetRecentVenuesAPIView.as_view()),
    path('profile', PlayerProfileAPIView.as_view()),
    path('profile/change_image', ChangePlayerProfilePicAPIView.as_view()),
    path('profile/post', PlayerPOSTProfileAPIView.as_view()),
    path('oauth/login/', SocialLoginView.as_view()),
    path('auth/', include('rest_framework_social_oauth2.urls')),
    path('newsletter', NewsLetterAPIView.as_view()),

    path('skills/get', PlayerSkillsAPIView.as_view()),
    path('skills/post', PersonalSkillsEDITAPIView.as_view()),
    path('delete_skills', PersonalSkillsDeleteAPIView.as_view()),

    path("add-player-interest", AddPlayerInterestAPIView.as_view()),
    path("get-player-interest", AddPlayerInterestAPIView.as_view()),
    path("delete-player-interest/<str:id>",
         AddPlayerInterestAPIView.as_view()),

    path("my_joins", PlayerMyJoinsAPIView.as_view()),
    path("join_action", ChangeJoinStatusAPIView.as_view()),
    path("join_by_id", ViewJoinByIDAPIView.as_view()),
    path("player_autocomplete", PlayerAutoCompleteAPIView.as_view()),
    path("invite", WebsiteInvitationAPIView.as_view()),

    path('find_opponent/post', FindOpponentAPIView.as_view()),
    path('get_all_player_skills', GetPlayerSkillsIDAPIView.as_view()),

    path('invite_opponent', InviteOpponentAPIView.as_view()),

    path('get_contact/get', GetContactsAPIView.as_view()),
    path('notify-contact', NotifyContactsAPIView.as_view()),
    path('availability', POSTAvailabilityAPIView.as_view()),
    path('get_field_price', GetFieldPriceAPIView.as_view()),

    path('delete_availability', DeleteAvailabilityAPIView.as_view()),

    path('venue_city', GetVenueBasedOnCityAPIView.as_view()),

    path('player_availability_notification',
         PlayerAvailabilityNotifyContactAPIView.as_view()),

    path('venue/post', PlayerVenueListPostApiView.as_view()),
    path('players/getAll', PlayerAllGetAPIView.as_view()),
    path('player/get', PlayerGetOnIDAPIView.as_view()),
    path('new_my_profile/get', PlayerMyProfileGETAPIView.as_view()),
    path('new_my_profile/edit', PlayerMyProfileEDITAPIView.as_view()),

    # Team Create
    path('team_detail', TeamDetailAPIView.as_view()),
    path('team_contact', TeamContactAPIView.as_view()),
    path('team_calendar', TeamCalendarAPIView.as_view()),
    path('team_member', TeamMembersAPIView.as_view()),
    path('get_team_member', GetTeamMemberAPIView.as_view()),

    path('get_all_teams', GetTeamsApiView.as_view()),


    path('team_global', GlobalTeamCreateGETAPIView.as_view()),

    # MY profile with images
    path("my_profile", MyProfileGetAPIView.as_view()),

    path("my_contacts/get_and_filter", MyContactsGetAndFilterAPIView.as_view()),
    path("my_contacts/invite_email",
         MyContactsSendFriendInvitationEmailAPIView.as_view()),
    path("my_contacts/edit_status", TeamMemberChangeStatusAPIView.as_view()),

]

# Book Match
field_details = [

    path('get_venue', GetVenueAPIView.as_view()),
    path('get_venue_map', GetVenueMapView.as_view()),
    path('venue_details', VenueDetailsAPIView.as_view()),

    path('get_field', FieldDetailsAPIView.as_view()),

    path('get_fav_field_filter', GetPlayerFavVenuesFilterAPIView.as_view()),
    path('get_field_autocomplete', GetFieldAutoCompleteAPIView.as_view()),
    path("book_match/get_player_venues", GetPlayerFavVenuesAPIView.as_view()),
    # same for both steps
    path("book_match/step", BookMatchStepAPIView.as_view()),
    path("book_match/send_user", BookMatchUserPOSTAPIView.as_view()),
    path("book_match/summary", BookMatchStep4APIView.as_view()),

    path("services_get", GetFieldServicesAPIView.as_view()),
    path("get_contacts", GetContactsPlayersAPIView.as_view()),
    path("send_requests", SendMatchRequestAPIView.as_view()),

    path("global_get", GlobalBookMatchAPIView.as_view()),


    # Nwe Booking Summery Apis
    path("book_match_new/post", BookMatchNewPOSTAPIView.as_view()),
    path("book_match_new_step2/post", BookMatchNew2POSTAPIView.as_view()),
    path("book_match_new_by_id/get", GetAllBookMatchSevicesAPIView.as_view()),
    path("book_match_new_step2/get", GetAllFieldAndVenueSevicesAPIView.as_view()),
    path("book_match_summery/get", BookMatchSummeryAPIView.as_view()),
    path("venue_paid_service_quantity/edit",
         VenuePaidServiceQuantityEDITAPIView.as_view()),
    path("venuefield_service_quantity/edit",
         VenueFieldServiceQuantityEDITAPIView.as_view()),
    path("venue_paid_service_quantity/delete",
         VenuePaidServiceQuantityDeleteAPIView.as_view()),
    path("venuefield_service_quantity/delete",
         VenueFieldServiceQuantityDeleteAPIView.as_view()),
    path("player_list/get",
         PlayersListAPIView.as_view()),
    path("player_send_email/post",
         SendPlayerInvitationEmailAPIView.as_view()),
    path("friend_send_email/post",
         SendFriendInvitationEmailAPIView.as_view()),

    # Player Second Way of Booking
    path("all_venues/get",
         GetAllVenuesAPIView.as_view()),
    path("all_venueservices/get",
         GetAllVenueServicesAPIView.as_view()),
    path("all_schedule_slots/get",
         GetScheduleSlotsAPIView.as_view()),
    path("popup_field_type/get",
         GetPopUpFieldTypeAPIView.as_view()),
    path("all_services_by_sports/get",
         GetFieldAndVenueSevicesBySportsAPIView.as_view()),

    # Player Suggestion api
    path("player_suggestions",
         GetSuggestionsForPlayerAPIView.as_view()),


]

search_team = [
    path('', FindTeamAPIView.as_view()),
    path('filter', FindATeamAPIView.as_view()),
    path('join_invite', JoinTeamAPIView.as_view()),
    path('join_team', JoinTeamFindATeamAPIView.as_view()),
    path('match_team_invite', MatchTeamRequestAPIView.as_view()),
    path('match_team', MatchTeamAPIView.as_view()),
    path('view_team_details', ViewTeamDetailsAPIView.as_view()),
]


my_matches = [
    path('filter', MyMatchesFilterAPIView.as_view()),
    path('games_played_graph', GamesPlayedByPlayerGraphCountAPIView.as_view()),
    path('win/loose_graph', GamesStatusByPlayerGraphCountAPIView.as_view()),
    path('mymatches_status/edit', MyMatchesEditInfoAPIView.as_view()),
]

urlpatterns = [
    path('auth/', include(auth_urls)),
    path('', include(player_urls)),
    path("field_details/", include(field_details)),
    path("find_team/", include(search_team)),
    path("my_matches/", include(my_matches))
]
