from django.db import models

from django.core.validators import MaxValueValidator, MinValueValidator
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.auth.models import User
from storages.backends.s3boto import S3BotoStorage

from django.core.files.images import get_image_dimensions
from django.core.exceptions import ValidationError

from static_data.models import *


def validate_image(image):
    w, h = get_image_dimensions(image)
    if w < 400 and h < 400:
        raise ValidationError("Image is not of correct Dimensions")


def validate_service_image(image):
    w, h = get_image_dimensions(image)
    if w != 1920 and h != 1201:
        raise ValidationError("Image is not of correct Dimensions")


class S3CustomStorage(S3BotoStorage):
    def _normalize_name(self, name):
        """
        S3 Storage function for storing files
        """
        print(name)
        return name


class Professional(models.Model):
    """ Professional user details reside here """
    GENDER_CHOICES = (
        ('M', 'Male'),
        ('F', 'Female'),
        ('O', 'Other')
    )
    STATUS_CHOICES = (
        ('I', 'In Progress'),
        ('R', 'Rejected'),
        ('N', 'New'),
        ('A', 'Approve'),
        ('C', 'Complete')
    )
    user = models.ForeignKey(
        User, on_delete=models.CASCADE, blank=True, null=True)
    first_name = models.CharField(
        max_length=100, null=True, blank=True, help_text="user first name")
    middle_name = models.CharField(
        max_length=100, null=True, blank=True, help_text="user middle name")
    last_name = models.CharField(
        max_length=100, null=True, blank=True, help_text="user last name")

    email = models.CharField(max_length=100, null=True,
                             blank=True, help_text="user email")
    new_email = models.CharField(
        max_length=100, null=True, blank=True, help_text="user new email")
    gender = models.CharField(max_length=1, null=True, blank=True,
                              choices=GENDER_CHOICES, help_text="M - Male, F - Female, O - Other")
    mobile = models.CharField(
        max_length=100, null=True, blank=True, help_text="mobile number")

    new_mobile = models.CharField(
        max_length=100, null=True, blank=True, help_text="new mobile number")
    telephone = models.CharField(max_length=50, blank=True, null=True)
    telephone2 = models.CharField(max_length=50, blank=True, null=True)
    address1 = models.CharField(max_length=150, blank=True, null=True,
                                help_text="Apartment, Suite, Unit, building, floor,etc.")
    tagline = models.CharField(
        max_length=50, blank=True, null=True, help_text="TagLine")
    address2 = models.CharField(
        max_length=150, blank=True, null=True, help_text="Street & Number, P.O.box c/o")
    #city = models.CharField(max_length=50, blank=True,null=True, help_text="city")
    #country = models.CharField(max_length=50, blank=True, null=True, help_text="country")
    user_code = models.CharField(max_length=6, null=True, blank=True)
    city = models.ForeignKey(
        City, on_delete=models.CASCADE, blank=True, null=True)
    country = models.ForeignKey(
        Country, on_delete=models.CASCADE, blank=True, null=True)
    area = models.ForeignKey(
        Area, on_delete=models.CASCADE, blank=True, null=True)
    state = models.ForeignKey(
        State, on_delete=models.SET_NULL, blank=True, null=True)
    street = models.CharField(max_length=50, blank=True,
                              null=True, help_text="street")
    number = models.CharField(max_length=50, blank=True,
                              null=True, help_text="number")
    payment_method = models.ForeignKey(
        PaymentMethod, on_delete=models.CASCADE, blank=True, null=True)
    business_name = models.CharField(max_length=50, blank=True, null=True)
    TaxOffices = models.CharField(
        max_length=10, blank=True, null=True, help_text="TaxOffices")
    buisness_vat = models.CharField(
        max_length=10, blank=True, null=True, help_text="user Buisness Vat")
    served_areas = models.ManyToManyField(
        Area, blank=True, related_name="professional_served_areas", help_text="serverd areas format [1,2]")
    latitude = models.FloatField(blank=True, null=True)
    longitude = models.FloatField(blank=True, null=True)
    google_address = models.CharField(max_length=450, blank=True, null=True)
    place_id = models.CharField(max_length=450, blank=True, null=True)
    status = models.CharField(
        max_length=10, default=STATUS_CHOICES[2][0], choices=STATUS_CHOICES, blank=True, null=True, help_text="Is document Verified")

    pincode = models.CharField(
        max_length=10, blank=True, null=True, help_text="code")
    image = models.ImageField(upload_to="professional_images",
                              blank=True, null=True, validators=[validate_image])
    dob = models.DateField(
        blank=True, null=True, help_text='date of birth format YYYY-MM-DD', verbose_name='date of birth')

    # service = models.CharField(
    #     max_length=100, blank=True, null=True, help_text="Service")
    service = models.ManyToManyField(Profession, blank=True)

    is_deleted = models.BooleanField(
        default=False, help_text="activate/deactivate user")
    is_email_verified = models.BooleanField(default=False)

    # created_by = models.ForeignKey(InternalUser, on_delete=models.CASCADE, blank=True, null=True)

    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)
    updated_at = models.DateTimeField(auto_now=True, blank=True, null=True)

    def __str__(self):
        return "{}-{}-{}".format(self.pk, self.last_name, self.first_name)

    class Meta:
        unique_together = ('user', 'email')
        verbose_name_plural = "Professionals"
        db_table = 'professional'


class ProfessionalDailyTiming(models.Model):
    """
    Weekly Timing of Professional
    """
    professional = models.ForeignKey(
        Professional, on_delete=models.CASCADE, blank=True, null=True)
    day = models.ForeignKey(
        DayofWeek, on_delete=models.SET_NULL, blank=True, null=True, help_text="Day")
    start_time = models.TimeField(
        blank=True, null=True, help_text="Start time")
    end_time = models.TimeField(blank=True, null=True, help_text="Start time")
    is_active = models.BooleanField(blank=True, null=True)
    is_deleted = models.BooleanField(default=False, null=True, blank=True)
    created_at = models.DateTimeField(auto_now=True, blank=True, null=True)

    def __str__(self):
        return "{}-{}-{}-{}".format(self.professional, self.day, self.start_time, self.end_time)


class ProfessionalServicePhoto(models.Model):
    """
    Professional Service Photos Data
    """
    main_image = models.ImageField(upload_to="service_photos",  help_text="Main Photo",
                                   blank=True, null=True, validators=[validate_service_image])
    # Mainimage  = models.CharField(max_length=100,help_text="Main Photo", blank = True, null = True)

    is_deleted = models.BooleanField(
        default=False, help_text="activate/deactivate Service Photo")
    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)

    created_by = models.ForeignKey(
        User, on_delete=models.CASCADE, blank=True, null=True)

    def __str__(self):
        return "{}-{}".format(self.main_image, self.created_by)

    class Meta:
        verbose_name_plural = "Service Photo"
        db_table = 'professional_service_photo'


class ProfessionalStepServiceData(models.Model):
    """
    Professional Step Service  Data
    """
    user = models.ForeignKey(
        User, on_delete=models.CASCADE, blank=True, null=True)
    professions = models.ManyToManyField(
        Profession, blank=True)

    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)
    is_deleted = models.BooleanField(
        default=False, help_text="activate/deactivate skills")

    service_description = models.TextField(
        max_length=500, blank=True, null=True)

    def __str__(self):
        return "{}".format(self.service_description)

    class Meta:
        verbose_name_plural = "Step Service Data"
        db_table = 'professional_step_data'


class ProfessionalService(models.Model):
    """
    Professional Services
    """
    profession = models.ForeignKey(
        Professional, on_delete=models.CASCADE, blank=True, null=True)
    service = models.ForeignKey(
        StaticProfessionalService, on_delete=models.CASCADE, blank=True, null=True)
    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)
    is_deleted = models.BooleanField(
        default=False, help_text="activate/deactivate skills")
    cost = models.IntegerField(blank=True, null=True, help_text="Costing")

    def __str__(self):
        return "{}-{}".format(self.service, self.profession)

    class Meta:
        verbose_name_plural = "Professional Service"
        db_table = 'professionalservice'


class ProfessionalCertificate(models.Model):
    """
    Professional Certificates Data
    """

    certificate_name = models.CharField(
        max_length=100,  help_text="Certificates", blank=True, null=True)
    certificate = models.FileField(
        upload_to="professional_certificates",  help_text="Certificates", blank=True, null=True)

    # certificate  = models.CharField(max_length=100,help_text="Main Photo", blank = True, null = True)
    is_deleted = models.BooleanField(
        default=False, help_text="activate/deactivate Certificate")
    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)

    created_by = models.ForeignKey(
        User, on_delete=models.CASCADE, blank=True, null=True)

    def __str__(self):
        return "{}-{}".format(self.certificate, self.created_by)

    class Meta:
        verbose_name_plural = "Certificate"
        db_table = 'professional_certificate'


class ProfessionalStepSkill(models.Model):
    """
    Professional Step Skills
    """

    skil = models.ForeignKey(
        Skill, on_delete=models.CASCADE, blank=True, null=True)
    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)

    is_deleted = models.BooleanField(
        default=False, help_text="activate/deactivate skills")
    created_by = models.ForeignKey(
        User, on_delete=models.CASCADE, blank=True, null=True)

    def __str__(self):
        return "{}-{}".format(self.skil, self.created_by)

    class Meta:
        verbose_name_plural = "Step Skills"
        db_table = 'professional_step_skill'


class ProfessionServiceAvailability(models.Model):
    """
    Profession Service
    """
    professional = models.ForeignKey(
        Professional, on_delete=models.CASCADE, blank=True, null=True)
    date = models.DateField(blank=True, null=True)
    service = models.ForeignKey(
        ProfessionalService, on_delete=models.CASCADE, blank=True, null=True)
    is_deleted = models.BooleanField(
        default=False, help_text="activate/deactivate skills")
    start_hour = models.TimeField(blank=True, null=True)
    end_hour = models.TimeField(blank=True, null=True)
    is_active = models.BooleanField(default=False, blank=True, null=True)
    day = models.ForeignKey(
        DayofWeek, on_delete=models.CASCADE, blank=True, null=True)

    def __str__(self):
        return "{}-{}".format(self.day, self.service)

    class Meta:
        verbose_name_plural = "Step Service Availability"
        db_table = 'step_service_availability'


class ProfessionalFavList(models.Model):
    """
    Professional Fav Lists
    """
    fav_profile = models.ForeignKey(
        User, on_delete=models.CASCADE, blank=True, related_name="Professional")
    created_by = models.ForeignKey(
        User, on_delete=models.CASCADE, blank=True, related_name="User_Name")

    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)
    is_deleted = models.BooleanField(
        default=False, help_text="activate/deactivate skills")

    def __str__(self):
        return "{}-{}".format(self.fav_profile, self.created_by)

    class Meta:
        verbose_name_plural = "Favourite"
        db_table = 'professional_favourite'


class ProfessionalServedArea(models.Model):
    latitude = models.FloatField(blank=True, null=True)
    longitude = models.FloatField(blank=True, null=True)
    google_address = models.CharField(max_length=450, blank=True, null=True)
    place_id = models.CharField(max_length=450, blank=True, null=True)
    created_by = models.ForeignKey(
        User, on_delete=models.CASCADE, blank=True, null=True)
    is_deleted = models.BooleanField(
        default=False, help_text="activate/deactivate Serve Area")
    professional = models.ForeignKey(
        Professional, on_delete=models.CASCADE, blank=True, null=True)

    class Meta:
        verbose_name_plural = "ServedArea"
        db_table = 'professional_ServedArea'

    def __str__(self):
        return "{}-{}-{}-{}-{}".format(self.professional, self.latitude, self.longitude, self.google_address, self.place_id)



class ProfessionalDocument(models.Model):
    """
    Professional Document Data
    """

    document_name = models.CharField(
        max_length=100,  help_text="Certificates", blank=True, null=True)
    document = models.FileField(
        upload_to="professional_document",  help_text="Documents", blank=True, null=True)

    # certificate  = models.CharField(max_length=100,help_text="Main Photo", blank = True, null = True)
    is_deleted = models.BooleanField(
        default=False, help_text="activate/deactivate Certificate")
    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)

    created_by = models.ForeignKey(
        User, on_delete=models.CASCADE, blank=True, null=True)

    def __str__(self):
        return "{}-{}".format(self.document_name, self.created_by)

    class Meta:
        verbose_name_plural = "Professional Documents"
        db_table = 'professional_document'


class ProfessionalCollaborationType(models.Model):
    professional = models.ForeignKey(Professional, on_delete=models.CASCADE, blank=True, null=True)
    collaboration_type = models.ForeignKey(CollaborationTypes, on_delete=models.CASCADE, blank=True, null=True)

    def __str__(self):
        return "{}-{}".format(self.professional, self.collaboration_type)

    class Meta:
        verbose_name_plural = "Professional Collaboration Types"
        db_table = 'professional_collaboration_type'



class ProfessionalContract(models.Model):
    professional = models.ForeignKey(Professional, on_delete=models.CASCADE, blank=True, null=True)
    contract_file = models.FileField(
        upload_to="professional_contract",  help_text="Contract", blank=True, null=True)
    is_deleted = models.BooleanField(default=False, blank=True, null=True)

    def __str__(self):
        return "{}".format(self.professional)

    class Meta:
        verbose_name_plural = "Professional Contracts"
        db_table = 'professional_contract'
