from rest_framework import serializers
from django.contrib.auth.models import User, Group
from rest_framework.authtoken.models import Token
from django.db.models import Q
from datetime import date
from utils.api_handle_errors import handle_errors
from .models import *
from static_data.models import Skill, Profession
from static_data.serializers import *
from reviews.models import Review
from commonAPI.models import TelephoneNumbers, ContactInfo, BankInfo, Contact_Person
from commonAPI.serializers import ContactInfoProfileSerializer, BankInfoProfileSerializer, ContactPersonSerializer
from django.db import transaction


class ProfessionalGETSerializer(serializers.ModelSerializer):
    user = serializers.CharField(source='user.username')
    service = ProfessionSerializer(
        required=False, read_only=True, many=True)
    served_areas = AreaGetSerializer(
        required=False, read_only=True, many=True)
    country = CountrySerializer(read_only=True)
    city = CityGetSerializer(read_only=True)
    state = StateGetSerializer(read_only=True)
    area = AreaGetSerializer(read_only=True)

    class Meta:
        model = Professional
        fields = '__all__'
        extra_kwargs = {
            'email': {'required': False, 'help_text': 'Email'},
            'user': {'required': False, 'help_text': 'User'},
        }


class ProfessionalSAVESerializer(serializers.ModelSerializer):
    class Meta:
        model = Professional
        fields = '__all__'


class ProfessionalDailyAvailibilitySerializer(serializers.ModelSerializer):
    class Meta:
        model = ProfessionalDailyTiming
        fields = '__all__'


class ServedAreaSerializer(serializers.ModelSerializer):
    class Meta:
        model = ProfessionalServedArea
        # fields = ["latitude","longitude","google_address","place_id"]
        fields = '__all__'


class deleteServedAreaSerializer(serializers.ModelSerializer):

    id = serializers.IntegerField(required=True)

    def validate_id(self, value):
        if ProfessionalServedArea.objects.filter(id=value, is_deleted=False).exists():
            return value
        else:
            raise serializers.FieldDoesNotExist("No Such Record Exists")

    class Meta:
        model = ProfessionalServedArea
        fields = ("id",)


class ProfessionalPOSTSerializer(serializers.ModelSerializer):
    Telephone_number = serializers.JSONField(
        required=False, help_text='{"mobile":["847123654","35135654"]}')
    iban = serializers.CharField(required=False)
    bank = serializers.CharField(required=False)

    acc_holder_first_name = serializers.CharField(required=False)
    acc_holder_last_name = serializers.CharField(required=False)
    acc_type = serializers.CharField(required=False)
    bank_identification_no = serializers.CharField(required=False)
    iban_certificate = models.FileField(
        upload_to='pdfs/', null=True, blank=True, help_text='IBAN CERTIFICATE')

    contact_person = serializers.JSONField(
        required=False, help_text='[{"first_name":"Test","last_name":"asd","telephone":1234567,"email":"email"},{"first_name":"Test","last_name":"asd","telephone":1234567,"email":"email"}]')
    mobile = serializers.CharField(required=False, help_text="Mobile number")

    timing = serializers.JSONField(
        required=False, help_text='[{"day":1,"start_time":12:00:00,"is_active":1,"end_time":1:00:00}]')

    ServedAreas = serializers.JSONField(
        required=False, help_text='[{"latitude":00.00, "longitude":00.00, "google_address":"FFF", "place_id":"ppp"}]')
    service = serializers.JSONField(required=False, help_text="[1,2,3]")
    state = serializers.IntegerField(
        required=False, help_text="state")
    area = serializers.IntegerField(
        required=False, help_text="area")
    city = serializers.IntegerField(
        required=False, help_text="city")
    country = serializers.IntegerField(
        required=False, help_text="country")

    def validate_mobile(self, value):
        if Professional.objects.filter(mobile=value, is_deleted=False).exists():
            # raise serializers.ValidationError("Mobile number is already linked")
            pass

        if len(value) < 10:
            raise serializers.ValidationError(
                "Mobile Number must be filled in.")
        else:
            return value

    @transaction.atomic
    def create(self, validated_data):
        '''
        Profession
        '''
        bool_data = False
        print(validated_data)
        if validated_data.get('user',False):
            bool_data = True
            self.context['user'] = validated_data.pop('user')

        validated_data['user'] = self.context['user'].id
        validated_data['email'] = self.context['user'].email
            

        serialized_data = ProfessionalSAVESerializer(instance=Professional.objects.filter(
            user=self.context['user']).first(), data=validated_data)

        if serialized_data.is_valid():
            professional_data = serialized_data.save()
        else:
            raise serializers.ValidationError(
                handle_errors(serialized_data.errors))

        '''
        multiple profession service data
        '''


        if ProfessionalStepServiceData.objects.filter(user=self.context['user']).exists():
            step_service_data = ProfessionalStepServiceData.objects.get(
                user=self.context['user'])
            step_service_data.professions.set(validated_data['service'])
            step_service_data.save()

            for pr2 in validated_data['service']:
                if ProfessionalStepSkill.objects.filter(~Q(skil__profession=pr2), created_by=self.context['user']).exists():
                    ProfessionalStepSkill.objects.filter(
                        ~Q(skil__profession=pr2), created_by=self.context['user']).delete()

        else:
            dataobj = ProfessionalStepServiceData.objects.create(
                user=self.context['user'])
            dataobj.professions.set(validated_data['service'])
            dataobj.save()
        print("done 1")
        '''
        Contact Info
        '''

        serialized_data = ContactInfoProfileSerializer(
            data=validated_data, context={"user": self.context['user']})
        if serialized_data.is_valid():
            contact_obj = Professional.objects.filter(
                user=self.context['user'])
            contact_contentType = ContentType.objects.get_for_model(
                Professional)
            ContactInfo.objects.filter(
                content_type__pk=contact_contentType.id, object_id__in=contact_obj).delete()
            serialized_data.validated_data['content_object'] = professional_data
            contact_info_data = serialized_data.save()
        else:

            raise serializers.ValidationError(
                handle_errors(serialized_data.errors))
        print("done 2")

        '''
        Timing
        '''
        if "timing" in validated_data:

            for i in validated_data['timing']:
                i['professional'] = professional_data.id
            timing_serializer = ProfessionalDailyAvailibilitySerializer(
                data=validated_data['timing'], many=True)

            if timing_serializer.is_valid():
                ProfessionalDailyTiming.objects.filter(
                    professional=professional_data.id).delete()
                timing_data = timing_serializer.save()
            else:
                print(timing_serializer.errors)
                raise serializers.ValidationError(
                    handle_errors(timing_serializer.errors))
        print("done 3")

        '''
        ServedArea
        '''
        if "ServedAreas" in validated_data:
            for i in validated_data['ServedAreas']:
                # ProfessionalServedArea.objects.filter(professional_id__pk=professional_data.id,professional_id__in=professional_data)
                i['professional'] = professional_data.id
            servearea_serializer = ServedAreaSerializer(
                data=validated_data['ServedAreas'], many=True)

            if servearea_serializer.is_valid():
                ProfessionalServedArea.objects.filter(
                    professional=professional_data.id).delete()
                servarea_data = servearea_serializer.save()
            else:
                print(servearea_serializer.errors)
                raise serializers.ValidationError(
                    handle_errors(servearea_serializer.errors))
        print("done 4")

        '''
        Contact Person
        '''
        if "contact_person" in validated_data:
            contact_persons = []
            Contact_Person.objects.filter(
                content_type__pk=contact_contentType.id, object_id__in=contact_obj).delete()
            # Contact_Person.objects.filter(content_type__pk=contact_contentType.id,object_id__in=contact_obj).all()

            for i in validated_data['contact_person']:
                if i['first_name'] == "" and i['last_name'] == "":
                    pass
                else:
                    contact_persons.append(i)

            serialized_data = ContactPersonSerializer(
                data=contact_persons, many=True)

            if serialized_data.is_valid():
                for i in serialized_data.validated_data:
                    i['content_object'] = professional_data
                serialized_data.save()
            else:
                raise serializers.ValidationError(
                    handle_errors(serialized_data.errors))
        print("done 5")

        '''
        Contact Numbers
        '''
        if "Telephone_number" in validated_data:
            TelephoneNumbers.objects.filter(
                contact_id=contact_info_data).delete()
            contact_numbers = [TelephoneNumbers(contact_id=contact_info_data,
                                                contact_number=i
                                                ) for i in validated_data['Telephone_number']['mobile']]

            TelephoneNumbers.objects.bulk_create(contact_numbers)
        print("done 6")

        '''
        Bank Info
        '''
        if 'bank' in validated_data:
            BankInfo.objects.filter(
                content_type__pk=contact_contentType.id, object_id__in=contact_obj).delete()
            serialized_data = BankInfoProfileSerializer(data=validated_data)
            if serialized_data.is_valid():
                serialized_data.validated_data['content_object'] = professional_data
                serialized_data.save()
            else:
                raise serializers.ValidationError(
                    handle_errors(serialized_data.errors))
        print("done 7")
        return professional_data.id


    class Meta:
        model = Professional
        fields = '__all__'
        extra_kwargs = {
            'new_email': {"read_only": True},
            'user': {"required": False}
        }


class UserSerializer(serializers.ModelSerializer):
    class Meta:
        model = User
        fields = '__all__'


class ServicePhotosSerializer(serializers.ModelSerializer):
    uploaded_by = serializers.CharField(
        source='created_by.username', read_only=True)

    class Meta:
        model = ProfessionalServicePhoto
        fields = "__all__"


class ServiceDataCustomSerializer(serializers.Serializer):
    # image part
    Mainimage = serializers.ListField(
        help_text='send array of photos', required=False)
    created_by = serializers.CharField(required=False)
    # Step service part
    #profession = serializers.CharField(help_text="id of the profession")
    professions = serializers.ListField(
        required=False, help_text="send same as main_image")
    serviceDescription = serializers.CharField()

    def validate_created_by(self, value):
        if User.objects.filter(username=value).exists():
            return value
        else:
            raise serializers.ValidationError("No User Founnd")

    def validate_professions(self, value):
        tp = []
        for p in value:
            print("outside: "+p)
            if Profession.objects.filter(id=p).exists():
                print("inside: "+p)
                tp.append(p)
                pass
            else:
                raise serializers.ValidationError("No Profession Found")
        return tuple(tp)

    def create(self, validated_data, **kwargs):
        validated_data['created_by'] = User.objects.get(
            username=validated_data['created_by'])

        '''validated_data['profession'] = Profession.objects.get(
            id=validated_data['profession'])'''

        if 'Mainimage' in validated_data:
            photo = [ProfessionalServicePhoto(
                main_image=i,
                created_by=validated_data['created_by']
            )for i in validated_data['Mainimage']]

            ProfessionalServicePhoto.objects.bulk_create(photo)
        prof = validated_data['professions']
        print(prof)
        validated_data.pop('professions')
        user_professional = Professional.objects.get(
            user=validated_data['created_by'])
        user_professional.service.set(prof)
        user_professional.save()
        ProfessionalStepServiceData.objects.filter(
            user=validated_data['created_by']).delete()
        data = ProfessionalStepServiceData.objects.create(
            user=validated_data['created_by'], service_description=validated_data['serviceDescription'])
        data.professions.set(prof)
        return "Success"


class ServiceDataSerializer(serializers.ModelSerializer):
    professions = ProfessionSerializer(
        required=False, read_only=True, many=True)

    class Meta:
        model = ProfessionalStepServiceData
        fields = '__all__'


class ServiceDataPhotoSerializer(serializers.ModelSerializer):
    professions = ProfessionSerializer(
        required=False, read_only=True, many=True)
    
    service_photo = serializers.SerializerMethodField("_get_service_photo")

    def _get_service_photo(self, instance):
        service_photo = ProfessionalServicePhoto.objects.filter(created_by=instance.user)
        service_class = ServicePhotosSerializer(service_photo, many=True)
        return service_class.data


    class Meta:
        model = ProfessionalStepServiceData
        fields = '__all__'




class DailyTimingGlobalSerializer(serializers.ModelSerializer):
    class Meta:
        model = ProfessionalDailyTiming
        fields = "__all__"


class GlobalGetSerializer(serializers.Serializer):
    UserId = serializers.IntegerField(write_only=True, required=False)

    def validate_UserId(self, value):
        if value:
            # if ProfessionalServicePhoto.objects.filter(created_by=value, is_deleted=False).exists() and ProfessionalStepServiceData.objects.filter(user=value, is_deleted=False).exists():
            if Professional.objects.filter(id=value, is_deleted=False).exists():
                return value
            else:
                raise serializers.ValidationError("No user found")


class DeleteServicePhotoSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(required=True)

    class Meta:
        model = ProfessionalServicePhoto
        fields = ['id']


class ProfessionalStepSkillCostEditSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(required=True)

    class Meta:
        model = ProfessionalStepSkill
        fields = ['id', 'cost']

class AddProfessionalServiceCostSerializer(serializers.Serializer):
    cost_obj = serializers.JSONField(required=True, help_text='[{"id":4,"cost":50}, {"professional_service_id":5,"cost":60}]')
    profession = serializers.PrimaryKeyRelatedField(queryset=Profession.objects.all(), required=True)

    def __update_record(self, id, cost):
        professoinal_service_obj = ProfessionalService.objects.filter(id=id)
        if professoinal_service_obj.exists():
            professoinal_service_obj = professoinal_service_obj.last()
        else:
            raise serializers.ValidationError("Incorrect Input Update")
        professoinal_service_obj.cost = cost
        professoinal_service_obj.save()
        return True
    
    def __create_record(self, id, cost):
        service_obj = StaticProfessionalService.objects.filter(id=id)
        if service_obj.exists():
            service_obj = service_obj.last()
        else:
            raise serializers.ValidationError("Incorrect Input")
        ProfessionalService.objects.create(service=service_obj, cost=cost, profession=self.context['professional_obj'])
        return True

    def save(self):
        cost_obj = self.validated_data['cost_obj']
        profession = self.validated_data['profession']
        update_ids = [i['professional_service_id'] for i in self.validated_data['cost_obj'] if "professional_service_id" in i]
        self.Meta.model.objects.filter(~Q(id__in=update_ids), profession=self.context['professional_obj'], service__profession=profession).delete()
        for i in self.validated_data['cost_obj']:
            if "id" in i:
                self.__create_record(i['id'], i['cost'])
            elif "professional_service_id" in i:

                self.__update_record(i['professional_service_id'], i['cost'])
        return True

    class Meta:
        model = ProfessionalService
        fields = ['cost_obj']
        

class GetServiceSkillSerializer(serializers.ModelSerializer):
    profession = serializers.CharField(source="skil.profession.professionName", required=False)
    skill_name = serializers.CharField(source="skil.skill_name", required=False)
    class Meta:
        model = ProfessionalStepSkill
        fields = '__all__'

class SkillsAvailabilitySerializer(serializers.Serializer):
    """
    Serializers for Skills Availability
    """
    METHOD_CHOICE = (
        ("WEEK", "WEEK"),
        ("MONTH", "MONTH"),
        ("DAY", "DAY")
    )
    start_date = serializers.DateField(required=True)
    method = serializers.ChoiceField(choices=METHOD_CHOICE, required=True)
    service = serializers.PrimaryKeyRelatedField(queryset=ProfessionalService.objects.all())


class SaveSkillsAvailabilitySerializer(serializers.ModelSerializer):
    """
    Skill Availability Store Serializer
    """
    availability_obj = serializers.JSONField(required=True)

    def save(self):
        validated_datas = self.validated_data['availability_obj']
        all_ids = [i['id'] for i in validated_datas]
        all_professional = [i['professional'] for i in validated_datas]
        self.Meta.model.objects.filter(~Q(id__in=all_ids), professional__in=all_professional).update(is_deleted=True)
        can_update_records = ['is_active']
        for validated_data in validated_datas:
            service_availability_obj = self.Meta.model.objects.filter(id=validated_data['id'], professional=validated_data['professional'])
            if service_availability_obj.exists():
                service_availability_obj = service_availability_obj.last()
                for (key, value) in validated_data.items():
                    if key in can_update_records:
                        setattr(service_availability_obj, key, value)
                service_availability_obj.save()
        
        return True

    class Meta:
        model = ProfessionServiceAvailability
        fields = ['availability_obj']

class ProfessionalServiceAvailabilitySerializer(serializers.ModelSerializer):

    is_active = serializers.SerializerMethodField("_get_is_active")

    def _get_is_active(self, obj):
        if self.Meta.model.objects.filter(**self.context, is_active=True).exists():
            return obj.is_active
        return True

    class Meta:
        model = ProfessionServiceAvailability
        fields = "__all__"

class StepSkillsSerializer(serializers.Serializer):
    skill_name = serializers.CharField(
        required=False, help_text="Skill1,skill2,skill3")
    profession_id = serializers.IntegerField(required=False, write_only=True)
    created_by = serializers.CharField(required=False, write_only=True)

    def validate_profession_id(self, value):
        profession_obj = Profession.objects.filter(id=value)
        if profession_obj.exists():
            return profession_obj.last()
        raise serializers.ValidationError("Invalid Profession")

    def validate_skill_name(self, value):
        skill_name = [Skill.objects.get(id=i)
                      for i in value.split(',')]
        return skill_name

    def create(self, validated_data, **kwargs):
        try:
            created_by = User.objects.get(username=self.context['user'])
            skills_obj = [ProfessionalStepSkill(
                skil=i,
                created_by=self.context['user']
            ) for i in validated_data['skill_name']]
            data = ProfessionalStepSkill.objects.bulk_create(skills_obj)
            return "Success"

        except Exception as e:
            raise serializers.ValidationError(
                "Error Occureed in serializer "+str(e))
        # return data


class StepSkillsReadSerializer(serializers.ModelSerializer):
    skill_id = serializers.CharField(source='skill_name.id')
    skill_name = serializers.CharField(source='skill_name.skillName')
    created_by = serializers.CharField(source='created_by.username')

    class Meta:
        model = ProfessionalStepSkill
        fields = "__all__"


class StepSkillsGlobalSerializer(serializers.ModelSerializer):
    skil = SkillsSetSerializer(
        required=False, read_only=True)

    class Meta:
        model = ProfessionalStepSkill
        fields = '__all__'


class StepSkillsDeleteSerializer(serializers.Serializer):
    id = serializers.IntegerField()

    def validate_id(self, value):
        if ProfessionalStepSkill.objects.filter(id=value).exists():
            return value
        else:
            raise serializers.ValidationError("Already selected skill ")


class CertificateGlobalSerializer(serializers.ModelSerializer):
    class Meta:
        model = ProfessionalCertificate
        fields = "__all__"


class CertificatesSerializer(serializers.ModelSerializer):
    uploaded_by = serializers.CharField(
        source='created_by.username', read_only=True)
    certificate_file = serializers.ListField(required=False, write_only=True)
    file_data = serializers.JSONField(
        required=False, help_text='[{"doc_name1":"file_name1","user_input":"test"},{"doc_name2":"file_name2","user_input":"test"}]', write_only=True)
    professional_email = serializers.CharField(required=False, write_only=True)

    def create(self, validated_data, format=None):
        try:
            certificate = [ProfessionalCertificate(
                certificate=i,
                certificate_name=next(
                    item for item in validated_data['file_data'] if item["doc_name1"] == i.name)['user_input'],
                created_by=self.context['user']
            ) for i in validated_data['certificate_file']]
            certificate = ProfessionalCertificate.objects.bulk_create(
                certificate)
            return True

        except Exception as e:
            raise serializers.ValidationError(
                "Error Occureed in serializer "+str(e))

    class Meta:
        model = ProfessionalCertificate
        fields = "__all__"
        read_only_fields = ['is_deleted', 'certificate', 'certificate_name']


class DeleteCertificateSerializer(serializers.Serializer):
    id = serializers.IntegerField(required=True)

    def validate_id(self, value):
        if ProfessionalCertificate.objects.filter(id=value).exists():
            return value
        else:
            raise serializers.ValidationError("No Record Found")


class getProfessionSerializer(serializers.ModelSerializer):
    profession = serializers.CharField(source='profession.professionName')
    profession_id = serializers.CharField(source='profession.id')

    class Meta:
        model = ProfessionalStepServiceData
        fields = ['profession', 'profession_id']


class MySkillsSerializer(serializers.Serializer):
    skill_name = serializers.CharField(
        required=False, help_text="Skill1,skill2,skill3")
    created_by = serializers.CharField(required=False)
    certificate = serializers.ListField(required=False,)
    file_data = serializers.JSONField(
        required=False, help_text='{"doc_name1":"file_name1","doc_name2":"file_name2"}')
    profession = serializers.CharField(
        help_text="ID of profession", write_only=True, required=False)

    def validate_profession(self, value):
        profession = Profession.objects.get(id=value)
        return profession

    def validate_skill_name(self, value):
        skill_name = [Skill.objects.get(id=i)
                      for i in value.split(',')]
        return skill_name

    def validate_created_by(self, value):
        user = User.objects.get(username=value)
        return user

    def create(self, validated_data, **kwargs):
        try:

            if 'skill_name' in validated_data:
                ProfessionalStepSkill.objects.filter(
                    created_by=validated_data['created_by']).delete()
                skills = [ProfessionalStepSkill(
                    skill_name=i,
                    created_by=validated_data['created_by']
                ) for i in validated_data['skill_name']]
                ProfessionalStepSkill.objects.bulk_create(skills)

            # Certificate
            if 'certificate' in validated_data:
                for i in validated_data['certificate']:
                    if i.size == 0:
                        raise serializers.ValidationError(
                            "Empty file cannot be uploaded")

                certificate = [ProfessionalCertificate(
                    certificate=i,
                    certificate_name=i.name,
                    created_by=validated_data['created_by']
                ) for i in validated_data['certificate']]
                certificate = ProfessionalCertificate.objects.bulk_create(
                    certificate)

            if 'profession' in validated_data:
                if ProfessionalStepServiceData.objects.filter(user=validated_data['created_by']).exists():
                    profession = ProfessionalStepServiceData.objects.filter(
                        user=validated_data['created_by']).first()
                    profession.profession = validated_data['profession']
                    profession.save(update_fields=['profession'])
                else:
                    data = ProfessionalStepServiceData(
                        user=validated_data['created_by'], profession=validated_data['profession'])
                    data.save()
            return "Success"
        except Exception as e:
            raise serializers.ValidationError(
                "Error Occureed in serializer "+str(e))


class FavListGETSerializer(serializers.ModelSerializer):
    created_by = serializers.CharField(required=False)
    fav_profile = serializers.IntegerField(
        required=False, help_text="Keep Empty to get all")

    def validate_created_by(self, value):
        created_by = User.objects.get(username=value).id
        return created_by

    def validate_fav_profile(self, value):
        if User.objects.filter(id=value).exists():
            return value

    class Meta:
        model = ProfessionalFavList
        fields = '__all__'


class FavListSerializer(serializers.ModelSerializer):
    created_by = serializers.CharField(source="created_by.username")
    fav_profile = serializers.CharField(source="fav_profile.username")

    class Meta:
        model = ProfessionalFavList
        fields = '__all__'


class FavListDeleteSerializer(serializers.Serializer):
    id = serializers.IntegerField()

    def validate_id(self, value):
        if ProfessionalFavList.objects.filter(id=value).exists():
            return value
        else:
            raise serializers.ValidationError("Already Favourite")


class FavListAddSerializer(serializers.ModelSerializer):
    created_by = serializers.CharField(required=False, help_text="keep Blank")
    fav_profile = serializers.IntegerField(
        required=True, help_text="ID of the Profile")
    is_deleted = serializers.BooleanField(read_only=True)

    def create(self, validated_data, **kwargs):
        validated_data['created_by'] = User.objects.get(
            username=validated_data['created_by'])
        validated_data['fav_profile'] = User.objects.get(
            id=validated_data['fav_profile'])
        if not ProfessionalFavList.objects.filter(created_by=validated_data['created_by'], fav_profile=validated_data['fav_profile'], is_deleted=False).exists():
            Fav = ProfessionalFavList.objects.create(
                created_by=validated_data['created_by'], fav_profile=validated_data['fav_profile'])
            return Fav
        else:
            raise serializers.ValidationError("Already Favourite")

    class Meta:
        model = ProfessionalFavList
        fields = '__all__'


class DegreeCertificateSerializer(serializers.ModelSerializer):
    certificate_file = serializers.FileField(
        allow_empty_file=False, required=True)
    cname = serializers.CharField(required=False, help_text="filename")

    def create(self, validated_data, format=None):
        ProfessionalCertificate.objects.create(
            certificate=validated_data['certificate_file'], certificate_name=validated_data['cname'], created_by=self.context['user'])
        return True

    class Meta:
        model = ProfessionalCertificate
        fields = "__all__"
        read_only_fields = ['is_deleted', 'certificate', 'certificate_name']



class GetProfessionalCardSerializer(serializers.ModelSerializer):
    """
    Professional Card Serializer
    """
    address = serializers.SerializerMethodField("_get_address")
    age = serializers.SerializerMethodField("_get_age")
    stars = serializers.SerializerMethodField("_get_stars")
    service_image = serializers.SerializerMethodField("_get_service_image")
    services = serializers.SerializerMethodField("_get_services")

    def _get_service_image(self, obj):
        service_pic = ProfessionalServicePhoto.objects.filter(created_by=obj.user, is_deleted=False).first()
        if service_pic is not None:
            service_pic_class = ServicePhotosSerializer(service_pic, many=False).data
            print(service_pic_class)
            return service_pic_class['main_image']
        return ""

    def _get_services(self, obj):
        service_data = ProfessionalStepServiceData.objects.filter(user=obj.user, is_deleted=False)
        service_data_class = ServiceDataSerializer(service_data, many=True)
        return service_data_class.data


    def calculateAge(self, born):
        if not born:
            return ""
        today = date.today()
        try: 
            birthday = born.replace(year = today.year)
        except ValueError: 
            birthday = born.replace(year = today.year,
                    month = born.month + 1, day = 1)
    
        if birthday > today:
            return today.year - born.year - 1
        else:
            return today.year - born.year
    
    def _get_stars(self, obj):
        from reviews.models import ProfessionalReviewAndRating
        from django.db.models import Avg

        data = ProfessionalReviewAndRating.objects.filter(
                professional=obj).aggregate(Avg("rating"))
        if data['rating__avg'] == None:
            return 0
        return data['rating__avg']

    def _get_address(self, obj):
        if obj.address1 and obj.address2:
            return str(obj.address1) + "|" + str(obj.address2)
        return ""

    def _get_age(self, obj):
        return self.calculateAge(obj.dob)

    class Meta:
        model = Professional
        fields = ['tagline', 'stars', 'first_name', 'age', 'address', 'user', 'service_image', 'services']


class DocumentsSerializer(serializers.ModelSerializer):
    class Meta:
        model = ProfessionalDocument
        fields = '__all__'

class ProfessionalContractSerializer(serializers.ModelSerializer):
    class Meta:
        model = ProfessionalContract
        fields = '__all__'

        

class ProfessionalDocumentSerializer(serializers.ModelSerializer):


    class Meta:
        model = ProfessionalDocument
        fields = '__all__'



class ProfessionalCertificateSerializer(serializers.ModelSerializer):


    class Meta:
        model = ProfessionalCertificate
        fields = '__all__'
