import pytz
import json
from django.http import HttpResponse
from django.template import Template, Context
from django.template.loader import get_template
from django.db import transaction
from django.urls import reverse

from django.contrib.auth import login as django_login
import datetime
from rest_framework.authtoken.models import Token
from rest_framework import status, generics
from rest_framework.response import Response
from rest_framework.authentication import TokenAuthentication
from rest_framework.permissions import IsAuthenticated, DjangoModelPermissions
from utils.mixins import PlatformAPICallMixin
from utils.api_handle_errors import handle_errors

from professionals.serializers import *
from professionals.models import *
from reviews.serializers import ReviewGETSerializer
from reviews.models import Review


class FavGETAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    '''
    GET All Fav List
    '''
    authentication_classes = (TokenAuthentication,)
    permission_classes = (DjangoModelPermissions,)
    queryset = Professional.objects.none()
    serializer_class = FavListGETSerializer
    get_Serializer_class = FavListSerializer

    def post(self, request, format=None):

        response = {}
        responsedata = []
        request.data._mutable = True
        request.data['created_by'] = str(request.user)
        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            if 'fav_profile' in serializers.validated_data:
                data = ProfessionalFavList.objects.filter(
                    created_by=serializers.data['created_by'], is_deleted=False, fav_profile=serializers.data['fav_profile'])
                SerializeData = self.get_Serializer_class(data, many=True)
            else:
                data = ProfessionalFavList.objects.filter(
                    created_by=serializers.data['created_by'], is_deleted=False)
                SerializeData = self.get_Serializer_class(data, many=True)
            response['result'] = 1
            response['data'] = SerializeData.data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializers.errors)
            response['result'] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class FavAddAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    '''
    Add Fav List
    '''
    authentication_classes = (TokenAuthentication,)
    # permission_classes = (DjangoModelPermissions,)
    permission_classes = (IsAuthenticated,)
    queryset = Professional.objects.none()
    serializer_class = FavListAddSerializer

    def post(self, request, format=None):

        response = {}
        responsedata = []
        request.data._mutable = True
        request.data['created_by'] = str(request.user)
        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            serializers.save()
            response['result'] = 1
            responsedata.append("Successfully Inserted")
            response['errors'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializers.errors)
            response['result'] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class FavDeleteAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    '''
    Delete Fav List
    '''
    authentication_classes = (TokenAuthentication,)
    permission_classes = (DjangoModelPermissions,)
    queryset = Professional.objects.none()
    serializer_class = FavListDeleteSerializer

    def post(self, request, format=None):

        response = {}
        responsedata = []
        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            fav = ProfessionalFavList.objects.filter(
                id=serializers.data['id']).first()
            fav.is_deleted = True
            fav.save(update_fields=['is_deleted'])
            response['result'] = 1
            responsedata.append("Successfully Deleted")
            response['errors'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)
        else:
            response = handle_errors(serializers.errors)
            response['result'] = 1
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)
