import pytz
import json
from django.http import HttpResponse
from django.template import Template, Context
from django.template.loader import get_template
from django.db import transaction
from django.urls import reverse

from django.contrib.auth import login as django_login
import datetime
from rest_framework.authtoken.models import Token
from rest_framework import status, generics
from rest_framework.response import Response
from rest_framework.authentication import TokenAuthentication
from rest_framework.permissions import IsAuthenticated, DjangoModelPermissions
from utils.mixins import PlatformAPICallMixin
from utils.api_handle_errors import handle_errors

from professionals.serializers import *
from professionals.models import *
from reviews.serializers import ReviewGETSerializer
from reviews.models import Review


class DocumentsAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = DocumentsSerializer

    def get_professional_obj(self, user):
        professional_obj = Professional.objects.filter(user=user)
        assert professional_obj.exists(), "No Such Player Found"
        return professional_obj.last()

    def get(self, request, format=None):
        """
        Document Get Api
        """
        try:
            response = {}
            responsedata = []
            professional_obj = self.get_professional_obj(request.user)
            doc_obj = self.serializer_class.Meta.model.objects.filter(is_deleted=False, created_by=request.user)
            doc_class = self.serializer_class(doc_obj, many=True)
            response["result"] = 1
            response["data"] = doc_class.data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        except AssertionError as ae:
            response["result"] = 0
            response["errors"] = [str(ae)]
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_401_UNAUTHORIZED)
        except Exception as e:
            response["result"] = 0
            response["errors"] = [str(e)]
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
    
    def post(self, request, format=None):
        """ 
        Documents Create API View
        """ 
        response = {}
        responsedata = []
        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            serializers.save()
            response["result"] = 1
            response["message"] = ["Successfully Uploaded"]
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializers.errors)
            response['result'] = 0
            self.add_api_call(request, response)
            return Response(response,status=status.HTTP_200_OK)

    def delete(self, request, format=None):
        """
        Document Delete API View
        """
        try:
            response = {}
            responsedata = []
            id = request.GET.get("id", False)
            assert id, "No Document Selected"
            doc_obj = ProfessionalDocument.objects.filter(id=id, created_by=request)
            assert doc_obj.exists(), "Invalid Document Selected"
            doc_obj = doc_obj.last()
            doc_obj.is_deleted=True
            doc_obj.save()
            response["result"] = 1
            response["message"] = ["Successfully Deleted"]
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        except Exception as e:
            response["result"] = 0
            response["errors"] = [str(e)]
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

from rest_framework.mixins import CreateModelMixin, RetrieveModelMixin, ListModelMixin, DestroyModelMixin
class ProfessionalConListView(CreateModelMixin, RetrieveModelMixin, ListModelMixin,DestroyModelMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    queryset = ProfessionalContract.objects.all()
    serializer_class = ProfessionalContractSerializer

    def perform_create(self, serializer):
        serializer.validated_data['professional'] = Professional.objects.filter(user=self.user).last()
        super().perform_create(serializer)

    def create(self,request, *args, **kwargs):
        super().create(request, *args, **kwargs)
        response = {"message":"Successfully Created", "result":1}
        return Response(response, status=status.HTTP_201_CREATED)
        
    def post(self, request, *args, **kwargs):
        self.user = request.user
        return self.create(request, *args, **kwargs)
    
    def get(self, request, *args, **kwargs):
        return self.list(request, *args, **kwargs)


class ProfessionalConDetailView(RetrieveModelMixin,DestroyModelMixin, generics.GenericAPIView):
    queryset = ProfessionalContract.objects.all()
    serializer_class = ProfessionalContractSerializer

    def destroy(self, request, *args, **kwargs):
        super().destroy(request, *args, **kwargs)
        response = {"message":"Successfully Deleted", "result":1}
        return Response(response, status=status.HTTP_201_CREATED)

    
    def get(self, request, *args, **kwargs):
        return self.retrieve(request, *args, **kwargs)

    def delete(self, request, *args, **kwargs):
        return self.destroy(request, *args, **kwargs)