import pytz
import json
from django.http import HttpResponse
from django.template import Template, Context
from django.template.loader import get_template
from django.db import transaction
from django.db.models import Avg
from django.urls import reverse

from django.contrib.auth import login as django_login
import datetime
from rest_framework.authtoken.models import Token
from rest_framework import status, generics
from rest_framework.response import Response
from rest_framework.authentication import TokenAuthentication
from rest_framework.permissions import IsAuthenticated, DjangoModelPermissions
from utils.mixins import PlatformAPICallMixin
from utils.api_handle_errors import handle_errors

from professionals.serializers import *
from professionals.models import *
from reviews.serializers import ReviewGETSerializer
from reviews.models import Review
from utils.decorator import check_permission
from commonAPI.serializers import ContactPersonSerializer
from commonAPI.models import Contact_Person, ContractTerms


class GetProfessionalAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    '''
    Get Professional Data
    '''
    authentication_classes = (TokenAuthentication,)
    permission_classes = (DjangoModelPermissions,)
    serializer_class = ProfessionalGETSerializer
    # dailySerializer_class = DailyTimingGlobalSerializer
    dailySerializer_class = ProfessionalDailyAvailibilitySerializer
    servedarea_class = ServedAreaSerializer
    photo_class = ServicePhotosSerializer
    serviceData_class = ServiceDataSerializer
    queryset = Professional.objects.none()

    # Get Professional
    def get(self, request, format=None):

        response = {}
        responsedata = []
        ProfessionalInfo = Professional.objects.filter(
            user=request.user, is_deleted=False)

        serverd_area = ProfessionalServedArea.objects.filter(
            professional_id__in=ProfessionalInfo, is_deleted=False)
        serverd_area_serializer = ServedAreaSerializer(serverd_area, many=True)

        professional_data = ContentType.objects.get_for_model(Professional)

        working_hours = ProfessionalDailyTiming.objects.filter(
            professional_id__in=ProfessionalInfo)

        serviceData = ProfessionalStepServiceData.objects.filter(
            user=ProfessionalInfo[0].user, is_deleted=False)

        serviceDataSerializer = self.serviceData_class(
            serviceData, many=True).data

        working_hours_serializer = ProfessionalDailyAvailibilitySerializer(
            working_hours, many=True)

        content_type = ContentType.objects.get_for_model(Professional)
        contact_person = Contact_Person.objects.filter(
            content_type__pk=content_type.id, object_id__in=ProfessionalInfo)

        contact_person_serializer = ContactPersonSerializer(
            contact_person, many=True)

        professional_photo = ProfessionalServicePhoto.objects.filter(
            created_by=request.user, is_deleted=False)
        professional_photo_class = self.photo_class(
            professional_photo, many=True).data

        serializer = self.serializer_class(ProfessionalInfo, many=True)
        responsedata = [{"contact_person": contact_person_serializer.data}, {
            "working_hours": working_hours_serializer.data}, {"service_data": serviceDataSerializer}, {"service_photo": professional_photo_class}, {"serverd_area": serverd_area_serializer.data}]

        responsedata = serializer.data + responsedata

        response['result'] = 1
        response['data'] = responsedata
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


class POSTProfessionalAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    '''
    Update Professional Data
    '''
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = ProfessionalPOSTSerializer
    # servedarea_class = ServedAreaSerializer
    queryset = Professional.objects.none()

    def post(self, request, format=None):
        print(request)
        response = {}
        responsedata = []
        serializers = self.serializer_class(
            data=request.data, context={"user": request.user})
        # print(serializers)
        if serializers.is_valid():
            data = serializers.save()
            # print(data)
            response['result'] = 1
            responsedata.append("Successfully Updated")
            response['errors'] = responsedata
            response['ID'] = data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializers.errors)
            response['result'] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class DeleteServeAreaAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = deleteServedAreaSerializer

    def post(self, request, format=None):
        ProfessionalInfo = Professional.objects.filter(
            user=request.user, is_deleted=False)
        response = {}
        responsedata = []
        serializer = self.serializer_class(data=request.data)
        if serializer.is_valid():
            if ProfessionalServedArea.objects.filter(created_by=request.user, id=serializer.data['id'], professional_id__in=ProfessionalInfo, is_deleted=False).exists():
                data = ProfessionalServedArea.objects.filter(
                    created_by=request.user, id=serializer.data['id'], is_deleted=False).first()
            data.is_deleted = True
            data.save(update_fields=['is_deleted'])
            response['result'] = 1
            responsedata.append("Successfully Deleted")
            response['errors'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response['result'] = 1
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class GlobalGETAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    '''
    Global Professional Get
    '''
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    # permission_classes = (DjangoModelPermissions,)
    queryset = Professional.objects.none()
    PhotoSerializer_class = ServicePhotosSerializer
    dailySerializer_class = DailyTimingGlobalSerializer
    certificate_class = CertificateGlobalSerializer
    reviewSerializer_class = ReviewGETSerializer
    serviceskillSerializer_class = StepSkillsGlobalSerializer
    serviceData_class = ServiceDataSerializer
    serializer_class = GlobalGetSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []
        serializer_data = self.serializer_class(data=request.data)
        if serializer_data.is_valid():
            if 'UserId' in serializer_data.validated_data:

                professional = Professional.objects.filter(
                    id=serializer_data.validated_data['UserId'])
                photo = ProfessionalServicePhoto.objects.filter(
                    created_by=professional[0].user, is_deleted=False)
                certificate = ProfessionalCertificate.objects.filter(
                    created_by=request.user, is_deleted=False)
                step_skills = ProfessionalStepSkill.objects.filter(
                    created_by=professional[0].user, is_deleted=False)
                reviewTo_contentType = ContentType.objects.get_for_model(
                    Professional)
                review = Review.objects.filter(
                    rated_to_type__pk=reviewTo_contentType.id, rated_to_id__in=professional, is_deleted=False)
                rating_avg = review.aggregate(Avg("rating"))
                if rating_avg['rating__avg'] == None:
                    rating_avg = 0
                serviceData = ProfessionalStepServiceData.objects.filter(
                    user=professional[0].user, is_deleted=False)
                working_hours = ProfessionalDailyTiming.objects.filter(
                    professional=professional.first(), is_deleted=False)
                cancel_query_set = ""
                if ContractTerms.objects.filter(created_for=professional[0].user).exists():
                    contract_obj = ContractTerms.objects.get(
                        created_for=professional[0].user)
                    print(contract_obj.cancellation_policy.all())
                    cancel_query_set = contract_obj.cancellation_policy.all()

            else:
                professional = Professional.objects.filter(user=request.user)
                photo = ProfessionalServicePhoto.objects.filter(
                    created_by=request.user, is_deleted=False)
                certificate = ProfessionalCertificate.objects.filter(
                    created_by=request.user, is_deleted=False)
                step_skills = ProfessionalStepSkill.objects.filter(
                    created_by=request.user, is_deleted=False)
                reviewTo_contentType = ContentType.objects.get_for_model(
                    Professional)
                review = Review.objects.filter(
                    rated_to_type__pk=reviewTo_contentType.id, rated_to_id__in=professional, is_deleted=False)
                rating_avg = review.aggregate(Avg("rating"))
                if rating_avg['rating__avg'] == None:
                    rating_avg = 0

                serviceData = ProfessionalStepServiceData.objects.filter(
                    user=request.user, is_deleted=False)
                working_hours = ProfessionalDailyTiming.objects.filter(
                    professional=professional.first(), is_deleted=False)
                cancel_query_set = ""
                if ContractTerms.objects.filter(created_for=professional[0].user).exists():
                    contract_obj = ContractTerms.objects.get(
                        created_for=professional[0].user)
                    print(contract_obj.cancellation_policy.all())
                    cancel_query_set = contract_obj.cancellation_policy.all()

            photoSerializer = self.PhotoSerializer_class(photo, many=True)
            reviewSerializer = self.reviewSerializer_class(review, many=True)
            serviceDataSerializer = self.serviceData_class(
                serviceData, many=True)
            skillSerializer = self.serviceskillSerializer_class(
                step_skills, many=True)
            certificateSerializer = self.certificate_class(
                certificate, many=True)
            dailySerializer = self.dailySerializer_class(
                working_hours, many=True)
            serializers_CancellationPolicy = CancellationPolicyySerializer(
                cancel_query_set, many=True)

            professional = professional.first()
            address = str(professional.address1) + " " + str(professional.address2) + \
                ", " + str(professional.city) + ", " + \
                str(professional.country)
            address = address.replace("None", "").strip()
            description = ""
            if serviceData is not None:
                description = serviceData.first().service_description
            #description = serviceData.first().service_description

            responsedata = [{'servicePhoto': photoSerializer.data}, {"avg_rating": rating_avg}, {"description": description}, {"working_hours": dailySerializer.data}, {"address": address}, {
                'review': reviewSerializer.data}, {'serviceData': serviceDataSerializer.data}, {'certificate': certificateSerializer.data}, {"skill": skillSerializer.data}, {"cancellation_policy": serializers_CancellationPolicy.data}]
            response['data'] = responsedata
            response['first_name'] = professional.first_name
            response['last_name'] = professional.last_name
            response['lat'] = professional.latitude
            response['long'] = professional.longitude
            response['result'] = 1
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer_data.errors)
            response['result'] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class GetProfessionalCardAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = GetProfessionalCardSerializer

    def get(self, request, format=None):
        """ 
        Get Professional Card APIView
        """
        response = {}
        responsedata = []

        professional_obj = Professional.objects.filter(
            user=request.user).last()
        profssional_class = self.serializer_class(
            professional_obj, many=False).data

        response["result"] = 1
        response["data"] = profssional_class
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)
