import pytz
import json
from django.http import HttpResponse
from django.template import Template, Context
from django.template.loader import get_template
from django.db import transaction
from django.urls import reverse

from django.contrib.auth import login as django_login
import datetime
from rest_framework.authtoken.models import Token
from rest_framework import status, generics
from rest_framework.response import Response
from rest_framework.authentication import TokenAuthentication
from rest_framework.permissions import IsAuthenticated,DjangoModelPermissions
from utils.mixins import PlatformAPICallMixin
from utils.api_handle_errors import handle_errors

from professionals.serializers import *
from professionals.models import *
from reviews.serializers import ReviewGETSerializer
from reviews.models import Review
from static_data.models import Skill, Profession
from static_data.serializers import GetProfessionalProfessionSerializer
import calendar


class serviceImageAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    '''
    Add Service Image
    '''
    authentication_classes = (TokenAuthentication,)
    permission_classes = (DjangoModelPermissions,)
    queryset = Professional.objects.none()
    serializer_class = ServicePhotosSerializer
    # Add Photo
    def get(self, request, format=None):
        response = {}
        responsedata = []
        data = ProfessionalServicePhoto.objects.filter(
            created_by=request.user, is_deleted=False)
        serializer = self.serializer_class(data, many=True)
        response['result'] = 1
        response['data'] = serializer.data
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


    def post(self,request,format=None):
        response = {}
        responsedata = []
        request.data._mutable = True
        user = User.objects.get(username=request.user)
        request.data['created_by'] = user.id
        serializer = self.serializer_class(data=request.data)
        if serializer.is_valid():
            serializer.validated_data['created_by'] = user
            serializer.save()
            responsedata.append("Successfully Created")
            response['result'] = 1
            response['errors'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response['result'] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)



class ServiceDataAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    '''
    Add Service Image and Profession Data
    '''
    authentication_classes = (TokenAuthentication,)
    permission_classes = (DjangoModelPermissions,)
    queryset = Professional.objects.none()
    serializer_class = ServiceDataCustomSerializer

    def post(self,request,format=None):
        response = {}
        responsedata = []
        request.data._mutable = True
        request.data['created_by'] = str(request.user)
        serializer = self.serializer_class(data=request.data)
        if serializer.is_valid():
            serializer.save()
            responsedata.append("Successfully Created")
            response['result'] = 1
            response['errors'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response['result'] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        


class DeleteServiceImageAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    '''
    Delete Service Image  url=DeleteService
    '''
    authentication_classes = (TokenAuthentication,)
    permission_classes = (DjangoModelPermissions,)
    queryset = Professional.objects.none()
    serializer_class = DeleteServicePhotoSerializer
    @transaction.atomic
    def post(self, request, format=None):
        serializer = self.serializer_class(data=request.data)
        if serializer.is_valid():
            responsedata = []
            response = {}
            if ProfessionalServicePhoto.objects.filter(created_by=request.user, id=serializer.data['id'], is_deleted=False).exists():
                data = ProfessionalServicePhoto.objects.filter(
                    created_by=request.user, id=serializer.data['id'], is_deleted=False).first()
                data.is_deleted = True
                data.save(update_fields=['is_deleted'])
            response['result'] = 1
            response['data'] = "Successfully deleted"
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
    

class EditServiceDataAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    '''
    Edit Service Data
    '''
    authentication_classes = (TokenAuthentication,)
    permission_classes = (DjangoModelPermissions,)
    queryset = Professional.objects.none()
    serializer_class = ProfessionalStepSkillCostEditSerializer

    @transaction.atomic
    def post(self, request, format=None):
        serializer = self.serializer_class(data=request.data)
        if serializer.is_valid():
            responsedata = []
            response = {}
            professional_skill = ProfessionalStepSkill.objects.filter(id=serializer.validated_data['id'])
            if professional_skill.exists():
                skill_serializer = self.serializer_class(data=request.data, instance=professional_skill.last())
                if skill_serializer.is_valid():
                    skill_serializer.save()
                    response["result"] = 1
                    response["message"] = ["Successfully Edited"]
                    self.add_api_call(request, response)
                    return Response(response, status=status.HTTP_200_OK)
                else:
                    response = handle_errors(serializer.errors)
                    response["result"] = 0
                    self.add_api_call(request, response)
                    return Response(response, status=status.HTTP_200_OK)
                
            else:
                response["result"] = 0
                response["errors"] = ["Incorrect ID"]
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
    

class AddProfessionalServiceCostAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = AddProfessionalServiceCostSerializer
    
    def post(self, request, format=None):
        """ 
        Add Service Skill Cost
        """ 
        response = {}
        responsedata = []
        # Check for profession user
        profession_obj = Professional.objects.filter(user=request.user, is_deleted=False)
        if profession_obj.exists():
            profession_obj = profession_obj.last()
        else:
            response["result"] = 0
            response["errors"] = ["Not authorized to perform this action"]
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        serializers = self.serializer_class(data=request.data, context={"professional_obj":profession_obj})
        if serializers.is_valid():
            serializers.save()
            response["result"] = 1
            response["message"] = ["Successfully Updated"]
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializers.errors)
            response['result'] = 0
            self.add_api_call(request, response)
            return Response(response,status=status.HTTP_200_OK)




class GetServiceSkillAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = GetProfessionalProfessionSerializer
    
    def get(self, request, format=None):
        """ 
        Get Service Skill 
        """ 
        response = {}
        responsedata = []
        # Get Professional Daata 
        professional_obj = Professional.objects.filter(user=request.user, is_deleted=False)
        if professional_obj.exists():
            professional_obj = professional_obj.last()
        else:
            response["result"] = 0
            response["errors"] = ["No Professional Data found"]
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        
        # Get All Skills
        profession_obj = Profession.objects.all()
        skills_class = self.serializer_class(profession_obj, many=True, context={"professional":professional_obj})

        response["result"] = 1
        response["skills_data"] = skills_class.data
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)





class GetProfessionalServiceAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = GetProfessionServiceSerializer
    
    def get(self, request, format=None):
        """ 
        Get Service Skill 
        """ 
        response = {}
        responsedata = []
        # Get Professional Daata 
        professional_obj = Professional.objects.filter(user=request.user, is_deleted=False)
        if professional_obj.exists():
            professional_obj = professional_obj.last()
        else:
            response["result"] = 0
            response["errors"] = ["No Professional Data found"]
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        
        # Get All Skills
        profession_obj = Profession.objects.all()
        skills_class = self.serializer_class(profession_obj, many=True, context={"professional":professional_obj})

        response["result"] = 1
        response["skills_data"] = skills_class.data
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)




class GetSkillsAvailabilityAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = SkillsAvailabilitySerializer
    
    @transaction.atomic
    def post(self, request, format=None):
        """ 
        Skills Availability
        """ 
        response = {}
        responsedata = []
        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            professional_obj = Professional.objects.filter(user=request.user, is_deleted=False)
            if professional_obj.exists():
                professional_obj = professional_obj.last()
            else:
                response["result"] = 0
                response["errors"] = ["Not Authorized to perform this action"]
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)
            
            # Get Professional Working hours
            professional_daily = ProfessionalDailyTiming.objects.filter(professional=professional_obj, is_deleted=False, is_active=True)

            # Professional Daily Timing 
            no_of_days = 7 if serializers.validated_data['method'] == "WEEK" else 30 if serializers.validated_data['method'] == "MONTH" else 1
            start_date = serializers.validated_data['start_date']
            end_date = start_date + datetime.timedelta(days=no_of_days)
            date = start_date
            all_dates = list()
            # ProfessionServiceAvailability.objects.filter(service=serializers.validated_data['service']).update(is_deleted=True)

            while date < end_date:
                day_of_date = calendar.day_name[date.weekday()]
                professional_daily = ProfessionalDailyTiming.objects.filter(professional=professional_obj, is_deleted=False, day__dayofweek = day_of_date)
                if professional_daily.exists():
                    professional_daily = professional_daily.last()
                else:
                    date += datetime.timedelta(days=1)
                    continue
                
                all_dates.append(date)
                day_of_week = {professional_daily.day.dayofweek: list()}
                slot_time = 60
                start_time = professional_daily.start_time
                end_time = professional_daily.end_time

                hours = []
                time = start_time
                end = end_time
                while time < end:
                    new_time = (datetime.datetime.combine(datetime.date(1,1,1),time) + datetime.timedelta(minutes=slot_time)).time()
                    time_slot = {"start_hour":time.strftime("%H:%M"), "end_hour":new_time}
                    professional_Service_availability = ProfessionServiceAvailability.objects.filter(
                        service = serializers.validated_data['service'],**time_slot,day=DayofWeek.objects.filter(dayofweek=day_of_date).last(),
                        date=date,professional=professional_obj)
                    if not professional_Service_availability.exists():
                        hours.append(time_slot)
                    time = new_time

                day_of_week[professional_daily.day.dayofweek] = hours


                professional_availability = [ProfessionServiceAvailability(
                    **i,
                    service = serializers.validated_data['service'],
                    day=DayofWeek.objects.filter(dayofweek=day_of_date).last(),
                    date=date,
                    professional=professional_obj
                ) for i in hours]

                ProfessionServiceAvailability.objects.bulk_create(professional_availability)
                date += datetime.timedelta(days=1)

            # Get Data from backend
            for single_date in all_dates:
                response_dict  = dict()
                professional_availability = ProfessionServiceAvailability.objects.filter(date=single_date, is_deleted=False, professional=professional_obj, service = serializers.validated_data['service'])
                response_dict = {"date":single_date, "day":professional_availability.last().day.dayofweek, "slots":list()}
                professional_class = ProfessionalServiceAvailabilitySerializer(professional_availability, many=True, context={"professional":professional_obj, "service": serializers.validated_data['service']})
                response_dict['slots'] = professional_class.data
                responsedata.append(response_dict)

            response["result"] = 1
            response["data"] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        else:
            response = handle_errors(serializers.errors)
            response['result'] = 0
            self.add_api_call(request, response)
            return Response(response,status=status.HTTP_200_OK)


class SaveSkillsAvailabilityAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = SaveSkillsAvailabilitySerializer
    
    def post(self, request, format=None):
        """ 
        Comments
        """ 
        response = {}
        responsedata = []
        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            serializers.save()
            response["result"] = 1
            response["message"] = ["Successfully Updated"]
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializers.errors)
            response['result'] = 0
            self.add_api_call(request, response)
            return Response(response,status=status.HTTP_200_OK)