import pytz
import json
from django.http import HttpResponse
from django.template import Template, Context
from django.template.loader import get_template
from django.db import transaction
from django.urls import reverse

from django.contrib.auth import login as django_login
import datetime
from rest_framework.authtoken.models import Token
from rest_framework import status, generics
from rest_framework.response import Response
from rest_framework.authentication import TokenAuthentication
from rest_framework.permissions import IsAuthenticated, DjangoModelPermissions
from utils.mixins import PlatformAPICallMixin
from utils.api_handle_errors import handle_errors

from professionals.serializers import *
from professionals.models import *
from reviews.serializers import ReviewGETSerializer
from reviews.models import Review


class SkillsSelectAPIView(PlatformAPICallMixin, generics.GenericAPIView):

    authentication_classes = (TokenAuthentication,)
    permission_classes = (DjangoModelPermissions,)
    queryset = Professional.objects.none()
    serializer_class = StepSkillsSerializer
    # GET Skills

    def get(self, request, format=None):
        '''Get Skills'''
        response = {}
        responsedata = []
        data = ProfessionalStepSkill.objects.filter(
            created_by=request.user, is_deleted=False)
        # serializer = StepSkillsReadSerializer(data, many=True)
        serializer = StepSkillsSerializer(data, many=True)

        response['result'] = 1
        response['data'] = serializer.data
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)

    def post(self, request, format=None):
        '''ADd Skills'''
        response = {}
        responsedata = []
        data = {}
        serializer = self.serializer_class(
            data=request.data, context={'user': request.user})
        if serializer.is_valid():
            if "profession_id":
                ProfessionalStepSkill.objects.filter(
                    skil__profession=serializer.validated_data['profession_id'], created_by=request.user).delete()
                serializer.validated_data.pop("profession_id")
            else:
                ProfessionalStepSkill.objects.filter(
                    created_by=request.user).delete()
            data = serializer.save()
            response['result'] = 1
            response['message'] = "Successfully Updated"
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class SkillDeleteAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    '''
    Delete skills
    '''
    authentication_classes = (TokenAuthentication,)
    permission_classes = (DjangoModelPermissions,)
    queryset = ProfessionalStepSkill.objects.none()
    serializer_class = StepSkillsDeleteSerializer

    def post(self, request, format=None):

        response = {}
        responsedata = []
        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            skill = ProfessionalStepSkill.objects.filter(
                id=serializers.data['id']).first()
            skill.is_deleted = True
            skill.save(update_fields=['is_deleted'])
            response['result'] = 1
            responsedata.append("Successfully Deleted")
            response['errors'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializers.errors)
            response['result'] = 1
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class CertificateAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    '''
    Add/Update Certificates
    '''
    authentication_classes = (TokenAuthentication,)
    permission_classes = (DjangoModelPermissions,)
    queryset = Professional.objects.none()
    serializer_class = CertificatesSerializer
    # GET Certificate

    def get(self, request, format=None):
        '''
        Get Certificates
        '''
        response = {}
        responsedata = []
        certificates = ProfessionalCertificate.objects.filter(
            created_by=request.user, is_deleted=False)
        serializer = self.serializer_class(certificates, many=True)
        response['result'] = 1
        response['errors'] = serializer.data
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)

    def post(self, request, format=None):
        '''
        Upload Certificate
        '''
        response = {}
        responsedata = []
        request.data._mutable = True
        serializer = self.serializer_class(
            data=request.data, context={"user": request.user})
        if serializer.is_valid():
            serializer.save()
            responsedata.append("Successfully Uploaded")
            response['result'] = 1
            response['errors'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response['result'] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class CertificateDeleteAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    '''
    delete Certificates
    '''
    authentication_classes = (TokenAuthentication,)
    permission_classes = (DjangoModelPermissions,)
    queryset = Professional.objects.none()
    serializer_class = DeleteCertificateSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []
        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            ProfessionalCertificate.objects.get(
                id=serializers.validated_data['id']).delete()
            response['result'] = 1
            responsedata.append("Successfully Deleted")
            response['errors'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializers.errors)
            response['result'] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class MySkillsAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    '''
    MySkills Page (GET SKILLS and CERTIFICATES)
    '''
    authentication_classes = (TokenAuthentication,)
    permission_classes = (DjangoModelPermissions,)
    queryset = Professional.objects.all()
    #skillsSerializer_class      = StepSkillsReadSerializer
    certificateSerializer_class = CertificatesSerializer
    serializer_class = MySkillsSerializer
    profession_serializer = getProfessionSerializer

    def get(self, request, format=None):
        '''
        GET SKILLS AND CERTIFICATES
        '''
        response = {}
        responsedata = []
        # ccertificate
        certificates = ProfessionalCertificate.objects.filter(
            created_by=request.user, is_deleted=False)
        serializer_certificate = self.certificateSerializer_class(
            certificates, many=True)
        # skills
        skills = ProfessionalStepSkill.objects.filter(
            created_by=request.user, is_deleted=False)
        # serializer_skills = self.skillsSerializer_class(skills, many=True)
        serializer_skills = self.serializer_class(skills, many=True)
        # profession
        profession = ProfessionalStepServiceData.objects.filter(
            user=request.user)
        serializer_profession = self.profession_serializer(
            profession, many=True)
        # data send
        responsedata.append(serializer_certificate.data)
        responsedata.append(serializer_skills.data)
        responsedata.append(serializer_profession.data)
        response["result"] = 1
        response['errors'] = responsedata
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)

    def post(self, request, format=None):
        '''
        POST SKILLS AND CERTIFICATES
        '''
        response = {}
        responsedata = []
        request.data._mutable = True
        request.data['created_by'] = str(request.user)
        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            serializers.save()
            response['result'] = 1
            responsedata.append("Successful")
            response['errors'] = responsedata
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class CertificateAddAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    '''
    adding Certificates
    '''
    authentication_classes = (TokenAuthentication,)
    permission_classes = (DjangoModelPermissions,)
    queryset = Professional.objects.none()
    serializer_class = DegreeCertificateSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []
        request.data._mutable = True
        serializer = self.serializer_class(
            data=request.data, context={"user": request.user})
        if serializer.is_valid():
            serializer.save()
            responsedata.append("Successfully Uploaded")
            response['result'] = 1
            response['errors'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response['result'] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)
