from django.db import models

from django.core.validators import MaxValueValidator, MinValueValidator
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.auth.models import User
from storages.backends.s3boto import S3BotoStorage
from venues.models import Venue
from professionals.models import Professional
from static_data.models import State, Skill


# Create your models here.
class S3CustomStorage(S3BotoStorage):
    def _normalize_name(self, name):
        """
        S3 Storage function for storing files
        """
        print(name)
        return name


class Review(models.Model):
    """
    Review Data
    """
    rating = models.IntegerField(null=True, blank=True)
    review = models.TextField(max_length=100, null=True, blank=True)

    is_deleted = models.BooleanField(
        default=False, help_text="activate/deactivate user")
    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)

    rated_to_type = models.ForeignKey(
        ContentType, on_delete=models.CASCADE, blank=True, null=True, related_name="Rated_To")
    rated_to_id = models.PositiveIntegerField(blank=True, null=True)
    rated_to_object = GenericForeignKey('rated_to_type', 'rated_to_id')

    rated_by_type = models.ForeignKey(
        ContentType, on_delete=models.CASCADE, blank=True, null=True, related_name="Rated_By")
    rated_by_id = models.PositiveIntegerField(blank=True, null=True)
    rated_by_object = GenericForeignKey('rated_by_type', 'rated_by_id')

    def __str__(self):
        return "{}-{}".format(self.rating, self.review)

    class Meta:
        verbose_name_plural = "Review"
        db_table = 'review'


class VenueReviewAndRating(models.Model):
    """ Venue Rating """

    venue = models.ForeignKey(
        Venue, on_delete=models.CASCADE, blank=True, null=True)

    rating = models.IntegerField(blank=True, null=True)
    review = models.TextField(blank=True, null=True)

    created_by = models.ForeignKey(
        User, on_delete=models.CASCADE, blank=True, null=True)

    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)
    updated_at = models.DateTimeField(auto_now=True, blank=True, null=True)
    is_deleted = models.BooleanField(
        default=False, help_text="activate/deactivate user")

    def __str__(self):
        return "{}-{}-{}".format(self.venue, self.rating, self.created_by)

    class Meta:
        verbose_name_plural = "Venue Reviews And Rating"
        db_table = 'venue_review_rating'

class ProfessionalReviewAndRating(models.Model):
    """ Professional Rating """

    professional = models.ForeignKey(
        Professional, on_delete=models.CASCADE, blank=True, null=True)

    rating = models.IntegerField(blank=True, null=True)
    review = models.TextField(blank=True, null=True)

    created_by = models.ForeignKey(
        User, on_delete=models.CASCADE, blank=True, null=True)

    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)
    updated_at = models.DateTimeField(auto_now=True, blank=True, null=True)
    is_deleted = models.BooleanField(
        default=False, help_text="activate/deactivate user")

    def __str__(self):
        return "{}-{}-{}".format(self.professional, self.rating, self.created_by)

    class Meta:
        verbose_name_plural = "Professional Reviews And Rating"
        db_table = 'professional_review_rating'
