from rest_framework import serializers
from .models import *
from venue_owners.models import VenueOwner
from venues.models import Venue
from advertisers.models import Advertiser
from players.models import Player
from professionals.models import Professional


class CustomReviewSerializer(serializers.Serializer):
    rated_to = serializers.CharField(
        help_text="ID of the profile(venueowner,player,professional)")
    rated_to_mode = serializers.CharField(
        help_text="Venue Owner, Professional, Player, Advertiser, Venue")

    def validate(self, obj):
        if obj['rated_to_mode'] == "Venue Owner":
            if VenueOwner.objects.filter(id=obj['rated_to']).exists():
                obj['rated_to'] = VenueOwner.objects.filter(id=obj['rated_to'])
            else:
                raise serializers.FieldDoesNotExist("Not a valid ID")

        elif obj['rated_to_mode'] == "Venue":
            if Venue.objects.filter(id=obj['rated_to']).exists():
                obj['rated_to'] = Venue.objects.filter(id=obj['rated_to'])
            else:
                raise serializers.FieldDoesNotExist("Not a valid ID")

        elif obj['rated_to_mode'] == "Professional":
            if Professional.objects.filter(id=obj['rated_to']).exists():
                obj['rated_to'] = Professional.objects.filter(
                    id=obj['rated_to'])
            else:
                raise serializers.FieldDoesNotExist("Not a valid ID")

        elif obj['rated_to_mode'] == "Player":
            if Player.objects.filter(id=obj['rated_to']).exists():
                obj['rated_to'] = Player.objects.filter(id=obj['rated_to'])
            else:
                raise serializers.FieldDoesNotExist("Not a valid ID")

        elif obj['rated_to_mode'] == "Advertiser":
            if Advertiser.objects.filter(id=obj['rated_to']).exists():
                obj['rated_to'] = Advertiser.objects.filter(id=obj['rated_to'])
            else:
                raise serializers.FieldDoesNotExist("Not a valid ID")
        return obj


class ReviewGETSerializer(serializers.ModelSerializer):

    class Meta:
        model = Review
        fields = '__all__'


class RatedByObjectRelatedField(serializers.RelatedField):
    def to_representation(self, value):
        return [{"first_name": value.first_name, "last_name": str(value.image)}]


class ReviewGETPlayerSerializer(serializers.ModelSerializer):
    rated_by_object = RatedByObjectRelatedField(read_only=True)

    class Meta:
        model = Review
        fields = '__all__'


class ReviewAddSerializer(serializers.ModelSerializer):
    rated_to = serializers.CharField()
    rated_to_mode = serializers.CharField(
        help_text="Venue Owner, Professional, Player, Advertiser")

    rated_by = serializers.CharField(
        required=False, help_text="ID of the reviewer(venueowner,player,professional)")
    rated_by_mode = serializers.CharField(
        help_text="Venue Owner, Professional, Player, Advertiser")

    def validate(self, obj):
        if obj['rated_to_mode'] == "Venue Owner":
            if VenueOwner.objects.filter(id=obj['rated_to']).exists():
                obj['rated_to'] = VenueOwner.objects.get(id=obj['rated_to'])
            else:
                raise serializers.FieldDoesNotExist("Not a valid ID")

        elif obj['rated_to_mode'] == "Professional":
            if Professional.objects.filter(id=obj['rated_to']).exists():
                obj['rated_to'] = Professional.objects.get(id=obj['rated_to'])
            else:
                raise serializers.FieldDoesNotExist("Not a valid ID")

        elif obj['rated_to_mode'] == "Player":
            if Player.objects.filter(id=obj['rated_to']).exists():
                obj['rated_to'] = Player.objects.get(id=obj['rated_to'])
            else:
                raise serializers.FieldDoesNotExist("Not a valid ID")

        elif obj['rated_to_mode'] == "Advertiser":
            if Advertiser.objects.filter(id=obj['rated_to']).exists():
                obj['rated_to'] = Advertiser.objects.get(id=obj['rated_to'])
            else:
                raise serializers.FieldDoesNotExist("Not a valid ID")

        if obj['rated_by_mode'] == "Venue Owner":
            if VenueOwner.objects.filter(id=obj['rated_by']).exists():
                obj['rated_by'] = VenueOwner.objects.get(id=obj['rated_by'])
            else:
                raise serializers.FieldDoesNotExist("Not a valid ID")
        elif obj['rated_by_mode'] == "Professional":
            if Professional.objects.filter(id=obj['rated_by']).exists():
                obj['rated_by'] = Professional.objects.get(id=obj['rated_by'])
            else:
                raise serializers.FieldDoesNotExist("Not a valid ID")
        elif obj['rated_by_mode'] == "Player":
            if Player.objects.filter(id=obj['rated_by']).exists():
                obj['rated_by'] = Player.objects.get(id=obj['rated_by'])
            else:
                raise serializers.FieldDoesNotExist("Not a valid ID")
        elif obj['rated_by_mode'] == "Advertiser":
            if Advertiser.objects.filter(id=obj['rated_by']).exists():
                obj['rated_by'] = Advertiser.objects.get(id=obj['rated_by'])
            else:
                raise serializers.FieldDoesNotExist("Not a valid ID")
        return obj

    class Meta:
        model = Review
        fields = '__all__'
        read_only_fields = ['rated_by', 'is_deleted']


class ReviewDeleteSerializer(serializers.Serializer):
    id = serializers.IntegerField(
        required=True, help_text='ID of the review')


class ReviewAndRatingGetSerializer(serializers.ModelSerializer):
    class Meta:
        model = VenueReviewAndRating
        fields = '__all__'


class GetCustomReviewSerializer(serializers.Serializer):
    id = serializers.IntegerField(
        help_text="ID of the venue")

class ProfessionalReviewAndRatingGetSerializer(serializers.ModelSerializer):
    class Meta:
        model = ProfessionalReviewAndRating
        fields = '__all__'

class ProfessionalGetCustomReviewSerializer(serializers.Serializer):
    id = serializers.IntegerField(
        help_text="ID of the Professional")

class PostReviewAddSerializer(serializers.ModelSerializer):
    mode = serializers.CharField(
        help_text="Player,Professional")

    created_by = serializers.CharField(
        required=False, help_text="ID of the reviewer (player)")

    def validate(self, obj):
        if obj['mode'] == "Player":
            obj['created_by'] = self.context.get("user")
            if Player.objects.filter(user=obj['created_by']).exists():
                pass
            else:
                raise serializers.FieldDoesNotExist("Not a valid player ID")

        if obj['mode'] == "Professional":
            obj['created_by'] = self.context.get("user")
            if Professional.objects.filter(user=obj['created_by']).exists():
                pass
            else:
                raise serializers.FieldDoesNotExist("Not a valid Professional")


            
        return obj

    class Meta:
        model = VenueReviewAndRating
        fields = '__all__'
        read_only_fields = ['created_at', 'updated_at']

class ProfessionalPostReviewAddSerializer(serializers.ModelSerializer):
    mode = serializers.CharField(
        help_text="Player,Professional")

    created_by = serializers.CharField(
        required=False, help_text="ID of the reviewer (player)")

    def validate(self, obj):
        if obj['mode'] == "Player":
            obj['created_by'] = self.context.get("user")
            if Player.objects.filter(user=obj['created_by']).exists():
                pass
            else:
                raise serializers.FieldDoesNotExist("Not a valid player ID")

        if obj['mode'] == "Professional":
            obj['created_by'] = self.context.get("user")
            if Professional.objects.filter(user=obj['created_by']).exists():
                pass
            else:
                raise serializers.FieldDoesNotExist("Not a valid Professional")


            
        return obj

    class Meta:
        model = ProfessionalReviewAndRating
        fields = '__all__'
        read_only_fields = ['created_at', 'updated_at']