import pytz
import json
from django.http import HttpResponse
from django.template import Template, Context
from django.template.loader import get_template
from django.db import transaction
from django.urls import reverse
from django.db.models import Avg, Count

from django.contrib.auth import login as django_login
import datetime
from rest_framework.authtoken.models import Token
from rest_framework import status, generics
from rest_framework.response import Response
from rest_framework.authentication import TokenAuthentication
from rest_framework.permissions import IsAuthenticated
from utils.mixins import PlatformAPICallMixin
from utils.api_handle_errors import handle_errors
from django.contrib.contenttypes.models import ContentType

from reviews.serializers import *
from reviews.models import *

from venue_owners.models import VenueOwner
from venues.models import Venue
from advertisers.models import Advertiser
from players.models import Player
from professionals.models import Professional


class ReviewAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    '''
    Get Review
    '''
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = CustomReviewSerializer
    GETSerializer = ReviewGETSerializer

    # Get Professional
    def post(self, request, format=None):
        try:
            response = {}
            responsedata = []
            serializers = self.serializer_class(data=request.data)
            if serializers.is_valid():

                reviewTo_contentType = ContentType.objects.get_for_model(VenueOwner) if serializers.validated_data['rated_to_mode'] == "Venue Owner" else ContentType.objects.get_for_model(Player) if serializers.validated_data['rated_to_mode'] == "Player" else ContentType.objects.get_for_model(
                    Professional) if serializers.validated_data['rated_to_mode'] == "Professional" else ContentType.objects.get_for_model(Advertiser) if serializers.validated_data['rated_to_mode'] == "Advertiser" else ContentType.objects.get_for_model(Venue)
                data = Review.objects.filter(rated_to_type__pk=reviewTo_contentType.id,
                                             rated_to_id__in=serializers.validated_data['rated_to'], is_deleted=False)

                serializeData = self.GETSerializer(data, many=True)
                response['result'] = 1
                response['data'] = serializeData.data
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)
            else:
                response = handle_errors(serializers.errors)
                response['result'] = 0
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_400_BAD_REQUEST)
        except Exception as e:
            response["result"] = 0
            responsedata.append(str(e))
            response['errors'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class ReviewAddAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    '''
    Add Review
    '''
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = ReviewAddSerializer

    def post(self, request, format=None):
        try:
            responsedata = []
            response = {}
            request.data._mutable = True
            serializer = self.serializer_class(data=request.data)
            if serializer.is_valid():
                serializer.validated_data.pop('rated_to_mode')
                serializer.validated_data.pop('rated_by_mode')
                serializer.validated_data['rated_by_object'] = serializer.validated_data['rated_by']
                serializer.validated_data['rated_to_object'] = serializer.validated_data['rated_to']
                serializer.validated_data.pop('rated_to')
                serializer.validated_data.pop('rated_by')

                print(serializer.validated_data)
                serializer.save()
                response['result'] = 1
                responsedata.append("Successfully Inserted")
                response['errors'] = responsedata
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)
            else:
                response = handle_errors(serializers.errors)
                response['result'] = 1
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_400_BAD_REQUEST)
        except Exception as e:
            response["result"] = 0
            responsedata.append(str(e))
            response['errors'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class ReviewDeleteAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    '''
    Delete Review
    '''
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = ReviewDeleteSerializer

    def post(self, request, format=None):
        try:
            response = {}
            responsedata = []
            serializers = self.serializer_class(data=request.data)
            if serializers.is_valid():
                if Review.objects.filter(id=serializers.data['id'], is_deleted=False).exists():
                    data = Review.objects.filter(
                        id=serializers.data['id'], is_deleted=False).first()
                    data.is_deleted = True
                    data.save(update_fields=['is_deleted'])
                    response['result'] = 1
                    responsedata.append("successfully deleted")
                    response['errors'] = responsedata
                    self.add_api_call(request, response)
                    return Response(response, status=status.HTTP_200_OK)
                else:
                    response['result'] = 0
                    responsedata.append("No Record Found")
                    response['errors'] = responsedata
                    self.add_api_call(request, response)
                    return Response(response, status=status.HTTP_400_BAD_REQUEST)
            else:
                response = handle_errors(serializers.serializer_errors)
                response['result'] = 0
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_400_BAD_REQUEST)
        except Exception as e:
            response["result"] = 0
            responsedata.append(str(e))
            response['errors'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class GetReviewAndRatingAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    """
    Get Review
    """
    #permission_classes = (IsAuthenticated,)
    serializer_class = GetCustomReviewSerializer
    GETSerializer = ReviewAndRatingGetSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []
        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            data = VenueReviewAndRating.objects.filter(
                venue=serializers.validated_data['id'])

            serializer = self.GETSerializer(data, many=True)
            response['result'] = 1
            response['data'] = serializer.data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response['result'] = 0
            responsedata.append("No Record Found")
            response['errors'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class PostReviewAndRatingAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    """
    Add Review
    """
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = PostReviewAddSerializer

    def post(self, request, format=None):

        response = {}
        responsedata = []
        request.data._mutable = True
        serializers = self.serializer_class(
            data=request.data, context={'user': request.user})

        if serializers.is_valid():
            serializers.validated_data.pop('mode')
            serializers.validated_data['created_by'] = serializers.validated_data['created_by']
            serializers.validated_data['venue'] = serializers.validated_data['venue']

            data = serializers.save()
            responsedata.append("Successfully Inserted")
            response["result"] = 1
            response['data'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class ProfessionalGetReviewAndRatingAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    """
    Get Review
    """ 
    permission_classes = (IsAuthenticated,)
    serializer_class = ProfessionalReviewAndRatingGetSerializer


    def post(self, request, format=None):
        response = {}
        responsedata = []
        
        
        data = ProfessionalReviewAndRating.objects.filter(
                professional=Professional.objects.get(email=request.user.email).id)

        serializer = self.serializer_class(data, many=True)
        
        response['result'] = 1
        response['data'] = serializer.data
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)
      

class ProfessionalPostReviewAndRatingAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    """
    Add Review
    """
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = ProfessionalPostReviewAddSerializer

    def post(self, request, format=None):
        
        response = {}
        responsedata = []
        request.data._mutable = True
        serializers = self.serializer_class(data=request.data,context={'user': request.user})

        if serializers.is_valid():
            serializers.validated_data.pop('mode')
            serializers.validated_data['created_by'] = serializers.validated_data['created_by']
            serializers.validated_data['professional'] = Professional.objects.get(email=request.user.email)
            data = serializers.save()
            responsedata.append("Successfully Inserted")
            response["result"] = 1
            response['data'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)