import pytz
import json
from django.http import HttpResponse
from django.template import Template, Context
from django.template.loader import get_template
from django.db import transaction
from django.urls import reverse
from django.contrib.contenttypes.models import ContentType

from django.contrib.auth import login as django_login
import datetime
from rest_framework.authtoken.models import Token
from rest_framework import status, generics
from rest_framework.response import Response
from rest_framework.authentication import TokenAuthentication
from rest_framework.permissions import IsAuthenticated
import uuid
from django.db.models import Q
from django.contrib.auth.models import User
from salesman.serializers import *
from salesman.models import *
from utils.mixins import *
from utils.api_handle_errors import handle_errors
from venue_owners.serializers import *
from venues.serializers import *
from venues.models import *
from commonAPI.models import ContractTerms
from venue_owners.models import *
from static_data.models import *
from static_data.serializers import *


class SalesmanPostAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = SalesmanPostSerializer

    def post(self, request, format=None):
        '''
        Add and Update Salesman
        '''
        responsedata = []
        response = {}

        if 'id' in request.data:
            if Salesman.objects.filter(id=request.data['id'],  is_deleted=False).exists():
                serializer = self.serializer_class(instance=Salesman.objects.filter(
                    id=request.data['id'], is_deleted=False).first(), data=request.data)
            message = "Updated Successfully"
        else:
            serializer = self.serializer_class(data=request.data)
            message = "Created Successfully"

        if serializer.is_valid():
            serializer.save()
            response["result"] = 1
            response['data'] = [message]
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class SalesmanGetAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    '''
    Salesman Get
    '''
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = SalesmanGetSerializer

    def post(self, request, format=None):
        response = {}
        serializer = self.serializer_class(data=request.data)
        if serializer.is_valid():
            sales_obj = Salesman.objects.filter(
                id=serializer.validated_data['id'], is_deleted=False)
            serializer_data = SalesmanSerializer(sales_obj, many=True)
            response['result'] = 1
            response['data'] = serializer_data.data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response['result'] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class SalesmanUsercodeAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    """
    User_Code
    """
    serializer_class = SalesmanSerializer

    def get(self, request, format=None):
        response = {}
        responsedata = {}

        # Convert UUID format to a Python string.
        random = str(uuid.uuid4())

        # Make all characters uppercase.
        random = random.upper()

        # Remove the UUID '-'.
        random = random.replace("-", "")

        # Return the random string.
        user_code = random[0:6]
        responsedata['user_code'] = user_code

        response['result'] = 1
        response['data'] = responsedata
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


# class SalesmanGetAndFilterAPIView(PlatformAPICallMixin, generics.GenericAPIView):
#     authentication_classes = (TokenAuthentication,)
#     permission_classes = (IsAuthenticated,)
#     serializer_class = SalesmanGetandFilterSerializer

#     def post(self, request, format=None):
#         '''
#         Get and Filter Salsman
#         '''
#         response = {}
#         responsedata = []

#         serializer = self.serializer_class(data=request.data)
#         serializer_ln = self.serializer_class(data=request.data)
#         serializer_vo = self.serializer_class(data=request.data)
#         serializer_pf = self.serializer_class(data=request.data)
#         serializer_ad = self.serializer_class(data=request.data)
#         if serializer.is_valid() and serializer_ln.is_valid() and serializer_vo.is_valid() and serializer_pf.is_valid() and serializer_ad.is_valid():

#             pagination = serializer.validated_data['page']
#             serializer.validated_data.pop("page")
#             serializer_ad.validated_data.pop("page")
#             serializer_pf.validated_data.pop("page")
#             serializer_vo.validated_data.pop("page")
#             serializer_ln.validated_data.pop("page")

#             if "vats" in serializer.validated_data:
#                 serializer.validated_data['vat__icontains'] = serializer.validated_data["vats"]
#                 serializer_ad.validated_data['vat__icontains'] = serializer.validated_data["vats"]
#                 serializer_pf.validated_data['vat__icontains'] = serializer.validated_data["vats"]
#                 serializer_vo.validated_data['vat__icontains'] = serializer.validated_data["vats"]
#                 serializer_ln.validated_data['vat__icontains'] = serializer.validated_data["vats"]
#                 serializer.validated_data.pop("vats")
#                 serializer_ad.validated_data.pop("vats")
#                 serializer_pf.validated_data.pop("vats")
#                 serializer_vo.validated_data.pop("vats")
#                 serializer_ln.validated_data.pop("vats")

#             abc = True
#             if "keyword" in serializer.validated_data:
#                 serializer.validated_data['first_name__icontains'] = serializer.validated_data["keyword"]
#                 serializer_ln.validated_data['last_name__icontains'] = serializer.validated_data["keyword"]
#                 try:
#                     serializer_vo.validated_data['venue_owner_number'] = int(
#                         serializer.validated_data["keyword"])
#                     serializer_pf.validated_data['professional_number'] = int(
#                         serializer.validated_data["keyword"])
#                     serializer_ad.validated_data['advertiser_number'] = int(
#                         serializer.validated_data["keyword"])
#                 except Exception as e:
#                     pass

#                 serializer.validated_data.pop("keyword")
#                 serializer_ln.validated_data.pop("keyword")
#                 serializer_vo.validated_data.pop("keyword")
#                 serializer_pf.validated_data.pop("keyword")
#                 serializer_ad.validated_data.pop("keyword")
#                 abc = False

#             start_data = (pagination - 1) * 10
#             end_data = start_data + 10

#             if abc:
#                 salesman_data = Salesman.objects.filter(
#                     **serializer.validated_data)
#             else:
#                 salesman_data = Salesman.objects.filter(
#                     Q(**serializer.validated_data) | Q(**serializer_ln.validated_data) | Q(**serializer_ad.validated_data) | Q(**serializer_pf.validated_data) | Q(**serializer_vo.validated_data))

#             salesman_count = salesman_data.count()
#             salesman_data = salesman_data[start_data:end_data]

#             serializer = self.serializer_class(salesman_data, many=True)

#             response["result"] = 1
#             response['data'] = serializer.data
#             response['total_page'] = salesman_count
#             self.add_api_call(request, response)
#             return Response(response, status=status.HTTP_200_OK)
#         else:
#             response = handle_errors(serializer.errors)
#             response["result"] = 0
#             self.add_api_call(request, response)
#             return Response(response, status=status.HTTP_400_BAD_REQUEST)


class SalesmanAllGetAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    """
    All Salesman
    """
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = SalesmanSerializer

    def get(self, request, format=None):
        response = {}
        responsedata = []
        data = Salesman.objects.all()
        serializer = self.serializer_class(data, many=True)
        response['result'] = 1
        response['data'] = serializer.data
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


class SalesmanNewGetAndFilterAPIView(PlatformAPICallMixin, generics.GenericAPIView):

    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = SalesmanGetandFilterSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []
        serializers = self.serializer_class(
            data=request.data, context={"user": request.user})

        if serializers.is_valid():
            pagination = serializers.validated_data['page']
            serializers.validated_data.pop("page")

            vat = ""
            if "vats" in serializers.validated_data:
                vat = serializers.validated_data["vats"]
                serializers.validated_data.pop("vats")

            k = ""
            if "keyword" in serializers.validated_data:
                k = serializers.validated_data["keyword"]
                serializers.validated_data.pop("keyword")

            if Salesman.objects.filter(**serializers.validated_data).exists():
                Salesman_obj = Salesman.objects.filter(
                    **serializers.validated_data).values()

                for i in Salesman_obj:
                    data = {}
                    data['id'] = i['id']
                    data['first_name'] = i['first_name']
                    data['last_name'] = i['last_name']
                    data['vat'] = i['vat']
                    data['mobile'] = i['mobile']
                    data['phone_number'] = i['phone_number']
                    data['id_card_number'] = i['id_card_number']
                    data['user_code'] = i['user_code']
                    data['street'] = i['street']
                    data['number'] = i['number']
                    data['country'] = Country.objects.get(
                        id=i['country_id']).name
                    data['state'] = State.objects.get(id=i['state_id']).name
                    data['city'] = City.objects.get(id=i['city_id']).name
                    data['area'] = Area.objects.get(id=i['area_id']).name
                    data['pincode'] = i['pincode']
                    data['is_deleted'] = i['is_deleted']
                    data['created_at'] = i['created_at']
                    data['updated_at'] = i['updated_at']

                    if ContractTerms.objects.filter(salesman=i['id']).exists():
                        contract_obj = ContractTerms.objects.filter(
                            salesman=i['id'])
                        a = 0
                        v = 0
                        p = 0
                        for j in contract_obj:
                            grp_obj = j.created_for.groups.all()[0]
                            if grp_obj.name == 'Venue Owner':
                                v = v + 1
                            elif grp_obj.name == 'Advertiser':
                                a = a + 1
                            elif grp_obj.name == 'Professional':
                                p = p + 1
                        data['venue_owner_number'] = v
                        data['advertiser_number'] = a
                        data['professional_number'] = p
                    else:
                        data['venue_owner_number'] = 0
                        data['advertiser_number'] = 0
                        data['professional_number'] = 0

                    if vat != "":
                        if vat in data['vat']:
                            if k != "":
                                try:
                                    kk = int(k)
                                    if data['venue_owner_number'] == kk or data['advertiser_number'] == kk or data['professional_number'] == kk or k.lower() in data['first_name'].lower() or k.lower() in data['last_name'].lower():
                                        responsedata.append(data)
                                except Exception as e:
                                    if k.lower() in data['first_name'].lower() or k.lower() in data['last_name'].lower():
                                        responsedata.append(data)
                            else:
                                responsedata.append(data)

                    else:
                        if k != "":
                            try:
                                kk = int(k)
                                if data['venue_owner_number'] == kk or data['advertiser_number'] == kk or data['professional_number'] == kk or k.lower() in data['first_name'].lower() or k.lower() in data['last_name'].lower():
                                    responsedata.append(data)
                            except Exception as e:
                                if k.lower() in data['first_name'].lower() or k.lower() in data['last_name'].lower():
                                    responsedata.append(data)
                        else:
                            responsedata.append(data)

                start_data = (pagination - 1) * 10
                end_data = start_data + 10
                salesman_count = len(responsedata)
                salesman_data = responsedata[start_data:end_data]

                response["result"] = 1
                response['data'] = salesman_data
                response['total_page'] = salesman_count
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)
            else:
                response['result'] = 0
                response['errors'] = ["No record available"]
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializers.errors)
            response['result'] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
