from django.db import models
from django.db import models


class UserType(models.Model):
    USER_TYPE = (
        ('VENUE OWNER', "Venue Owner"),
        ('PROFESSIONAL', 'Professional'),
        ('PLAYER', 'Player'),
        ('ADVERTISER', 'Advertiser'),
        ('VISITOR', 'Visitor'),
    )

    name = models.CharField(max_length=50, choices=USER_TYPE,
                            null=True, blank=True, help_text='user type choice')
    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)

    def __str__(self):
        return "{}".format(self.name)

    class Meta:
        verbose_name_plural = 'User Types'
        db_table = 'user_type'


class Country(models.Model):
    """
Static Countries
"""
    name = models.CharField(max_length=100, blank=True, null=True)
    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)

    def __str__(self):
        return "{}".format(self.name)

    class Meta:
        verbose_name_plural = 'Countries'
        db_table = 'country'


class State(models.Model):
    """
States
"""
    name = models.CharField(max_length=100, blank=True, null=True)
    country = models.ForeignKey(
        Country, on_delete=models.CASCADE, blank=True, null=True)
    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)

    def __str__(self):
        return "{}-{}".format(self.name, self.country)

    class Meta:
        verbose_name_plural = 'State'
        db_table = 'state'


class City(models.Model):
    """
Static City
"""
    name = models.CharField(max_length=100, blank=True, null=True)
    state = models.ForeignKey(
        State, on_delete=models.CASCADE, blank=True, null=True)
    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)

    def __str__(self):
        return "{}-{}".format(self.name, self.state)

    class Meta:
        verbose_name_plural = "City"
        db_table = "city"


class Area(models.Model):
    """
Static Area
"""
    name = models.CharField(max_length=800, blank=True, null=True)
    city = models.ForeignKey(
        City, on_delete=models.CASCADE, blank=True, null=True)
    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)

    def __str__(self):
        return "{}-{}".format(self.name, self.city)

    class Meta:
        verbose_name_plural = 'Areas'
        db_table = 'area'


class DayofWeek(models.Model):
    # SUNDAY, MONDAY, TUESDAY, WEDNESDAY, THURSDAY, FRIDAY, SATURDAY = range(7)
    weekday_choices = (
        ('SUNDAY', "Sunday"),
        ('MONDAY', 'Monday'),
        ('TUESDAY', 'Tuesday'),
        ('WEDNESDAY', 'Wednesday'),
        ('THURSDAY', 'Thursday'),
        ('FRIDAY', 'Friday'),
        ('SATURDAY', 'Saturday'),
    )
    dayofweek = models.CharField(
        max_length=50, choices=weekday_choices, blank=True, null=True)

    def __str__(self):
        return "{}".format(self.dayofweek)

    class Meta:
        verbose_name_plural = 'Day of Week'
        db_table = 'dayofweek'


class Sport(models.Model):
    """
Static Sports
"""
    STATUS_CHOICES = (
        ('T', 'Team'),
        ('I', 'Individual'),
    )
    CATAGORY_CHOICES = (
        ('Sports', 'Sports'),
        ('Winter Sports', 'Winter Sports'),
        ('Summer Sports', 'Summer Sports'),
        ('All Seasons Sports', 'All Seasons Sports'),
    )
    name = models.CharField(max_length=100, blank=True, null=True)
    play_type = models.CharField(
        max_length=15, default=STATUS_CHOICES[1][0], choices=STATUS_CHOICES, blank=True, null=True,
        help_text="Type of Play - Team/Individual")
    catagory = models.CharField(
        max_length=20,  choices=CATAGORY_CHOICES, blank=True, null=True,
        help_text="Sports=Sports,Winter Sports=Winter Sports,Summer Sports,All Seasons Sports")
    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)
    show_on_homepage = models.BooleanField(
        default=False, help_text="True=show on homepage/False=don't show")
    parent_sport = models.ForeignKey(
        'self', on_delete=models.CASCADE, blank=True, null=True)

    def __str__(self):
        return "{}-{}".format(self.name, self.play_type)

    class Meta:
        verbose_name_plural = 'Sports'
        db_table = 'sport'


class Position(models.Model):
    """
Static Positions of players
"""

    sport = models.ForeignKey(
        Sport, on_delete=models.CASCADE, blank=True, null=True)
    position = models.CharField(max_length=100, blank=True, null=True)
    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)

    def __str__(self):
        return "{}".format(self.position)

    class Meta:
        verbose_name_plural = 'Sports_Position'
        db_table = 'sport_position'


class FieldType(models.Model):
    """
Static Field Types
"""
    name = models.CharField(max_length=100, blank=True, null=True)
    sport = models.ManyToManyField(
        Sport, blank=True)
    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)

    def __str__(self):
        return "{}".format(self.name)

    class Meta:
        verbose_name_plural = 'Field Types'
        db_table = 'field_type'


class FloorType(models.Model):
    """
Static Floor Types
"""
    name = models.CharField(max_length=100, blank=True, null=True)
    sport = models.ManyToManyField(
        Sport, blank=True)
    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)

    def __str__(self):
        return "{}".format(self.name)

    class Meta:
        verbose_name_plural = 'Floor Type'
        db_table = 'floor_type'


class FacilityType(models.Model):
    """
Static Facility Types
"""
    name = models.CharField(max_length=100, blank=True, null=True)
    sport = models.ManyToManyField(
        Sport, blank=True)
    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)

    def __str__(self):
        return "{}".format(self.name)

    class Meta:
        verbose_name_plural = 'Facility Type'
        db_table = 'facility_type'


class Lights(models.Model):
    """
Static Lights
"""
    name = models.CharField(max_length=100, blank=True, null=True)
    sport = models.ForeignKey(
        Sport, on_delete=models.CASCADE, blank=True, null=True)
    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)

    def __str__(self):
        return "{}".format(self.name)

    class Meta:
        verbose_name_plural = 'Lights'
        db_table = 'lights'


class FieldAttribute(models.Model):
    """
Static Field Attributes
"""
    name = models.CharField(max_length=100, blank=True, null=True)
    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)

    def __str__(self):
        return "{}".format(self.name)

    class Meta:
        verbose_name_plural = 'Field Attributes'
        db_table = 'field_attribute'


class VenueService(models.Model):
    """
Static Venue Service
"""
    TYPE_CHOICES = (
        ('P', 'Paid'),
        ('F', 'Free')
    )
    SERVICE_AVAILABLE_CHOICES = (
        ('V', 'Venue'),
        ('F', 'Field'),
        ('B', 'Both')
    )
    name = models.CharField(max_length=100, blank=True, null=True)
    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)
    logo = models.ImageField(upload_to="service_icons", blank=True, null=True)
    type = models.CharField(
        max_length=10, default=TYPE_CHOICES[1][0], choices=TYPE_CHOICES, blank=True, null=True)
    service_available = models.CharField(
        max_length=10, default=SERVICE_AVAILABLE_CHOICES[0][0], choices=SERVICE_AVAILABLE_CHOICES, blank=True,
        null=True)

    def __str__(self):
        return "{}-{}-{}-{}".format(self.id, self.name, self.type, self.service_available)

    class Meta:
        verbose_name_plural = 'Venue Services'
        db_table = 'venue_service'


class Profession(models.Model):
    """
Static Professions
"""
    professionName = models.CharField(max_length=200)
    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)

    def __str__(self):
        return "{}".format(self.professionName)

    class Meta:
        verbose_name_plural = 'Static Profession'
        db_table = 'static_profession'


class Skill(models.Model):
    """
Static Skills
"""
    skill_name = models.CharField(max_length=100, blank=True)
    profession = models.ForeignKey(
        Profession, on_delete=models.CASCADE, blank=True, null=True)
    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)

    def __str__(self):
        return "{}".format(self.skill_name)

    class Meta:
        verbose_name_plural = 'Step Skills'
        db_table = 'step_skills'


class StaticProfessionalService(models.Model):
    """
    Static Professional Services 
    """
    profession = models.ForeignKey(
        Profession, on_delete=models.CASCADE, blank=True, null=True)
    service = models.CharField(max_length=100, blank=True, null=True)
    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)

    def __str__(self):
        return "{}".format(self.service)

    class Meta:
        verbose_name_plural = 'Professional Services'
        db_table = 'static_professional_services'


class VenuePermission(models.Model):
    """
Static Venue Permissions
"""
    name = models.CharField(max_length=100, blank=True, null=True)
    logo = models.ImageField(
        upload_to="venue_permission", blank=True, null=True)

    def __str__(self):
        return "{}".format(self.name)

    class Meta:
        verbose_name_plural = 'Venue Entry'
        db_table = 'venue_permission'


class NotificationIcon(models.Model):
    """
    Notification Icons
    """
    name = models.CharField(max_length=100, blank=True, null=True)
    icons = models.ImageField(
        upload_to="notification_icons", blank=True, null=True)

    def __str__(self):
        return "{}".format(self.name)

    class Meta:
        verbose_name_plural = 'Notification Icons'
        db_table = 'notification_icon'


class AdminIcons(models.Model):
    """
    Temporary Admin Icons
    """
    icon = models.ImageField(upload_to="admin_icons", blank=True, null=True)

    class Meta:
        verbose_name_plural = 'Admin Icons'
        db_table = 'admin_icon'


class BankName(models.Model):
    """
    Bank Name
    """
    name = models.CharField(max_length=100, blank=True, null=True)
    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)

    class Meta:
        verbose_name_plural = 'Bank Name'
        db_table = 'bank_name'


class TelephoneCountryCode(models.Model):
    """
Country Codes
"""
    country_name = models.CharField(max_length=100, blank=True, null=True)
    country_code = models.CharField(max_length=100, blank=True, null=True)

    def __str__(self):
        return "{}-{}-{}".format(self.id, self.country_name, self.country_code)

    class Meta:
        verbose_name_plural = 'Country Codes'
        db_table = 'country_code'


class Region(models.Model):
    """
Static Region
"""
    name = models.CharField(max_length=100, blank=True, null=True)
    country = models.ForeignKey(
        Country, on_delete=models.CASCADE, blank=True, null=True)
    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)

    def __str__(self):
        return "{}-{}".format(self.name, self.country)

    class Meta:
        verbose_name_plural = "Region"
        db_table = "region"


class TeamLevel(models.Model):
    """
    Static Team Level
    """
    name = models.CharField(max_length=100, blank=True, null=True)

    def __str__(self):
        return "{}-{}".format(self.id, self.name)

    class Meta:
        verbose_name_plural = "Team Level"
        db_table = "team_level"


class PlayerLevel(models.Model):
    """
    Static player Level
    """
    name = models.CharField(max_length=100, blank=True, null=True)

    def __str__(self):
        return "{}-{}".format(self.id, self.name)

    class Meta:
        verbose_name_plural = "Player Level"
        db_table = "player_level"


class PaymentMethod(models.Model):
    """
Payment Methods
"""
    name = models.CharField(max_length=100, blank=True, null=True)
    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)

    def __str__(self):
        return "{}".format(self.name)

    class Meta:
        verbose_name_plural = 'Payment Method'
        db_table = 'payment_method'


class BookingCancellation(models.Model):
    """
Booking Cancellation
    """
    cancellation_reason = models.CharField(
        max_length=600, blank=True, null=True)
    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)

    def __str__(self):
        return "{}".format(self.cancellation_reason)

    class Meta:
        verbose_name_plural = 'Booking Cancellation'
        db_table = 'Booking_Cancellation'


class CancellationPolicy(models.Model):
    """
    Cancellation Policy
    """
    charge_rate = models.CharField(max_length=100, blank=True, null=True)
    start_time = models.IntegerField(blank=True, null=True)
    end_time = models.IntegerField(blank=True, null=True)

    def __str__(self):
        return "{}-{}-{}".format(self.charge_rate, self.start_time, self.end_time)

    class Meta:
        verbose_name_plural = 'Cancellation Policy'
        db_table = 'Cancellation_Policy'


class CollaborationTypes(models.Model):
    """
    Types of Collaboration
    """
    collaboration_name = models.CharField(
        max_length=500, blank=True, null=True)

    def __str__(self):
        return "{}".format(self.collaboration_name)

    class Meta:
        verbose_name_plural = 'Collaboration Types'
        db_table = 'Collaboration_Types'


class Covid19Measures(models.Model):
    """
    Types of Covid19Measures
    """
    description = models.CharField(max_length=500, blank=True, null=True)

    def __str__(self):
        return "{}".format(self.description)

    class Meta:
        verbose_name_plural = 'Covid19Measures'
        db_table = 'Covid19Measures'


class Template_Images(models.Model):
    """
Static Templates Images
"""
    template_image = models.FileField(
        upload_to='template_images/', null=True, blank=True, help_text='template images')

    def __str__(self):
        return "{}".format(self.template_image)

    class Meta:
        verbose_name_plural = "Template Image"
        db_table = "template_images"


class MeasurementUnits(models.Model):
    """
        model for measurement unit
    """
    measurement_code = models.CharField(
        max_length=50, blank=True, null=True, unique=True)
    measurement_description = models.CharField(
        max_length=500, blank=True, null=True)

    def __str__(self):
        return "{}-{}".format(self.measurement_code, self.measurement_description)

    class Meta:
        verbose_name_plural = "Measurement Unit"
        db_table = "measurement_units"


class VatExemptionOrder(models.Model):
    """
        model for vat exemption order
    """
    vat_exemption_order_code = models.CharField(
        max_length=50, blank=True, null=True, unique=True)
    vat_exemption_order_description = models.CharField(
        max_length=500, blank=True, null=True)

    def __str__(self):
        return "{}-{}".format(self.vat_exemption_order_code, self.vat_exemption_order_description)

    class Meta:
        verbose_name_plural = "vat exemption order"
        db_table = "vat_exemption_order"


class Vat(models.Model):
    """
        model for vat
    """
    vat_code = models.CharField(
        max_length=50, blank=True, null=True, unique=True)
    vat_description = models.CharField(max_length=500, blank=True, null=True)
    vat_standard = models.CharField(max_length=500, blank=True, null=True)
    vat_reduced = models.CharField(max_length=500, blank=True, null=True)
    vat_exemption = models.CharField(max_length=500, blank=True, null=True)

    def __str__(self):
        return "{}-{}".format(self.vat_code, self.vat_description, self.vat_standard,
                              self.vat_reduced, self.vat_exemption)

    class Meta:
        verbose_name_plural = "vat"
        db_table = "vat"


class BookingPaymentMethod(models.Model):
    """
        model for booking payment method
    """
    payment_method_code = models.CharField(
        max_length=50, blank=True, null=True, unique=True)
    payment_method_description = models.CharField(
        max_length=500, blank=True, null=True)

    def __str__(self):
        return "{}-{}".format(self.payment_method_code, self.payment_method_description)

    class Meta:
        verbose_name_plural = "booking payment method"
        db_table = "booking_payment_method"


class MIXED_GENDER(models.Model):
    """
Static MIXED GENDERS
"""
    name = models.CharField(max_length=100, blank=True, null=True)
    sport = models.ForeignKey(
        Sport, on_delete=models.CASCADE, blank=True, null=True)
    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)

    def __str__(self):
        return "{}-{}".format(self.name, self.sport)

    class Meta:
        verbose_name_plural = 'Mixed Genders'
        db_table = 'mixed_gender'


class AVAILABILITY_IN_USE(models.Model):
    """
Static AVAILABILITY_IN_USE
"""
    name = models.CharField(max_length=100, blank=True, null=True)
    sport = models.ForeignKey(
        Sport, on_delete=models.CASCADE, blank=True, null=True)
    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)

    def __str__(self):
        return "{}-{}".format(self.name, self.sport)

    class Meta:
        verbose_name_plural = 'Availability In Use'
        db_table = 'availability_in_use'


class DocNumberingSeries(models.Model):
    """
    Document Numbering Series
    """
    code = models.CharField(max_length=5, blank=True,
                            null=True, help_text="Code")
    description = models.CharField(
        max_length=100, blank=True, null=True, help_text="Description")
    is_deleted = models.BooleanField(
        default=False, blank=True, null=True, help_text="If the record is deleted")
    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)

    def __str__(self):
        return "{}-{}".format(self.pk, self.code)

    class Meta:
        verbose_name_plural = 'Doc Numbering Series'
        db_table = 'doc_numbering_series'


class DocumentType(models.Model):
    """
    Document Type
    """
    code = models.CharField(max_length=5, blank=True,
                            null=True, help_text="Code")
    description = models.CharField(
        max_length=100, blank=True, null=True, help_text="Description")
    # doc_numbering_series = models.ForeignKey(DocNumberingSeries,on_delete=models.CASCADE, blank=True, null=True, help_text="Numbering Series")
    is_deleted = models.BooleanField(
        default=False, blank=True, null=True, help_text="If the record is deleted")
    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)

    class Meta:
        verbose_name_plural = "Document Types"
        db_table = "document_type"


class DocumentTypeDocNumberingSeries(models.Model):
    """
    Many to many Document Numbering Series for Document Type
    """
    document_type = models.ForeignKey(
        DocumentType, on_delete=models.CASCADE, help_text="Document Type")
    document_numbering_series = models.ForeignKey(
        DocNumberingSeries, on_delete=models.CASCADE, help_text="Document Numbering System")

    class Meta:
        verbose_name_plural = "Document Type Numbering Series"
        db_table = "document_type_numbering_series"


class TransferPurpose(models.Model):
    """
    Transfer Purpose
    """
    code = models.CharField(max_length=5, blank=True,
                            null=True, help_text="Code")
    description = models.CharField(
        max_length=100, blank=True, null=True, help_text="Description")
    is_deleted = models.BooleanField(
        default=False, blank=True, null=True, help_text="If the record is deleted")
    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)

    def __str__(self):
        return "{}-{}".format(self.pk, self.code)

    class Meta:
        verbose_name_plural = 'Transfer Purposes'
        db_table = 'transfer_purpose'


class TransferMethod(models.Model):
    """
    Transfer Method
    """
    code = models.CharField(max_length=5, blank=True,
                            null=True, help_text="Code")
    description = models.CharField(
        max_length=100, blank=True, null=True, help_text="Description")
    is_deleted = models.BooleanField(
        default=False, blank=True, null=True, help_text="If the record is deleted")
    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)

    def __str__(self):
        return "{}-{}".format(self.pk, self.code)

    class Meta:
        verbose_name_plural = 'Transfer Method'
        db_table = 'transfer_method'


class MeansOfTransport(models.Model):
    """
    Means Of Transport
    """
    code = models.CharField(max_length=5, blank=True,
                            null=True, help_text="Code")
    description = models.CharField(
        max_length=100, blank=True, null=True, help_text="Description")
    is_deleted = models.BooleanField(
        default=False, blank=True, null=True, help_text="If the record is deleted")
    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)

    def __str__(self):
        return "{}-{}".format(self.pk, self.code)

    class Meta:
        verbose_name_plural = 'Means Of Transports'
        db_table = 'means_of_transport'
