from rest_framework import serializers
from django.contrib.auth.models import User, Group
from rest_framework.authtoken.models import Token

from datetime import date
import re
from .models import *
from static_data.functionalities.invoice import save_edit_invoices, validate_code, validate_id
from static_data.models import StaticProfessionalService
from professionals.models import ProfessionalStepSkill, ProfessionalService
from venues.models import *


class VenueServiceSerializer(serializers.ModelSerializer):
    class Meta:
        model = VenueService
        fields = '__all__'


class VenuePermissionSerializer(serializers.ModelSerializer):
    class Meta:
        model = VenuePermission
        fields = '__all__'


class SportSerializer(serializers.ModelSerializer):
    class Meta:
        model = Sport
        fields = ['id', 'name', 'show_on_homepage']


class SportAdminSerializer(serializers.ModelSerializer):
    class Meta:
        model = Sport
        fields = '__all__'


class SportCategorySerializer(serializers.ModelSerializer):
    class Meta:
        model = Sport
        fields = ['name']


class ProfessionSerializer(serializers.ModelSerializer):
    class Meta:
        model = Profession
        fields = '__all__'


class FloorTypeSerializer(serializers.ModelSerializer):
    class Meta:
        model = FloorType
        fields = '__all__'
        # fields = ['id', 'name']


class FieldTypeSerializer(serializers.ModelSerializer):
    sport = SportSerializer(read_only=True, many=True)

    class Meta:
        model = FieldType
        fields = '__all__'
        # fields = ['id', 'name', 'sport']


class FieldAttributeSerializer(serializers.ModelSerializer):
    class Meta:
        model = FieldAttribute
        fields = '__all__'


class PositionSerializer(serializers.ModelSerializer):
    class Meta:
        model = Position
        fields = '__all__'


class DayofWeekSerializer(serializers.ModelSerializer):
    class Meta:
        model = DayofWeek
        fields = "__all__"


class UserTypeSerializer(serializers.ModelSerializer):
    class Meta:
        model = UserType
        fields = "__all__"


class CitySerializer(serializers.ModelSerializer):
    class Meta:
        model = City
        fields = "__all__"


class CountrySerializer(serializers.ModelSerializer):
    class Meta:
        model = Country
        fields = "__all__"


class BankNameSerializer(serializers.ModelSerializer):
    class Meta:
        model = BankName
        fields = "__all__"


class CountryCodeSerializer(serializers.ModelSerializer):
    class Meta:
        model = TelephoneCountryCode
        fields = '__all__'


class GetProfessionalSkillsSerializer(serializers.ModelSerializer):
    is_selected = serializers.SerializerMethodField("_get_is_selected")
    professional_skill_id = serializers.SerializerMethodField(
        "_get_professional_skill_id")

    def _get_professional_skill_id(self, obj):
        professional_obj = self.context['professional']
        professional_skill_id = 0
        if ProfessionalStepSkill.objects.filter(skil=obj, created_by=professional_obj.user, is_deleted=False).exists():
            professional_skill_id = ProfessionalStepSkill.objects.filter(
                skil=obj, created_by=professional_obj.user, is_deleted=False).last()
            professional_skill_id = professional_skill_id.id
        return professional_skill_id

    def _get_is_selected(self, obj):
        professional_obj = self.context['professional']
        if ProfessionalStepSkill.objects.filter(skil=obj, created_by=professional_obj.user, is_deleted=False).exists():
            return True
        return False

    class Meta:
        model = Skill
        fields = "__all__"


class GetProfessionalProfessionSerializer(serializers.ModelSerializer):
    skills = serializers.SerializerMethodField("_get_skills")

    def _get_skills(self, obj):
        skill_obj = Skill.objects.filter(profession=obj)
        skills_class = GetProfessionalSkillsSerializer(
            skill_obj, many=True, context=self.context)
        return skills_class.data

    class Meta:
        model = Profession
        fields = '__all__'


class GetProfessionServicesSerializer(serializers.ModelSerializer):
    is_selected = serializers.SerializerMethodField("_get_is_selected")
    price = serializers.SerializerMethodField("_get_price")
    profession_service_id = serializers.SerializerMethodField(
        "_get_profession_service_id")

    def _get_is_selected(self, obj):
        professional_obj = self.context['professional']
        if ProfessionalService.objects.filter(service=obj, profession=professional_obj, is_deleted=False).exists():
            return True
        return False

    def _get_profession_service_id(self, obj):
        profession_service_id = 0
        professional_obj = self.context['professional']
        if ProfessionalService.objects.filter(service=obj, profession=professional_obj, is_deleted=False).exists():
            professional_service = ProfessionalService.objects.filter(
                service=obj, profession=professional_obj, is_deleted=False).last()
            profession_service_id = professional_service.id
        return profession_service_id

    def _get_price(self, obj):
        price = 0
        professional_obj = self.context['professional']
        if ProfessionalService.objects.filter(service=obj, profession=professional_obj, is_deleted=False).exists():
            professional_service = ProfessionalService.objects.filter(
                service=obj, profession=professional_obj, is_deleted=False).last()
            price = professional_service.cost if professional_service.cost != None else 0
        return price

    class Meta:
        model = StaticProfessionalService
        fields = "__all__"


class GetProfessionServiceSerializer(serializers.ModelSerializer):
    service = serializers.SerializerMethodField("_get_services")

    def _get_services(self, obj):
        service_obj = StaticProfessionalService.objects.filter(profession=obj)
        service_class = GetProfessionServicesSerializer(
            service_obj, many=True, context=self.context)
        return service_class.data

    class Meta:
        model = Profession
        fields = '__all__'


class CountryCodePOSTSerializer(serializers.ModelSerializer):
    country_name = serializers.CharField(required=True)
    country_code = serializers.CharField(required=True)

    class Meta:
        model = TelephoneCountryCode
        fields = '__all__'


class RegionSerializer(serializers.ModelSerializer):
    country = serializers.CharField(source="country.name", required=False)

    class Meta:
        model = Region
        fields = '__all__'


class TeamLevelSerializer(serializers.ModelSerializer):
    class Meta:
        model = TeamLevel
        fields = '__all__'


class SkillsSetSerializer(serializers.ModelSerializer):
    profession = ProfessionSerializer(
        required=False, read_only=True)

    class Meta:
        model = Skill
        fields = '__all__'


class RequestSkillsSerializer(serializers.ModelSerializer):

    def save(self):
        from admin_app.models import ProfessionalRequestSkill
        ProfessionalRequestSkill.objects.create(
            professional=self.context['professional'],
            profession=self.validated_data['profession'],
            skill_name=self.validated_data['skill_name'],

        )
        return True

    class Meta:
        model = Skill
        fields = '__all__'


class RequestServiceSerializer(serializers.ModelSerializer):

    def save(self):
        from admin_app.models import ProfessionalRequestService
        ProfessionalRequestService.objects.create(
            professional=self.context['professional'],
            profession=self.validated_data['profession'],
            service_name=self.validated_data['service'],

        )
        return True

    class Meta:
        model = StaticProfessionalService
        fields = '__all__'


class StateGetSerializer(serializers.ModelSerializer):
    country = serializers.CharField(
        required=False)

    class Meta:
        model = State
        fields = '__all__'
        read_only_fields = ['name']


class CityGetSerializer(serializers.ModelSerializer):
    state = serializers.CharField(
        required=False)

    class Meta:
        model = City
        fields = '__all__'
        read_only_fields = ['name']


class AreaGetSerializer(serializers.ModelSerializer):
    city = serializers.CharField(
        required=False)

    class Meta:
        model = Area
        fields = '__all__'
        read_only_fields = ['name']


class FacilitysSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(
        required=False)

    class Meta:
        model = FacilityType
        fields = '__all__'
        read_only_fields = ['name', 'sport']


class LightsSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(
        required=False)

    class Meta:
        model = Lights
        fields = '__all__'
        read_only_fields = ['name', 'sport']


class LightsGetSerializer(serializers.ModelSerializer):
    class Meta:
        model = Lights
        fields = '__all__'


class FacilitysGetSerializer(serializers.ModelSerializer):
    class Meta:
        model = FacilityType
        fields = '__all__'


class PaymentMethodSerializer(serializers.ModelSerializer):
    class Meta:
        model = PaymentMethod
        fields = '__all__'


class BookingCancellationWithPageSerializer(serializers.ModelSerializer):
    page = serializers.IntegerField(
        required=False, default=1, help_text="Page number")

    class Meta:
        model = BookingCancellation
        fields = '__all__'
        read_only_fields = ['cancellation_reason', 'created_at']


class BookingCancellationSerializer(serializers.ModelSerializer):
    class Meta:
        model = BookingCancellation
        fields = '__all__'


class Template_ImagesSerializer(serializers.ModelSerializer):
    class Meta:
        model = Template_Images
        fields = '__all__'


class MeasurementUnitsPOSTSerializer(serializers.ModelSerializer):
    class Meta:
        model = MeasurementUnits
        fields = '__all__'


class MeasurementUnitsGetbyIDSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(
        required=True)

    class Meta:
        model = MeasurementUnits
        fields = '__all__'
        read_only_fields = ['measurement_code', 'measurement_description']


class MeasurementUnitsWithPageSerializer(serializers.ModelSerializer):
    page = serializers.IntegerField(
        required=False, default=1, help_text="Page number")

    class Meta:
        model = MeasurementUnits
        fields = '__all__'
        read_only_fields = ['measurement_code', 'measurement_description']


class MeasurementUnitsWithoutPageSerializer(serializers.ModelSerializer):

    class Meta:
        model = MeasurementUnits
        fields = '__all__'


class EditMeasurementUnitsSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(
        required=True, help_text="Id of MeasurementUnits")

    class Meta:
        model = MeasurementUnits
        fields = ['id', 'measurement_description']


class DeleteMeasurementUnitsSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(
        required=True, help_text="Id of MeasurementUnits")

    class Meta:
        model = MeasurementUnits
        fields = ['id']


class VatExemptionOrderPOSTorEDITSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(
        required=False, help_text="Id of VatExemptionOrder")

    class Meta:
        model = VatExemptionOrder
        fields = ['id', 'vat_exemption_order_code',
                  'vat_exemption_order_description']


class VatExemptionOrderGetbyIDSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(
        required=True)

    class Meta:
        model = VatExemptionOrder
        fields = '__all__'
        read_only_fields = ['vat_exemption_order_code',
                            'vat_exemption_order_description']


class VatExemptionOrderWithPageSerializer(serializers.ModelSerializer):
    page = serializers.IntegerField(
        required=False, default=1, help_text="Page number")

    class Meta:
        model = VatExemptionOrder
        fields = '__all__'
        read_only_fields = ['vat_exemption_order_code',
                            'vat_exemption_order_description']


class DeleteVatExemptionOrderSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(
        required=True, help_text="Id of VatExemptionOrder")

    class Meta:
        model = MeasurementUnits
        fields = ['id']


class BookingPaymentMethodPOSTorEDITSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(
        required=False, help_text="Id of BookingPaymentMethod")

    class Meta:
        model = BookingPaymentMethod
        fields = ['id', 'payment_method_code', 'payment_method_description']


class BookingPaymentMethodGetbyIDSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(
        required=True)

    class Meta:
        model = BookingPaymentMethod
        fields = '__all__'
        read_only_fields = ['payment_method_code',
                            'payment_method_description']


class BookingPaymentMethodWithPageSerializer(serializers.ModelSerializer):
    page = serializers.IntegerField(
        required=False, default=1, help_text="Page number")

    class Meta:
        model = BookingPaymentMethod
        fields = '__all__'
        read_only_fields = ['payment_method_code',
                            'payment_method_description']


class DeleteBookingPaymentMethodSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(
        required=True, help_text="Id of BookingPaymentMethod")

    class Meta:
        model = BookingPaymentMethod
        fields = ['id']


class VatPOSTorEDITSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(
        required=False, help_text="Id of Vat")

    class Meta:
        model = Vat
        fields = ['id', 'vat_code', 'vat_description', 'vat_standard',
                  'vat_reduced', 'vat_exemption']


class VatGetbyIDSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(
        required=True)

    class Meta:
        model = Vat
        fields = '__all__'
        read_only_fields = ['vat_code', 'vat_description',
                            'vat_standard', 'vat_reduced', 'vat_exemption']


class VatWithPageSerializer(serializers.ModelSerializer):
    page = serializers.IntegerField(
        required=False, default=1, help_text="Page number")

    class Meta:
        model = Vat
        fields = '__all__'
        read_only_fields = ['vat_code', 'vat_description',
                            'vat_standard', 'vat_reduced', 'vat_exemption']


class VatWithoutPageSerializer(serializers.ModelSerializer):

    class Meta:
        model = Vat
        fields = '__all__'


class DeleteVatSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(
        required=True, help_text="Id of Vat")

    class Meta:
        model = BookingPaymentMethod
        fields = ['id']


class MixedGenderSerializer(serializers.ModelSerializer):
    class Meta:
        model = MIXED_GENDER
        fields = '__all__'


class AvailabilityInUseSerializer(serializers.ModelSerializer):
    class Meta:
        model = AVAILABILITY_IN_USE
        fields = '__all__'


class DocNumberingSeriesSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(
        required=False, help_text="ID for PATCH Operation")
    description = serializers.CharField(
        required=False, help_text="Description")
    code = serializers.CharField(required=False, help_text="Code")

    def validate_id(self, value):
        try:
            return validate_id(self, value)
        except Exception as e:
            raise serializers.ValidationError(str(e))

    def validate_code(self, value):
        try:
            return validate_code(self, value)
        except Exception as e:
            raise serializers.ValidationError(e)

    def save(self):
        try:
            save_edit_invoices(self)
            return True
        except Exception as e:
            print(e)
            raise Exception(str(e))

    class Meta:
        model = DocNumberingSeries
        fields = '__all__'


class DocumentTypeDocNumberingSeriesSerializer(serializers.ModelSerializer):
    document_numbering_series = DocNumberingSeriesSerializer(required=False)

    class Meta:
        model = DocumentTypeDocNumberingSeries
        fields = '__all__'


class DocumentTypeSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(
        required=False, help_text="ID for PATCH Operation")
    description = serializers.CharField(
        required=False, help_text="Description")
    code = serializers.CharField(required=False, help_text="Code")
    doc_numbering_series = serializers.JSONField(
        required=False, help_text="[1,2,3,4]")

    def validate_id(self, value):
        try:
            return validate_id(self, value)
        except Exception as e:
            raise serializers.ValidationError(str(e))

    def validate_doc_numbering_series(self, value):
        data = []
        for i in value:
            doc_numbering_series = DocNumberingSeries.objects.filter(id=i)
            if doc_numbering_series.exists():
                data.append(doc_numbering_series.last())
            else:
                raise serializers.ValidationError(
                    f'Incorrect Doc Numbering ID {i}')
        return data

    def validate_code(self, value):
        try:
            return validate_code(self, value)
        except Exception as e:
            raise serializers.ValidationError(e)

    def save(self):
        try:
            document_numbering_series = self.validated_data.pop(
                "doc_numbering_series", False)

            obj_id = save_edit_invoices(self)
            if document_numbering_series:
                DocumentTypeDocNumberingSeries.objects.filter(
                    document_type=obj_id).delete()
                document_type_numbering_series = [DocumentTypeDocNumberingSeries(
                    document_type=obj_id,
                    document_numbering_series=i
                ) for i in document_numbering_series]
                DocumentTypeDocNumberingSeries.objects.bulk_create(
                    document_type_numbering_series)
            return True
        except Exception as e:
            raise Exception(e)

    class Meta:
        model = DocumentType
        fields = '__all__'


class GetDocNumberingStringForDocTypeSerializer(serializers.ModelSerializer):
    """
    Get Doc Numbering Strings of Document Type
    """
    class Meta:
        model = DocumentTypeDocNumberingSeries
        fields = '__all__'


class GetInvoiceStaticDataSerializer(serializers.ModelSerializer):
    doc_numbering_series = serializers.SerializerMethodField(
        "_get_numbering_series")

    def _get_numbering_series(self, obj):
        print(self.Meta.model)
        if self.Meta.model == DocumentType:
            doc_numbering_series = DocumentTypeDocNumberingSeries.objects.filter(
                document_type=obj)
            doc_numbering_class = GetDocNumberingStringForDocTypeSerializer(
                doc_numbering_series, many=True)
            return doc_numbering_class.data

    class Meta:
        model = None
        fields = '__all__'


class TransferPurposeSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(
        required=False, help_text="ID for PATCH Operation")
    description = serializers.CharField(
        required=False, help_text="Description")
    code = serializers.CharField(required=False, help_text="Code")

    def validate_id(self, value):
        try:
            return validate_id(self, value)
        except Exception as e:
            raise serializers.ValidationError(str(e))

    def validate_code(self, value):
        try:
            return validate_code(self, value)
        except Exception as e:
            raise serializers.ValidationError(e)

    def save(self):
        try:
            save_edit_invoices(self)
            return True

        except Exception as e:
            raise Exception(e)

    class Meta:
        model = TransferPurpose
        fields = '__all__'


class TransferMethodSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(
        required=False, help_text="ID for PATCH Operation")
    description = serializers.CharField(
        required=False, help_text="Description")
    code = serializers.CharField(required=False, help_text="Code")

    def validate_id(self, value):
        try:
            return validate_id(self, value)
        except Exception as e:
            raise serializers.ValidationError(str(e))

    def validate_code(self, value):
        try:
            return validate_code(self, value)
        except Exception as e:
            raise serializers.ValidationError(e)

    def save(self):
        try:
            save_edit_invoices(self)
            return True

        except Exception as e:
            raise Exception(e)

    class Meta:
        model = TransferMethod
        fields = '__all__'


class MeansOfTransportSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(
        required=False, help_text="ID for PATCH Operation")
    description = serializers.CharField(
        required=False, help_text="Description")
    code = serializers.CharField(required=False, help_text="Code")

    def validate_id(self, value):
        try:
            return validate_id(self, value)
        except Exception as e:
            raise serializers.ValidationError(str(e))

    def validate_code(self, value):
        try:
            return validate_code(self, value)
        except Exception as e:
            raise serializers.ValidationError(e)

    def save(self):
        try:
            save_edit_invoices(self)
            return True

        except Exception as e:
            raise Exception(e)

    class Meta:
        model = MeansOfTransport
        fields = '__all__'


class GetStaticNamesSerializer(serializers.ModelSerializer):
    """
    Get Name and id of all static serializer
    """
    code = serializers.CharField(required=False)
    name = serializers.CharField(required=False)

    class Meta:
        model = None
        fields = ['code', 'name', 'id']


class VatExemptionOrderWithoutPageSerializer(serializers.ModelSerializer):

    class Meta:
        model = VatExemptionOrder
        fields = '__all__'


class CancellationPolicyySerializer(serializers.ModelSerializer):

    class Meta:
        model = CancellationPolicy
        fields = '__all__'


class PlayerLevelSerializer(serializers.ModelSerializer):
    class Meta:
        model = PlayerLevel
        fields = '__all__'


class ServiceByProfesssionSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(
        required=False, help_text='id of profession')
    profession_name = serializers.CharField(
        required=False, read_only=True, source="profession.professionName")

    class Meta:
        model = StaticProfessionalService
        fields = '__all__'
        read_only_fields = ['service', 'profession']


class NestedSportsOrFieldtypeBySportsSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(
        required=False, help_text='id of sport')

    class Meta:
        model = Sport
        fields = '__all__'
        read_only_fields = ['name', 'play_type',
                            'category', 'show_on_homepage', 'parent_sport']


class VenueIDSerializer(serializers.Serializer):
    venue_id = serializers.IntegerField(
        required=False, help_text='id of Venue')
