import pytz
import json
from django.http import HttpResponse
from django.template import Template, Context
from django.template.loader import get_template
from django.db import transaction
from django.urls import reverse

from django.contrib.auth import login as django_login
import datetime
from rest_framework.authtoken.models import Token
from rest_framework import status, generics
from rest_framework.response import Response
from rest_framework.authentication import TokenAuthentication
from rest_framework.permissions import IsAuthenticated
from utils.mixins import PlatformAPICallMixin
from utils.api_handle_errors import handle_errors

from static_data.serializers import *
from static_data.models import *
from professionals.models import Professional

class SkillsSetAPIView(PlatformAPICallMixin,generics.GenericAPIView):
    '''
    Get  All Skills Set
    '''
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = SkillsSetSerializer
    # Get All Skills

    def post(self, request, format=None):
        response = {}
        responsedata = []
        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            skill_obj = Skill.objects.all()
            skill_serializer = self.serializer_class(skill_obj, many=True)
            #Send response
            response['result'] = 1
            response['data'] = skill_serializer.data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            return Response(response, status=status.HTTP_400_BAD_REQUEST)




class RequestSkillAPIView(PlatformAPICallMixin,generics.GenericAPIView):
    '''
    Get  All Skills Set
    '''
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = RequestSkillsSerializer
    # Get All Skills

    def post(self, request, format=None):
        response = {}
        responsedata = []
        professional_obj = Professional.objects.filter(user=request.user)
        if professional_obj.exists():
            professional_obj = professional_obj.last()
        else:
            response["result"] = 0
            response["errors"] = ["UnAuthorized"]
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        serializers = self.serializer_class(data=request.data, context={"professional":professional_obj})
        if serializers.is_valid():
            serializers.save()
            #Send response
            response['result'] = 1
            response['Message'] = "Successfully Requested"
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            return Response(response, status=status.HTTP_400_BAD_REQUEST)




class RequestServiceAPIView(PlatformAPICallMixin,generics.GenericAPIView):
    '''
    posot service
    '''
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = RequestServiceSerializer
    # Get All Skills

    def post(self, request, format=None):
        response = {}
        responsedata = []
        professional_obj = Professional.objects.filter(user=request.user)
        if professional_obj.exists():
            professional_obj = professional_obj.last()
        else:
            response["result"] = 0
            response["errors"] = ["UnAuthorized"]
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        serializers = self.serializer_class(data=request.data, context={"professional":professional_obj})
        if serializers.is_valid():
            serializers.save()
            #Send response
            response['result'] = 1
            response['Message'] = "Successfully Requested"
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            return Response(response, status=status.HTTP_400_BAD_REQUEST)






# class SkillsSetAPIView(PlatformAPICallMixin,generics.GenericAPIView):
#     '''
#     Get  All Skills Set
#     '''
#     # authentication_classes = (TokenAuthentication,)
#     # permission_classes = (IsAuthenticated,)
#     serializer_class = SkillsSetSerializer
#     # Get All Skills

#     def post(self,request,format=None):
#         try:
#             response = {}
#             responsedata = []
#             data = Skill.objects.all()
#             serializer = self.serializer_class(data,many=True)
#             response['result'] = 1
#             response['data'] = serializer.data
#             return Response(response, status=status.HTTP_200_OK)
#         except Exception as e:
#             response["result"] = 0
#             response['errors'] = responsedata.append(str(e))
#             self.add_api_call(request, response)
#             return Response(response, status=status.HTTP_400_BAD_REQUEST)
