import requests
import pytz
import json
from django.http import HttpResponse
from django.template import Template, Context
from django.template.loader import get_template
from django.db import transaction
from django.urls import reverse

from django.contrib.auth import login as django_login
import datetime
from rest_framework.authtoken.models import Token
from rest_framework import status, generics
from rest_framework.response import Response
from rest_framework.authentication import TokenAuthentication
from rest_framework.permissions import IsAuthenticated
from utils.mixins import PlatformAPICallMixin
from utils.api_handle_errors import handle_errors

from static_data.serializers import *
from static_data.models import *
from static_data.functionalities.invoice import *
from venues.models import *


class FloorTypeGetAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    '''
    Get  All Floor Types
    '''
    # authentication_classes = (TokenAuthentication,)
    # permission_classes = (IsAuthenticated,)
    serializer_class = FloorTypeSerializer

    def get(self, request, format=None):
        response = {}
        responsedata = []
        data = FloorType.objects.all()
        serializer = self.serializer_class(data, many=True)
        response['result'] = 1
        response['data'] = serializer.data
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


class FieldTypeGetAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    '''
    Get  All Field Types
    '''
    serializer_class = FieldTypeSerializer

    def get(self, request, format=None):
        response = {}
        responsedata = []
        data = FieldType.objects.all()
        serializer = self.serializer_class(data, many=True)
        response['result'] = 1
        response['data'] = serializer.data
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


class FieldAttributeGetAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    '''
    Get  All Field Attributes
    '''
    # authentication_classes = (TokenAuthentication,)
    # permission_classes = (IsAuthenticated,)
    serializer_class = FieldAttributeSerializer

    def get(self, request, format=None):
        response = {}
        responsedata = []
        data = FieldAttribute.objects.all()
        serializer = self.serializer_class(data, many=True)
        response['result'] = 1
        response['data'] = serializer.data
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


class VenuePermissionAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    """
    VenuePermission
    """
    serializer_class = VenuePermissionSerializer

    def get(self, request, format=None):
        response = {}
        responsedata = []
        data = VenuePermission.objects.all()
        serializer = self.serializer_class(data, many=True)
        response['result'] = 1
        response['data'] = serializer.data
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


class SportsPositionAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    """
    Sports Position
    """
    serializer_class = PositionSerializer

    def get(self, request, format=None):
        response = {}
        responsedata = []
        data = Position.objects.all()
        serializer = self.serializer_class(data, many=True)
        response['result'] = 1
        response['data'] = serializer.data
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


class GetDayofWeekAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    """
    Get Days of week
    """
    serializer_class = DayofWeekSerializer

    def get(self, request, format=None):
        response = {}
        responsedata = []
        data = DayofWeek.objects.all()
        serializer = self.serializer_class(data, many=True)
        response['result'] = 1
        response['data'] = serializer.data
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


class ServicesAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    """
    Venue Services
    """
    serializer_class = VenueServiceSerializer

    def get(self, request, format=None):
        response = {}
        responsedata = []
        data = VenueService.objects.all()
        serializer = self.serializer_class(data, many=True)
        response['result'] = 1
        response['data'] = serializer.data
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


class UserTypeAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    """
    User Type
    """
    serializer_class = UserTypeSerializer

    def get(self, request, format=None):
        response = {}
        responsedata = []
        data = UserType.objects.all()
        serializer = self.serializer_class(data, many=True)
        response['result'] = 1
        response['data'] = serializer.data
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


class CityCountryGetAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    """
    Get City and country
    """
    city_serializer_class = CitySerializer
    country_serializer_class = CountrySerializer

    def get(self, request, format=None):
        response = {}
        responsedata = []
        city_data = City.objects.all()
        country_data = Country.objects.all()

        city_serializer = self.city_serializer_class(city_data, many=True)
        country_serializer = self.country_serializer_class(
            country_data, many=True)

        response['result'] = 1
        response['data'] = city_serializer.data + country_serializer.data
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


class MailTestAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    def get(self, request, format=None):
        response = {}
        change = requests.post(
            "https://api.mailgun.net/v3/sandbox6267135107c6406a92ea100b99a6b99b.mailgun.org/messages",
            auth=("api", "e09ce36eb9b16508fad2d1aca600a272-ba042922-4f7d9d0e"),
            data={"from": "mailgun@sandbox6267135107c6406a92ea100b99a6b99b.mailgun.org",
                  "to": "yatin.pandit94@gmail.com",
                  "subject": "Hello",
                  "text": "Testing some Mailgun awesomness!"})
        print(change.status_code)
        print(change.content)

        response['result'] = 1
        response['data'] = "city_serializer.data + country_serializer.data"
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


class GetBankAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    """
    Get Bannk
    """
    serializer_class = BankNameSerializer

    def get(self, request, format=None):
        response = {}
        bank_data = BankName.objects.all()

        bank_serializer = self.serializer_class(bank_data, many=True)

        response['result'] = 1
        response['data'] = bank_serializer.data
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


class CountryCodeGetAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    """
    Country Codes
    """
    serializer_class = CountryCodeSerializer

    def get(self, request, format=None):
        response = {}
        responsedata = []
        data = TelephoneCountryCode.objects.all()
        serializer = self.serializer_class(data, many=True)
        response['result'] = 1
        response['data'] = serializer.data
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


class CountryCodePOSTAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    # authentication_classes = (TokenAuthentication,)
    # permission_classes = (IsAuthenticated,)
    serializer_class = CountryCodePOSTSerializer

    def post(self, request, format=None):

        response = {}
        responsedata = []
        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            data = serializers.save()
            responsedata.append("Successfully Inserted")
            response["result"] = 1
            response['data'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class RegionGetAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    """
    Regions
    """
    serializer_class = RegionSerializer

    def get(self, request, format=None):
        response = {}
        responsedata = []
        data = Region.objects.all()
        serializer = self.serializer_class(data, many=True)
        response['result'] = 1
        response['data'] = serializer.data
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


class TeamLevelGetAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    """
    Team Levels
    """
    serializer_class = TeamLevelSerializer

    def get(self, request, format=None):
        response = {}
        responsedata = []
        data = TeamLevel.objects.all()
        serializer = self.serializer_class(data, many=True)
        response['result'] = 1
        response['data'] = serializer.data
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


class GetStatesByCountryAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    # authentication_classes = (TokenAuthentication,)
    # permission_classes = (IsAuthenticated,)
    serializer_class = StateGetSerializer

    def post(self, request, format=None):
        '''
        Get States by Country
        '''
        response = {}
        responsedata = []

        serializer = self.serializer_class(data=request.data)
        if serializer.is_valid():

            if "country" in serializer.validated_data:
                if Country.objects.filter(id=serializer.validated_data["country"]).exists():
                    serializer.validated_data['country'] = Country.objects.get(
                        id=serializer.validated_data["country"])
                else:
                    response["result"] = 0
                    response['data'] = "Invalid country id"
                    self.add_api_call(request, response)
                    return Response(response, status=status.HTTP_200_OK)

            state_data = State.objects.filter(
                **serializer.validated_data)

            serializer = self.serializer_class(state_data, many=True)

            response["result"] = 1
            response['data'] = serializer.data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class GetCitysByStateAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    # authentication_classes = (TokenAuthentication,)
    # permission_classes = (IsAuthenticated,)
    serializer_class = CityGetSerializer

    def post(self, request, format=None):
        '''
        Get citys by State
        '''
        response = {}
        responsedata = []

        serializer = self.serializer_class(data=request.data)
        if serializer.is_valid():

            if "state" in serializer.validated_data:
                if State.objects.filter(id=serializer.validated_data["state"]).exists():
                    serializer.validated_data['state'] = State.objects.get(
                        id=serializer.validated_data["state"])
                else:
                    response["result"] = 0
                    response['data'] = "Invalid state id"
                    self.add_api_call(request, response)
                    return Response(response, status=status.HTTP_200_OK)

            city_data = City.objects.filter(
                **serializer.validated_data)

            serializer = self.serializer_class(city_data, many=True)

            response["result"] = 1
            response['data'] = serializer.data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class GetAreasByCityAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    # authentication_classes = (TokenAuthentication,)
    # permission_classes = (IsAuthenticated,)
    serializer_class = AreaGetSerializer

    def post(self, request, format=None):
        '''
        Get areas by city
        '''
        response = {}
        responsedata = []

        serializer = self.serializer_class(data=request.data)
        if serializer.is_valid():

            if "city" in serializer.validated_data:
                if City.objects.filter(id=serializer.validated_data["city"]).exists():
                    serializer.validated_data['city'] = City.objects.get(
                        id=serializer.validated_data["city"])
                else:
                    response["result"] = 0
                    response['error'] = "Invalid city id"
                    self.add_api_call(request, response)
                    return Response(response, status=status.HTTP_200_OK)

            area_data = Area.objects.filter(
                **serializer.validated_data)

            serializer = self.serializer_class(area_data, many=True)

            response["result"] = 1
            response['data'] = serializer.data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class CountryGetAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    """
    Countrys
    """
    serializer_class = CountrySerializer

    def get(self, request, format=None):
        response = {}
        responsedata = []
        data = Country.objects.all()
        serializer = self.serializer_class(data, many=True)
        response['result'] = 1
        response['data'] = serializer.data
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


class GetfacilitysAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    # authentication_classes = (TokenAuthentication,)
    # permission_classes = (IsAuthenticated,)
    serializer_class = FacilitysSerializer

    def post(self, request, format=None):
        '''
        Get all facility type
        '''
        response = {}
        responsedata = []

        serializer = self.serializer_class(data=request.data)
        if serializer.is_valid():

            if "id" in serializer.validated_data:
                if FacilityType.objects.filter(id=serializer.validated_data["id"]).exists():
                    serializer.validated_data['name'] = FacilityType.objects.get(
                        id=serializer.validated_data["id"])
                else:
                    response["result"] = 0
                    response['error'] = "Invalid facility id"
                    self.add_api_call(request, response)
                    return Response(response, status=status.HTTP_200_OK)

            facility_data = FacilityType.objects.filter(
                **serializer.validated_data)

            serializer = self.serializer_class(facility_data, many=True)

            response["result"] = 1
            response['data'] = serializer.data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class GetLightsAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    # authentication_classes = (TokenAuthentication,)
    # permission_classes = (IsAuthenticated,)
    serializer_class = LightsSerializer

    def post(self, request, format=None):
        '''
        Get all lights
        '''
        response = {}
        responsedata = []

        serializer = self.serializer_class(data=request.data)
        if serializer.is_valid():

            if "id" in serializer.validated_data:
                if Lights.objects.filter(id=serializer.validated_data["id"]).exists():
                    serializer.validated_data['name'] = Lights.objects.get(
                        id=serializer.validated_data["id"])
                else:
                    response["result"] = 0
                    response['error'] = "Invalid lights id"
                    self.add_api_call(request, response)
                    return Response(response, status=status.HTTP_200_OK)

            lights_data = Lights.objects.filter(
                **serializer.validated_data)

            serializer = self.serializer_class(lights_data, many=True)

            response["result"] = 1
            response['data'] = serializer.data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class PaymentMethodAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    """
    Payment Methods
    """
    serializer_class = PaymentMethodSerializer

    def get(self, request, format=None):
        response = {}
        responsedata = []
        data = PaymentMethod.objects.all()
        serializer = self.serializer_class(data, many=True)
        response['result'] = 1
        response['data'] = serializer.data
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


class BookingCancellationWithPageAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    """
    Get All data from Booking Cancellation table with pagination
    """
    serializer_class = BookingCancellationWithPageSerializer

    def post(self, request, format=None):
        # get page number
        if 'page' in request.data:
            start_count = int(request.data['page']) - 1
        else:
            start_count = 0
        start_count = start_count * 10
        end_count = start_count + 10
        response = {}
        responsedata = []
        data = BookingCancellation.objects.all()[start_count:end_count]
        total_count = BookingCancellation.objects.all().count()
        serializer = self.serializer_class(data, many=True)
        response['result'] = 1
        response['data'] = serializer.data
        response['total_count'] = total_count
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


class BookingCancellationAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    """
    Get All data from Booking Cancellation table
    """
    serializer_class = BookingCancellationSerializer

    def get(self, request, format=None):
        response = {}
        responsedata = []
        data = BookingCancellation.objects.all()
        serializer = self.serializer_class(data, many=True)
        response['result'] = 1
        response['data'] = serializer.data
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


class TemplateImagesGetAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    """
    Template images
    """
    serializer_class = Template_ImagesSerializer

    def get(self, request, format=None):
        response = {}
        responsedata = []
        data = Template_Images.objects.all()
        serializer = self.serializer_class(data, many=True)
        response['result'] = 1
        response['data'] = serializer.data
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


class MeasurementUnitsPOSTAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    # authentication_classes = (TokenAuthentication,)
    # permission_classes = (IsAuthenticated,)
    serializer_class = MeasurementUnitsPOSTSerializer

    def post(self, request, format=None):

        response = {}
        responsedata = []
        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            data = serializers.save()
            responsedata.append("Successfully Inserted")
            response["result"] = 1
            response['data'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class GetMeasurementUnitsAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    # authentication_classes = (TokenAuthentication,)
    # permission_classes = (IsAuthenticated,)
    serializer_class = MeasurementUnitsGetbyIDSerializer

    def post(self, request, format=None):
        '''
        Get MeasurementUnits based on id
         '''
        response = {}
        responsedata = []

        serializer = self.serializer_class(data=request.data)
        if serializer.is_valid():

            if "id" in serializer.validated_data:
                if MeasurementUnits.objects.filter(id=serializer.validated_data["id"]).exists():
                    serializer.validated_data['name'] = MeasurementUnits.objects.get(
                        id=serializer.validated_data["id"])
                else:
                    response["result"] = 0
                    response['error'] = "Invalid MeasurementUnits id"
                    self.add_api_call(request, response)
                    return Response(response, status=status.HTTP_200_OK)

            MeasurementUnits_data = MeasurementUnits.objects.get(
                id=request.data['id'])

            serializer = self.serializer_class(MeasurementUnits_data)

            response["result"] = 1
            response['data'] = serializer.data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class MeasurementUnitsWithPageAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    """
    Get All data from MeasurementUnits table with pagination
    """
    serializer_class = MeasurementUnitsWithPageSerializer

    def post(self, request, format=None):
        # get page number
        if 'page' in request.data:
            start_count = int(request.data['page']) - 1
        else:
            start_count = 0
        start_count = start_count * 10
        end_count = start_count + 10
        response = {}
        responsedata = []
        data = MeasurementUnits.objects.all()[start_count:end_count]
        total_count = MeasurementUnits.objects.all().count()
        serializer = self.serializer_class(data, many=True)
        response['result'] = 1
        response['data'] = serializer.data
        response['total_count'] = total_count
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


class MeasurementUnitsWithoutPageAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    """
    Get All data from MeasurementUnits
    """
    serializer_class = MeasurementUnitsWithoutPageSerializer

    def get(self, request, format=None):

        response = {}
        responsedata = []
        data = MeasurementUnits.objects.all()
        serializer = self.serializer_class(data, many=True)
        response['result'] = 1
        response['data'] = serializer.data
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


class MeasurementUnitsEditInfoAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = EditMeasurementUnitsSerializer

    def post(self, request, format=None):
        '''
        Update MeasurementUnits
        '''
        responsedata = []
        response = {}

        if MeasurementUnits.objects.filter(id=request.data['id']).exists():
            serializer = self.serializer_class(instance=MeasurementUnits.objects.get(
                id=request.data['id']), data=request.data)
            message = "Updated Successfully"
        else:
            response["result"] = 0
            response['data'] = "Invalid MeasurementUnits id"
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        if serializer.is_valid():
            print(serializer.validated_data)
            serializer.save()
            response["result"] = 1
            response['data'] = [message]
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class DeleteMeasurementUnitsAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = DeleteMeasurementUnitsSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []

        if not MeasurementUnits.objects.filter(id=request.data['id']).exists():
            response['results'] = 0
            response['error'] = "Not a valid ID"
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        serializers = self.serializer_class(data=request.data)

        if serializers.is_valid():
            MeasurementUnits_obj = MeasurementUnits.objects.get(
                id=serializers.validated_data['id']).delete()

            response['results'] = 1
            response['data'] = "Successfully Deleted"
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        else:
            response['results'] = 0
            response = handle_errors(serializers.errors)
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class VatExemptionOrderPOSTorEDITAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = VatExemptionOrderPOSTorEDITSerializer

    def post(self, request, format=None):
        '''
        Add and Update VatExemptionOrder
        '''
        responsedata = []
        response = {}

        if 'id' in request.data:
            if VatExemptionOrder.objects.filter(id=request.data['id']).exists():
                serializer = self.serializer_class(instance=VatExemptionOrder.objects.get(
                    id=request.data['id']), data=request.data)
                message = "Updated Successfully"
            else:
                response["result"] = 0
                response['data'] = "Invalid VatExemptionOrder id"
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)
        else:
            serializer = self.serializer_class(data=request.data)
            message = "Created Successfully"

        if serializer.is_valid():
            print(serializer.validated_data)
            serializer.save()
            response["result"] = 1
            response['data'] = [message]
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class GetVatExemptionOrderAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    # authentication_classes = (TokenAuthentication,)
    # permission_classes = (IsAuthenticated,)
    serializer_class = VatExemptionOrderGetbyIDSerializer

    def post(self, request, format=None):
        '''
        Get VatExemptionOrder based on id
         '''
        response = {}
        responsedata = []

        serializer = self.serializer_class(data=request.data)
        if serializer.is_valid():

            if "id" in serializer.validated_data:
                if VatExemptionOrder.objects.filter(id=serializer.validated_data["id"]).exists():
                    serializer.validated_data['name'] = VatExemptionOrder.objects.get(
                        id=serializer.validated_data["id"])
                else:
                    response["result"] = 0
                    response['error'] = "Invalid VatExemptionOrder id"
                    self.add_api_call(request, response)
                    return Response(response, status=status.HTTP_200_OK)

            VatExemptionOrder_data = VatExemptionOrder.objects.get(
                id=request.data['id'])

            serializer = self.serializer_class(VatExemptionOrder_data)

            response["result"] = 1
            response['data'] = serializer.data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class VatExemptionOrderWithPageAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    """
    Get All data from VatExemptionOrder table with pagination
    """
    serializer_class = VatExemptionOrderWithPageSerializer

    def post(self, request, format=None):
        # get page number
        if 'page' in request.data:
            start_count = int(request.data['page']) - 1
        else:
            start_count = 0
        start_count = start_count * 10
        end_count = start_count + 10
        response = {}
        responsedata = []
        data = VatExemptionOrder.objects.all()[start_count:end_count]
        total_count = VatExemptionOrder.objects.all().count()
        serializer = self.serializer_class(data, many=True)
        response['result'] = 1
        response['data'] = serializer.data
        response['total_count'] = total_count
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


class DeleteVatExemptionOrderAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = DeleteVatExemptionOrderSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []

        if not VatExemptionOrder.objects.filter(id=request.data['id']).exists():
            response['results'] = 0
            response['error'] = "Not a valid ID"
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        serializers = self.serializer_class(data=request.data)

        if serializers.is_valid():
            VatExemptionOrder_obj = VatExemptionOrder.objects.get(
                id=serializers.validated_data['id']).delete()

            response['results'] = 1
            response['data'] = "Successfully Deleted"
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        else:
            response['results'] = 0
            response = handle_errors(serializers.errors)
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class BookingPaymentMethodPOSTorEDITAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = BookingPaymentMethodPOSTorEDITSerializer

    def post(self, request, format=None):
        '''
        Add and Update BookingPaymentMethod
        '''
        responsedata = []
        response = {}

        if 'id' in request.data:
            if BookingPaymentMethod.objects.filter(id=request.data['id']).exists():
                serializer = self.serializer_class(instance=BookingPaymentMethod.objects.get(
                    id=request.data['id']), data=request.data)
                message = "Updated Successfully"
            else:
                response["result"] = 0
                response['data'] = "Invalid BookingPaymentMethod id"
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)
        else:
            serializer = self.serializer_class(data=request.data)
            message = "Created Successfully"

        if serializer.is_valid():
            print(serializer.validated_data)
            serializer.save()
            response["result"] = 1
            response['data'] = [message]
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class GetBookingPaymentMethodAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    # authentication_classes = (TokenAuthentication,)
    # permission_classes = (IsAuthenticated,)
    serializer_class = BookingPaymentMethodGetbyIDSerializer

    def post(self, request, format=None):
        '''
        Get BookingPaymentMethod based on id
         '''
        response = {}
        responsedata = []

        serializer = self.serializer_class(data=request.data)
        if serializer.is_valid():

            if "id" in serializer.validated_data:
                if BookingPaymentMethod.objects.filter(id=serializer.validated_data["id"]).exists():
                    serializer.validated_data['name'] = BookingPaymentMethod.objects.get(
                        id=serializer.validated_data["id"])
                else:
                    response["result"] = 0
                    response['error'] = "Invalid BookingPaymentMethod id"
                    self.add_api_call(request, response)
                    return Response(response, status=status.HTTP_200_OK)

            BookingPaymentMethod_data = BookingPaymentMethod.objects.get(
                id=request.data['id'])

            serializer = self.serializer_class(BookingPaymentMethod_data)

            response["result"] = 1
            response['data'] = serializer.data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class BookingPaymentMethodWithPageAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    """
    Get All data from BookingPaymentMethod table with pagination
    """
    serializer_class = BookingPaymentMethodWithPageSerializer

    def post(self, request, format=None):
        # get page number
        if 'page' in request.data:
            start_count = int(request.data['page']) - 1
        else:
            start_count = 0
        start_count = start_count * 10
        end_count = start_count + 10
        response = {}
        responsedata = []
        data = BookingPaymentMethod.objects.all()[start_count:end_count]
        total_count = BookingPaymentMethod.objects.all().count()
        serializer = self.serializer_class(data, many=True)
        response['result'] = 1
        response['data'] = serializer.data
        response['total_count'] = total_count
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


class DeleteBookingPaymentMethodAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = DeleteBookingPaymentMethodSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []

        if not BookingPaymentMethod.objects.filter(id=request.data['id']).exists():
            response['results'] = 0
            response['error'] = "Not a valid ID"
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        serializers = self.serializer_class(data=request.data)

        if serializers.is_valid():
            BookingPaymentMethod_obj = BookingPaymentMethod.objects.get(
                id=serializers.validated_data['id']).delete()

            response['results'] = 1
            response['data'] = "Successfully Deleted"
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        else:
            response['results'] = 0
            response = handle_errors(serializers.errors)
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class VatPOSTorEDITAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = VatPOSTorEDITSerializer

    def post(self, request, format=None):
        '''
        Add and Update Vat
        '''
        responsedata = []
        response = {}

        if 'id' in request.data:
            if Vat.objects.filter(id=request.data['id']).exists():
                serializer = self.serializer_class(instance=Vat.objects.get(
                    id=request.data['id']), data=request.data)
                message = "Updated Successfully"
            else:
                response["result"] = 0
                response['data'] = "Invalid Vat id"
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)
        else:
            serializer = self.serializer_class(data=request.data)
            message = "Created Successfully"

        if serializer.is_valid():
            print(serializer.validated_data)
            serializer.save()
            response["result"] = 1
            response['data'] = [message]
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class GetVatAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    # authentication_classes = (TokenAuthentication,)
    # permission_classes = (IsAuthenticated,)
    serializer_class = VatGetbyIDSerializer

    def post(self, request, format=None):
        '''
        Get Vat based on id
         '''
        response = {}
        responsedata = []

        serializer = self.serializer_class(data=request.data)
        if serializer.is_valid():

            if "id" in serializer.validated_data:
                if Vat.objects.filter(id=serializer.validated_data["id"]).exists():
                    serializer.validated_data['name'] = Vat.objects.get(
                        id=serializer.validated_data["id"])
                else:
                    response["result"] = 0
                    response['error'] = "Invalid Vat id"
                    self.add_api_call(request, response)
                    return Response(response, status=status.HTTP_200_OK)

            Vat_data = Vat.objects.get(id=request.data['id'])

            serializer = self.serializer_class(Vat_data)

            response["result"] = 1
            response['data'] = serializer.data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class VatWithPageAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    """
    Get All data from Vat table with pagination
    """
    serializer_class = VatWithPageSerializer

    def post(self, request, format=None):
        # get page number
        if 'page' in request.data:
            start_count = int(request.data['page']) - 1
        else:
            start_count = 0
        start_count = start_count * 10
        end_count = start_count + 10
        response = {}
        responsedata = []
        data = Vat.objects.all()[start_count:end_count]
        total_count = Vat.objects.all().count()
        serializer = self.serializer_class(data, many=True)
        response['result'] = 1
        response['data'] = serializer.data
        response['total_count'] = total_count
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


class VatWithoutPageAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    """
    Get All data from Vat 
    """
    serializer_class = VatWithoutPageSerializer

    def get(self, request, format=None):
        response = {}
        responsedata = []
        data = Vat.objects.all()
        serializer = self.serializer_class(data, many=True)
        response['result'] = 1
        response['data'] = serializer.data
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


class DeleteVatAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = DeleteVatSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []

        if not Vat.objects.filter(id=request.data['id']).exists():
            response['results'] = 0
            response['error'] = "Not a valid ID"
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        serializers = self.serializer_class(data=request.data)

        if serializers.is_valid():
            Vat_obj = Vat.objects.get(
                id=serializers.validated_data['id']).delete()

            response['results'] = 1
            response['data'] = "Successfully Deleted"
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        else:
            response['results'] = 0
            response = handle_errors(serializers.errors)
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class MixedGenderGetAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    """
    Mixed Gender
    """
    serializer_class = MixedGenderSerializer

    def get(self, request, format=None):
        response = {}
        responsedata = []
        data = MIXED_GENDER.objects.all()
        serializer = self.serializer_class(data, many=True)
        response['result'] = 1
        response['data'] = serializer.data
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


class AvailabilityInUseGetAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    """
    Availability In Use
    """
    serializer_class = AvailabilityInUseSerializer

    def get(self, request, format=None):
        response = {}
        responsedata = []
        data = AVAILABILITY_IN_USE.objects.all()
        serializer = self.serializer_class(data, many=True)
        response['result'] = 1
        response['data'] = serializer.data
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


class GetInvoiceStaticDataAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    serializer_class = GetInvoiceStaticDataSerializer

    def get(self, request, model_name, id, format=None):
        """
        Get Any Invoice Static Data
        """
        response = {}
        kwargs = {'is_deleted': False}

        if id != 0:
            kwargs['id'] = id

        from django.apps import apps
        try:
            model_name = apps.get_model('static_data', model_name=model_name)
            data = model_name.objects.filter(**kwargs)
        except LookupError as e:
            response["result"] = 0
            response["errors"] = ["Incorrect Model Name"]
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        self.serializer_class.Meta.model = model_name
        invoice_static_data = self.serializer_class(data, many=True)
        response['result'] = 1
        response['data'] = invoice_static_data.data
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


class DocNumberingSeriesAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    # authentication_classes = (TokenAuthentication,)
    # permission_classes = (IsAuthenticated,)
    serializer_class = DocNumberingSeriesSerializer

    response = {}
    responsedata = []

    def post(self, request, format=None, **kwargs):
        """ 
        Doc Numbering Series Add / Edit Admin 
        """
        kwargs = {}
        if request.data.get("id", False):
            serializers = self.serializer_class(data=request.data, context={
                                                'id': request.data.get("id")})
            message = "Doc Number Series Successfully Updated"
        else:
            serializers = self.serializer_class(data=request.data, context={})
            message = "Doc Number Series Successfully Created"
        if serializers.is_valid():
            serializers.save()
            self.response['result'] = 1
            self.response['message'] = message
            self.add_api_call(request, self.response)
            return Response(self.response, status=status.HTTP_200_OK)
        else:
            self.response = handle_errors(serializers.errors)
            self.response['result'] = 0
            self.add_api_call(request, self.response)
            return Response(self.response, status=status.HTTP_200_OK)

    def delete(self, request, format=None):
        """
        DocNumberingSeries Delete Admin
        """
        response = {}
        try:
            delete_record(self, request)
            response["result"] = 1
            response["message"] = ["Doc Numbering Series Successfully Deleted"]
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        except Exception as e:
            response["result"] = 0
            response["errors"] = [e.detail]
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class DocumentTypeDocNumberingSeriesAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    # authentication_classes = (TokenAuthentication,)
    # permission_classes = (IsAuthenticated,)
    serializer_class = DocumentTypeDocNumberingSeriesSerializer

    def get(self, request, format=None, ** kwargs):
        """
        Doc Numbering Series Get
        """
        response = {}
        document_type_id = request.GET.get("doc_type", False)
        print(document_type_id)
        if not document_type_id:
            raise Exception("No Valid Document Type Id found")

        document_type = DocumentType.objects.filter(
            id=document_type_id, is_deleted=False)
        if document_type.exists():
            document_type = document_type.last()
        else:
            raise Exception("No Valid Document Type Id found")

        # Get Numbering Series
        doc_numbering_series = self.serializer_class.Meta.model.objects.filter(
            document_type=document_type)
        doc_numbering_class = self.serializer_class(
            doc_numbering_series, many=True)

        response["result"] = 1
        response["data"] = doc_numbering_class.data
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


class DocumentTypeAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    # authentication_classes = (TokenAuthentication,)
    # permission_classes = (IsAuthenticated,)
    serializer_class = DocumentTypeSerializer

    response = {}
    responsedata = []

    def post(self, request, format=None, **kwargs):
        """ 
        Document Type Add / Edit Admin 
        """
        kwargs = {}
        if request.data.get("id", False):
            serializers = self.serializer_class(data=request.data, context={
                                                'id': request.data.get("id")})
            message = "Document Type Successfully Updated"
        else:
            serializers = self.serializer_class(data=request.data, context={})
            message = "Document Type Successfully Created"
        if serializers.is_valid():
            serializers.save()
            self.response['result'] = 1
            self.response['message'] = message
            self.add_api_call(request, self.response)
            return Response(self.response, status=status.HTTP_200_OK)
        else:
            self.response = handle_errors(serializers.errors)
            self.response['result'] = 0
            self.add_api_call(request, self.response)
            return Response(self.response, status=status.HTTP_200_OK)

    def delete(self, request, format=None):
        """
        Document Type Delete Admin
        """
        response = {}
        try:
            delete_record(self, request)
            response["result"] = 1
            response["message"] = ["Document Type Successfully Deleted"]
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        except Exception as e:
            response["result"] = 0
            response["errors"] = [e.detail]
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class TransferPurposeAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    # authentication_classes = (TokenAuthentication,)
    # permission_classes = (IsAuthenticated,)
    serializer_class = TransferPurposeSerializer

    response = {}
    responsedata = []

    def post(self, request, format=None, **kwargs):
        """ 
        Transfer Purposes Add / Edit Admin 
        """
        kwargs = {}
        if request.data.get("id", False):
            serializers = self.serializer_class(data=request.data, context={
                                                'id': request.data.get("id")})
            message = "Transfer Purpose Successfully Updated"
        else:
            serializers = self.serializer_class(data=request.data, context={})
            message = "Transfer Purpose Successfully Created"
        if serializers.is_valid():
            serializers.save()
            self.response['result'] = 1
            self.response['message'] = message
            self.add_api_call(request, self.response)
            return Response(self.response, status=status.HTTP_200_OK)
        else:
            self.response = handle_errors(serializers.errors)
            self.response['result'] = 0
            self.add_api_call(request, self.response)
            return Response(self.response, status=status.HTTP_200_OK)

    def delete(self, request, format=None):
        """
        Transfer Purposes Delete Admin
        """
        response = {}
        try:
            delete_record(self, request)
            response["result"] = 1
            response["message"] = ["Transfer Purpose Successfully Deleted"]
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        except Exception as e:
            response["result"] = 0
            response["errors"] = [e.detail]
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class TransferMethodAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    # authentication_classes = (TokenAuthentication,)
    # permission_classes = (IsAuthenticated,)
    serializer_class = TransferMethodSerializer

    response = {}
    responsedata = []

    def post(self, request, format=None, **kwargs):
        """ 
        Transfer Methods Add / Edit Admin 
        """
        kwargs = {}
        if request.data.get("id", False):
            serializers = self.serializer_class(data=request.data, context={
                                                'id': request.data.get("id")})
            message = "Transfer Methods Successfully Updated"
        else:
            serializers = self.serializer_class(data=request.data, context={})
            message = "Transfer Methods Successfully Created"
        if serializers.is_valid():
            serializers.save()
            self.response['result'] = 1
            self.response['message'] = message
            self.add_api_call(request, self.response)
            return Response(self.response, status=status.HTTP_200_OK)
        else:
            self.response = handle_errors(serializers.errors)
            self.response['result'] = 0
            self.add_api_call(request, self.response)
            return Response(self.response, status=status.HTTP_200_OK)

    def delete(self, request, format=None):
        """
        Transfer Methods Delete Admin
        """
        response = {}
        try:
            delete_record(self, request)
            response["result"] = 1
            response["message"] = ["Transfer Methods Successfully Deleted"]
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        except Exception as e:
            response["result"] = 0
            response["errors"] = [e.detail]
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class MeansOfTransportAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    # authentication_classes = (TokenAuthentication,)
    # permission_classes = (IsAuthenticated,)
    serializer_class = MeansOfTransportSerializer

    response = {}
    responsedata = []

    def post(self, request, format=None, **kwargs):
        """ 
        Means of Transfport Add / Edit Admin 
        """
        kwargs = {}
        if request.data.get("id", False):
            serializers = self.serializer_class(data=request.data, context={
                                                'id': request.data.get("id")})
            message = "Means of Transfport Successfully Updated"
        else:
            serializers = self.serializer_class(data=request.data, context={})
            message = "Means of Transfport Successfully Created"
        if serializers.is_valid():
            serializers.save()
            self.response['result'] = 1
            self.response['message'] = message
            self.add_api_call(request, self.response)
            return Response(self.response, status=status.HTTP_200_OK)
        else:
            self.response = handle_errors(serializers.errors)
            self.response['result'] = 0
            self.add_api_call(request, self.response)
            return Response(self.response, status=status.HTTP_200_OK)

    def delete(self, request, format=None):
        """
        Means of Transfport Delete Admin
        """
        response = {}
        try:
            delete_record(self, request)
            response["result"] = 1
            response["message"] = ["Means of Transfport Successfully Deleted"]
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        except Exception as e:
            response["result"] = 0
            response["errors"] = [str(e)]
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class VatExemptionOrderWithoutPageAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    """
    Get All data from VatExemptionOrder table
    """
    serializer_class = VatExemptionOrderWithoutPageSerializer

    def get(self, request, format=None):
        response = {}
        responsedata = []
        data = VatExemptionOrder.objects.all()
        serializer = self.serializer_class(data, many=True)
        response['result'] = 1
        response['data'] = serializer.data
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


class GetServiceByProfessionAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    # authentication_classes = (TokenAuthentication,)
    # permission_classes = (IsAuthenticated,)
    serializer_class = ServiceByProfesssionSerializer

    def post(self, request, format=None):
        '''
        Get all Service by profession
        '''
        response = {}
        responsedata = []

        serializer = self.serializer_class(data=request.data)
        if serializer.is_valid():

            if "id" in serializer.validated_data:
                if Profession.objects.filter(id=serializer.validated_data["id"]).exists():
                    serializer.validated_data['profession'] = Profession.objects.get(
                        id=serializer.validated_data["id"])
                else:
                    response["result"] = 0
                    response['error'] = "Invalid profession id"
                    self.add_api_call(request, response)
                    return Response(response, status=status.HTTP_200_OK)

                StaticProfessionalService_data = StaticProfessionalService.objects.filter(
                    profession=serializer.validated_data['profession'])

                serializer = self.serializer_class(
                    StaticProfessionalService_data, many=True)

            response["result"] = 1
            response['data'] = serializer.data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class ProfessionalStaticServiceListAPIView(PlatformAPICallMixin, generics.CreateAPIView, generics.ListAPIView):
    # authentication_classes = (TokenAuthentication,)
    # permission_classes = (IsAuthenticated,)
    serializer_class = ServiceByProfesssionSerializer
    queryset = StaticProfessionalService.objects.all()

    def create(self, request, *args, **kwargs):
        super().create(request, *args, **kwargs)
        response = {"result": 1, "message": "Successfully Created"}
        return Response(response, status=status.HTTP_201_CREATED)


class ProfessionalStaticServiceDetailAPIView(PlatformAPICallMixin, generics.RetrieveUpdateDestroyAPIView):
    # authentication_classes = (TokenAuthentication,)
    # permission_classes = (IsAuthenticated,)
    serializer_class = ServiceByProfesssionSerializer
    queryset = StaticProfessionalService.objects.all()

    def post(self, request, format=None):
        pass


class GetNestedSportsOrFieldtypeBySportsAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    # authentication_classes = (TokenAuthentication,)
    # permission_classes = (IsAuthenticated,)
    serializer_class = NestedSportsOrFieldtypeBySportsSerializer

    def post(self, request, format=None):
        '''
        Get nested sports or fieldtype by sports
        '''
        response = {}
        responsedata = []

        serializer = self.serializer_class(data=request.data)
        if serializer.is_valid():
            data = {}
            if Sport.objects.filter(parent_sport=serializer.validated_data["id"]).exists():
                sport_data = Sport.objects.filter(
                    parent_sport=serializer.validated_data["id"])
                serializer = SportSerializer(
                    sport_data, many=True)
                data['sports'] = serializer.data
                data['type'] = 'sports'
            else:
                fieldtype_data = FieldType.objects.filter(
                    sport=serializer.validated_data['id'])
                serializer = FieldTypeSerializer(fieldtype_data, many=True)
                data['field_types'] = serializer.data
                data['type'] = 'field_type'

            responsedata.append(data)
            response["result"] = 1
            response['data'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class GetSportsByVenueAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    # authentication_classes = (TokenAuthentication,)
    # permission_classes = (IsAuthenticated,)
    serializer_class = VenueIDSerializer

    def post(self, request, format=None):
        '''
        Get Sports by Venue
        '''
        response = {}
        responsedata = []

        serializer = self.serializer_class(data=request.data)
        if serializer.is_valid():

            if Venue.objects.filter(id=serializer.validated_data["venue_id"]).exists():
                Venue_obj = Venue.objects.get(
                    id=serializer.validated_data["venue_id"])

                sports_obj = Venue_obj.all_sports.all()
                for i in sports_obj:
                    data = {}
                    data['id'] = i.id
                    data['name'] = i.name
                    responsedata.append(data)
            else:
                response["result"] = 0
                response['error'] = "Invalid Venue id"
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)

            response["result"] = 1
            response['data'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)
