from task.models import TaskCommand, Task
import datetime
from campaigns.models import CampaignPart1, CampaignPart3, CampaignPart2
import traceback
from players.models import Player
from django.db.models import Q
from utils.mail import send_email_campaign_task

class Command(TaskCommand):
    def add_arguments(self, parser):
        parser.add_argument("campaign_mail_pk")
        parser.add_argument("--progress_callback")
        parser.add_argument("--task_id")

    def handle(self, *args, **options):
        campaign_mail_pk = options['campaign_mail_pk']
        self.add_callbacks(options)
        self.progress_callback(progress='Running', state=Task.RUNNING, ran_at=datetime.datetime.now(datetime.timezone.utc))
        try:
            campaign1 = CampaignPart1.objects.get(pk=campaign_mail_pk)

            # Get Notification Settings of Campaigns
            campaign_step2 = CampaignPart2.objects.get(campaign1=campaign1)
            if campaign_step2.notification == "P":
                pass
            else:
                raise Exception("Other User types selected")

            # Filter 
            player_obj = Player.objects.filter(is_deleted=False).values_list('email')
            player_obj = [i[0] for i in list(player_obj)]
            player_obj.append(campaign1.created_by.email)
            # player_obj = ['yatin.pandit94@gmail.com']

            #  Get Layout
            email_layout = CampaignPart3.objects.filter(campaign1=campaign1, is_deleted=False).values('email_logo','emailBody','email_background')
            email_layout = email_layout[0]

            # Mail Function
            send_email_campaign_task(player_obj, email_layout)
            self.progress_callback(progress='Success', state=Task.SUCCESSFUL, ran=True, finished_at=datetime.datetime.now(datetime.timezone.utc))
            
            # Sending Confirmation Mail
        except CampaignPart1.DoesNotExist:
            self.progress_callback(progress="No Record Found for PK", state=Task.FAILED, finished_at=datetime.datetime.now(datetime.timezone.utc), ran=True)

        except Exception as e:
            self.progress_callback(traceback.format_exc(), Task.FAILED, exception=e)