from django.core.management.base import BaseCommand, CommandError
from django.core.management import call_command
import time, logging
import traceback
import datetime

from task.models import Task, DuplicateTaskException, TaskAlreadyInProgress, progress_callback

class Command(BaseCommand):
    help = 'Run the tasks in the task queue if any'
    def handle(self, *args, **options):
        try:
            new_tasks = Task.get_next_task_in_queue()

            for a_task in new_tasks:
                argument_to_pass = a_task.argument_list if a_task.argument_list else ''

                if datetime.datetime.now(datetime.timezone.utc) > a_task.scheduled_time:
                    print('New task run with command {} at {}'.format(a_task.command, datetime.datetime.now(datetime.timezone.utc)))
                    call_command(a_task.command, argument_to_pass, progress_callback=progress_callback(a_task.pk), task_id=a_task.pk)
            
        except TaskAlreadyInProgress as e:
            print('Some task already in progess', datetime.datetime.now(datetime.timezone.utc))
        except Task.DoesNotExist as e:
            print('No Pending tasks found', datetime.datetime.now(datetime.timezone.utc))
        except Exception as e:
            print(traceback.print_exc(), datetime.datetime.now(datetime.timezone.utc))
