from rest_framework import serializers
from django.contrib.auth.models import User, Group
from rest_framework.authtoken.models import Token
from django.contrib.contenttypes.models import ContentType
from django.db import transaction
from utils.api_handle_errors import handle_errors
from .models import *
from static_data.models import *
from static_data.serializers import *
from venues.serializers import VenueFieldSerializer
from commonAPI.models import TelephoneNumbers, ContactInfo, Contact_Person, BankInfo
from commonAPI.serializers import *
from django.db import transaction
from venues.models import *
from venues.serializers import *
from django.utils import timezone
from players.models import *
import datetime
####### VENUE OWNER PROFILE #########################
from admin_app.serializers import *


class VenueOwnerGetSerializer(serializers.ModelSerializer):
    country = CountrySerializer(read_only=True)
    city = CityGetSerializer(read_only=True)
    state = StateGetSerializer(read_only=True)
    area = AreaGetSerializer(read_only=True)

    class Meta:
        model = VenueOwner
        fields = '__all__'
        extra_kwargs = {
            'email': {'required': False, 'help_text': 'Email'},
            'user': {'required': False, 'help_text': 'Username'}
        }


class VenueOwnerSAVESerializer(serializers.ModelSerializer):
    class Meta:
        model = VenueOwner
        fields = '__all__'


class VenueDailyAvailibilitySerializer(serializers.ModelSerializer):
    class Meta:
        model = VenueDailyTiming
        fields = '__all__'


class VenueOwnerPOSTSerializer(serializers.ModelSerializer):
    Telephone_number = serializers.JSONField(
        required=False, help_text='{"mobile":["847123654","35135654"]}')
    iban = serializers.CharField(required=False)
    business_name = serializers.CharField(required=False)
    bank = serializers.CharField(required=False)
    acc_holder_first_name = serializers.CharField(required=False)
    acc_holder_last_name = serializers.CharField(required=False)
    acc_type = serializers.CharField(required=False)
    bank_identification_no = serializers.CharField(required=False)
    contact_person = serializers.JSONField(
        required=False,
        help_text='[{"first_name":"Test","last_name":"asd","telephone":1234567,"email":"email"},{"first_name":"Test","last_name":"asd","telephone":1234567,"email":"email"}]')
    timing = serializers.JSONField(
        required=False, help_text='[{"day":1, "start_time":12:00:00, "is_active":1, "end_time":1:00:00}]')

    # Venue Details
    logo = serializers.ImageField(required=False, help_text="Venue Logo")
    name = serializers.CharField(required=False, help_text="Venue Name")
    street = serializers.CharField(required=False, help_text="Venue street")
    pincode = serializers.CharField(required=False, help_text="Venue pincode")
    mobile = serializers.CharField(required=False, help_text="Mobile number")

    venue_number = serializers.CharField(
        required=False, help_text="Venue number")
    buisness_vat = serializers.CharField(
        required=False, help_text="Venue buisness vat")
    tax_offices = serializers.CharField(
        required=False, help_text="Venue buisness vat")

    working_hours = serializers.CharField(
        required=False, help_text="working hours")

    state = serializers.IntegerField(
        required=False, help_text="state")
    area = serializers.IntegerField(
        required=False, help_text="area")
    city = serializers.IntegerField(
        required=False, help_text="city")
    country = serializers.IntegerField(
        required=False, help_text="country")
    latitude = serializers.DecimalField(
        required=False, max_digits=9, decimal_places=6, help_text="Latitude")
    longitude = serializers.DecimalField(
        required=False, max_digits=9, decimal_places=6, help_text="Longitude")

    def validate_mobile(self, value):
        if len(value) < 10:
            raise serializers.ValidationError(
                "Mobile Number must be Filled in.")
        else:
            return value

    def validate_timing(self, value):
        for items in value:
            if items['day']:
                if not (items['start_time'] and items['end_time']):
                    raise serializers.ValidationError(
                        "Start time and End time must be filled in.")
                else:
                    return value

    @transaction.atomic
    def create(self, validated_data):
        '''
        venue owner
        '''
        validated_data['user'] = self.context['user'].id
        validated_data['email'] = self.context['user'].email
        serialized_data = VenueOwnerSAVESerializer(instance=VenueOwner.objects.filter(
            user=self.context['user']).first(), data=validated_data)
        if serialized_data.is_valid():
            venueowner_data = serialized_data.save()
        else:
            raise serializers.ValidationError(
                handle_errors(serialized_data.errors))

        '''
        Venue
        '''
        validated_data.pop('street')
        validated_data.pop('venue_number')
        validated_data.pop('pincode')
        validated_data.pop('country')
        validated_data.pop('city')
        validated_data.pop('state')
        validated_data.pop('area')

        validated_data['created_by'] = validated_data['user']
        if Venue.objects.filter(created_by=validated_data['user'], is_deleted=False).exists():
            venue_serializer = VenueProfileSerializer(instance=Venue.objects.filter(
                created_by=validated_data['user'], is_deleted=False).first(), data=validated_data)
        else:
            venue_serializer = VenueProfileSerializer(data=validated_data)

        if venue_serializer.is_valid():
            venue_data = venue_serializer.save()
        else:
            raise serializers.ValidationError(
                handle_errors(venue_serializer.errors))

        '''
        buisness name
        
        '''
        if "business_name" in validated_data:
            venue_id = venue_data.id
            Venue.objects.filter(id=venue_id).update(
                business_name=validated_data['business_name'])

        '''
        timing
        '''

        if "timing" in validated_data:
            for i in validated_data['timing']:
                i['venue'] = venue_data.id
            timing_serializer = VenueDailyAvailibilitySerializer(
                data=validated_data['timing'], many=True)
            print(timing_serializer)
            if timing_serializer.is_valid():
                VenueDailyTiming.objects.filter(venue=venue_data.id).delete()
                timing_data = timing_serializer.save()

            else:
                print(timing_serializer.errors)
                raise serializers.ValidationError(
                    handle_errors(timing_serializer.errors))

        '''
        Contact Info
        '''
        serialized_data = ContactInfoProfileSerializer(data=validated_data)
        if serialized_data.is_valid():
            if len(serialized_data.validated_data) > 0:
                contact_obj = VenueOwner.objects.filter(
                    user=self.context['user'])
                contact_contentType = ContentType.objects.get_for_model(
                    VenueOwner)
                ContactInfo.objects.filter(
                    content_type__pk=contact_contentType.id, object_id__in=contact_obj).delete()
                serialized_data.validated_data['content_object'] = venueowner_data
                contact_info_data = serialized_data.save()

        else:
            raise serializers.ValidationError(
                handle_errors(serialized_data.errors))

        '''
        Contact Numbers
        '''
        if "Telephone_number" in validated_data:
            TelephoneNumbers.objects.filter(
                contact_id=contact_info_data).delete()
            telephone_number = []

            for i in validated_data['Telephone_number']['mobile']:
                if i == "":
                    pass
                else:
                    telephone_number.append(i)

            contact_numbers = [TelephoneNumbers(contact_id=contact_info_data,
                                                contact_number=i
                                                ) for i in telephone_number]

            TelephoneNumbers.objects.bulk_create(contact_numbers)

        '''
        Contact Person
        '''
        if "contact_person" in validated_data:
            contact_persons = []

            contact_contentType = ContentType.objects.get_for_model(Venue)
            contact_obj = VenueOwner.objects.filter(user=self.context['user'])

            if Contact_Person.objects.filter(content_type__pk=contact_contentType.id,
                                             object_id__in=contact_obj).exists():
                Contact_Person.objects.filter(
                    content_type__pk=contact_contentType.id, object_id__in=contact_obj).delete()

            for i in validated_data['contact_person']:
                if i['first_name'] == "" and i['last_name'] == "":
                    pass
                else:
                    contact_persons.append(i)

            serialized_data = ContactPersonSerializer(
                data=contact_persons, many=True)
            if serialized_data.is_valid():
                for i in serialized_data.validated_data:
                    i['content_object'] = venueowner_data
                serialized_data.save()

        '''
        Bank Info
        '''
        if "bank" in validated_data:
            serialized_data = BankInfoProfileSerializer(data=validated_data)
            BankInfo.objects.filter(
                object_id=venueowner_data.id).update(is_deleted=True)
            if serialized_data.is_valid():
                serialized_data.validated_data['content_object'] = venueowner_data
                serialized_data.save()
            else:
                raise serializers.ValidationError(
                    handle_errors(serialized_data.errors))
        return venueowner_data.id

    class Meta:
        model = VenueOwner
        fields = '__all__'
        extra_kwargs = {
            'email': {"read_only": True},
            'new_email': {"read_only": True},
            'user': {"read_only": True}
        }


class VenueOwnerUPDATESerializer(serializers.ModelSerializer):
    class Meta:
        model = VenueOwner
        fields = '__all__'
        extra_kwargs = {
            'email': {'read_only': True},
            'user': {'read_only': True}
        }


########################################################################

class UserSerializer(serializers.ModelSerializer):
    class Meta:
        model = User
        fields = ('pk', 'username', 'first_name',
                  'last_name', 'email', 'date_joined')


class PriceListStep1GETSerializer(serializers.ModelSerializer):
    id = serializers.CharField(required=False)
    created_by = serializers.CharField(source='created_by.username')
    fields = VenueFieldSerializer(read_only=True, many=True)

    def __init__(self, *args, **kwargs):
        super(PriceListStep1GETSerializer, self).__init__(*args, **kwargs)
        for field in self.fields:
            if field != 'id':
                self.fields[field].read_only = True

    class Meta:
        model = PriceListStep1
        fields = '__all__'


class PriceListStep1POSTSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(required=False)
    fields = serializers.PrimaryKeyRelatedField(
        required=False, many=True, queryset=VenueField.objects.all())
    default_price = serializers.IntegerField(required=False)

    class Meta:
        model = PriceListStep1
        fields = '__all__'


class PriceListStep2PartSerializer(serializers.ModelSerializer):
    day = serializers.CharField(source="day.dayofweek")

    class Meta:
        model = PriceListStep2
        fields = ['id', 'start_hour', 'end_hour', 'price', 'day']


class PriceListStep2SerializerGET(serializers.ModelSerializer):
    created_by = serializers.CharField(write_only=True)
    id = serializers.IntegerField(required=False)
    day = serializers.CharField(source='day.dayofweek')

    def validate_id(self, value):
        if id:
            if PriceListStep2.objects.filter(id=value).exists():
                return value
            else:
                raise serializers.ValidationError("No Such Form")

    def __init__(self, *args, **kwargs):
        super(PriceListStep2SerializerGET, self).__init__(*args, **kwargs)
        for field in self.fields:
            if field == 'id':
                self.fields[field].read_only = False
            elif field == 'campaign1':
                self.fields[field].read_only = False
            else:
                self.fields[field].read_only = True

    class Meta:
        model = PriceListStep2
        fields = '__all__'


class PriceListStep2BridgeSerializer(serializers.Serializer):
    priceList1ID = serializers.IntegerField(required=True)
    data = serializers.JSONField(
        required=False, help_text='[{"id": 230,"price": 30}, {"id": 231,"price": 20}, ...]')

    created_by = serializers.IntegerField(required=False)

    def validate_priceList1ID(self, value):
        if PriceListStep1.objects.filter(id=value).exists():
            return value
        else:
            raise serializers.ValidationError("No Step 1 data found")

    def save(self):
        # del self.validated_data['created_by']
        for i in self.validated_data['data']:
            PriceListStep2.objects.filter(
                id=i['id'], price_list1_id=self.validated_data['priceList1ID']).update(price=i['price'])
        return "Success"


class PriceListStep3POSTSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(
        required=False, help_text="Price List Step 1 ID")
    # fields = serializers.PrimaryKeyRelatedField(required=False,many=True,queryset=VenueField.objects.all())
    fields = serializers.JSONField(required=False, help_text="[12,13,14,15]")

    def validate_fields(self, value):
        data = []
        for i in value:
            if VenueField.objects.filter(id=i).exists():
                data.append(VenueField.objects.get(id=i))
            else:
                raise serializers.ValidationError(
                    "No Field with " + str(i) + " data found")
        return data

    class Meta:
        model = PriceListStep1
        fields = ['id', 'fields']


class PriceListGlobalGetSerializer(serializers.Serializer):
    id = serializers.IntegerField(
        required=False, help_text="Primary Id of Pricelist 1")

    def validate_id(self, value):
        if PriceListStep1.objects.filter(id=value).exists():
            return value
        else:
            raise serializers.ValidationError("No Record with ID")

    class Meta:
        fields = ['id']


# Schedule Template #######################3

class PlanRangeGETSerializer(serializers.Serializer):
    range_choice = (
        ("D", "Daily"),
        ("W", "Weekly"),
        ("M", "Monthly")
    )
    plan_range = serializers.ChoiceField(
        choices=range_choice, help_text="D=Day,M=Monthly,W=weekly")
    start_hour = serializers.TimeField(
        required=False, help_text="Format = HH:MM:SS")
    end_hour = serializers.TimeField(
        required=False, help_text="Format = HH:MM:SS")
    interval = serializers.IntegerField(required=False, help_text="in Minutes")
    gap = serializers.IntegerField(required=False, help_text="in Minutes")


class ScheduleTemplateStep1GETSerializer(serializers.ModelSerializer):
    class Meta:
        model = ScheduleTemplateStep1
        fields = '__all__'


class ScheduleTemplateStep1POSTSerializer(serializers.ModelSerializer):
    slots = serializers.JSONField(
        required=False,
        help_text='each slot [{ "start_hour": "10:00", "end_hour": "11:00","day":1,"is_available":True }, { "start_hour": "11:00","end_hour": "12:00","day":1,"is_available":True}]')

    def validate_Template_Name(self, value, ):
        if ScheduleTemplateStep1.objects.filter(Template_Name=value, created_by=self.context['user']).exists():
            raise serializers.ImproperlyConfigured(
                "Template Name already exists")
        else:
            return value

    @transaction.atomic
    def create(self, validated_data, **kwargs):
        slots = validated_data['slots']
        validated_data.pop('slots')
        schedule_template = ScheduleTemplateStep1(
            **validated_data, created_by=self.context['user'])
        schedule_template.save()
        data = [ScheduleTemplateSlot(
            **i,
            template=schedule_template
        ) for i in slots]
        ScheduleTemplateSlot.objects.bulk_create(data)
        return "Success"

    class Meta:
        model = ScheduleTemplateStep1
        fields = '__all__'


class ScheduleTemplate1GETSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(
        required=False, help_text="Id of the schedule Template")

    def validate_Template_Name(self, value):
        if ScheduleTemplateStep1.objects.filter(Template_Name=value, created_by=self.context['user']).exists():
            return value
        else:
            raise serializers.FieldDoesNotExist("No Record with TemplateName")

    def validate_id(self, value):
        if ScheduleTemplateStep1.objects.filter(id=value, created_by=self.context['user']).exists():
            return value
        else:
            raise serializers.FieldDoesNotExist("No Record with ID")

    def __init__(self, *args, **kwargs):
        super(ScheduleTemplate1GETSerializer, self).__init__(*args, **kwargs)
        for field in self.fields:
            if field != 'Template_Name' and field != 'id':
                self.fields[field].read_only = True

    class Meta:
        model = ScheduleTemplateStep1
        fields = '__all__'


class ScheduleTemplateSlotsSerialzier(serializers.ModelSerializer):
    class Meta:
        model = ScheduleTemplateSlot
        fields = '__all__'


class ScheduleSlotSerializer(serializers.ModelSerializer):
    class Meta:
        model = Schedule_Slot
        fields = "__all__"


class ScheduleTemplateStep1UPDATESerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(
        required=False, help_text='Insert ID to update')
    created_by = serializers.IntegerField(required=False)
    slots = serializers.JSONField(
        required=False,
        help_text='each slot [{ "start_hour": "10:00", "end_hour": "11:00","day":1,"is_available":True }, { "start_hour": "11:00","end_hour": "12:00","day":1,"is_available":True}]')

    class Meta:
        model = ScheduleTemplateStep1
        fields = '__all__'


################### FLIGHTPLAN ####################


class FlightPlanPopUpSerializer(serializers.Serializer):
    id = serializers.IntegerField(
        required=False, help_text="Id of the schedule slot")


class FlightPlanBookMatchSerializer(serializers.ModelSerializer):
    # created_by = serializers.CharField(source='created_by.user')

    class Meta:
        model = BookMatch
        fields = "__all__"


class FlightPlanScheduleSlotSerializer(serializers.ModelSerializer):
    date = serializers.DateField(required=False)

    class Meta:
        model = Schedule_Slot
        fields = "__all__"
        read_only_fields = ['start_hour', 'end_hour',
                            'cost', 'schedule_id', 'day']


class FlightPlanWeeklyScheduleSlotSerializer(serializers.ModelSerializer):
    start_date = serializers.DateField(required=False)
    end_date = serializers.DateField(required=False)

    class Meta:
        model = Schedule_Slot
        fields = "__all__"
        read_only_fields = ['start_hour', 'end_hour',
                            'cost', 'schedule_id', 'day', 'date', 'start_date', 'end_date']


class FlightPlanMonthlyScheduleSlotSerializer(serializers.ModelSerializer):
    year = serializers.IntegerField(required=False, help_text="2021/2020...")
    month = serializers.IntegerField(required=False, help_text="01/02/03...")

    class Meta:
        model = Schedule_Slot
        fields = "__all__"
        read_only_fields = ['start_hour', 'end_hour',
                            'cost', 'schedule_id', 'day', 'date', 'month_year']


class FlightPlanSecondPopupSerializer(serializers.ModelSerializer):
    date = serializers.DateField(required=False)
    id = serializers.IntegerField(
        required=False, help_text="Id of the venue field")

    class Meta:
        model = Schedule_Slot
        fields = "__all__"
        read_only_fields = ['start_hour', 'end_hour',
                            'cost', 'schedule_id', 'day']


################### SCHEDULE ####################


class ScheduleGETSerializer(serializers.Serializer):
    Template_Id = serializers.IntegerField(required=False)
    Pricelist_Id = serializers.IntegerField(required=False)
    start_date = serializers.DateField(required=False)
    id = serializers.IntegerField(
        required=False, help_text="While editing Schedule")
    time_slot_price = serializers.JSONField(
        required=False, help_text='[{"start_hour":"13:00:00","end_hour":"15:00:00","cost":30}]')

    def validate_Template_Id(self, value):
        if ScheduleTemplateStep1.objects.filter(id=value, created_by=self.context['user']).exists():
            return value
        else:
            raise serializers.FieldDoesNotExist("No Template with ID")

    def validate_Pricelist_Id(self, value):
        if PriceListStep1.objects.filter(id=value).exists():
            return value
        else:
            raise serializers.FieldDoesNotExist("No Pricelist with ID")


class PriceList2ScheduleSerializer(serializers.ModelSerializer):
    created_by = serializers.CharField(write_only=True)
    id = serializers.IntegerField(required=False)
    day = serializers.CharField(source='day.dayofweek')

    class Meta:
        model = PriceListStep2
        fields = '__all__'


class ScheduleTemplateScheduleSerializer(serializers.ModelSerializer):
    class Meta:
        model = ScheduleTemplateStep1
        fields = '__all__'


class SchedulePOSTSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(required=False, help_text="Id if edit")
    slots = serializers.JSONField(
        required=True,
        help_text='[{"start_hour":"13:00:00","end_hour":"15:00:00","date":"2020-08-12","day":"Tuesday","is_available":"True","cost":30}]')
    confirmation = serializers.JSONField(
        required=False, help_text='[{"id":2},{"id":3}]')
    end_date = serializers.CharField(required=False, help_text='Do not send')
    time_slot_price = serializers.JSONField(
        required=False, help_text='[{"start_hour":"13:00:00","end_hour":"15:00:00","cost":30}]')

    @transaction.atomic
    def create(self, validated_data):
        print(validated_data)
        slots = validated_data['slots']
        validated_data.pop("slots")
        if 'time_slot_price' in validated_data:
            time_slots = validated_data['time_slot_price']
            validated_data.pop("time_slot_price")
        else:
            time_slots = []
        schedule = Schedule(**validated_data, created_by=self.context['user'])
        schedule.save()
        # Email Cron
        email_cron_obj = Schedule_Email_Cron.objects.create(
            schedule_id=schedule)
        email_cron_obj.schedule_end_date = schedule.end_date
        email_cron_obj.run_date = datetime.datetime.strptime(email_cron_obj.schedule_end_date, "%Y-%m-%d").date() - \
            datetime.timedelta(days=5)
        email_cron_obj.save()

        for i in slots:
            i['day'] = DayofWeek.objects.get(dayofweek=i['day'])

        slots = [Schedule_Slot(
            **i,
            schedule_id=schedule
        ) for i in slots]

        if len(time_slots) > 0:
            tslots = [Time_Slot_Price(**j, schedule_id=schedule)
                      for j in time_slots]
            Time_Slot_Price.objects.bulk_create(tslots)

        Schedule_Slot.objects.bulk_create(slots)
        return True

    class Meta:
        model = Schedule
        fields = "__all__"
        extra_kwargs = {
            "schedule_name": {
                "required": False
            },
            "field": {
                "required": False
            }
        }


# class SchedulePOSTSerializer(serializers.ModelSerializer):
#     id = serializers.IntegerField(required=False, help_text="Id if edit")
#     slots = serializers.JSONField(
#         required=True, help_text='[{"start_hour":"13:00:00","end_hour":"15:00:00","date":"2020-08-12","day":"Tuesday","is_available":"True","cost":30}]')
#     confirmation = serializers.JSONField(
#         required=False, help_text='[{"id":2},{"id":3}]')
#     end_date = serializers.DateField(required=False, help_text='end_date')

#     @transaction.atomic
#     def create(self, validated_data):
#         slots = validated_data['slots']
#         validated_data.pop("slots")
#         schedule = Schedule(**validated_data, created_by=self.context['user'])
#         schedule.save()
#         for i in slots:
#             i['day'] = DayofWeek.objects.get(dayofweek=i['day'])

#         slots = [Schedule_Slot(
#             **i,
#             schedule_id=schedule
#         ) for i in slots]

#         Schedule_Slot.objects.bulk_create(slots)
#         return True

#     class Meta:
#         model = Schedule
#         fields = "__all__"
#         extra_kwargs = {
#             "schedule_name": {
#                 "required": False
#             },
#             "field": {
#                 "required": False
#             }
#         }


class Schedule_SlotSerializer(serializers.ModelSerializer):
    class Meta:
        model = Schedule_Slot
        fields = "__all__"


class Schedule_SlotSummarySerializer(serializers.ModelSerializer):
    class Meta:
        model = Schedule_Slot
        fields = ['id', 'cost', 'start_hour', 'end_hour']


################### LIST VIEWS #####################################


class PricelistLISTSerializer(serializers.ModelSerializer):
    assortment = serializers.CharField(required=False, help_text="Field Name")
    page = serializers.IntegerField(
        required=False, default=1, help_text="Page number")
    keyword = serializers.CharField(
        required=False, write_only=True, help_text="Keyword")
    in_use = serializers.SerializerMethodField('is_in_use')
    filter_in_use = serializers.CharField(
        required=False, write_only=True, help_text="yes/no")

    def is_in_use(self, obj):
        if Schedule.objects.filter(pricelist__id=obj.id).exists():
            return "yes"
        else:
            return "no"

    class Meta:
        model = PriceListStep1
        fields = '__all__'
        read_only_fields = ['list_name', 'interval',
                            'gap_period', 'created_by', 'fields']


class PricelistDELETESerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(required=True)

    def validate_id(self, value):
        if PriceListStep1.objects.filter(id=value).exists():
            return value
        else:
            raise serializers.ImproperlyConfigured("No Record Found")

    class Meta:
        model = PriceListStep1
        fields = ['id', 'is_deleted']


class ScheduleTemplateLISTSerializer(serializers.ModelSerializer):
    #assortment = serializers.CharField(required=False, help_text="Field Name")
    page = serializers.IntegerField(
        required=False, default=1, help_text="Page number")
    keyword = serializers.CharField(
        required=False, write_only=True, help_text="Keyword")
    date_range = serializers.CharField(
        required=False, write_only=True, help_text="plan range")
    in_use = serializers.SerializerMethodField('is_in_use')
    filter_in_use = serializers.CharField(
        required=False, write_only=True, help_text="yes/no")

    def is_in_use(self, obj):
        if Schedule.objects.filter(template__id=obj.id).exists():
            return "yes"
        else:
            return "no"

    class Meta:
        model = ScheduleTemplateStep1
        fields = '__all__'
        read_only_fields = ['template_Name', 'interval',
                            'gap_period', 'plan_range', 'day', 'is_available']


class ScheduleLISTSerializer(serializers.ModelSerializer):
    assortment = serializers.CharField(
        required=False, help_text="start_date / ")
    start_date = serializers.CharField(
        required=False, help_text="starting date")
    active = serializers.CharField(
        required=False, help_text="Inuse - Yes / No")
    keyword = serializers.CharField(
        required=False, help_text="Search keywords")
    page = serializers.IntegerField(
        required=False, default=1, help_text="Page number")

    class Meta:
        model = Schedule
        fields = '__all__'
        read_only_fields = ['schedule_name', 'start_date', 'template',
                            'pricelist', 'start_date', 'end_date', 'created_by']


class ScheduleTemplateLISTDELETESerializer(serializers.Serializer):
    id = serializers.IntegerField(
        required=False, help_text="ID of the TemplateList")

    def validate_id(self, value):
        if ScheduleTemplateStep1.objects.filter(id=value, created_by=self.context['user']).exists():
            return value
        else:
            raise serializers.FieldDoesNotExist("No Record Found")

    @transaction.atomic
    def update(self, instance, validated_data):
        instance.is_deleted = True
        instance.save()
        ScheduleTemplateSlot.objects.filter(template=instance).delete()
        return instance


class ScheduleLISTDELETESerializer(serializers.Serializer):
    id = serializers.IntegerField(
        required=False, help_text="ID of the Schedule")

    def validate_id(self, value):
        if Schedule.objects.filter(id=value, created_by=self.context['user']).exists():
            return value
        else:
            raise serializers.FieldDoesNotExist("No Record Found")

    @transaction.atomic
    def update(self, instance, validated_data):
        instance.is_deleted = True
        instance.save()
        # ScheduleTemplateSlot.objects.filter(template=instance).delete()
        return instance


class ScheduleGetFieldSerializer(serializers.ModelSerializer):
    class Meta:
        model = VenueField
        fields = ['id', 'name']


class PricelistLISTWithoutPaginationSerializer(serializers.ModelSerializer):
    assortment = serializers.CharField(required=False, help_text="Field Name")
    keyword = serializers.CharField(
        required=False, write_only=True, help_text="Keyword")
    in_use = serializers.BooleanField(
        required=False, default=False, write_only=True, help_text="In Use")

    class Meta:
        model = PriceListStep1
        fields = '__all__'
        read_only_fields = ['list_name', 'interval',
                            'gap_period', 'created_by', 'fields']


class ScheduleTemplateLISTWithoutPaginationSerializer(serializers.ModelSerializer):
    assortment = serializers.CharField(required=False, help_text="Field Name")
    keyword = serializers.CharField(
        required=False, write_only=True, help_text="Keyword")

    class Meta:
        model = ScheduleTemplateStep1
        fields = '__all__'
        read_only_fields = ['template_Name', 'interval',
                            'gap_period', 'plan_range', 'day', 'is_available']


class VenueOwnerNewPOSTSerializer(serializers.ModelSerializer):
    state = serializers.IntegerField(
        required=False, help_text="state")
    area = serializers.IntegerField(
        required=False, help_text="area")
    city = serializers.IntegerField(
        required=False, help_text="city")
    country = serializers.IntegerField(
        required=False, help_text="country")
    street = serializers.CharField(
        required=False, help_text="VenueOwner street")
    pincode = serializers.CharField(
        required=False, help_text="VenueOwner pincode")
    venue_number = serializers.CharField(
        required=False, help_text="Venue number")
    timing = serializers.JSONField(
        required=False, help_text='[{"day":1, "start_time":12:00:00, "is_active":1, "end_time":1:00:00}]')

    # Venue Details
    logo = serializers.ImageField(required=False, help_text="Venue Logo")
    name = serializers.CharField(required=False, help_text="Venue Name")
    buisness_vat = serializers.CharField(
        required=False, help_text="Venue buisness vat")
    tax_offices = serializers.CharField(
        required=False, help_text="Venue buisness vat")
    business_name = serializers.CharField(required=False)

    def validate_timing(self, value):
        for items in value:
            if items['day']:
                if not (items['start_time'] and items['end_time']):
                    raise serializers.ValidationError(
                        "Please enter start time and end time")
                else:
                    return value

    @transaction.atomic
    def create(self, validated_data):

        validated_data['user'] = self.context['user'].id
        validated_data['email'] = self.context['user'].email
        venue = Venue.objects.filter(created_by=self.context['user']).first()
        venue.area = Area.objects.get(
            id=validated_data['area'])
        venue.city = City.objects.get(
            id=validated_data['city'])
        venue.state = State.objects.get(
            id=validated_data['state'])
        venue.country = Country.objects.get(
            id=validated_data['country'])

        venue.buisness_vat = validated_data['buisness_vat']
        venue.business_name = validated_data['business_name']
        venue.tax_offices = validated_data['tax_offices']
        venue.name = validated_data['name']
        if 'logo' in validated_data:
            venue.logo = validated_data['logo']

        venue.save()

        venueowner = VenueOwner.objects.filter(
            user=self.context['user']).first()
        venueowner.street = validated_data['street']
        venueowner.venue_number = validated_data['venue_number']
        venueowner.pincode = validated_data['pincode']
        venueowner.city = City.objects.get(
            id=validated_data['city'])
        venueowner.country = Country.objects.get(
            id=validated_data['country'])

        venueowner.save()

        if "timing" in validated_data:
            for i in validated_data['timing']:
                i['venue'] = venue.id
            timing_serializer = VenueDailyAvailibilitySerializer(
                data=validated_data['timing'], many=True)
            print(timing_serializer)
            if timing_serializer.is_valid():
                VenueDailyTiming.objects.filter(venue=venue.id).delete()
                timing_data = timing_serializer.save()

            else:
                print(timing_serializer.errors)
                raise serializers.ValidationError(
                    handle_errors(timing_serializer.errors))

        return venueowner.id

    class Meta:
        model = VenueOwner
        fields = ['city', 'street', 'country', 'pincode', 'venue_number', 'logo',
                  'buisness_vat', 'name', 'timing', 'tax_offices', 'business_name', 'state', 'area']
        extra_kwargs = {
            'email': {"read_only": True},
            'new_email': {"read_only": True},
            'user': {"read_only": True}
        }


class PackagesGetSerializer(serializers.ModelSerializer):
    sport_relivance = serializers.CharField(
        required=False, help_text="Sport ID")
    sport_name = serializers.CharField(
        required=False, read_only=True, source="sport.name")
    sport_id = serializers.CharField(
        required=False, read_only=True, source="sport_relivance.id")
    keyword = serializers.CharField(
        required=False, help_text="Search Text box")
    page = serializers.IntegerField(
        required=False, default=1, help_text="Page number. Default = 1")
    package_type = serializers.CharField(
        required=False, )
    package_type_id = serializers.CharField(
        required=False, read_only=True, source="package_type.id")
    package_duration = serializers.CharField(required=False)
    package_duration_id = serializers.CharField(
        required=False, read_only=True, source="package_duration.id")
    status_name = serializers.CharField(
        required=False, write_only=True, help_text="UNDER APPROVAL/ON GOING/WAITING/REJECTED")
    package_availabiity = serializers.NullBooleanField(
        required=False, help_text="Package Availability")

    class Meta:
        model = Packages
        fields = '__all__'
        read_only_fields = ['package_image', 'package_description',
                            'number_sessions', 'session_duration', 'package_quantity', 'level', 'price', 'name']


class PackagesPostSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(required=False)
    level = serializers.JSONField(required=False, help_text="Level IDs[1,2,3]")

    class Meta:
        model = Packages
        fields = '__all__'


class ScheduleEditGetSerializer(serializers.Serializer):
    id = serializers.IntegerField(required=True)


class ScheduleEditPOSTSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(required=True, help_text="Id if edit")
    slots = serializers.JSONField(
        required=True,
        help_text='[{"start_hour":"13:00:00","end_hour":"15:00:00","date":"2020-08-12","day":"Tuesday","is_available":"True","cost":30}]')

    @transaction.atomic
    def create(self, validated_data):
        slots = validated_data['slots']
        validated_data.pop("slots")
        schedule = Schedule.objects.get(id=validated_data['id'])
        '''schedule = Schedule(**validated_data, created_by=self.context['user'])
        schedule.save()'''
        Schedule_Slot.objects.filter(schedule_id=schedule).delete()
        for i in slots:
            i['day'] = DayofWeek.objects.get(dayofweek=i['day'])

        slots = [Schedule_Slot(
            **i,
            schedule_id=schedule
        ) for i in slots]

        Schedule_Slot.objects.bulk_create(slots)
        return True

    class Meta:
        model = Schedule
        fields = "__all__"


class ScheduleEditPOST2Serializer(serializers.ModelSerializer):
    id = serializers.IntegerField(required=True, help_text="Id if edit")
    slots = serializers.JSONField(
        required=True,
        help_text='[{"start_hour":"13:00:00","end_hour":"15:00:00","date":"2020-08-12","day":"Tuesday","is_available":"True","cost":30}]')
    time_slot_price = serializers.JSONField(
        required=False, help_text='[{"start_hour":"13:00:00","end_hour":"15:00:00","cost":30}]')

    def create(self, validated_data):
        print(validated_data)
        slots = validated_data['slots']
        validated_data.pop("slots")
        if 'time_slot_price' in validated_data:
            time_slots = validated_data['time_slot_price']
            validated_data.pop("time_slot_price")
        else:
            time_slots = []

        schedule = Schedule.objects.get(id=validated_data['id'])
        schedule.schedule_name = validated_data['schedule_name']
        schedule.template = validated_data['template']
        schedule.field = validated_data['field']
        schedule.start_date = validated_data['start_date']
        schedule.end_date = validated_data['end_date']
        schedule.save()
        # Email_cron
        email_cron_obj = Schedule_Email_Cron.objects.filter(
            schedule_id=schedule).first()
        print(email_cron_obj)
        email_cron_obj.schedule_end_date = schedule.end_date
        email_cron_obj.run_date = email_cron_obj.schedule_end_date - \
            datetime.timedelta(days=5)
        email_cron_obj.save()

        Schedule_Slot.objects.filter(schedule_id=schedule).delete()
        Time_Slot_Price.objects.filter(schedule_id=schedule).delete()
        for i in slots:
            i['day'] = DayofWeek.objects.get(dayofweek=i['day'])

        slots = [Schedule_Slot(
            **i,
            schedule_id=schedule
        ) for i in slots]

        Schedule_Slot.objects.bulk_create(slots)
        if len(time_slots) > 0:
            tslots = [Time_Slot_Price(**j, schedule_id=schedule)
                      for j in time_slots]
            Time_Slot_Price.objects.bulk_create(tslots)

        return True

    class Meta:
        model = Schedule
        fields = "__all__"


class AdminVenueApprovalSerializer(serializers.ModelSerializer):
    class Meta:
        model = Venue
        fields = ['id', 'name']


class CancellationReasonSerializer(serializers.ModelSerializer):
    class Meta:
        model = BookingCancellation
        fields = ['id', 'cancellation_reason']


class BookingCancellationVenueFieldSerializer(serializers.ModelSerializer):
    venue = AdminVenueApprovalSerializer(read_only=True)

    class Meta:
        model = VenueField
        fields = ['id', 'name', 'venue']


# class BookingCancellationRequestAllSerializer(serializers.ModelSerializer):
#     venue_field = BookingCancellationVenueFieldSerializer(read_only=True)
#     cancellation_reason = CancellationReasonSerializer(read_only=True)

#     class Meta:
#         model = RequestBookingCancellation
#         fields = "__all__"

class BookingCancellationRequestAllSerializer(serializers.ModelSerializer):
    venue_field = BookingCancellationVenueFieldSerializer(
        read_only=True, many=True)
    cancellation_reason = CancellationReasonSerializer(read_only=True)
    venue_name = serializers.SerializerMethodField('getVenueName')

    def getVenueName(self, obj):
        if Venue.objects.filter(created_by=obj.created_by).exists():
            venue_obj = Venue.objects.filter(created_by=obj.created_by).last()
            if venue_obj.name is not None:
                return venue_obj.name
            else:
                return "-"
        else:
            return "-"

    class Meta:
        model = RequestBookingCancellation
        fields = "__all__"


class BookingCancellationRequestPostSerializer(serializers.ModelSerializer):
    status = serializers.CharField(read_only=True)
    start_time = serializers.TimeField(required=True)
    end_time = serializers.TimeField(required=True)
    start_date = serializers.DateField(required=True)
    end_date = serializers.DateField(required=True)
    venue_field = serializers.JSONField(required=False, help_text="[1,2]")

    class Meta:
        model = RequestBookingCancellation
        fields = '__all__'
        read_only_fields = ['created_by']


class Schedule_ParameterGetSerializer(serializers.ModelSerializer):
    class Meta:
        model = Schedule_Parameter
        fields = '__all__'


class BookingCancellationFilterSerializer(serializers.Serializer):
    venue_field = serializers.JSONField(required=False, help_text="[1,2]")
    cancellation_reason = serializers.CharField(required=False)
    start_date = serializers.DateField(required=False)
    end_date = serializers.DateField(required=False)
    status = serializers.CharField(required=False, write_only=True)
    page = serializers.JSONField(
        required=False, default=1, help_text="Page number")


class Schedule_ParameterPostSerializer(serializers.ModelSerializer):

    def create(self, validated_data):
        data = Schedule_Parameter.objects.create(**validated_data)
        return data

    class Meta:
        model = Schedule_Parameter
        fields = '__all__'
