import pytz
import json
from django.http import HttpResponse
from django.template import Template, Context
from django.template.loader import get_template
from django.db import transaction
from django.urls import reverse

from django.contrib.auth import login as django_login
import datetime
from rest_framework.authtoken.models import Token
from rest_framework import status, generics
from rest_framework.response import Response
from rest_framework.authentication import TokenAuthentication
from rest_framework.permissions import IsAuthenticated
from utils.mixins import *
from utils.api_handle_errors import handle_errors
from static_data.models import DayofWeek
from venue_owners.serializers import *
import copy
import itertools 

############################ -- PriceListStep1 -- ###############################

class PriceListStep1GETAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = PriceListStep1GETSerializer

    def post(self,request,format=None):
        '''
        GET PriceList 1 value
        '''
        try:
            response = {}
            responsedata = []
            serializers = self.serializer_class(data=request.data)
            if serializers.is_valid():
                if 'id' in serializers.validated_data:
                    data = PriceListStep1.objects.filter(
                        id=serializers.validated_data['id'], is_deleted=False)
                else:
                    data = PriceListStep1.objects.filter(
                        created_by=request.user, is_deleted=False)
            else:
                response['result'] = 0
                response = handle_errors(serializers.errors)
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_400_BAD_REQUEST)
            serializer = self.serializer_class(data, many=True)
            response['result'] = 1
            response['data'] = serializer.data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        except Exception as e:
            response["result"] = 0
            responsedata.append(str(e))
            response['errors'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)




class PriceListStep1POSTAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = PriceListStep1POSTSerializer

    # Add PriceList Part1
    @transaction.atomic
    def post(self,request,format=None):
        '''
        PriceList Part1 Update/Create
        '''
        try:
            response = {}
            responsedata = []
            request.data._mutable = True
            request.data['created_by'] = User.objects.get(username=str(request.user)).id
            if (PriceListStep1.objects.filter(list_name=request.data['list_name'], interval=request.data['interval'] ,gap_period=request.data['gap_period'], start_hour=request.data['start_hour'], end_hour=request.data['end_hour']).exists()):
                raise Exception("Duplicate record found")
            if 'id' in request.data:
                if PriceListStep1.objects.filter(id=request.data['id'],is_deleted=False).exists():
                    # Check if schedule template is assigned
                    if Schedule.objects.filter(pricelist=request.data['id'], is_deleted=False).exists():
                        response["result"] = 0
                        responsedata.append("Price list is in use is in use")
                        response['errors'] = responsedata
                        self.add_api_call(request, response)
                        return Response(response, status=status.HTTP_200_OK)

                    serializers = self.serializer_class(instance=PriceListStep1.objects.filter(id=request.data['id'],is_deleted=False).first(),data= request.data)
                    message = "Successfully Updated"
                else:
                    raise Exception("No Such Record to update")
            else:
                serializers = self.serializer_class(data= request.data)
                message = 'Successfully Inserted'
            if serializers.is_valid():
                if serializers.validated_data['end_hour'] < serializers.validated_data['start_hour']:
                    response["result"] = 0
                    response["errors"] = ["Incorrect Timing"]
                    self.add_api_call(request, response)
                    return Response(response, status=status.HTTP_200_OK)
                    
                if "default_price" in  serializers.validated_data:
                    price = serializers.validated_data['default_price']
                    serializers.validated_data.pop("default_price")
                else:
                    price = 0

                save_data = serializers.save()

                #creating slots
                if "id" in serializers.validated_data:
                    PriceListStep2.objects.filter(price_list1=save_data.id).delete()

                data = []
                start_hour_list = []
                t = serializers.validated_data['start_hour']
                daysofweek = DayofWeek.objects.all()
                #Reducing the end time by 1 hour for perfect while loop

                serializers.validated_data['end_hour'] = (datetime.datetime.combine(datetime.date.today(), serializers.validated_data['end_hour']) - datetime.timedelta(minutes=serializers.validated_data['interval'])).time()
                
                while t <= serializers.validated_data['end_hour']:
                    start_hour_list.append(t)
                    dictdata = {"start_hour":t.strftime('%H:%M')}
                    t = (datetime.datetime.combine(datetime.date.today(), t) + datetime.timedelta(minutes=serializers.validated_data['interval'])).time()
                    dictdata['end_hour'] = t.strftime('%H:%M')
                    t = (datetime.datetime.combine(datetime.date.today(), t) + datetime.timedelta(minutes=serializers.validated_data['gap_period'])).time()
                    dictdata['created_by'] = serializers.validated_data['created_by']
                    dictdata['created_by'] = serializers.validated_data['created_by']
                    dictdata['priceList1ID'] = save_data.id

                    objs = [
                        PriceListStep2(
                            price_list1=PriceListStep1.objects.get(id=dictdata['priceList1ID']),
                            start_hour= dictdata['start_hour'],
                            end_hour=dictdata['end_hour'],
                            created_by=dictdata['created_by'],
                            price= price,
                            day=i
                        )
                        for i in daysofweek
                    ]

                    PriceListStep2.objects.bulk_create(objs)
                
                # Day of week object
                day_of_week_obj = DayofWeek.objects.all()
                # print(day_of_week_obj)

                response_data = []
                
                for i in start_hour_list: 
                    single_hour_dict = {}
                    for j in day_of_week_obj:
                        PriceListStep2IDs = PriceListStep2.objects.get(price_list1=save_data.id, day=j.id,start_hour=i)
                        pricelist2_serializer = PriceListStep2PartSerializer(PriceListStep2IDs)
                        single_hour_dict[j.dayofweek] = pricelist2_serializer.data

                    response_data.append(single_hour_dict)


                # PriceListStep2IDs = PriceListStep2.objects.filter(price_list1=save_data.id)
                # pricelist2_serializer = PriceListStep2PartSerializer(PriceListStep2IDs,many=True)


                response["result"] = 1
                responsedata.append(message)
                response['errors'] = responsedata
                response['id'] = save_data.id
                response['PriceList2ID'] = response_data
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)
            else:
                response = handle_errors(serializers.errors)
                response['result'] = 0
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)

        except Exception as e:
            response["result"] = 0
            responsedata.append(str(e))
            response['errors'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


#################### PriceList 2##############################3



class PriceListStep2GETAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = PriceListStep2SerializerGET

    def post(self,request,format=None):
        '''
        GET PriceList 2 value
        '''
        try:
            response = {}
            responsedata = []
            serializers = self.serializer_class(data=request.data)
            if serializers.is_valid():
                if 'id' in serializers.validated_data:
                    data = PriceListStep2.objects.filter(
                        id=serializers.validated_data['id'], is_deleted=False)
                else:
                    data = PriceListStep2.objects.filter(
                        created_by=request.user, is_deleted=False)
            else:
                response['result'] = 0
                response = handle_errors(serializers.errors)
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_400_BAD_REQUEST)
            serializer = self.serializer_class(data, many=True)
            response['result'] = 1
            response['data'] = serializer.data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        except Exception as e:
            response["result"] = 0
            responsedata.append(str(e))
            response['errors'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class PriceListStep2POSTAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = PriceListStep2BridgeSerializer
    # Add PriceList Part1
    def post(self,request,format=None):
        '''
        PriceList Part2 Profile Update/Create
        '''
        try:
            response = {}
            responsedata = []
            request.data._mutable = True
            serializers = self.serializer_class(data= request.data)
            message = 'Successfully Updated'
            if serializers.is_valid():
                if Schedule.objects.filter(pricelist=serializers.validated_data['priceList1ID'], is_deleted=False).exists():
                    response["result"] = 0
                    response['errors'] = "Price list is currently active in schedule"
                    self.add_api_call(request, response)
                    return Response(response, status=status.HTTP_200_OK)

                save_data = serializers.save()
                response["result"] = 1
                responsedata.append(message)
                response['errors'] = responsedata
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)
            else:
                response = handle_errors(serializers.errors)
                response['result'] = 0
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)
        except Exception as e:
            response["result"] = 0
            responsedata.append(str(e))
            response['errors'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


############### PART3 ###########################



class PriceListStep3POSTAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = PriceListStep3POSTSerializer

    # Add PriceList Part1
    def post(self,request,format=None):
        '''
        PriceList Part3 Profile Update/Create
        '''
        try:
            response = {}
            responsedata = []
            request.data._mutable = True
            request.data['created_by'] = User.objects.get(username=str(request.user)).id
            if PriceListStep1.objects.filter(id=request.data['id'],is_deleted=False).exists():
                serializers = self.serializer_class(instance=PriceListStep1.objects.filter(id=request.data['id'],is_deleted=False).first(),data= request.data)
                message = "Successfully Updated"
            else:
                raise Exception("No Such Record to update")
            if serializers.is_valid():
                save_data = serializers.save()
                response["result"] = 1
                responsedata.append(message)
                response['errors'] = responsedata
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)
            else:
                response = handle_errors(serializers.errors)
                response['result'] = 0
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)

        except Exception as e:
            response["result"] = 0
            responsedata.append(str(e))
            response['errors'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


############################ Global Get ##############################

class PriceListGlobalGetAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes      = (TokenAuthentication,)
    permission_classes          = (IsAuthenticated,)
    serializer_class            = PriceListGlobalGetSerializer
    ListStep1Serializer_class   = PriceListStep1GETSerializer
    ListStep2Serializer_class   = PriceListStep2SerializerGET
    def post(self,request,format=None):
        try:
            response = {}
            responsedata = []
            GlobalData = []
            serializers =  self.serializer_class(data=request.data)
            if serializers.is_valid():
                if 'id' in serializers.validated_data:
                    data_part1 = PriceListStep1.objects.filter(
                        id=serializers.validated_data['id'], is_deleted=False)
                    data_part2 = PriceListStep2.objects.filter(
                        price_list1=serializers.validated_data['id'], is_deleted=False)
                else:
                    data_part1 = PriceListStep1.objects.filter(
                        created_by=request.user, is_deleted=False)
                    data_part2 = PriceListStep2.objects.filter(
                        created_by=request.user, is_deleted=False)
            else:
                response = handle_errors(serializers.errors)
                response['result'] = 0
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_400_BAD_REQUEST)
            serialized_data1 = self.ListStep1Serializer_class(data_part1,many=True)
            serialized_data2 = self.ListStep2Serializer_class(data_part2,many=True)
            GlobalData.append({"PriceListStep1":serialized_data1.data,"PriceListStep2":serialized_data2.data})
            response['data'] = GlobalData
            response['result'] = 1
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        except Exception as e:
            response["result"] = 0
            responsedata.append(str(e))
            response['errors'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)