import pytz
import json
from django.http import HttpResponse
from django.template import Template, Context
from django.template.loader import get_template
from django.db import transaction
from django.urls import reverse
from django.db.models import Q
from django.contrib.auth import login as django_login
import datetime
import calendar
from rest_framework.authtoken.models import Token
from rest_framework import status, generics
from rest_framework.response import Response
from rest_framework.authentication import TokenAuthentication
from rest_framework.permissions import IsAuthenticated
from utils.mixins import *
from utils.api_handle_errors import handle_errors
from static_data.models import DayofWeek
from venue_owners.serializers import *
from players.models import *
from players.serializers import *
from venues.models import *


class ScheduleGETTableAPIView(PlatformAPICallMixin, generics.GenericAPIView):

    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = ScheduleGETSerializer
    templateSerializer_class = ScheduleTemplateScheduleSerializer
    SlotSerializer_class = ScheduleTemplateSlotsSerialzier
    pricelistSerializer_class = PriceList2ScheduleSerializer
    schedule_slots_class = ScheduleSlotSerializer

    def post(self, request, format=None):
        try:
            response = {}
            responsedata = []
            serializers = self.serializer_class(
                data=request.data, context={"user": request.user})
            if serializers.is_valid():

                if "id" in serializers.validated_data:
                    schedule_obj = Schedule.objects.get(
                        id=serializers.validated_data['id'])
                    scheudle_slots = Schedule_Slot.objects.filter(
                        schedule_id=schedule_obj, is_deleted=False)
                    template_obj = schedule_obj.template

                    start_hour = template_obj.start_hour
                    end_hour = template_obj.end_hour

                    plan_range = 1 if template_obj.plan_range == "D" else 7 if template_obj.plan_range == "W" else 31

                    response_data = []
                    token_1 = 0
                    while start_hour < end_hour:

                        data = {}
                        token = 0
                        start_date = schedule_obj.start_date
                        end_date = schedule_obj.end_date
                        break_down = 0

                        while start_date < end_date:
                            if (datetime.datetime.combine(datetime.date.today(), start_hour) + datetime.timedelta(minutes=template_obj.interval) + datetime.timedelta(minutes=template_obj.gap_period)).time() > end_hour:
                                break
                            scheudle_slots = Schedule_Slot.objects.filter(
                                schedule_id=schedule_obj.id, date=start_date, start_hour=start_hour, is_deleted=False).first()
                            scheudle_slots_class = self.schedule_slots_class(
                                scheudle_slots)
                            data[token] = scheudle_slots_class.data
                            start_date = start_date + \
                                datetime.timedelta(days=1)
                            token = token + 1

                            if break_down > 500:
                                break
                            else:
                                break_down = break_down + 1

                        start_hour = (datetime.datetime.combine(datetime.date.today(), start_hour) + datetime.timedelta(
                            minutes=template_obj.interval) + datetime.timedelta(minutes=template_obj.gap_period)).time()
                        if len(data) == 0:
                            pass
                        else:
                            response_data.append(data)
                        token_1 = token_1 + 1

                    response['results'] = 1
                    response['data'] = response_data
                    response['plan_range'] = plan_range
                    # response['schedule_data'] = scheudle_slots_class.data
                    self.add_api_call(request, response)
                    return Response(response, status=status.HTTP_200_OK)

                time_slots_price = serializers.validated_data['time_slot_price']
                template = ScheduleTemplateStep1.objects.get(
                    id=serializers.validated_data['Template_Id'], created_by=request.user)
                plan_range = 1 if template.plan_range == "D" else 7 if template.plan_range == "W" else 31
                start_date = serializers.validated_data['start_date']
                end_date = start_date + datetime.timedelta(days=plan_range)

                start_hour = template.start_hour
                end_hour = template.end_hour

                datalist = []
                break_down = 0

                if plan_range == 7:
                    date_day_array = []
                    for d in range(0, 7):
                        wd = {}
                        wd['date'] = start_date+datetime.timedelta(days=d)
                        wd['day'] = calendar.day_name[wd['date'].weekday()]
                        date_day_array.append(wd)
                    workinghours = VenueDailyTiming.objects.filter(
                        venue__created_by=request.user)
                    '''days_array=[]
                    for w in range(0,workinghours.count()):
                        days_array.append(workinghours[w].day)

                    print(days_array)'''
                    print(date_day_array)
                    slots = ScheduleTemplateSlot.objects.filter(
                        template=template.id)
                    slotsSerializer = self.SlotSerializer_class(
                        slots, many=True)
                    if len(slotsSerializer.data) > 0:
                        for index, i in enumerate(slotsSerializer.data):
                            data = {}
                            dayname = ""
                            if i['day'] == 0:
                                dayname = "Monday"
                            if i['day'] == 1:
                                dayname = "Tuesday"
                            if i['day'] == 2:
                                dayname = "Wednesday"
                            if i['day'] == 3:
                                dayname = "Thursday"
                            if i['day'] == 4:
                                dayname = "Friday"
                            if i['day'] == 5:
                                dayname = "Saturday"
                            if i['day'] == 6:
                                dayname = "Sunday"

                            for rec in date_day_array:
                                if rec['day'] == dayname:
                                    current_date = rec['date']

                            data['date'] = current_date
                            data['day'] = dayname
                            data['is_available'] = i['is_available']
                            data['start_hour'] = i['start_hour']
                            data['end_hour'] = i['end_hour']
                            which_has_max_weightage = []
                            for cnt in range(0, len(time_slots_price)):
                                sh = datetime.datetime.strptime(
                                    data['start_hour'], '%H:%M:%S')
                                eh = datetime.datetime.strptime(
                                    data['end_hour'], '%H:%M:%S')

                                dt = {}
                                count_of_weight = 0
                                while sh <= eh:
                                    if datetime.datetime.strptime(time_slots_price[cnt]['start_hour'], '%H:%M:%S') < sh and sh < datetime.datetime.strptime(time_slots_price[cnt]['end_hour'], '%H:%M:%S'):
                                        count_of_weight = count_of_weight+1
                                    sh = sh+datetime.timedelta(seconds=60)
                                dt["weight"] = count_of_weight
                                dt["slot_sh"] = data["start_hour"]
                                dt["slot_eh"] = data["end_hour"]
                                dt["duration"] = time_slots_price[cnt]
                                which_has_max_weightage.append(dt)

                            # print(which_has_max_weightage)
                            winner = 0
                            for z in range(0, len(which_has_max_weightage)):
                                if which_has_max_weightage[z]['weight'] > which_has_max_weightage[winner]['weight']:
                                    winner = z
                            if which_has_max_weightage[winner]['weight'] == 0:
                                data['cost'] = 0
                            else:
                                data['cost'] = time_slots_price[winner]['cost']
                            datalist.append(data)

                else:
                    while start_hour < end_hour:

                        if (datetime.datetime.combine(datetime.date.today(), start_hour) + datetime.timedelta(minutes=template.interval)).time() > end_hour:
                            break

                        slots = ScheduleTemplateSlot.objects.filter(
                            template=template.id, start_hour=start_hour)
                        slotsSerializer = self.SlotSerializer_class(
                            slots, many=True)

                        if len(slotsSerializer.data) == 0:
                            # start_hour = slots[0].end_hour + datetime.timedelta(minutes=template.gap_period)
                            break

                        token = 0
                        main_obj = {}

                        for index, i in enumerate(slotsSerializer.data):
                            data = {}
                            current_date = start_date + \
                                datetime.timedelta(days=i['day'])
                            data['date'] = current_date
                            data['day'] = calendar.day_name[current_date.weekday()]
                            data['is_available'] = i['is_available']
                            data['start_hour'] = i['start_hour']
                            data['end_hour'] = i['end_hour']
                            which_has_max_weightage = []
                            for cnt in range(0, len(time_slots_price)):
                                sh = datetime.datetime.strptime(
                                    data['start_hour'], '%H:%M:%S')
                                eh = datetime.datetime.strptime(
                                    data['end_hour'], '%H:%M:%S')

                                dt = {}
                                count_of_weight = 0
                                while sh <= eh:
                                    if datetime.datetime.strptime(time_slots_price[cnt]['start_hour'], '%H:%M:%S') < sh and sh < datetime.datetime.strptime(time_slots_price[cnt]['end_hour'], '%H:%M:%S'):
                                        count_of_weight = count_of_weight+1
                                    sh = sh+datetime.timedelta(seconds=60)
                                dt["weight"] = count_of_weight
                                dt["slot_sh"] = data["start_hour"]
                                dt["slot_eh"] = data["end_hour"]
                                dt["duration"] = time_slots_price[cnt]
                                which_has_max_weightage.append(dt)

                            # print(which_has_max_weightage)
                            winner = 0
                            for z in range(0, len(which_has_max_weightage)):
                                if which_has_max_weightage[z]['weight'] > which_has_max_weightage[winner]['weight']:
                                    winner = z
                            if which_has_max_weightage[winner]['weight'] == 0:
                                data['cost'] = 0
                            else:
                                data['cost'] = time_slots_price[winner]['cost']
                            main_obj[token] = data
                            token = token + 1

                        datalist.append(main_obj)
                        start_hour = (datetime.datetime.combine(datetime.date.today(
                        ), slots[0].end_hour) + datetime.timedelta(minutes=template.gap_period)).time()

                        if break_down > 500:
                            break
                        else:
                            break_down = break_down + 1

                responsedata.append(datalist)
                response['results'] = 1
                response['data'] = datalist
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)
            else:
                response = handle_errors(serializers.errors)
                response['result'] = 0
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_400_BAD_REQUEST)
        except Exception as e:
            response["result"] = 0
            responsedata.append(str(e))
            response['errors'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class SchedulePOSTTableAPIView(PlatformAPICallMixin, generics.GenericAPIView):

    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = SchedulePOSTSerializer

    SlotSerializer_class = Schedule_SlotSerializer
    templateSerializer_class = ScheduleTemplateScheduleSerializer
    pricelistSerializer_class = PriceList2ScheduleSerializer

    def post(self, request, format=None):
        try:
            response = {}
            responsedata = []
            serializers = self.serializer_class(
                data=request.data, context={"user": request.user})
            if serializers.is_valid():
                if "id" in serializers.validated_data:
                    # Check if id exists for slot update
                    if Schedule.objects.filter(id=serializers.validated_data['id'], is_deleted=False).exists():
                        schedule_obj = Schedule.objects.get(
                            id=serializers.validated_data['id'], is_deleted=False)
                        # UPdate Slots
                        schedule_slot_obj = Schedule_Slot.objects.filter(
                            schedule_id=schedule_obj, is_deleted=False).update(is_deleted=True)

                        for i in serializers.validated_data['slots']:
                            i['day'] = DayofWeek.objects.get(
                                dayofweek=(i['day']))

                        slots = [Schedule_Slot(
                            **i,
                            schedule_id=schedule_obj
                        ) for i in serializers.validated_data['slots']]

                        Schedule_Slot.objects.bulk_create(slots)

                        response['result'] = 1
                        response['errors'] = ["Successfully Updated"]
                        self.add_api_call(request, response)
                        return Response(response, status=status.HTTP_200_OK)

                    else:
                        response['result'] = 0
                        response['errors'] = ["No Such Schedule Exists"]
                        self.add_api_call(request, response)
                        return Response(response, status=status.HTTP_200_OK)

                else:
                    if "schedule_name" not in serializers.validated_data or "field" not in serializers.validated_data:
                        response['result'] = 0
                        response['errors'] = [
                            "No Schedule Name and/or Field sent"]
                        self.add_api_call(request, response)
                        return Response(response, status=status.HTTP_200_OK)

                if 'confirmation' in serializers.validated_data:
                    for i in serializers.validated_data['confirmation']:
                        delete_slot = Schedule_Slot.objects.get(id=i['id'])
                        delete_slot.is_deleted = True
                        delete_slot.save()

                days = 1 if serializers.validated_data[
                    'template'].plan_range == "D" else 7 if serializers.validated_data['template'].plan_range == "W" else 31

                end_date = serializers.validated_data['end_date']
                schedule = Schedule.objects.filter(Q(start_date__range=(serializers.validated_data['start_date'], end_date), created_by=request.user, is_deleted=False, field=serializers.validated_data['field']) | Q(
                    end_date__range=(serializers.validated_data['start_date'], end_date), created_by=request.user, is_deleted=False, field=serializers.validated_data['field']))
                if (len(schedule) > 0):
                    reserved_slots = []
                    ignore_slots = []
                    for i in serializers.validated_data['slots']:
                        for j in schedule:
                            # scheduleSlot = Schedule_Slot.objects.filter(date = i['date'],start_hour__lte=i['start_hour'],schedule_id=j,is_deleted=False).first()
                            scheduleSlot = Schedule_Slot.objects.filter(Q(start_hour=i['start_hour'], end_hour=i['end_hour'], schedule_id=j, is_deleted=False) | Q(
                                start_hour__lte=i['start_hour'], end_hour__gt=i['start_hour'], schedule_id=j, is_deleted=False) | Q(start_hour__lt=i['end_hour'], end_hour__gt=i['end_hour'], schedule_id=j, is_deleted=False)).first()
                            slot_serializer = self.SlotSerializer_class(
                                scheduleSlot)
                            if scheduleSlot is not None:
                                if 'confirmation' in serializers.validated_data:
                                    ignore_slots.append(slot_serializer.data)
                                else:
                                    reserved_slots.append(slot_serializer.data)
                            else:
                                serializers.validated_data['end_date'] = end_date
                                serializers.save()
                                response['result'] = 1
                                response['errors'] = ["Successful"]
                                self.add_api_call(request, response)
                                return Response(response, status=status.HTTP_200_OK)

                    if 'confirmation' in serializers.validated_data:
                        for i in ignore_slots:
                            serializers.validated_data['slots'] = [x for x in serializers.validated_data['slots'] if str(x['start_hour']) != str(
                                i['start_hour']) or str(x['end_hour']) != str(i['end_hour']) or str(x['date']) != str(i['date'])]
                        serializers.validated_data.pop('confirmation')
                        serializers.save()
                        response['result'] = 1
                        response['errors'] = ["Successful"]
                        self.add_api_call(request, response)
                        return Response(response, status=status.HTTP_200_OK)
                    else:
                        response['result'] = 2
                        response['message'] = ["Time slot already reserved"]
                        response['errors'] = reserved_slots
                        self.add_api_call(request, response)
                        return Response(response, status=status.HTTP_200_OK)
                else:
                    serializers.validated_data['end_date'] = end_date
                    serializers.save()
                    response['result'] = 1
                    response['errors'] = ["Successful"]
                    self.add_api_call(request, response)
                    return Response(response, status=status.HTTP_200_OK)
            else:
                response = handle_errors(serializers.errors)
                response['result'] = 0
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)
        except Exception as e:
            response["result"] = 0
            responsedata.append(str(e))
            response['errors'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class GetFieldsAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = ScheduleGetFieldSerializer

    def get(self, request, format=None):
        response = {}
        responsedata = []

        # get user
        user_obj = request.user

        # Get fields of user
        venue_obj = Venue.objects.filter(created_by=user_obj).last()
        # fields = VenueField.objects.filter(created_by=user_obj, venue=venue_obj, is_deleted=False)
        fields = VenueField.objects.filter(
            created_by=user_obj, is_deleted=False)
        fields_class = self.serializer_class(fields, many=True)

        response['result'] = 1
        response['data'] = fields_class.data
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


class FlightPlanAPIView(PlatformAPICallMixin, generics.GenericAPIView):

    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = FlightPlanScheduleSlotSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []
        responsebookeddata = []
        responseavailabledata = []

        serializers = self.serializer_class(
            data=request.data, context={"user": request.user})
        if serializers.is_valid():

            if Schedule_Slot.objects.filter(date=serializers.validated_data['date'], schedule_id__created_by=request.user).exists():

                available_schedule_slots = Schedule_Slot.objects.filter(
                    date=serializers.validated_data['date'], schedule_id__created_by=request.user, is_available=True, is_deleted=False)
                serializer = self.serializer_class(
                    available_schedule_slots, many=True)
                print(available_schedule_slots)

                unavailable_schedule_slots = Schedule_Slot.objects.filter(
                    date=serializers.validated_data['date'], schedule_id__created_by=request.user, is_available=False, is_deleted=False)
                serializer1 = self.serializer_class(
                    unavailable_schedule_slots, many=True)
                print(unavailable_schedule_slots)

                book = 0
                av = 0
                booked = {}
                available = {}
                booked_list = []
                available_list = []

                for index, i in enumerate(serializer.data):
                    data = {}
                    # print(i)
                    if BookMatch.objects.filter(schedule_slot=i['id']).exists():
                        bookmatch_cost = BookMatch.objects.get(
                            schedule_slot=i['id'])
                        print('id', bookmatch_cost.id)
                        data['status'] = "B"
                        data['id'] = i['id']
                        data['schedule_id'] = i['schedule_id']
                        data['start_hour'] = i['start_hour']
                        data['end_hour'] = i['end_hour']
                        data['cost'] = i['cost']
                        book = book + bookmatch_cost.total_cost
                        booked['cost'] = book

                        field_obj = Schedule.objects.get(id=i['schedule_id'])
                        print(field_obj)
                        # print(field_obj.field.name)
                        data['field_id'] = field_obj.field.id
                        data['field_name'] = field_obj.field.name

                        if field_obj.field.id not in booked_list:
                            booked_list.append(field_obj.field.id)
                            booked['field_count'] = len(booked_list)

                        data['Campaign'] = "No"

                        field_paid_service_list = []
                        if VenueFieldServiceQuantity.objects.filter(book_match=bookmatch_cost.id, book_match__schedule_slot=i['id'], ).exists():
                            fieldServiceObj = VenueFieldServiceQuantity.objects.filter(
                                book_match=bookmatch_cost.id, book_match__schedule_slot=i['id'], )
                            print('field', fieldServiceObj)

                            for l in fieldServiceObj:

                                services_obj = {}
                                service_id = l.venue_field_service.venue_service.id
                                service_name = l.venue_field_service.venue_service.name
                                services_obj['service_id'] = service_id
                                services_obj['service_name'] = service_name

                                field_paid_service_list.append(
                                    services_obj)

                                print(field_paid_service_list)
                                data['paid_services'] = field_paid_service_list
                        else:
                            data['paid_services'] = []

                        if VenuePaidServiceQuantity.objects.filter(book_match=bookmatch_cost.id, book_match__schedule_slot=i['id'], ).exists():
                            venueServiceObj = VenuePaidServiceQuantity.objects.filter(
                                book_match=bookmatch_cost.id, book_match__schedule_slot=i['id'], )
                            print('venue', venueServiceObj)

                            venue_paid_service_list = []
                            for l in venueServiceObj:

                                services_obj = {}
                                service_id = l.venue_service_cost.venue_service.id
                                service_name = l.venue_service_cost.venue_service.name
                                services_obj['service_id'] = service_id
                                services_obj['service_name'] = service_name

                                venue_paid_service_list.append(
                                    services_obj)

                                print(venue_paid_service_list)
                                data['paid_services'] = field_paid_service_list + \
                                    venue_paid_service_list
                        else:
                            data['paid_services'] = [] + \
                                field_paid_service_list

                    else:
                        data['status'] = "A"
                        data['id'] = i['id']
                        data['schedule_id'] = i['schedule_id']
                        data['start_hour'] = i['start_hour']
                        data['end_hour'] = i['end_hour']
                        data['cost'] = i['cost']
                        av = av + i['cost']
                        available['cost'] = av

                        field_obj = Schedule.objects.get(id=i['schedule_id'])
                        data['field_id'] = field_obj.field.id
                        data['field_name'] = field_obj.field.name

                        if field_obj.field.id not in available_list:
                            available_list.append(field_obj.field.id)
                            available['field_count'] = len(available_list)

                        data['paid_services'] = []
                        data['Campaign'] = "No"
                    responsedata.append(data)
                # print(book)
                # print(booked)
                responsebookeddata.append(booked)
                responseavailabledata.append(available)

                for index, i in enumerate(serializer1.data):
                    data = {}
                    data['status'] = "U"
                    data['id'] = i['id']
                    data['schedule_id'] = i['schedule_id']
                    data['start_hour'] = i['start_hour']
                    data['end_hour'] = i['end_hour']
                    data['cost'] = i['cost']
                    field_obj = Schedule.objects.get(id=i['schedule_id'])
                    data['field_id'] = field_obj.field.id
                    data['field_name'] = field_obj.field.name
                    data['paid_services'] = []
                    data['Campaign'] = "No"
                    responsedata.append(data)

                response["result"] = 1
                response['datas'] = responsedata
                response['booked'] = responsebookeddata
                response['available'] = responseavailabledata
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)

            else:
                response['result'] = 0
                response['errors'] = ["No record available"]
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)

        else:
            response = handle_errors(serializers.errors)
            response['result'] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class FlightPlanPopUpAPIView(PlatformAPICallMixin, generics.GenericAPIView):

    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = FlightPlanPopUpSerializer
    bookmatch_class = FlightPlanBookMatchSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []
        serializers = self.serializer_class(
            data=request.data, context={"user": request.user})
        if serializers.is_valid():

            if Schedule_Slot.objects.filter(id=serializers.validated_data['id'], schedule_id__created_by=request.user, is_available=True).exists():

                schedule_slots_id = serializers.validated_data['id']

                if BookMatch.objects.filter(schedule_slot=schedule_slots_id).exists():
                    bookmatch_obj = BookMatch.objects.get(
                        schedule_slot=schedule_slots_id)

                    data = {}
                    data['name'] = bookmatch_obj.created_by.first_name + " " +\
                        bookmatch_obj.created_by.last_name
                    data['mobile'] = bookmatch_obj.created_by.mobile
                    data['email'] = bookmatch_obj.created_by.user.username
                    data['field_name'] = bookmatch_obj.venue_field.name
                    print(bookmatch_obj.schedule_slot.filter(
                        id=schedule_slots_id))
                    slot_obj = bookmatch_obj.schedule_slot.get(
                        id=schedule_slots_id)
                    data['date'] = slot_obj.date
                    # data['start_hour'] = slot_obj.start_hour
                    # data['end_hour'] = slot_obj.end_hour
                    #data['services_cost'] = bookmatch_obj.services_cost

                    Schedule_slot_obj = bookmatch_obj.schedule_slot.filter()
                    print(Schedule_slot_obj)

                    first = Schedule_slot_obj.first()
                    last = Schedule_slot_obj.last()
                    start_hour = first.start_hour
                    end_hour = last.end_hour
                    data['start_hour'] = start_hour
                    data['end_hour'] = end_hour

                    # Duration
                    value = 0
                    duration = datetime.timedelta(minutes=value)
                    for i in Schedule_slot_obj:
                        end = i.end_hour
                        beginning = i.start_hour

                        difference = datetime.datetime.combine(
                            date.min, end) - datetime.datetime.combine(date.min, beginning)
                        print('diff', difference)
                        duration = duration + difference
                        print('dur', duration)

                    total_seconds = duration.total_seconds()
                    minutes = total_seconds/60
                    hr = minutes/60
                    print('hr', hr)

                    data['duration'] = hr

                    # Paid Services
                    venue_paid_service_obj = VenuePaidServiceQuantity.objects.filter(
                        book_match=bookmatch_obj)
                    paid_serialized_data = VenuePaidServiceQuantityerializer(
                        venue_paid_service_obj, many=True)

                    venue_field_service_obj = VenueFieldServiceQuantity.objects.filter(
                        book_match=bookmatch_obj)
                    field_serialized_data = VenueFieldServiceQuantitySerializer(
                        venue_field_service_obj, many=True)

                    # Total Cost Schedule
                    totalcost = 0
                    value = 0
                    cost = value
                    for i in Schedule_slot_obj:
                        schedule_cost = i.cost
                        cost = cost + schedule_cost
                    print('cost', cost)

                    # Total Cost Paid Service
                    paid_service_list = []
                    val = 0
                    p_cost = val
                    for i in venue_paid_service_obj:
                        paid_obj = {}

                        paid_service_cost = i.venue_service_cost.price_per_hour
                        paid_service_name = i.venue_service_cost.venue_service.name

                        quantity = i.quantity
                        paid_service_cost_calc = (
                            paid_service_cost*quantity)*hr

                        paid_obj['venue_service_name'] = paid_service_name
                        paid_obj['venue_service_cost'] = paid_service_cost_calc

                        paid_service_list.append(paid_obj)

                        p_cost = p_cost + (paid_service_cost*quantity)*hr
                        print('pcost', p_cost)

                    data['venue_services'] = paid_service_list

                    # Total Cost Field Service
                    field_service_list = []
                    fval = 0
                    f_cost = fval
                    for j in venue_field_service_obj:
                        field_obj = {}

                        field_service_cost = j.venue_field_service.price_per_hour
                        field_service_name = j.venue_field_service.venue_service.name
                        quantity = j.quantity

                        field_service_cost_calc = (
                            field_service_cost*quantity)*hr

                        field_obj['field_service_name'] = field_service_name
                        field_obj['field_service_cost'] = field_service_cost_calc

                        field_service_list.append(field_obj)

                        f_cost = f_cost + (field_service_cost*quantity)*hr

                    data['field_services'] = field_service_list

                    print('fcost', f_cost)

                    totalcost = totalcost + cost + p_cost + f_cost

                    data['schedule_cost'] = cost
                    # data['venueService_cost'] = p_cost
                    # data['fieldService_cost'] = f_cost
                    data['total_cost'] = totalcost

                    responsedata.append(data)
                else:
                    response['result'] = 0
                    response['errors'] = ["No record available in bookmatch"]
                    self.add_api_call(request, response)
                    return Response(response, status=status.HTTP_200_OK)

                response["result"] = 1
                response['data'] = responsedata
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)
            else:
                response['result'] = 0
                response['errors'] = ["No record available"]
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializers.errors)
            response['result'] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class FlightPlanWeeklyAPIView(PlatformAPICallMixin, generics.GenericAPIView):

    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = FlightPlanWeeklyScheduleSlotSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []
        responsebookeddata = []
        responseavailabledata = []

        serializers = self.serializer_class(
            data=request.data, context={"user": request.user})
        if serializers.is_valid():

            if Schedule_Slot.objects.filter(date__range=[serializers.validated_data['start_date'], serializers.validated_data['end_date']], schedule_id__created_by=request.user).exists():

                available_schedule_slots = Schedule_Slot.objects.filter(
                    date__range=[serializers.validated_data['start_date'], serializers.validated_data['end_date']], schedule_id__created_by=request.user, is_available=True, is_deleted=False)
                serializer = self.serializer_class(
                    available_schedule_slots, many=True)

                unavailable_schedule_slots = Schedule_Slot.objects.filter(
                    date__range=[serializers.validated_data['start_date'], serializers.validated_data['end_date']], schedule_id__created_by=request.user, is_available=False, is_deleted=False)
                serializer1 = self.serializer_class(
                    unavailable_schedule_slots, many=True)

                book = 0
                av = 0
                booked = {}
                available = {}
                booked_list = []
                available_list = []

                for index, i in enumerate(serializer.data):
                    data = {}
                    # print(i)
                    if BookMatch.objects.filter(schedule_slot=i['id']).exists():
                        bookmatch_cost = BookMatch.objects.get(
                            schedule_slot=i['id'])
                        # print(bookmatch_cost.total_cost)
                        data['status'] = "B"
                        data['id'] = i['id']
                        data['schedule_id'] = i['schedule_id']
                        data['date'] = i['date']
                        data['start_hour'] = i['start_hour']
                        data['end_hour'] = i['end_hour']
                        data['cost'] = i['cost']
                        book = book + bookmatch_cost.total_cost
                        booked['cost'] = book

                        field_obj = Schedule.objects.get(id=i['schedule_id'])
                        print(field_obj)
                        data['field_id'] = field_obj.field.id
                        data['field_name'] = field_obj.field.name

                        if field_obj.field.id not in booked_list:
                            booked_list.append(field_obj.field.id)
                            booked['field_count'] = len(booked_list)

                        data['Campaign'] = "No"
                        field_paid_service_list = []
                        if VenueFieldServiceQuantity.objects.filter(book_match=bookmatch_cost.id, book_match__schedule_slot=i['id'], ).exists():
                            fieldServiceObj = VenueFieldServiceQuantity.objects.filter(
                                book_match=bookmatch_cost.id, book_match__schedule_slot=i['id'], )
                            print('field', fieldServiceObj)

                            for l in fieldServiceObj:

                                services_obj = {}
                                service_id = l.venue_field_service.venue_service.id
                                service_name = l.venue_field_service.venue_service.name
                                services_obj['service_id'] = service_id
                                services_obj['service_name'] = service_name

                                field_paid_service_list.append(
                                    services_obj)

                                print(field_paid_service_list)
                                data['paid_services'] = field_paid_service_list
                        else:
                            data['paid_services'] = []

                        if VenuePaidServiceQuantity.objects.filter(book_match=bookmatch_cost.id, book_match__schedule_slot=i['id'], ).exists():
                            venueServiceObj = VenuePaidServiceQuantity.objects.filter(
                                book_match=bookmatch_cost.id, book_match__schedule_slot=i['id'], )
                            print('venue', venueServiceObj)

                            venue_paid_service_list = []
                            for l in venueServiceObj:

                                services_obj = {}
                                service_id = l.venue_service_cost.venue_service.id
                                service_name = l.venue_service_cost.venue_service.name
                                services_obj['service_id'] = service_id
                                services_obj['service_name'] = service_name

                                venue_paid_service_list.append(
                                    services_obj)

                                print(venue_paid_service_list)
                                data['paid_services'] = field_paid_service_list + \
                                    venue_paid_service_list
                        else:
                            data['paid_services'] = [] + \
                                field_paid_service_list

                    else:
                        data['status'] = "A"
                        data['id'] = i['id']
                        data['schedule_id'] = i['schedule_id']
                        data['date'] = i['date']
                        data['start_hour'] = i['start_hour']
                        data['end_hour'] = i['end_hour']
                        data['cost'] = i['cost']
                        av = av + i['cost']
                        available['cost'] = av

                        field_obj = Schedule.objects.get(id=i['schedule_id'])
                        data['field_id'] = field_obj.field.id
                        data['field_name'] = field_obj.field.name

                        if field_obj.field.id not in available_list:
                            available_list.append(field_obj.field.id)
                            available['field_count'] = len(available_list)

                        data['paid_services'] = []
                        data['Campaign'] = "No"
                    responsedata.append(data)
                # print(book)
                # print(booked)
                responsebookeddata.append(booked)
                responseavailabledata.append(available)

                for index, i in enumerate(serializer1.data):
                    data = {}
                    data['status'] = "U"
                    data['id'] = i['id']
                    data['schedule_id'] = i['schedule_id']
                    data['date'] = i['date']
                    data['start_hour'] = i['start_hour']
                    data['end_hour'] = i['end_hour']
                    data['cost'] = i['cost']
                    field_obj = Schedule.objects.get(id=i['schedule_id'])
                    data['field_id'] = field_obj.field.id
                    data['field_name'] = field_obj.field.name
                    data['paid_services'] = []
                    data['Campaign'] = "No"
                    responsedata.append(data)

                response["result"] = 1
                response['datas'] = responsedata
                response['booked'] = responsebookeddata
                response['available'] = responseavailabledata
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)

            else:
                response['result'] = 0
                response['errors'] = ["No record available"]
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)

        else:
            response = handle_errors(serializers.errors)
            response['result'] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class GetSchedule1(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = ScheduleEditGetSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []
        user_obj = request.user
        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            if Schedule.objects.filter(id=serializers.validated_data['id'], created_by=user_obj, is_deleted=False).exists():
                data = []
                schedule_data = Schedule.objects.get(
                    id=serializers.validated_data['id'])
                start_date = schedule_data.start_date
                end_date = schedule_data.end_date
                current_date = start_date
                while current_date <= end_date:
                    inner_data = {}
                    slots_for_date = Schedule_Slot.objects.filter(
                        schedule_id=schedule_data.id, date=current_date, is_deleted=False)
                    if slots_for_date.count() > 0:
                        inner_data['date_of_day'] = current_date.strftime(
                            "%d/%m/%Y")
                        inner_data['day_name'] = calendar.day_name[current_date.weekday()]
                        slots_list = []
                        for i in range(0, slots_for_date.count()):
                            obj = {}
                            obj['date'] = slots_for_date[i].date
                            obj['day'] = slots_for_date[i].day.dayofweek
                            obj['start_hour'] = slots_for_date[i].start_hour
                            obj['end_hour'] = slots_for_date[i].end_hour
                            obj['cost'] = slots_for_date[i].cost
                            obj['is_available'] = slots_for_date[i].is_available
                            slots_list.append(obj)
                        inner_data['slots'] = slots_list
                    else:
                        inner_data['date_of_day'] = current_date.strftime(
                            "%d/%m/%Y")
                        inner_data['day_name'] = calendar.day_name[current_date.weekday()]
                        inner_data['slots'] = []

                    data.append(inner_data)
                    current_date = current_date+datetime.timedelta(days=1)

                time_slots_prices = Time_Slot_Price.objects.filter(
                    schedule_id=schedule_data)
                sl_list = []
                for z in range(0, time_slots_prices.count()):
                    o = {}
                    o['start_hour'] = time_slots_prices[z].start_hour
                    o['end_hour'] = time_slots_prices[z].end_hour
                    o['cost'] = time_slots_prices[z].cost
                    sl_list.append(o)

                dd = (datetime.datetime.now(datetime.timezone.utc) -
                      schedule_data.created_at).days
                schedule_details = {}
                schedule_details['name'] = schedule_data.schedule_name
                schedule_details['field'] = schedule_data.field.id
                schedule_details['field_name'] = schedule_data.field.name
                schedule_details['schedule_template'] = schedule_data.template.id
                schedule_details['schedule_template_name'] = schedule_data.template.template_name
                schedule_details['start_date'] = schedule_data.start_date.strftime(
                    "%d-%m-%Y")
                schedule_details['end_date'] = schedule_data.end_date.strftime(
                    "%d-%m-%Y")
                schedule_details['days_from_creation'] = dd
                # schedule_details['days_from_creation']=11
                schedule_details['plan_range'] = schedule_data.template.plan_range
                schedule_details['prices'] = sl_list
                response['result'] = 1
                response['data'] = data
                l = []
                l.append(schedule_details)
                print(l)
                print(schedule_details)
                response['schedule_details'] = l
                # response['pricelist']=pricelist

                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)
            else:
                response['result'] = 0
                response['error'] = ["No such schedule exist"]
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)

        else:
            response['result'] = 0
            response['error'] = ["Schedule Id is not valid"]
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class EditScheduleOnlyHours(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = ScheduleEditPOSTSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []
        user_obj = request.user
        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            serializers.save()
            response['result'] = 1
            response['message'] = ["Schedule Edited Successfully"]
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response['result'] = 0
            response['error'] = ["Schedule Id is not valid"]
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class EditScheduleAll(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = ScheduleEditPOST2Serializer

    def post(self, request, format=None):
        response = {}
        responsedata = []
        user_obj = request.user
        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            if Schedule.objects.filter(id=serializers.validated_data['id'], created_by=user_obj).exists():
                schedule = Schedule.objects.filter(Q(start_date__range=(serializers.validated_data['start_date'], serializers.validated_data['end_date']), created_by=request.user, is_deleted=False, field=serializers.validated_data['field']) | Q(
                    end_date__range=(serializers.validated_data['start_date'], serializers.validated_data['end_date']), created_by=request.user, is_deleted=False, field=serializers.validated_data['field']))
                if (len(schedule) > 1):
                    response['result'] = 0
                    response['message'] = [
                        "Some schedule is already applied on the selected field for the particular date durations"]
                    self.add_api_call(request, response)
                    return Response(response, status=status.HTTP_200_OK)
                elif (len(schedule) == 1):
                    if schedule[0].id == serializers.validated_data['id']:
                        serializers.save()
                        response['result'] = 1
                        response['message'] = ["Successfully Edited schedule"]
                        self.add_api_call(request, response)
                        return Response(response, status=status.HTTP_200_OK)
                    else:
                        response['result'] = 0
                        response['message'] = [
                            "Some schedule is already applied on the selected field for the particular date durations"]
                        self.add_api_call(request, response)
                        return Response(response, status=status.HTTP_200_OK)

                else:
                    serializers.save()
                    response['result'] = 1
                    response['message'] = ["Successfully Edited schedule"]
                    self.add_api_call(request, response)
                    return Response(response, status=status.HTTP_200_OK)
            else:
                response['result'] = 0
                response['message'] = ["No such schedule found"]
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)
        else:
            response['result'] = 0
            response['message'] = ["Schedule Id is not valid"]
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class FlightPlanMonthlyAPIView(PlatformAPICallMixin, generics.GenericAPIView):

    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = FlightPlanMonthlyScheduleSlotSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []
        responsemonthlydata = []
        responsebookeddata = []
        responseavailabledata = []
        responsecoveragedata = []

        serializers = self.serializer_class(
            data=request.data, context={"user": request.user})
        if serializers.is_valid():

            if Schedule_Slot.objects.filter(date__year=serializers.validated_data['year'], date__month=serializers.validated_data['month'], schedule_id__created_by=request.user).exists():

                d_fmt = "{0:>02}.{1:>02}.{2}"
                date_from = datetime.datetime.strptime(
                    d_fmt.format(1, serializers.validated_data['month'], serializers.validated_data['year']), '%d.%m.%Y').date()
                last_day_of_month = calendar.monthrange(
                    serializers.validated_data['year'], serializers.validated_data['month'])[1]
                date_to = datetime.datetime.strptime(
                    d_fmt.format(last_day_of_month, serializers.validated_data['month'], serializers.validated_data['year']), '%d.%m.%Y').date()

                datewise_data = date_to - date_from
                # print(datewise_data.days)

                book = 0
                av = 0
                booked = {}
                available = {}
                coverage_rate = {}
                booked_list = []
                available_list = []
                total_booked_slot = 0
                total_available_slot = 0
                total_unavailable_slot = 0

                for i in range(datewise_data.days + 1):

                    date_obj = date_from + datetime.timedelta(days=i)
                    # print(date_obj)

                    if VenueField.objects.filter(created_by=request.user).exists():
                        field_obj = VenueField.objects.filter(
                            created_by=request.user)

                        # print(field_obj)
                        for j in range(field_obj.count()):
                            monthly_data = {}

                            bookmatch_slots = BookMatch.objects.filter(
                                schedule_slot__date=date_obj, venue_field=field_obj[j])

                            datewise_available_schedule_slots = Schedule_Slot.objects.filter(
                                date=date_obj,  schedule_id__field=field_obj[j], is_available=True, is_deleted=False)

                            datewise_unavailable_schedule_slots = Schedule_Slot.objects.filter(
                                date=date_obj,  schedule_id__field=field_obj[j], is_available=False, is_deleted=False)

                            if bookmatch_slots.count() > 0 or datewise_available_schedule_slots.count() > 0 or datewise_unavailable_schedule_slots.count() > 0:

                                if bookmatch_slots.count() > 0:
                                    monthly_data['booked'] = "Yes"
                                    monthly_data['field_id'] = field_obj[j].id
                                    monthly_data['field_name'] = field_obj[j].name
                                    monthly_data['date'] = date_obj
                                    monthly_data['campaign'] = "No"
                                    monthly_data['booked_slots'] = bookmatch_slots.count(
                                    )
                                    total_booked_slot = total_booked_slot + bookmatch_slots.count()

                                    if field_obj[j].id not in booked_list:
                                        booked_list.append(field_obj[j].id)
                                        booked['field_count'] = len(
                                            booked_list)

                                    # To get bookmatch details
                                    bookmatch_cost = BookMatch.objects.filter(
                                        schedule_slot__date=date_obj, venue_field=field_obj[j])
                                    print(bookmatch_cost)

                                    for k in bookmatch_cost:
                                        book = book + k.total_cost
                                        bookmatch_id = k.id

                                    # To get available slot details
                                    datewise_available_schedule_slots_to_get_service = Schedule_Slot.objects.filter(
                                        date=date_obj,  schedule_id__field=field_obj[j], is_available=True, is_deleted=False)

                                    for m in datewise_available_schedule_slots_to_get_service:
                                        slot_id = m.id
                                        print(slot_id)

                                    booked['cost'] = book
                                    field_paid_service_list = []
                                    if VenueFieldServiceQuantity.objects.filter(book_match=bookmatch_id, book_match__schedule_slot=slot_id, ).exists():
                                        fieldServiceObj = VenueFieldServiceQuantity.objects.filter(
                                            book_match=bookmatch_id, book_match__schedule_slot=slot_id, )
                                        print('field', fieldServiceObj)

                                        for l in fieldServiceObj:

                                            services_obj = {}
                                            service_id = l.venue_field_service.venue_service.id
                                            service_name = l.venue_field_service.venue_service.name
                                            services_obj['service_id'] = service_id
                                            services_obj['service_name'] = service_name

                                            field_paid_service_list.append(
                                                services_obj)

                                            print(field_paid_service_list)
                                            monthly_data['paid_services'] = field_paid_service_list
                                        else:
                                            monthly_data['paid_services'] = []

                                    if VenuePaidServiceQuantity.objects.filter(book_match=bookmatch_id, book_match__schedule_slot=slot_id, ).exists():
                                        venueServiceObj = VenuePaidServiceQuantity.objects.filter(
                                            book_match=bookmatch_id, book_match__schedule_slot=slot_id, )
                                        print('venue', venueServiceObj)

                                        venue_paid_service_list = []
                                        for l in venueServiceObj:

                                            services_obj = {}
                                            service_id = l.venue_service_cost.venue_service.id
                                            service_name = l.venue_service_cost.venue_service.name
                                            services_obj['service_id'] = service_id
                                            services_obj['service_name'] = service_name

                                            venue_paid_service_list.append(
                                                services_obj)

                                            print(venue_paid_service_list)
                                            monthly_data['paid_services'] = field_paid_service_list + \
                                                venue_paid_service_list
                                    else:
                                        monthly_data['paid_services'] = [] + \
                                            field_paid_service_list

                                else:
                                    monthly_data['booked'] = "No"
                                    monthly_data['field_id'] = field_obj[j].id
                                    monthly_data['field_name'] = field_obj[j].name
                                    monthly_data['date'] = date_obj
                                    monthly_data['campaign'] = "No"
                                    monthly_data['booked_slots'] = 0
                                    monthly_data['paid_services'] = []

                                if datewise_available_schedule_slots.count() > 0:
                                    monthly_data['available'] = "Yes"
                                    monthly_data['field_id'] = field_obj[j].id
                                    monthly_data['field_name'] = field_obj[j].name
                                    monthly_data['date'] = date_obj
                                    monthly_data['campaign'] = "No"
                                    monthly_data['available_slots'] = datewise_available_schedule_slots.count(

                                    )

                                    total_available_slot = total_available_slot + \
                                        datewise_available_schedule_slots.count()

                                    if field_obj[j].id not in available_list:
                                        available_list.append(field_obj[j].id)
                                        available['field_count'] = len(
                                            available_list)

                                    schedule_cost = Schedule_Slot.objects.filter(
                                        date=date_obj, schedule_id__field=field_obj[j])

                                    for l in schedule_cost:
                                        av = av + l.cost

                                    available['cost'] = av

                                else:
                                    monthly_data['available'] = "No"
                                    monthly_data['field_id'] = field_obj[j].id
                                    monthly_data['field_name'] = field_obj[j].name
                                    monthly_data['date'] = date_obj
                                    monthly_data['campaign'] = "No"
                                    monthly_data['available_slots'] = 0

                                if datewise_unavailable_schedule_slots.count() > 0:
                                    monthly_data['unavailable'] = "Yes"
                                    monthly_data['field_id'] = field_obj[j].id
                                    monthly_data['field_name'] = field_obj[j].name
                                    monthly_data['date'] = date_obj
                                    monthly_data['campaign'] = "No"
                                    monthly_data['unavailable_slots'] = datewise_unavailable_schedule_slots.count(
                                    )

                                    total_unavailable_slot = total_unavailable_slot + \
                                        datewise_unavailable_schedule_slots.count()

                                else:
                                    monthly_data['unavailable'] = "No"
                                    monthly_data['field_id'] = field_obj[j].id
                                    monthly_data['field_name'] = field_obj[j].name
                                    monthly_data['date'] = date_obj
                                    monthly_data['campaign'] = "No"
                                    monthly_data['unavailable_slots'] = 0

                                responsemonthlydata.append(monthly_data)

                    else:
                        response['result'] = 0
                        response['errors'] = ["No record available"]
                        self.add_api_call(request, response)
                        return Response(response, status=status.HTTP_200_OK)
                # print(available)
                # print(total_available_slot)
                # print(total_unavailable_slot)
                # print(total_booked_slot)
                total_slot_av_uav = total_available_slot + total_unavailable_slot

                percentage_of_booked_slot = (
                    total_booked_slot/total_slot_av_uav)*100

                # print(percentage_of_booked_slot)
                coverage_rate['coverage_rate'] = percentage_of_booked_slot

                responseavailabledata.append(available)
                responsebookeddata.append(booked)
                responsecoveragedata.append(coverage_rate)

                response["result"] = 1
                response['data'] = responsemonthlydata
                response['booked'] = responsebookeddata
                response['available'] = responseavailabledata
                response['coverage'] = responsecoveragedata
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)

            else:
                response['result'] = 0
                response['errors'] = ["No record available"]
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)

        else:
            response = handle_errors(serializers.errors)
            response['result'] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class FlightPlanSecondPopupAPIView(PlatformAPICallMixin, generics.GenericAPIView):

    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = FlightPlanSecondPopupSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []

        serializers = self.serializer_class(
            data=request.data, context={"user": request.user})
        if serializers.is_valid():

            if Schedule_Slot.objects.filter(date=serializers.validated_data['date'], schedule_id__field=serializers.validated_data['id'], schedule_id__created_by=request.user).exists():

                available_schedule_slots = Schedule_Slot.objects.filter(
                    date=serializers.validated_data['date'], schedule_id__field=serializers.validated_data['id'], schedule_id__created_by=request.user, is_available=True, is_deleted=False)
                serializer = self.serializer_class(
                    available_schedule_slots, many=True)

                unavailable_schedule_slots = Schedule_Slot.objects.filter(
                    date=serializers.validated_data['date'], schedule_id__field=serializers.validated_data['id'], schedule_id__created_by=request.user, is_available=False, is_deleted=False)
                serializer1 = self.serializer_class(
                    unavailable_schedule_slots, many=True)

                for index, i in enumerate(serializer.data):
                    data = {}
                    # print(i)
                    field_obj = Schedule.objects.get(id=i['schedule_id'])

                    if BookMatch.objects.filter(schedule_slot__id=i['id'], venue_field=field_obj.field).exists():

                        data['status'] = "B"
                        data['id'] = i['id']
                        data['schedule_id'] = i['schedule_id']
                        data['start_hour'] = i['start_hour']
                        data['end_hour'] = i['end_hour']
                        data['cost'] = i['cost']
                        data['date'] = i['date']

                        #field_obj = Schedule.objects.get(id=i['schedule_id'])
                        # print(field_obj)
                        data['field_id'] = field_obj.field.id
                        data['field_name'] = field_obj.field.name

                        bookmatch_cost = BookMatch.objects.filter(
                            schedule_slot__id=i['id'], venue_field=field_obj.field)
                        print(bookmatch_cost)

                        for k in bookmatch_cost:
                            bookmatch_id = k.id

                        service_list = []
                        if VenueFieldServiceQuantity.objects.filter(book_match=bookmatch_id, book_match__schedule_slot=i['id'], ).exists():
                            fieldServiceObj = VenueFieldServiceQuantity.objects.filter(
                                book_match=bookmatch_id, book_match__schedule_slot=i['id'], )
                            print('field', fieldServiceObj)

                            for m in fieldServiceObj:
                                service_list.append(m.venue_field_service)

                        if VenuePaidServiceQuantity.objects.filter(book_match=bookmatch_id, book_match__schedule_slot=i['id'], ).exists():
                            venueServiceObj = VenuePaidServiceQuantity.objects.filter(
                                book_match=bookmatch_id, book_match__schedule_slot=i['id'], )
                            print('venue', venueServiceObj)

                            for k in venueServiceObj:
                                service_list.append(k.venue_service_cost)
                        print(service_list)

                        if len(service_list) > 0:
                            data['service'] = True
                        else:
                            data['service'] = False

                    else:
                        data['status'] = "A"
                        data['id'] = i['id']
                        data['schedule_id'] = i['schedule_id']
                        data['start_hour'] = i['start_hour']
                        data['end_hour'] = i['end_hour']
                        data['cost'] = i['cost']
                        data['date'] = i['date']

                        #field_obj = Schedule.objects.get(id=i['schedule_id'])
                        data['field_id'] = field_obj.field.id
                        data['field_name'] = field_obj.field.name
                        data['service'] = False

                    responsedata.append(data)

                for index, i in enumerate(serializer1.data):
                    data = {}

                    data['status'] = "U"
                    data['id'] = i['id']
                    data['schedule_id'] = i['schedule_id']
                    data['start_hour'] = i['start_hour']
                    data['end_hour'] = i['end_hour']
                    data['cost'] = i['cost']
                    data['date'] = i['date']

                    field_obj = Schedule.objects.get(id=i['schedule_id'])
                    data['field_id'] = field_obj.field.id
                    data['field_name'] = field_obj.field.name
                    data['service'] = False

                    responsedata.append(data)

                response["result"] = 1
                response['data'] = responsedata
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)

            else:
                response['result'] = 0
                response['errors'] = ["No record available"]
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)

        else:
            response = handle_errors(serializers.errors)
            response['result'] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class Schedule_ParameterGetAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    """
    Schedule_Parameters
    """
    serializer_class = Schedule_ParameterGetSerializer

    def get(self, request, format=None):
        response = {}
        responsedata = []
        data = Schedule_Parameter.objects.all()
        serializer = self.serializer_class(data, many=True)
        response['result'] = 1
        response['data'] = serializer.data
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


class Schedule_ParameterPostAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = Schedule_ParameterPostSerializer

    # Add Schedule Parameter
    def post(self, request, format=None):
        response = {}
        responsedata = []

        if Schedule_Parameter.objects.filter().exists():
            parameter_object = Schedule_Parameter.objects.filter()
            print(parameter_object)
            parameter_object.delete()

            serializers = self.serializer_class(data=request.data)
            message = 'Successfully Updated'
        else:
            serializers = self.serializer_class(data=request.data)
            message = 'Successfully Inserted'

        if serializers.is_valid():
            save_data = serializers.save()
            response["result"] = 1
            responsedata.append(message)
            response['data'] = responsedata
            response['id'] = save_data.id
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializers.errors)
            response['result'] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
