import pytz
import json
from django.http import HttpResponse
from django.template import Template, Context
from django.template.loader import get_template
from django.db import transaction
from django.urls import reverse

from django.contrib.auth import login as django_login
import datetime
from rest_framework.authtoken.models import Token
from rest_framework import status, generics
from rest_framework.response import Response
from rest_framework.authentication import TokenAuthentication
from rest_framework.permissions import IsAuthenticated
from utils.mixins import *
from utils.api_handle_errors import handle_errors
from static_data.models import DayofWeek
from venue_owners.serializers import *
from venue_owners.models import *


class PlanRangeGETAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    # authentication_classes = (TokenAuthentication,)
    # permission_classes = (IsAuthenticated,)
    serializer_class = PlanRangeGETSerializer

    def post(self, request, format=None):
        '''
        GEt Plan Range
        '''
        try:
            response = {}
            responsedata = []
            serializers = self.serializer_class(data=request.data)
            if serializers.is_valid():
                if serializers.validated_data['end_hour'] < serializers.validated_data['start_hour']:
                    response["result"] = 0
                    response["errors"] = ["Incorrect Timing"]
                    self.add_api_call(request, response)
                    return Response(response, status=status.HTTP_200_OK)

                plan_range = serializers.validated_data['plan_range']
                days = 1 if plan_range == 'D' else 7 if plan_range == "W" else 31
                start_time = serializers.validated_data['start_hour']
                # end_time = (datetime.datetime.combine(datetime.date.today(), serializers.validated_data['end_time']) - datetime.timedelta(minutes=60)).time()
                end_time = serializers.validated_data['end_hour']

                hours = []
                break_down = 0
                while start_time < end_time:
                    data = {"start_hour": start_time.strftime('%H:%M')}
                    start_time = (datetime.datetime.combine(datetime.date.today(
                    ), start_time) + datetime.timedelta(minutes=serializers.validated_data['interval'])).time()
                    data['end_hour'] = start_time.strftime('%H:%M')
                    start_time = (datetime.datetime.combine(datetime.date.today(), start_time) + datetime.timedelta(
                        minutes=serializers.validated_data['gap'])).time()          # gap period
                    if start_time > end_time:
                        pass
                    else:
                        hours.append(data)

                    if break_down > 500:
                        response['result'] = 1
                        response['data'] = ['Infinite Loop Detected']
                        self.add_api_call(request, response)
                        return Response(response, status=status.HTTP_200_OK)
                    else:
                        break_down = break_down + 1

                days_list = []

                for i in hours:
                    day_obj = {}

                    for j in range(days):
                        day_obj[j] = i

                    days_list.append(day_obj)

                # [days_list.append({i:hours}) for i in range(days)]

                response['result'] = 1
                response['data'] = days_list
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)
            else:
                response['result'] = 0
                response = handle_errors(serializers.errors)
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_400_BAD_REQUEST)
        except Exception as e:
            response["result"] = 0
            responsedata.append(str(e))
            response['errors'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class ScheduleTemplate1APIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = ScheduleTemplateStep1POSTSerializer

    def post(self, request, format=None):
        '''
        Add Schedule Template
        '''
        try:
            response = {}
            responsedata = []
            serializers = self.serializer_class(
                data=request.data, context={"user": request.user})
            if serializers.is_valid():
                serializers.save()
                response["result"] = 1
                responsedata.append("Successfully Inserted")
                response['errors'] = responsedata
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)
            else:
                response = handle_errors(serializers.errors)
                response["result"] = 0
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_400_BAD_REQUEST)
        except Exception as e:
            response["result"] = 0
            responsedata.append(str(e))
            response['errors'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class ScheduleTemplate1GETAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = ScheduleTemplate1GETSerializer
    slots_serializer = ScheduleTemplateSlotsSerialzier

    def post(self, request, format=None):
        '''
        Get Schedule Template
        '''
        try:
            response = {}
            responsedata = []
            user = request.user
            serializers = self.serializer_class(
                data=request.data, context={"user": user})
            if serializers.is_valid():

                template_data = ScheduleTemplateStep1.objects.get(
                    **serializers.validated_data, created_by=request.user)

                days = 1 if template_data.plan_range == 'D' else 7 if template_data.plan_range == "W" else 31
                serializers = self.serializer_class(template_data, many=False)
                slots_start_times = ScheduleTemplateSlot.objects.filter(
                    template=template_data).values_list('start_hour').distinct()

                template_slots = []

                if len(serializers.data) == 0:
                    template_slots = []
                else:
                    if template_data.plan_range == "W":
                        print("w")
                        slots = ScheduleTemplateSlot.objects.filter(
                            template=template_data)
                        slotSerializer = self.slots_serializer(
                            slots, many=True)
                        sdata = slotSerializer.data
                        print(sdata)
                        template_slots = sdata
                    else:
                        for k in slots_start_times:
                            temp_data = {}
                            break_down = 0

                            for i in range(days):
                                slots = ScheduleTemplateSlot.objects.get(
                                    template=template_data, day=i, start_hour=k[0].strftime('%H:%M:%S'))
                                slotSerializer = self.slots_serializer(
                                    slots, many=False)
                                temp_data[i] = slotSerializer.data
                                template_slots.append(temp_data)

                                if break_down > 500:
                                    break
                                else:
                                    break_down = break_down + 1

                response["result"] = 1
                responsedata.append({"ScheduleTemplate1": serializers.data})
                responsedata.append({"ScheduleTemplateSlot": template_slots})
                response['data'] = responsedata
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)
            else:
                response = handle_errors(serializers.errors)
                response["result"] = 0
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_400_BAD_REQUEST)
        except Exception as e:
            response["result"] = 0
            responsedata.append(str(e))
            response['errors'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class ScheduleTemplate1UPDATEAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = ScheduleTemplateStep1UPDATESerializer

    @transaction.atomic
    def post(self, request, format=None):
        '''
        For update send entire record again
        '''
        try:
            response = {}
            responsedata = []
            serializers = self.serializer_class(instance=ScheduleTemplateStep1.objects.get(
                id=request.data['id']), data=request.data, context={"user": request.user})
            if serializers.is_valid():

                # Check if schedule template is assigned
                if Schedule.objects.filter(template=serializers.validated_data["id"], is_deleted=False).exists():
                    response["result"] = 0
                    responsedata.append("Schedule Template is in use")
                    response['errors'] = responsedata
                    self.add_api_call(request, response)
                    return Response(response, status=status.HTTP_200_OK)

                # Append Slots
                slots = serializers.validated_data['slots']
                serializers.validated_data.pop("slots")

                template_obj = serializers.save()

                # Slots
                ScheduleTemplateSlot.objects.filter(
                    template=template_obj).delete()
                data = [ScheduleTemplateSlot(
                    **i,
                    template=template_obj
                ) for i in slots]
                ScheduleTemplateSlot.objects.bulk_create(data)

                response["result"] = 1
                responsedata.append("Successfully Updated")
                response['errors'] = responsedata
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)
            else:
                response = handle_errors(serializers.errors)
                response["result"] = 0
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_400_BAD_REQUEST)
        except Exception as e:
            response["result"] = 0
            responsedata.append(str(e))
            response['errors'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)
