import pytz
import json
from django.http import HttpResponse
from django.template import Template, Context
from django.template.loader import get_template
from django.db import transaction
from django.urls import reverse
from django.contrib.contenttypes.models import ContentType

from django.contrib.auth import login as django_login
import datetime
from rest_framework.authtoken.models import Token
from rest_framework import status, generics
from rest_framework.response import Response
from rest_framework.authentication import TokenAuthentication
from rest_framework.permissions import IsAuthenticated
from utils.mixins import *
from utils.api_handle_errors import handle_errors
from venue_owners.serializers import *
from venues.serializers import *
from venue_owners.models import *
from commonAPI.serializers import ContactPersonSerializer
from commonAPI.models import Contact_Person

############################ -- GET -- ###############################
from admin_app.serializers import VenueOwnerVenueFieldSerializer

from static_data.models import BookingCancellation

from utils.mail import onboard_Complete_mail

from utils.mail import venue_owner_booking_cancel_Complete_mail


class VenueOwnerGetAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = VenueOwnerGetSerializer

    def get(self, request, format=None):
        '''
        Get Venue owners
        '''
        try:
            response = {}
            errors = []
            data = Venue.objects.filter(created_by=request.user)
            OwnerInfo = VenueOwner.objects.filter(
                user=request.user, is_deleted=False)
            serializer = self.serializer_class(OwnerInfo, many=True)

            content_type = ContentType.objects.get_for_model(VenueOwner)
            working_hours = VenueDailyTiming.objects.filter(venue_id__in=data)
            print(working_hours)

            working_hours_serializer = VenueDailyAvailibilitySerializer(
                working_hours, many=True)
            contact_person = Contact_Person.objects.filter(
                content_type__pk=content_type.id, object_id__in=OwnerInfo)
            contact_person_serializer = ContactPersonSerializer(
                contact_person, many=True)

            responsedata = [{"contact_person": contact_person_serializer.data}, {
                "working_hours": working_hours_serializer.data}]
            responsedata = serializer.data + responsedata

            response['result'] = 1
            response['data'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        except Exception as e:
            response["result"] = 0
            errors.append(str(e))
            response['errors'] = errors
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class VenueOwnerPOSTAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = VenueOwnerPOSTSerializer

    def post(self, request, format=None):
        '''
        Create New Venue Owner
        '''
        try:
            response = {}
            errors = []
            request.data._mutable = True
            serializers = self.serializer_class(
                data=request.data, context={"user": request.user})
            if serializers.is_valid():
                data = serializers.save()
                response["result"] = 1
                errors.append("Successfully Inserted")
                response["errors"] = errors
                response['ID'] = data
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)
            else:
                response["result"] = 1
                response = handle_errors(serializers.errors)
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_400_BAD_REQUEST)
        except Exception as e:
            response["result"] = 0
            errors.append(str(e))
            response['errors'] = errors
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class VenueOwerUPDATEAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = VenueOwnerUPDATESerializer

    def post(self, request, format=None):
        '''
        Create New Venue Owner
        '''
        try:
            response = {}
            errors = []
            serializers = self.serializer_class(
                instance=VenueOwner.objects.get(user=request.user), data=request.data)
            if serializers.is_valid():
                serializers.save()
                response["result"] = 1
                errors.append("Successfully Updated")
                response["errors"] = errors
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)
            else:
                response["result"] = 1
                response = handle_errors(serializers.errors)
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_400_BAD_REQUEST)
        except Exception as e:
            response["result"] = 0
            errors.append(str(e))
            response['errors'] = errors
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class VenueOwnerNewGetAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)

    def get(self, request, format=None):
        '''
        Get Venue owners
        '''
        try:
            response = {}
            errors = []
            data = Venue.objects.filter(created_by=request.user).first()
            OwnerInfo = VenueOwner.objects.filter(
                user=request.user, is_deleted=False).first()

            so = VenueOwnerGetSerializer(OwnerInfo)
            do = VenueProfileSerializer(data)
            working_hours = VenueDailyTiming.objects.filter(venue_id=data)
            working_hours_serializer = VenueDailyAvailibilitySerializer(
                working_hours, many=True)
            responsedata = [{"venue_data": do.data}, {"venueowner_data": so.data}, {
                "working_hours": working_hours_serializer.data}]
            response['result'] = 1
            response['data'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        except Exception as e:
            response["result"] = 0
            errors.append(str(e))
            response['errors'] = errors
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class VenueOwnerNewPOSTAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = VenueOwnerNewPOSTSerializer

    def post(self, request, format=None):
        '''
        Create New Venue Owner
        '''
        try:
            response = {}
            errors = []
            request.data._mutable = True
            serializers = self.serializer_class(
                data=request.data, context={"user": request.user})
            if serializers.is_valid():
                data = serializers.save()
                response["result"] = 1
                errors.append("Successfully Inserted")
                response["errors"] = errors
                response['ID'] = data
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)
            else:
                response["result"] = 1
                response = handle_errors(serializers.errors)
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_400_BAD_REQUEST)
        except Exception as e:
            response["result"] = 0
            errors.append(str(e))
            response['errors'] = errors
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class PackagesGETAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = PackagesGetSerializer

    def post(self, request, format=None):
        '''
        Get Packages
        '''
        response = {}
        responsedata = []
        request.data._mutable = True
        request.data['created_by'] = User.objects.get(
            username=str(request.user)).id

        serializer = self.serializer_class(data=request.data)
        print(request.data)
        if serializer.is_valid():

            pagination = serializer.validated_data['page']
            serializer.validated_data.pop("page")

            # if "sport_relivance" in serializer.validated_data:
            #     serializer.validated_data['sport_relivance__in'] = serializer.validated_data["sport_relivance"]
            #     serializer.validated_data.pop("sport_relivance")

            # if "package_type" in serializer.validated_data:
            #     if serializer.validated_data['package_type'] == "Subscription":
            #         serializer.validated_data['package_type__in'] = 'S'

            #     elif serializer.validated_data['package_type'] == "Lessons":
            #         serializer.validated_data['package_type__in'] = 'L'

            #     elif serializer.validated_data['package_type'] == "Bookings Package":
            #         serializer.validated_data['package_type__in'] = 'B'
            #     serializer.validated_data.pop("package_type")

            if "status_name" in serializer.validated_data:
                if serializer.validated_data["status_name"] == "UNDER APPROVAL":
                    serializer.validated_data['status'] = "O"

                elif serializer.validated_data["status_name"] == "ON GOING":
                    serializer.validated_data['status'] = "A"

                elif serializer.validated_data["status_name"] == "WAITING":
                    serializer.validated_data['status'] = "A"

                elif serializer.validated_data["status_name"] == "REJECTED":
                    serializer.validated_data['status'] = "R"
                serializer.validated_data.pop("status_name")

            if "keyword" in serializer.validated_data:
                serializer.validated_data['name__icontains'] = serializer.validated_data['keyword']
                serializer.validated_data.pop("keyword")

            # if "package_duration" in serializer.validated_data:
            #     serializer.validated_data['package_duration__in'] = [
            #         serializer.validated_data["package_duration"]]
            #     serializer.validated_data.pop("package_duration")

            if "package_availabiity" in serializer.validated_data:
                package_availabiity = serializer.validated_data.pop(
                    "package_availabiity")
            else:
                package_availabiity = None

            start_data = (pagination - 1) * 10
            end_data = start_data + 10

            package_data = Packages.objects.filter(
                **serializer.validated_data).order_by("-created_at")

            # package_availabiity
            for item in package_data:
                if package_availabiity == True:
                    if item.package_quantity > 0:
                        pass
                    else:
                        package_data = package_data.exclude(id=item.id)
                elif package_availabiity == False:
                    if item.package_quantity == 0:
                        pass
                    else:
                        package_data = package_data.exclude(id=item.id)
                else:
                    pass

            package_count = package_data.count()
            package_data = package_data[start_data:end_data]

            serializer = self.serializer_class(package_data, many=True)

            response["result"] = 1
            response['data'] = serializer.data
            response['total_page'] = package_count
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class PackagesPostAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = PackagesPostSerializer

    def post(self, request, format=None):
        '''
        Add and Update Packages
        '''
        responsedata = []
        response = {}
        request.data._mutable = True
        request.data['created_by'] = User.objects.get(
            username=str(request.user)).id
        if 'id' in request.data:
            if Packages.objects.filter(id=request.data['id'], is_deleted=False).exists():
                serializer = self.serializer_class(instance=Packages.objects.filter(
                    id=request.data['id'], is_deleted=False).first(), data=request.data)
            message = "Updated Successfully"
        else:
            serializer = self.serializer_class(data=request.data)
            message = "Created Successfully"

        if serializer.is_valid():
            serializer.save()
            response["result"] = 1
            response['data'] = [message]
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class BookingCancellationRequestPOSTAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = BookingCancellationRequestPostSerializer

    def post(self, request, format=None):
        '''
        BookingCancellationRequest post API
        '''
        try:
            response = {}
            responsedata = []
            request.data._mutable = True
            serializers = self.serializer_class(data=request.data)
            venue_owner = VenueOwner.objects.get(user=request.user)
            images = Template_Images.objects.filter().values('template_image')
            logo = images[0]
            logo2 = images[1]

            if serializers.is_valid():
                serializers.validated_data['created_by'] = request.user
                data = serializers.save()
                msg = "We received your bookings cancellation request.It will be processed shortly."
                venue_owner_booking_cancel_Complete_mail(
                    venue_owner.first_name, venue_owner.email, msg, logo, logo2)
                response["result"] = 1
                responsedata.append("Successfully Inserted")
                response["data"] = responsedata
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)
            else:
                response["result"] = 1
                response = handle_errors(serializers.errors)
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_400_BAD_REQUEST)
        except Exception as e:
            response["result"] = 0
            responsedata.append(str(e))
            response['errors'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class BookingCancellationFilterAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    '''
    Booking cancellation filter API View
    '''

    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = BookingCancellationFilterSerializer

    @transaction.atomic
    def post(self, request, format=None):
        response = {}
        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            # get page number
            start_count = serializers.validated_data['page'] - 1
            start_count = start_count * 10
            end_count = start_count + 10
            extra_args = {}

            if "cancellation_reason" in serializers.validated_data:
                extra_args['cancellation_reason'] = serializers.validated_data['cancellation_reason']

            if "start_date" in serializers.validated_data:
                extra_args['start_date'] = serializers.validated_data['start_date']

            if "end_date" in serializers.validated_data:
                extra_args['end_date'] = serializers.validated_data['end_date']

            if "status" in serializers.validated_data:
                extra_args['status'] = serializers.validated_data['status']

            if "venue_field" in serializers.validated_data:
                extra_args['venue_field__in'] = serializers.validated_data['venue_field']

                book_cancel_obj = RequestBookingCancellation.objects.filter(created_by=request.user,
                                                                            **extra_args).distinct().order_by("-created_at")
            else:
                book_cancel_obj = RequestBookingCancellation.objects.filter(created_by=request.user,
                                                                            **extra_args).order_by("-created_at")
            total_count = book_cancel_obj.count()
            book_cancel_obj = book_cancel_obj[start_count:end_count]
            booking_cancel_serializer = BookingCancellationRequestAllSerializer(
                book_cancel_obj, many=True)

            data = book_cancel_obj[start_count:end_count]
            response['result'] = 1
            response['data'] = booking_cancel_serializer.data
            response['total_count'] = total_count
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        # Serializer Errors
        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)
