from django.db import models

from django.core.validators import MaxValueValidator, MinValueValidator

from django.contrib.auth.models import User
from storages.backends.s3boto import S3BotoStorage

from static_data.models import *

#from sportsfieldbooking_backend.field_booking.static_data.models import VenueService


class S3CustomStorage(S3BotoStorage):
    def _normalize_name(self, name):
        """
        S3 Storage function for storing files
        """
        return name


class TestModel(models.Model):
    name = models.TextField(max_length=100)

    def __str__(self):
        return self.name


class Venue(models.Model):
    """ Venue Details """

    name = models.CharField(max_length=100, null=True)
    logo = models.ImageField(upload_to="venue_logo", blank=True, null=True)

    main_image = models.ImageField(
        upload_to="venue_main_image", blank=True, null=True)

    business_name = models.CharField(max_length=100, null=True)

    professional_allowed = models.ManyToManyField(Profession, blank=True)
    all_sports = models.ManyToManyField(Sport, blank=True)
    allowed_entry = models.ManyToManyField(VenuePermission, blank=True)

    all_services = models.ManyToManyField(VenueService, blank=True)
    covid19Measure_description = models.ManyToManyField(
        Covid19Measures, blank=True)

    description = models.TextField(max_length=5000, blank=True, null=True)

    street = models.TextField(max_length=150, blank=True, null=True)
    #city = models.CharField(max_length=50, blank=True, null=True)
    pincode = models.CharField(max_length=10, blank=True, null=True)
    #country = models.CharField(max_length=100, blank=True, null=True)
    country = models.ForeignKey(
        Country, on_delete=models.CASCADE, blank=True, null=True)
    city = models.ForeignKey(
        City, on_delete=models.CASCADE, blank=True, null=True)
    state = models.ForeignKey(
        State, on_delete=models.CASCADE, blank=True, null=True)
    area = models.ForeignKey(
        Area, on_delete=models.CASCADE, blank=True, null=True)

    latitude = models.FloatField(blank=True, null=True)
    longitude = models.FloatField(blank=True, null=True)
    google_address = models.CharField(max_length=450, blank=True, null=True)
    place_id = models.CharField(max_length=450, blank=True, null=True)

    venue_number = models.CharField(max_length=50, blank=True, null=True)
    buisness_vat = models.CharField(
        max_length=50, blank=True, null=True, help_text="user buisness vat")
    tax_offices = models.CharField(
        max_length=100, null=True, blank=True, help_text="user buisness Tax office")
    working_hours = models.CharField(
        max_length=100, null=True, blank=True, help_text="user buisness working hours")

    is_deleted = models.BooleanField(default=False)

    created_by = models.ForeignKey(
        User, on_delete=models.CASCADE, blank=True, null=True)

    #sport = models.ForeignKey(Sport, on_delete=models.CASCADE,related_name='venue_sports', blank=True, null=True)

    #area = models.ForeignKey(Area, on_delete=models.CASCADE, related_name='area_sports', blank=True, null=True)
    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)
    updated_at = models.DateTimeField(auto_now=True, blank=True, null=True)

    def __str__(self):
        return "{}-{}-{}".format(self.name, self.created_by, self.created_at)

    class Meta:
        verbose_name_plural = "Venue"
        db_table = 'venue'


class VenueImage(models.Model):
    """ Venue images """

    venue = models.ForeignKey(
        Venue, on_delete=models.CASCADE, blank=True, null=True)
    image = models.ImageField(upload_to="venue_images",
                              blank=True, null=True, max_length=6000)
    description = models.TextField(max_length=100, blank=True, null=True)

    is_deleted = models.BooleanField(default=False)
    created_by = models.ForeignKey(
        User, on_delete=models.CASCADE, blank=True, null=True)

    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)
    updated_at = models.DateTimeField(auto_now=True, blank=True, null=True)

    def __str__(self):
        return "{}-{}-{}".format(self.venue, self.is_deleted, self.created_by)

    class Meta:
        verbose_name_plural = "Venue Images"
        db_table = 'venue_image'


class VenueDailyTiming(models.Model):
    """
    Weekly Timing of Venue
    """
    venue = models.ForeignKey(
        Venue, on_delete=models.CASCADE, blank=True, null=True)
    day = models.ForeignKey(
        DayofWeek, on_delete=models.SET_NULL, blank=True, null=True, help_text="Day")
    start_time = models.TimeField(
        blank=True, null=True, help_text="Start time")
    end_time = models.TimeField(blank=True, null=True, help_text="Start time")
    is_active = models.BooleanField(blank=True, null=True)
    is_deleted = models.BooleanField(default=False, null=True, blank=True)
    created_at = models.DateTimeField(auto_now=True, blank=True, null=True)

    def __str__(self):
        return "{}-{}-{}-{}".format(self.id, self.day, self.start_time, self.end_time)

    class Meta:
        verbose_name_plural = "Venue Daily Timing"
        db_table = 'venue_timing'


class VenueContactPerson(models.Model):
    """ Venue Contact Persons """

    venue = models.ForeignKey(
        Venue, on_delete=models.CASCADE, blank=True, null=True)

    first_name = models.CharField(max_length=50, blank=True, null=True)
    last_name = models.CharField(max_length=50, blank=True, null=True)
    telephone = models.CharField(max_length=50, blank=True, null=True)
    mobile = models.IntegerField(blank=True, null=True)
    email = models.CharField(max_length=50, blank=True, null=True)

    is_deleted = models.BooleanField(default=False)

    created_by = models.ForeignKey(
        User, on_delete=models.CASCADE, blank=True, null=True)

    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)
    updated_at = models.DateTimeField(auto_now=True, blank=True, null=True)

    def __str__(self):
        return "{}-{}-{}".format(self.venue, self.is_deleted, self.created_by)

    class Meta:
        verbose_name_plural = "Venue Contact Persons"
        db_table = 'venue_contact_persons'


class VenueBankInfo(models.Model):
    """ Venue Contact Persons """

    venue = models.ForeignKey(
        Venue, on_delete=models.CASCADE, blank=True, null=True)

    iban = models.CharField(max_length=50, blank=True, null=True)
    bank = models.CharField(max_length=50, blank=True, null=True)

    first_name = models.CharField(max_length=50, blank=True, null=True)
    last_name = models.CharField(max_length=50, blank=True, null=True)

    is_deleted = models.BooleanField(default=False)

    created_by = models.ForeignKey(
        User, on_delete=models.CASCADE, blank=True, null=True)

    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)
    updated_at = models.DateTimeField(auto_now=True, blank=True, null=True)

    def __str__(self):
        return "{}-{}-{}".format(self.venue, self.is_deleted, self.created_by)

    class Meta:
        verbose_name_plural = "Venue Bank Info"
        db_table = 'venue_bank_info'


class VenueOpeningHour(models.Model):
    """ Venue Opening Hours """

    venue = models.ForeignKey(
        Venue, on_delete=models.CASCADE, blank=True, null=True)

    day_of_week = models.ForeignKey(
        DayofWeek, on_delete=models.CASCADE, blank=True, null=True)

    start_time = models.TimeField(blank=True, null=True)
    end_time = models.TimeField(blank=True, null=True)

    created_by = models.ForeignKey(
        User, on_delete=models.CASCADE, blank=True, null=True)

    is_deleted = models.BooleanField(default=False, blank=True, null=True)

    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)
    updated_at = models.DateTimeField(auto_now=True, blank=True, null=True)

    def __str__(self):
        return "{}-{}-{}".format(self.venue, self.is_deleted, self.created_by)

    class Meta:
        verbose_name_plural = "Venue Opening Hours"
        db_table = 'venue_opening_hours'


class VenueRating(models.Model):
    """ Venue Rating """

    venue = models.ForeignKey(
        Venue, on_delete=models.CASCADE, blank=True, null=True)

    rating = models.IntegerField(blank=True, null=True)

    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)
    updated_at = models.DateTimeField(auto_now=True, blank=True, null=True)

    def __str__(self):
        return "{}-{}-{}".format(self.venue, self.rating, self.created_at)

    class Meta:
        verbose_name_plural = "Venue Rating"
        db_table = 'venue_rating'


class VenueReview(models.Model):
    """ Venue Rating """

    venue = models.ForeignKey(
        Venue, on_delete=models.CASCADE, blank=True, null=True)

    rating = models.IntegerField(blank=True, null=True)
    review = models.TextField(blank=True, null=True)

    created_by = models.ForeignKey(
        User, on_delete=models.CASCADE, blank=True, null=True)

    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)
    updated_at = models.DateTimeField(auto_now=True, blank=True, null=True)

    def __str__(self):
        return "{}-{}-{}".format(self.venue, self.rating, self.created_by)

    class Meta:
        verbose_name_plural = "Venue Reviews"
        db_table = 'venue_review'


class VenueServiceCost(models.Model):
    """ Venue Service Costs """

    venue = models.ForeignKey(
        Venue, on_delete=models.CASCADE, blank=True, null=True)
    venue_service = models.ForeignKey(
        VenueService, on_delete=models.CASCADE, blank=True, null=True)

    price_per_hour = models.FloatField(default=0)

    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)
    updated_at = models.DateTimeField(auto_now=True, blank=True, null=True)

    def __str__(self):
        return "{}-{}-{}".format(self.venue, self.venue_service, self.price_per_hour)

    class Meta:
        verbose_name_plural = "Venue Service Cost"
        db_table = 'venue_service_cost'


class VenueServiceCostQuantity(models.Model):
    """ Venue Service Costs quantity """

    venue_service = models.ForeignKey(
        VenueServiceCost, on_delete=models.CASCADE, blank=True, null=True)

    quantity = models.IntegerField(blank=True, null=True)

    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)
    updated_at = models.DateTimeField(auto_now=True, blank=True, null=True)

    def __str__(self):
        return "{}-{}".format(self.quantity, self.venue_service)

    class Meta:
        verbose_name_plural = "Venue Service Cost Quantity"
        db_table = 'venue_service_cost_quantity'


class VenueField(models.Model):
    """ Venue Fields """
    STATUS_CHOICES = (
        ('A', 'Accepted'),
        ('R', 'Rejected'),
        ('O', 'On Hold')
    )

    venue = models.ForeignKey(
        Venue, on_delete=models.CASCADE, blank=True, null=True)
    name = models.CharField(max_length=100, null=True)
    main_image = models.ImageField(
        upload_to="venue_main_image", blank=True, null=True)

    field_attributes = models.ManyToManyField(FieldAttribute, blank=True)
    field_type = models.ForeignKey(
        FieldType, on_delete=models.CASCADE, blank=True, null=True)
    sport = models.ForeignKey(
        Sport, on_delete=models.CASCADE, blank=True, null=True)
    seat_capacity = models.CharField(max_length=100, blank=True, null=True)
    description = models.TextField(blank=True, null=True)

    floor_type = models.ForeignKey(
        FloorType, blank=True, null=True, on_delete=models.CASCADE)
    is_deleted = models.BooleanField(default=False)

    created_by = models.ForeignKey(
        User, on_delete=models.CASCADE, blank=True, null=True)
    status = models.CharField(
        max_length=10, default=STATUS_CHOICES[2][0], choices=STATUS_CHOICES, blank=True, null=True, help_text="Is document Verified")
    facility_type = models.ForeignKey(
        FacilityType, blank=True, null=True, on_delete=models.CASCADE)
    lights = models.ForeignKey(
        Lights, blank=True, null=True, on_delete=models.CASCADE)
    mixed_gender = models.ForeignKey(
        MIXED_GENDER, blank=True, null=True, on_delete=models.CASCADE)
    availability_in_use = models.ForeignKey(
        AVAILABILITY_IN_USE, blank=True, null=True, on_delete=models.CASCADE)

    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)
    updated_at = models.DateTimeField(auto_now=True, blank=True, null=True)

    def __str__(self):
        return "{}-{}-{}".format(self.name, self.created_by, self.created_at)

    class Meta:
        verbose_name_plural = "Venue Fields"
        db_table = 'venue_fields'


class VenueFieldImage(models.Model):
    """ Venue field images """
    venue_field = models.ForeignKey(
        VenueField, on_delete=models.CASCADE, blank=True, null=True)
    image = models.ImageField(upload_to="venue_images", blank=True, null=True)
    is_deleted = models.BooleanField(default=False)

    created_by = models.ForeignKey(
        User, on_delete=models.CASCADE, blank=True, null=True)

    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)
    updated_at = models.DateTimeField(auto_now=True, blank=True, null=True)

    def __str__(self):
        return "{}-{}-{}".format(self.venue_field, self.is_deleted, self.created_by)

    class Meta:
        verbose_name_plural = "Venue Field Images"
        db_table = 'venue_field_image'


class VenueFieldService(models.Model):
    """ Venue field services """
    venue_field = models.ForeignKey(
        VenueField, on_delete=models.CASCADE, blank=True, null=True)
    venue_service = models.ForeignKey(
        VenueService, on_delete=models.CASCADE, blank=True, null=True)

    price_per_hour = models.FloatField(default=0)

    is_deleted = models.BooleanField(default=False, null=True, blank=True)
    created_by = models.ForeignKey(
        User, on_delete=models.CASCADE, blank=True, null=True)

    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)
    updated_at = models.DateTimeField(auto_now=True, blank=True, null=True)

    def __str__(self):
        return "{}-{}-{}-{}".format(self.venue_field, self.price_per_hour, self.venue_service, self.created_by)

    class Meta:
        verbose_name_plural = "Venue Field Services"
        db_table = 'venue_field_services'


class VenuePriceList(models.Model):
    """ Venue Price List """
    venue = models.ForeignKey(
        Venue, on_delete=models.CASCADE, blank=True, null=True)

    start_time = models.TimeField(blank=True, null=True)
    end_time = models.TimeField(blank=True, null=True)

    time_interval = models.IntegerField(default=0)
    gap_period = models.IntegerField(default=0)
    default_price = models.FloatField(default=0)

    created_by = models.ForeignKey(
        User, on_delete=models.CASCADE, blank=True, null=True)

    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)
    updated_at = models.DateTimeField(auto_now=True, blank=True, null=True)

    def __str__(self):
        return "{}-{}-{}".format(self.venue_field, self.price_per_hour, self.created_by)

    class Meta:
        verbose_name_plural = "Venue Price List"
        db_table = 'venue_price_list'


class VenuePriceRender(models.Model):
    """ Venue Price List """
    venue = models.ForeignKey(
        Venue, on_delete=models.CASCADE, blank=True, null=True)

    start_time = models.TimeField(blank=True, null=True)
    end_time = models.TimeField(blank=True, null=True)

    time_interval = models.IntegerField(default=0)
    gap_period = models.IntegerField(default=0)
    default_price = models.FloatField(default=0)

    created_by = models.ForeignKey(
        User, on_delete=models.CASCADE, blank=True, null=True)

    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)
    updated_at = models.DateTimeField(auto_now=True, blank=True, null=True)

    def __str__(self):
        return "{}-{}-{}".format(self.venue_field, self.price_per_hour, self.created_by)

    class Meta:
        verbose_name_plural = "Venue Price Render"
        db_table = 'venue_price_render'
