from rest_framework import serializers
from django.contrib.auth.models import User, Group
from rest_framework.authtoken.models import Token

from datetime import date

from .models import *
from static_data.serializers import *
from static_data.models import *
from django.core import serializers as djangoserializer


class VenueImageSerializer(serializers.ModelSerializer):
    professional_allowed = ProfessionSerializer(read_only=True, many=True)

    class Meta:
        model = Venue
        fields = ['professional_allowed']


class VenueProfileSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(required=False)

    all_sports = SportSerializer(read_only=True, many=True)
    total_fields = serializers.SerializerMethodField("_get_field_count")

    def _get_field_count(self, obj):
        venue_obj = VenueField.objects.filter(
            venue=obj, is_deleted=False).count()
        print(venue_obj)
        return venue_obj

    class Meta:
        model = Venue
        fields = '__all__'


class VenueBankInfoSerializer(serializers.ModelSerializer):
    venue = serializers.CharField(source='venue.name', read_only=True)
    created_by = serializers.CharField(
        source='created_by.username', read_only=True)

    class Meta:
        model = VenueBankInfo
        fields = '__all__'


class VenueTimingSerializer(serializers.ModelSerializer):
    day = serializers.CharField(source="day.dayofweek", required=False)

    class Meta:
        model = VenueDailyTiming
        fields = ['id', 'day', 'start_time', 'end_time']


class VenueServicePreviewSerializer(serializers.ModelSerializer):
    """
    Serializer for venue service to show in create field preiview
    """
    venue_service_name = serializers.CharField(source="venue_service.name")
    venue_service_logo = serializers.ImageField(source="venue_service.logo")

    class Meta:
        model = VenueFieldService
        fields = ['id', 'venue_service_name', 'venue_service_logo']


class VenueServicePreview1Serializer(serializers.ModelSerializer):
    """
    Serializer for venue service to show in create field preiview
    """
    venue_service = VenueServiceSerializer(required=False, read_only=True)

    class Meta:
        model = VenueFieldService
        fields = '__all__'


class VenueFieldServicePreviewSerializer(serializers.ModelSerializer):
    """
    Serializer for venue field service to show in create field preiview
    """

    class Meta:
        model = VenueFieldService
        fields = '__all__'


class VenueFieldServicePreviewSerializer1(serializers.ModelSerializer):
    """
    Serializer for venue field service to show in create field preiview
    """
    venue_service_name = serializers.CharField(source="venue_service.name")
    venue_service_logo = serializers.ImageField(source="venue_service.logo")

    class Meta:
        model = VenueFieldService
        fields = '__all__'


class VenueFieldSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(required=False)
    services = serializers.JSONField(
        required=False, write_only=True,
        help_text='[{"venue_service":1,"price_per_hour":50},{"venue_service":2,"price_per_hour":60}]')
    images = serializers.ListField(
        required=False, write_only=True, help_text="Field Images")

    # sport = serializers.JSONField(required=False, help_text="[1,2,3]")

    def validate_services(self, data):
        for i in range(len(data)):
            if VenueService.objects.filter(id=data[i]['venue_service']).exists():
                data[i]['venue_service'] = VenueService.objects.get(
                    id=data[i]['venue_service'])
            else:
                raise serializers.FieldDoesNotExist(
                    "No record with ID " + str(data[i]['venue_service']))
        return data

    def create(self, validated_data):

        # import pdb
        # pdb.set_trace()
        return_data_list = {}

        field_attributes = validated_data['field_attributes']
        attributes = []

        venue_obj = Venue.objects.filter(
            created_by=self.context['user']).first()

        if 'services' in validated_data:
            attributes.append("services")
            services = validated_data['services']
            validated_data.pop('services')

        if 'images' in validated_data:
            attributes.append("images")

            images = validated_data['images']
            validated_data.pop('images')

        # if 'sport' in validated_data:
        # sport = validated_data['sport']
        # validated_data.pop('sport')

        validated_data.pop('field_attributes')

        validated_data['venue'] = venue_obj
        venue_FieldData = VenueField.objects.create(**validated_data)
        # venue_FieldData.sport.set(sport)
        # venue_FieldData.save()
        return_data = {
            "id": venue_FieldData.id,
            "description": venue_FieldData.description,
            "field_name": venue_FieldData.name,
            "field_type": venue_FieldData.field_type.name,
            "floor_type": venue_FieldData.floor_type.name,
            "seat_capacity": venue_FieldData.seat_capacity,
            "lights": venue_FieldData.lights.name,
            "facility_type": venue_FieldData.facility_type.name,
            "facility_type_id": venue_FieldData.facility_type.id,
            "sport": venue_FieldData.sport.name,

        }
        main_image_class = VenueFieldSerializerVer1(
            venue_FieldData, many=False)
        return_data['main_image'] = main_image_class.data

        if len(field_attributes) > 0:
            venue_FieldData.field_attributes.set(field_attributes)

        if 'services' in attributes:
            venue_fieldService = [VenueFieldService(
                **i,
                venue_field=venue_FieldData,
                created_by=self.context['user']
            ) for i in services]
            service_obj = VenueFieldService.objects.bulk_create(
                venue_fieldService)
            services_class = VenueServicePreviewSerializer(
                service_obj, many=True)
            return_data['services'] = services_class.data

        if 'images' in attributes:
            venueField_image = [VenueFieldImage(
                image=i,
                venue_field=venue_FieldData,
                created_by=self.context['user']
            )
                for i in images]
            image_obj = VenueFieldImage.objects.bulk_create(venueField_image)
            images_class = VenueFieldImageSerializer(image_obj, many=True)
            return_data['images'] = images_class.data

        return return_data

    class Meta:
        model = VenueField
        fields = '__all__'


class VenueFieldSerializerVer1(serializers.ModelSerializer):
    class Meta:
        model = VenueField
        fields = ['main_image']


class VenueBookmatchSerializer(serializers.ModelSerializer):
    country = CountrySerializer(read_only=True)
    city = CitySerializer(read_only=True)
    state = StateGetSerializer(read_only=True)
    area = AreaGetSerializer(read_only=True)

    class Meta:
        model = Venue
        fields = '__all__'


class VenueFieldBookingMatchSerializer(serializers.ModelSerializer):
    venue = VenueBookmatchSerializer()

    class Meta:
        model = VenueField
        fields = ['name', 'venue']


class VenueFieldServiceSerializer(serializers.ModelSerializer):
    name = serializers.CharField(source="venue_service.name")
    logo = serializers.ImageField(source="venue_service.logo")

    class Meta:
        model = VenueFieldService
        fields = '__all__'


class VenueServicesCostSerializer(serializers.ModelSerializer):
    venue_service = serializers.CharField(
        source="venue_service.name", required=False)

    class Meta:
        model = VenueServiceCost
        fields = "__all__"


class VenueServicesCost1Serializer(serializers.ModelSerializer):
    venue_service = VenueServiceSerializer(required=False, read_only=True)

    class Meta:
        model = VenueServiceCost
        fields = "__all__"


class VenueServicesCost1Serializer(serializers.ModelSerializer):
    venue_service = VenueServiceSerializer(required=False, read_only=True)

    class Meta:
        model = VenueServiceCost
        fields = "__all__"


class VenueFieldImageSerializer(serializers.ModelSerializer):
    class Meta:
        model = VenueFieldImage
        fields = '__all__'


class VenueFieldGETAdminSerializer(serializers.ModelSerializer):
    venue = serializers.CharField(source='venue.name', read_only=True)
    id = serializers.IntegerField(
        required=False, help_text="ID of the venue Field")
    field_attributes = FieldAttributeSerializer(many=True)
    field_service = serializers.SerializerMethodField()
    field_images = serializers.SerializerMethodField()
    field_type = serializers.SerializerMethodField()
    floor_type = serializers.SerializerMethodField()
    facility_type = serializers.SerializerMethodField()
    lights = serializers.SerializerMethodField()
    sport = serializers.SerializerMethodField()
    availability_in_use = AvailabilityInUseSerializer(read_only=True)
    mixed_gender = MixedGenderSerializer(read_only=True)

    def get_field_images(self, obj):
        data = VenueFieldImage.objects.filter(venue_field=obj)
        serialized_data = VenueFieldImageSerializer(data, many=True)
        return serialized_data.data

    def get_field_service(self, obj):
        data = VenueFieldService.objects.filter(venue_field=obj)
        serialized_data = VenueFieldServiceSerializer(data, many=True)
        return serialized_data.data

    def get_field_type(self, obj):
        serialized_data = FieldTypeSerializer(obj.field_type)
        return serialized_data.data

    def get_floor_type(self, obj):
        serialized_data = FloorTypeSerializer(obj.floor_type)
        return serialized_data.data

    def get_facility_type(self, obj):
        serialized_data = FacilitysGetSerializer(obj.facility_type)
        return serialized_data.data

    def get_lights(self, obj):
        serialized_data = LightsGetSerializer(obj.lights)
        return serialized_data.data

    def get_sport(self, obj):
        serialized_data = SportSerializer(obj.sport)
        return serialized_data.data

    def validate_id(self, value):
        if VenueField.objects.filter(id=value).exists():
            return value
        else:
            raise serializers.FieldDoesNotExist("No record with ID found")

    def __init__(self, *args, **kwargs):
        super(VenueFieldGETAdminSerializer, self).__init__(*args, **kwargs)
        for field in self.fields:
            if field != 'id':
                self.fields[field].read_only = True

    class Meta:
        model = VenueField
        fields = '__all__'


class VenueFieldPaginationSerializer(serializers.Serializer):
    page = serializers.IntegerField(
        required=False, default=1, help_text="Page number")


# class VenueServiceSerializer(serializers.ModelSerializer):
# 	class Meta:
# 		models = VenueService
# 		fields = ['name','logo']


class VenueFieldGETSerializer(serializers.ModelSerializer):
    venue = serializers.CharField(source='venue.name', read_only=True)
    id = serializers.IntegerField(
        required=False, help_text="ID of the venue Field")
    sport = SportSerializer(read_only=True)
    field_attributes = FieldAttributeSerializer(many=True)
    field_service = serializers.SerializerMethodField()
    field_images = serializers.SerializerMethodField()

    # field_type = serializers.CharField(
    #     source="field_type.name", required=False)
    # floor_type = serializers.CharField(
    #     source="floor_type.name", required=False)
    field_type = serializers.SerializerMethodField()
    floor_type = serializers.SerializerMethodField()
    facility_type = serializers.SerializerMethodField()
    lights = serializers.SerializerMethodField()
    availability_in_use = AvailabilityInUseSerializer(read_only=True)
    mixed_gender = MixedGenderSerializer(read_only=True)
    mode = serializers.CharField(
        required=False, help_text="Advertiser (for all fields)")
    page = serializers.IntegerField(
        required=False, default=1, help_text="Page number")

    # sport = serializers.SerializerMethodField()

    def get_field_type(self, obj):
        serialized_data = FieldTypeSerializer(obj.field_type)
        return serialized_data.data

    def get_floor_type(self, obj):
        serialized_data = FloorTypeSerializer(obj.floor_type)
        return serialized_data.data

    def get_facility_type(self, obj):
        serialized_data = FacilitysGetSerializer(obj.facility_type)
        return serialized_data.data

    def get_lights(self, obj):
        serialized_data = LightsGetSerializer(obj.lights)
        return serialized_data.data

    def get_field_images(self, obj):
        data = VenueFieldImage.objects.filter(venue_field=obj)
        serialized_data = VenueFieldImageSerializer(data, many=True)
        return serialized_data.data

    def get_field_service(self, obj):
        data = VenueFieldService.objects.filter(venue_field=obj)
        serialized_data = VenueFieldServiceSerializer(data, many=True)
        return serialized_data.data

    def validate_id(self, value):
        if VenueField.objects.filter(id=value).exists():
            return value
        else:
            raise serializers.FieldDoesNotExist("No record with ID found")

    def __init__(self, *args, **kwargs):
        super(VenueFieldGETSerializer, self).__init__(*args, **kwargs)
        for field in self.fields:
            if field != 'id' and field != "mode" and field != "page":
                self.fields[field].read_only = True

    class Meta:
        model = VenueField
        fields = '__all__'


class VenueContactPersonSerializer(serializers.ModelSerializer):
    created_by = serializers.CharField(
        source='created_by.username', read_only=True)

    venueName = serializers.CharField(source='venue.name', read_only=True)
    venue = serializers.IntegerField(write_only=True)

    class Meta:
        model = VenueContactPerson
        fields = '__all__'


class VenuePhotoSerializer(serializers.ModelSerializer):
    images = serializers.ListField(help_text="Array of images", required=False)
    latitude = serializers.FloatField(required=False)
    longitude = serializers.FloatField(required=False)
    google_address = serializers.CharField(required=False)
    place_id = serializers.CharField(required=False)
    professional_allowed = serializers.JSONField(
        required=False, help_text="[1,2,3]")
    all_sports = serializers.JSONField(required=False, help_text="[1,2,3]")
    main_image = serializers.ImageField(required=False)
    street = serializers.CharField(required=False)
    state = serializers.IntegerField(
        required=False, help_text="state")
    area = serializers.IntegerField(
        required=False, help_text="area")
    city = serializers.IntegerField(
        required=False, help_text="city")
    country = serializers.IntegerField(
        required=False, help_text="country")
    venue_number = serializers.CharField(required=False)
    pincode = serializers.CharField(required=False)
    name = serializers.CharField(required=False)

    def create(self, validated_data):

        venue = Venue.objects.filter(created_by=self.context['user']).first()

        latitude = validated_data.get("latitude", None)
        longitude = validated_data.get("longitude", None)
        google_address = validated_data.get("google_address", "")
        place_id = validated_data.get("place_id", "")
        sports = validated_data.get("all_sports", "")
        professional_allowed = validated_data.get("professional_allowed", "")

        # main_image = validated_data.get("main_image", "")

        # Update lat long
        venue.latitude = latitude
        venue.longitude = longitude
        venue.google_address = google_address
        venue.place_id = place_id
        venue.description = validated_data['description']
        venue.street = validated_data['street']
        venue.area = Area.objects.get(
            id=validated_data['area'])
        venue.city = City.objects.get(
            id=validated_data['city'])
        venue.state = State.objects.get(
            id=validated_data['state'])
        venue.country = Country.objects.get(
            id=validated_data['country'])
        venue.pincode = validated_data['pincode']
        venue.venue_number = validated_data['venue_number']
        venue.name = validated_data['name']

        if 'main_image' in validated_data:
            venue.main_image = validated_data["main_image"]

        venue.professional_allowed.set(professional_allowed)
        venue.all_sports.set(sports)
        venue.save()

        if 'images' in validated_data:
            # VenueImage.objects.filter(venue=venue).delete()
            obj = [VenueImage(
                created_by=self.context['user'],
                venue=venue,
                image=image
            ) for image in validated_data['images']]

            VenueImage.objects.bulk_create(obj)

        return True

    class Meta:
        model = VenueImage
        fields = '__all__'
        read_only_fields = ['venue', 'image', 'created_by']


class VenueProfileServiceSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(required=False)
    all_services = serializers.JSONField(required=False, help_text="[1,2,3]")

    class Meta:
        model = Venue
        fields = ['all_services', 'id']


class VenueProfileServiceGETSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(required=False)
    all_services = VenueServiceSerializer(
        required=False, many=True, read_only=True)

    class Meta:
        model = Venue
        fields = ['all_services', 'id']


class VenueEntrySerializers(serializers.ModelSerializer):
    id = serializers.IntegerField(required=False)
    allowed_entry = serializers.JSONField(
        required=False, help_text="[1,2,3,4]")

    class Meta:
        model = Venue
        fields = ['allowed_entry', 'id']


class VenueEntryGETSerializers(serializers.ModelSerializer):
    id = serializers.IntegerField(required=True)
    allowed_entry = VenuePermissionSerializer(
        required=False, read_only=True, many=True)

    class Meta:
        model = Venue
        fields = ['allowed_entry', 'id']


class Covid19MeasuresSerializer(serializers.ModelSerializer):
    class Meta:
        model = Covid19Measures
        fields = "__all__"


############## GLOBAL GET ###############


class GlobalGETSerializer(serializers.Serializer):
    id = serializers.IntegerField(required=False)

    def validate_id(self, value):
        if Venue.objects.filter(id=value, is_deleted=False).exists():
            return value
        else:
            raise serializers.FieldDoesNotExist(
                "NO Record with ID " + str(value))


class FieldNobookingSerializer(serializers.Serializer):
    id = serializers.PrimaryKeyRelatedField(
        queryset=VenueField.objects.all(), required=True)


class VenueProfileGlobalSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(required=False)
    created_by = serializers.CharField(
        source="created_by.username", read_only=True)
    all_sports = SportSerializer(read_only=True, many=True)
    all_services = VenueServiceSerializer(read_only=True, many=True)
    allowed_entry = VenuePermissionSerializer(read_only=True, many=True)
    professional_allowed = ProfessionSerializer(read_only=True, many=True)

    country = CountrySerializer(read_only=True)
    city = CityGetSerializer(read_only=True)
    state = StateGetSerializer(read_only=True)
    area = AreaGetSerializer(read_only=True)
    covid19Measure_description = Covid19MeasuresSerializer(
        read_only=True, many=True)

    class Meta:
        model = Venue
        fields = '__all__'


class DeleteFieldSerializer(serializers.Serializer):
    id = serializers.IntegerField(required=True, help_text="Id of the field")


class FieldServicesSerializer(serializers.ModelSerializer):
    class Meta:
        model = VenueService
        fields = '__all__'


class VenueFieldGlobalSerializer(serializers.ModelSerializer):
    field_attributes = FieldAttributeSerializer(many=True, required=False)
    field_type = serializers.CharField(
        source="field_type.name", required=False)
    floor_type = serializers.CharField(
        source="floor_type.name", required=False)

    class Meta:
        model = VenueField
        fields = '__all__'


# TO GET VENUE PHOTOS IN GLOBAL
class VenueFieldImageGlobalSerializer(serializers.ModelSerializer):
    class Meta:
        model = VenueFieldImage
        fields = '__all__'


class VenuePhotoGETSerializer(serializers.ModelSerializer):
    venue = serializers.CharField(source="venue.name", read_only=True)

    class Meta:
        model = VenueImage
        fields = '__all__'


# Teestt

class CreateDemoSerializer(serializers.Serializer):
    email_id = serializers.ListField(required=True, help_text="Email id")
    password = serializers.CharField(required=True, help_text="Password")
    mode = serializers.CharField(required=True, help_text="User type")
    confirm = serializers.CharField(required=True, help_text="Check")


# SERIALIZER CLASS


class DeleteVenuePhotoSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(required=True)

    class Meta:
        model = VenueImage
        fields = ['id']


# SERIALIZER CLASS


class DeleteVenueFieldPhotoSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(required=True)

    class Meta:
        model = VenueFieldImage
        fields = ['id']


class VenuePhotoNewSerializer(serializers.ModelSerializer):
    images = serializers.ListField(help_text="Array of images", required=False)
    latitude = serializers.FloatField(required=False)
    longitude = serializers.FloatField(required=False)
    google_address = serializers.CharField(required=False)
    place_id = serializers.CharField(required=False)
    professional_allowed = serializers.JSONField(
        required=False, help_text="[1,2,3]")
    all_sports = serializers.JSONField(required=False, help_text="[1,2,3]")
    main_image = serializers.ImageField(required=False)
    street = serializers.CharField(required=False)
    state = serializers.IntegerField(
        required=False, help_text="state")
    area = serializers.IntegerField(
        required=False, help_text="area")
    city = serializers.IntegerField(
        required=False, help_text="city")
    country = serializers.IntegerField(
        required=False, help_text="country")
    venue_number = serializers.CharField(required=False)
    pincode = serializers.CharField(required=False)
    name = serializers.CharField(required=False)

    allowed_entry = serializers.JSONField(
        required=False, help_text="[1,2,3,4]")
    all_services = serializers.JSONField(required=False, help_text="[1,2,3]")

    def create(self, validated_data):

        venue = Venue.objects.filter(created_by=self.context['user']).first()

        latitude = validated_data.get("latitude", None)
        longitude = validated_data.get("longitude", None)
        google_address = validated_data.get("google_address", "")
        place_id = validated_data.get("place_id", "")
        sports = validated_data.get("all_sports", "")
        professional_allowed = validated_data.get("professional_allowed", "")
        allowed_entry = validated_data.get("allowed_entry", "")
        all_services = validated_data.get("all_services", "")
        # main_image = validated_data.get("main_image", "")

        # Update lat long
        venue.latitude = latitude
        venue.longitude = longitude
        venue.google_address = google_address
        venue.place_id = place_id
        venue.street = validated_data['street']
        venue.area = Area.objects.get(
            id=validated_data['area'])
        venue.city = City.objects.get(
            id=validated_data['city'])
        venue.state = State.objects.get(
            id=validated_data['state'])
        venue.country = Country.objects.get(
            id=validated_data['country'])
        venue.description = validated_data['description']
        venue.street = validated_data['street']
        venue.pincode = validated_data['pincode']
        venue.venue_number = validated_data['venue_number']
        venue.name = validated_data['name']

        if 'main_image' in validated_data:
            venue.main_image = validated_data["main_image"]

        if 'allowed_entry' in validated_data:
            venue.allowed_entry.set(allowed_entry)

        venue.professional_allowed.set(professional_allowed)
        venue.all_sports.set(sports)
        # venue.allowed_entry.set(allowed_entry)
        venue.all_services.set(all_services)
        venue.save()

        if 'images' in validated_data:
            # VenueImage.objects.filter(venue=venue).delete()
            obj = [VenueImage(
                created_by=self.context['user'],
                venue=venue,
                image=image
            ) for image in validated_data['images']]

            VenueImage.objects.bulk_create(obj)

        return True

    class Meta:
        model = VenueImage
        fields = '__all__'
        read_only_fields = ['venue', 'image', 'created_by']


class VenueServiceSerializer1(serializers.ModelSerializer):
    class Meta:
        model = VenueService
        fields = '__all__'
        read_only_fields = ['type', 'service_available', 'logo', 'name']


class VenueOwnerServiceSerializer(serializers.ModelSerializer):
    venue_service = serializers.JSONField(
        required=False, write_only=True, help_text="venue services ID-[1,2,3]")

    class Meta:
        model = VenueServiceCost
        fields = '__all__'
        read_only_fields = ['venue', 'price_per_hour']


class VenueServiceSerializerForServices(serializers.ModelSerializer):
    class Meta:
        model = VenueService
        fields = '__all__'


class VenueOwnerPaidServiceCostSerializer(serializers.ModelSerializer):
    venue_service = VenueServiceSerializerForServices(read_only=True)

    class Meta:
        model = VenueServiceCost
        fields = ['venue_service', 'venue', 'price_per_hour']


class VenueServiceSerializer(serializers.ModelSerializer):
    price_per_hour = serializers.FloatField(required=False)

    class Meta:
        model = VenueService
        fields = '__all__'


class VenueFieldPaidServiceSerializer(serializers.ModelSerializer):
    venue_field = serializers.IntegerField(help_text="Venue Field ID")

    class Meta:
        model = VenueFieldService
        fields = '__all__'
        read_only_fields = ['is_deleted', 'price_per_hour',
                            'venue_service', 'created_by']


class VenueFieldPaidServiceSerializer1(serializers.ModelSerializer):
    venue_service = VenueServiceSerializerForServices(read_only=True)

    class Meta:
        model = VenueFieldService
        fields = '__all__'


class VenueFieldSerializerPostService(serializers.ModelSerializer):
    venue_service = VenueServiceSerializerForServices(read_only=True)

    class Meta:
        model = VenueField
        fields = '__all__'


class VenueSAVESerializer(serializers.ModelSerializer):
    class Meta:
        model = VenueServiceCost
        fields = '__all__'


class VenueFieldPOSTServicesSerializer(serializers.ModelSerializer):
    venue_field = serializers.IntegerField(help_text="Venue Field ID")
    services = serializers.JSONField(
        required=False, write_only=True,
        help_text='[{"venue_service":1,"price_per_hour":50},{"venue_service":2,"price_per_hour":60}]')

    def validate_services(self, data):

        for i in range(len(data)):
            if VenueService.objects.filter(id=data[i]['venue_service']).exists():
                data[i]['venue_service'] = VenueService.objects.get(
                    id=data[i]['venue_service'])
            else:
                raise serializers.FieldDoesNotExist(
                    "No record with ID " + str(data[i]['venue_service']))
        return data

    def create(self, validated_data):

        attributes = []

        venue_field_obj = VenueField.objects.filter(
            id=validated_data['venue_field']).first()

        venue_field_service_object = VenueFieldService.objects.filter(
            venue_field=validated_data['venue_field'])
        venue_field_service_object.delete()

        if 'services' in validated_data:
            attributes.append("services")
            services = validated_data['services']
            validated_data.pop('services')

        validated_data['venue_field'] = venue_field_obj

        return_data = {}

        if 'services' in attributes:
            venue_fieldService = [VenueFieldService(
                **i,
                venue_field=venue_field_obj,
                created_by=self.context['user']
            ) for i in services]

            service_obj = VenueFieldService.objects.bulk_create(
                venue_fieldService)
            services_class = VenueFieldServicePreviewSerializer(
                service_obj, many=True)
            return_data['VenueFieldService'] = services_class.data

        return return_data

    class Meta:
        model = VenueFieldService
        fields = '__all__'
        read_only_fields = ['venue_service',
                            'price_per_hour', 'is_deleted', 'created_by']


class VenuePOSTServicesSerializer(serializers.ModelSerializer):
    services = serializers.JSONField(
        required=True, write_only=True,
        help_text='[{"venue_service":1,"price_per_hour":50},{"venue_service":2,"price_per_hour":60}]')

    def validate_services(self, data):

        for i in range(len(data)):
            if VenueService.objects.filter(id=data[i]['venue_service']).exists():
                data[i]['venue_service'] = VenueService.objects.get(
                    id=data[i]['venue_service'])
            else:
                raise serializers.FieldDoesNotExist(
                    "No record with ID " + str(data[i]['venue_service']))
        return data

    def create(self, validated_data):

        validated_data['user'] = self.context['user'].id

        attributes = []

        venue_obj = Venue.objects.get(created_by=validated_data['user'])

        venue_service_cost_object = VenueServiceCost.objects.filter(
            venue=venue_obj.id)
        venue_service_cost_object.delete()

        if 'services' in validated_data:
            attributes.append("services")
            services = validated_data['services']
            validated_data.pop('services')

        validated_data['venue'] = venue_obj

        return_data = {}

        if 'services' in attributes:
            venue_Service_cost = [VenueServiceCost(
                **i,
                venue=venue_obj
            ) for i in services]

            service_obj = VenueServiceCost.objects.bulk_create(
                venue_Service_cost)
            services_class = VenueSAVESerializer(
                service_obj, many=True)
            return_data['VenueService'] = services_class.data

        return return_data

    class Meta:
        model = VenueServiceCost
        fields = '__all__'
        read_only_fields = ['venue', 'venue_service',
                            'price_per_hour', 'is_deleted', 'created_by']


class VenueSerializer(serializers.ModelSerializer):
    class Meta:
        model = Venue
        fields = '__all__'


class VenueServiceSerializerForPaidServices(serializers.ModelSerializer):
    class Meta:
        model = VenueService
        fields = '__all__'


class VenueServicesCostSerializerForPaidServices(serializers.ModelSerializer):
    venue_service = VenueServiceSerializerForPaidServices(read_only=True)

    class Meta:
        model = VenueServiceCost
        fields = "__all__"


# class AllVenuegetSerializer(serializers.ModelSerializer):
#     page = serializers.IntegerField(
#         required=False, default=1, help_text="Page number")
#     all_sports = SportSerializer(read_only=True, many=True)
#     all_services = VenueServiceSerializer(read_only=True, many=True)
#     allowed_entry = VenuePermissionSerializer(read_only=True, many=True)
#     professional_allowed = ProfessionSerializer(read_only=True, many=True)

#     country = CountrySerializer(read_only=True)
#     city = CityGetSerializer(read_only=True)
#     state = StateGetSerializer(read_only=True)
#     area = AreaGetSerializer(read_only=True)
#     covid19Measure_description = Covid19MeasuresSerializer(
#         read_only=True, many=True)

#     class Meta:
#         model = Venue
#         fields = "__all__"
#         read_only_fields = ['name', 'logo',
#                             'main_image', 'business_name', 'description', 'street', 'pincode', 'latitude', 'longitude', 'google_address', 'place_id', 'venue_number', 'buisness_vat', 'tax_offices', 'working_hours', 'is_deleted', 'country', 'city', 'state', 'area', 'created_by', 'professional_allowed', 'all_sports', 'allowed_entry', 'all_services', 'covid19Measure_description']


class AllVenuegetSerializer(serializers.ModelSerializer):
    page = serializers.IntegerField(
        required=False, default=1, help_text="Page number")
    all_sports = SportSerializer(read_only=True, many=True)
    all_services = VenueServiceSerializer(read_only=True, many=True)
    # allowed_entry = VenuePermissionSerializer(read_only=True, many=True)
    # professional_allowed = ProfessionSerializer(read_only=True, many=True)

    # country = CountrySerializer(read_only=True)
    #city = serializers.CharField(read_only=True, source='city.name')
    # state = StateGetSerializer(read_only=True)
    # area = AreaGetSerializer(read_only=True)
    covid19Measure_description = Covid19MeasuresSerializer(
        read_only=True, many=True)

    class Meta:
        model = Venue
        fields = "__all__"
        read_only_fields = ['name', 'logo',
                            'main_image', 'business_name', 'description', 'street', 'pincode', 'latitude', 'longitude', 'google_address', 'place_id', 'venue_number', 'buisness_vat', 'tax_offices', 'working_hours', 'is_deleted', 'country', 'city', 'state', 'area', 'created_by', 'professional_allowed', 'all_sports', 'allowed_entry', 'all_services', 'covid19Measure_description']


class GetAllVenueFilterSerializer(serializers.Serializer):

    all_sports = serializers.CharField(
        required=False, help_text='Sport ID = 1')
    star_ratings = serializers.JSONField(required=False, help_text='[1,2,3]')
    page = serializers.IntegerField(
        required=False, default=1, help_text="Page number")
    # field_type = serializers.CharField(
    #     required=False, help_text="Field type ID")
    location = serializers.JSONField(required=False, help_text='[1,2,3]')
    attributes = serializers.JSONField(
        required=False, help_text='["INDOOR","OUTDOOR","YES","NO"]')


############## Suggestions  ################

class GetSuggestionsSerializer(serializers.Serializer):
    key_word = serializers.CharField(
        required=True, help_text="Send names")


class SuggestionsAreaGetSerializer(serializers.ModelSerializer):

    class Meta:
        model = Area
        fields = ['name']
        #read_only_fields = ['name']


class SuggestionsCitySerializer(serializers.ModelSerializer):
    class Meta:
        model = City
        fields = ['name']


class SuggestionsStateGetSerializer(serializers.ModelSerializer):

    class Meta:
        model = State
        fields = ['name']


class SuggestionsCountrySerializer(serializers.ModelSerializer):
    class Meta:
        model = Country
        fields = ['name']


class VenueSuggestionsSerializer(serializers.ModelSerializer):

    class Meta:
        model = Venue
        fields = ['name']
