import json
from django.http import HttpResponse
from django.template import Template, Context
from django.db import transaction
from django.urls import reverse
from django.contrib.contenttypes.models import ContentType

from django.contrib.auth import login as django_login
import datetime
from rest_framework.authtoken.models import Token
from rest_framework import status, generics

from rest_framework.response import Response
from rest_framework.authentication import TokenAuthentication
from rest_framework.permissions import IsAuthenticated

from utils.mixins import PlatformAPICallMixin
from utils.api_handle_errors import handle_errors
from commonAPI.serializers import ContactPersonSerializer
from commonAPI.models import Contact_Person
from venues.serializers import *
from venues.models import *
from venue_owners.serializers import *
from venue_owners.models import VenueOwner
from static_data.models import *
from static_data.serializers import VenueServiceSerializer
from django.db.models import Avg, Count
from reviews.models import *
############################ -- GET -- ###############################


class VenueGetAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = VenueProfileSerializer
    # Get Venue

    def get(self, request, format=None):
        '''
        Venue Profile GET
        '''

        response = {}
        responsedata = []
        data = Venue.objects.filter(created_by=request.user)
        OwnerInfo = Venue.objects.filter(
            created_by=request.user, is_deleted=False)
        serializer = self.serializer_class(OwnerInfo, many=True)
        working_hours = VenueDailyTiming.objects.filter(venue_id__in=data)
        venue_image = VenueImage.objects.filter(venue_id__in=data)
        venue_image_serializer = VenueImageSerializer(venue_image, many=True)
        # venue_owner = VenueOwner.objects.filter(user=request.user, is_deleted=False)

        content_type = ContentType.objects.get_for_model(Venue)
        contact_person = Contact_Person.objects.filter(
            content_type__pk=content_type.id, object_id__in=OwnerInfo)
        contact_person_serializer = ContactPersonSerializer(
            contact_person, many=True)
        working_hours_serializer = VenueDailyAvailibilitySerializer(
            working_hours, many=True)

        responsedata = [{"contact_person": contact_person_serializer.data}, {
            "working_hours": working_hours_serializer.data}]
        responsedata = serializer.data + responsedata + venue_image_serializer.data

        response['result'] = 1
        response['data'] = responsedata
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)

    # Edit Venue

    def post(self, request, format=None):
        '''
        Edit Venue Profile
        '''
        responsedata = []
        response = {}
        if 'id' in request.data:
            if Venue.objects.filter(id=request.data['id'], created_by=request.user, is_deleted=False).exists():
                serializer = self.serializer_class(instance=Venue.objects.filter(
                    created_by=request.user, id=request.data['id'], is_deleted=False).first(), data=request.data)
            message = "Updated Successfully"
        else:
            serializer = self.serializer_class(data=request.data)
            message = "Created Successfully"

        if serializer.is_valid():
            serializer.save()
            response["result"] = 1
            response['errors'] = [message]
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class VenueBankInfoGetAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = VenueBankInfoSerializer
    # GET Bank Info

    def get(self, request, format=None):
        '''
        Get Venue Bank Info
        '''
        responsedata = []
        response = {}
        BankInfo = VenueBankInfo.objects.filter(
            created_by=request.user, is_deleted=False)
        serializers = self.serializer_class(BankInfo, many=True)
        response['result'] = 1
        response['data'] = serializers.data
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)

    # Update Bank Info

    def post(self, request, format=None):
        '''
        Update Venue Bank Info
        '''
        responsedata = []
        response = {}
        if VenueBankInfo.objects.filter(created_by=request.user, is_deleted=False).exists():
            serializer = self.serializer_class(instance=VenueBankInfo.objects.filter(
                created_by=request.user, is_deleted=False).first(), data=request.data)
            if serializer.is_valid():
                serializer.save()
                response["result"] = 1
                response['errors'] = str("Updated Successfully")
                return Response(response, status=status.HTTP_200_OK)
            else:
                response = handle_errors(serializer.errors)
                response["result"] = 0
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)
        else:
            responsedata.append("No such Record")
            response['errors'] = responsedata
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class VenueFieldPOSTInfoAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = VenueFieldSerializer

    # Update/ADD Field Info
    def post(self, request, format=None):
        '''
        Post request for Updating the field

        '''
        responsedata = []
        response = {}
        request.data._mutable = True
        request.data['created_by'] = User.objects.get(
            username=str(request.user)).id
        if 'id' in request.data:
            if VenueField.objects.filter(created_by=request.user, id=request.data['id'], is_deleted=False).exists():
                serializer = self.serializer_class(instance=VenueField.objects.filter(
                    created_by=request.user, id=request.data['id'], is_deleted=False).first(), data=request.data)
                message = "Updated Successfully"
            else:
                response['errors'] = "No Record Found with the ID"
                response["result"] = 1
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)

        else:
            serializer = self.serializer_class(
                data=request.data, context={"user": request.user})
            message = "Inserted Successfully"

        # save data
        if serializer.is_valid():
            if "id" in serializer.validated_data:
                # services = []

                # if "services" in serializer.validated_data:
                #     services = serializer.validated_data['services']
                #     serializer.validated_data.pop("services")

                field_obj = serializer.save()
                return_data = {
                    "id": field_obj.id,
                    "description": field_obj.description,
                    "field_name": field_obj.name,
                    "field_type": field_obj.field_type.name,
                    "floor_type": field_obj.floor_type.name,
                    "seat_capacity": field_obj.seat_capacity,
                    "lights": field_obj.lights.name,
                    "facility_type": field_obj.facility_type.name,
                    "facility_type_id": field_obj.facility_type.id,
                    "sport": field_obj.sport.name,
                }
                main_image_class = VenueFieldSerializerVer1(
                    field_obj, many=False)
                return_data['main_image'] = main_image_class.data

                if "services" in serializer.validated_data:
                    VenueFieldService.objects.filter(
                        created_by=request.user, venue_field=field_obj).delete()
                    venue_fieldService = [VenueFieldService(
                        **i,
                        venue_field=field_obj,
                        created_by=request.user
                    ) for i in serializer.validated_data['services']]
                    service_obj = VenueFieldService.objects.bulk_create(
                        venue_fieldService)
                    services_class = VenueServicePreviewSerializer(
                        service_obj, many=True)
                    return_data['services'] = services_class.data

                if 'images' in serializer.validated_data:
                    venueField_image = [VenueFieldImage(
                        image=i,
                        venue_field=field_obj,
                        created_by=field_obj['created_by']
                    ) for i in serializer.validated_data['images']]
                    image_obj = VenueFieldImage.objects.bulk_create(
                        venueField_image)

                sub_image_obj = VenueFieldImage.objects.filter(
                    venue_field=field_obj)
                images_class = VenueFieldImageSerializer(
                    sub_image_obj, many=True)
                return_data['images'] = images_class.data

            else:
                field_obj = serializer.save()
                return_data = field_obj

            responsedata.append(message)
            response['results'] = responsedata
            response["result"] = 1
            response['data'] = return_data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class VenueFieldGETInfoAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = VenueFieldGETSerializer
    # sport_serializer_class = SportSerializer

    # Get Field Info

    def post(self, request, format=None):
        '''
        POST request for getting the field
        '''
        try:
            responsedata = []
            response = {}
            if 'id' in request.data:
                serializers = self.serializer_class(data=request.data)
                if serializers.is_valid():
                    start_count = (serializers.validated_data['page'] - 1) * 10
                    end_count = start_count + 10
                    serializers.validated_data.pop("page")

                    FieldInfo = VenueField.objects.filter(
                        id=serializers.validated_data['id'], is_deleted=False)[start_count:end_count]

                    total_count = VenueField.objects.filter(
                        id=serializers.validated_data['id'], is_deleted=False).count()
                else:
                    response = handle_errors(serializers.errors)
                    response["result"] = 0
                    self.add_api_call(request, response)
                    return Response(response, status=status.HTTP_400_BAD_REQUEST)
            else:
                # Check for mode
                serializers = VenueFieldPaginationSerializer(data=request.data)
                if serializers.is_valid():
                    start_count = (serializers.validated_data['page'] - 1) * 10
                    end_count = start_count + 10

                if "mode" in request.data:
                    FieldInfo = VenueField.objects.filter(is_deleted=False)[
                        start_count:end_count]
                    total_count = VenueField.objects.filter(
                        is_deleted=False).count()
                else:
                    FieldInfo = VenueField.objects.filter(
                        created_by=request.user, is_deleted=False)[start_count:end_count]
                    total_count = VenueField.objects.filter(
                        created_by=request.user, is_deleted=False).count()

            serializers = self.serializer_class(FieldInfo, many=True)

            response['result'] = 1
            response['data'] = serializers.data
            response['total_count'] = total_count
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        except Exception as e:
            response["result"] = 0
            responsedata.append(str(e))
            response['errors'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_401_UNAUTHORIZED)


class VenueContactAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = VenueContactPersonSerializer

    # Get Contact Info
    def get(self, request, format=None):
        '''
        GET request for getting the Contact Information
        '''
        try:
            responsedata = []
            response = {}
            FieldInfo = VenueContactPerson.objects.filter(
                created_by=request.user, is_deleted=False).first()
            serializers = self.serializer_class(FieldInfo, many=False)
            response['result'] = 1
            response['data'] = serializers.data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        except Exception as e:
            response["result"] = 0
            responsedata.append(str(e))
            response['errors'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_401_UNAUTHORIZED)

    # Update Contact Info
    def post(self, request, format=None):
        '''
        Post request for Updating the Contact Information field
        '''
        try:
            responsedata = []
            response = {}
            request.data._mutable = True
            request.data['venue'] = Venue.objects.get(
                name=request.data['venue']).id
            if VenueContactPerson.objects.filter(created_by=request.user, venue=request.data['venue'], is_deleted=False).exists():
                serializer = self.serializer_class(instance=VenueContactPerson.objects.filter(
                    created_by=request.user, venue=request.data['venue'], is_deleted=False).first(), data=request.data)
                if serializer.is_valid():
                    serializer.validated_data['venue'] = Venue.objects.get(
                        id=serializer.validated_data['venue'])
                    serializer.save()
                    responsedata.append('Updated Successfully')
                    response['errors'] = responsedata
                    response["result"] = 1
                    return Response(response, status=status.HTTP_200_OK)
                else:
                    response = handle_errors(serializer.errors)
                    response["result"] = 0
                    self.add_api_call(request, response)
                    return Response(response, status=status.HTTP_400_BAD_REQUEST)

            else:
                response = handle_errors(serializer.errors)
                response["result"] = 0
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_400_BAD_REQUEST)
        except Exception as e:
            response["result"] = 0
            responsedata.append(str(e))
            response['errors'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class VenuePhotosPOSTAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = VenuePhotoSerializer

    def post(self, request, format=None):
        try:
            response = {}
            responsedata = []
            serializers = self.serializer_class(
                data=request.data, context={'user': request.user})
            if serializers.is_valid():
                serializers.save()
            else:
                response = handle_errors(serializers.errors)
                response["result"] = 0
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_400_BAD_REQUEST)
            response["result"] = 1
            responsedata.append("Successfully Inserted")
            response['errors'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        except Exception as e:
            response["result"] = 0
            responsedata.append(str(e))
            response['errors'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class VenueServicesGETAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = VenueProfileServiceGETSerializer

    def post(self, request, format=None):
        '''
        GET Venue Services
        '''
        try:
            responsedata = []
            response = {}
            serializers = self.serializer_class(data=request.data)
            if serializers.is_valid():
                venue_extra = Venue.objects.get(
                    id=serializers.validated_data['id'])
                venue_class = self.serializer_class(venue_extra)
                response["result"] = 1
                response['data'] = venue_class.data
                return Response(response, status=status.HTTP_200_OK)
            else:
                response = handle_errors(serializers.errors)
                response["result"] = 0
                return Response(response, status=status.HTTP_400_BAD_REQUEST)
        except Exception as e:
            response["result"] = 0
            responsedata.append(str(e))
            response['errors'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class VenueServicesPOSTAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = VenueProfileServiceSerializer

    def post(self, request, format=None):
        '''
        Edit Venue Services
        '''
        try:
            responsedata = []
            response = {}
            if Venue.objects.filter(created_by=request.user, is_deleted=False).exists():
                serializer = self.serializer_class(instance=Venue.objects.filter(
                    created_by=request.user, is_deleted=False).first(), data=request.data)
            else:
                raise Exception("No Such Venue")
            message = "Updated Successfully"
            if serializer.is_valid():
                serializer.validated_data['created_by'] = request.user
                serializer.save()
                response["result"] = 1
                response['errors'] = ["Updated Successfully"]
                return Response(response, status=status.HTTP_200_OK)
            else:
                response = handle_errors(serializer.errors)
                response["result"] = 0
                return Response(response, status=status.HTTP_400_BAD_REQUEST)
        except Exception as e:
            response["result"] = 0
            responsedata.append(str(e))
            response['errors'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class VenueEntryPOSTAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = VenueEntrySerializers

    def post(self, request, format=None):
        '''
        Edit Venue Entry Permission
        '''
        try:
            responsedata = []
            response = {}
            if Venue.objects.filter(created_by=request.user, is_deleted=False).exists():
                serializer = self.serializer_class(instance=Venue.objects.filter(
                    created_by=request.user, is_deleted=False).first(), data=request.data)
            else:
                raise Exception("No Such Venue")
            message = "Updated Successfully"
            if serializer.is_valid():
                serializer.save()
                response["result"] = 1
                response['errors'] = ["Updated Successfully"]
                return Response(response, status=status.HTTP_200_OK)
            else:
                response = handle_errors(serializer.errors)
                response["result"] = 0
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_400_BAD_REQUEST)
        except Exception as e:
            response["result"] = 0
            responsedata.append(str(e))
            response['errors'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class VenueEntryGETAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = VenueEntryGETSerializers

    def post(self, request, format=None):
        '''
        Edit Venue Entry Permission
        '''
        try:
            responsedata = []
            response = {}
            if VenueField.objects.filter(id=request.data['id'], is_deleted=False).exists():
                serializer = self.serializer_class(data=request.data)
            else:
                raise Exception("No Such Venue")
            message = "Updated Successfully"
            if serializer.is_valid():
                venue_field_object = VenueField.objects.get(
                    id=serializer.validated_data['id'])
                venue_object = venue_field_object.venue
                venue_class = self.serializer_class(venue_object)
                response["result"] = 1
                response['data'] = venue_class.data
                return Response(response, status=status.HTTP_200_OK)
            else:
                response = handle_errors(serializer.errors)
                response["result"] = 0
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_400_BAD_REQUEST)
        except Exception as e:
            response["result"] = 0
            responsedata.append(str(e))
            response['errors'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class GlobalGetVenueFieldAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = GlobalGETSerializer
    venuePhoto_class = VenuePhotoGETSerializer
    venue_class = VenueProfileGlobalSerializer
    field_class = VenueFieldGlobalSerializer

    def post(self, request, format=None):
        '''Global GET '''
        try:
            response = {}
            responsedata = []
            serializers = self.serializer_class(data=request.data)
            if serializers.is_valid():
                if "id" in serializers.validated_data:
                    venue_data = Venue.objects.filter(
                        id=serializers.validated_data['id'])
                    image_data = VenueImage.objects.filter(
                        venue=serializers.validated_data['id'], is_deleted=False)
                    field_obj = VenueField.objects.filter(
                        venue=venue_data.last(), is_deleted=False)

                else:
                    venue_data = Venue.objects.filter(created_by=request.user)
                    # venue=venue_data.first(),
                    image_data = VenueImage.objects.filter(
                        created_by=request.user, is_deleted=False)
                    field_obj = VenueField.objects.filter(
                        created_by=request.user, is_deleted=False)

                serializers_venue = self.venue_class(venue_data, many=True)
                serializers_image = self.venuePhoto_class(
                    image_data, many=True)
                serializers_fields = self.field_class(field_obj, many=True)

                # Venue attributes
                venue_attributes = []
                venue_fields = serializers_fields.data
                for i in venue_fields:
                    i['image'] = VenueFieldImageGlobalSerializer(VenueFieldImage.objects.filter(
                        venue_field=i['id'], is_deleted=False), many=True).data
                    if 'floor_type' in i.keys():
                        venue_attributes.append(i['floor_type'])
                    if 'field_type' in i.keys():
                        venue_attributes.append(i['field_type'])
                    if 'field_type' not in i.keys() and 'floor_type' not in i.keys():
                        VenueField.objects.filter(id=i['id']).delete()

                venue_attributes = list(set(venue_attributes))

                # Venue Availability
                availability = VenueDailyTiming.objects.filter(
                    venue=venue_data.last(), is_active=True, is_deleted=False)
                timing = VenueTimingSerializer(availability, many=True)

                venue = venue_data.first()
                venue_serializer = VenueSerializer(venue)
                venue_services_cost = VenueServiceCost.objects.filter(
                    venue=venue_serializer.data['id'])

                paid_service_list = []
                for item in venue_services_cost:
                    paid_service_list.append(item.venue_service)
                venue_paid_services_serializer = VenueServiceSerializer(
                    paid_service_list, many=True)

                response['result'] = 1
                responsedata.append({'venue_profile': serializers_venue.data})
                responsedata.append(
                    {'venue_paid_services': venue_paid_services_serializer.data})
                responsedata.append({'venue_images': serializers_image.data})
                responsedata.append({'venue_fields': venue_fields})
                responsedata.append({'venue_calender': timing.data})
                responsedata.append({'venue_attributes': venue_attributes})
                response['data'] = responsedata
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_200_OK)
            else:
                response = handle_errors(serializers.errors)
                response["result"] = 0
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_400_BAD_REQUEST)
        except Exception as e:
            response["result"] = 0
            responsedata.append(str(e))
            response['errors'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class FieldNoBookingAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    # authentication_classes = (TokenAuthentication,)
    # permission_classes = (IsAuthenticated,)
    serializer_class = FieldNobookingSerializer

    venue_field_serializer_class = VenueFieldGETSerializer
    venue_field_image_serializer = VenueFieldImageSerializer
    venue_field_service_serializer = VenueFieldServiceSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []

        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():

            venue_field = serializers.validated_data['id']
            serializer_data = self.venue_field_serializer_class(venue_field)

            # venue Field images
            field_images = VenueFieldImage.objects.filter(
                venue_field=serializers.validated_data['id'], is_deleted=False)
            serializer_field_images = self.venue_field_image_serializer(
                field_images, many=True)

            # VenueFieldService
            venue_field_services = VenueFieldService.objects.filter(
                venue_field=serializers.validated_data['id'], is_deleted=False)
            serializer_field_services = self.venue_field_service_serializer(
                venue_field_services, many=True)

            # Venue Availability
            availability = VenueDailyTiming.objects.filter(
                venue=venue_field.venue, is_active=True, is_deleted=False)
            timing = VenueTimingSerializer(availability, many=True)

            # Venue Address
            venue_obj = venue_field.venue
            vdata = VenueProfileGlobalSerializer(venue_obj, many=False)

            data = {"field_data": serializer_data.data, "field_images": serializer_field_images.data,
                    "timing": timing.data, "address": venue_obj.google_address,
                    "latitude": venue_obj.latitude, "longitude": venue_obj.longitude, "venue_data": vdata.data}
            covid19Measure_count = venue_obj.covid19Measure_description.count()

            response['results'] = 1
            response['data'] = data
            response['total_Covid19Measures_count'] = covid19Measure_count
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        else:
            response['results'] = 0
            response = handle_errors(serializers.errors)
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class DeleteFieldAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = DeleteFieldSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []

        if not VenueField.objects.filter(id=request.data['id'], created_by=request.user).exists():
            response['results'] = 0
            response['error'] = "Not a valid Field"
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        serializers = self.serializer_class(data=request.data)

        if serializers.is_valid():
            VenueField.objects.filter(
                id=serializers.validated_data['id']).delete()

            response['results'] = 1
            response['error'] = "Successfully Deleted"
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        else:
            response['results'] = 0
            response = handle_errors(serializers.errors)
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class GetFieldDisabledServicesAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = FieldServicesSerializer

    def get(self, request, format=None):
        """
        Field Services
        """
        response = {}
        responsedata = []

        all_field_serivces = VenueService.objects.all()

        user_field_services = VenueFieldService.objects.filter(
            venue_field__created_by=request.user, venue_field__is_deleted=False, is_deleted=False).values_list("venue_service", flat=True).distinct()

        all_field_service_class = self.serializer_class(
            all_field_serivces, many=True)

        for i in all_field_service_class.data:
            if i['id'] in user_field_services:
                i['valid'] = False
            else:
                i['valid'] = True

        response['results'] = 1
        response['error'] = all_field_service_class.data
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


class DeleteVenueImageAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    '''
    Delete Venue Image  url=DeleteVenueImage
    '''
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = DeleteVenuePhotoSerializer

    @ transaction.atomic
    def post(self, request, format=None):
        serializer = self.serializer_class(data=request.data)
        if serializer.is_valid():
            responsedata = []
            response = {}
            if VenueImage.objects.filter(created_by=request.user, id=serializer.data['id'], is_deleted=False).exists():
                data = VenueImage.objects.filter(
                    created_by=request.user, id=serializer.data['id'], is_deleted=False).first()
                data.is_deleted = True
                data.save(update_fields=['is_deleted'])
            response['result'] = 1
            response['data'] = "Successfully deleted"
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class DeleteVenueFieldImageAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    '''
    Delete Venue Field Image
    '''
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = DeleteVenueFieldPhotoSerializer

    @ transaction.atomic
    def post(self, request, format=None):
        serializer = self.serializer_class(data=request.data)
        if serializer.is_valid():
            responsedata = []
            response = {}
            if VenueFieldImage.objects.filter(created_by=request.user, id=serializer.data['id'], is_deleted=False).exists():
                data = VenueFieldImage.objects.filter(
                    created_by=request.user, id=serializer.data['id'], is_deleted=False).first()
                data.is_deleted = True
                data.save(update_fields=['is_deleted'])
            response['result'] = 1
            response['data'] = "Successfully deleted"
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class VenuePhotosNewPOSTAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = VenuePhotoNewSerializer

    def post(self, request, format=None):
        try:
            response = {}
            responsedata = []

            serializers = self.serializer_class(
                data=request.data, context={'user': request.user})
            if serializers.is_valid():
                serializers.save()
            else:
                response = handle_errors(serializers.errors)
                response["result"] = 0
                self.add_api_call(request, response)
                return Response(response, status=status.HTTP_400_BAD_REQUEST)
            response["result"] = 1
            responsedata.append("Successfully Inserted")
            response['errors'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        except Exception as e:
            response["result"] = 0
            responsedata.append(str(e))
            response['errors'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class VenueServicesGETAPIView1(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = VenueServiceSerializer1

    def post(self, request, format=None):
        try:
            response = {}
            responsedata = []

            venue_service_objects = VenueService.objects.filter(
                type='P', service_available='F')
            serializers = self.serializer_class(
                venue_service_objects, many=True)

            response["result"] = 1
            response['data'] = serializers.data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        except Exception as e:
            response["result"] = 0
            responsedata.append(str(e))
            response['errors'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class VenueServicesGETAPIView2(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = VenueServiceSerializer1

    def post(self, request, format=None):
        try:
            response = {}
            responsedata = []

            venue_service_objects = VenueService.objects.filter(
                type='F', service_available='V')
            serializers = self.serializer_class(
                venue_service_objects, many=True)

            response["result"] = 1
            response['data'] = serializers.data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        except Exception as e:
            response["result"] = 0
            responsedata.append(str(e))
            response['errors'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class VenueServicesGETAPIView3(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = VenueServiceSerializer1

    def post(self, request, format=None):
        try:
            response = {}
            responsedata = []

            venue_service_objects = VenueService.objects.filter(
                type='P', service_available='V')
            serializers = self.serializer_class(
                venue_service_objects, many=True)

            response["result"] = 1
            response['data'] = serializers.data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)

        except Exception as e:
            response["result"] = 0
            responsedata.append(str(e))
            response['errors'] = responsedata
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class VenueOwnerServicesPOSTAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = VenueOwnerServiceSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []
        venue_owner = VenueOwner.objects.get(
            user=request.user, is_deleted=False)
        venue = Venue.objects.filter(created_by=venue_owner.user).first()
        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            venue_service_cost_object = [VenueServiceCost(
                venue_service=VenueService.objects.filter(id=i).last(),
                price_per_hour="0",
                venue=venue,
            )
                for i in serializers.validated_data['venue_service']]

            VenueServiceCost.objects.bulk_create(
                venue_service_cost_object)
            response["result"] = 1
            response['message'] = "Added Successfully"
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class VenueOwnerPaidServiceGETAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = VenueOwnerPaidServiceCostSerializer

    def get(self, request, format=None):
        response = {}
        responsedata = []
        venue_service = []
        venue_owner = VenueOwner.objects.get(
            user=request.user, is_deleted=False)
        venue = Venue.objects.filter(created_by=venue_owner.user).first()
        venue_service_cost_object = VenueServiceCost.objects.filter(
            venue=venue.id)
        serializers = self.serializer_class(
            venue_service_cost_object, many=True)
        print(serializers.data)

        for item in serializers.data:
            data = {}
            data['venue_paid_service'] = item['venue_service']
            data['price_per_hr'] = item['price_per_hour']
            responsedata.append(data)

        response["result"] = 1
        response['data'] = responsedata
        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


class VenueFieldPaidServiceGETAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = VenueFieldPaidServiceSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []
        serializer = self.serializer_class(request.data)
        venue_field_object = VenueFieldService.objects.filter(
            venue_field=serializer.data['venue_field'])
        serializers = VenueFieldPaidServiceSerializer1(
            venue_field_object, many=True)

        for item in serializers.data:
            data = {}
            data['venue_paid_service'] = item['venue_service']
            data['price_per_hr'] = item['price_per_hour']
            responsedata.append(data)

        response['result'] = 1
        response['data'] = responsedata

        self.add_api_call(request, response)
        return Response(response, status=status.HTTP_200_OK)


class VenueFieldPOSTServicesSerializerAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = VenueFieldPOSTServicesSerializer

    def post(self, request, format=None):
        '''
        Post request for Updating the field into VenueFieldService

        '''
        responsedata = []
        response = {}
        request.data._mutable = True
        request.data['created_by'] = User.objects.get(
            username=str(request.user)).id

        serializer = self.serializer_class(
            data=request.data, context={"user": request.user})
        message = "Inserted Successfully"

        # save data
        if serializer.is_valid():
            field_obj = serializer.save()
            return_data = field_obj

            responsedata.append(message)
            response['results'] = responsedata
            response["result"] = 1
            response['data'] = return_data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class VenuePOSTServicesSerializerAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)
    serializer_class = VenuePOSTServicesSerializer

    def post(self, request, format=None):
        '''
        Post request for Updating the field into VenueFieldService

        '''
        responsedata = []
        response = {}
        request.data._mutable = True

        venue = Venue.objects.get(created_by=request.user)

        serializer = self.serializer_class(
            data=request.data, context={"user": request.user})
        message = "Inserted Successfully"

        # save data
        if serializer.is_valid():
            field_obj = serializer.save()
            return_data = field_obj

            responsedata.append(message)
            response['results'] = responsedata
            response["result"] = 1
            response['data'] = return_data
            response['data'] = return_data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializer.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


# class AllVenueGetAPIView(PlatformAPICallMixin, generics.GenericAPIView):
#     """
#     Venue
#     """
#     serializer_class = AllVenuegetSerializer

#     def post(self, request, format=None):
#         response = {}
#         responsedata = []
#         # get page number
#         if 'page' in request.data:
#             start_count = int(request.data['page']) - 1
#         else:
#             start_count = 0
#         start_count = start_count * 10
#         end_count = start_count + 10
#         data = Venue.objects.all()[start_count:end_count]
#         total_count = Venue.objects.all().count()
#         serializer = self.serializer_class(data, many=True)
#         response['result'] = 1
#         response['data'] = serializer.data
#         response['total_count'] = total_count
#         self.add_api_call(request, response)
#         return Response(response, status=status.HTTP_200_OK)


class AllVenueGetAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    """
    Venue
    """
    serializer_class = GetAllVenueFilterSerializer

    def post(self, request, format=None):
        response = {}
        responsedata = []
        serializers = self.serializer_class(data=request.data)

        if serializers.is_valid():

            start_count = (serializers.validated_data['page'] - 1) * 10
            end_count = start_count + 10
            serializers.validated_data.pop("page")

            star_rating = None
            if "star_ratings" in serializers.validated_data:
                star_rating = serializers.validated_data['star_ratings']
                if (star_rating) == 0:
                    star_rating = None
                serializers.validated_data.pop("star_ratings")

            if "all_sports" in serializers.validated_data:
                if serializers.validated_data['all_sports'] == "any":
                    serializers.validated_data.pop("all_sports")
                else:
                    if Sport.objects.filter(id=serializers.validated_data["all_sports"]).exists():
                        serializers.validated_data['all_sports'] = Sport.objects.get(
                            id=serializers.validated_data['all_sports'])

            attributes = None
            if "attributes" in serializers.validated_data:
                attributes = serializers.validated_data['attributes']
                serializers.validated_data.pop("attributes")

            field_type = None
            if "field_type" in serializers.validated_data:
                if FieldType.objects.filter(id=serializers.validated_data["field_type"]).exists():
                    serializers.validated_data['field_type'] = FieldType.objects.get(
                        id=serializers.validated_data['field_type'])
                    field_type = serializers.validated_data['field_type']
                serializers.validated_data.pop("field_type")

            if "location" in serializers.validated_data:
                location = serializers.validated_data['location']
                serializers.validated_data.pop("location")

                # Get Venue
                venue_data = Venue.objects.filter(city__in=location,
                                                  is_deleted=False, **serializers.validated_data)
            else:
                venue_data = Venue.objects.filter(
                    is_deleted=False, **serializers.validated_data)

            serialized_data = AllVenuegetSerializer(venue_data, many=True)

            for i in serialized_data.data:
                i['address'] = str(i['city']) + " " + str(i['street']) + \
                    " " + str(i['country']) + " " + str(i['pincode'])
                i['address'] = i['address'].replace(
                    "None", "").strip().replace(" ", ", ")

                venue_content = ContentType.objects.get_for_model(Venue)
                venue_content_to = Venue.objects.filter(
                    id=i['id'], is_deleted=False)
                venue_ratings = VenueReviewAndRating.objects.filter(venue=i['id'], is_deleted=False).values(
                    'rating').aggregate(rating=Avg('rating'))
                print(venue_ratings)

                field_obj = VenueField.objects.filter(
                    venue=i['id'],  is_deleted=False)

                i['no_of_fields'] = len(field_obj)
                data = {}
                if star_rating is not None:
                    # print(star_rating[-1])
                    if venue_ratings['rating'] is not None:
                        if venue_ratings['rating'] >= star_rating[0] and venue_ratings['rating'] <= star_rating[-1]:
                            i['ratings'] = venue_ratings['rating']
                            data['id'] = i['id']
                            data['page'] = i['page']
                            data['all_sports'] = i['all_sports']
                            data['all_services'] = i['all_services']
                            data['covid19Measure_description'] = i['covid19Measure_description']
                            data['name'] = i['name']
                            data['ratings'] = i['ratings']
                            data['no_of_fields'] = i['no_of_fields']
                            data['address'] = i['address']
                            data['logo'] = i['logo']
                            data['main_image'] = i['main_image']
                            data['ratings'] = i['ratings']
                            data['no_of_fields'] = i['no_of_fields']
                            data['address'] = i['address']
                            data['business_name'] = i['business_name']
                            data['area'] = i['area']
                            data['city'] = i['city']
                            data['state'] = i['state']
                            data['country'] = i['country']
                            responsedata.append(data)

                        else:
                            continue
                else:
                    if venue_ratings['rating'] == None:
                        venue_ratings['rating'] = 0
                    i['ratings'] = venue_ratings['rating']
                    data['id'] = i['id']
                    data['page'] = i['page']
                    data['all_sports'] = i['all_sports']
                    data['all_services'] = i['all_services']
                    data['covid19Measure_description'] = i['covid19Measure_description']
                    data['name'] = i['name']
                    data['logo'] = i['logo']
                    data['main_image'] = i['main_image']
                    data['ratings'] = i['ratings']
                    data['no_of_fields'] = i['no_of_fields']
                    data['address'] = i['address']
                    data['business_name'] = i['business_name']
                    data['area'] = i['area']
                    data['city'] = i['city']
                    data['state'] = i['state']
                    data['country'] = i['country']

                    # responsedata.append(data)

                    if attributes or field_type is not None:
                        if VenueField.objects.filter(venue=i['id']).exists():
                            new_field_obj = VenueField.objects.filter(
                                venue=i['id'])
                            print(new_field_obj)
                            # Get fields ids
                            field_ids = [i.id for i in new_field_obj]
                            if attributes:
                                venue_facility_obj = VenueField.objects.filter(
                                    facility_type__name__in=attributes, id__in=field_ids, is_deleted=False)
                                print('venue_facility_obj---',
                                      venue_facility_obj)

                                venue_lights_obj = VenueField.objects.filter(
                                    lights__name__in=attributes, id__in=field_ids, is_deleted=False)
                                print('venue_lights_obj---', venue_lights_obj)

                                if venue_facility_obj.count() > 0 or venue_lights_obj.count() > 0:

                                    i['ratings'] = venue_ratings['rating']
                                    data['id'] = i['id']
                                    data['page'] = i['page']
                                    data['all_sports'] = i['all_sports']
                                    data['all_services'] = i['all_services']
                                    data['covid19Measure_description'] = i['covid19Measure_description']
                                    data['name'] = i['name']
                                    data['logo'] = i['logo']
                                    data['main_image'] = i['main_image']
                                    data['ratings'] = i['ratings']
                                    data['no_of_fields'] = i['no_of_fields']
                                    data['address'] = i['address']
                                    data['business_name'] = i['business_name']
                                    data['area'] = i['area']
                                    data['city'] = i['city']
                                    data['state'] = i['state']
                                    data['country'] = i['country']

                                    responsedata.append(data)
                            if field_type:

                                venue_field_type_obj = VenueField.objects.filter(
                                    field_type=field_type, id__in=field_ids, is_deleted=False)
                                print('venue_field_type_obj---',
                                      venue_field_type_obj)
                                if venue_field_type_obj.count() > 0:

                                    i['ratings'] = venue_ratings['rating']
                                    data['id'] = i['id']
                                    data['page'] = i['page']
                                    data['all_sports'] = i['all_sports']
                                    data['all_services'] = i['all_services']
                                    data['covid19Measure_description'] = i['covid19Measure_description']
                                    data['name'] = i['name']
                                    data['logo'] = i['logo']
                                    data['main_image'] = i['main_image']
                                    data['ratings'] = i['ratings']
                                    data['no_of_fields'] = i['no_of_fields']
                                    data['address'] = i['address']
                                    data['business_name'] = i['business_name']
                                    data['area'] = i['area']
                                    data['city'] = i['city']
                                    data['state'] = i['state']
                                    data['country'] = i['country']

                                    responsedata.append(data)
                    else:
                        responsedata.append(data)

                    # responsedata.append(i)
                    # i['ratings'] = venue_ratings['rating']

            response['result'] = 1
            response['data'] = responsedata[start_count:end_count]
            response['total_count'] = len(responsedata)
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response['results'] = 0
            response = handle_errors(serializers.errors)
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)


class GetNameSuggestionsAPIView(PlatformAPICallMixin, generics.GenericAPIView):
    '''
    Get Names Suggestions
    '''
    # authentication_classes = (TokenAuthentication,)
    # permission_classes = (IsAuthenticated,)
    serializer_class = GetSuggestionsSerializer

    def post(self, request, format=None):

        serializers = self.serializer_class(data=request.data)
        if serializers.is_valid():
            response = {}
            responsedata = []

            # Area
            data = Area.objects.filter(
                name__icontains=serializers.validated_data['key_word'])
            Area_data = SuggestionsAreaGetSerializer(data, many=True)

            # City
            data = City.objects.filter(
                name__icontains=serializers.validated_data['key_word'])
            City_data = SuggestionsCitySerializer(data, many=True)

            # State
            data = State.objects.filter(
                name__icontains=serializers.validated_data['key_word'])
            State_data = SuggestionsStateGetSerializer(data, many=True)

            # Country
            data = Country.objects.filter(
                name__icontains=serializers.validated_data['key_word'])
            Country_data = SuggestionsCountrySerializer(data, many=True)

            # Venue
            data = Venue.objects.filter(
                name__icontains=serializers.validated_data['key_word'])
            Venue_data = VenueSuggestionsSerializer(data, many=True)

            response['result'] = 1
            response['data'] = Country_data.data + \
                State_data.data + City_data.data + Area_data.data + Venue_data.data
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = handle_errors(serializers.errors)
            response["result"] = 0
            self.add_api_call(request, response)
            return Response(response, status=status.HTTP_200_OK)
